@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/attribute_manager.cc,v 1.6 2004/06/10 22:40:28 hodson Exp $"
+#ident "$XORP: xorp/bgp/attribute_manager.cc,v 1.9 2005/03/25 02:52:38 pavlin Exp $"
 
 //#define DEBUG_LOGGING
 #include "bgp_module.h"
@@ -39,19 +39,19 @@ AttributeManager<A>::add_attribute_list(
     debug_msg("AttributeManager<A>::add_attribute_list\n");
     StoredAttributeList<A> *new_att =
 	new StoredAttributeList<A>(attribute_list);
-    typedef typename set<StoredAttributeList<A>*>::iterator Iter;
+    typedef typename set<StoredAttributeList<A>*, Att_Ptr_Cmp<A> >::iterator Iter;
     Iter i = _attribute_lists.find(new_att);
     if (i == _attribute_lists.end()) {
 	new_att->clone_data();
 	_attribute_lists.insert(new_att);
-	debug_msg("ATMgr: Inserting new attribute %x\n",
-	       (u_int)(new_att->attribute()));
+	debug_msg("ATMgr: Inserting new attribute %p\n",
+		  new_att->attribute());
 	return new_att->attribute();
     } else {
 	(*i)->increase();
 	delete new_att;
-	debug_msg("** (+) ref count for %x now %d\n",
-	       (u_int)((*i)->attribute()), (*i)->references());
+	debug_msg("** (+) ref count for %p now %d\n",
+		  (*i)->attribute(), (*i)->references());
 	return (*i)->attribute();
     }
     debug_msg("done\n");
@@ -66,13 +66,13 @@ AttributeManager<A>::delete_attribute_list(
 	new StoredAttributeList<A>(attribute_list);
     debug_msg("AttributeManager<A>::delete_attribute_list %p\n",
 	      del_att->attribute());
-    typedef typename set<StoredAttributeList<A>*>::iterator Iter;
+    typedef typename set<StoredAttributeList<A>*, Att_Ptr_Cmp<A> >::iterator Iter;
     Iter i = _attribute_lists.find(del_att);
     assert(i != _attribute_lists.end());
     delete del_att;
     (*i)->decrease();
-    debug_msg("** (-) ref count for %x now %d\n",
-	   (u_int)((*i)->attribute()), (*i)->references());
+    debug_msg("** (-) ref count for %p now %d\n",
+	      (*i)->attribute(), (*i)->references());
     if ((*i)->references() == 0) {
 	del_att = (*i);
 	_attribute_lists.erase(i);
