@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/bgp.cc,v 1.37 2004/06/10 22:40:28 hodson Exp $"
+#ident "$XORP: xorp/bgp/bgp.cc,v 1.50 2005/03/25 02:52:38 pavlin Exp $"
 
 // #define DEBUG_MAXIMUM_DELAY
 // #define DEBUG_LOGGING
@@ -30,6 +31,7 @@
 #include "path_attribute.hh"
 #include "iptuple.hh"
 #include "xrl_target.hh"
+#include "profile_vars.hh"
 
 // ----------------------------------------------------------------------------
 // Static class members
@@ -53,7 +55,7 @@ BGPMain::BGPMain()
 
     wait_until_xrl_router_is_ready(eventloop(), *_xrl_router);
 
-    _rib_ipc_handler = new RibIpcHandler(_xrl_router, *this);
+    _rib_ipc_handler = new RibIpcHandler(*_xrl_router, *this);
     _next_hop_resolver_ipv4 = new NextHopResolver<IPv4>(_xrl_router,
 							eventloop(),
 							*this);
@@ -63,17 +65,23 @@ BGPMain::BGPMain()
     _plumbing_unicast = new BGPPlumbing(SAFI_UNICAST,
 					_rib_ipc_handler,
 					*_next_hop_resolver_ipv4,
-					*_next_hop_resolver_ipv6);
+					*_next_hop_resolver_ipv6,
+					_policy_filters,
+					*this);
     _plumbing_multicast = new BGPPlumbing(SAFI_MULTICAST,
 					  _rib_ipc_handler,
 					  *_next_hop_resolver_ipv4,
-					  *_next_hop_resolver_ipv6);
+					  *_next_hop_resolver_ipv6,
+					  _policy_filters,
+					  *this);
     _rib_ipc_handler->set_plumbing(_plumbing_unicast, _plumbing_multicast);
 
     _process_watch = new ProcessWatch(_xrl_router, eventloop(),
 				      bgp_mib_name().c_str(),
 				      ::callback(this,
 						 &BGPMain::terminate));
+
+    initialize_profiling_variables(_profile);
 }
 
 BGPMain::~BGPMain()
@@ -89,7 +97,9 @@ BGPMain::~BGPMain()
 
     debug_msg("-------------------------------------------\n");
     debug_msg("Waiting for all peers to go to idle\n");
-    while (_peerlist->not_all_idle() || _rib_ipc_handler->busy()) {
+    while (_peerlist->not_all_idle() || _rib_ipc_handler->busy() ||
+	   DeleteAllNodes<IPv4>::running() || 
+	   DeleteAllNodes<IPv6>::running()) {
 	eventloop().run();
     }
     /*
@@ -97,7 +107,7 @@ BGPMain::~BGPMain()
     */
     if (eventloop().timer_list_length() > 1)
 	XLOG_INFO("EVENT: timers %u",
-		  static_cast<uint32_t>(eventloop().timer_list_length()));
+		  XORP_UINT_CAST(eventloop().timer_list_length()));
 
     /*
     ** Force the table de-registration from the RIB. Otherwise the
@@ -256,16 +266,18 @@ BGPMain::local_config(const uint32_t& as, const IPv4& id)
 */
 void
 BGPMain::connect_attempt(int fd, SelectorMask m,
-			 struct in_addr laddr, uint16_t lport)
+			 string laddr, uint16_t lport)
 {
     if (SEL_RD != m) {
 	XLOG_WARNING("Unexpected SelectorMask %#x", m);
 	return;
     }
 
-    struct sockaddr_in cliaddr;
-    socklen_t clilen = sizeof(cliaddr);
-    int connfd = accept(fd,(struct sockaddr *) &cliaddr, &clilen);
+    char buf[Socket::SOCKET_BUFFER_SIZE];
+    struct sockaddr *cliaddr = reinterpret_cast<struct sockaddr *>(buf);
+    socklen_t clilen = sizeof(buf);
+
+    int connfd = accept(fd, cliaddr, &clilen);
 
     debug_msg("Incoming connection attempt %d\n", connfd);
 
@@ -274,6 +286,13 @@ BGPMain::connect_attempt(int fd, SelectorMask m,
 	return;
     }
 
+    // Extract the peers IP address.
+    char peername[1024];
+    int error;
+    if ((error = getnameinfo(cliaddr, clilen , peername, sizeof(peername),
+			     0, 0, NI_NUMERICHOST)))
+	XLOG_FATAL("getnameinfo() failed: %s", gai_strerror(error));
+
     /*
     ** Loop through our peer list and see if we are configured for
     ** this guy.
@@ -288,8 +307,8 @@ BGPMain::connect_attempt(int fd, SelectorMask m,
 	debug_msg("Trying: %s ", iptuple.str().c_str());
 
 	if (iptuple.get_local_port() == lport &&
-	   iptuple.get_local_addr().s_addr == laddr.s_addr &&
-	   iptuple.get_peer_addr().s_addr == cliaddr.sin_addr.s_addr) {
+	   iptuple.get_local_addr() == laddr &&
+	   iptuple.get_peer_addr() == peername) {
 	    debug_msg(" Succeeded\n");
 	    (*i)->connected(connfd);
 	    return;
@@ -297,7 +316,7 @@ BGPMain::connect_attempt(int fd, SelectorMask m,
 	debug_msg(" Failed\n");
     }
 
-    XLOG_WARNING("Connection by %s denied", inet_ntoa(cliaddr.sin_addr));
+    XLOG_WARNING("Connection by %s denied", peername);
     if (-1 == close(connfd))
 	XLOG_WARNING("Close failed %s", strerror(errno));
 
@@ -316,9 +335,6 @@ BGPMain::connect_attempt(int fd, SelectorMask m,
     ** drop the peering.
     */
 #if	0
-    string peername = "Peer-";
-    peername += inet_ntoa(cliaddr.sin_addr);
-
     // create new peer
     debug_msg("Creating new peer after socket connection\n");
     BGPPeer *peer = new BGPPeer(_localdata, new BGPPeerData, this, connfd);
@@ -387,7 +403,9 @@ BGPMain::create_peer(BGPPeerData *pd)
 	return false;
     }
 
-    SocketClient *sock = new SocketClient(pd->iptuple(), eventloop());
+    bool md5sig = !pd->get_md5_password().empty();
+
+    SocketClient *sock = new SocketClient(pd->iptuple(), eventloop(), md5sig);
 
     BGPPeer *p = new BGPPeer(&_local_data, pd, sock, this);
     //    sock->set_eventloop(eventloop());
@@ -452,6 +470,34 @@ BGPMain::disable_peer(const Iptuple& iptuple)
     return true;
 }
 
+bool
+BGPMain::set_nexthop6(const Iptuple& iptuple, const IPv6& next_hop)
+{
+    BGPPeer *peer = find_peer(iptuple);
+
+    if (peer == 0) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
+	return false;
+    }
+    const_cast<BGPPeerData*>(peer->peerdata())->set_v6_local_addr(next_hop);
+
+    return true;
+}
+
+bool
+BGPMain::get_nexthop6(const Iptuple& iptuple, IPv6& next_hop)
+{
+    BGPPeer *peer = find_peer(iptuple);
+
+    if (peer == 0) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
+	return false;
+    }
+    next_hop = peer->peerdata()->get_v6_local_addr();
+
+    return true;
+}
+
 bool
 BGPMain::set_peer_state(const Iptuple& iptuple, bool state)
 {
@@ -470,6 +516,24 @@ BGPMain::set_peer_state(const Iptuple& iptuple, bool state)
     return activate(iptuple);
 }
 
+bool
+BGPMain::set_peer_md5_password(const Iptuple& iptuple, const string& password)
+{
+    BGPPeer *peer = find_peer(iptuple);
+
+    if (peer == NULL) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
+	return false;
+    }
+
+    // The md5-password property has to belong to BGPPeerData, because
+    // it is instantiated before BGPPeer and before SocketClient.
+    BGPPeerData* pd = const_cast<BGPPeerData*>(peer->peerdata());
+    pd->set_md5_password(password);
+
+    return true;
+}
+
 bool
 BGPMain::activate(const Iptuple& iptuple)
 {
@@ -511,16 +575,18 @@ BGPMain::next_hop_rewrite_filter(const Iptuple& iptuple, const IPv4& next_hop)
 }
 
 bool
-BGPMain::get_peer_list_start(uint32_t& token) {
+BGPMain::get_peer_list_start(uint32_t& token)
+{
     return _peerlist->get_peer_list_start(token);
 }
 
 bool
 BGPMain::get_peer_list_next(const uint32_t& token,
-			IPv4& local_ip,
+			string& local_ip,
 			uint32_t& local_port,
-			IPv4& peer_ip,
-			uint32_t& peer_port) {
+			string& peer_ip,
+			uint32_t& peer_port)
+{
     return _peerlist->get_peer_list_next(token, local_ip, local_port,
 					 peer_ip, peer_port);
 }
@@ -529,8 +595,12 @@ bool
 BGPMain::get_peer_id(const Iptuple& iptuple, IPv4& peer_id)
 {
     BGPPeer *peer = find_peer(iptuple);
-    if (peer == NULL)
+
+    if (0 == peer) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
 	return false;
+    }
+
     peer_id = peer->peerdata()->id();
     return true;
 }
@@ -541,8 +611,12 @@ BGPMain::get_peer_status(const Iptuple& iptuple,
 			 uint32_t& admin_status)
 {
     BGPPeer *peer = find_peer(iptuple);
-    if (peer == NULL)
+
+    if (0 == peer) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
 	return false;
+    }
+
     peer_state = peer->state();
     //XXX state stopped is only for our internal use.
     if (peer_state == STATESTOPPED)
@@ -557,8 +631,11 @@ BGPMain::get_peer_negotiated_version(const Iptuple& iptuple,
 				     int32_t& neg_version)
 {
     BGPPeer *peer = find_peer(iptuple);
-    if (peer == NULL)
+
+    if (0 == peer) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
 	return false;
+    }
 
     if (peer->state() == STATEESTABLISHED)
 	neg_version = 4; /* we only support BGP 4 */
@@ -573,8 +650,12 @@ bool
 BGPMain::get_peer_as(const Iptuple& iptuple, uint32_t& peer_as)
 {
     BGPPeer *peer = find_peer(iptuple);
-    if (peer == NULL)
+
+    if (0 == peer) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
 	return false;
+    }
+
     const BGPPeerData* pd = peer->peerdata();
 
     // XXX is it appropriate to return an extended AS number in this
@@ -594,8 +675,12 @@ BGPMain::get_peer_msg_stats(const Iptuple& iptuple,
 			    uint32_t& in_update_elapsed)
 {
     BGPPeer *peer = find_peer(iptuple);
-    if (peer == NULL)
+
+    if (0 == peer) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
 	return false;
+    }
+
     peer->get_msg_stats(in_updates, out_updates, in_msgs, out_msgs,
 			last_error, in_update_elapsed);
     return true;
@@ -607,8 +692,12 @@ BGPMain::get_peer_established_stats(const Iptuple& iptuple,
 				    uint32_t& established_time)
 {
     BGPPeer *peer = find_peer(iptuple);
-    if (peer == NULL)
+
+    if (0 == peer) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
 	return false;
+    }
+
     transitions = peer->get_established_transitions();
     established_time = peer->get_established_time();
     return true;
@@ -625,8 +714,12 @@ BGPMain::get_peer_timer_config(const Iptuple& iptuple,
 			       uint32_t& min_route_adv_interval)
 {
     BGPPeer *peer = find_peer(iptuple);
-    if (peer == NULL)
+
+    if (0 == peer) {
+	XLOG_WARNING("Could not find peer: %s", iptuple.str().c_str());
 	return false;
+    }
+
     const BGPPeerData* pd = peer->peerdata();
     retry_interval = (pd->get_retry_duration() / 1000);
     hold_time = (pd->get_hold_duration() / 1000);
@@ -690,8 +783,8 @@ BGPMain::start_server(const Iptuple& iptuple)
 	for (j = i->_tuples.begin(); j != i->_tuples.end(); j++) {
 	    if (*j == iptuple)	// This tuple already in list.
 		return;
-	    if ((j->get_local_addr().s_addr ==
-		iptuple.get_local_addr().s_addr &&
+	    if ((j->get_local_addr() ==
+		iptuple.get_local_addr() &&
 		j->get_local_port() == iptuple.get_local_port()))
 		match = true;
 	}
@@ -744,17 +837,21 @@ BGPMain::stop_server(const Iptuple& iptuple)
 void
 BGPMain::stop_all_servers()
 {
+    debug_msg("stop_all_servers\n");
+
     list<Server>::iterator i;
-    for (i = _serverfds.begin(); i != _serverfds.end(); i++) {
+    for (i = _serverfds.begin(); i != _serverfds.end();) {
+	debug_msg("%s\n", i->str().c_str());
 	eventloop().remove_selector(i->_serverfd);
 	::close(i->_serverfd);
-	i = _serverfds.erase(i);
+	_serverfds.erase(i++);
     }
 }
 
 bool
 BGPMain::originate_route(const IPv4Net& nlri, const IPv4& next_hop,
-			  const bool& unicast, const bool& multicast)
+			  const bool& unicast, const bool& multicast,
+			  const PolicyTags& policytags)
 {
     debug_msg("nlri %s next hop %s unicast %d multicast %d\n",
 	      nlri.str().c_str(), next_hop.str().c_str(), unicast, multicast);
@@ -762,12 +859,14 @@ BGPMain::originate_route(const IPv4Net& nlri, const IPv4& next_hop,
     AsPath aspath;
 
     return _rib_ipc_handler->originate_route(INCOMPLETE, aspath, nlri,
-					     next_hop, unicast, multicast);
+					     next_hop, unicast,
+					     multicast, policytags);
 }
 
 bool
 BGPMain::originate_route(const IPv6Net& nlri, const IPv6& next_hop,
-			  const bool& unicast, const bool& multicast)
+			  const bool& unicast, const bool& multicast,
+			  const PolicyTags& policytags)
 {
     debug_msg("nlri %s next hop %s unicast %d multicast %d\n",
 	      nlri.str().c_str(), next_hop.str().c_str(), unicast, multicast);
@@ -775,7 +874,8 @@ BGPMain::originate_route(const IPv6Net& nlri, const IPv6& next_hop,
     AsPath aspath;
 
     return _rib_ipc_handler->originate_route(INCOMPLETE, aspath, nlri,
-					      next_hop, unicast, multicast);
+					      next_hop, unicast,
+					      multicast, policytags);
 }
 
 bool
@@ -820,8 +920,9 @@ BGPMain::rib_client_route_info_changed4(const IPv4& addr,
 					const uint32_t& metric)
 {
     debug_msg("rib_client_route_info_changed4:"
-	      " addr %s prefix_len %d nexthop %s metric %d\n",
-	      addr.str().c_str(), prefix_len, nexthop.str().c_str(), metric);
+	      " addr %s prefix_len %u nexthop %s metric %u\n",
+	      addr.str().c_str(), XORP_UINT_CAST(prefix_len),
+	      nexthop.str().c_str(), XORP_UINT_CAST(metric));
 
     return plumbing_unicast()->plumbing_ipv4().
 	next_hop_resolver().rib_client_route_info_changed(addr, prefix_len,
@@ -835,8 +936,9 @@ BGPMain::rib_client_route_info_changed6(const IPv6& addr,
 					const uint32_t& metric)
 {
     debug_msg("rib_client_route_info_changed6:"
-	      " addr %s prefix_len %d nexthop %s metric %d\n",
-	      addr.str().c_str(), prefix_len, nexthop.str().c_str(), metric);
+	      " addr %s prefix_len %u nexthop %s metric %u\n",
+	      addr.str().c_str(), XORP_UINT_CAST(prefix_len),
+	      nexthop.str().c_str(), XORP_UINT_CAST(metric));
 
     return plumbing_unicast()->plumbing_ipv6().
 	next_hop_resolver().rib_client_route_info_changed(addr, prefix_len,
@@ -848,7 +950,8 @@ BGPMain::rib_client_route_info_invalid4(const IPv4& addr,
 					const uint32_t& prefix_len)
 {
     debug_msg("rib_client_route_info_invalid4:"
-	      " addr %s prefix_len %d\n", addr.str().c_str(), prefix_len);
+	      " addr %s prefix_len %u\n", addr.str().c_str(),
+	      XORP_UINT_CAST(prefix_len));
 
     return plumbing_unicast()->plumbing_ipv4().
 	next_hop_resolver().rib_client_route_info_invalid(addr, prefix_len);
@@ -859,14 +962,16 @@ BGPMain::rib_client_route_info_invalid6(const IPv6& addr,
 					const uint32_t& prefix_len)
 {
     debug_msg("rib_client_route_info_invalid6:"
-	      " addr %s prefix_len %d\n", addr.str().c_str(), prefix_len);
+	      " addr %s prefix_len %u\n", addr.str().c_str(),
+	      XORP_UINT_CAST(prefix_len));
 
     return plumbing_unicast()->plumbing_ipv6().
 	next_hop_resolver().rib_client_route_info_invalid(addr, prefix_len);
 }
 
 bool
-BGPMain::set_parameter(const Iptuple& iptuple , const string& parameter)
+BGPMain::set_parameter(const Iptuple& iptuple , const string& parameter,
+		       const bool toggle)
 {
     BGPPeer *peer;
     // BGPPeerData *peerdata;
@@ -876,6 +981,7 @@ BGPMain::set_parameter(const Iptuple& iptuple , const string& parameter)
 	return false;
     }
 
+    ParameterNode node;
     if (strcmp(parameter.c_str(),"Refresh_Capability") == 0) {
 	/*
 	** This is a place holder and example as to how to set
@@ -887,32 +993,48 @@ BGPMain::set_parameter(const Iptuple& iptuple , const string& parameter)
 // 		  iptuple.str().c_str());
 // 	peerdata = const_cast<BGPPeerData *const>(peer->peerdata());
 // 	peerdata->add_sent_parameter( new BGPRefreshCapability() );
+    } else if (strcmp(parameter.c_str(),"MultiProtocol.IPv4.Unicast") == 0) {
+ 	debug_msg("IPv4 Unicast\n");
+	node = new BGPMultiProtocolCapability(AFI_IPV4, SAFI_UNICAST);
     } else if (strcmp(parameter.c_str(),"MultiProtocol.IPv4.Multicast") == 0) {
- 	debug_msg("Setting Multiprotocol IPv4 Multicast for peer %s.",
-		  iptuple.str().c_str());
- 	BGPPeerData *peerdata =
-	    const_cast<BGPPeerData *const>(peer->peerdata());
- 	peerdata->add_sent_parameter(
-	     new BGPMultiProtocolCapability(AFI_IPV4, SAFI_MULTICAST));
+ 	debug_msg("IPv4 Multicast\n");
+	node = new BGPMultiProtocolCapability(AFI_IPV4, SAFI_MULTICAST);
     } else if (strcmp(parameter.c_str(),"MultiProtocol.IPv6.Unicast") == 0) {
- 	debug_msg("Setting Multiprotocol IPv6 Unicast for peer %s.",
-		  iptuple.str().c_str());
- 	BGPPeerData *peerdata =
-	    const_cast<BGPPeerData *const>(peer->peerdata());
- 	peerdata->add_sent_parameter(
-	     new BGPMultiProtocolCapability(AFI_IPV6, SAFI_UNICAST));
+ 	debug_msg("IPv6 Unicast\n");
+	node = new BGPMultiProtocolCapability(AFI_IPV6, SAFI_UNICAST);
     } else if (strcmp(parameter.c_str(),"MultiProtocol.IPv6.Multicast") == 0) {
- 	debug_msg("Setting Multiprotocol IPv6 Multicast for peer %s.",
-		  iptuple.str().c_str());
- 	BGPPeerData *peerdata =
-	    const_cast<BGPPeerData *const>(peer->peerdata());
- 	peerdata->add_sent_parameter(
-	     new BGPMultiProtocolCapability(AFI_IPV6, SAFI_MULTICAST));
+ 	debug_msg("IPv6 Multicast\n");
+	node = new BGPMultiProtocolCapability(AFI_IPV6, SAFI_MULTICAST);
     } else {
 	XLOG_WARNING("Unable to set unknown parameter: <%s>.",
 		     parameter.c_str());
 	return false;
     }
+    
+    debug_msg("Peer %s. %s\n", iptuple.str().c_str(),toggle ? "true" :"false");
 
+    BGPPeerData *peerdata = const_cast<BGPPeerData *const>(peer->peerdata());
+    if (toggle) {
+	peerdata->add_sent_parameter(node);
+    } else {
+	peerdata->remove_sent_parameter(node);
+    }
     return true;
 }
+
+void
+BGPMain::configure_filter(const uint32_t& filter, const string& conf) {
+    _policy_filters.configure(filter,conf);
+}
+
+void
+BGPMain::reset_filter(const uint32_t& filter) {
+    _policy_filters.reset(filter);
+}
+
+void
+BGPMain::push_routes() {
+    _plumbing_unicast->push_routes();
+    _plumbing_multicast->push_routes();
+}
+
