@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/dummy_main.cc,v 1.8 2004/06/10 22:40:29 hodson Exp $"
+#ident "$XORP: xorp/bgp/dummy_main.cc,v 1.10 2005/03/25 02:52:39 pavlin Exp $"
 
 #include <sys/time.h>
 
@@ -59,7 +59,7 @@ BGPMain::local_config(const uint32_t&, const IPv4&)
 */
 void 
 BGPMain::connect_attempt(int, SelectorMask,
-			 struct in_addr, uint16_t)
+			 string, uint16_t)
 {
 }
 
