@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/test_peer.cc,v 1.26 2004/06/10 22:40:40 hodson Exp $"
+#ident "$XORP: xorp/bgp/harness/test_peer.cc,v 1.31 2005/03/25 02:52:54 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -97,7 +97,8 @@ XrlTestPeerTarget::test_peer_0_1_register(const string& coordinator,
     debug_msg("\n");
 
     if(_trace)
-	printf("register(%s,%u)\n", coordinator.c_str(), genid);
+	printf("register(%s,%u)\n", coordinator.c_str(),
+	       XORP_UINT_CAST(genid));
 
     _test_peer.register_coordinator(coordinator);
     _test_peer.register_genid(genid);
@@ -127,7 +128,7 @@ XrlTestPeerTarget::test_peer_0_1_connect(const string&	host,
     debug_msg("\n");
 
     if(_trace)
-	printf("connect(%s,%u)\n", host.c_str(), port);
+	printf("connect(%s,%u)\n", host.c_str(), XORP_UINT_CAST(port));
 
     string error_string;
     if(!_test_peer.connect(host, port, error_string)) {
@@ -144,7 +145,7 @@ XrlTestPeerTarget::test_peer_0_1_listen(const string& address,
     debug_msg("\n");
 
     if(_trace)
-	printf("listen(%s,%u)\n", address.c_str(), port);
+	printf("listen(%s,%u)\n", address.c_str(), XORP_UINT_CAST(port));
 
     string error_string;
     if(!_test_peer.listen(address, port, error_string)) {
@@ -161,7 +162,7 @@ XrlTestPeerTarget::test_peer_0_1_bind(const string& address,
     debug_msg("\n");
 
     if(_trace)
-	printf("bind(%s,%u)\n", address.c_str(), port);
+	printf("bind(%s,%u)\n", address.c_str(), XORP_UINT_CAST(port));
 
     string error_string;
     if(!_test_peer.bind(address, port, error_string)) {
@@ -271,7 +272,7 @@ TestPeer::register_coordinator(const string& name)
 void
 TestPeer::register_genid(const uint32_t& genid)
 {
-    debug_msg("Test Peer: genid = %u\n", genid);
+    debug_msg("Test Peer: genid = %u\n", XORP_UINT_CAST(genid));
 
     _genid = genid;
 }
@@ -314,25 +315,23 @@ TestPeer::connect(const string& host, const uint32_t& port,
 	return false;
     }
 
-    int s = ::socket(PF_INET, SOCK_STREAM, 0);
+    char buf[Socket::SOCKET_BUFFER_SIZE];
+    struct sockaddr *peer = reinterpret_cast<struct sockaddr *>(buf);
+    size_t len = sizeof(buf);
+    try {
+	Socket::init_sockaddr(host, port, peer, len);
+    } catch(UnresolvableHost e) {
+	error_string = e.why();
+	return false;
+    }
+
+    int s = ::socket(peer->sa_family, SOCK_STREAM, 0);
     if(-1 == s) {
 	error_string = c_format("socket call failed: %s", strerror(errno));
 	return false;
     }
 
-    struct sockaddr_in peer;
-    try {
-	Socket::init_sockaddr(&peer,
-			      Iptuple::get_addr(host.c_str()),
-			      htons(port));
-    } catch(UnresolvableHost e) {
-	::close(s);
-	error_string = e.why();
-	return false;
-    }
-
-    if(-1 == ::connect(s, reinterpret_cast<struct sockaddr *>(&peer),
-		       sizeof(peer))) {
+    if(-1 == ::connect(s, peer, len)) {
 	::close(s);
 	return false;
     }
@@ -383,20 +382,20 @@ TestPeer::listen(const string& host, const uint32_t& port,
 	return false;
     }
 
-    int s = ::socket(PF_INET, SOCK_STREAM, 0);
-    if(-1 == s) {
-	error_string = c_format("socket call failed: %s", strerror(errno));
+
+    char buf[Socket::SOCKET_BUFFER_SIZE];
+    struct sockaddr *local = reinterpret_cast<struct sockaddr *>(buf);
+    size_t len = sizeof(buf);
+    try {
+	Socket::init_sockaddr(host, port, local, len);
+    } catch(UnresolvableHost e) {
+	error_string = e.why();
 	return false;
     }
 
-    struct sockaddr_in local;
-    try {
-	Socket::init_sockaddr(&local,
-			      Iptuple::get_addr(host.c_str()),
-			      htons(port));
-    } catch(UnresolvableHost e) {
-	::close(s);
-	error_string = e.why();
+    int s = ::socket(local->sa_family, SOCK_STREAM, 0);
+    if(-1 == s) {
+	error_string = c_format("socket call failed: %s", strerror(errno));
 	return false;
     }
 
@@ -406,8 +405,7 @@ TestPeer::listen(const string& host, const uint32_t& port,
 	return false;
     }
 
-    if(-1 == ::bind(s, reinterpret_cast<struct sockaddr *>(&local),
-		  sizeof(local))) {
+    if(-1 == ::bind(s, local, len)) {
 	::close(s);
 	error_string = c_format("Bind failed: %s", strerror(errno));
 	return false;
@@ -452,20 +450,19 @@ TestPeer::bind(const string& host, const uint32_t& port,
 	return false;
     }
 
-    int s = ::socket(PF_INET, SOCK_STREAM, 0);
-    if(-1 == s) {
-	error_string = c_format("socket call failed: %s", strerror(errno));
+    char buf[Socket::SOCKET_BUFFER_SIZE];
+    struct sockaddr *local = reinterpret_cast<struct sockaddr *>(buf);
+    size_t len = sizeof(buf);
+    try {
+	Socket::init_sockaddr(host, port, local, len);
+    } catch(UnresolvableHost e) {
+	error_string = e.why();
 	return false;
     }
 
-    struct sockaddr_in local;
-    try {
-	Socket::init_sockaddr(&local,
-			      Iptuple::get_addr(host.c_str()),
-			      htons(port));
-    } catch(UnresolvableHost e) {
-	::close(s);
-	error_string = e.why();
+    int s = ::socket(local->sa_family, SOCK_STREAM, 0);
+    if(-1 == s) {
+	error_string = c_format("socket call failed: %s", strerror(errno));
 	return false;
     }
 
@@ -475,8 +472,7 @@ TestPeer::bind(const string& host, const uint32_t& port,
 	return false;
     }
 
-    if(-1 == ::bind(s, reinterpret_cast<struct sockaddr *>(&local),
-		  sizeof(local))) {
+    if(-1 == ::bind(s, local, len)) {
 	::close(s);
 	error_string = c_format("Bind failed: %s", strerror(errno));
 	return false;
@@ -490,7 +486,7 @@ TestPeer::bind(const string& host, const uint32_t& port,
 bool
 TestPeer::send(const vector<uint8_t>& data, string& error_string)
 {
-    debug_msg("len: %u\n", (uint32_t)data.size());
+    debug_msg("len: %u\n", XORP_UINT_CAST(data.size()));
     if(UNCONNECTED == _s) {
 	XLOG_WARNING("Not connected");
 	error_string = "Not connected";
@@ -524,7 +520,7 @@ TestPeer::send_complete(AsyncFileWriter::Event ev, const uint8_t *buf,
 	break;
     case AsyncFileOperator::FLUSHING:
 	debug_msg("event: flushing\n");
-	debug_msg("Freeing Buffer for sent packet: %x\n", (uint)buf);
+	debug_msg("Freeing Buffer for sent packet: %p\n", buf);
 	delete[] buf;
 	break;
     case AsyncFileOperator::ERROR_CHECK_ERRNO:
@@ -752,7 +748,7 @@ TestPeer::receive(int fd, SelectorMask m)
 void
 TestPeer::datain(status st, uint8_t *ptr, size_t len, string error)
 {
-    debug_msg("status = %d len = %u error = %s\n", st, (uint32_t)len,
+    debug_msg("status = %d len = %u error = %s\n", st, XORP_UINT_CAST(len),
 	      error.c_str());
 
     if(_verbose) {
@@ -815,7 +811,7 @@ TestPeer::sendit()
 
     XrlDatainV0p1Client datain(&_xrlrouter);
 
-    debug_msg("%u\n", (uint32_t)q.v.size());
+    debug_msg("%u\n", XORP_UINT_CAST(q.v.size()));
     XLOG_ASSERT(q.len == q.v.size());
 
     _flying++;
@@ -877,7 +873,7 @@ main(int argc, char **argv)
     xlog_start();
 
     int c;
-    string finder_host = FINDER_DEFAULT_HOST.str();
+    string finder_host = FinderConstants::FINDER_DEFAULT_HOST().str();
     const char *server = SERVER;
     bool verbose = false;
     bool trace = false;
