@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_peering1.sh,v 1.26 2004/06/08 00:21:36 atanu Exp $
+# $XORP: xorp/bgp/harness/test_peering1.sh,v 1.30 2005/03/09 02:48:01 atanu Exp $
 #
 
 #
@@ -95,14 +95,14 @@ configure_bgp()
     PEER=$PEER3
     NEXT_HOP=192.150.187.78
     add_peer $LOCALHOST $PORT3 $PEER $PEER3_PORT $PEER3_AS $NEXT_HOP $HOLDTIME
-    set_parameter $LOCALHOST $PORT3 $PEER $PEER3_PORT MultiProtocol.IPv6.Unicast
+    set_parameter $LOCALHOST $PORT3 $PEER $PEER3_PORT MultiProtocol.IPv6.Unicast true
     enable_peer $LOCALHOST $PORT3 $PEER $PEER3_PORT
 
     # IBGP - IPV6
     PEER=$PEER4
     NEXT_HOP=192.150.187.78
     add_peer $LOCALHOST $PORT4 $PEER $PEER4_PORT $PEER4_AS $NEXT_HOP $HOLDTIME
-    set_parameter $LOCALHOST $PORT4 $PEER $PEER4_PORT MultiProtocol.IPv6.Unicast
+    set_parameter $LOCALHOST $PORT4 $PEER $PEER4_PORT MultiProtocol.IPv6.Unicast true
     enable_peer $LOCALHOST $PORT4 $PEER $PEER4_PORT
 }
 
@@ -226,10 +226,8 @@ test5()
     disable_peer $LOCALHOST $PORT1 $PEER1 $PEER1_PORT $PEER1_AS
     enable_peer $LOCALHOST $PORT1 $PEER1 $PEER1_PORT $PEER1_AS
 
-    echo "Sleeping $HOLDTIME"
-    sleep $HOLDTIME
-
-    coord peer1 expect packet notify $HOLD_TIMER
+    sleep 2
+    coord peer1 assert established
 }
 
 test6()
@@ -923,7 +921,7 @@ test27_ipv6()
     echo "TEST27 (IPV6) - Verify that routes originated by BGP reach an IBGP peer"
 
     coord reset
-    coord target $HOST $PORT1
+    coord target $HOST $PORT3
     coord initialise attach peer1
 
     # Introduce a route
@@ -941,11 +939,23 @@ test27_ipv6()
 test28()
 {
     echo "TEST28 - Verify that routes originated by BGP reach an EBGP peer"
+    echo "Also verify that the nexthop in the originate route is honoured"
 
     coord reset
     coord target $HOST $PORT2
     coord initialise attach peer1
 
+    coord peer1 expect packet open asnum $AS bgpid $ID holdtime $HOLDTIME
+
+    coord peer1 expect packet keepalive
+
+    coord peer1 expect packet update \
+	origin 2 \
+	nexthop 20.20.20.20 \
+	aspath 65008 \
+	med 1 \
+	nlri 10.10.10.0/24
+
     # Introduce a route
     originate_route4 10.10.10.0/24 20.20.20.20 true false
 
@@ -961,11 +971,24 @@ test28()
 test28_ipv6()
 {
     echo "TEST28 (IPV6) - Verify that routes originated by BGP reach an EBGP peer"
+    echo "Also verify that the nexthop in the originate route is honoured"
 
     coord reset
-    coord target $HOST $PORT2
+    coord target $HOST $PORT4
     coord initialise attach peer1
 
+    coord peer1 expect packet open asnum $AS bgpid $ID holdtime $HOLDTIME \
+	afi 2 safi 1
+
+    coord peer1 expect packet keepalive
+
+    coord peer1 expect packet update \
+	origin 2 \
+	aspath 65008 \
+	med 1 \
+	nlri6 2000::/3 \
+	nexthop6 20:20:20:20:20:20:20:20
+
     # Introduce a route
     originate_route6 2000::/3 20:20:20:20:20:20:20:20 true false
 
@@ -1029,7 +1052,7 @@ test30()
     enable_peer $LOCALHOST $PORT1 $PEER1 $PEER1_PORT $PEER1_AS
 
     # Need to find out that the BGP process still exists so make a peering.
-    BASE=peer1 ./xrl_shell_funcs.sh reset
+    BASE=peer1 ${srcdir}/xrl_shell_funcs.sh reset
     sleep 2
 
     coord reset
