@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_routing1.sh,v 1.15 2003/11/05 06:32:44 atanu Exp $
+# $XORP: xorp/bgp/harness/test_routing1.sh,v 1.16 2004/12/17 01:31:02 atanu Exp $
 #
 
 #
@@ -119,7 +119,7 @@ configure_bgp()
     PORT=$PORT1_IPV6;PEER_PORT=$PEER_PORT1_IPV6;PEER_AS=$PEER1_AS_IPV6
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
-    set_parameter $IPTUPLE MultiProtocol.IPv6.Unicast
+    set_parameter $IPTUPLE MultiProtocol.IPv6.Unicast true
     enable_peer $IPTUPLE
 
     # EBGP - IPV6
@@ -127,7 +127,7 @@ configure_bgp()
     PORT=$PORT2_IPV6;PEER_PORT=$PEER_PORT2_IPV6;PEER_AS=$PEER2_AS_IPV6
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
-    set_parameter $IPTUPLE MultiProtocol.IPv6.Unicast
+    set_parameter $IPTUPLE MultiProtocol.IPv6.Unicast true
     enable_peer $IPTUPLE
 
     # IBGP - IPV6
@@ -135,7 +135,7 @@ configure_bgp()
     PORT=$PORT3_IPV6;PEER_PORT=$PEER_PORT3_IPV6;PEER_AS=$PEER3_AS_IPV6
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE  $PEER_AS $NEXT_HOP $HOLDTIME
-    set_parameter $IPTUPLE MultiProtocol.IPv6.Unicast
+    set_parameter $IPTUPLE MultiProtocol.IPv6.Unicast true
     enable_peer $IPTUPLE
 }
 
