@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/internal_message.cc,v 1.6 2004/06/10 22:40:30 hodson Exp $"
+#ident "$XORP: xorp/bgp/internal_message.cc,v 1.8 2005/03/25 02:52:40 pavlin Exp $"
 
 #include <string>
 #include "bgp_module.h"
@@ -53,7 +53,7 @@ string
 InternalMessage<A>::str() const
 {
     string s;
-    s += c_format("GenID is %d\n", _genid);
+    s += c_format("GenID is %d\n", XORP_INT_CAST(_genid));
     if (_changed)
 	s += "CHANGED flag is set\n";
     if (_push)
