@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/next_hop_resolver.cc,v 1.31 2004/06/10 22:40:30 hodson Exp $"
+#ident "$XORP: xorp/bgp/next_hop_resolver.cc,v 1.41 2005/03/25 02:52:41 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -30,6 +30,7 @@
 #include "next_hop_resolver.hh"
 #include "route_table_nhlookup.hh"
 #include "route_table_decision.hh"
+#include "profile_vars.hh"
 
 template <class A>
 NextHopResolver<A>::NextHopResolver(XrlStdRouter *xrl_router,
@@ -179,8 +180,13 @@ NextHopResolver<A>::rib_client_route_info_changed(const A& addr,
 						  const A& nexthop,
 						  const uint32_t& metric)
 {
-    debug_msg("addr %s prefix_len %d nexthop %s metric %d\n",
-	      addr.str().c_str(), real_prefix_len, nexthop.str().c_str(), metric);
+    debug_msg("addr %s prefix_len %u nexthop %s metric %u\n",
+	      addr.str().c_str(), XORP_UINT_CAST(real_prefix_len),
+	      nexthop.str().c_str(), XORP_UINT_CAST(metric));
+    XLOG_TRACE(_bgp.profile().enabled(trace_nexthop_resolution),
+	       "addr %s prefix_len %u nexthop %s metric %u\n",
+	       addr.str().c_str(), XORP_UINT_CAST(real_prefix_len),
+	       nexthop.str().c_str(), XORP_UINT_CAST(metric));
 
     UNUSED(nexthop);
 
@@ -201,7 +207,11 @@ bool
 NextHopResolver<A>::rib_client_route_info_invalid(const A& addr,
 						  const uint32_t& prefix_len)
 {
-    debug_msg("addr %s prefix_len %d\n", addr.str().c_str(), prefix_len);
+    debug_msg("addr %s prefix_len %u\n", addr.str().c_str(),
+	      XORP_UINT_CAST(prefix_len));
+    XLOG_TRACE(_bgp.profile().enabled(trace_nexthop_resolution),
+	       "addr %s prefix_len %u\n", addr.str().c_str(),
+	       XORP_UINT_CAST(prefix_len));
 
     /*
     ** This is a callback from the RIB telling us that this
@@ -224,8 +234,8 @@ NextHopResolver<A>::rib_client_route_info_invalid(const A& addr,
 	if (_next_hop_rib_request.premature_invalid(addr, prefix_len)) {
 	    return true;
 	}
-	XLOG_WARNING("address not found in next hop cache: %s/%d",
-		   addr.str().c_str(), prefix_len);
+	XLOG_WARNING("address not found in next hop cache: %s/%u",
+		   addr.str().c_str(), XORP_UINT_CAST(prefix_len));
 	return false;
     }
 
@@ -682,9 +692,24 @@ NextHopRibRequest<A>::register_nexthop(A nexthop, IPNet<A> net_from_route,
 
 template<class A>
 void
-NextHopRibRequest<A>::deregister_from_rib(const A& base_addr, 
+NextHopRibRequest<A>::deregister_from_rib(const A& base_addr,
 					  uint32_t prefix_len)
 {
+    /*
+    ** Its possible that we are already trying to deregister for this
+    ** addr/prefix. In which case just return.
+    */
+    typename list<RibRequestQueueEntry<A> *>::iterator i = _queue.begin();
+    for ( ;i != _queue.end(); i++) {
+	RibDeregisterQueueEntry<A> *dreg = 
+	    dynamic_cast<RibDeregisterQueueEntry<A> *>(*i);
+	if (dreg) {
+	    if (dreg->base_addr() == base_addr && 
+		dreg->prefix_len() == prefix_len)
+		return;
+	}
+    }
+
     /*
     ** Construct a request.
     */
@@ -710,7 +735,9 @@ void
 NextHopRibRequest<IPv4>::register_interest(IPv4 nexthop)
 {
     debug_msg("nexthop %s\n", nexthop.str().c_str());
-    if(0 == _xrl_router)	// The test code sets _xrl_router to zero
+    XLOG_TRACE(_bgp.profile().enabled(trace_nexthop_resolution),
+	       "nexthop %s\n", nexthop.str().c_str());
+    if (0 == _xrl_router)	// The test code sets _xrl_router to zero
 	return;
 
     XrlRibV0p1Client rib(_xrl_router);
@@ -727,7 +754,9 @@ void
 NextHopRibRequest<IPv6>::register_interest(IPv6 nexthop)
 {
     debug_msg("nexthop %s\n", nexthop.str().c_str());
-    if(0 == _xrl_router)	// The test code sets _xrl_router to zero
+    XLOG_TRACE(_bgp.profile().enabled(trace_nexthop_resolution),
+	       "nexthop %s\n", nexthop.str().c_str());
+    if (0 == _xrl_router)	// The test code sets _xrl_router to zero
 	return;
 
     XrlRibV0p1Client rib(_xrl_router);
@@ -783,11 +812,19 @@ NextHopRibRequest<A>::register_interest_response(const XrlError& error,
     }
 
     debug_msg("Error %s resolves %d addr %s "
-	      "prefix_len %d real prefix_len %d actual nexthop %s metric %d %s\n",
+	      "prefix_len %u real prefix_len %u actual nexthop %s metric %d %s\n",
  	      error.str().c_str(), *resolves, addr->str().c_str(),
-	      *prefix_len, *real_prefix_len, actual_nexthop->str().c_str(), *metric,
+	      XORP_UINT_CAST(*prefix_len), XORP_UINT_CAST(*real_prefix_len),
+	      actual_nexthop->str().c_str(), XORP_UINT_CAST(*metric),
 	      comment.c_str());
-
+    XLOG_TRACE(_bgp.profile().enabled(trace_nexthop_resolution),
+	       "Error %s resolves %d addr %s "
+	      "prefix_len %u real prefix_len %u "
+	       "actual nexthop %s metric %d %s\n",
+	       error.str().c_str(), *resolves, addr->str().c_str(),
+	       XORP_UINT_CAST(*prefix_len), XORP_UINT_CAST(*real_prefix_len),
+	       actual_nexthop->str().c_str(), XORP_UINT_CAST(*metric),
+	       comment.c_str());
 
     XLOG_ASSERT(*real_prefix_len <= A::addr_bitlen());
 
@@ -865,16 +902,16 @@ NextHopRibRequest<A>::register_interest_response(const XrlError& error,
 								lookup_succeeded, m)) {
 		XLOG_ASSERT(rr->new_register() || rr->reregister());
 		/*
-		** See if this request was caused by a downcall from the next hop
-		** table.
-	    */
+		** See if this request was caused by a downcall from
+		** the next hop table.
+		*/
 		if (rr->new_register()) {
 		    NHRequest<A>* request_data = &rr->requests();
 		    /*
 		    ** If nobody is interested then don't register or run
 		    ** the callbacks.
 		    */
-		    if(0 != request_data->requests()) {
+		    if (0 != request_data->requests()) {
 			_next_hop_cache.register_nexthop(rr->nexthop(),
 							 request_data->requests());
 
@@ -890,10 +927,10 @@ NextHopRibRequest<A>::register_interest_response(const XrlError& error,
 		    }
 		}
 		/*
-	    ** See if this request was caused by an upcall from the
-	    ** RIB. If it was then notify decision that this next hop
-	    ** has changed.
-	    */
+		** See if this request was caused by an upcall from the
+		** RIB. If it was then notify decision that this next hop
+		** has changed.
+		*/
 		if (rr->reregister() && 0 != rr->ref_cnt()) {
 		    _next_hop_cache.register_nexthop(rr->nexthop(), rr->ref_cnt());
 		    /*
@@ -910,23 +947,8 @@ NextHopRibRequest<A>::register_interest_response(const XrlError& error,
 		break;
 	    }
 	} else {
-	    RibDeregisterQueueEntry<A> *rd
-		= dynamic_cast<RibDeregisterQueueEntry<A> *>(*i);
-	    assert(rd != NULL);
-	    if ((rd->base_addr() == *addr) 
-		&& (rd->prefix_len() == *prefix_len)) {
-		//We got an answer back, and there was a matching
-		//deregister in the queue.  Remove the deregister from
-		//the queue.  It's possible this was actually needed,
-		//but at this point we can't tell for sure.  Later
-		//we'll validate the entry, and back a deregister if
-		//it was needed.
-		delete rd;
-		i = _queue.erase(i);
-	    } else {
-		//skip the deregister
-		i++;
-	    }
+	    //skip the deregister
+	    i++;
 	}
     }
 
@@ -940,12 +962,9 @@ NextHopRibRequest<A>::register_interest_response(const XrlError& error,
     */
     if (!_next_hop_cache.validate_entry(*addr, first_nexthop, *prefix_len,
 					*real_prefix_len)) {
-	RibDeregisterQueueEntry<A> *rd
-	    = new RibDeregisterQueueEntry<A>(*addr, *prefix_len);
-	_queue.push_back(rd);
+	deregister_from_rib(*addr, *prefix_len);
     }
 
-
     /*
     ** There are entries left on the queue, so, fire off another request.
     */
@@ -985,13 +1004,64 @@ NextHopRibRequest<A>::premature_invalid(const A& addr,
     if (!_busy)
 	return false;
 
-    XLOG_ASSERT(_busy);
-    XLOG_ASSERT(!_invalid);
+    /*
+    ** An invalid has been received for an entry that we don't have in
+    ** our cache.
+    * 1) The current request may have generated an invalid. Extremely
+    * irritating, we make a request and before receiving the response
+    * we get an invalid from the RIB. In which case set the _invalid
+    * flag and _invalid_net and all should be well.
+    * 2) We receive an invalid for an entry that we are no longer
+    * interested in but the deregister is still in the request queue. In
+    * which case remove the deregister from the queue and continue.
+    */
 
-    _invalid = true;
-    _invalid_net = IPNet<A>(addr, prefix_len);
+    XLOG_ASSERT(!_queue.empty());
+    RibRegisterQueueEntry<A> *first_rr =
+	dynamic_cast<RibRegisterQueueEntry<A> *>(_queue.front());
+    if (first_rr && 
+	(IPNet<A>(addr, prefix_len) ==
+	 IPNet<A>(first_rr->nexthop(), prefix_len))) {
 
-    return true;
+	XLOG_ASSERT(_busy);
+	XLOG_ASSERT(!_invalid);
+
+	_invalid = true;
+	_invalid_net = IPNet<A>(addr, prefix_len);
+
+	return true;
+    }
+
+    typename list<RibRequestQueueEntry<A> *>::iterator i = _queue.begin();
+    for ( ;i != _queue.end(); i++) {
+	RibDeregisterQueueEntry<A> *dreg = 
+	    dynamic_cast<RibDeregisterQueueEntry<A> *>(*i);
+	if (dreg) {
+	    if (dreg->base_addr() == addr && 
+		dreg->prefix_len() == prefix_len) {
+		/*
+		** Don't erase the first entry in the queue this is an
+		** ongoing transaction.
+		*/
+		XLOG_INFO("invalid addr %s prefix len %u matched delete %s",
+			  cstring(addr), XORP_UINT_CAST(prefix_len),
+			  (i == _queue.begin()) ? "front" : "not front");
+		if (i == _queue.begin()) {
+		    XLOG_ASSERT(_busy);
+		    XLOG_ASSERT(!_invalid);
+
+		    _invalid = true;
+		    _invalid_net = IPNet<A>(addr, prefix_len);
+		} else {
+		    delete dreg;
+		    _queue.erase(i);
+		}
+		return true;
+	    }
+	}
+    }
+
+    return false;
 }
 
 template<class A>
@@ -1001,7 +1071,6 @@ NextHopRibRequest<A>::deregister_nexthop(A nexthop, IPNet<A> net_from_route,
 {
     debug_msg("nexthop %s net %s requested %p\n",
 	      nexthop.str().c_str(), net_from_route.str().c_str(), requester);
-
     typename list<RibRequestQueueEntry<A> *>::iterator i;
 
     /*
@@ -1026,9 +1095,9 @@ void
 NextHopRibRequest<A>::reregister_nexthop(A nexthop, uint32_t ref_cnt,
 					 bool resolvable, uint32_t metric)
 {
-    debug_msg(" nexthop %s ref_cnf %d resolvable %d metric %d\n",
-	      nexthop.str().c_str(), ref_cnt, resolvable, metric);
-
+    debug_msg(" nexthop %s ref_cnf %u resolvable %d metric %u\n",
+	      nexthop.str().c_str(), XORP_UINT_CAST(ref_cnt), resolvable,
+	      XORP_UINT_CAST(metric));
     /*
     ** It may be the case that we already have an entry that covers
     ** this next hop.
@@ -1101,8 +1170,9 @@ NextHopRibRequest<A>::lookup(const A& nexthop,
 	if (rr && rr->reregister() && rr->nexthop() == nexthop) {
 	    resolvable = rr->resolvable();
 	    metric = rr->metric();
-	    debug_msg("nexthop %s resolvable %d metric %d\n",
-		      nexthop.str().c_str(), resolvable, metric);
+	    debug_msg("nexthop %s resolvable %d metric %u\n",
+		      nexthop.str().c_str(), resolvable,
+		      XORP_UINT_CAST(metric));
 	    return true;
 	}
     }
@@ -1117,8 +1187,11 @@ void
 NextHopRibRequest<IPv4>::deregister_interest(IPv4 addr, 
 					     uint32_t prefix_len)
 {
-    debug_msg("addr %s/%d\n", addr.str().c_str(), prefix_len);
-    if(0 == _xrl_router)	// The test code sets _xrl_router to zero
+    debug_msg("addr %s/%u\n", addr.str().c_str(), XORP_UINT_CAST(prefix_len));
+    XLOG_TRACE(_bgp.profile().enabled(trace_nexthop_resolution),
+	       "addr %s/%u\n", addr.str().c_str(),
+	       XORP_UINT_CAST(prefix_len));	       
+    if (0 == _xrl_router)	// The test code sets _xrl_router to zero
 	return;
 
     XrlRibV0p1Client rib(_xrl_router);
@@ -1128,8 +1201,9 @@ NextHopRibRequest<IPv4>::deregister_interest(IPv4 addr,
 				  prefix_len,
 	    ::callback(this,&NextHopRibRequest::deregister_interest_response,
 		       addr, prefix_len,
-		       c_format("deregister_from_rib: addr %s/%d",
-				addr.str().c_str(), prefix_len)));
+		       c_format("deregister_from_rib: addr %s/%u",
+				addr.str().c_str(),
+				XORP_UINT_CAST(prefix_len))));
 }
 
 template<>
@@ -1137,8 +1211,11 @@ void
 NextHopRibRequest<IPv6>::deregister_interest(IPv6 addr, 
 					     uint32_t prefix_len)
 {
-    debug_msg("addr %s/%d\n", addr.str().c_str(), prefix_len);
-    if(0 == _xrl_router)	// The test code sets _xrl_router to zero
+    debug_msg("addr %s/%u\n", addr.str().c_str(), XORP_UINT_CAST(prefix_len));
+    XLOG_TRACE(_bgp.profile().enabled(trace_nexthop_resolution),
+	       "addr %s/%u\n", addr.str().c_str(),
+	       XORP_UINT_CAST(prefix_len));	       
+    if (0 == _xrl_router)	// The test code sets _xrl_router to zero
 	return;
 
     XrlRibV0p1Client rib(_xrl_router);
@@ -1148,8 +1225,9 @@ NextHopRibRequest<IPv6>::deregister_interest(IPv6 addr,
 				  prefix_len,
 	    ::callback(this,&NextHopRibRequest::deregister_interest_response,
 		       addr, prefix_len,
-		       c_format("deregister_from_rib: addr %s/%d",
-				addr.str().c_str(), prefix_len)));
+		       c_format("deregister_from_rib: addr %s/%u",
+				addr.str().c_str(),
+				XORP_UINT_CAST(prefix_len))));
 }
 
 template <class A>
@@ -1192,15 +1270,28 @@ NextHopRibRequest<A>::deregister_interest_response(const XrlError& error,
 	return;
 	break;
     case SEND_FAILED:
-	XLOG_FATAL("%s %s", comment.c_str(), error.str().c_str());
+	XLOG_FATAL("callback: %s %s", comment.c_str(), error.str().c_str());
 	break;
     case SEND_FAILED_TRANSIENT:
     case NO_SUCH_METHOD:
     case BAD_ARGS:
-    case COMMAND_FAILED:
     case INTERNAL_ERROR:
 	XLOG_FATAL("callback: %s %s",  comment.c_str(), error.str().c_str());
 	break;
+    case COMMAND_FAILED:
+	/*
+	** If it possible that we were de-registering interest in this
+	** nexthop when it became invalid. In this case the command
+	** may fail as the RIB has already told us it is invalid.
+	*/
+	if (_invalid) {
+	    XLOG_ASSERT(addr == _invalid_net.masked_addr() &&
+			prefix_len == _invalid_net.prefix_len());
+	    _invalid = false;
+	} else
+	    XLOG_FATAL("callback: %s %s",  comment.c_str(),
+		       error.str().c_str());
+	break;
     }
 
     //remove this request from the queue
