@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +12,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/peer.cc,v 1.79 2004/06/10 22:40:31 hodson Exp $"
+#ident "$XORP: xorp/bgp/peer.cc,v 1.92 2005/04/10 06:23:15 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
+// #define SAVE_PACKETS
 #define CHECK_TIME
 
 #include "bgp_module.h"
@@ -28,6 +29,7 @@
 
 #include "peer.hh"
 #include "bgp.hh"
+#include "profile_vars.hh"
 
 #define DEBUG_BGPPeer
 
@@ -45,6 +47,7 @@ BGPPeer::BGPPeer(LocalData *ld, BGPPeerData *pd, SocketClient *sock,
     _SocketClient = sock;
     _output_queue_was_busy = false;
     _handler = NULL;
+    _peername = c_format("Peer-%s", peerdata()->iptuple().str().c_str());
 
     _in_updates = 0;
     _out_updates = 0;
@@ -74,8 +77,14 @@ BGPPeer::~BGPPeer()
  */
 bool
 BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
-			 size_t length)
+		     size_t length)
 {
+    if (main()->profile().enabled(profile_message_in))
+	main()->profile().log(profile_message_in,
+			      c_format("message on %s len %u",
+				       str().c_str(),
+				       XORP_UINT_CAST(length)));
+	
     TIMESPENT();
 
     switch (status) {
@@ -96,6 +105,83 @@ BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
 	return false;
     }
 
+#ifdef	SAVE_PACKETS
+    // XXX
+    // This is a horrible hack to try and find a problem before the
+    // 1.1 Release.
+    // All packets will be stored in fname, in MRTD format, if they
+    // arrived on an IPv4 peering. 
+    // The route_btoa program can be used to print the packets.
+    // TODO after the release:
+    // 1) Move the structures into a header file in libxorp.
+    // 2) Add an XRL to enable the saving of incoming packets on a
+    // per peer basis.
+    // 3) Handle IPv6 peerings.
+    // 4) Save all the values that are required by mrt_update.
+    // 5) Move all this code into a separate method.
+    // 6) Keep the file open.
+    // 7) Don't call gettimeofday directly, get the time from the eventloop.
+
+    string fname = "/tmp/bgpin.mrtd";
+
+    FILE *fp = fopen(fname.c_str(), "a");
+    if(0 == fp)
+	XLOG_FATAL("fopen of %s failed: %s", fname.c_str(),
+		   strerror(errno));
+
+    struct mrt_header {
+	uint32_t time;
+	uint16_t type;
+	uint16_t subtype;
+	uint32_t length;
+    };
+
+    struct mrt_update {
+	uint16_t source_as;
+	uint16_t dest_as;
+	uint16_t ifindex;
+	uint16_t af;
+	uint32_t source_ip;
+	uint32_t dest_ip;
+    };
+
+    timeval now;
+    gettimeofday(&now, 0);
+    TimeVal tv(now);
+    
+    mrt_header mrt_header;
+    mrt_header.time = htonl(tv.sec());
+    mrt_header.type = htons(16);
+    mrt_header.subtype = htons(1);
+    mrt_header.length = htonl(length + sizeof(mrt_update));
+    
+    if(fwrite(&mrt_header, sizeof(mrt_header), 1, fp) != 1)
+	XLOG_FATAL("fwrite of %s failed: %s", fname.c_str(), strerror(errno));
+
+    mrt_update update;
+    memset(&update, 0, sizeof(update));
+    update.af = htons(1);	/* IPv4 */
+
+    string peer_addr = peerdata()->iptuple().get_peer_addr();
+    try {
+	update.source_as = htons(peerdata()->as().as());
+	update.source_ip = IPv4(peer_addr.c_str()).addr();
+
+	if(fwrite(&update, sizeof(update), 1, fp) != 1)
+	    XLOG_FATAL("fwrite of %s failed: %s", fname.c_str(),
+		       strerror(errno));
+
+	if(fwrite(buf, length, 1, fp) != 1)
+	    XLOG_FATAL("fwrite of %s failed: %s", fname.c_str(),
+		       strerror(errno));
+    } catch(InvalidFamily &e) {
+	XLOG_ERROR("%s might not be an IPv4 address %s", peer_addr.c_str(),
+		   e.str().c_str());
+    }
+	
+    fclose(fp);
+#endif
+
     _in_total_messages++;
 
     /*
@@ -115,6 +201,8 @@ BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
 	case MESSAGETYPEOPEN: {
 	    debug_msg("OPEN Packet RECEIVED\n");
 	    OpenPacket pac(buf, length);
+	    XLOG_TRACE(main()->profile().enabled(trace_message_in),
+		       cstring(pac));
 	    // All decode errors should throw a CorruptMessage.
 	    debug_msg(pac.str().c_str());
 	    // want unified decode call. now need to get peerdata out.
@@ -124,9 +212,12 @@ BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
 	    break;
 	}
 	case MESSAGETYPEKEEPALIVE: {
-	    debug_msg("KEEPALIVE Packet RECEIVED %u\n", (uint32_t)length);
+	    debug_msg("KEEPALIVE Packet RECEIVED %u\n",
+		      XORP_UINT_CAST(length));
 	    // Check that the length is correct or throw an exception
 	    KeepAlivePacket pac(buf, length);
+	    XLOG_TRACE(main()->profile().enabled(trace_message_in),
+		       cstring(pac));
 	    // debug_msg(pac.str().c_str());
 	    event_keepmess();
 	    TIMESPENT_CHECK();
@@ -137,13 +228,16 @@ BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
 	    _in_updates++;
 	    _mainprocess->eventloop().current_time(_in_update_time);
 	    UpdatePacket pac(buf, length);
+	    XLOG_TRACE(main()->profile().enabled(trace_message_in),
+		       cstring(pac));
 	    // All decode errors should throw a CorruptMessage.
 	    debug_msg(pac.str().c_str());
 	    event_recvupdate(pac);
 	    TIMESPENT_CHECK();
-	    if(TIMESPENT_OVERLIMIT()) {
-		XLOG_WARNING("Processing packet took longer than %d second %s",
-			     TIMESPENT_LIMIT, pac.str().c_str());
+	    if (TIMESPENT_OVERLIMIT()) {
+		XLOG_WARNING("Processing packet took longer than %u second %s",
+			     XORP_UINT_CAST(TIMESPENT_LIMIT),
+			     pac.str().c_str());
 	    }
 	    break;
 	}
@@ -151,6 +245,8 @@ BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
 	    debug_msg("NOTIFICATION Packet RECEIVED\n");
 	    try {
 		NotificationPacket pac(buf, length);
+		XLOG_TRACE(main()->profile().enabled(trace_message_in),
+			   cstring(pac));
 		// All decode errors should throw an InvalidPacket
 		debug_msg(pac.str().c_str());
 		event_recvnotify(pac);
@@ -233,7 +329,7 @@ BGPPeer::send_message(const BGPPacket& p)
     ** This buffer is dynamically allocated and should be freed.
     */
     buf = (const uint8_t *)p.encode(ccnt);
-    debug_msg("Buffer for sent packet is %x\n", (uint)buf);
+    debug_msg("Buffer for sent packet is %p\n", buf);
 
 
     /*
@@ -280,7 +376,7 @@ BGPPeer::send_message_complete(SocketClient::Event ev, const uint8_t *buf)
 	/*drop through to next case*/
     case SocketClient::FLUSHING:
 	debug_msg("event: flushing\n");
-	debug_msg("Freeing Buffer for sent packet: %x\n", (uint)buf);
+	debug_msg("Freeing Buffer for sent packet: %p\n", buf);
 	delete[] buf;
 	TIMESPENT_CHECK();
 	break;
@@ -298,7 +394,7 @@ BGPPeer::send_message_complete(SocketClient::Event ev, const uint8_t *buf)
 }
 
 void
-BGPPeer::send_notification(const NotificationPacket& p, bool error)
+BGPPeer::send_notification(const NotificationPacket& p, bool restart)
 {
     debug_msg(p.str().c_str());
 
@@ -326,13 +422,13 @@ BGPPeer::send_notification(const NotificationPacket& p, bool error)
      * This buffer is dynamically allocated and should be freed.
      */
     const uint8_t *buf = (const uint8_t *)p.encode(ccnt);
-    debug_msg("Buffer for sent packet is %x\n", (uint)buf);
+    debug_msg("Buffer for sent packet is %p\n", buf);
 
     /*
     ** This write is async. So we can't free the data now,
     ** we will deal with it in the complete routine.  */
     bool ret =_SocketClient->send_message(buf, ccnt,
-	       callback(this, &BGPPeer::send_notification_complete, error));
+	       callback(this, &BGPPeer::send_notification_complete, restart));
 
     if (!ret) {
 	delete[] buf;
@@ -343,7 +439,7 @@ BGPPeer::send_notification(const NotificationPacket& p, bool error)
 
 void
 BGPPeer::send_notification_complete(SocketClient::Event ev,
-				    const uint8_t* buf, bool error)
+				    const uint8_t* buf, bool restart)
 {
     TIMESPENT();
 
@@ -352,7 +448,7 @@ BGPPeer::send_notification_complete(SocketClient::Event ev,
 	debug_msg("Notification sent\n");
 	XLOG_ASSERT(STATESTOPPED == _state);
 	delete[] buf;
-	set_state(STATEIDLE, error);
+	set_state(STATEIDLE, restart);
 	break;
     case SocketClient::FLUSHING:
 	delete[] buf;
@@ -360,7 +456,7 @@ BGPPeer::send_notification_complete(SocketClient::Event ev,
     case SocketClient::ERROR:
 	debug_msg("Notification not sent\n");
 	XLOG_ASSERT(STATESTOPPED == _state);
-	set_state(STATEIDLE, error);
+	set_state(STATEIDLE, restart);
 	/* Don't free the message here we'll get it in the flush */
 	break;
     }
@@ -382,7 +478,7 @@ BGPPeer::hook_stopped()
     /*
     ** If the original notification was not an error such as sending a
     ** CEASE. If we arrived here due to a timeout, something has gone
-    ** wrong so unconditionally set the error to true.
+    ** wrong so unconditionally set the restart to true.
     */
     set_state(STATEIDLE, true);
 }
@@ -437,7 +533,7 @@ BGPPeer::event_start()			// EVENTBGPSTART
  * stop event. No data associated with the event.
  */
 void
-BGPPeer::event_stop()			// EVENTBGPSTOP
+BGPPeer::event_stop(bool restart)	// EVENTBGPSTOP
 { 
     TIMESPENT();
 
@@ -451,7 +547,7 @@ BGPPeer::event_stop()			// EVENTBGPSTOP
 	/*FALLTHROUGH*/
 
     case STATEACTIVE:
-	set_state(STATEIDLE);
+	set_state(STATEIDLE, restart);
 	break;
 
     case STATEOPENSENT:
@@ -459,14 +555,14 @@ BGPPeer::event_stop()			// EVENTBGPSTOP
     case STATEESTABLISHED: {
 	// Send Notification Message with error code of CEASE.
 	NotificationPacket np(CEASE);
-	send_notification(np, false);
-	set_state(STATESTOPPED);
+	send_notification(np, restart);
+	set_state(STATESTOPPED, restart);
 	break;
     }
     case STATESTOPPED:
 	// a second stop will cause us to give up on sending the CEASE
 	flush_transmit_queue();		// ensure callback can't happen
-	set_state(STATEIDLE);
+	set_state(STATEIDLE, restart);
 	break;
     }
 }
@@ -502,7 +598,8 @@ BGPPeer::event_open()	// EVENTBGPTRANOPEN
 	ParameterList::const_iterator
 	    pi = _peerdata->parameter_sent_list().begin();
 	while(pi != _peerdata->parameter_sent_list().end()) {
-	    open_packet.add_parameter(*pi);
+	    if ((*pi)->send())
+		open_packet.add_parameter(*pi);
 	    pi++;
 	}
 
@@ -1014,8 +1111,8 @@ BGPPeer::check_open_packet(const OpenPacket *p) throw(CorruptMessage)
 		  _peerdata->as().str().c_str());
 	xorp_throw(CorruptMessage,
 		   c_format("Wrong AS %s expecting %s",
-		      p->as().str().c_str(),
-		      _peerdata->as().str().c_str()),
+			    p->as().str().c_str(),
+			    _peerdata->as().str().c_str()),
 		   OPENMSGERROR, BADASPEER);
     }
 
@@ -1066,12 +1163,29 @@ BGPPeer::check_open_packet(const OpenPacket *p) throw(CorruptMessage)
 }
 
 #define	REMOVE_UNNEGOTIATED_NLRI
+inline
+bool
+check_multiprotocol_nlri(const UpdatePacket *, const BGPUpdateAttribList& pa,
+			 bool neg)
+{
+    if (!pa.empty() && !neg) {
+#ifdef	REMOVE_UNNEGOTIATED_NLRI
+	// We didn't advertise this capability, so strip this sucker out.
+	const_cast<BGPUpdateAttribList *>(&pa)->clear();
+#else
+	// We didn't advertise this capability, drop peering.
+ 	return false;
+#endif
+    }
+
+    return true;
+}
 
 inline
 bool
 check_multiprotocol_nlri(const UpdatePacket *p, PathAttribute *pa, bool neg)
 {
-    if(pa && !neg) {
+    if (pa && !neg) {
 #ifdef	REMOVE_UNNEGOTIATED_NLRI
 	// We didn't advertise this capability, so strip this sucker out.
 	PathAttributeList<IPv4>& tmp = 
@@ -1129,33 +1243,43 @@ BGPPeer::check_update_packet(const UpdatePacket *p)
 //     BGPPeerData::Direction dir = BGPPeerData::NEGOTIATED;
     BGPPeerData::Direction dir = BGPPeerData::SENT;
     bool bad_nlri = false;
-    if(!check_multiprotocol_nlri(p, p->mpreach<IPv4>(SAFI_MULTICAST),
-				 peerdata()->
-				 multiprotocol<IPv4>(SAFI_MULTICAST, dir)))
+#define STOP_IPV4_UNICAST
+#ifdef	STOP_IPV4_UNICAST
+    if (!check_multiprotocol_nlri(p, p->nlri_list(),
+				  peerdata()->
+				  multiprotocol<IPv4>(SAFI_UNICAST, dir)))
 	bad_nlri = true;
-    if(!check_multiprotocol_nlri(p, p->mpunreach<IPv4>(SAFI_MULTICAST),
-				 peerdata()->
-				 multiprotocol<IPv4>(SAFI_MULTICAST, dir)))
+    if (!check_multiprotocol_nlri(p, p->wr_list(),
+				  peerdata()->
+				  multiprotocol<IPv4>(SAFI_UNICAST, dir)))
 	bad_nlri = true;
-
-    if(!check_multiprotocol_nlri(p, p->mpreach<IPv6>(SAFI_UNICAST),
-				 peerdata()->
-				 multiprotocol<IPv6>(SAFI_UNICAST, dir)))
+#endif
+    if (!check_multiprotocol_nlri(p, p->mpreach<IPv4>(SAFI_MULTICAST),
+				  peerdata()->
+				  multiprotocol<IPv4>(SAFI_MULTICAST, dir)))
 	bad_nlri = true;
-    if(!check_multiprotocol_nlri(p, p->mpunreach<IPv6>(SAFI_UNICAST),
-				 peerdata()->
-				 multiprotocol<IPv6>(SAFI_UNICAST, dir)))
+    if (!check_multiprotocol_nlri(p, p->mpunreach<IPv4>(SAFI_MULTICAST),
+				  peerdata()->
+				  multiprotocol<IPv4>(SAFI_MULTICAST, dir)))
 	bad_nlri = true;
-    if(!check_multiprotocol_nlri(p, p->mpreach<IPv6>(SAFI_MULTICAST),
-				 peerdata()->
-				 multiprotocol<IPv6>(SAFI_MULTICAST, dir)))
+    if (!check_multiprotocol_nlri(p, p->mpreach<IPv6>(SAFI_UNICAST),
+				  peerdata()->
+				  multiprotocol<IPv6>(SAFI_UNICAST, dir)))
 	bad_nlri = true;
-    if(!check_multiprotocol_nlri(p, p->mpunreach<IPv6>(SAFI_MULTICAST),
-				 peerdata()->
-				 multiprotocol<IPv6>(SAFI_MULTICAST, dir)))
+    if (!check_multiprotocol_nlri(p, p->mpunreach<IPv6>(SAFI_UNICAST),
+				  peerdata()->
+				  multiprotocol<IPv6>(SAFI_UNICAST, dir)))
+	bad_nlri = true;
+    if (!check_multiprotocol_nlri(p, p->mpreach<IPv6>(SAFI_MULTICAST),
+				  peerdata()->
+				  multiprotocol<IPv6>(SAFI_MULTICAST, dir)))
+	bad_nlri = true;
+    if (!check_multiprotocol_nlri(p, p->mpunreach<IPv6>(SAFI_MULTICAST),
+				  peerdata()->
+				  multiprotocol<IPv6>(SAFI_MULTICAST, dir)))
 	bad_nlri = true;
 #ifndef	REMOVE_UNNEGOTIATED_NLRI
-    if(bad_nlri)
+    if (bad_nlri)
 	return new NotificationPacket(UPDATEMSGERR, OPTATTR);
 #endif
 
@@ -1296,6 +1420,7 @@ BGPPeer::check_update_packet(const UpdatePacket *p)
 	// address of our peer
 	// if so, we'd discard the route, but not send a notification
 
+#if	0
 	if (ibgp() && !local_pref)
 	    XLOG_WARNING("%s Update packet from ibgp with no LOCAL_PREF",
 			 this->str().c_str());
@@ -1303,7 +1428,7 @@ BGPPeer::check_update_packet(const UpdatePacket *p)
 	if (!ibgp() && local_pref)
 	    XLOG_WARNING("%s Update packet from ebgp with LOCAL_PREF",
 			 this->str().c_str());
-
+#endif
 	// XXX Check Network layer reachability fields
 
     }
@@ -1369,7 +1494,7 @@ BGPPeer::connected(int sock)
     ** then it may be necessary to deal with the STATESTOPPED
     ** explictly.
     */
-//     if(is_connected()) {
+//     if (is_connected()) {
 // 	debug_msg("Already connected dropping this connection\n");
 // 	::close(sock);
 // 	return;
@@ -1398,8 +1523,8 @@ BGPPeer::clear_all_timers()
 void
 BGPPeer::start_connect_retry_timer()
 {
-    debug_msg("Start Connect Retry timer after %d ms\n",
-	      _peerdata->get_retry_duration());
+    debug_msg("Start Connect Retry timer after %u ms\n",
+	      XORP_UINT_CAST(_peerdata->get_retry_duration()));
 
     _timer_connect_retry = _mainprocess->eventloop().
 	new_oneoff_after_ms(_peerdata->get_retry_duration(),
@@ -1417,8 +1542,8 @@ BGPPeer::clear_connect_retry_timer()
 void
 BGPPeer::restart_connect_retry_timer()
 {
-    debug_msg("restart Connect Retry timer after %d ms\n",
-	      _peerdata->get_retry_duration());
+    debug_msg("restart Connect Retry timer after %u ms\n",
+	      XORP_UINT_CAST(_peerdata->get_retry_duration()));
 
     clear_connect_retry_timer();
     start_connect_retry_timer();
@@ -1435,9 +1560,9 @@ BGPPeer::start_hold_timer()
     uint32_t duration = _peerdata->get_hold_duration();
 
     if (duration != 0) {
-	/* Add another half a second to give the remote keepalive a chance*/
+	/* Add another half a second to give the remote keepalive a chance */
 	duration += 500;
-	debug_msg("Holdtimer started %d\n", duration);
+	debug_msg("Holdtimer started %u\n", XORP_UINT_CAST(duration));
 	_timer_hold_time = _mainprocess->eventloop().
 	    new_oneoff_after_ms(duration,
 	    callback(this, &BGPPeer::event_holdexp));
@@ -1464,8 +1589,8 @@ void
 BGPPeer::start_keepalive_timer()
 {
     uint32_t duration = _peerdata->get_keepalive_duration();
-    debug_msg("KeepAlive timer started with duration %d\n",
-	      duration);
+    debug_msg("KeepAlive timer started with duration %u\n",
+	      XORP_UINT_CAST(duration));
 
     if (duration > 0)
 	_timer_keep_alive = _mainprocess->eventloop().
@@ -1528,13 +1653,15 @@ BGPPeer::release_resources()
     return true;
 }
 
+#if	0
 string
-BGPPeer::str()
+BGPPeer::str() const
 {
     return c_format("Peer(%d)-%s",
 		    get_sock(),
 		    peerdata()->iptuple().str().c_str());
 }
+#endif
 
 const char *
 BGPPeer::pretty_print_state(FSMState s)
@@ -1559,7 +1686,7 @@ BGPPeer::pretty_print_state(FSMState s)
 }
 
 void
-BGPPeer::set_state(FSMState s, bool error)
+BGPPeer::set_state(FSMState s, bool restart)
 {
     TIMESPENT();
 
@@ -1580,7 +1707,7 @@ BGPPeer::set_state(FSMState s, bool error)
 	    clear_all_timers();
 	    // Release resources - which includes a disconnect
 	    release_resources();
-	    if (error) {
+	    if (restart) {
 		// re-start timer = 60;
 		// or exp growth see section 8
 
@@ -1644,7 +1771,7 @@ void
 trap_callback(const XrlError& error, const char *comment)
 {
     debug_msg("trap_callback %s %s\n", comment, error.str().c_str());
-    if(XrlError::OKAY() != error) {
+    if (XrlError::OKAY() != error) {
 	XLOG_WARNING("trap_callback: %s %s", comment, error.str().c_str());
     }
 }
@@ -1698,7 +1825,7 @@ BGPPeer::get_msg_stats(uint32_t& in_updates,
 bool 
 BGPPeer::remote_ip_ge_than(const BGPPeer& peer)
 {
-    const IPv4& this_remote_ip = peerdata()->iptuple().get_peer_addr();
-    const IPv4& other_remote_ip = peer.peerdata()->iptuple().get_peer_addr();
+    string this_remote_ip = peerdata()->iptuple().get_peer_addr();
+    string other_remote_ip = peer.peerdata()->iptuple().get_peer_addr();
     return (this_remote_ip >= other_remote_ip);     
 }
