@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/peer_handler.cc,v 1.35 2004/06/10 22:40:32 hodson Exp $"
+#ident "$XORP: xorp/bgp/peer_handler.cc,v 1.38 2005/03/25 02:52:43 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -404,11 +404,11 @@ PeerHandler::start_packet(bool ibgp)
 int
 PeerHandler::add_route(const SubnetRoute<IPv4> &rt, Safi safi)
 {
-    debug_msg("PeerHandler::add_route(IPv4) %x\n", (u_int)(&rt));
+    debug_msg("PeerHandler::add_route(IPv4) %p\n", &rt);
     XLOG_ASSERT(_packet != NULL);
 
-    // Check this peer want this NLRI
-    if (!multiprotocol<IPv4>(safi, BGPPeerData::RECEIVED))
+    // Check this peer wants this NLRI
+    if (!multiprotocol<IPv4>(safi, BGPPeerData::NEGOTIATED))
 	return 0;
 
     if (_packet->big_enough()) {
@@ -465,8 +465,8 @@ PeerHandler::add_route(const SubnetRoute<IPv6> &rt, Safi safi)
     debug_msg("PeerHandler::add_route(IPv6) %p\n", &rt);
     XLOG_ASSERT(_packet != NULL);
 
-    // Check this peer want this NLRI
-    if (!multiprotocol<IPv6>(safi, BGPPeerData::RECEIVED))
+    // Check this peer wants this NLRI
+    if (!multiprotocol<IPv6>(safi, BGPPeerData::NEGOTIATED))
 	return 0;
 
     if (_packet->big_enough()) {
@@ -527,11 +527,11 @@ PeerHandler::replace_route(const SubnetRoute<IPv6> &old_rt,
 int
 PeerHandler::delete_route(const SubnetRoute<IPv4> &rt, Safi safi)
 {
-    debug_msg("PeerHandler::delete_route(IPv4) %x\n", (u_int)(&rt));
+    debug_msg("PeerHandler::delete_route(IPv4) %p\n", &rt);
     XLOG_ASSERT(_packet != NULL);
 
-    // Check this peer want this NLRI
-    if (!multiprotocol<IPv4>(safi, BGPPeerData::RECEIVED))
+    // Check this peer wants this NLRI
+    if (!multiprotocol<IPv4>(safi, BGPPeerData::NEGOTIATED))
 	return 0;
 
     if (_packet->big_enough()) {
@@ -566,8 +566,8 @@ PeerHandler::delete_route(const SubnetRoute<IPv6>& rt, Safi safi)
     debug_msg("PeerHandler::delete_route(IPv6) %p\n", &rt);
     XLOG_ASSERT(_packet != NULL);
 
-    // Check this peer want this NLRI
-    if (!multiprotocol<IPv6>(safi, BGPPeerData::RECEIVED))
+    // Check this peer wants this NLRI
+    if (!multiprotocol<IPv6>(safi, BGPPeerData::NEGOTIATED))
 	return 0;
 
     if (_packet->big_enough()) {
