@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +13,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/plumbing.cc,v 1.54 2004/06/10 22:40:32 hodson Exp $"
+#ident "$XORP: xorp/bgp/plumbing.cc,v 1.65 2005/03/25 02:52:44 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
+#define CHECK_TIME
 
 #include "bgp_module.h"
 #include "config.h"
@@ -27,20 +29,26 @@
 
 #include "plumbing.hh"
 #include "bgp.hh"
+#include "profile_vars.hh"
+#include "libxorp/timespent.hh"
 
 BGPPlumbing::BGPPlumbing(const Safi safi,
 			 RibIpcHandler* ribhandler,
 			 NextHopResolver<IPv4>& next_hop_resolver_ipv4,
-			 NextHopResolver<IPv6>& next_hop_resolver_ipv6)
+			 NextHopResolver<IPv6>& next_hop_resolver_ipv6,
+			 PolicyFilters& pfs,
+			 BGPMain& bgp)
     : _rib_handler(ribhandler),
       _next_hop_resolver_ipv4(next_hop_resolver_ipv4),
       _next_hop_resolver_ipv6(next_hop_resolver_ipv6),
       _safi(safi),
+      _policy_filters(pfs),
       _plumbing_ipv4("[IPv4:" + string(pretty_string_safi(safi)) + "]", *this,
 		     _next_hop_resolver_ipv4),
       _plumbing_ipv6("[IPv6:" + string(pretty_string_safi(safi)) + "]", *this, 
 		     _next_hop_resolver_ipv6),
-      _my_AS_number(AsNum::AS_INVALID)
+      _my_AS_number(AsNum::AS_INVALID),
+      _bgp(bgp)
 {
 }
 
@@ -74,8 +82,11 @@ BGPPlumbing::peering_went_down(PeerHandler* peer_handler)
 {
     debug_msg("BGPPlumbing::peering_went_down\n");
     int result = 0;
+    TIMESPENT();
     result |= plumbing_ipv4().peering_went_down(peer_handler);
+    TIMESPENT_CHECK();
     result |= plumbing_ipv6().peering_went_down(peer_handler);
+    TIMESPENT_CHECK();
     return result;
 }
 
@@ -112,6 +123,9 @@ BGPPlumbing::add_route(const InternalMessage<IPv4> &rtmsg,
 		       PeerHandler* peer_handler) 
 {
     debug_msg("BGPPlumbing::add_route IPv4\n");
+    if (main().profile().enabled(profile_route_ribin))
+	main().profile().log(profile_route_ribin, 
+			     c_format("add %s", rtmsg.net().str().c_str()));
     return plumbing_ipv4().add_route(rtmsg, peer_handler);
 }
 
@@ -120,6 +134,9 @@ BGPPlumbing::add_route(const InternalMessage<IPv6> &rtmsg,
 		       PeerHandler* peer_handler)  
 {
     debug_msg("BGPPlumbing::add_route IPv6\n");
+    if (main().profile().enabled(profile_route_ribin))
+	main().profile().log(profile_route_ribin,
+			     c_format("add %s", rtmsg.net().str().c_str()));
     return plumbing_ipv6().add_route(rtmsg, peer_handler);
 }
 
@@ -127,6 +144,9 @@ int
 BGPPlumbing::delete_route(const InternalMessage<IPv4> &rtmsg, 
 			  PeerHandler* peer_handler) 
 {
+    if (main().profile().enabled(profile_route_ribin))
+	main().profile().log(profile_route_ribin,
+			     c_format("delete %s", rtmsg.net().str().c_str()));
     return plumbing_ipv4().delete_route(rtmsg, peer_handler);
 }
 
@@ -134,6 +154,9 @@ int
 BGPPlumbing::delete_route(const InternalMessage<IPv6> &rtmsg, 
 			  PeerHandler* peer_handler) 
 {
+    if (main().profile().enabled(profile_route_ribin))
+	main().profile().log(profile_route_ribin,
+			     c_format("delete %s", rtmsg.net().str().c_str()));
     return plumbing_ipv6().delete_route(rtmsg, peer_handler);
 }
 
@@ -141,6 +164,9 @@ int
 BGPPlumbing::delete_route(const IPNet<IPv4>& net,
 			  PeerHandler* peer_handler) 
 {
+    if (main().profile().enabled(profile_route_ribin))
+	main().profile().log(profile_route_ribin,
+			     c_format("delete %s", net.str().c_str()));
     return plumbing_ipv4().delete_route(net, peer_handler);
 }
 
@@ -148,6 +174,9 @@ int
 BGPPlumbing::delete_route(const IPNet<IPv6>& net,
 			  PeerHandler* peer_handler) 
 {
+    if (main().profile().enabled(profile_route_ribin))
+	main().profile().log(profile_route_ribin,
+			     c_format("delete %s", net.str().c_str()));
     return plumbing_ipv6().delete_route(net, peer_handler);
 }
 
@@ -232,6 +261,12 @@ BGPPlumbing::status(string& reason) const
     return true;
 }
 
+void
+BGPPlumbing::push_routes() {
+    plumbing_ipv4().push_routes();
+    plumbing_ipv6().push_routes();
+}
+
 /***********************************************************************/
 
 template <class A>
@@ -269,11 +304,19 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 			     _next_hop_resolver);
     _next_hop_resolver.add_decision(_decision_table);
 
+    _policy_sourcematch_table =
+	new PolicyTableSourceMatch<A>(ribname + "PolicyExportSourceMatchTable",
+				      _master.safi(),
+				      _decision_table,
+				      _master.policy_filters());
+    _decision_table->set_next_table(_policy_sourcematch_table);
+
+
     _fanout_table = 
 	new FanoutTable<A>(ribname + "FanoutTable", 
 			   _master.safi(),
-			   _decision_table);
-    _decision_table->set_next_table(_fanout_table);
+			   _policy_sourcematch_table);
+    _policy_sourcematch_table->set_next_table(_fanout_table);
 
     /*
      * Plumb the input branch
@@ -285,6 +328,8 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 			  _master.rib_handler());
     _in_map[_master.rib_handler()] = _ipc_rib_in_table;
 
+    
+
     FilterTable<A>* filter_in =
 	new FilterTable<A>(_ribname + "IpcChannelInputFilter",
 			   _master.safi(),
@@ -292,12 +337,19 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 			   _next_hop_resolver);
     _ipc_rib_in_table->set_next_table(filter_in);
     
+    PolicyTableImport<A>* policy_filter_in =
+	new PolicyTableImport<A>(_ribname + "IpcChannelInputPolicyFilter",
+				 _master.safi(),
+				 filter_in,
+				 _master.policy_filters());
+    filter_in->set_next_table(policy_filter_in);
+
     CacheTable<A>* cache_in = 
 	new CacheTable<A>(_ribname + "IpcChannelInputCache",
 			  _master.safi(),
-			  filter_in,
+			  policy_filter_in,
 			  _master.rib_handler());
-    filter_in->set_next_table(cache_in);
+    policy_filter_in->set_next_table(cache_in);
 
     NhLookupTable<A> *nexthop_in =
 	new NhLookupTable<A>(_ribname + "IpcChannelNhLookup",
@@ -311,7 +363,9 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 				_ipc_rib_in_table->genid());
 
     _tables.insert(filter_in);
+    _tables.insert(policy_filter_in);
     _tables.insert(cache_in);
+    _tables.insert(nexthop_in);
 
     /*
      * Plumb the output branch
@@ -322,16 +376,24 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 			   _master.safi(),
 			   _fanout_table,
 			   _next_hop_resolver);
-    _fanout_table->add_next_table(filter_out, _master.rib_handler(),
-				  _ipc_rib_in_table->genid());
     _tables.insert(filter_out);
 
+    PolicyTable<A>* policy_filter_out =
+	new PolicyTable<A>(ribname + "IpcChannelOutputPolicyFilter",
+			   _master.safi(),
+			   filter_out,
+			   _master.policy_filters(),
+			   filter::EXPORT);
+    filter_out->set_next_table(policy_filter_out);
+    _tables.insert(policy_filter_out);
+
+
     CacheTable<A> *cache_out =
 	new CacheTable<A>(ribname + "IpcChannelOutputCache",
 			  _master.safi(),
-			  filter_out,
+			  policy_filter_out,
 			  _master.rib_handler());
-    filter_out->set_next_table(cache_out);
+    policy_filter_out->set_next_table(cache_out);
     _tables.insert(cache_out);
 
     _ipc_rib_out_table =
@@ -341,9 +403,9 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 			   _master.rib_handler());
     _out_map[_master.rib_handler()] = _ipc_rib_out_table;
     cache_out->set_next_table(_ipc_rib_out_table);
-    
-    _tables.insert(filter_out);
-    _tables.insert(cache_out);
+
+    _fanout_table->add_next_table(filter_out, _master.rib_handler(),
+				  _ipc_rib_in_table->genid());
 }
 
 template <class A>
@@ -354,6 +416,7 @@ BGPPlumbingAF<A>::~BGPPlumbingAF()
 	delete (*i);
     }
     delete _decision_table;
+    delete _policy_sourcematch_table;
     delete _fanout_table;
     delete _ipc_rib_in_table;
     delete _ipc_rib_out_table;
@@ -400,12 +463,19 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
 			   _next_hop_resolver);
     rib_in->set_next_table(filter_in);
     
+    PolicyTableImport<A>* policy_filter_in =
+	new PolicyTableImport<A>(_ribname + "PeerInputPolicyFilter" + peername,
+				 _master.safi(),
+				 filter_in,
+				 _master.policy_filters());
+    filter_in->set_next_table(policy_filter_in);			   
+
     CacheTable<A>* cache_in = 
 	new CacheTable<A>(_ribname + "PeerInputCache" + peername,
 			  _master.safi(),
-			  filter_in,
+			  policy_filter_in,
 			  peer_handler);
-    filter_in->set_next_table(cache_in);
+    policy_filter_in->set_next_table(cache_in);
 
     NhLookupTable<A> *nexthop_in =
 	new NhLookupTable<A>(_ribname + "NhLookup" + peername,
@@ -419,6 +489,7 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
 
     _tables.insert(rib_in);
     _tables.insert(filter_in);
+    _tables.insert(policy_filter_in);
     _tables.insert(cache_in);
     _tables.insert(nexthop_in);
 
@@ -429,6 +500,7 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
     const AsNum& his_AS_number = peer_handler->AS_number();
     const AsNum& my_AS_number = _master.my_AS_number();
 
+    
     /* 1. configure the loop filters */
     filter_in->add_simple_AS_filter(my_AS_number);
 
@@ -446,21 +518,26 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
     /*
      * Plumb the output branch
      */
-    
     FilterTable<A>* filter_out =
 	new FilterTable<A>(_ribname + "PeerOutputFilter" + peername,
 			   _master.safi(),
 			   _fanout_table,
 			   _next_hop_resolver);
-    _fanout_table->add_next_table(filter_out, peer_handler,
-				  rib_in->genid());
-    
+
+    PolicyTable<A>* policy_filter_out =
+	new PolicyTable<A>(_ribname + "PeerOutputPolicyFilter" + peername,
+			   _master.safi(),
+			   filter_out,
+			   _master.policy_filters(),
+			   filter::EXPORT);
+    filter_out->set_next_table(policy_filter_out);
+   
     CacheTable<A>* cache_out = 
 	new CacheTable<A>(_ribname + "PeerOutputCache" + peername,
 			  _master.safi(),
-			  filter_out,
+			  policy_filter_out,
 			  peer_handler);
-    filter_out->set_next_table(cache_out);
+    policy_filter_out->set_next_table(cache_out);
 
     RibOutTable<A>* rib_out =
 	new RibOutTable<A>(_ribname + "RibOut" + peername,
@@ -472,10 +549,10 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
     _reverse_out_map[rib_out] = peer_handler;
 
     _tables.insert(filter_out);
+    _tables.insert(policy_filter_out);
     _tables.insert(cache_out);
     _tables.insert(rib_out);
 
-    
     /*
      * Start things up on the output branch
      */
@@ -524,10 +601,13 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
 
     /* 9. load up damping filters */
     /* TBD */
-    
-    /* 10. cause the routing table to be dumped to the new peer */
+
+    /* 10. finally plumb in the output branch */
+    _fanout_table->add_next_table(filter_out, peer_handler, rib_in->genid());
+
+    /* 11. cause the routing table to be dumped to the new peer */
     dump_entire_table(filter_out, _ribname);
-    if(_awaits_push)
+    if (_awaits_push)
 	push(peer_handler);
 
     return 0;
@@ -545,14 +625,15 @@ BGPPlumbingAF<A>::stop_peering(PeerHandler* peer_handler)
     typename map <PeerHandler*, RibOutTable<A>*>::iterator iter;
     iter = _out_map.find(peer_handler);
     if (iter == _out_map.end()) 
-	XLOG_FATAL("BGPPlumbingAF<IPv%u,%s>::stop_peering: peer %#x not found",
-		   A::ip_version(),  pretty_string_safi(_master.safi()),
-		   (u_int)peer_handler);
+	XLOG_FATAL("BGPPlumbingAF<IPv%u,%s>::stop_peering: peer %p not found",
+		   XORP_UINT_CAST(A::ip_version()),
+		   pretty_string_safi(_master.safi()),
+		   peer_handler);
     rt = iter->second;
     prevrt = rt;
     while (rt != _fanout_table) {
-	debug_msg("rt=%x (%s), _fanout_table=%x\n", 
-	       (u_int)rt, rt->tablename().c_str(), (u_int)_fanout_table);
+	debug_msg("rt=%p (%s), _fanout_table=%p\n", 
+		  rt, rt->tablename().c_str(), _fanout_table);
 	if (rt->type() == CACHE_TABLE)
 	    ((CacheTable<A>*)rt)->flush_cache();
 	prevrt = rt;
@@ -585,10 +666,14 @@ BGPPlumbingAF<A>::peering_went_down(PeerHandler* peer_handler)
     RibInTable<A> *rib_in;
     rib_in = iter->second;
     //peering went down will be propagated downstream by the RIB-In.
+    TIMESPENT();
     rib_in->ribin_peering_went_down();
+    TIMESPENT_CHECK();
+    
 
     //stop_peering shuts down and disconnects the output branch for this peer
     stop_peering(peer_handler);
+    TIMESPENT_CHECK();
 
     /* we don't flush the input caches - lookup requests should still
        be answered until the DeletionTable gets round to telling the
@@ -617,13 +702,13 @@ BGPPlumbingAF<A>::peering_came_up(PeerHandler* peer_handler)
     typename map <PeerHandler*, RibOutTable<A>*>::iterator iter;
     iter = _out_map.find(peer_handler);
     if (iter == _out_map.end()) 
-	XLOG_FATAL("BGPPlumbingAF<A>::peering_came_up: peer %#x not found",
-		(u_int)peer_handler);
+	XLOG_FATAL("BGPPlumbingAF<A>::peering_came_up: peer %p not found",
+		   peer_handler);
     rt = iter->second;
     prevrt = rt;
     while (rt != NULL) {
-	debug_msg("rt=%x (%s), _fanout_table=%x\n", 
-	       (u_int)rt, rt->tablename().c_str(), (u_int)_fanout_table);
+	debug_msg("rt=%p (%s), _fanout_table=%p\n", 
+		  rt, rt->tablename().c_str(), _fanout_table);
 	prevrt = rt;
 	rt = rt->parent();
     }
@@ -632,13 +717,13 @@ BGPPlumbingAF<A>::peering_came_up(PeerHandler* peer_handler)
     FilterTable<A> *filter_out = dynamic_cast<FilterTable<A> *>(prevrt);
     XLOG_ASSERT(filter_out != NULL);
 
-    _fanout_table->add_next_table(filter_out, peer_handler, rib_in->genid());
     filter_out->set_parent(_fanout_table);
+    _fanout_table->add_next_table(filter_out, peer_handler, rib_in->genid());
 
     //do the route dump
     dump_entire_table(filter_out, _ribname);
 
-    if(_awaits_push)
+    if (_awaits_push)
 	push(peer_handler);
     return 0;
 }
@@ -700,8 +785,8 @@ BGPPlumbingAF<A>::delete_peering(PeerHandler* peer_handler)
     typename map <PeerHandler*, RibOutTable<A>*>::iterator iter;
     iter = _out_map.find(peer_handler);
     if (iter == _out_map.end())
-	XLOG_FATAL("BGPPlumbingAF<A>::drop_peering: peer %#x not found",
-		(u_int)peer_handler);
+	XLOG_FATAL("BGPPlumbingAF<A>::drop_peering: peer %p not found",
+		   peer_handler);
 
     iter = _out_map.find(peer_handler);
     rt = iter->second;
@@ -721,7 +806,8 @@ void
 BGPPlumbingAF<A>::dump_entire_table(FilterTable<A> *filter_out, string ribname)
 {
     debug_msg("BGPPlumbingAF<IPv%u:%s>::dump_entire_table\n",
-	      A::ip_version(), pretty_string_safi(_master.safi()));
+	      XORP_UINT_CAST(A::ip_version()),
+	      pretty_string_safi(_master.safi()));
 
     _fanout_table->dump_entire_table(filter_out, _master.safi(), ribname);
 
@@ -757,7 +843,8 @@ template <class A>
 void
 BGPPlumbingAF<A>::flush(PeerHandler* peer_handler) 
 {
-    debug_msg("BGPPlumbingAF<IPv%u:%s>::flush\n", A::ip_version(),
+    debug_msg("BGPPlumbingAF<IPv%u:%s>::flush\n",
+	      XORP_UINT_CAST(A::ip_version()),
 	      pretty_string_safi(_master.safi()));
 
     RibInTable<A> *rib_in;
@@ -766,7 +853,8 @@ BGPPlumbingAF<A>::flush(PeerHandler* peer_handler)
     if (iter == _in_map.end())
 	XLOG_FATAL("BGPPlumbingAF<IPv%u:%s>: "
 		   "flush called for a PeerHandler "
-		   "that has no associated RibIn", A::ip_version(),
+		   "that has no associated RibIn",
+		   XORP_UINT_CAST(A::ip_version()),
 		   pretty_string_safi(_master.safi()));
 
     rib_in = iter->second;
@@ -780,7 +868,8 @@ int
 BGPPlumbingAF<A>::add_route(const InternalMessage<A> &rtmsg, 
 			    PeerHandler* peer_handler) 
 {
-    debug_msg("BGPPlumbingAF<IPv%u:%s>::add_route\n", A::ip_version(),
+    debug_msg("BGPPlumbingAF<IPv%u:%s>::add_route\n",
+	      XORP_UINT_CAST(A::ip_version()),
 	      pretty_string_safi(_master.safi()));
 
     int result = 0;
@@ -790,7 +879,8 @@ BGPPlumbingAF<A>::add_route(const InternalMessage<A> &rtmsg,
     if (iter == _in_map.end())
 	XLOG_FATAL("BGPPlumbingAF<IPv%u:%s>: "
 		   "add_route called for a PeerHandler "
-		   "that has no associated RibIn", A::ip_version(),
+		   "that has no associated RibIn",
+		   XORP_UINT_CAST(A::ip_version()),
 		   pretty_string_safi(_master.safi()));
 
     rib_in = iter->second;
@@ -867,11 +957,12 @@ template <class A>
 void
 BGPPlumbingAF<A>::push(PeerHandler* peer_handler) 
 {
-    debug_msg("BGPPlumbingAF<IPv%u:%s>::push\n", A::ip_version(),
+    debug_msg("BGPPlumbingAF<IPv%u:%s>::push\n",
+	      XORP_UINT_CAST(A::ip_version()),
 	      pretty_string_safi(_master.safi()));
     if (_awaits_push == false) {
 	XLOG_WARNING("push <IPv%u:%s> when none needed",
-		     A::ip_version(),
+		     XORP_UINT_CAST(A::ip_version()),
 		     pretty_string_safi(_master.safi()));
 	return;
     }
@@ -981,5 +1072,16 @@ BGPPlumbingAF<A>::status(string&) const
     return true;
 }
 
+
+template <class A>
+void
+BGPPlumbingAF<A>::push_routes() {
+    list<const PeerTableInfo<A>*> peer_list;
+
+    _fanout_table->peer_table_info(peer_list);
+
+    _policy_sourcematch_table->push_routes(peer_list);
+}
+
 template class BGPPlumbingAF<IPv4>;
 template class BGPPlumbingAF<IPv6>;
