@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/rib_ipc_handler.cc,v 1.53 2004/06/10 22:40:33 hodson Exp $"
+#ident "$XORP: xorp/bgp/rib_ipc_handler.cc,v 1.63 2005/03/25 02:52:44 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -24,14 +25,15 @@
 
 #include "bgp.hh"
 #include "rib_ipc_handler.hh"
+#include "profile_vars.hh"
 
-RibIpcHandler::RibIpcHandler(XrlStdRouter *xrl_router, BGPMain& bgp) 
+RibIpcHandler::RibIpcHandler(XrlStdRouter& xrl_router, BGPMain& bgp) 
     : PeerHandler("RIBIpcHandler", NULL, NULL, NULL), 
       _ribname(""),
       _xrl_router(xrl_router),
-      _v4_queue(this, xrl_router, &bgp),
-      _v6_queue(this, xrl_router, &bgp),
-      _fake_id((unsigned int)0)
+      _v4_queue(*this, xrl_router, bgp),
+      _v6_queue(*this, xrl_router, bgp),
+      _fake_id(IPv4::ZERO())
 {
 }
 
@@ -48,7 +50,7 @@ RibIpcHandler::~RibIpcHandler()
 
     set_plumbing(NULL, NULL);
 
-    if("" != _ribname)
+    if (!_ribname.empty())
 	XLOG_WARNING("Deleting RibIpcHandler while still registered with RIB");
     /*
     ** If would be great to de-register from the RIB here. The problem
@@ -63,22 +65,22 @@ RibIpcHandler::register_ribname(const string& r)
     if (_ribname == r)
 	return true;
 
-    if ("" == r) {
-	_ribname.erase();
-	return unregister_rib();
-    }
-
+    string previous_ribname = _ribname;
     _ribname = r;
 
-    XrlRibV0p1Client rib(_xrl_router);
+    if (r.empty()) {
+	return unregister_rib(previous_ribname);
+    }
+
+    XrlRibV0p1Client rib(&_xrl_router);
     //create our tables
     //ebgp - v4
     //name - "ebgp"
     //unicast - true
     //multicast - true
     rib.send_add_egp_table4(_ribname.c_str(),
-		"ebgp", _xrl_router->class_name(),
-                 _xrl_router->instance_name(), true, true,
+		"ebgp", _xrl_router.class_name(),
+                 _xrl_router.instance_name(), true, true,
                  callback(this, 
 			  &RibIpcHandler::rib_command_done,"add_table"));
     //ibgp - v4
@@ -86,8 +88,8 @@ RibIpcHandler::register_ribname(const string& r)
     //unicast - true
     //multicast - true
     rib.send_add_egp_table4(_ribname.c_str(),
-		"ibgp", _xrl_router->class_name(),
-                _xrl_router->instance_name(), true, true,
+		"ibgp", _xrl_router.class_name(),
+                _xrl_router.instance_name(), true, true,
 		callback(this, 
 			 &RibIpcHandler::rib_command_done,"add_table"));
 
@@ -97,8 +99,8 @@ RibIpcHandler::register_ribname(const string& r)
     //unicast - true
     //multicast - true
     rib.send_add_egp_table6(_ribname.c_str(),
-                "ebgp",  _xrl_router->class_name(),
-                _xrl_router->instance_name(), true, true,
+                "ebgp",  _xrl_router.class_name(),
+                _xrl_router.instance_name(), true, true,
 		callback(this, 
  		         &RibIpcHandler::rib_command_done,"add_table"));
     //ibgp - v6
@@ -106,8 +108,8 @@ RibIpcHandler::register_ribname(const string& r)
     //unicast - true
     //multicast - true
     rib.send_add_egp_table6(_ribname.c_str(),
-		"ibgp", _xrl_router->class_name(),
-                _xrl_router->instance_name(), true, true,
+		"ibgp", _xrl_router.class_name(),
+                _xrl_router.instance_name(), true, true,
 		callback(this,
 			 &RibIpcHandler::rib_command_done,"add_table"));
 
@@ -115,18 +117,18 @@ RibIpcHandler::register_ribname(const string& r)
 }
 
 bool
-RibIpcHandler::unregister_rib()
+RibIpcHandler::unregister_rib(string ribname)
 {
-    XrlRibV0p1Client rib(_xrl_router);
+    XrlRibV0p1Client rib(&_xrl_router);
     
     //delete our tables
     //ebgp - v4
     //name - "ebgp"
     //unicast - true
     //multicast - true
-    rib.send_delete_egp_table4(_ribname.c_str(),
-			       "ebgp", _xrl_router->class_name(),
-                               _xrl_router->instance_name(),
+    rib.send_delete_egp_table4(ribname.c_str(),
+			       "ebgp", _xrl_router.class_name(),
+                               _xrl_router.instance_name(),
 			       true, true,
 			       callback(this,
 					&RibIpcHandler::rib_command_done,
@@ -135,9 +137,9 @@ RibIpcHandler::unregister_rib()
     //name - "ibgp"
     //unicast - true
     //multicast - true
-    rib.send_delete_egp_table4(_ribname.c_str(),
-			       "ibgp", _xrl_router->class_name(),
-                               _xrl_router->instance_name(), 
+    rib.send_delete_egp_table4(ribname.c_str(),
+			       "ibgp", _xrl_router.class_name(),
+                               _xrl_router.instance_name(), 
 			       true, true,
 			       callback(this,
 					&RibIpcHandler::rib_command_done,
@@ -148,9 +150,9 @@ RibIpcHandler::unregister_rib()
     //name - "ebgp"
     //unicast - true
     //multicast - true
-    rib.send_delete_egp_table6(_ribname.c_str(),
- 			       "ebgp", _xrl_router->class_name(),
-                               _xrl_router->instance_name(), 
+    rib.send_delete_egp_table6(ribname.c_str(),
+ 			       "ebgp", _xrl_router.class_name(),
+                               _xrl_router.instance_name(), 
 			       true, true,
  			       callback(this,
  				  	&RibIpcHandler::rib_command_done,
@@ -160,9 +162,9 @@ RibIpcHandler::unregister_rib()
     //name - "ibgp"
     //unicast - true
     //multicast - true
-    rib.send_delete_egp_table6(_ribname.c_str(),
- 			       "ibgp", _xrl_router->class_name(),
-                               _xrl_router->instance_name(), 
+    rib.send_delete_egp_table6(ribname.c_str(),
+ 			       "ibgp", _xrl_router.class_name(),
+                               _xrl_router.instance_name(), 
 			       true, true,
  			       callback(this,
  					&RibIpcHandler::rib_command_done,
@@ -182,12 +184,13 @@ RibIpcHandler::start_packet(bool ibgp)
 int 
 RibIpcHandler::add_route(const SubnetRoute<IPv4> &rt, Safi safi)
 {
-    debug_msg("RibIpcHandler::add_route(IPv4) %x\n", (u_int)(&rt));
+    debug_msg("RibIpcHandler::add_route(IPv4) %p\n", &rt);
 
-    if("" == _ribname)
+    if (_ribname.empty())
 	return 0;
 
-    _v4_queue.queue_add_route(_ribname, _ibgp, safi, rt.net(), rt.nexthop());
+    _v4_queue.queue_add_route(_ribname, _ibgp, safi, rt.net(),
+			      rt.nexthop(), rt.policytags());
 
     return 0;
 }
@@ -197,10 +200,11 @@ RibIpcHandler::add_route(const SubnetRoute<IPv6>& rt, Safi safi)
 {
     debug_msg("RibIpcHandler::add_route(IPv6) %p\n", &rt);
 
-    if ("" == _ribname)
+    if (_ribname.empty())
 	return 0;
 
-    _v6_queue.queue_add_route(_ribname, _ibgp, safi, rt.net(), rt.nexthop());
+    _v6_queue.queue_add_route(_ribname, _ibgp, safi, rt.net(), rt.nexthop(),
+			      rt.policytags());
 
     return 0;
 }
@@ -230,9 +234,9 @@ RibIpcHandler::replace_route(const SubnetRoute<IPv6> &old_rt,
 int 
 RibIpcHandler::delete_route(const SubnetRoute<IPv4> &rt, Safi safi)
 {
-    debug_msg("RibIpcHandler::delete_route(IPv4) %x\n", (u_int)(&rt));
+    debug_msg("RibIpcHandler::delete_route(IPv4) %p\n", &rt);
 
-    if("" == _ribname)
+    if (_ribname.empty())
 	return 0;
 
     _v4_queue.queue_delete_route(_ribname, _ibgp, safi, rt.net());
@@ -245,7 +249,7 @@ RibIpcHandler::delete_route(const SubnetRoute<IPv6>& rt, Safi safi)
 {
     debug_msg("RibIpcHandler::delete_route(IPv6) %p\n", &rt);
     UNUSED(rt);
-    if("" == _ribname)
+    if (_ribname.empty())
 	return 0;
 
     _v6_queue.queue_delete_route(_ribname, _ibgp, safi, rt.net());
@@ -282,7 +286,8 @@ RibIpcHandler::push_packet()
 bool 
 RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
 			       const IPv4Net& nlri, const IPv4& next_hop,
-			       const bool& unicast, const bool& multicast)
+			       const bool& unicast, const bool& multicast, 
+			       const PolicyTags& policytags)
 {
     debug_msg("origin %d aspath %s nlri %s next hop %s unicast %d"
 	      " multicast %d\n",
@@ -299,7 +304,8 @@ RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
     */
     SubnetRoute<IPv4>* msg_route 
 	= new SubnetRoute<IPv4>(nlri, &pa_list, NULL);
-    
+    msg_route->set_policytags(policytags);
+   
     /*
     ** Make an internal message.
     */
@@ -326,7 +332,8 @@ RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
 bool 
 RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
 			       const IPv6Net& nlri, const IPv6& next_hop,
-			       const bool& unicast, const bool& multicast)
+			       const bool& unicast, const bool& multicast,
+			       const PolicyTags& policytags)
 {
     debug_msg("origin %d aspath %s nlri %s next hop %s unicast %d"
 	      " multicast %d\n",
@@ -343,6 +350,7 @@ RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
     */
     SubnetRoute<IPv6>* msg_route 
 	= new SubnetRoute<IPv6>(nlri, &pa_list, NULL);
+    msg_route->set_policytags(policytags);
     
     /*
     ** Make an internal message.
@@ -374,6 +382,8 @@ RibIpcHandler::withdraw_route(const IPv4Net& nlri, const bool& unicast,
     debug_msg("nlri %s unicast %d multicast %d\n", nlri.str().c_str(),
 	      unicast, multicast);
 
+// XXX: bug... wrong function called
+#if 0
     /*
     ** Create a subnet route
     */
@@ -388,17 +398,18 @@ RibIpcHandler::withdraw_route(const IPv4Net& nlri, const bool& unicast,
     /*
     ** Inject the message into the plumbing.
     */
+#endif    
     if (unicast) {
-	_plumbing_unicast->delete_route(msg, this);
+	_plumbing_unicast->delete_route(nlri, this);
 	_plumbing_unicast->push<IPv4>(this);
     }
 
     if (multicast) {
-	_plumbing_multicast->delete_route(msg, this);
+	_plumbing_multicast->delete_route(nlri, this);
 	_plumbing_multicast->push<IPv4>(this);
     }
 
-    msg_route->unref();
+//    msg_route->unref();
 
     return true;
 }
@@ -410,6 +421,8 @@ RibIpcHandler::withdraw_route(const IPv6Net& nlri, const bool& unicast,
     debug_msg("nlri %s unicast %d multicast %d\n", nlri.str().c_str(),
 	      unicast, multicast);
 
+// XXX: bug... wrong function called
+#if 0
     /*
     ** Create a subnet route
     */
@@ -424,24 +437,25 @@ RibIpcHandler::withdraw_route(const IPv6Net& nlri, const bool& unicast,
     /*
     ** Inject the message into the plumbing.
     */
+#endif    
     if (unicast) {
-	_plumbing_unicast->delete_route(msg, this);
+	_plumbing_unicast->delete_route(nlri, this);
 	_plumbing_unicast->push<IPv6>(this);
     }
 
     if (multicast) {
-	_plumbing_multicast->delete_route(msg, this);
+	_plumbing_multicast->delete_route(nlri, this);
 	_plumbing_multicast->push<IPv6>(this);
     }
 
-    msg_route->unref();
+//    msg_route->unref();
 
     return true;
 }
 
 template<class A>
-XrlQueue<A>::XrlQueue(RibIpcHandler *rib_ipc_handler,
-		      XrlStdRouter *xrl_router, BGPMain *bgp)
+XrlQueue<A>::XrlQueue(RibIpcHandler& rib_ipc_handler,
+		      XrlStdRouter& xrl_router, BGPMain& bgp)
     : _rib_ipc_handler(rib_ipc_handler),
       _xrl_router(xrl_router), _bgp(bgp),
       _flying(0)
@@ -452,16 +466,21 @@ template<class A>
 EventLoop& 
 XrlQueue<A>::eventloop() const 
 { 
-    return _rib_ipc_handler->eventloop(); 
+    return _rib_ipc_handler.eventloop(); 
 }
 
 template<class A>
 void
 XrlQueue<A>::queue_add_route(string ribname, bool ibgp, Safi safi,
-			     const IPNet<A>& net, const A& nexthop)
+			     const IPNet<A>& net, const A& nexthop, 
+			     const PolicyTags& policytags)
 {
     Queued q;
 
+    if (_bgp.profile().enabled(profile_route_rpc_in))
+	_bgp.profile().log(profile_route_rpc_in,
+			   c_format("add %s", net.str().c_str()));
+
     q.add = true;
     q.ribname = ribname;
     q.ibgp = ibgp;
@@ -475,6 +494,7 @@ XrlQueue<A>::queue_add_route(string ribname, bool ibgp, Safi safi,
 		 safi,
 		 net.str().c_str(),
 		 nexthop.str().c_str());
+    q.policytags = policytags;
 
     _xrl_queue.push_back(q);
 
@@ -488,6 +508,10 @@ XrlQueue<A>::queue_delete_route(string ribname, bool ibgp, Safi safi,
 {
     Queued q;
 
+    if (_bgp.profile().enabled(profile_route_rpc_in))
+	_bgp.profile().log(profile_route_rpc_in,
+			   c_format("delete %s", net.str().c_str()));
+
     q.add = false;
     q.ribname = ribname;
     q.ibgp = ibgp;
@@ -529,12 +553,12 @@ XrlQueue<A>::start()
     // route commands as possible as possible.
 
     for(;;) {
-	debug_msg("queue length %u\n", (uint32_t)_xrl_queue.size());
+	debug_msg("queue length %u\n", XORP_UINT_CAST(_xrl_queue.size()));
 
 	if(_xrl_queue.empty()) {
 	    debug_msg("Output no longer busy\n");
 #if	0
-	    _rib_ipc_handler->output_no_longer_busy();
+	    _rib_ipc_handler.output_no_longer_busy();
 #endif
 	    return;
 	}
@@ -588,17 +612,24 @@ XrlQueue<IPv4>::sendit_spec(Queued& q, const char *bgp)
 	break;
     }
 
-    XrlRibV0p1Client rib(_xrl_router);
+    XrlRibV0p1Client rib(&_xrl_router);
     if(q.add) {
 	debug_msg("adding route from %s peer to rib\n", bgp);
+	if (_bgp.profile().enabled(profile_route_rpc_out))
+	    _bgp.profile().log(profile_route_rpc_out, 
+			       c_format("add %s", q.net.str().c_str()));
 	sent = rib.send_add_route4(q.ribname.c_str(),
 			    bgp,
 			    unicast, multicast,
-			    q.net, q.nexthop, /*metric*/0, 
+			    q.net, q.nexthop, /*metric*/0,
+			    q.policytags.xrl_atomlist(),
 			    callback(this, &XrlQueue::route_command_done,
 				     q.comment));
     } else {
 	debug_msg("deleting route from %s peer to rib\n", bgp);
+	if (_bgp.profile().enabled(profile_route_rpc_out))
+	    _bgp.profile().log(profile_route_rpc_out, 
+			       c_format("delete %s", q.net.str().c_str()));
 	sent = rib.send_delete_route4(q.ribname.c_str(),
 				      bgp,
 				      unicast, multicast,
@@ -628,17 +659,24 @@ XrlQueue<IPv6>::sendit_spec(Queued& q, const char *bgp)
 	break;
     }
 
-    XrlRibV0p1Client rib(_xrl_router);
+    XrlRibV0p1Client rib(&_xrl_router);
     if(q.add) {
 	debug_msg("adding route from %s peer to rib\n", bgp);
+	if (_bgp.profile().enabled(profile_route_rpc_out))
+	    _bgp.profile().log(profile_route_rpc_out, 
+			       c_format("add %s", q.net.str().c_str()));
 	sent = rib.send_add_route6(q.ribname.c_str(),
 			    bgp,
 			    unicast, multicast,
 			    q.net, q.nexthop, /*metric*/0, 
+			    q.policytags.xrl_atomlist(),
 			    callback(this, &XrlQueue::route_command_done,
 				     q.comment));
     } else {
 	debug_msg("deleting route from %s peer to rib\n", bgp);
+	if (_bgp.profile().enabled(profile_route_rpc_out))
+	    _bgp.profile().log(profile_route_rpc_out, 
+			       c_format("delete %s", q.net.str().c_str()));
 	sent = rib.send_delete_route6(q.ribname.c_str(),
 			       bgp,
 			       unicast, multicast,
@@ -678,7 +716,7 @@ XrlQueue<A>::route_command_done(const XrlError& error,
     case NO_FINDER:
 	// XXX - Temporarily code dump if this condition occurs.
 	XLOG_FATAL("NO FINDER");
-	_bgp->finder_death(__FILE__, __LINE__);
+	_bgp.finder_death(__FILE__, __LINE__);
 	break;
 
     case BAD_ARGS:
