@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_debug.cc,v 1.10 2004/06/10 22:40:33 hodson Exp $"
+#ident "$XORP: xorp/bgp/route_table_debug.cc,v 1.13 2005/03/25 02:52:45 pavlin Exp $"
 
 //#define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -30,6 +30,7 @@ DebugTable<A>::DebugTable(string table_name,
     _print_tablename = false;
     _msgs = 0;
     _ofile = NULL;
+    _get_on_wakeup = false;
 }
 
 template<class A>
@@ -149,6 +150,13 @@ DebugTable<A>::route_used(const SubnetRoute<A>* rt, bool in_use){
     this->_parent->route_used(rt, in_use);
 }
 
+template<class A>
+void
+DebugTable<A>::wakeup() {
+    if (_get_on_wakeup)
+	while (this->_parent->get_next_message(this)) {}
+}
+
 template<class A>
 string
 DebugTable<A>::str() const {
