@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,20 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_decision.cc,v 1.28 2004/06/25 12:32:08 mjh Exp $"
+#ident "$XORP: xorp/bgp/route_table_decision.cc,v 1.37 2005/03/25 02:52:45 pavlin Exp $"
 
-//#define DEBUG_LOGGING
+// #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
+// All routes hold some internal state for example: is this route
+// resolvable? This internal state is required because the state of
+// a route may change. For example a route may go from being
+// resolvable to unresolvable. To maintain a consistent internal state
+// in the plumbing it is important when considering routes on other
+// branches to look at the cached state in the route itself, rather
+// than the actual state. Never change the state of a route that is
+// not on the current peering.
+
 #include "bgp_module.h"
 #include "libxorp/xlog.h"
 #include "route_table_decision.hh"
@@ -36,12 +45,25 @@ DecisionTable<A>::DecisionTable(string table_name,
 {
 }
 
+template<class A>
+DecisionTable<A>::~DecisionTable()
+{
+    //clean up - this should really be done by removing the parent
+    //when the peering goes down, but we don't do that currently - we
+    //reuse the existing branch because that makes deleting old routes
+    //much easier.  
+    typename map<BGPRouteTable<A>*, PeerTableInfo<A>* >::iterator i;
+    for (i = _parents.begin();  i != _parents.end();  i++) {
+	delete i->second;
+    }
+}
+
 template<class A>
 int
 DecisionTable<A>::add_parent(BGPRouteTable<A> *new_parent,
 			     PeerHandler *peer_handler,
 			     uint32_t genid) {
-    debug_msg("DecisionTable<A>::add_parent: %x\n", (u_int)new_parent);
+    debug_msg("DecisionTable<A>::add_parent: %p\n", new_parent);
     if (_parents.find(new_parent)!=_parents.end()) {
 	//the parent is already in the set
 	return -1;
@@ -57,7 +79,7 @@ DecisionTable<A>::add_parent(BGPRouteTable<A> *new_parent,
 template<class A>
 int
 DecisionTable<A>::remove_parent(BGPRouteTable<A> *ex_parent) {
-    debug_msg("DecisionTable<A>::remove_parent: %x\n", (u_int)ex_parent);
+    debug_msg("DecisionTable<A>::remove_parent: %p\n", ex_parent);
     typename map<BGPRouteTable<A>*, PeerTableInfo<A>* >::iterator i;
     i = _parents.find(ex_parent);
     PeerTableInfo<A> *pti = i->second;
@@ -80,6 +102,8 @@ DecisionTable<A>::add_route(const InternalMessage<A> &rtmsg,
 
     //if the nexthop isn't resolvable, don't even consider the route
     debug_msg("testing resolvability\n");
+    XLOG_ASSERT(rtmsg.route()->nexthop_resolved() ==
+		resolvable(rtmsg.route()->nexthop()));
     if (!resolvable(rtmsg.route()->nexthop())) {
 	debug_msg("route not resolvable\n");
     	return ADD_UNUSED;
@@ -108,12 +132,12 @@ DecisionTable<A>::add_route(const InternalMessage<A> &rtmsg,
 	//the new route wins by default
 	new_winner = &new_route;
     }
-    assert(new_winner != NULL);
+    XLOG_ASSERT(new_winner != NULL);
 
     if (old_winner_clone != NULL) {
 	if (old_winner_clone->route() == new_winner->route()) {
 	    //the winner didn't change.
-	    assert(old_winner_clone != NULL);
+	    XLOG_ASSERT(old_winner_clone != NULL);
 	    delete old_winner_clone;
 	    return ADD_UNUSED;
 	}
@@ -175,6 +199,7 @@ DecisionTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
 	//preserve this, because the original old_winner's data will
 	//be clobbered by route_wins.
 	old_winner_clone = new RouteData<A>(*old_winner);
+
     } else if (old_rtmsg.route()->is_winner()) {
 	//the route being deleted was the old winner
 	old_winner_clone = new RouteData<A>(old_rtmsg.route(), caller,
@@ -220,7 +245,8 @@ DecisionTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
     const InternalMessage<A> *old_rtmsg_p, *new_rtmsg_p;
     if (old_winner_clone->route() == old_rtmsg.route()) {
 	old_rtmsg.force_clear_push();
-	old_rtmsg.route()->set_is_not_winner();
+	// FIXME: hack to enable policy route pushing.
+//	old_rtmsg.route()->set_is_not_winner();
 	old_rtmsg_p = &old_rtmsg;
     } else {
 	old_rtmsg_p = new InternalMessage<A>(old_winner_clone->route(), 
@@ -274,7 +300,7 @@ DecisionTable<A>::delete_route(const InternalMessage<A> &rtmsg,
     debug_msg("delete route: %s\n",
 	      rtmsg.route()->str().c_str());
     PARANOID_ASSERT(_parents.find(caller) != _parents.end());
-    assert(this->_next_table != NULL);
+    XLOG_ASSERT(this->_next_table != NULL);
 
     //find the alternative routes, and the old winner if there was one.
     RouteData<A> *old_winner = NULL, *old_winner_clone = NULL;
@@ -310,7 +336,7 @@ DecisionTable<A>::delete_route(const InternalMessage<A> &rtmsg,
 	if (new_winner != NULL
 	    && old_winner_clone->route() == new_winner->route()) {
 	    //the winner didn't change.
-	    assert(old_winner_clone != NULL);
+	    XLOG_ASSERT(old_winner_clone != NULL);
 	    delete old_winner_clone;
 	    return -1;
 	}
@@ -435,7 +461,7 @@ DecisionTable<A>::find_alternative_routes(
 						    pti->peer_handler(),
 						    found_genid));
 		if (found_route->is_winner()) {
-		    assert(previous_winner == NULL);
+		    XLOG_ASSERT(previous_winner == NULL);
 		    previous_winner = &(alternatives.back());
 		}
 	    }
@@ -515,9 +541,9 @@ DecisionTable<A>::igp_distance(const A nexthop) const
 	XLOG_FATAL("This next hop must be known %s", nexthop.str().c_str());
 
     if (resolvable)
-	debug_msg("Decision: IGP distance for %s is %d\n",
+	debug_msg("Decision: IGP distance for %s is %u\n",
 		  nexthop.str().c_str(),
-		  metric);
+		  XORP_UINT_CAST(metric));
     else
 	debug_msg("Decision: IGP distance for %s is unknown\n",
 		  nexthop.str().c_str());
@@ -525,297 +551,6 @@ DecisionTable<A>::igp_distance(const A nexthop) const
     return metric;
 }
 
-/*
-** The main decision process.
-**
-** return true if test_route is better than current_route.
-**        resolved is set to true if the winning route resolves.
-*/
-template<class A>
-bool 
-DecisionTable<A>::route_wins(const SubnetRoute<A> *test_route,
-			     const PeerHandler *test_peer,
-			     list<RouteData<A> >& alternatives) const
-{
-    typename list<RouteData<A> >::iterator i;
-
-    /* The spec seems pretty odd.  In our architecture, it seems
-       reasonable to do phase 2 before phase 1, because if a route
-       isn't resolvable we don't want to consider it */
-
-    /*
-    ** Phase 2: Route Selection.  */
-    /*
-    ** Check if routes resolve.
-    */
-    bool test_resolvable = resolvable(test_route->nexthop());
-
-    /*We should never even call route_is_better if the test_route
-      isn't resolvable */
-    assert(test_resolvable);
-
-    for (i=alternatives.begin(); i!=alternatives.end();) {
-	if (!resolvable(i->route()->nexthop())) {
-	    i = alternatives.erase(i);
-	} else {
-	    i++;
-	}
-    }
-    
-    /* If there are no resolvable alternatives, the test route wins */
-    if (alternatives.empty()) {
-	return true;
-    }
-
-    /* By this point, all remaining routes are resolvable */
-
-    /* 
-    ** Phase 1: Calculation of degree of preference.
-    */
-    int test_pref = local_pref(test_route, test_peer);
-    for (i=alternatives.begin(); i!=alternatives.end();) {
-	int lp = local_pref(i->route(), i->peer_handler());
-	assert(lp >= 0);
-	//prefer higher preference
-	if (lp < test_pref) {
-	    i = alternatives.erase(i);
-	} else if (lp > test_pref) {
-	    return false;
-	} else {
-	    i++;
-	}
-    }
-
-    if(alternatives.empty()) {
-	return true;
-    }
-
-    /*
-    ** If we got here the preferences for both routes are the same.
-    */
-
-    /*
-    ** Here we are crappy tie breaking.
-    */
-    
-    debug_msg("tie breaking\n");
-    debug_msg("AS path test\n");
-    /*
-    ** Shortest AS path length.
-    */
-    int test_aspath_length = test_route->attributes()->
-	aspath().path_length();
-    debug_msg("%s length %d\n", test_route->attributes()->
-	      aspath().str().c_str(), test_aspath_length);
-
-    for (i=alternatives.begin(); i!=alternatives.end();) {
-	int len = i->route()->attributes()->aspath().path_length();
-	assert(len >= 0);
-	//prefer shortest path
-	if (len > test_aspath_length) {
-	    i = alternatives.erase(i);
-	} else if (len < test_aspath_length) {
-	    return false;
-	} else {
-	    i++;
-	}
-    }
-
-    
-    if(alternatives.empty()) {
-	return true;
-    }
-
-    debug_msg("origin test\n");
-    /*
-    ** Lowest origin value.
-    */
-    int test_origin = test_route->attributes()->origin();
-    for (i=alternatives.begin(); i!=alternatives.end();) {
-	int origin = i->route()->attributes()->origin();
-	//prefer lower origin
-	if (origin > test_origin) {
-	    i = alternatives.erase(i);
-	} else if (origin < test_origin) {
-	    return false;
-	} else {
-	    i++;
-	}
-    }
-
-    if(alternatives.empty()) {
-	return true;
-    }
-
-    debug_msg("MED test\n");
-    /*
-    ** Compare meds if both routes came from the same neighbour AS.
-    */
-
-    AsNum test_asnum(test_route->attributes()->aspath().first_asnum());
-    int test_med = med(test_route);
-    for (i=alternatives.begin(); i!=alternatives.end();) {
-	AsNum asnum = (i->route()->attributes()->aspath().first_asnum());
-	int alt_med = med(i->route());
-	if (test_asnum == asnum) {
-	    //prefer lower MED
-	    if (test_med < alt_med) {
-		i = alternatives.erase(i);
-	    } else if (test_med > alt_med) {
-		return false;
-	    } else {
-		i++;
-	    }
-	} else {
-	    i++;
-	}
-    }
-
-    if(alternatives.empty()) {
-	return true;
-    }
-
-    //remove any other routes from consideration if they come from the
-    //same AS as another route, and loses on MED to that other route.
-    typename list <RouteData<A> >::iterator j;
-    for (i=alternatives.begin(); i!=alternatives.end();) {
-	AsNum asnum1 = (i->route()->attributes()->aspath().first_asnum());
-	int med1 = med(i->route());
-	bool del_i = false;
-	for (j=alternatives.begin(); j!=alternatives.end();) {
-	    bool del_j = false;
-	    if (i != j) {
-		AsNum asnum2 
-		    = (j->route()->attributes()->aspath().first_asnum());
-		int med2 = med(j->route());
-		if (asnum1 == asnum2) {
-		    if (med1 > med2) {
-			i = alternatives.erase(i);
-			del_i = true;
-			break;
-		    } else if (med1 < med2) {
-			j = alternatives.erase(j);
-			del_j = true;
-		    }
-		} 
-	    }
-	    //move to the next j if we didn't already do it using erase
-	    if (del_j == false) {
-		j++;
-	    }
-	}	
-	//move to the next i if we didn't already do it using erase
-	if (del_i == false) {
-	    i++;
-	}
-    }
-
-    if(alternatives.empty()) {
-	//the loop above can't remove the last alternative
-	XLOG_UNREACHABLE();
-    }
-
-
-    //Sanity checking.  If this conditions is hit, then the
-    //input sanity checks haven't done their job.
-    if (!test_peer->ibgp() && (test_asnum != test_peer->AS_number()))
-	XLOG_FATAL("AS Path %s received from EBGP peer with AS %s\n",
-		   test_route->attributes()->aspath().str().c_str(),
-		   test_peer->AS_number().str().c_str());
-    
-    debug_msg("EBGP vs IBGP test\n");
-    /*
-    ** Prefer routes from external peers over internal peers.
-    */
-    bool test_ibgp = test_peer->ibgp();
-    for (i=alternatives.begin(); i!=alternatives.end();) {
-	bool ibgp = i->peer_handler()->ibgp();
-	if ((!test_ibgp) && ibgp) {
-	    //test route is external, alternative is internal
-	    i = alternatives.erase(i);
-	} else if (test_ibgp && !ibgp) {
-	    //test route is internal, alternative is external
-	    return false;
-	} else {
-	    i++;
-	}
-    }
-
-    if (alternatives.empty()) {
-	return true;
-    }
-
-
-    debug_msg("IGP distance test\n");
-    /*
-    ** Compare IGP distances.
-    */
-    int test_igp_distance = igp_distance(test_route->nexthop());
-    for (i=alternatives.begin(); i!=alternatives.end(); ) {
-	int igp_dist = igp_distance(i->route()->nexthop());
-	//prefer lower IGP distance
-	if (test_igp_distance < igp_dist) {
-	    i = alternatives.erase(i);
-	} else if (test_igp_distance > igp_dist) {
-	    return false;
-	} else {
-	    i++;
-	}
-    }
-
-    if (alternatives.empty()) {
-	return true;
-    }
-
-
-    debug_msg("BGP ID test\n");
-    /*
-    ** Choose the route from the neighbour with the lowest BGP ID.
-    */
-    IPv4 test_id = test_peer->id();
-    for (i=alternatives.begin(); i!=alternatives.end();) {
-	IPv4 id = i->peer_handler()->id();
-	//prefer lower ID distance
-	if (test_id < id) {
-	    i = alternatives.erase(i);
-	} else if (test_id > id) {
-	    return false;
-	} else {
-	    i++;
-	}
-    }
-
-    if (alternatives.empty()) {
-	return true;
-    }
-
-
-    debug_msg("neighbour addr test\n");
-    /*
-    ** Choose the route from the neighbour with the lowest neighbour
-    ** address.
-    */
-    int test_address = test_peer->neighbour_address();
-    for (i=alternatives.begin(); i!=alternatives.end();) {
-	int address = i->peer_handler()->neighbour_address();
-	//prefer lower address
-	if (test_address < address) {
-	    i = alternatives.erase(i);
-	} else if (test_address > address) {
-	    return false;
-	} else {
-	    i++;
-	}
-    }
-
-    if (alternatives.empty()) {
-	return true;
-    }
-
-    //We can get here if we compare two identically rated routes.  
-    return false;
-}
-
 /*
 ** The main decision process.
 **
@@ -838,7 +573,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
     ** Check if routes resolve.
     */
     for (i=alternatives.begin(); i!=alternatives.end();) {
-	if (!resolvable(i->route()->nexthop())) {
+	if (!i->route()->nexthop_resolved()) {
 	    i = alternatives.erase(i);
 	} else {
 	    i++;
@@ -863,7 +598,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
     i = alternatives.begin(); i++;
     while(i!=alternatives.end()) {
 	int lp = local_pref(i->route(), i->peer_handler());
-	assert(lp >= 0);
+	XLOG_ASSERT(lp >= 0);
 	//prefer higher preference
 	if (lp < test_pref) {
 	    i = alternatives.erase(i);
@@ -900,7 +635,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
     i = alternatives.begin(); i++;
     while(i!=alternatives.end()) {
 	int len = i->route()->attributes()->aspath().path_length();
-	assert(len >= 0);
+	XLOG_ASSERT(len >= 0);
 	//prefer shortest path
 	if (len > test_aspath_length) {
 	    i = alternatives.erase(i);
