@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_deletion.cc,v 1.18 2004/06/10 22:40:34 hodson Exp $"
+#ident "$XORP: xorp/bgp/route_table_deletion.cc,v 1.20 2005/03/25 02:52:46 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -49,8 +49,8 @@ int
 DeletionTable<A>::add_route(const InternalMessage<A> &rtmsg,
 			    BGPRouteTable<A> *caller)
 {
-    debug_msg("DeletionTable<A>::add_route %x on %s\n",
-	      (u_int)(&rtmsg), this->tablename().c_str());
+    debug_msg("DeletionTable<A>::add_route %p on %s\n",
+	      &rtmsg, this->tablename().c_str());
     XLOG_ASSERT(caller == this->_parent);
     XLOG_ASSERT(this->_next_table != NULL);
 
@@ -97,8 +97,8 @@ DeletionTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
 				const InternalMessage<A> &new_rtmsg,
 				BGPRouteTable<A> *caller)
 {
-    debug_msg("DeletionTable<A>::replace_route %x -> %x on %s\n",
-	      (u_int)(&old_rtmsg), (u_int)(&new_rtmsg), this->tablename().c_str());
+    debug_msg("DeletionTable<A>::replace_route %p -> %p on %s\n",
+	      &old_rtmsg, &new_rtmsg, this->tablename().c_str());
     XLOG_ASSERT(caller == this->_parent);
     XLOG_ASSERT(this->_next_table != NULL);
     XLOG_ASSERT(old_rtmsg.net() == new_rtmsg.net());
@@ -134,8 +134,8 @@ int
 DeletionTable<A>::delete_route(const InternalMessage<A> &rtmsg,
 			       BGPRouteTable<A> *caller)
 {
-    debug_msg("DeletionTable<A>::delete_route %x on %s\n",
-	      (u_int)(&rtmsg), this->tablename().c_str());
+    debug_msg("DeletionTable<A>::delete_route %p on %s\n",
+	      &rtmsg, this->tablename().c_str());
     XLOG_ASSERT(caller == this->_parent);
     XLOG_ASSERT(this->_next_table != NULL);
     // we should never see a delete for a net that's in the deletion cache
