@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_fanout.cc,v 1.37 2004/06/10 22:40:34 hodson Exp $"
+#ident "$XORP: xorp/bgp/route_table_fanout.cc,v 1.51 2005/04/08 22:45:59 mjh Exp $"
 
 //#define DEBUG_LOGGING
 //#define DEBUG_PRINT_FUNCTION_NAME
@@ -65,10 +66,10 @@ NextTableMap<A>::erase(iterator& iter)
     typename map<BGPRouteTable<A> *, PeerTableInfo<A>* >::iterator i;
     i = _next_tables.find(prpair->route_table());
     XLOG_ASSERT(i != _next_tables.end());
+    uint32_t id = i->second->peer_handler()->id().addr();
     _next_tables.erase(i);
 
-    typename map <uint32_t, PeerTableInfo<A>* >::iterator j;
-    uint32_t id = i->second->peer_handler()->id().addr();
+    typename multimap<uint32_t, PeerTableInfo<A>* >::iterator j;
     j = _next_table_order.find(id);
     while (j->first == id  && j->second != prpair) {
 	//find the right one.
@@ -90,7 +91,7 @@ NextTableMap<A>::find(BGPRouteTable<A> *next_table)
     if (i == _next_tables.end())
 	return end();
     PeerTableInfo<A>* prpair = i->second;
-    typename map <uint32_t, PeerTableInfo<A>* >::iterator j;
+    typename multimap<uint32_t, PeerTableInfo<A>* >::iterator j;
     uint32_t id = i->second->peer_handler()->id().addr();
     j = _next_table_order.find(id);
     while (j->first == id  && j->second != prpair) {
@@ -130,15 +131,17 @@ int
 FanoutTable<A>::add_next_table(BGPRouteTable<A> *new_next_table,
 			       const PeerHandler *ph, uint32_t genid) 
 {
-    debug_msg("FanoutTable<IPv%u:%s>::add_next_table %x %s\n",
-	      A::ip_version(), pretty_string_safi(this->safi()),
-	      (u_int)new_next_table, new_next_table->tablename().c_str());
+    debug_msg("FanoutTable<IPv%u:%s>::add_next_table %p %s\n",
+	      XORP_UINT_CAST(A::ip_version()),
+	      pretty_string_safi(this->safi()),
+	      new_next_table, new_next_table->tablename().c_str());
 
     if (_next_tables.find(new_next_table) != _next_tables.end()) {
 	// the next_table is already in the set
 	return -1;
     }
     _next_tables.insert(new_next_table, ph, genid);
+    new_next_table->peering_came_up(ph, genid, this);
     return 0;
 }
 
@@ -206,42 +209,41 @@ FanoutTable<A>::add_route(const InternalMessage<A> &rtmsg,
     const PeerHandler *origin_peer = rtmsg.origin_peer();
 
     typename NextTableMap<A>::iterator i = _next_tables.begin();
-    int result = ADD_USED, r;
     list <PeerTableInfo<A>*> queued_peers;
     while (i != _next_tables.end()) {
 	const PeerHandler *next_peer = i.second().peer_handler();
 	if (origin_peer == next_peer) {
 	    // don't send the route back to the peer it came from
-	    debug_msg("FanoutTable<IPv%u%s>::add_route %x.\n  Don't send back to %s\n",
-		      A::ip_version(), pretty_string_safi(this->safi()),
-		      (u_int)(&rtmsg), (i.first())->tablename().c_str());
+	    debug_msg("FanoutTable<IPv%u%s>::add_route %p.\n  Don't send back to %s\n",
+		      XORP_UINT_CAST(A::ip_version()),
+		      pretty_string_safi(this->safi()),
+		      &rtmsg, (i.first())->tablename().c_str());
 	} else {
-	    debug_msg("FanoutTable<IPv%u%s>::add_route %x to %s\n",
-		      A::ip_version(), pretty_string_safi(this->safi()),
-		      (u_int)(&rtmsg), (i.first())->tablename().c_str());
-
-	    if (i.second().busy()) {
-		debug_msg("Fanout: queuing route, queue len is %u\n",
-			  (uint32_t)queued_peers.size());
-		queued_peers.push_back(&(i.second()));
-		r = ADD_USED;
-	    } else {
-		debug_msg("Fanout: passing route on\n");
-		r = i.first()->add_route(rtmsg, (BGPRouteTable<A>*)this);
-	    }
+	    debug_msg("FanoutTable<IPv%u%s>::add_route %p to %s\n",
+		      XORP_UINT_CAST(A::ip_version()),
+		      pretty_string_safi(this->safi()),
+		      &rtmsg, (i.first())->tablename().c_str());
+	    //we now always add to the queue, and require a
+	    //get_next_message from downstream to liberate a queued
+	    //entry.
+	    debug_msg("Fanout: queuing route, queue len is %u\n",
+		      XORP_UINT_CAST(queued_peers.size()));
+	    queued_peers.push_back(&(i.second()));
 
 	    // we don't bother to return ADD_UNUSED or ADD_FILTERED,
 	    // but if there's a failure, we'll return the last failure
 	    // code.  Not clear how useful this is, but best not to
 	    // hide the failure completely.
-	    if (r != ADD_USED && r != ADD_UNUSED && r != ADD_FILTERED)
-		result = r;
 	}
 	i++;
     }
-    if (queued_peers.empty() == false)
+    //queue it if anyone needs it.
+    if (queued_peers.empty() == false) {
 	add_to_queue(RTQUEUE_OP_ADD, rtmsg, queued_peers);
-    return result;
+	wakeup_downstream(queued_peers);
+    }
+
+    return ADD_USED;
 }
 
 template<class A>
@@ -272,7 +274,6 @@ FanoutTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
     const PeerHandler *origin_peer = old_rtmsg.origin_peer();
     XLOG_ASSERT(origin_peer == new_rtmsg.origin_peer());
 
-    int result = ADD_USED, r;
     list <PeerTableInfo<A>*> queued_peers;
     typename NextTableMap<A>::iterator i;
     for (i = _next_tables.begin();  i != _next_tables.end();  i++) {
@@ -280,31 +281,23 @@ FanoutTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
 	if (origin_peer == next_peer) {
 	    // don't send the route back to the peer it came from
 	} else {
-	    debug_msg("FanoutTable<IPv%u:%s>::replace_route %x -> %x to %s\n",
-		      A::ip_version(), pretty_string_safi(this->safi()),
-		      (u_int)(&old_rtmsg), (u_int)(&new_rtmsg),
+	    debug_msg("FanoutTable<IPv%u:%s>::replace_route %p -> %p to %s\n",
+		      XORP_UINT_CAST(A::ip_version()),
+		      pretty_string_safi(this->safi()),
+		      &old_rtmsg, &new_rtmsg,
 		      (i.first())->tablename().c_str());
 
-	    if (i.second().busy()) {
-		debug_msg("queueing replace_route\n");
-		queued_peers.push_back(&(i.second()));
-		r = ADD_USED;
-	    } else {
-		debug_msg("not queuing replace_route\n");
-		r = i.first()->replace_route(old_rtmsg, new_rtmsg,
-					    (BGPRouteTable<A>*)this);
-	    }
+	    debug_msg("queueing replace_route\n");
+	    queued_peers.push_back(&(i.second()));
 
-	    // we don't bother to return ADD_UNUSED, but if there's a
-	    // failure, we'll return the last failure code.  Not clear how
-	    // useful this is, but best not to hide the failure completely.
-	    if (r != ADD_USED && r != ADD_UNUSED && r != ADD_FILTERED)
-		result = r;
 	}
     }
-    if (queued_peers.empty() == false)
+    if (queued_peers.empty() == false) {
 	add_replace_to_queue(old_rtmsg, new_rtmsg, queued_peers);
-    return result;
+	wakeup_downstream(queued_peers);
+    }
+
+    return ADD_USED;
 }
 
 template<class A>
@@ -329,23 +322,24 @@ FanoutTable<A>::delete_route(const InternalMessage<A> &rtmsg,
     for (i = _next_tables.begin();  i != _next_tables.end();  i++) {
 	const PeerHandler *next_peer = i.second().peer_handler();
 	if (origin_peer == next_peer) {
-	    debug_msg("FanoutTable<IPv%u:%s>::delete_route %x.\n  Don't send back to %s\n",
-		      A::ip_version(), pretty_string_safi(this->safi()),
-		      (u_int)(&rtmsg), (i.first())->tablename().c_str());
+	    debug_msg("FanoutTable<IPv%u:%s>::delete_route %p.\n  Don't send back to %s\n",
+		      XORP_UINT_CAST(A::ip_version()),
+		      pretty_string_safi(this->safi()),
+		      &rtmsg, (i.first())->tablename().c_str());
 	} else {
-	    debug_msg("FanoutTable<IPv%u:%s>::delete_route %x to %s\n",
-		      A::ip_version(), pretty_string_safi(this->safi()),
-		      (u_int)(&rtmsg), (i.first())->tablename().c_str());
-	    if (i.second().busy()) {
-		queued_peers.push_back(&(i.second()));
-	    } else {
-		XLOG_ASSERT(!i.second().has_queued_data());
-		i.first()->delete_route(rtmsg, (BGPRouteTable<A>*)this);
-	    }
+	    debug_msg("FanoutTable<IPv%u:%s>::delete_route %p to %s\n",
+		      XORP_UINT_CAST(A::ip_version()),
+		      pretty_string_safi(this->safi()),
+		      &rtmsg, (i.first())->tablename().c_str());
+	    queued_peers.push_back(&(i.second()));
+
 	}
     }
-    if (queued_peers.empty() == false)
+    if (queued_peers.empty() == false) {
 	add_to_queue(RTQUEUE_OP_DELETE, rtmsg, queued_peers);
+	wakeup_downstream(queued_peers);
+    }
+
     return 0;
 }
 
@@ -367,7 +361,6 @@ FanoutTable<A>::route_dump(const InternalMessage<A> &rtmsg,
 	}
     }
     XLOG_ASSERT(i != _next_tables.end());
-    XLOG_ASSERT(i.second().busy());
 
     int result;
     result = dump_child->route_dump(rtmsg, (BGPRouteTable<A>*)this, dump_peer);
@@ -388,21 +381,33 @@ FanoutTable<A>::push(BGPRouteTable<A> *caller)
 	// a push needs to go to all peers because an add may cause a
 	// delete (or vice versa) of a route that originated from a
 	// different peer.
-	if (i.second().busy()) {
-	    queued_peers.push_back(&(i.second()));
-	} else {
-	    i.first()->push((BGPRouteTable<A>*)this);
-	}
+	queued_peers.push_back(&(i.second()));
     }
 
-    if (queued_peers.empty() == false)
+    if (queued_peers.empty() == false) {
 	// if the origin peer we send to add_push_to_queue is NULL, the
 	// push will go to all peers
 	add_push_to_queue(queued_peers, NULL);
+	wakeup_downstream(queued_peers);
+    }
 
     return 0;
 }
 
+template<class A>
+void
+FanoutTable<A>::wakeup_downstream(list <PeerTableInfo<A>*>& queued_peers)
+{
+    //send a wakeup downstream if they're currently waiting
+    typename list <PeerTableInfo<A>*>::iterator i;
+    for (i = queued_peers.begin(); i != queued_peers.end(); i++) {
+	if ((*i)->is_ready()) {
+	    (*i)->wakeup_sent();
+	    (*i)->route_table()->wakeup();
+	}
+    }
+}
+
 template<class A>
 const SubnetRoute<A>*
 FanoutTable<A>::lookup_route(const IPNet<A> &net, uint32_t& genid) const 
@@ -437,6 +442,18 @@ FanoutTable<A>::remove_dump_table(DumpTable<A> *dump_table)
     _dump_tables.erase(i);
 }
 
+template<class A>
+void
+FanoutTable<A>::peer_table_info(list<const PeerTableInfo<A>*>& peer_list) {
+    typename NextTableMap<A>::iterator i;
+    
+    for (i = _next_tables.begin(); i != _next_tables.end(); i++) {
+	if (i.second().peer_handler() != NULL)
+	    peer_list.push_back(&(i.second()));
+    }
+
+}
+
 template<class A>
 int
 FanoutTable<A>::dump_entire_table(BGPRouteTable<A> *child_to_dump_to,
@@ -474,7 +491,6 @@ FanoutTable<A>::dump_entire_table(BGPRouteTable<A> *child_to_dump_to,
 	    peer_info = &(i.second());
     }
     XLOG_ASSERT(peer_info != NULL);
-    peer_info->set_busy(true);
 
     add_dump_table(dump_table);
 
@@ -499,7 +515,8 @@ FanoutTable<A>::add_to_queue(RouteQueueOp operation,
     set_queue_positions(queued_peers);
 
     if (rtmsg.push())
-	add_push_to_queue(queued_peers, rtmsg.origin_peer());
+	queue_entry->set_push(true);
+
 }
 
 template<class A>
@@ -526,17 +543,20 @@ FanoutTable<A>::add_replace_to_queue(const InternalMessage<A> &old_rtmsg,
 					 RTQUEUE_OP_REPLACE_NEW);
     queue_entry->set_origin_peer(new_rtmsg.origin_peer());
     queue_entry->set_genid(new_rtmsg.genid());
+    
     _output_queue.push_back(queue_entry);
 
 
     if (new_rtmsg.push()) {
 	if (new_rtmsg.origin_peer() == old_rtmsg.origin_peer())
-	    add_push_to_queue(queued_peers, new_rtmsg.origin_peer());
+	    //add_push_to_queue(queued_peers, new_rtmsg.origin_peer());
+	    queue_entry->set_push(true);
 	else
 	    // if the origin peer we send to add_push_to_queue, the push will
 	    // go to all peers
 	    add_push_to_queue(queued_peers, NULL);
     }
+
 }
 
 template<class A>
@@ -559,7 +579,7 @@ FanoutTable<A>::set_queue_positions(const list<PeerTableInfo<A>*>&
 {
     typename list<PeerTableInfo<A>*>::const_iterator i;
     for (i = queued_peers.begin(); i != queued_peers.end(); i++) {
-	if ((*i)->busy() && ((*i)->has_queued_data() == false)) {
+	if ((*i)->has_queued_data() == false) {
 	    /* set the queue position to the current last element */
 	    (*i)->set_queue_position( --(_output_queue.end()) );
 	    (*i)->set_has_queued_data(true);
@@ -567,30 +587,6 @@ FanoutTable<A>::set_queue_positions(const list<PeerTableInfo<A>*>&
     }
 }
 
-template<class A>
-void
-FanoutTable<A>::output_state(bool busy, BGPRouteTable<A> *next_table) 
-{
-    debug_msg("%s \n next table: %s\n", this->tablename().c_str(),
-	      next_table->tablename().c_str());
-
-    typename NextTableMap<A>::iterator i;
-    i = _next_tables.find(next_table);
-    XLOG_ASSERT(i != _next_tables.end());
-
-    PeerTableInfo<A> *peer_info = &(i.second());
-
-    if (busy)
-	debug_msg("Fanout: Peer output state is BUSY\n");
-    else {
-	// downstream must drain the queue using get_next_message or
-	// skip_entire_queue before we reset the output state to NOT BUSY
-	XLOG_ASSERT(peer_info->has_queued_data() == false);
-	debug_msg("Fanout: Peer output state is not busy\n");
-    }
-    peer_info->set_busy(busy);
-    //    peer_info->set_has_queued_data(false);
-}
 
 template<class A>
 bool
@@ -603,8 +599,10 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
     XLOG_ASSERT(i != _next_tables.end());
 
     PeerTableInfo<A> *peer_info = &(i.second());
+    peer_info->received_get();
     if (peer_info->has_queued_data() == false) {
 	debug_msg("no data queued\n");
+	peer_info->set_is_ready();
 	return false;
     }
 
@@ -620,6 +618,7 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
 	InternalMessage<A> rtmsg((*queue_ptr)->route(),
 				 (*queue_ptr)->origin_peer(),
 				 (*queue_ptr)->genid());
+	if ((*queue_ptr)->push()) rtmsg.set_push();
 	next_table->add_route(rtmsg, (BGPRouteTable<A>*)this);
 	break;
     }
@@ -628,6 +627,7 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
 	InternalMessage<A> rtmsg((*queue_ptr)->route(),
 				 (*queue_ptr)->origin_peer(),
 				 (*queue_ptr)->genid());
+	if ((*queue_ptr)->push()) rtmsg.set_push();
 	next_table->delete_route(rtmsg, (BGPRouteTable<A>*)this);
 	break;
     }
@@ -643,6 +643,7 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
 	InternalMessage<A> new_rtmsg((*queue_ptr)->route(),
 				     (*queue_ptr)->origin_peer(),
 				     (*queue_ptr)->genid());
+	if ((*queue_ptr)->push()) new_rtmsg.set_push();
 	next_table->replace_route(old_rtmsg, new_rtmsg,
 				  (BGPRouteTable<A>*)this);
 	skipped = 2;
@@ -749,6 +750,7 @@ FanoutTable<A>::skip_entire_queue(BGPRouteTable<A> *next_table)
     XLOG_ASSERT(i != _next_tables.end());
 
     PeerTableInfo<A> *peer_info = &(i.second());
+    peer_info->peer_reset();
     if (peer_info->has_queued_data() == false)
 	return;
 
@@ -784,10 +786,10 @@ FanoutTable<A>::skip_entire_queue(BGPRouteTable<A> *next_table)
 	// skip over stuff that's not for this peer
 	while ((queue_ptr != _output_queue.end()) &&
 	       ((*queue_ptr)->origin_peer() == peer_info->peer_handler())) {
+	    queue_ptr++;
 	    if (queue_ptr == _output_queue.end()) {
 		break;
 	    } else {
-		queue_ptr++;
 		if ((*queue_ptr)->op() == RTQUEUE_OP_REPLACE_NEW)
 		    queue_ptr++;
 	    }
@@ -858,8 +860,8 @@ FanoutTable<A>::peering_down_complete(const PeerHandler *peer,
 {
     XLOG_ASSERT(this->_parent == caller);
 
-    debug_msg("peer: %s genid: %d caller: %s\n",
-	      peer->peername().c_str(), genid,
+    debug_msg("peer: %s genid: %u caller: %s\n",
+	      peer->peername().c_str(), XORP_UINT_CAST(genid),
 	      caller->tablename().c_str());
 
     print_queue();
