@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_filter.cc,v 1.22 2004/06/10 22:40:35 hodson Exp $"
+#ident "$XORP: xorp/bgp/route_table_filter.cc,v 1.27 2005/03/25 02:52:47 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -53,6 +54,17 @@ BGPRouteFilter<A>::propagate_flags(const InternalMessage<A> *rtmsg,
 	new_rtmsg->set_from_previous_peering();
 }
 
+
+template<class A>
+void
+BGPRouteFilter<A>::propagate_flags(const SubnetRoute<A>& route,
+				   SubnetRoute<A>& new_route) const {
+    new_route.set_filtered(route.is_filtered());
+    new_route.set_policytags(route.policytags());
+    if(route.is_winner())
+	new_route.set_is_winner(route.igp_metric());
+}				   
+
 /*************************************************************************/
 
 template<class A>
@@ -104,6 +116,10 @@ ASPrependFilter<A>::filter(const InternalMessage<A> *rtmsg,
 	= new SubnetRoute<A>(rtmsg->net(), &palist, 
 			     rtmsg->route()->original_route(), 
 			     rtmsg->route()->igp_metric());
+
+    // policy needs this
+    propagate_flags(*(rtmsg->route()),*new_route);
+    
     InternalMessage<A> *new_rtmsg = 
 	new InternalMessage<A>(new_route, rtmsg->origin_peer(), 
 			       rtmsg->genid());
@@ -133,6 +149,10 @@ const InternalMessage<A>*
 NexthopRewriteFilter<A>::filter(const InternalMessage<A> *rtmsg,
 				bool &modified) const
 {
+    // If we originated this route don't rewrite the nexthop
+    if (rtmsg->origin_peer()->originate_route_handler())
+	return rtmsg;
+
     //Form a new path attribute list containing the new nexthop
     PathAttributeList<A> palist(*(rtmsg->route()->attributes()));
     palist.replace_nexthop(_local_nexthop);
@@ -143,9 +163,12 @@ NexthopRewriteFilter<A>::filter(const InternalMessage<A> *rtmsg,
 	= new SubnetRoute<A>(rtmsg->net(), &palist,
 			     rtmsg->route()->original_route(), 
 			     rtmsg->route()->igp_metric());
+    
+    // policy needs this
+    propagate_flags(*(rtmsg->route()),*new_route);
 
-    debug_msg("NexthopRewriteFilter: new route: %x with attributes %x\n",
-	   (uint)new_route, (uint)(new_route->attributes()));
+    debug_msg("NexthopRewriteFilter: new route: %p with attributes %p\n",
+	      new_route, new_route->attributes());
     InternalMessage<A> *new_rtmsg = 
 	new InternalMessage<A>(new_route, rtmsg->origin_peer(), 
 			       rtmsg->genid());
@@ -217,6 +240,9 @@ LocalPrefInsertionFilter<A>::filter(const InternalMessage<A> *rtmsg,
 			     rtmsg->route()->original_route(), 
 			     rtmsg->route()->igp_metric());
 
+    // policy needs this
+    propagate_flags(*(rtmsg->route()),*new_route);
+
     InternalMessage<A> *new_rtmsg = 
 	new InternalMessage<A>(new_route, rtmsg->origin_peer(), 
 			       rtmsg->genid());
@@ -262,6 +288,9 @@ LocalPrefRemovalFilter<A>::filter(const InternalMessage<A> *rtmsg,
 			     rtmsg->route()->original_route(), 
 			     rtmsg->route()->igp_metric());
 
+    // policy needs this
+    propagate_flags(*(rtmsg->route()),*new_route);
+
     InternalMessage<A> *new_rtmsg = 
 	new InternalMessage<A>(new_route, rtmsg->origin_peer(), 
 			       rtmsg->genid());
@@ -309,6 +338,9 @@ MEDInsertionFilter<A>::filter(const InternalMessage<A> *rtmsg,
 	= new SubnetRoute<A>(rtmsg->net(), &palist,
 			     rtmsg->route()->original_route(), 
 			     rtmsg->route()->igp_metric());
+    
+    // policy needs this
+    propagate_flags(*(rtmsg->route()),*new_route);
 
     InternalMessage<A> *new_rtmsg = 
 	new InternalMessage<A>(new_route, rtmsg->origin_peer(), 
@@ -355,6 +387,9 @@ MEDRemovalFilter<A>::filter(const InternalMessage<A> *rtmsg,
 			     rtmsg->route()->original_route(), 
 			     rtmsg->route()->igp_metric());
 
+    // policy needs this
+    propagate_flags(*(rtmsg->route()),*new_route);
+    
     InternalMessage<A> *new_rtmsg = 
 	new InternalMessage<A>(new_route, rtmsg->origin_peer(), 
 			       rtmsg->genid());
@@ -396,6 +431,9 @@ UnknownFilter<A>::filter(const InternalMessage<A> *rtmsg,
 			     rtmsg->route()->original_route(), 
 			     rtmsg->route()->igp_metric());
 
+    // policy needs this
+    propagate_flags(*(rtmsg->route()),*new_route);
+    
     InternalMessage<A> *new_rtmsg = 
 	new InternalMessage<A>(new_route, rtmsg->origin_peer(), 
 			       rtmsg->genid());
@@ -451,6 +489,11 @@ OriginateRouteFilter<A>::filter(const InternalMessage<A> *rtmsg,
 	= new SubnetRoute<A>(rtmsg->net(), &palist, 
 			     rtmsg->route()->original_route(), 
 			     rtmsg->route()->igp_metric());
+    
+    // policy needs this
+    propagate_flags(*(rtmsg->route()),*new_route);
+    
+    
     InternalMessage<A> *new_rtmsg = 
 	new InternalMessage<A>(new_route, rtmsg->origin_peer(), 
 			       rtmsg->genid());
@@ -839,16 +882,6 @@ FilterTable<A>::apply_filters(const InternalMessage<A> *rtmsg) const
 }
 
 
-/* mechanisms to implement flow control in the output plumbing */
-template<class A>
-void 
-FilterTable<A>::output_state(bool busy, BGPRouteTable<A> *next_table)
-{
-    XLOG_ASSERT(this->_next_table == next_table);
-
-    this->_parent->output_state(busy, this);
-}
-
 template<class A>
 bool 
 FilterTable<A>::get_next_message(BGPRouteTable<A> *next_table)
