@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_ribout.cc,v 1.19 2004/06/10 22:40:36 hodson Exp $"
+#ident "$XORP: xorp/bgp/route_table_ribout.cc,v 1.27 2005/03/25 02:52:48 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -30,7 +30,7 @@ RibOutTable<A>::RibOutTable(string table_name,
     this->_parent = init_parent;
     _peer = peer;
     _peer_busy = false;
-    _upstream_queue_exists = false;
+    _peer_is_up = false;
 }
 
 template<class A>
@@ -159,8 +159,8 @@ RibOutTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
 			      const InternalMessage<A> &new_rtmsg,
 			      BGPRouteTable<A> *caller) 
 {
-    debug_msg("%s::replace_route %x %x\n", this->tablename().c_str(),
-	      (u_int)(&old_rtmsg), (u_int)(&new_rtmsg));
+    debug_msg("%s::replace_route %p %p\n", this->tablename().c_str(),
+	      &old_rtmsg, &new_rtmsg);
     XLOG_ASSERT(old_rtmsg.push() == false);
 
     delete_route(old_rtmsg, caller);
@@ -200,26 +200,30 @@ RibOutTable<A>::delete_route(const InternalMessage<A> &rtmsg,
 	entry->set_origin_peer(rtmsg.origin_peer());
 	_queue.push_back(entry);
     } else if (queued_entry->op() == RTQUEUE_OP_ADD) {
-	// XXX this probably shouldn't happen.
-	XLOG_UNREACHABLE();
-
-	// if it does, then here's how to handle it:
+	// The happens when a route with the same nexthop has been
+	// introduced on two peers and the resolvability changes.
+	// The RIB-IN sends a delete followed by an add. If this is
+	// the better route then the other peer will send an add then
+	// a delete.
+	// RIB-IN				RIB-OUT
+	// Peer A -> delete<A>
+	//					add<B>
+	// Peer A -> add<A>
+	//					delete<B>
+	
 	_queue.erase(i);
 	delete queued_entry;
     } else if (queued_entry->op() == RTQUEUE_OP_DELETE) {
 	// This should not happen.
 	XLOG_UNREACHABLE();
     } else if (queued_entry->op() == RTQUEUE_OP_REPLACE_OLD) {
-	// XXX this probably shouldn't happen.
-	XLOG_UNREACHABLE();
+	// The happens when a route with the same nexthop has been
+	// introduced on two peers and the resolvability changes.
 
-	// if it does, then here's how to handle it:
-	typename list<const RouteQueueEntry<A>*>::iterator i2 = i;
-	i++;
-	_queue.erase(i2);
+	i = _queue.erase(i);
 	XLOG_ASSERT((*i)->op() == RTQUEUE_OP_REPLACE_NEW);
-	_queue.erase(i);
 	delete *i;
+	_queue.erase(i);
 
 	entry = new RouteQueueEntry<A>(queued_entry->route(),
 				       RTQUEUE_OP_DELETE);
@@ -245,7 +249,6 @@ RibOutTable<A>::push(BGPRouteTable<A> *caller)
     // have the same Path Attributes, and send them together in an
     // Update message.  We repeatedly do this until the queue is empty.
 
-    bool peer_busy = false;
     while (_queue.empty() == false) {
 
 	// go through _queue and move all of the queue elements that
@@ -291,8 +294,8 @@ RibOutTable<A>::push(BGPRouteTable<A> *caller)
 		}
 	    }
 	}
-	debug_msg("%d elements with attr %x moved to tmp queue\n", ctr,
-		  (u_int)attributes);
+	debug_msg("%d elements with attr %p moved to tmp queue\n", ctr,
+		  attributes);
 	print_queue(tmp_queue);
 
 	// at this point we pass the tmp_queue to the output BGP
@@ -334,55 +337,46 @@ RibOutTable<A>::push(BGPRouteTable<A> *caller)
 
 	/* push the packet */
 	/* if the peer is busy (a queue has built up), there's not
-           much the RibOut can do for the things already in it's
+           much the RibOut can do for the things already in its
            output queue - we just keep sending those to the
-           peer_handler.  But we want to signal back upstream to stop
-           sending us new data */
+           peer_handler.  But we want to not request any more data for now. */
 	if (_peer->push_packet() == PEER_OUTPUT_BUSY)
-	    peer_busy = true;
+	    _peer_busy = true;
 
 
 	debug_msg("************************************\n");
     }
 
-    /* signal our state back upstream */
-    if ((peer_busy == true) && (_peer_busy == false)) {
-	debug_msg("RibOut signalling peer busy upstream\n");
-	this->_parent->output_state(true, this);
-
-	// we now have to assume that a queue will build upstream
-	_upstream_queue_exists = true;
-    }
-    _peer_busy = peer_busy;
-
     return 0;
 }
 
+/* value is a tradeoff between not too many calls to timers, and not
+   being away from eventloop for too long - probably needs tuning*/
+#define MAX_MSGS_IN_BATCH 10
 
 template<class A>
 void
-RibOutTable<A>::output_no_longer_busy() 
+RibOutTable<A>::wakeup()
 {
-    debug_msg("%s: output_no_longer_busy\n", this->tablename().c_str());
-    if (_peer_busy == false) return;
-    debug_msg("%s: _peer_busy true->false\n", this->tablename().c_str());
+    /* don't do anything if we can't sent to the peer handler yet */
+    if (_peer_busy == true)
+	return;
+
+    /* we're down, so ignore this wakeup */
+    if (_peer_is_up == false)
+	return;
+
+    for (int msgs = 0; msgs < MAX_MSGS_IN_BATCH; msgs++) {
+	/* only request a limited about of messages, so we don't hog
+	   the eventloop for too long */
 
-    _peer_busy = false;
-    while (1) {
 	/* get_next_message will cause the upstream queue to send
            another update (add, delete, replace or push) through the
            normal RouteTable interfaces*/
-
 	bool upstream_queue_exists = this->_parent->get_next_message(this);
 
 	if (upstream_queue_exists == false) {
 	    /*the queue upstream has now drained*/
-	    if (_peer_busy == false) {
-		/*if the queue downstream is no longer busy then go
-                  back to normal operation*/
-		_upstream_queue_exists = false;
-		this->_parent->output_state(false, this);
-	    }
 	    /*there's nothing left to do here*/
 	    return;
 	}
@@ -392,6 +386,29 @@ RibOutTable<A>::output_no_longer_busy()
 	    return;
 	}
     }
+    reschedule_self();
+}
+
+template<class A>
+void
+RibOutTable<A>::output_no_longer_busy() 
+{
+    debug_msg("%s: output_no_longer_busy\n", this->tablename().c_str());
+    debug_msg("%s: _peer_busy true->false\n", this->tablename().c_str());
+    _peer_busy = false;
+    wakeup();
+}
+
+template<class A>
+void
+RibOutTable<A>::reschedule_self()
+{
+    /*call back immediately, but after network events or expired
+      timers */    
+    if (_wakeup_timer.scheduled())
+	return;
+    _wakeup_timer = _peer->eventloop().
+	new_oneoff_after_ms(0, callback(this, &RibOutTable<A>::wakeup));
 }
 
 
@@ -409,7 +426,10 @@ RibOutTable<A>::peering_went_down(const PeerHandler *peer, uint32_t genid,
 {
     XLOG_ASSERT(this->_parent == caller);
     UNUSED(genid);
-    UNUSED(peer);
+
+    if (peer == _peer) {
+	_peer_is_up = false;
+    }
 }
 
 template<class A>
@@ -423,6 +443,20 @@ RibOutTable<A>::peering_down_complete(const PeerHandler *peer,
     UNUSED(peer);
 }
 
+template<class A>
+void
+RibOutTable<A>::peering_came_up(const PeerHandler *peer, uint32_t genid,
+				  BGPRouteTable<A> *caller) 
+{
+    XLOG_ASSERT(this->_parent == caller);
+    UNUSED(genid);
+
+    if (peer == _peer) {
+	_peer_is_up = true;
+	_peer_busy = false;
+    }
+}
+
 template<class A>
 string
 RibOutTable<A>::str() const 
