@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/subnet_route.cc,v 1.13 2004/06/10 22:40:36 hodson Exp $"
+#ident "$XORP: xorp/bgp/subnet_route.cc,v 1.16 2005/03/25 02:52:48 pavlin Exp $"
 
 #include "bgp_module.h"
 #include "libxorp/xlog.h"
@@ -24,7 +25,7 @@ template<class A> AttributeManager<A> SubnetRoute<A>::_att_mgr;
 template<class A>
 SubnetRoute<A>::SubnetRoute(const SubnetRoute<A>& route_to_clone) 
 {
-    debug_msg("SubnetRoute constructor1 giving %x\n", (uint)this);
+    debug_msg("SubnetRoute constructor1 giving %p\n", this);
     //note that we need an explicit constructor here, rather than
     //relying in C++ for the default constructor, or we get the
     //reference counts wrong in the attribute manager
@@ -46,6 +47,8 @@ SubnetRoute<A>::SubnetRoute(const SubnetRoute<A>& route_to_clone)
     if (_parent_route)
 	_parent_route->bump_refcount(1);
     _igp_metric = route_to_clone._igp_metric;
+
+    _policytags = route_to_clone._policytags;
 }
 
 template<class A>
@@ -53,7 +56,7 @@ SubnetRoute<A>::SubnetRoute(const IPNet<A> &n,
 			       const PathAttributeList<A>* atts,
 			       const SubnetRoute<A>* parent_route)
     : _net(n), _parent_route(parent_route) {
-    debug_msg("SubnetRoute constructor2 giving %x\n", (uint)this);
+    debug_msg("SubnetRoute constructor2 giving %p\n", this);
     //the attribute manager handles memory management, and ensuring
     //that only one copy of each attribute list is ever stored
     _attributes = _att_mgr.add_attribute_list(atts);
@@ -78,7 +81,7 @@ SubnetRoute<A>::SubnetRoute(const IPNet<A> &n,
 			       const SubnetRoute<A>* parent_route,
 			       uint32_t igp_metric)
     : _net(n), _parent_route(parent_route) {
-    debug_msg("SubnetRoute constructor3 giving %x\n", (uint)this);
+    debug_msg("SubnetRoute constructor3 giving %p\n", this);
     //the attribute manager handles memory management, and ensuring
     //that only one copy of each attribute list is ever stored
     _attributes = _att_mgr.add_attribute_list(atts);
@@ -110,7 +113,7 @@ SubnetRoute<A>::operator==(const SubnetRoute<A>& them) const {
 
 template<class A>
 SubnetRoute<A>::~SubnetRoute() {
-    debug_msg("SubnetRoute destructor called for %x\n", (uint)this);
+    debug_msg("SubnetRoute destructor called for %p\n", this);
     _att_mgr.delete_attribute_list(_attributes);
 
     assert(refcount() == 0);
@@ -240,10 +243,12 @@ SubnetRoute<A>::str() const {
     s += "  Net: " + _net.str() + "\n";
     s += "  PAList: " + _attributes->str();
 #ifdef DEBUG_FLAGS
+    s += "\n";
+    s += "  Policytags: " + _policytags.str() + "\n";
     if (is_winner()) {
-	s += "  route is winner";
+	s += "  route is winner\n";
     } else {
-	s += "  route is not winner";
+	s += "  route is not winner\n";
     }
     if (is_filtered()) {
 	s += "  route is filtered";
