@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_cache.cc,v 1.20 2004/06/10 22:40:36 hodson Exp $"
+#ident "$XORP: xorp/bgp/test_cache.cc,v 1.23 2005/03/25 02:52:48 pavlin Exp $"
 
 #include "bgp_module.h"
 #include "config.h"
@@ -38,9 +38,12 @@ test_cache(TestInfo& /*info*/)
     BGPMain bgpmain;
     // EventLoop* eventloop = bgpmain.eventloop();
     LocalData localdata;
-    BGPPeer peer1(&localdata, NULL, NULL, &bgpmain);
+    Iptuple iptuple;
+    BGPPeerData *pd1 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeer peer1(&localdata, pd1, NULL, &bgpmain);
     PeerHandler handler1("test1", &peer1, NULL, NULL);
-    BGPPeer peer2(&localdata, NULL, NULL, &bgpmain);
+    BGPPeerData *pd2 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeer peer2(&localdata, pd2, NULL, &bgpmain);
     PeerHandler handler2("test2", &peer2, NULL, NULL);
 
     // trivial plumbing
@@ -56,7 +59,7 @@ test_cache(TestInfo& /*info*/)
 
     // create a load of attributes
     IPNet<IPv4> net1("1.0.1.0/24");
-    //    IPNet<IPv4> net2("1.0.2.0/24");
+    IPNet<IPv4> net2("1.0.2.0/24");
 
     IPv4 nexthop1("2.0.0.1");
     NextHopAttribute<IPv4> nhatt1(nexthop1);
@@ -310,8 +313,18 @@ test_cache(TestInfo& /*info*/)
     assert(msg->route() == NULL);
     delete msg;
 
-    assert(cache_table->route_count() == 1);
+    sr1 = new SubnetRoute<IPv4>(net2, palist2, NULL);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg->set_changed();
+    cache_table->add_route(*msg, NULL);
+    assert(msg->route() == NULL);
+    delete msg;
+
+    assert(cache_table->route_count() == 2);
     cache_table->flush_cache();
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
     assert(cache_table->route_count() == 0);
 
 
