@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_dump.cc,v 1.41 2004/06/10 22:40:37 hodson Exp $"
+#ident "$XORP: xorp/bgp/test_dump.cc,v 1.45 2005/03/25 02:52:49 pavlin Exp $"
 
 #include "bgp_module.h"
 #include "config.h"
@@ -141,14 +141,17 @@ test_dump(TestInfo& /*info*/)
     debug_table1->set_output_file(filename);
     debug_table1->set_canned_response(ADD_USED);
     debug_table1->enable_tablename_printing();
+    debug_table1->set_get_on_wakeup(true);
 
     debug_table2->set_output_file(debug_table1->output_file());
     debug_table2->set_canned_response(ADD_USED);
     debug_table2->enable_tablename_printing();
+    debug_table2->set_get_on_wakeup(true);
 
     debug_table3->set_output_file(debug_table1->output_file());
     debug_table3->set_canned_response(ADD_USED);
     debug_table3->enable_tablename_printing();
+    debug_table3->set_get_on_wakeup(true);
 
     //create a load of attributes 
     IPNet<IPv4> net0("1.0.0.0/24");
@@ -302,6 +305,10 @@ test_dump(TestInfo& /*info*/)
     while (bgpmain.eventloop().timers_pending()) {
 	bgpmain.eventloop().run();
     }
+    while (debug_table3->parent()->get_next_message(debug_table3)) {}
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     //delete the routes
     debug_table1->write_separator();
@@ -389,6 +396,10 @@ test_dump(TestInfo& /*info*/)
     while (bgpmain.eventloop().timers_pending()) {
 	bgpmain.eventloop().run();
     }
+    while (debug_table3->parent()->get_next_message(debug_table3)) {}
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     //delete the routes
     debug_table1->write_separator();
@@ -495,8 +506,8 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_separator();
     debug_table1->write_comment("PEER 2 GOES DOWN");
     fanout_table->remove_next_table(debug_table2);
+    debug_table1->write_comment("XXXX");
     ribin_table2->ribin_peering_went_down();
-
     debug_table1->write_separator();
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     while (bgpmain.eventloop().timers_pending()) {
@@ -588,15 +599,12 @@ test_dump(TestInfo& /*info*/)
     fanout_table->dump_entire_table(debug_table3, SAFI_UNICAST, "ribname");
 
     debug_table1->write_separator();
-    debug_table1->write_comment("ONE EVENT RUN");
+    debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
-
-    debug_table1->write_separator();
-    debug_table1->write_comment("TWO EVENT RUN");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
-    bgpmain.eventloop().run();
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     debug_table1->write_separator();
     debug_table1->write_comment("PEER 2 GOES DOWN");
@@ -772,9 +780,11 @@ test_dump(TestInfo& /*info*/)
     fanout_table->dump_entire_table(debug_table3, SAFI_UNICAST, "ribname");
 
     debug_table1->write_separator();
-    debug_table1->write_comment("ONE EVENT RUN");
+    debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     debug_table1->write_separator();
     debug_table1->write_comment("PEER 2 GOES DOWN");
@@ -1064,20 +1074,13 @@ test_dump(TestInfo& /*info*/)
     delete msg;
 
     debug_table1->write_separator();
-    debug_table1->write_comment("ONE EVENT RUN");
+    debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
-
-    debug_table1->write_separator();
-    debug_table1->write_comment("ONE EVENT RUN");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
-
-    debug_table1->write_separator();
-    debug_table1->write_comment("TWO EVENT RUN");
     debug_table1->write_comment("EXPECT ADD 1.0.3.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
-    bgpmain.eventloop().run();
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     debug_table1->write_separator();
     debug_table1->write_comment("SENDING FROM PEER 2");
@@ -1253,15 +1256,12 @@ test_dump(TestInfo& /*info*/)
     delete msg;
 
     debug_table1->write_separator();
-    debug_table1->write_comment("ONE EVENT RUN");
+    debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
-
-    debug_table1->write_separator();
-    debug_table1->write_comment("TWO EVENT RUN");
     debug_table1->write_comment("EXPECT ADD 1.0.3.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
-    bgpmain.eventloop().run();
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     debug_table1->write_separator();
     debug_table1->write_comment("SENDING FROM PEER 2");
@@ -1382,6 +1382,7 @@ test_dump(TestInfo& /*info*/)
 
     debug_table1->write_separator();
     debug_table1->write_comment("TAKE PEER 1 DOWN");
+    int genid = ribin_table1->genid();
     fanout_table->remove_next_table(debug_table1);
     ribin_table1->ribin_peering_went_down();
 
@@ -1392,6 +1393,8 @@ test_dump(TestInfo& /*info*/)
 				 ribin_table3->genid());
     debug_table3->set_parent(fanout_table);
     fanout_table->dump_entire_table(debug_table3, SAFI_UNICAST, "ribname");
+    ((DumpTable<IPv4>*)debug_table3->parent())
+	->peering_is_down(&handler1, genid);
 
     debug_table1->write_separator();
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
@@ -1487,7 +1490,9 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.3.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     debug_table1->write_separator();
     while (bgpmain.eventloop().timers_pending()) {
@@ -1625,7 +1630,9 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.4.0/24 RECEIVED BY PEER 3");
 
-    bgpmain.eventloop().run();
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
     debug_table1->write_separator();
     while (bgpmain.eventloop().timers_pending()) {
 	bgpmain.eventloop().run();
@@ -1700,11 +1707,12 @@ test_dump(TestInfo& /*info*/)
     fanout_table->dump_entire_table(debug_table3, SAFI_UNICAST, "ribname");
 
     debug_table1->write_separator();
-    debug_table1->write_comment("SINGLE STEP EVENT LOOP");
+    debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
-    debug_table1->write_comment("STEP AGAIN (MOVES TO DUMPING NEXT PEER)");
-    bgpmain.eventloop().run();
+    debug_table1->write_comment("EXPECT (MOVES TO DUMPING NEXT PEER)");
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     debug_table1->write_separator();
     debug_table1->write_comment("SENDING FROM PEER 1");
@@ -1798,9 +1806,11 @@ test_dump(TestInfo& /*info*/)
     fanout_table->dump_entire_table(debug_table3, SAFI_UNICAST, "ribname");
 
     debug_table1->write_separator();
-    debug_table1->write_comment("SINGLE STEP EVENT LOOP");
+    debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("NO ROUTES IN 1st PEER (MOVES TO DUMPING NEXT PEER)");
-    bgpmain.eventloop().run();
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     debug_table1->write_separator();
     debug_table1->write_comment("SENDING FROM PEER 1");
@@ -2322,7 +2332,9 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_separator();
     debug_table1->write_comment("STEP EVENT LOOP ONCE TO DUMP ONE ROUTE");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
-    bgpmain.eventloop().run();
+    while (bgpmain.eventloop().timers_pending()) {
+	bgpmain.eventloop().run();
+    }
 
     debug_table1->write_separator();
     //take peer1 down
