@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_fanout.cc,v 1.22 2004/06/10 22:40:37 hodson Exp $"
+#ident "$XORP: xorp/bgp/test_fanout.cc,v 1.25 2005/03/25 02:52:49 pavlin Exp $"
 
 #include "bgp_module.h"
 #include "config.h"
@@ -132,6 +132,8 @@ test_fanout(TestInfo& /*info*/)
     debug_table1->write_comment("TEST 1");
     debug_table1->write_comment("ADD AND DELETE");
     debug_table1->write_comment("SENDING FROM PEER 1");
+    debug_table1->set_get_on_wakeup(true);
+    debug_table2->set_get_on_wakeup(true);
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
     sr1->set_nexthop_resolved(true);
     msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
@@ -171,7 +173,8 @@ test_fanout(TestInfo& /*info*/)
     debug_table1->write_comment("SENDING FROM PEER 1");
 
     //set output state on peer 2 to be busy
-    fanout_table->output_state(true, debug_table2);
+    //    fanout_table->output_state(true, debug_table2);
+    debug_table2->set_get_on_wakeup(false);
 
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
     sr1->set_nexthop_resolved(true);
@@ -204,7 +207,7 @@ test_fanout(TestInfo& /*info*/)
     debug_table1->write_comment("SENDING FROM PEER 1");
 
     //set output state on peer 2 to be busy
-    fanout_table->output_state(true, debug_table2);
+    debug_table2->set_get_on_wakeup(false);
 
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
     sr1->set_nexthop_resolved(true);
@@ -238,8 +241,10 @@ test_fanout(TestInfo& /*info*/)
     debug_table1->write_comment("SENDING FROM PEER 1");
 
     //set output state on both peers to be busy
-    fanout_table->output_state(true, debug_table1);
-    fanout_table->output_state(true, debug_table2);
+    //fanout_table->output_state(true, debug_table1);
+    //fanout_table->output_state(true, debug_table2);
+    debug_table1->set_get_on_wakeup(false);
+    debug_table2->set_get_on_wakeup(false);
 
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
     sr1->set_nexthop_resolved(true);
@@ -294,8 +299,10 @@ test_fanout(TestInfo& /*info*/)
     debug_table1->write_comment("SENDING FROM PEER 1");
 
     //set output state on both peers to be busy
-    fanout_table->output_state(true, debug_table1);
-    fanout_table->output_state(true, debug_table2);
+    //fanout_table->output_state(true, debug_table1);
+    //fanout_table->output_state(true, debug_table2);
+    debug_table1->set_get_on_wakeup(false);
+    debug_table2->set_get_on_wakeup(false);
 
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
     sr1->set_nexthop_resolved(true);
@@ -354,18 +361,19 @@ test_fanout(TestInfo& /*info*/)
 	 = new DebugTable<IPv4>("D3", (BGPRouteTable<IPv4>*)fanout_table);
     fanout_table->add_next_table(debug_table3, &handler3, 1);
 
-    debug_table3->set_output_file(debug_table1->output_file());
+    //debug_table3->set_output_file(debug_table1->output_file());
     debug_table3->set_canned_response(ADD_USED);
     debug_table3->enable_tablename_printing();
+    debug_table3->set_output_file(debug_table1->output_file());
 
     debug_table1->write_comment("******************************************");
     debug_table1->write_comment("TEST 5");
     debug_table1->write_comment("INTERLEAVED FLOW CONTROL - THREE PEERS BUSY");
     debug_table1->write_comment("SENDING FROM PEER 1");
 
-    fanout_table->output_state(true, debug_table1);
-    fanout_table->output_state(true, debug_table2);
-    fanout_table->output_state(true, debug_table3);
+    //fanout_table->output_state(true, debug_table1);
+    //fanout_table->output_state(true, debug_table2);
+    //fanout_table->output_state(true, debug_table3);
 
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
     sr1->set_nexthop_resolved(true);
@@ -432,6 +440,204 @@ test_fanout(TestInfo& /*info*/)
     sr1->unref();
     sr2->unref();
 
+    //================================================================
+    //Test6: test of queuing
+    //================================================================
+    //add a route
+    debug_table1->write_comment("******************************************");
+    debug_table1->write_comment("TEST 6");
+    debug_table1->write_comment("TEST QUEUING");
+
+    debug_table1->set_get_on_wakeup(false);
+    debug_table2->set_get_on_wakeup(false);
+    debug_table3->set_get_on_wakeup(false);
+
+    debug_table1->write_comment("SENDING FROM PEER 1");
+    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
+    sr1->set_nexthop_resolved(true);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg->set_push();
+    fanout_table->add_route(*msg, NULL);
+    debug_table1->write_separator();
+    delete msg;
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("SENDING FROM PEER 2");
+    sr2 = new SubnetRoute<IPv4>(net2, palist2, NULL);
+    sr2->set_nexthop_resolved(true);
+    msg = new InternalMessage<IPv4>(sr2, &handler2, 0);
+    msg->set_push();
+    fanout_table->add_route(*msg, NULL);
+    debug_table1->write_separator();
+    delete msg;
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 3 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET A ROUTE");
+    fanout_table->get_next_message(debug_table3);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 3 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET A ROUTE");
+    fanout_table->get_next_message(debug_table3);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 3 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET NO ROUTE");
+    fanout_table->get_next_message(debug_table3);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 2 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET A ROUTE");
+    fanout_table->get_next_message(debug_table2);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 2 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET NO ROUTE");
+    fanout_table->get_next_message(debug_table2);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 1 SKIP ENTIRE QUEUE");
+    debug_table1->write_comment("EXPECT TO GET NO ROUTE");
+    fanout_table->remove_next_table(debug_table1);
+    fanout_table->add_next_table(debug_table1, &handler1, 1);
+    debug_table1->write_separator();
+
+    sr1->unref();
+    sr2->unref();
+
+    //================================================================
+    //Test7: test of queuing
+    //================================================================
+    //add a route
+    debug_table1->write_comment("******************************************");
+    debug_table1->write_comment("TEST 7");
+    debug_table1->write_comment("TEST QUEUING");
+
+    debug_table1->set_get_on_wakeup(false);
+    debug_table2->set_get_on_wakeup(false);
+    debug_table3->set_get_on_wakeup(false);
+
+    debug_table1->write_comment("SENDING FROM PEER 1");
+    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
+    sr1->set_nexthop_resolved(true);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg->set_push();
+    fanout_table->add_route(*msg, NULL);
+    debug_table1->write_separator();
+    delete msg;
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("SENDING FROM PEER 2");
+    sr2 = new SubnetRoute<IPv4>(net2, palist2, NULL);
+    sr2->set_nexthop_resolved(true);
+    msg = new InternalMessage<IPv4>(sr2, &handler2, 0);
+    msg->set_push();
+    fanout_table->add_route(*msg, NULL);
+    debug_table1->write_separator();
+    delete msg;
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 3 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET A ROUTE");
+    fanout_table->get_next_message(debug_table3);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 1 SKIP ENTIRE QUEUE");
+    debug_table1->write_comment("EXPECT TO GET NO ROUTE");
+    fanout_table->remove_next_table(debug_table1);
+    fanout_table->add_next_table(debug_table1, &handler1, 1);
+    debug_table1->write_separator();
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 3 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET A ROUTE");
+    fanout_table->get_next_message(debug_table3);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 3 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET NO ROUTE");
+    fanout_table->get_next_message(debug_table3);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 2 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET A ROUTE");
+    fanout_table->get_next_message(debug_table2);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 2 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET NO ROUTE");
+    fanout_table->get_next_message(debug_table2);
+
+    sr1->unref();
+    sr2->unref();
+
+    //================================================================
+    //Test8: test of queuing
+    //================================================================
+    //add a route
+    debug_table1->write_comment("******************************************");
+    debug_table1->write_comment("TEST 8");
+    debug_table1->write_comment("TEST QUEUING");
+
+    debug_table1->set_get_on_wakeup(false);
+    debug_table2->set_get_on_wakeup(false);
+    debug_table3->set_get_on_wakeup(false);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("SENDING FROM PEER 2");
+    sr2 = new SubnetRoute<IPv4>(net2, palist2, NULL);
+    sr2->set_nexthop_resolved(true);
+    msg = new InternalMessage<IPv4>(sr2, &handler2, 0);
+    msg->set_push();
+    fanout_table->add_route(*msg, NULL);
+    debug_table1->write_separator();
+    delete msg;
+
+    debug_table1->write_comment("SENDING FROM PEER 1");
+    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
+    sr1->set_nexthop_resolved(true);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg->set_push();
+    fanout_table->add_route(*msg, NULL);
+    debug_table1->write_separator();
+    delete msg;
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 3 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET A ROUTE");
+    fanout_table->get_next_message(debug_table3);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 1 SKIP ENTIRE QUEUE");
+    debug_table1->write_comment("EXPECT TO GET NO ROUTE");
+    fanout_table->remove_next_table(debug_table1);
+    fanout_table->add_next_table(debug_table1, &handler1, 1);
+    debug_table1->write_separator();
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 3 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET A ROUTE");
+    fanout_table->get_next_message(debug_table3);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 3 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET NO ROUTE");
+    fanout_table->get_next_message(debug_table3);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 2 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET A ROUTE");
+    fanout_table->get_next_message(debug_table2);
+
+    debug_table1->write_separator();
+    debug_table1->write_comment("PEER 2 GET_NEXT_MESSAGE");
+    debug_table1->write_comment("EXPECT TO GET NO ROUTE");
+    fanout_table->get_next_message(debug_table2);
+
+    sr1->unref();
+    sr2->unref();
+
     //================================================================
 
     debug_table1->write_comment("SHUTDOWN AND CLEAN UP");
@@ -450,7 +656,7 @@ test_fanout(TestInfo& /*info*/)
 	fclose(file);
 	return false;
    }
-#define BUFSIZE 8192
+#define BUFSIZE 16384
     char testout[BUFSIZE];
     memset(testout, 0, BUFSIZE);
     int bytes1 = fread(testout, 1, BUFSIZE, file);
