@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_peer_data.cc,v 1.6 2004/06/10 22:40:38 hodson Exp $"
+#ident "$XORP: xorp/bgp/test_peer_data.cc,v 1.8 2005/03/25 02:52:50 pavlin Exp $"
 
 #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -135,6 +135,9 @@ test2(TestInfo& info)
     */
     if(!pd.parameter_recv_list().empty()) {
 	DOUT(info) << "The receive list should be empty\n";
+	ParameterList pl = pd.parameter_recv_list();
+	for(ParameterList::iterator i = pl.begin(); i != pl.end(); i++)
+	    DOUT(info) << "Parameter = " << (*i)->str() << endl;
 	return false;
     }
 
