@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +13,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/xrl_target.cc,v 1.26 2004/06/10 22:40:39 hodson Exp $"
+#ident "$XORP: xorp/bgp/xrl_target.cc,v 1.37 2005/03/25 02:52:51 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
+#define PROFILE_UTILS_REQUIRED
 
 #include "config.h"
 #include "bgp_module.h"
@@ -26,10 +28,13 @@
 
 #include "libxorp/eventloop.hh"
 #include "libxipc/xrl_std_router.hh"
+#include "libxipc/xrl_error.hh"
+#include "xrl/interfaces/profile_client_xif.hh"
 
 #include "bgp.hh"
 #include "iptuple.hh"
 #include "xrl_target.hh"
+//#include "profile_vars.hh"
 
 XrlBgpTarget::XrlBgpTarget(XrlRouter *r, BGPMain& bgp)
 	: XrlBgpTargetBase(r),
@@ -92,7 +97,7 @@ XrlBgpTarget::bgp_0_2_local_config(
 				   const uint32_t&	as, 
 				   const IPv4&		id)
 {
-    debug_msg("as %d id %s\n", as, id.str().c_str());
+    debug_msg("as %u id %s\n", XORP_UINT_CAST(as), id.str().c_str());
 
     /*
     ** We may already be configured so don't allow a reconfiguration.
@@ -112,7 +117,7 @@ XrlBgpTarget::bgp_0_2_set_local_as(
 				   // Input values, 
 				   const uint32_t&	as)
 {
-    debug_msg("as %d\n", as);
+    debug_msg("as %u\n", XORP_UINT_CAST(as));
 
     /*
     ** We may already be configured so don't allow a reconfiguration.
@@ -186,10 +191,12 @@ XrlBgpTarget::bgp_0_2_add_peer(
 	const IPv4&	next_hop,
 	const uint32_t&	holdtime)
 {
-    debug_msg("local ip %s local port %d peer ip %s peer port %d as %d"
-	      " next_hop %s holdtime %d\n",
-	      local_ip.c_str(), local_port, peer_ip.c_str(), peer_port,
-	      as, next_hop.str().c_str(), holdtime);
+    debug_msg("local ip %s local port %u peer ip %s peer port %u as %u"
+	      " next_hop %s holdtime %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      XORP_UINT_CAST(as), next_hop.str().c_str(),
+	      XORP_UINT_CAST(holdtime));
 
     if(_awaiting_config)
 	return XrlCmdError::COMMAND_FAILED("BGP Not configured!!!");
@@ -197,11 +204,17 @@ XrlBgpTarget::bgp_0_2_add_peer(
     if(!_bgp.processes_ready())
 	return XrlCmdError::COMMAND_FAILED("FEA or RIB not running");
 
-    Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
-
-    BGPPeerData *pd = new BGPPeerData(iptuple,
-				      AsNum(static_cast<uint16_t>(as)),
-				      next_hop, holdtime);
+    
+    BGPPeerData *pd = 0;
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+	pd = new BGPPeerData(iptuple, AsNum(static_cast<uint16_t>(as)),
+			     next_hop, holdtime);
+    } catch(XorpException& e) {
+	delete pd;
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
 
     if(!_bgp.create_peer(pd)) {
 	delete pd;
@@ -219,8 +232,9 @@ XrlBgpTarget::bgp_0_2_delete_peer(
 	const string&	peer_ip, 
 	const uint32_t&	peer_port)
 {
-    debug_msg("local ip %s local port %d peer ip %s peer port %d\n",
-	      local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    debug_msg("local ip %s local port %u peer ip %s peer port %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port));
 
     XLOG_WARNING("Not currently implemented\n");
 
@@ -235,13 +249,19 @@ XrlBgpTarget::bgp_0_2_enable_peer(
 	const string&	peer_ip, 
 	const uint32_t&	peer_port)
 {
-    debug_msg("local ip %s local port %d peer ip %s peer port %d\n",
-	      local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    debug_msg("local ip %s local port %u peer ip %s peer port %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port));
 
-    Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
 
-    if(!_bgp.enable_peer(iptuple))
-	return XrlCmdError::COMMAND_FAILED();
+	if(!_bgp.enable_peer(iptuple))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
 
     return XrlCmdError::OKAY();
 }
@@ -254,13 +274,73 @@ XrlBgpTarget::bgp_0_2_disable_peer(
 	const string&	peer_ip, 
 	const uint32_t&	peer_port)
 {
-    debug_msg("local ip %s local port %d peer ip %s peer port %d\n",
-	      local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    debug_msg("local ip %s local port %u peer ip %s peer port %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port));
 
-    Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
 
-    if(!_bgp.disable_peer(iptuple))
-	return XrlCmdError::COMMAND_FAILED();
+	if(!_bgp.disable_peer(iptuple))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlBgpTarget::bgp_0_2_set_nexthop6(
+	// Input values, 
+	const string&	local_ip, 
+	const uint32_t&	local_port, 
+	const string&	peer_ip, 
+	const uint32_t&	peer_port,
+	const IPv6&	next_hop)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u "
+	      "next-hop %s\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      cstring(next_hop));
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.set_nexthop6(iptuple, next_hop))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlBgpTarget::bgp_0_2_get_nexthop6(
+	// Input values, 
+	const string&	local_ip, 
+	const uint32_t&	local_port, 
+	const string&	peer_ip, 
+	const uint32_t&	peer_port,
+	IPv6&	next_hop)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port));
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.get_nexthop6(iptuple, next_hop))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
 
     return XrlCmdError::OKAY();
 }
@@ -274,13 +354,46 @@ XrlBgpTarget::bgp_0_2_set_peer_state(
 	const uint32_t&	peer_port, 
 	const bool& toggle)
 {
-    debug_msg("local ip %s local port %d peer ip %s peer port %d toggle %d\n",
-	  local_ip.c_str(), local_port, peer_ip.c_str(), peer_port, toggle);
+    debug_msg("local ip %s local port %u peer ip %s peer port %u toggle %d\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port), toggle);
 
-    Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
 
-    if(!_bgp.set_peer_state(iptuple, toggle))
-	return XrlCmdError::COMMAND_FAILED();
+	if(!_bgp.set_peer_state(iptuple, toggle))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlBgpTarget::bgp_0_2_set_peer_md5_password(
+	// Input values, 
+	const string&	local_ip, 
+	const uint32_t&	local_port, 
+	const string&	peer_ip, 
+	const uint32_t&	peer_port, 
+	const string& password)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u password %s\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      password.c_str());
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.set_peer_md5_password(iptuple, password))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
 
     return XrlCmdError::OKAY();
 }
@@ -293,13 +406,19 @@ XrlBgpTarget::bgp_0_2_activate(
 	const string&	peer_ip, 
 	const uint32_t&	peer_port)
 {
-    debug_msg("local ip %s local port %d peer ip %s peer port %d\n",
-	  local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    debug_msg("local ip %s local port %u peer ip %s peer port %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port));
 
-    Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
 
-    if(!_bgp.activate(iptuple))
-	return XrlCmdError::COMMAND_FAILED();
+	if(!_bgp.activate(iptuple))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
 
     return XrlCmdError::OKAY();
 }
@@ -313,15 +432,21 @@ XrlBgpTarget::bgp_0_2_next_hop_rewrite_filter(
 	const uint32_t&	peer_port,
 	const IPv4& 	next_hop)
 {
-    debug_msg("local ip %s local port %d peer ip %s peer port %d "
+    debug_msg("local ip %s local port %u peer ip %s peer port %u "
 	      "next hop %s\n",
-	      local_ip.c_str(), local_port, peer_ip.c_str(), peer_port,
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
 	      next_hop.str().c_str());
 
-    Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
 
-    if(!_bgp.next_hop_rewrite_filter(iptuple, next_hop))
-	return XrlCmdError::COMMAND_FAILED();
+	if(!_bgp.next_hop_rewrite_filter(iptuple, next_hop))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
 
     return XrlCmdError::OKAY();
 }
@@ -337,7 +462,7 @@ XrlBgpTarget::bgp_0_2_originate_route4(
     debug_msg("nlri %s next hop %s unicast %d multicast %d\n",
 	      nlri.str().c_str(), next_hop.str().c_str(), unicast, multicast);
 
-    if (!_bgp.originate_route(nlri, next_hop, unicast, multicast))
+    if (!_bgp.originate_route(nlri, next_hop, unicast, multicast,PolicyTags()))
 	return XrlCmdError::COMMAND_FAILED();
 
     return XrlCmdError::OKAY();
@@ -354,7 +479,7 @@ XrlBgpTarget::bgp_0_2_originate_route6(
     debug_msg("nlri %s next hop %s unicast %d multicast %d\n",
 	      nlri.str().c_str(), next_hop.str().c_str(), unicast, multicast);
 
-    if (!_bgp.originate_route(nlri, next_hop, unicast, multicast))
+    if (!_bgp.originate_route(nlri, next_hop, unicast, multicast,PolicyTags()))
 	return XrlCmdError::COMMAND_FAILED();
 
     return XrlCmdError::OKAY();
@@ -407,92 +532,133 @@ XrlBgpTarget::bgp_0_2_get_peer_list_next(
 					 // Input values, 
 					 const uint32_t&	token, 
 					 // Output values, 
-					 IPv4&	local_ip, 
+					 string&	local_ip, 
 					 uint32_t&	local_port, 
-					 IPv4&	peer_ip, 
+					 string&	peer_ip, 
 					 uint32_t&	peer_port, 
 					 bool&	more)
 {
     more = _bgp.get_peer_list_next(token, local_ip, local_port, peer_ip, 
 				   peer_port);
+
+    debug_msg("local ip %s local port %u peer ip %s peer port %u "
+	      "token %#x %s\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      XORP_UINT_CAST(token),
+	      more ? "more" : "");
+
     return  XrlCmdError::OKAY();
 }
 
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_get_peer_id(
 				  // Input values, 
-				  const IPv4&	local_ip, 
+				  const string&	local_ip, 
 				  const uint32_t&	local_port, 
-				  const IPv4&	peer_ip, 
+				  const string&	peer_ip, 
 				  const uint32_t&	peer_port, 
 				  // Output values, 
 				  IPv4&	peer_id)
 {
-    Iptuple iptuple(local_ip, local_port, peer_ip, peer_port);
-    if (!_bgp.get_peer_id(iptuple, peer_id)) {
-	return XrlCmdError::COMMAND_FAILED();
+    debug_msg("local ip %s local port %u peer ip %s peer port %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port));
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if (!_bgp.get_peer_id(iptuple, peer_id)) {
+	    return XrlCmdError::COMMAND_FAILED();
+	}
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
     }
+
     return XrlCmdError::OKAY();
 }
 
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_get_peer_status(
 				      // Input values, 
-				      const IPv4&	local_ip, 
+				      const string&	local_ip, 
 				      const uint32_t&	local_port, 
-				      const IPv4&	peer_ip, 
+				      const string&	peer_ip, 
 				      const uint32_t&	peer_port, 
 				      // Output values, 
 				      uint32_t&	peer_state, 
 				      uint32_t&	admin_status)
 {
-    Iptuple iptuple(local_ip, local_port, peer_ip, peer_port);
-    if (!_bgp.get_peer_status(iptuple, peer_state, admin_status)) {
-	return XrlCmdError::COMMAND_FAILED();
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if (!_bgp.get_peer_status(iptuple, peer_state, admin_status)) {
+	    return XrlCmdError::COMMAND_FAILED();
+	}
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
     }
+
     return XrlCmdError::OKAY();
 }
 
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_get_peer_negotiated_version(
 						  // Input values, 
-						  const IPv4& local_ip, 
+						  const string& local_ip, 
 						  const uint32_t& local_port, 
-						  const IPv4& peer_ip, 
+						  const string& peer_ip, 
 						  const uint32_t& peer_port, 
 						  // Output values, 
 						  int32_t& neg_version)
 {
-    Iptuple iptuple(local_ip, local_port, peer_ip, peer_port);
-    if (!_bgp.get_peer_negotiated_version(iptuple, neg_version)) {
-	return XrlCmdError::COMMAND_FAILED();
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if (!_bgp.get_peer_negotiated_version(iptuple, neg_version)) {
+	    return XrlCmdError::COMMAND_FAILED();
+	}
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
     }
+
     return XrlCmdError::OKAY();
 }
 
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_get_peer_as(
 				  // Input values, 
-				  const IPv4& local_ip, 
+				  const string& local_ip, 
 				  const uint32_t& local_port, 
-				  const IPv4&	peer_ip, 
+				  const string&	peer_ip, 
 				  const uint32_t& peer_port, 
 				  // Output values, 
 				  uint32_t& peer_as)
 {
-    Iptuple iptuple(local_ip, local_port, peer_ip, peer_port);
-    if (!_bgp.get_peer_as(iptuple, peer_as)) {
-	return XrlCmdError::COMMAND_FAILED();
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if (!_bgp.get_peer_as(iptuple, peer_as)) {
+	    return XrlCmdError::COMMAND_FAILED();
+	}
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
     }
+
     return XrlCmdError::OKAY();
 }
 
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_get_peer_msg_stats(
 					 // Input values, 
-					 const IPv4& local_ip, 
+					 const string& local_ip, 
 					 const uint32_t& local_port, 
-					 const IPv4& peer_ip, 
+					 const string& peer_ip, 
 					 const uint32_t& peer_port, 
 					 // Output values, 
 					 uint32_t&	in_updates, 
@@ -502,32 +668,45 @@ XrlBgpTarget::bgp_0_2_get_peer_msg_stats(
 					 uint32_t&	last_error, 
 					 uint32_t&	in_update_elapsed)
 {
-    Iptuple iptuple(local_ip, local_port, peer_ip, peer_port);
-    uint16_t last_error_short;
-    if (!_bgp.get_peer_msg_stats(iptuple, in_updates, out_updates,
-				 in_msgs, out_msgs, last_error_short, 
-				 in_update_elapsed)) {
-	return XrlCmdError::COMMAND_FAILED();
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	uint16_t last_error_short;
+	if (!_bgp.get_peer_msg_stats(iptuple, in_updates, out_updates,
+				     in_msgs, out_msgs, last_error_short, 
+				     in_update_elapsed)) {
+	    return XrlCmdError::COMMAND_FAILED();
+	}
+	last_error = last_error_short;
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
     }
-    last_error = last_error_short;
+
     return XrlCmdError::OKAY();
 }
 
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_get_peer_established_stats(
 						 // Input values, 
-						 const IPv4& local_ip, 
+						 const string& local_ip, 
 						 const uint32_t& local_port, 
-						 const IPv4& peer_ip, 
+						 const string& peer_ip, 
 						 const uint32_t& peer_port, 
 						 // Output values, 
 						 uint32_t& transitions, 
 						 uint32_t& established_time)
 {
-    Iptuple iptuple(local_ip, local_port, peer_ip, peer_port);
-    if (!_bgp.get_peer_established_stats(iptuple, transitions, 
-					 established_time)) {
-	return XrlCmdError::COMMAND_FAILED();
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if (!_bgp.get_peer_established_stats(iptuple, transitions,
+					     established_time)) {
+	    return XrlCmdError::COMMAND_FAILED();
+	}
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
     }
     return XrlCmdError::OKAY();
 }
@@ -535,10 +714,10 @@ XrlBgpTarget::bgp_0_2_get_peer_established_stats(
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_get_peer_timer_config(
 					    // Input values, 
-					    const IPv4&	local_ip, 
-					    const uint32_t& local_port, 
-					    const IPv4&	peer_ip, 
-					    const uint32_t& peer_port, 
+					    const string&	local_ip, 
+					    const uint32_t&	local_port, 
+					    const string&	peer_ip, 
+					    const uint32_t&	peer_port, 
 					    // Output values, 
 					    uint32_t& retry_interval, 
 					    uint32_t& hold_time, 
@@ -548,12 +727,17 @@ XrlBgpTarget::bgp_0_2_get_peer_timer_config(
 					    uint32_t& min_as_origin_interval,
 					    uint32_t& min_route_adv_interval)
 {
-    Iptuple iptuple(local_ip, local_port, peer_ip, peer_port);
-    if (!_bgp.get_peer_timer_config(iptuple, retry_interval, hold_time,
-				    keep_alive, hold_time_conf,
-				    keep_alive_conf, min_as_origin_interval,
-				    min_route_adv_interval)) {
-	return XrlCmdError::COMMAND_FAILED();
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+	if (!_bgp.get_peer_timer_config(iptuple, retry_interval, hold_time,
+					keep_alive, hold_time_conf,
+					keep_alive_conf, min_as_origin_interval,
+					min_route_adv_interval)) {
+	    return XrlCmdError::COMMAND_FAILED();
+	}
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
     }
     return XrlCmdError::OKAY();
 }
@@ -703,9 +887,9 @@ XrlCmdError XrlBgpTarget::rib_client_0_1_route_info_changed4(
 {
     IPNet<IPv4> net(addr, prefix_len);
     debug_msg("IGP route into changed for net %s\n", net.str().c_str());
-    debug_msg("Nexthop: %s Metric: %d AdminDistance: %d ProtocolOrigin: %s\n",
-	      nexthop.str().c_str(), metric, admin_distance,
-	      protocol_origin.c_str());
+    debug_msg("Nexthop: %s Metric: %u AdminDistance: %u ProtocolOrigin: %s\n",
+	      nexthop.str().c_str(), XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance), protocol_origin.c_str());
     
     // TODO: admin_distance and protocol_origin are not used
     if(!_bgp.rib_client_route_info_changed4(addr, prefix_len, nexthop, metric))
@@ -725,9 +909,9 @@ XrlCmdError XrlBgpTarget::rib_client_0_1_route_info_changed6(
 {
     IPNet<IPv6> net(addr, prefix_len);
     debug_msg("IGP route into changed for net %s\n", net.str().c_str());
-    debug_msg("Nexthop: %s Metric: %d AdminDistance: %d ProtocolOrigin: %s\n",
-	      nexthop.str().c_str(), metric, admin_distance,
-	      protocol_origin.c_str());
+    debug_msg("Nexthop: %s Metric: %u AdminDistance: %u ProtocolOrigin: %s\n",
+	      nexthop.str().c_str(), XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance), protocol_origin.c_str());
     
     // TODO: admin_distance and protocol_origin are not used
     if(!_bgp.rib_client_route_info_changed6(addr, prefix_len, nexthop, metric))
@@ -764,22 +948,30 @@ XrlCmdError XrlBgpTarget::rib_client_0_1_route_info_invalid6(
     return XrlCmdError::OKAY();
 }
 
- 
 XrlCmdError XrlBgpTarget::bgp_0_2_set_parameter(
 				  // Input values,
 				  const string&	local_ip, 
 				  const uint32_t&	local_port, 
 				  const string&	peer_ip, 
 				  const uint32_t&	peer_port, 
-				  const string& parameter)
+				  const string& parameter,
+				  const bool&	toggle)
 {
-    debug_msg("local ip %s local port %d peer ip %s peer port %d paremeter %s\n",
-	      local_ip.c_str(), local_port, peer_ip.c_str(), peer_port, parameter.c_str());
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " parameter %s state %s\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      parameter.c_str(), toggle ? "set" : "unset");
 
-    Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(), peer_port);
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
 
-    if(!_bgp.set_parameter(iptuple,parameter))
-	return XrlCmdError::COMMAND_FAILED();
+	if(!_bgp.set_parameter(iptuple,parameter, toggle))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
 
     return XrlCmdError::OKAY();
 }
@@ -812,6 +1004,159 @@ XrlBgpTarget::finder_event_observer_0_1_xrl_target_death(
     return XrlCmdError::OKAY();
 }
 
+XrlCmdError
+XrlBgpTarget::policy_backend_0_1_configure(const uint32_t& filter, 
+					   const string& conf) {
+    try {
+	_bgp.configure_filter(filter,conf);
+    } catch(const PolicyException& e) {
+	return XrlCmdError::COMMAND_FAILED("Filter configure failed: " +
+					   e.str());
+    }
+    return XrlCmdError::OKAY();					   
+}
+
+XrlCmdError
+XrlBgpTarget::policy_backend_0_1_reset(const uint32_t& filter) {
+    try {
+	_bgp.reset_filter(filter);
+    } catch(const PolicyException& e){ 
+	return XrlCmdError::COMMAND_FAILED("Filter reset failed: " +
+					   e.str());
+    }
+    return XrlCmdError::OKAY();					   
+}
+
+XrlCmdError
+XrlBgpTarget::policy_backend_0_1_push_routes() {
+    _bgp.push_routes();
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlBgpTarget::policy_redist4_0_1_add_route4(
+        const IPv4Net&	    network,
+        const bool&	    unicast,
+        const bool&	    multicast,
+        const IPv4&	    nexthop,
+        const uint32_t&	    metric,
+        const XrlAtomList&  policytags) {
+
+    UNUSED(metric);
+    _bgp.originate_route(network,nexthop,unicast,multicast,policytags);
+    return XrlCmdError::OKAY();
+	
+}	
+        
+XrlCmdError 
+XrlBgpTarget::policy_redist4_0_1_delete_route4(
+        const IPv4Net&	network,
+        const bool&     unicast,
+        const bool&     multicast) {
+
+    _bgp.withdraw_route(network,unicast,multicast);
+    return XrlCmdError::OKAY();
+}	
+        
+XrlCmdError 
+XrlBgpTarget::policy_redist6_0_1_add_route6(
+        const IPv6Net&	    network,
+        const bool&	    unicast,
+        const bool&	    multicast,
+        const IPv6&	    nexthop,
+        const uint32_t&	    metric,
+        const XrlAtomList&  policytags) {
+    
+    UNUSED(metric);
+    _bgp.originate_route(network,nexthop,unicast,multicast,policytags);
+    return XrlCmdError::OKAY();
+}	
+        
+XrlCmdError 
+XrlBgpTarget::policy_redist6_0_1_delete_route6(
+        const IPv6Net&  network,
+        const bool&     unicast,
+        const bool&     multicast) {
+    
+    _bgp.withdraw_route(network,unicast,multicast);
+    return XrlCmdError::OKAY();
+}	
+
+XrlCmdError
+XrlBgpTarget::profile_0_1_enable(const string& pname)
+{
+    debug_msg("profile variable %s\n", pname.c_str());
+    try {
+	_bgp.profile().enable(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::profile_0_1_disable(const string&	pname)
+{
+    debug_msg("profile variable %s\n", pname.c_str());
+    try {
+	_bgp.profile().disable(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlBgpTarget::profile_0_1_get_entries(const string& pname,
+				      const string& instance_name)
+{
+    debug_msg("profile variable %s instance %s\n", pname.c_str(),
+	      instance_name.c_str());
+
+    // Lock and initialize.
+    try {
+	_bgp.profile().lock_log(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    ProfileUtils::transmit_log(pname,
+			       _bgp.get_router(), instance_name,
+			       &_bgp.profile());
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::profile_0_1_clear(const string& pname)
+{
+    debug_msg("profile variable %s\n", pname.c_str());
+    try {
+	_bgp.profile().clear(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::profile_0_1_list(string& info)
+{
+    debug_msg("\n");
+    
+    info = _bgp.profile().list();
+    return XrlCmdError::OKAY();
+}
+
 bool 
 XrlBgpTarget::waiting()
 {
