@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_client.cc,v 1.24 2004/06/12 03:35:03 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_client.cc,v 1.27 2005/03/25 02:52:55 pavlin Exp $"
 
 
 //
@@ -21,15 +21,19 @@
 
 
 #include "cli_module.h"
-#include "cli_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
 #include "libxorp/eventloop.hh"
 #include "libxorp/token.hh"
 #include "libxorp/utils.hh"
+
 #include "libcomm/comm_api.h"
+
 #include "cli_client.hh"
 #include "cli_command_pipe.hh"
-
-
+#include "cli_private.hh"
 
 
 //
@@ -1222,10 +1226,10 @@ CliClient::command_line_help(const char *line, int word_end)
     cli_print(" \b");	// XXX: A hack to delete the '?'
 }
 
-CliCommand *
-CliClient::multi_command_find(const string& command_line)
+bool
+CliClient::is_multi_command_prefix(const string& command_line)
 {
-    return (_current_cli_command->multi_command_find(command_line));
+    return (_current_cli_command->is_multi_command_prefix(command_line));
 }
 
 int
