@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_command.cc,v 1.10 2004/06/12 00:31:44 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_command.cc,v 1.13 2005/03/25 02:52:56 pavlin Exp $"
 
 
 //
@@ -21,9 +21,13 @@
 
 
 #include "cli_module.h"
-#include "cli_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
 #include "libxorp/token.hh"
 #include "libxorp/utils.hh"
+
 #include "cli_command_pipe.hh"
 
 
@@ -571,16 +575,18 @@ CliCommand::command_find(const string& token)
     return (NULL);
 }
 
-// Find a multi-level command
-// TODO: consider/ignore the command arguments as appropriate
-CliCommand *
-CliCommand::multi_command_find(const string& command_line)
+//
+// Test if the command line is a prefix for a multi-level command.
+// Note that if there is an exact match, then the return value is false.
+//
+bool
+CliCommand::is_multi_command_prefix(const string& command_line)
 {
     string token;
     string token_line = command_line;
     CliCommand *parent_cli_command = this;
     CliCommand *child_cli_command = NULL;
-    
+
     for (token = pop_token(token_line);
 	 ! token.empty();
 	 token = pop_token(token_line)) {
@@ -589,16 +595,21 @@ CliCommand::multi_command_find(const string& command_line)
 	    parent_cli_command = child_cli_command;
 	    continue;
 	}
-	
-	if (parent_cli_command->has_cli_process_callback()) {
-	    // The parent command has processing function, so the rest
-	    // of the tokens could be arguments for that function
-	    child_cli_command = parent_cli_command;
+
+	// Test if the token is a prefix for a child command
+	list<CliCommand *>::const_iterator iter;
+	for (iter = parent_cli_command->child_command_list().begin();
+	     iter != parent_cli_command->child_command_list().end();
+	     ++iter) {
+	    child_cli_command = *iter;
+	    if (child_cli_command->is_same_prefix(token))
+		return true;
 	}
+
 	break;
     }
-    
-    return (child_cli_command);
+
+    return (false);
 }
 
 // Tests if the string in @token can be a prefix for this command
