@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_node_net.cc,v 1.28 2004/06/12 00:33:58 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_node_net.cc,v 1.34 2005/03/25 02:52:56 pavlin Exp $"
 
 
 //
@@ -21,7 +21,7 @@
 
 
 #include "cli_module.h"
-#include "cli_private.hh"
+#include "libxorp/xorp.h"
 
 #include <errno.h>
 #include <termios.h>
@@ -29,12 +29,16 @@
 #include <arpa/telnet.h>
 #include <sys/ioctl.h>
 
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
 #include "libxorp/c_format.hh"
 #include "libxorp/token.hh"
 
 #include "libcomm/comm_api.h"
 
 #include "cli_client.hh"
+#include "cli_private.hh"
 
 
 //
@@ -72,11 +76,11 @@ CliNode::sock_serv_open()
     // Open the socket
     switch (family()) {
     case AF_INET:
-	_cli_socket = comm_bind_tcp4(NULL, _cli_port);
+	_cli_socket = comm_bind_tcp4(NULL, _cli_port, COMM_SOCK_NONBLOCKING);
 	break;
 #ifdef HAVE_IPV6
     case AF_INET6:
-	_cli_socket = comm_bind_tcp6(NULL, _cli_port);
+	_cli_socket = comm_bind_tcp6(NULL, _cli_port, COMM_SOCK_NONBLOCKING);
 	break;
 #endif // HAVE_IPV6
     default:
@@ -210,7 +214,7 @@ CliNode::add_connection(int client_socket)
 	uint32_t i = 0;
 	
 	for (i = 0; i < CLI_MAX_CONNECTIONS; i++) {
-	    term_name = c_format("cli%u", i);
+	    term_name = c_format("cli%u", XORP_UINT_CAST(i));
 	    if (find_cli_by_term_name(term_name) == NULL)
 		break;
 	}
@@ -237,7 +241,7 @@ CliNode::add_connection(int client_socket)
 	if (i >= CLI_MAX_CONNECTIONS) {
 	    // This should not happen: there are available session slots,
 	    // but no available session IDs.
-	    assert(0 && "Cannot assign CLI session ID");
+	    XLOG_FATAL("Cannot assign CLI session ID");
 	    return (NULL);
 	}
 	cli_client->set_cli_session_session_id(session_id);
@@ -384,7 +388,7 @@ CliClient::start_connection()
 		cli_print(c_format("Invalid window width (%u); "
 				   "window width unchanged (%u)\n",
 				   new_window_width,
-				   static_cast<uint32_t>(window_width())));
+				   XORP_UINT_CAST(window_width())));
 	    }
 	    if (new_window_height > 0) {
 		set_window_height(new_window_height);
@@ -392,14 +396,14 @@ CliClient::start_connection()
 		cli_print(c_format("Invalid window height (%u); "
 				   "window height unchanged (%u)\n",
 				   new_window_height,
-				   static_cast<uint32_t>(window_height())));
+				   XORP_UINT_CAST(window_height())));
 	    }
 
 	    gl_terminal_size(gl(), window_width(), window_height());
 	    debug_msg("Client window size changed to width = %u "
 		      "height = %u\n",
-		      (uint32_t)window_width(),
-		      (uint32_t)window_height());
+		      XORP_UINT_CAST(window_width()),
+		      XORP_UINT_CAST(window_height()));
 	}
     }
 
@@ -611,9 +615,7 @@ CliClient::preprocess_char(uint8_t val)
 	int tmp_buff_curpos = buff_curpos();
 	char *tmp_line = (char *)command_buffer().data();
 	string command_line = string(tmp_line, tmp_buff_curpos);
-	string token_line = command_line;
-	string token = pop_token(token_line);
-	if (token.empty() || (multi_command_find(command_line) != NULL)) {
+	if (! is_multi_command_prefix(command_line)) {
 	    // Un-bind the "SPACE" to complete-word
 	    // Don't ask why we need six '\' to specify the ASCII value
 	    // of 'SPACE'...
@@ -681,7 +683,7 @@ CliClient::process_telnet_option(int val)
 			cli_print(c_format("Invalid window width (%u); "
 					   "window width unchanged (%u)\n",
 					   new_window_width,
-					   static_cast<uint32_t>(window_width())));
+					   XORP_UINT_CAST(window_width())));
 		    }
 		    if (new_window_height > 0) {
 			set_window_height(new_window_height);
@@ -689,14 +691,14 @@ CliClient::process_telnet_option(int val)
 			cli_print(c_format("Invalid window height (%u); "
 					   "window height unchanged (%u)\n",
 					   new_window_height,
-					   static_cast<uint32_t>(window_height())));
+					   XORP_UINT_CAST(window_height())));
 		    }
 
 		    gl_terminal_size(gl(), window_width(), window_height());
 		    debug_msg("Client window size changed to width = %u "
 			      "height = %u\n",
-			      static_cast<uint32_t>(window_width()),
-			      static_cast<uint32_t>(window_height()));
+			      XORP_UINT_CAST(window_width()),
+			      XORP_UINT_CAST(window_height()));
 		}
 		break;
 	    default:
