@@ -1,7 +1,8 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
+/* vim:set sts=4 ts=8: */
 
 /*
- * Copyright (c) 2001-2004 International Computer Science Institute
+ * Copyright (c) 2001-2005 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -14,4 +15,4 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/devnotes/template.c,v 1.3 2004/06/10 22:40:45 hodson Exp $"
+#ident "$XORP: xorp/devnotes/template.c,v 1.5 2005/03/25 02:52:59 pavlin Exp $"
