@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +12,10 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig_entry_set.cc,v 1.3 2004/06/10 22:40:49 hodson Exp $"
-
+#ident "$XORP: xorp/fea/fticonfig_entry_set.cc,v 1.9 2005/03/25 02:53:03 pavlin Exp $"
 
 #include "fea_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
@@ -32,7 +32,8 @@
 FtiConfigEntrySet::FtiConfigEntrySet(FtiConfig& ftic)
     : _is_running(false),
       _ftic(ftic),
-      _in_configuration(false)
+      _in_configuration(false),
+      _is_primary(true)
 {
     
 }
@@ -43,7 +44,27 @@ FtiConfigEntrySet::~FtiConfigEntrySet()
 }
 
 void
-FtiConfigEntrySet::register_ftic()
+FtiConfigEntrySet::register_ftic_primary()
 {
-    _ftic.register_ftic_entry_set(this);
+    _ftic.register_ftic_entry_set_primary(this);
+}
+
+void
+FtiConfigEntrySet::register_ftic_secondary()
+{
+    _ftic.register_ftic_entry_set_secondary(this);
+
+    //
+    // XXX: push the current config into the new secondary
+    //
+    if (_is_running) {
+	// XXX: nothing to do. 
+	//
+	// XXX: note that the forwarding table state in the secondary methods
+	// is pushed by FtiConfigTableSet::register_ftic_secondary(), hence
+	// we don't need to do it here again. However, this is based on the
+	// assumption that for each FtiConfigEntrySet secondary method
+	// there is a corresponding FtiConfigTableSet secondary method.
+	//
+    }
 }
