@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +12,13 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_observer.cc,v 1.3 2004/06/10 22:40:52 hodson Exp $"
-
+#ident "$XORP: xorp/fea/ifconfig_observer.cc,v 1.7 2005/03/25 02:53:06 pavlin Exp $"
 
 #include "fea_module.h"
+
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
-#include "libxorp/xorp.h"
 
 #include "ifconfig.hh"
 #include "ifconfig_observer.hh"
@@ -32,7 +32,8 @@
 
 IfConfigObserver::IfConfigObserver(IfConfig& ifc)
     : _is_running(false),
-      _ifc(ifc)
+      _ifc(ifc),
+      _is_primary(true)
 {
     
 }
@@ -43,7 +44,13 @@ IfConfigObserver::~IfConfigObserver()
 }
 
 void
-IfConfigObserver::register_ifc()
+IfConfigObserver::register_ifc_primary()
 {
-    _ifc.register_ifc_observer(this);
+    _ifc.register_ifc_observer_primary(this);
+}
+
+void
+IfConfigObserver::register_ifc_secondary()
+{
+    _ifc.register_ifc_observer_secondary(this);
 }
