@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +12,10 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_set_ioctl.cc,v 1.22 2004/06/17 03:59:06 pavlin Exp $"
-
+#ident "$XORP: xorp/fea/ifconfig_set_ioctl.cc,v 1.36 2005/03/25 02:53:08 pavlin Exp $"
 
 #include "fea_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
@@ -80,30 +80,48 @@ IfConfigSetIoctl::IfConfigSetIoctl(IfConfig& ifc)
     : IfConfigSet(ifc), _s4(-1), _s6(-1)
 {
 #ifdef HAVE_IOCTL_SIOCGIFCONF
-    register_ifc();
+    register_ifc_primary();
 #endif
 }
 
 IfConfigSetIoctl::~IfConfigSetIoctl()
 {
-    stop();
+    string error_msg;
+
+    if (stop(error_msg) != XORP_OK) {
+	XLOG_ERROR("Cannot stop the ioctl(2) mechanism to set "
+		   "information about network interfaces into the underlying "
+		   "system: %s",
+		   error_msg.c_str());
+    }
 }
 
 int
-IfConfigSetIoctl::start()
+IfConfigSetIoctl::start(string& error_msg)
 {
-    if (_s4 < 0) {
-	_s4 = socket(AF_INET, SOCK_DGRAM, 0);
+    if (_is_running)
+	return (XORP_OK);
+
+    if (ifc().have_ipv4()) {
 	if (_s4 < 0) {
-	    XLOG_FATAL("Could not initialize IPv4 ioctl() socket");
+	    _s4 = socket(AF_INET, SOCK_DGRAM, 0);
+	    if (_s4 < 0) {
+		error_msg = c_format("Could not initialize IPv4 ioctl() "
+				     "socket: %s", strerror(errno));
+		XLOG_FATAL("%s", error_msg.c_str());
+	    }
 	}
     }
     
 #ifdef HAVE_IPV6
-    if (_s6 < 0) {
-	_s6 = socket(AF_INET6, SOCK_DGRAM, 0);
+    if (ifc().have_ipv6()) {
 	if (_s6 < 0) {
-	    XLOG_FATAL("Could not initialize IPv6 ioctl() socket");
+	    _s6 = socket(AF_INET6, SOCK_DGRAM, 0);
+	    if (_s6 < 0) {
+		error_msg = c_format("Could not initialize IPv6 ioctl() "
+				     "socket: %s", strerror(errno));
+		XLOG_FATAL("%s", error_msg.c_str());
+	    }
 	}
     }
 #endif // HAVE_IPV6
@@ -114,31 +132,178 @@ IfConfigSetIoctl::start()
 }
 
 int
-IfConfigSetIoctl::stop()
+IfConfigSetIoctl::stop(string& error_msg)
 {
+    int ret_value4 = XORP_OK;
+    int ret_value6 = XORP_OK;
+
     if (! _is_running)
 	return (XORP_OK);
 
     if (_s4 >= 0) {
-	close(_s4);
+	ret_value4 = close(_s4);
 	_s4 = -1;
+	if (ret_value4 < 0) {
+	    error_msg = c_format("Could not close IPv4 ioctl() "
+				 "socket: %s", strerror(errno));
+	}
     }
     if (_s6 >= 0) {
-	close(_s6);
+	ret_value6 = close(_s6);
 	_s6 = -1;
+	if ((ret_value6 < 0) && (ret_value4 >= 0)) {
+	    error_msg = c_format("Could not close IPv6 ioctl() "
+				 "socket: %s", strerror(errno));
+	}
     }
 
+    if ((ret_value4 < 0) || (ret_value6 < 0))
+	return (XORP_ERROR);
+
     _is_running = false;
 
     return (XORP_OK);
 }
 
+bool
+IfConfigSetIoctl::is_discard_emulated(const IfTreeInterface& i) const
+{
+#if defined(HOST_OS_BSDI) || defined(HOST_OS_FREEBSD) || defined(HOST_OS_MACOSX) || defined(HOST_OS_NETBSD) || defined(HOST_OS_OPENBSD)
+    return (true);
+#else
+    return (false);
+#endif
+    UNUSED(i);
+}
+
 #ifndef HAVE_IOCTL_SIOCGIFCONF
+int
+IfConfigSetIoctl::config_begin(string& error_msg)
+{
+    debug_msg("config_begin\n");
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetIoctl::config_end(string& error_msg)
+{
+    debug_msg("config_end\n");
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetIoctl::add_interface(const string& ifname,
+				uint16_t if_index,
+				string& error_msg)
+{
+    debug_msg("add_interface "
+	      "(ifname = %s if_index = %u)\n",
+	      ifname.c_str(), if_index);
+
+    UNUSED(ifname);
+    UNUSED(if_index);
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetIoctl::add_vif(const string& ifname,
+			  const string& vifname,
+			  uint16_t if_index,
+			  string& error_msg)
+{
+    debug_msg("add_vif "
+	      "(ifname = %s vifname = %s if_index = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index);
+
+    UNUSED(ifname);
+    UNUSED(vifname);
+    UNUSED(if_index);
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetIoctl::config_interface(const string& ifname,
+				   uint16_t if_index,
+				   uint32_t flags,
+				   bool is_up,
+				   bool is_deleted,
+				   string& error_msg)
+{
+    debug_msg("config_interface "
+	      "(ifname = %s if_index = %u flags = 0x%x is_up = %s "
+	      "is_deleted = %s)\n",
+	      ifname.c_str(), if_index, flags, (is_up)? "true" : "false",
+	      (is_deleted)? "true" : "false");
+
+    UNUSED(ifname);
+    UNUSED(if_index);
+    UNUSED(flags);
+    UNUSED(is_up);
+    UNUSED(is_deleted);
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetIoctl::config_vif(const string& ifname,
+			     const string& vifname,
+			     uint16_t if_index,
+			     uint32_t flags,
+			     bool is_up,
+			     bool is_deleted,
+			     bool broadcast,
+			     bool loopback,
+			     bool point_to_point,
+			     bool multicast,
+			     string& error_msg)
+{
+    debug_msg("config_vif "
+	      "(ifname = %s vifname = %s if_index = %u flags = 0x%x "
+	      "is_up = %s is_deleted = %s broadcast = %s loopback = %s "
+	      "point_to_point = %s multicast = %s)\n",
+	      ifname.c_str(), vifname.c_str(), if_index, flags,
+	      (is_up)? "true" : "false",
+	      (is_deleted)? "true" : "false",
+	      (broadcast)? "true" : "false",
+	      (loopback)? "true" : "false",
+	      (point_to_point)? "true" : "false",
+	      (multicast)? "true" : "false");
+
+    UNUSED(ifname);
+    UNUSED(vifname);
+    UNUSED(if_index);
+    UNUSED(flags);
+    UNUSED(is_up);
+    UNUSED(is_deleted);
+    UNUSED(broadcast);
+    UNUSED(loopback);
+    UNUSED(point_to_point);
+    UNUSED(multicast);
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
 int
 IfConfigSetIoctl::set_interface_mac_address(const string& ifname,
 					    uint16_t if_index,
 					    const struct ether_addr& ether_addr,
-					    string& reason)
+					    string& error_msg)
 {
     debug_msg("set_interface_mac "
 	      "(ifname = %s if_index = %u mac = %s)\n",
@@ -148,7 +313,7 @@ IfConfigSetIoctl::set_interface_mac_address(const string& ifname,
     UNUSED(if_index);
     UNUSED(ether_addr);
 
-    reason = "method not supported";
+    error_msg = "method not supported";
 
     return (XORP_ERROR);
 }
@@ -157,7 +322,7 @@ int
 IfConfigSetIoctl::set_interface_mtu(const string& ifname,
 				    uint16_t if_index,
 				    uint32_t mtu,
-				    string& reason)
+				    string& error_msg)
 {
     debug_msg("set_interface_mtu "
 	      "(ifname = %s if_index = %u mtu = %u)\n",
@@ -167,48 +332,31 @@ IfConfigSetIoctl::set_interface_mtu(const string& ifname,
     UNUSED(if_index);
     UNUSED(mtu);
 
-    reason = "method not supported";
+    error_msg = "method not supported";
 
     return (XORP_ERROR);
 }
 
 int
-IfConfigSetIoctl::set_interface_flags(const string& ifname,
-				      uint16_t if_index,
-				      uint32_t flags,
-				      string& reason)
-{
-    debug_msg("set_interface_flags "
-	      "(ifname = %s if_index = %u flags = 0x%x)\n",
-	      ifname.c_str(), if_index, flags);
-
-    UNUSED(ifname);
-    UNUSED(if_index);
-    UNUSED(flags);
-
-    reason = "method not supported";
-
-    return (XORP_ERROR);
-}
-
-int
-IfConfigSetIoctl::set_vif_address(const string& ifname,
+IfConfigSetIoctl::add_vif_address(const string& ifname,
+				  const string& vifname,
 				  uint16_t if_index,
 				  bool is_broadcast,
 				  bool is_p2p,
 				  const IPvX& addr,
 				  const IPvX& dst_or_bcast,
 				  uint32_t prefix_len,
-				  string& reason)
+				  string& error_msg)
 {
-    debug_msg("set_vif_address "
-	      "(ifname = %s if_index = %u is_broadcast = %s is_p2p = %s "
-	      "addr = %s dst/bcast = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index, (is_broadcast)? "true" : "false",
-	      (is_p2p)? "true" : "false", addr.str().c_str(),
-	      dst_or_bcast.str().c_str(), prefix_len);
+    debug_msg("add_vif_address "
+	      "(ifname = %s vifname = %s if_index = %u is_broadcast = %s "
+	      "is_p2p = %s addr = %s dst/bcast = %s prefix_len = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index,
+	      (is_broadcast)? "true" : "false", (is_p2p)? "true" : "false",
+	      addr.str().c_str(), dst_or_bcast.str().c_str(), prefix_len);
 
     UNUSED(ifname);
+    UNUSED(vifname);
     UNUSED(if_index);
     UNUSED(is_broadcast);
     UNUSED(is_p2p);
@@ -216,39 +364,179 @@ IfConfigSetIoctl::set_vif_address(const string& ifname,
     UNUSED(dst_or_bcast);
     UNUSED(prefix_len);
 
-    reason = "method not supported";
+    error_msg = "method not supported";
 
     return (XORP_ERROR);
 }
 
 int
 IfConfigSetIoctl::delete_vif_address(const string& ifname,
+				     const string& vifname,
 				     uint16_t if_index,
 				     const IPvX& addr,
 				     uint32_t prefix_len,
-				     string& reason)
+				     string& error_msg)
 {
     debug_msg("delete_vif_address "
-	      "(ifname = %s if_index = %u addr = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index, addr.str().c_str(), prefix_len);
+	      "(ifname = %s vifname = %s if_index = %u addr = %s "
+	      "prefix_len = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index, addr.str().c_str(),
+	      prefix_len);
 
     UNUSED(ifname);
+    UNUSED(vifname);
     UNUSED(if_index);
     UNUSED(addr);
     UNUSED(prefix_len);
 
-    reason = "method not supported";
+    error_msg = "method not supported";
 
     return (XORP_ERROR);
 }
 
 #else // HAVE_IOCTL_SIOCGIFCONF
 
+int
+IfConfigSetIoctl::config_begin(string& error_msg)
+{
+    debug_msg("config_begin\n");
+
+    // XXX: nothing to do
+
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
+int
+IfConfigSetIoctl::config_end(string& error_msg)
+{
+    debug_msg("config_end\n");
+
+    // XXX: nothing to do
+
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
+int
+IfConfigSetIoctl::add_interface(const string& ifname,
+				uint16_t if_index,
+				string& error_msg)
+{
+    debug_msg("add_interface "
+	      "(ifname = %s if_index = %u)\n",
+	      ifname.c_str(), if_index);
+
+    // XXX: nothing to do
+
+    UNUSED(ifname);
+    UNUSED(if_index);
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
+int
+IfConfigSetIoctl::add_vif(const string& ifname,
+			  const string& vifname,
+			  uint16_t if_index,
+			  string& error_msg)
+{
+    debug_msg("add_vif "
+	      "(ifname = %s vifname = %s if_index = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index);
+
+    // XXX: nothing to do
+
+    UNUSED(ifname);
+    UNUSED(vifname);
+    UNUSED(if_index);
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
+int
+IfConfigSetIoctl::config_interface(const string& ifname,
+				   uint16_t if_index,
+				   uint32_t flags,
+				   bool is_up,
+				   bool is_deleted,
+				   string& error_msg)
+{
+    struct ifreq ifreq;
+
+    debug_msg("config_interface "
+	      "(ifname = %s if_index = %u flags = 0x%x is_up = %s "
+	      "is_deleted = %s)\n",
+	      ifname.c_str(), if_index, flags, (is_up)? "true" : "false",
+	      (is_deleted)? "true" : "false");
+
+    UNUSED(if_index);
+    UNUSED(is_up);
+    UNUSED(is_deleted);
+
+    memset(&ifreq, 0, sizeof(ifreq));
+    strncpy(ifreq.ifr_name, ifname.c_str(), sizeof(ifreq.ifr_name) - 1);
+
+    ifreq.ifr_flags = flags;
+    if (ioctl(_s4, SIOCSIFFLAGS, &ifreq) < 0) {
+	error_msg = c_format("%s", strerror(errno));
+	return (XORP_ERROR);
+    }
+
+    return (XORP_OK);
+}
+
+int
+IfConfigSetIoctl::config_vif(const string& ifname,
+			     const string& vifname,
+			     uint16_t if_index,
+			     uint32_t flags,
+			     bool is_up,
+			     bool is_deleted,
+			     bool broadcast,
+			     bool loopback,
+			     bool point_to_point,
+			     bool multicast,
+			     string& error_msg)
+{
+    debug_msg("config_vif "
+	      "(ifname = %s vifname = %s if_index = %u flags = 0x%x "
+	      "is_up = %s is_deleted = %s broadcast = %s loopback = %s "
+	      "point_to_point = %s multicast = %s)\n",
+	      ifname.c_str(), vifname.c_str(), if_index, flags,
+	      (is_up)? "true" : "false",
+	      (is_deleted)? "true" : "false",
+	      (broadcast)? "true" : "false",
+	      (loopback)? "true" : "false",
+	      (point_to_point)? "true" : "false",
+	      (multicast)? "true" : "false");
+
+    // XXX: nothing to do
+
+    UNUSED(ifname);
+    UNUSED(vifname);
+    UNUSED(if_index);
+    UNUSED(flags);
+    UNUSED(is_up);
+    UNUSED(is_deleted);
+    UNUSED(broadcast);
+    UNUSED(loopback);
+    UNUSED(point_to_point);
+    UNUSED(multicast);
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
 int
 IfConfigSetIoctl::set_interface_mac_address(const string& ifname,
 					    uint16_t if_index,
 					    const struct ether_addr& ether_addr,
-					    string& reason)
+					    string& error_msg)
 {
     struct ifreq ifreq;
 
@@ -271,7 +559,7 @@ IfConfigSetIoctl::set_interface_mac_address(const string& ifname,
     ifreq.ifr_addr.sa_len = ETHER_ADDR_LEN;
 #endif
     if (ioctl(_s4, SIOCSIFLLADDR, &ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	return (XORP_ERROR);
     }
     return (XORP_OK);
@@ -286,7 +574,7 @@ IfConfigSetIoctl::set_interface_mac_address(const string& ifname,
     ifreq.ifr_hwaddr.sa_len = ETH_ALEN;
 #endif
     if (ioctl(_s4, SIOCSIFHWADDR, &ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	return (XORP_ERROR);
     }
     return (XORP_OK);
@@ -299,7 +587,8 @@ IfConfigSetIoctl::set_interface_mac_address(const string& ifname,
     // setting the MAC address.
     //
     UNUSED(ether_addr);
-    reason = c_format("No mechanism to set the MAC address on an interface");
+    error_msg = c_format("No mechanism to set the MAC address "
+			 "on an interface");
     return (XORP_ERROR);
 #endif
 }
@@ -308,7 +597,7 @@ int
 IfConfigSetIoctl::set_interface_mtu(const string& ifname,
 				    uint16_t if_index,
 				    uint32_t mtu,
-				    string& reason)
+				    string& error_msg)
 {
     struct ifreq ifreq;
 
@@ -323,63 +612,40 @@ IfConfigSetIoctl::set_interface_mtu(const string& ifname,
 
     ifreq.ifr_mtu = mtu;
     if (ioctl(_s4, SIOCSIFMTU, &ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	return (XORP_ERROR);
     }
     return (XORP_OK);
 }
 
 int
-IfConfigSetIoctl::set_interface_flags(const string& ifname,
-				      uint16_t if_index,
-				      uint32_t flags,
-				      string& reason)
-{
-    struct ifreq ifreq;
-
-    debug_msg("set_interface_flags "
-	      "(ifname = %s if_index = %u flags = 0x%x)\n",
-	      ifname.c_str(), if_index, flags);
-
-    UNUSED(if_index);
-
-    memset(&ifreq, 0, sizeof(ifreq));
-    strncpy(ifreq.ifr_name, ifname.c_str(), sizeof(ifreq.ifr_name) - 1);
-
-    ifreq.ifr_flags = flags;
-    if (ioctl(_s4, SIOCSIFFLAGS, &ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
-	return (XORP_ERROR);
-    }
-    return (XORP_OK);
-}
-
-int
-IfConfigSetIoctl::set_vif_address(const string& ifname,
+IfConfigSetIoctl::add_vif_address(const string& ifname,
+				  const string& vifname,
 				  uint16_t if_index,
 				  bool is_broadcast,
 				  bool is_p2p,
 				  const IPvX& addr,
 				  const IPvX& dst_or_bcast,
 				  uint32_t prefix_len,
-				  string& reason)
+				  string& error_msg)
 {
-    debug_msg("set_vif_address "
-	      "(ifname = %s if_index = %u is_broadcast = %s is_p2p = %s "
-	      "addr = %s dst/bcast = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index, (is_broadcast)? "true" : "false",
-	      (is_p2p)? "true" : "false", addr.str().c_str(),
-	      dst_or_bcast.str().c_str(), prefix_len);
+    debug_msg("add_vif_address "
+	      "(ifname = %s vifname = %s if_index = %u is_broadcast = %s "
+	      "is_p2p = %s addr = %s dst/bcast = %s prefix_len = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index,
+	      (is_broadcast)? "true" : "false", (is_p2p)? "true" : "false",
+	      addr.str().c_str(), dst_or_bcast.str().c_str(), prefix_len);
 
     switch (addr.af()) {
     case AF_INET:
-	return set_vif_address4(ifname, if_index, is_broadcast, is_p2p,
-				addr, dst_or_bcast, prefix_len, reason);
+	return add_vif_address4(ifname, vifname, if_index, is_broadcast,
+				is_p2p, addr, dst_or_bcast, prefix_len,
+				error_msg);
 	break;
 #ifdef HAVE_IPV6
     case AF_INET6:
-	return set_vif_address6(ifname, if_index, is_p2p,
-				addr, dst_or_bcast, prefix_len, reason);
+	return add_vif_address6(ifname, vifname, if_index, is_p2p,
+				addr, dst_or_bcast, prefix_len, error_msg);
 	break;
 #endif // HAVE_IPV6
     default:
@@ -392,31 +658,39 @@ IfConfigSetIoctl::set_vif_address(const string& ifname,
 }
 
 /**
- * Set IPv4 interface addresses
+ * Add an IPv4 address to an interface.
  *
  * Note: if the system has ioctl(SIOCAIFADDR) (e.g., FreeBSD), then
  * the interface address is added as an alias, otherwise it overwrites the
  * previous address (if such exists).
  */
 int
-IfConfigSetIoctl::set_vif_address4(const string& ifname,
+IfConfigSetIoctl::add_vif_address4(const string& ifname,
+				   const string& vifname,
 				   uint16_t if_index,
 				   bool is_broadcast,
 				   bool is_p2p,
 				   const IPvX& addr,
 				   const IPvX& dst_or_bcast,
 				   uint32_t prefix_len,
-				   string& reason)
+				   string& error_msg)
 {
-    debug_msg("set_vif_address4 "
-	      "(ifname = %s if_index = %u is_broadcast = %s is_p2p = %s "
-	      "addr = %s dst/bcast = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index, (is_broadcast)? "true" : "false",
-	      (is_p2p)? "true" : "false", addr.str().c_str(),
-	      dst_or_bcast.str().c_str(), prefix_len);
+    debug_msg("add_vif_address4 "
+	      "(ifname = %s vifname = %s if_index = %u is_broadcast = %s "
+	      "is_p2p = %s addr = %s dst/bcast = %s prefix_len = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index,
+	      (is_broadcast)? "true" : "false", (is_p2p)? "true" : "false",
+	      addr.str().c_str(), dst_or_bcast.str().c_str(), prefix_len);
 
+    UNUSED(vifname);
     UNUSED(if_index);
     UNUSED(is_broadcast);
+
+    if (! ifc().have_ipv4()) {
+	error_msg = "IPv4 is not supported";
+	return (XORP_ERROR);
+    }
+
 #ifdef SIOCAIFADDR
     //
     // Add an alias address
@@ -432,11 +706,12 @@ IfConfigSetIoctl::set_vif_address4(const string& ifname,
 	dst_or_bcast.copy_out(ifra.ifra_broadaddr);
     IPvX::make_prefix(addr.af(), prefix_len).copy_out(ifra.ifra_mask);
     if (ioctl(_s4, SIOCAIFADDR, &ifra) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	return (XORP_ERROR);
     }
+    return (XORP_OK);
 
-#endif // SIOCAIFADDR
+#else // ! SIOCAIFADDR
 
     //
     // Set a new address
@@ -449,14 +724,14 @@ IfConfigSetIoctl::set_vif_address4(const string& ifname,
     // Set the address
     addr.copy_out(ifreq.ifr_addr);
     if (ioctl(_s4, SIOCSIFADDR, &ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	return (XORP_ERROR);
     }
 
     // Set the netmask
     IPvX::make_prefix(addr.af(), prefix_len).copy_out(ifreq.ifr_addr);
     if (ioctl(_s4, SIOCSIFNETMASK, &ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	return (XORP_ERROR);
     }
 
@@ -464,58 +739,67 @@ IfConfigSetIoctl::set_vif_address4(const string& ifname,
     if (is_p2p) {
 	dst_or_bcast.copy_out(ifreq.ifr_dstaddr);
 	if (ioctl(_s4, SIOCSIFDSTADDR, &ifreq) < 0) {
-	    reason = c_format("%s", strerror(errno));
+	    error_msg = c_format("%s", strerror(errno));
 	    return (XORP_ERROR);
 	}
     } else {
 	dst_or_bcast.copy_out(ifreq.ifr_broadaddr);
 	if (ioctl(_s4, SIOCSIFBRDADDR, &ifreq) < 0) {
-	    reason = c_format("%s", strerror(errno));
+	    error_msg = c_format("%s", strerror(errno));
 	    return (XORP_ERROR);
 	}
     }
     return (XORP_OK);
+#endif // ! SIOCAIFADDR
 }
 
 /**
- * Set IPv6 interface addresses
+ * Add an IPv6 address to an interface.
  */
 int
-IfConfigSetIoctl::set_vif_address6(const string& ifname,
+IfConfigSetIoctl::add_vif_address6(const string& ifname,
+				   const string& vifname,
 				   uint16_t if_index,
 				   bool is_p2p,
 				   const IPvX& addr,
 				   const IPvX& dst,
 				   uint32_t prefix_len,
-				   string& reason)
+				   string& error_msg)
 {
-    debug_msg("set_vif_address6 "
-	      "(ifname = %s if_index = %u is_p2p = %s "
+    debug_msg("add_vif_address6 "
+	      "(ifname = %s vifname = %s if_index = %u is_p2p = %s "
 	      "addr = %s dst = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index,
+	      ifname.c_str(), vifname.c_str(), if_index,
 	      (is_p2p)? "true" : "false", addr.str().c_str(),
 	      dst.str().c_str(), prefix_len);
 
 #ifndef HAVE_IPV6
     UNUSED(ifname);
+    UNUSED(vifname);
     UNUSED(if_index);
     UNUSED(is_p2p);
     UNUSED(addr);
     UNUSED(dst);
     UNUSED(prefix_len);
     
-    reason = "IPv6 is not supported";
+    error_msg = "IPv6 is not supported";
 
     return (XORP_ERROR);
 
 #else // HAVE_IPV6
 
+    if (! ifc().have_ipv6()) {
+	error_msg = "IPv6 is not supported";
+	return (XORP_ERROR);
+    }
+
 #ifdef SIOCAIFADDR_IN6
     //
     // Add an alias address
     //
     struct in6_aliasreq ifra;
 
+    UNUSED(vifname);
     UNUSED(if_index);
 
     memset(&ifra, 0, sizeof(ifra));
@@ -529,12 +813,12 @@ IfConfigSetIoctl::set_vif_address6(const string& ifname,
     ifra.ifra_lifetime.ia6t_pltime = ND6_INFINITE_LIFETIME;
 
     if (ioctl(_s6, SIOCAIFADDR_IN6, &ifra) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	return (XORP_ERROR);
     }
     return (XORP_OK);
 
-#else // !SIOCAIFADDR_IN6
+#else // ! SIOCAIFADDR_IN6
 
     //
     // Set a new address
@@ -548,6 +832,7 @@ IfConfigSetIoctl::set_vif_address6(const string& ifname,
     struct in6_ifreq in6_ifreq;
 
     UNUSED(ifname);
+    UNUSED(vifname);
 
     memset(&in6_ifreq, 0, sizeof(in6_ifreq));
     in6_ifreq.ifr6_ifindex = if_index;
@@ -556,7 +841,7 @@ IfConfigSetIoctl::set_vif_address6(const string& ifname,
     addr.copy_out(in6_ifreq.ifr6_addr);
     in6_ifreq.ifr6_prefixlen = prefix_len;
     if (ioctl(_s6, SIOCSIFADDR, &in6_ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	return (XORP_ERROR);
     }
 
@@ -564,7 +849,7 @@ IfConfigSetIoctl::set_vif_address6(const string& ifname,
     if (is_p2p) {
 	dst.copy_out(in6_ifreq.ifr6_addr);
 	if (ioctl(_s6, SIOCSIFDSTADDR, &in6_ifreq) < 0) {
-	    reason = c_format("%s", strerror(errno));
+	    error_msg = c_format("%s", strerror(errno));
 	    return (XORP_ERROR);
 	}
     }
@@ -576,27 +861,57 @@ IfConfigSetIoctl::set_vif_address6(const string& ifname,
 
 int
 IfConfigSetIoctl::delete_vif_address(const string& ifname,
+				     const string& vifname,
 				     uint16_t if_index,
 				     const IPvX& addr,
 				     uint32_t prefix_len,
-				     string& reason)
+				     string& error_msg)
 {
     debug_msg("delete_vif_address "
-	      "(ifname = %s if_index = %u addr = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index, addr.str().c_str(), prefix_len);
+	      "(ifname = %s vifname = %s if_index = %u addr = %s "
+	      "prefix_len = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index, addr.str().c_str(),
+	      prefix_len);
+
+    // Check that the family is supported
+    switch (addr.af()) {
+    case AF_INET:
+	if (! ifc().have_ipv4()) {
+	    error_msg = "IPv4 is not supported";
+	    return (XORP_ERROR);
+	}
+	break;
+
+#ifdef HAVE_IPV6
+    case AF_INET6:
+	if (! ifc().have_ipv6()) {
+	    error_msg = "IPv6 is not supported";
+	    return (XORP_ERROR);
+	}
+	break;
+#endif // HAVE_IPV6
+
+    default:
+	XLOG_UNREACHABLE();
+	break;
+    }
 
     switch (addr.af()) {
     case AF_INET:
     {
 	struct ifreq ifreq;
 
+	UNUSED(vifname);
+	UNUSED(if_index);
+	UNUSED(prefix_len);
+
 	memset(&ifreq, 0, sizeof(ifreq));
 	strncpy(ifreq.ifr_name, ifname.c_str(), sizeof(ifreq.ifr_name) - 1);
 
 #ifndef HOST_OS_LINUX
 	addr.copy_out(ifreq.ifr_addr);
 	if (ioctl(_s4, SIOCDIFADDR, &ifreq) < 0) {
-	    reason = c_format("%s", strerror(errno));
+	    error_msg = c_format("%s", strerror(errno));
 	    return (XORP_ERROR);
 	}
 #else // HOST_OS_LINUX
@@ -607,7 +922,7 @@ IfConfigSetIoctl::delete_vif_address(const string& ifname,
 	UNUSED(addr);
 	IPv4::ZERO().copy_out(ifreq.ifr_addr);
 	if (ioctl(_s4, SIOCSIFADDR, &ifreq) < 0) {
-	    reason = c_format("%s", strerror(errno));
+	    error_msg = c_format("%s", strerror(errno));
 	    return (XORP_ERROR);
 	}
 #endif // HOST_OS_LINUX
@@ -631,12 +946,12 @@ IfConfigSetIoctl::delete_vif_address(const string& ifname,
 	addr.copy_out(in6_ifreq.ifr_addr);
 
 	if (ioctl(_s6, SIOCDIFADDR_IN6, &in6_ifreq) < 0) {
-	    reason = c_format("%s", strerror(errno));
+	    error_msg = c_format("%s", strerror(errno));
 	    return (XORP_ERROR);
 	}
 	return (XORP_OK);
 
-#else // !SIOCDIFADDR_IN6
+#else // ! SIOCDIFADDR_IN6
 	//
 	// XXX: Linux uses a weird struct in6_ifreq to do this, and this
 	// name clashes with the KAME in6_ifreq.
@@ -652,7 +967,7 @@ IfConfigSetIoctl::delete_vif_address(const string& ifname,
 	in6_ifreq.ifr6_prefixlen = prefix_len;
 
 	if (ioctl(_s6, SIOCDIFADDR, &in6_ifreq) < 0) {
-	    reason = c_format("%s", strerror(errno));
+	    error_msg = c_format("%s", strerror(errno));
 	    return (XORP_ERROR);
 	}
 	return (XORP_OK);
