@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +12,10 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_set_netlink.cc,v 1.8 2004/06/10 22:40:54 hodson Exp $"
-
+#ident "$XORP: xorp/fea/ifconfig_set_netlink.cc,v 1.21 2005/03/25 02:53:08 pavlin Exp $"
 
 #include "fea_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
@@ -55,25 +55,41 @@ IfConfigSetNetlink::IfConfigSetNetlink(IfConfig& ifc)
       _ns_reader(*(NetlinkSocket4 *)this, *(NetlinkSocket6 *)this)
 {
 #ifdef HAVE_NETLINK_SOCKETS
-    register_ifc();
+    register_ifc_primary();
 #endif
 }
 
 IfConfigSetNetlink::~IfConfigSetNetlink()
 {
-    stop();
+    string error_msg;
+
+    if (stop(error_msg) != XORP_OK) {
+	XLOG_ERROR("Cannot stop the netlink(7) sockets mechanism to set "
+		   "information about network interfaces into the underlying "
+		   "system: %s",
+		   error_msg.c_str());
+    }
 }
 
 int
-IfConfigSetNetlink::start()
+IfConfigSetNetlink::start(string& error_msg)
 {
-    if (NetlinkSocket4::start() < 0)
-	return (XORP_ERROR);
+    if (_is_running)
+	return (XORP_OK);
+
+    if (ifc().have_ipv4()) {
+	if (NetlinkSocket4::start(error_msg) < 0)
+	    return (XORP_ERROR);
+    }
     
 #ifdef HAVE_IPV6
-    if (NetlinkSocket6::start() < 0) {
-	NetlinkSocket4::stop();
-	return (XORP_ERROR);
+    if (ifc().have_ipv6()) {
+	if (NetlinkSocket6::start(error_msg) < 0) {
+	    string error_msg2;
+	    if (ifc().have_ipv4())
+		NetlinkSocket4::stop(error_msg2);
+	    return (XORP_ERROR);
+	}
     }
 #endif
 
@@ -83,7 +99,7 @@ IfConfigSetNetlink::start()
 }
 
 int
-IfConfigSetNetlink::stop()
+IfConfigSetNetlink::stop(string& error_msg)
 {
     int ret_value4 = XORP_OK;
     int ret_value6 = XORP_OK;
@@ -91,10 +107,16 @@ IfConfigSetNetlink::stop()
     if (! _is_running)
 	return (XORP_OK);
 
-    ret_value4 = NetlinkSocket4::stop();
+    if (ifc().have_ipv4())
+	ret_value4 = NetlinkSocket4::stop(error_msg);
     
 #ifdef HAVE_IPV6
-    ret_value6 = NetlinkSocket6::stop();
+    if (ifc().have_ipv6()) {
+	string error_msg2;
+	ret_value6 = NetlinkSocket6::stop(error_msg2);
+	if ((ret_value6 < 0) && (ret_value4 >= 0))
+	    error_msg = error_msg2;	// XXX: update the error message
+    }
 #endif
     
     if ((ret_value4 < 0) || (ret_value6 < 0))
@@ -105,12 +127,145 @@ IfConfigSetNetlink::stop()
     return (XORP_OK);
 }
 
+bool
+IfConfigSetNetlink::is_discard_emulated(const IfTreeInterface& i) const
+{
+#ifdef HOST_OS_LINUX
+    return (true);
+#else
+    return (false);
+#endif
+    UNUSED(i);
+}
+
 #ifndef HAVE_NETLINK_SOCKETS
+int
+IfConfigSetNetlink::config_begin(string& error_msg)
+{
+    debug_msg("config_begin\n");
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetNetlink::config_end(string& error_msg)
+{
+    debug_msg("config_end\n");
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetNetlink::add_interface(const string& ifname,
+				  uint16_t if_index,
+				  string& error_msg)
+{
+    debug_msg("add_interface "
+	      "(ifname = %s if_index = %u)\n",
+	      ifname.c_str(), if_index);
+
+    UNUSED(ifname);
+    UNUSED(if_index);
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetNetlink::add_vif(const string& ifname,
+			    const string& vifname,
+			    uint16_t if_index,
+			    string& error_msg)
+{
+    debug_msg("add_vif "
+	      "(ifname = %s vifname = %s if_index = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index);
+
+    UNUSED(ifname);
+    UNUSED(vifname);
+    UNUSED(if_index);
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetNetlink::config_interface(const string& ifname,
+				     uint16_t if_index,
+				     uint32_t flags,
+				     bool is_up,
+				     bool is_deleted,
+				     string& error_msg)
+{
+    debug_msg("config_interface "
+	      "(ifname = %s if_index = %u flags = 0x%x is_up = %s "
+	      "is_deleted = %s)\n",
+	      ifname.c_str(), if_index, flags, (is_up)? "true" : "false",
+	      (is_deleted)? "true" : "false");
+
+    UNUSED(ifname);
+    UNUSED(if_index);
+    UNUSED(flags);
+    UNUSED(is_up);
+    UNUSED(is_deleted);
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+IfConfigSetNetlink::config_vif(const string& ifname,
+			       const string& vifname,
+			       uint16_t if_index,
+			       uint32_t flags,
+			       bool is_up,
+			       bool is_deleted,
+			       bool broadcast,
+			       bool loopback,
+			       bool point_to_point,
+			       bool multicast,
+			       string& error_msg)
+{
+    debug_msg("config_vif "
+	      "(ifname = %s vifname = %s if_index = %u flags = 0x%x "
+	      "is_up = %s is_deleted = %s broadcast = %s loopback = %s "
+	      "point_to_point = %s multicast = %s)\n",
+	      ifname.c_str(), vifname.c_str(), if_index, flags,
+	      (is_up)? "true" : "false",
+	      (is_deleted)? "true" : "false",
+	      (broadcast)? "true" : "false",
+	      (loopback)? "true" : "false",
+	      (point_to_point)? "true" : "false",
+	      (multicast)? "true" : "false");
+
+    UNUSED(ifname);
+    UNUSED(vifname);
+    UNUSED(if_index);
+    UNUSED(flags);
+    UNUSED(is_up);
+    UNUSED(is_deleted);
+    UNUSED(broadcast);
+    UNUSED(loopback);
+    UNUSED(point_to_point);
+    UNUSED(multicast);
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
 int
 IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
 					      uint16_t if_index,
 					      const struct ether_addr& ether_addr,
-					      string& reason)
+					      string& error_msg)
 {
     debug_msg("set_interface_mac "
 	      "(ifname = %s if_index = %u mac = %s)\n",
@@ -120,7 +275,7 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
     UNUSED(if_index);
     UNUSED(ether_addr);
 
-    reason = "method not supported";
+    error_msg = "method not supported";
 
     return (XORP_ERROR);
 }
@@ -129,7 +284,7 @@ int
 IfConfigSetNetlink::set_interface_mtu(const string& ifname,
 				      uint16_t if_index,
 				      uint32_t mtu,
-				      string& reason)
+				      string& error_msg)
 {
     debug_msg("set_interface_mtu "
 	      "(ifname = %s if_index = %u mtu = %u)\n",
@@ -139,48 +294,31 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
     UNUSED(if_index);
     UNUSED(mtu);
 
-    reason = "method not supported";
+    error_msg = "method not supported";
 
     return (XORP_ERROR);
 }
 
 int
-IfConfigSetNetlink::set_interface_flags(const string& ifname,
-					uint16_t if_index,
-					uint32_t flags,
-					string& reason)
-{
-    debug_msg("set_interface_flags "
-	      "(ifname = %s if_index = %u flags = 0x%x)\n",
-	      ifname.c_str(), if_index, flags);
-
-    UNUSED(ifname);
-    UNUSED(if_index);
-    UNUSED(flags);
-
-    reason = "method not supported";
-
-    return (XORP_ERROR);
-}
-
-int
-IfConfigSetNetlink::set_vif_address(const string& ifname,
+IfConfigSetNetlink::add_vif_address(const string& ifname,
+				    const string& vifname,
 				    uint16_t if_index,
 				    bool is_broadcast,
 				    bool is_p2p,
 				    const IPvX& addr,
 				    const IPvX& dst_or_bcast,
 				    uint32_t prefix_len,
-				    string& reason)
+				    string& error_msg)
 {
-    debug_msg("set_vif_address "
-	      "(ifname = %s if_index = %u is_broadcast = %s is_p2p = %s "
-	      "addr = %s dst/bcast = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index, (is_broadcast)? "true" : "false",
-	      (is_p2p)? "true" : "false", addr.str().c_str(),
-	      dst_or_bcast.str().c_str(), prefix_len);
+    debug_msg("add_vif_address "
+	      "(ifname = %s vifname = %s if_index = %u is_broadcast = %s "
+	      "is_p2p = %s addr = %s dst/bcast = %s prefix_len = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index,
+	      (is_broadcast)? "true" : "false", (is_p2p)? "true" : "false",
+	      addr.str().c_str(), dst_or_bcast.str().c_str(), prefix_len);
 
     UNUSED(ifname);
+    UNUSED(vifname);
     UNUSED(if_index);
     UNUSED(is_broadcast);
     UNUSED(is_p2p);
@@ -188,39 +326,247 @@ IfConfigSetNetlink::set_vif_address(const string& ifname,
     UNUSED(dst_or_bcast);
     UNUSED(prefix_len);
 
-    reason = "method not supported";
+    error_msg = "method not supported";
 
     return (XORP_ERROR);
 }
 
 int
 IfConfigSetNetlink::delete_vif_address(const string& ifname,
+				       const string& vifname,
 				       uint16_t if_index,
 				       const IPvX& addr,
 				       uint32_t prefix_len,
-				       string& reason)
+				       string& error_msg)
 {
     debug_msg("delete_vif_address "
-	      "(ifname = %s if_index = %u addr = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index, addr.str().c_str(), prefix_len);
+	      "(ifname = %s vifname = %s if_index = %u addr = %s "
+	      "prefix_len = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index, addr.str().c_str(),
+	      prefix_len);
 
     UNUSED(ifname);
+    UNUSED(vifname);
     UNUSED(if_index);
     UNUSED(addr);
     UNUSED(prefix_len);
 
-    reason = "method not supported";
+    error_msg = "method not supported";
 
     return (XORP_ERROR);
 }
 
 #else // HAVE_NETLINK_SOCKETS
 
+int
+IfConfigSetNetlink::config_begin(string& error_msg)
+{
+    debug_msg("config_begin\n");
+
+    // XXX: nothing to do
+
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
+int
+IfConfigSetNetlink::config_end(string& error_msg)
+{
+    debug_msg("config_end\n");
+
+    // XXX: nothing to do
+
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
+int
+IfConfigSetNetlink::add_interface(const string& ifname,
+				  uint16_t if_index,
+				  string& error_msg)
+{
+    debug_msg("add_interface "
+	      "(ifname = %s if_index = %u)\n",
+	      ifname.c_str(), if_index);
+
+    // XXX: nothing to do
+
+    UNUSED(ifname);
+    UNUSED(if_index);
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
+int
+IfConfigSetNetlink::add_vif(const string& ifname,
+			    const string& vifname,
+			    uint16_t if_index,
+			    string& error_msg)
+{
+    debug_msg("add_vif "
+	      "(ifname = %s vifname = %s if_index = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index);
+
+    // XXX: nothing to do
+
+    UNUSED(ifname);
+    UNUSED(vifname);
+    UNUSED(if_index);
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
+int
+IfConfigSetNetlink::config_interface(const string& ifname,
+				     uint16_t if_index,
+				     uint32_t flags,
+				     bool is_up,
+				     bool is_deleted,
+				     string& error_msg)
+{
+    debug_msg("config_interface "
+	      "(ifname = %s if_index = %u flags = 0x%x is_up = %s "
+	      "is_deleted = %s)\n",
+	      ifname.c_str(), if_index, flags, (is_up)? "true" : "false",
+	      (is_deleted)? "true" : "false");
+
+#ifndef HAVE_NETLINK_SOCKETS_SET_FLAGS_IS_BROKEN
+    static const size_t	buffer_size = sizeof(struct nlmsghdr)
+	+ sizeof(struct ifinfomsg) + 2*sizeof(struct rtattr) + 512;
+    uint8_t		buffer[buffer_size];
+    struct sockaddr_nl	snl;
+    struct nlmsghdr*	nlh;
+    struct ifinfomsg*	ifinfomsg;
+    NetlinkSocket4&	ns4 = *this;
+
+    UNUSED(ifname);
+    UNUSED(is_up);
+    UNUSED(is_deleted);
+
+    memset(buffer, 0, sizeof(buffer));
+
+    // Set the socket
+    memset(&snl, 0, sizeof(snl));
+    snl.nl_family = AF_NETLINK;
+    snl.nl_pid    = 0;		// nl_pid = 0 if destination is the kernel
+    snl.nl_groups = 0;
+
+    //
+    // Set the request
+    //
+    nlh = reinterpret_cast<struct nlmsghdr*>(buffer);
+    nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifinfomsg));
+    nlh->nlmsg_type = RTM_NEWLINK;
+    nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
+    nlh->nlmsg_seq = ns4.seqno();
+    nlh->nlmsg_pid = ns4.pid();
+    ifinfomsg = reinterpret_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
+    ifinfomsg->ifi_family = AF_UNSPEC;
+    ifinfomsg->ifi_type = IFLA_UNSPEC;
+    ifinfomsg->ifi_index = if_index;
+    ifinfomsg->ifi_flags = flags;
+    ifinfomsg->ifi_change = 0xffffffff;
+
+    if (NLMSG_ALIGN(nlh->nlmsg_len) > sizeof(buffer)) {
+	XLOG_FATAL("AF_NETLINK buffer size error: %d instead of %d",
+		   sizeof(buffer), NLMSG_ALIGN(nlh->nlmsg_len));
+    }
+    nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len);
+
+    if (ns4.sendto(buffer, nlh->nlmsg_len, 0,
+		   reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
+	!= (ssize_t)nlh->nlmsg_len) {
+	error_msg = c_format("error writing to netlink socket: %s",
+			     strerror(errno));
+	return (XORP_ERROR);
+    }
+    if (NlmUtils::check_netlink_request(_ns_reader, ns4, nlh->nlmsg_seq,
+					error_msg) < 0) {
+	return (XORP_ERROR);
+    }
+    return (XORP_OK);
+
+#else // HAVE_NETLINK_SOCKETS_SET_FLAGS_IS_BROKEN
+    //
+    // XXX: a work-around in case the kernel doesn't support setting
+    // the MTU on an interface by using netlink.
+    // In this case, the work-around is to use ioctl(). Sigh...
+    //
+    struct ifreq ifreq;
+    int s = socket(AF_INET, SOCK_DGRAM, 0);
+
+    UNUSED(is_up);
+    UNUSED(is_deleted);
+
+    if (s < 0) {
+	XLOG_FATAL("Could not initialize IPv4 ioctl() socket");
+    }
+
+    memset(&ifreq, 0, sizeof(ifreq));
+    strncpy(ifreq.ifr_name, ifname.c_str(), sizeof(ifreq.ifr_name) - 1);
+    ifreq.ifr_flags = flags;
+    if (ioctl(s, SIOCSIFFLAGS, &ifreq) < 0) {
+	error_msg = c_format("%s", strerror(errno));
+	close(s);
+	return (XORP_ERROR);
+    }
+    close(s);
+
+    return (XORP_OK);
+#endif // HAVE_NETLINK_SOCKETS_SET_FLAGS_IS_BROKEN
+}
+
+int
+IfConfigSetNetlink::config_vif(const string& ifname,
+			       const string& vifname,
+			       uint16_t if_index,
+			       uint32_t flags,
+			       bool is_up,
+			       bool is_deleted,
+			       bool broadcast,
+			       bool loopback,
+			       bool point_to_point,
+			       bool multicast,
+			       string& error_msg)
+{
+    debug_msg("config_vif "
+	      "(ifname = %s vifname = %s if_index = %u flags = 0x%x "
+	      "is_up = %s is_deleted = %s broadcast = %s loopback = %s "
+	      "point_to_point = %s multicast = %s)\n",
+	      ifname.c_str(), vifname.c_str(), if_index, flags,
+	      (is_up)? "true" : "false",
+	      (is_deleted)? "true" : "false",
+	      (broadcast)? "true" : "false",
+	      (loopback)? "true" : "false",
+	      (point_to_point)? "true" : "false",
+	      (multicast)? "true" : "false");
+
+    // XXX: nothing to do
+
+    UNUSED(ifname);
+    UNUSED(vifname);
+    UNUSED(if_index);
+    UNUSED(flags);
+    UNUSED(is_up);
+    UNUSED(is_deleted);
+    UNUSED(broadcast);
+    UNUSED(loopback);
+    UNUSED(point_to_point);
+    UNUSED(multicast);
+    UNUSED(error_msg);
+
+    return (XORP_OK);
+}
+
 int
 IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
 					      uint16_t if_index,
 					      const struct ether_addr& ether_addr,
-					      string& reason)
+					      string& error_msg)
 {
     debug_msg("set_interface_mac "
 	      "(ifname = %s if_index = %u mac = %s)\n",
@@ -278,12 +624,12 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
     if (ns4.sendto(buffer, nlh->nlmsg_len, 0,
 		   reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
-	reason = c_format("error writing to netlink socket: %s",
-			  strerror(errno));
+	error_msg = c_format("error writing to netlink socket: %s",
+			     strerror(errno));
 	return (XORP_ERROR);
     }
     if (NlmUtils::check_netlink_request(_ns_reader, ns4, nlh->nlmsg_seq,
-					reason) < 0) {
+					error_msg) < 0) {
 	return (XORP_ERROR);
     }
     return (XORP_OK);
@@ -311,7 +657,7 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
     ifreq.ifr_hwaddr.sa_len = ETH_ALEN;
 #endif
     if (ioctl(s, SIOCSIFHWADDR, &ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	close(s);
 	return (XORP_ERROR);
     }
@@ -328,7 +674,7 @@ int
 IfConfigSetNetlink::set_interface_mtu(const string& ifname,
 				      uint16_t if_index,
 				      uint32_t mtu,
-				      string& reason)
+				      string& error_msg)
 {
     debug_msg("set_interface_mtu "
 	      "(ifname = %s if_index = %u mtu = %u)\n",
@@ -387,12 +733,12 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
     if (ns4.sendto(buffer, nlh->nlmsg_len, 0,
 		   reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
-	reason = c_format("error writing to netlink socket: %s",
-			  strerror(errno));
+	error_msg = c_format("error writing to netlink socket: %s",
+			     strerror(errno));
 	return (XORP_ERROR);
     }
     if (NlmUtils::check_netlink_request(_ns_reader, ns4, nlh->nlmsg_seq,
-					reason) < 0) {
+					error_msg) < 0) {
 	return (XORP_ERROR);
     }
     return (XORP_OK);
@@ -416,7 +762,7 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
     strncpy(ifreq.ifr_name, ifname.c_str(), sizeof(ifreq.ifr_name) - 1);
     ifreq.ifr_mtu = mtu;
     if (ioctl(s, SIOCSIFMTU, &ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
+	error_msg = c_format("%s", strerror(errno));
 	close(s);
 	return (XORP_ERROR);
     }
@@ -427,105 +773,15 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
 }
 
 int
-IfConfigSetNetlink::set_interface_flags(const string& ifname,
-					uint16_t if_index,
-					uint32_t flags,
-					string& reason)
-{
-    debug_msg("set_interface_flags "
-	      "(ifname = %s if_index = %u flags = 0x%x)\n",
-	      ifname.c_str(), if_index, flags);
-
-#ifndef HAVE_NETLINK_SOCKETS_SET_FLAGS_IS_BROKEN
-    static const size_t	buffer_size = sizeof(struct nlmsghdr)
-	+ sizeof(struct ifinfomsg) + 2*sizeof(struct rtattr) + 512;
-    uint8_t		buffer[buffer_size];
-    struct sockaddr_nl	snl;
-    struct nlmsghdr*	nlh;
-    struct ifinfomsg*	ifinfomsg;
-    NetlinkSocket4&	ns4 = *this;
-
-    UNUSED(ifname);
-
-    memset(buffer, 0, sizeof(buffer));
-
-    // Set the socket
-    memset(&snl, 0, sizeof(snl));
-    snl.nl_family = AF_NETLINK;
-    snl.nl_pid    = 0;		// nl_pid = 0 if destination is the kernel
-    snl.nl_groups = 0;
-
-    //
-    // Set the request
-    //
-    nlh = reinterpret_cast<struct nlmsghdr*>(buffer);
-    nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifinfomsg));
-    nlh->nlmsg_type = RTM_NEWLINK;
-    nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
-    nlh->nlmsg_seq = ns4.seqno();
-    nlh->nlmsg_pid = ns4.pid();
-    ifinfomsg = reinterpret_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
-    ifinfomsg->ifi_family = AF_UNSPEC;
-    ifinfomsg->ifi_type = IFLA_UNSPEC;
-    ifinfomsg->ifi_index = if_index;
-    ifinfomsg->ifi_flags = flags;
-    ifinfomsg->ifi_change = 0xffffffff;
-
-    if (NLMSG_ALIGN(nlh->nlmsg_len) > sizeof(buffer)) {
-	XLOG_FATAL("AF_NETLINK buffer size error: %d instead of %d",
-		   sizeof(buffer), NLMSG_ALIGN(nlh->nlmsg_len));
-    }
-    nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len);
-
-    if (ns4.sendto(buffer, nlh->nlmsg_len, 0,
-		   reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
-	!= (ssize_t)nlh->nlmsg_len) {
-	reason = c_format("error writing to netlink socket: %s",
-			  strerror(errno));
-	return (XORP_ERROR);
-    }
-    if (NlmUtils::check_netlink_request(_ns_reader, ns4, nlh->nlmsg_seq,
-					reason) < 0) {
-	return (XORP_ERROR);
-    }
-    return (XORP_OK);
-
-#else // HAVE_NETLINK_SOCKETS_SET_FLAGS_IS_BROKEN
-    //
-    // XXX: a work-around in case the kernel doesn't support setting
-    // the MTU on an interface by using netlink.
-    // In this case, the work-around is to use ioctl(). Sigh...
-    //
-    struct ifreq ifreq;
-    int s = socket(AF_INET, SOCK_DGRAM, 0);
-
-    if (s < 0) {
-	XLOG_FATAL("Could not initialize IPv4 ioctl() socket");
-    }
-
-    memset(&ifreq, 0, sizeof(ifreq));
-    strncpy(ifreq.ifr_name, ifname.c_str(), sizeof(ifreq.ifr_name) - 1);
-    ifreq.ifr_flags = flags;
-    if (ioctl(s, SIOCSIFFLAGS, &ifreq) < 0) {
-	reason = c_format("%s", strerror(errno));
-	close(s);
-	return (XORP_ERROR);
-    }
-    close(s);
-
-    return (XORP_OK);
-#endif // HAVE_NETLINK_SOCKETS_SET_FLAGS_IS_BROKEN
-}
-
-int
-IfConfigSetNetlink::set_vif_address(const string& ifname,
+IfConfigSetNetlink::add_vif_address(const string& ifname,
+				    const string& vifname,
 				    uint16_t if_index,
 				    bool is_broadcast,
 				    bool is_p2p,
 				    const IPvX& addr,
 				    const IPvX& dst_or_bcast,
 				    uint32_t prefix_len,
-				    string& reason)
+				    string& error_msg)
 {
     static const size_t	buffer_size = sizeof(struct nlmsghdr)
 	+ sizeof(struct ifinfomsg) + 2*sizeof(struct rtattr) + 512;
@@ -538,14 +794,39 @@ IfConfigSetNetlink::set_vif_address(const string& ifname,
     uint8_t*		data;
     NetlinkSocket*	ns_ptr = NULL;
 
-    debug_msg("set_vif_address "
-	      "(ifname = %s if_index = %u is_broadcast = %s is_p2p = %s "
-	      "addr = %s dst/bcast = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index, (is_broadcast)? "true" : "false",
+    debug_msg("add_vif_address "
+	      "(ifname = %s vifname = %s if_index = %u is_broadcast = %s "
+	      "is_p2p = %s addr = %s dst/bcast = %s prefix_len = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index,
+	      (is_broadcast)? "true" : "false",
 	      (is_p2p)? "true" : "false", addr.str().c_str(),
 	      dst_or_bcast.str().c_str(), prefix_len);
 
     UNUSED(ifname);
+    UNUSED(vifname);
+
+    // Check that the family is supported
+    switch (addr.af()) {
+    case AF_INET:
+	if (! ifc().have_ipv4()) {
+	    error_msg = "IPv4 is not supported";
+	    return (XORP_ERROR);
+	}
+	break;
+
+#ifdef HAVE_IPV6
+    case AF_INET6:
+	if (! ifc().have_ipv6()) {
+	    error_msg = "IPv6 is not supported";
+	    return (XORP_ERROR);
+	}
+	break;
+#endif // HAVE_IPV6
+
+    default:
+	XLOG_UNREACHABLE();
+	break;
+    }
 
     memset(buffer, 0, sizeof(buffer));
 
@@ -628,12 +909,12 @@ IfConfigSetNetlink::set_vif_address(const string& ifname,
     if (ns_ptr->sendto(buffer, nlh->nlmsg_len, 0,
 		       reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
-	reason = c_format("error writing to netlink socket: %s",
-			  strerror(errno));
+	error_msg = c_format("error writing to netlink socket: %s",
+			     strerror(errno));
 	return (XORP_ERROR);
     }
     if (NlmUtils::check_netlink_request(_ns_reader, *ns_ptr, nlh->nlmsg_seq,
-					reason) < 0) {
+					error_msg) < 0) {
 	return (XORP_ERROR);
     }
     return (XORP_OK);
@@ -641,10 +922,11 @@ IfConfigSetNetlink::set_vif_address(const string& ifname,
 
 int
 IfConfigSetNetlink::delete_vif_address(const string& ifname,
+				       const string& vifname,
 				       uint16_t if_index,
 				       const IPvX& addr,
 				       uint32_t prefix_len,
-				       string& reason)
+				       string& error_msg)
 {
     static const size_t	buffer_size = sizeof(struct nlmsghdr)
 	+ sizeof(struct ifinfomsg) + 2*sizeof(struct rtattr) + 512;
@@ -658,10 +940,36 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
     NetlinkSocket*	ns_ptr = NULL;
 
     debug_msg("delete_vif_address "
-	      "(ifname = %s if_index = %u addr = %s prefix_len = %u)\n",
-	      ifname.c_str(), if_index, addr.str().c_str(), prefix_len);
+	      "(ifname = %s vifname = %s if_index = %u addr = %s "
+	      "prefix_len = %u)\n",
+	      ifname.c_str(), vifname.c_str(), if_index, addr.str().c_str(),
+	      prefix_len);
 
     UNUSED(ifname);
+    UNUSED(vifname);
+
+    // Check that the family is supported
+    switch (addr.af()) {
+    case AF_INET:
+	if (! ifc().have_ipv4()) {
+	    error_msg = "IPv4 is not supported";
+	    return (XORP_ERROR);
+	}
+	break;
+
+#ifdef HAVE_IPV6
+    case AF_INET6:
+	if (! ifc().have_ipv6()) {
+	    error_msg = "IPv6 is not supported";
+	    return (XORP_ERROR);
+	}
+	break;
+#endif // HAVE_IPV6
+
+    default:
+	XLOG_UNREACHABLE();
+	break;
+    }
 
     memset(buffer, 0, sizeof(buffer));
 
@@ -724,12 +1032,12 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
     if (ns_ptr->sendto(buffer, nlh->nlmsg_len, 0,
 		       reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
-	reason = c_format("error writing to netlink socket: %s",
-			  strerror(errno));
+	error_msg = c_format("error writing to netlink socket: %s",
+			     strerror(errno));
 	return (XORP_ERROR);
     }
     if (NlmUtils::check_netlink_request(_ns_reader, *ns_ptr, nlh->nlmsg_seq,
-					reason) < 0) {
+					error_msg) < 0) {
 	return (XORP_ERROR);
     }
     return (XORP_OK);
