@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +12,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/libfeaclient_bridge.cc,v 1.10 2004/06/10 22:40:54 hodson Exp $"
-
-/*
-#define DEBUG_LOGGING
-*/
+#ident "$XORP: xorp/fea/libfeaclient_bridge.cc,v 1.14 2005/03/25 02:53:09 pavlin Exp $"
 
 #include "fea_module.h"
+
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
@@ -152,6 +150,7 @@ LibFeaClientBridge::interface_update(const string& ifname,
     // equivalent.
     //
     _rm->push(new IfMgrIfSetEnabled(ifname, ii->second.enabled()));
+    _rm->push(new IfMgrIfSetDiscard(ifname, ii->second.discard()));
     _rm->push(new IfMgrIfSetMtu(ifname, ii->second.mtu()));
     _rm->push(new IfMgrIfSetMac(ifname, ii->second.mac()));
     _rm->push(new IfMgrIfSetPifIndex(ifname, ii->second.pif_index()));
