@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,19 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_mrouter.cc,v 1.20 2004/06/10 22:40:55 hodson Exp $"
-
+#ident "$XORP: xorp/fea/mfea_mrouter.cc,v 1.29 2005/03/25 02:53:10 pavlin Exp $"
 
 //
 // Multicast routing kernel-access specific implementation.
 //
 
-
 #include "mfea_module.h"
+
 #include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+#include "libxorp/utils.hh"
 
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
@@ -42,15 +45,9 @@
 #include <netinet6/in6_var.h>
 #endif
 
-#include "mrt/include/ip_mroute.h"
-
-#include "libxorp/xlog.h"
-#include "libxorp/debug.h"
-#include "libxorp/ipvx.hh"
-#include "libxorp/utils.hh"
-
 #include "libcomm/comm_api.h"
 
+#include "mrt/include/ip_mroute.h"
 #include "mrt/max_vifs.h"
 #include "mrt/multicast_defs.h"
 
@@ -176,12 +173,12 @@ MfeaMrouter::~MfeaMrouter()
 int
 MfeaMrouter::start()
 {
-    if (is_up() || is_pending_up())
-	return (XORP_OK);
-
     // XXX: MfeaMrouter is automatically enabled by default
     ProtoUnit::enable();
     
+    if (is_up() || is_pending_up())
+	return (XORP_OK);
+
     if (ProtoUnit::start() < 0)
 	return (XORP_ERROR);
     
@@ -429,9 +426,9 @@ MfeaMrouter::adopt_mrouter_socket()
 	ICMP6_FILTER_SETBLOCKALL(&filter);
 	if (setsockopt(_mrouter_socket, IPPROTO_ICMPV6, ICMP6_FILTER,
 		       (void *)&filter, sizeof(filter)) < 0) {
+	    XLOG_ERROR("setsockopt(ICMP6_FILTER) failed: %s", strerror(errno));
 	    close(_mrouter_socket);
 	    _mrouter_socket = -1;
-	    XLOG_ERROR("setsockopt(ICMP6_FILTER) failed: %s", strerror(errno));
 	    return (XORP_ERROR);
 	}
 #endif // HAVE_IPV6_MULTICAST_ROUTING
@@ -443,7 +440,7 @@ MfeaMrouter::adopt_mrouter_socket()
 	return (XORP_ERROR);
     }
     
-    // Assign a function to read from this socket
+    // Assign a method to read from this socket
     if (mfea_node().eventloop().add_selector(_mrouter_socket, SEL_RD,
 				callback(this,
 					 &MfeaMrouter::mrouter_socket_read))
@@ -486,6 +483,9 @@ MfeaMrouter::close_mrouter_socket()
 	    break;
 	// If there is already IGMP or ICMPV6 socket, then don't close it.
 	if (_mrouter_socket == proto_comm->proto_socket()) {
+	    // Transfer the selector to the protocol socket
+	    mfea_node().eventloop().remove_selector(_mrouter_socket);
+	    proto_comm->add_proto_socket_selector();
 	    _mrouter_socket = -1;
 	    return (XORP_OK);
 	}
@@ -497,8 +497,8 @@ MfeaMrouter::close_mrouter_socket()
     
     // Close the socket and reset it to -1
     if (close(_mrouter_socket) < 0) {
-	_mrouter_socket = -1;
 	XLOG_ERROR("Cannot close mrouter socket: %s", strerror(errno));
+	_mrouter_socket = -1;
 	return (XORP_ERROR);
     }
     
@@ -557,7 +557,7 @@ MfeaMrouter::start_mrt()
     //
     // Configure advanced multicast API
     //
-#if defined(MRT_API_CONFIG) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT_API_CONFIG) && defined(ENABLE_ADVANCED_MULTICAST_API)
     if (family() == AF_INET) {
 	uint32_t mrt_api = 0;
 	
@@ -616,9 +616,9 @@ MfeaMrouter::start_mrt()
 #endif
 	
     }
-#endif // MRT_API_CONFIG && ENABLE_ADVANCED_MCAST_API
+#endif // MRT_API_CONFIG && ENABLE_ADVANCED_MULTICAST_API
     
-#if defined(MRT6_API_CONFIG) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT6_API_CONFIG) && defined(ENABLE_ADVANCED_MULTICAST_API)
 #ifdef HAVE_IPV6
     if (family == AF_INET6) {
 #ifndef HAVE_IPV6_MULTICAST_ROUTING
@@ -686,7 +686,7 @@ MfeaMrouter::start_mrt()
 #endif // HAVE_IPV6_MULTICAST_ROUTING	
     }
 #endif // HAVE_IPV6
-#endif // MRT6_API_CONFIG && ENABLE_ADVANCED_MCAST_API
+#endif // MRT6_API_CONFIG && ENABLE_ADVANCED_MULTICAST_API
     
     return (XORP_OK);
 }
@@ -1061,7 +1061,7 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
     switch (family()) {
     case AF_INET:
     {
-#if defined(HAVE_MFCCTL2) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(HAVE_MFCCTL2) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct mfcctl2 mc;
 #else
 	struct mfcctl mc;
@@ -1073,11 +1073,11 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	mc.mfcc_parent = iif_vif_index;
 	for (uint16_t i = 0; i < mfea_node().maxvifs(); i++) {
 	    mc.mfcc_ttls[i] = oifs_ttl[i];
-#if defined(HAVE_MFCC_FLAGS) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(HAVE_MFCC_FLAGS) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	    mc.mfcc_flags[i] = oifs_flags[i];
 #endif
 	}
-#if defined(HAVE_MFCC_RP) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(HAVE_MFCC_RP) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	if (_mrt_api_mrt_mfc_rp)
 	    rp_addr.copy_out(mc.mfcc_rp);
 #endif
@@ -1100,7 +1100,7 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	return (XORP_ERROR);
 #else
 	
-#if defined(HAVE_MF6CCTL2) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(HAVE_MF6CCTL2) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct mf6cctl2 mc;
 #else
 	struct mf6cctl mc;
@@ -1114,11 +1114,11 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	for (uint16_t i = 0; i < mfea_node().maxvifs(); i++) {
 	    if (oifs_ttl[i] > 0)
 		IF_SET(i, &mc.mf6cc_ifset);
-#if defined(HAVE_MF6CC_FLAGS) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(HAVE_MF6CC_FLAGS) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	    mc.mf6cc_flags[i] = oifs_flags[i];
 #endif
 	}
-#if defined(HAVE_MF6CC_RP) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(HAVE_MF6CC_RP) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	if (_mrt_api_mrt_mfc_rp)
 	    rp_addr.copy_out(mc.mf6cc_rp);
 #endif
@@ -1281,7 +1281,7 @@ MfeaMrouter::add_bw_upcall(const IPvX& source, const IPvX& group,
     switch (family()) {
     case AF_INET:
     {
-#if defined(MRT_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct bw_upcall bw_upcall;
 	
 	//
@@ -1318,7 +1318,7 @@ MfeaMrouter::add_bw_upcall(const IPvX& source, const IPvX& group,
 	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
-#endif // MRT_ADD_BW_UPCALL && ENABLE_ADVANCED_MCAST_API
+#endif // MRT_ADD_BW_UPCALL && ENABLE_ADVANCED_MULTICAST_API
     }
     break;
     
@@ -1332,7 +1332,7 @@ MfeaMrouter::add_bw_upcall(const IPvX& source, const IPvX& group,
 	return (XORP_ERROR);
 #else
 	
-#if defined(MRT6_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT6_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct bw6_upcall bw_upcall;
 	
 	//
@@ -1369,7 +1369,7 @@ MfeaMrouter::add_bw_upcall(const IPvX& source, const IPvX& group,
 	    XLOG_ERROR(("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
-#endif // MRT6_ADD_BW_UPCALL && ENABLE_ADVANCED_MCAST_API
+#endif // MRT6_ADD_BW_UPCALL && ENABLE_ADVANCED_MULTICAST_API
 #endif // HAVE_IPV6_MULTICAST_ROUTING
     }
     break;
@@ -1464,7 +1464,7 @@ MfeaMrouter::delete_bw_upcall(const IPvX& source, const IPvX& group,
     switch (family()) {
     case AF_INET:
     {
-#if defined(MRT_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct bw_upcall bw_upcall;
 	
 	//
@@ -1501,7 +1501,7 @@ MfeaMrouter::delete_bw_upcall(const IPvX& source, const IPvX& group,
 	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
-#endif // MRT_ADD_BW_UPCALL && ENABLE_ADVANCED_MCAST_API
+#endif // MRT_ADD_BW_UPCALL && ENABLE_ADVANCED_MULTICAST_API
     }
     break;
     
@@ -1515,7 +1515,7 @@ MfeaMrouter::delete_bw_upcall(const IPvX& source, const IPvX& group,
 	return (XORP_ERROR);
 #else
 	
-#if defined(MRT6_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT6_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct bw6_upcall bw_upcall;
 	
 	//
@@ -1552,7 +1552,7 @@ MfeaMrouter::delete_bw_upcall(const IPvX& source, const IPvX& group,
 	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
-#endif // MRT6_ADD_BW_UPCALL && ENABLE_ADVANCED_MCAST_API
+#endif // MRT6_ADD_BW_UPCALL && ENABLE_ADVANCED_MULTICAST_API
 #endif // HAVE_IPV6_MULTICAST_ROUTING
     }
     break;
@@ -1607,7 +1607,7 @@ MfeaMrouter::delete_all_bw_upcall(const IPvX& source, const IPvX& group,
     switch (family()) {
     case AF_INET:
     {
-#if defined(MRT_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct bw_upcall bw_upcall;
 	
 	//
@@ -1626,7 +1626,7 @@ MfeaMrouter::delete_all_bw_upcall(const IPvX& source, const IPvX& group,
 	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
-#endif // MRT_ADD_BW_UPCALL && ENABLE_ADVANCED_MCAST_API
+#endif // MRT_ADD_BW_UPCALL && ENABLE_ADVANCED_MULTICAST_API
     }
     break;
     
@@ -1640,7 +1640,7 @@ MfeaMrouter::delete_all_bw_upcall(const IPvX& source, const IPvX& group,
 	return (XORP_ERROR);
 #else
 	
-#if defined(MRT6_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT6_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct bw6_upcall bw_upcall;
 	
 	//
@@ -1659,7 +1659,7 @@ MfeaMrouter::delete_all_bw_upcall(const IPvX& source, const IPvX& group,
 	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
-#endif // MRT6_ADD_BW_UPCALL && ENABLE_ADVANCED_MCAST_API
+#endif // MRT6_ADD_BW_UPCALL && ENABLE_ADVANCED_MULTICAST_API
 #endif // HAVE_IPV6_MULTICAST_ROUTING
     }
     break;
@@ -1889,7 +1889,8 @@ MfeaMrouter::mrouter_socket_read(int fd, SelectorMask mask)
     if (nbytes < 0) {
 	if (errno == EINTR)
 	    return;		// OK: restart receiving
-	XLOG_ERROR("recvmsg() failed: %s", strerror(errno));
+	XLOG_ERROR("recvmsg() on socket %d failed: %s",
+		   _mrouter_socket, strerror(errno));
 	return;			// Error
     }
     
@@ -1903,7 +1904,8 @@ MfeaMrouter::mrouter_socket_read(int fd, SelectorMask mask)
 	if (nbytes < (ssize_t)sizeof(*igmpmsg)) {
 	    XLOG_WARNING("mrouter_socket_read() failed: "
 			 "kernel signal packet size %d is smaller than minimum size %u",
-			 nbytes, (uint32_t)sizeof(*igmpmsg));
+			 XORP_INT_CAST(nbytes),
+			 XORP_UINT_CAST(sizeof(*igmpmsg)));
 	    return;		// Error
 	}
 	if (igmpmsg->im_mbz == 0) {
@@ -1933,8 +1935,9 @@ MfeaMrouter::mrouter_socket_read(int fd, SelectorMask mask)
 	    && (nbytes < (ssize_t)sizeof(struct mld_hdr))) {
 	    XLOG_WARNING("mrouter_socket_read() failed: "
 			 "kernel signal packet size %d is smaller than minimum size %u",
-			 nbytes,
-			 min((uint32_t)sizeof(*mrt6msg), (uint32_t)sizeof(struct mld_hdr)));
+			 XORP_INT_CAST(nbytes),
+			 XORP_UINT_CAST(min(sizeof(*mrt6msg),
+					    sizeof(struct mld_hdr))));
 	    return;		// Error
 	}
 	if ((mrt6msg->im6_mbz == 0) || (_rcvmh.msg_controllen == 0)) {
@@ -2031,7 +2034,7 @@ MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
 	    }
 	}
 	break;
-#if defined(IGMPMSG_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(IGMPMSG_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	case IGMPMSG_BW_UPCALL:
 	    break;
 #endif
@@ -2053,7 +2056,7 @@ MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
 		return (XORP_ERROR);
 	    }
 	    break;
-#if defined(IGMPMSG_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(IGMPMSG_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	case IGMPMSG_BW_UPCALL:
 	    break;
 #endif
@@ -2121,7 +2124,7 @@ MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
 	    }
 	}
 	break;
-#if defined(MRT6MSG_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT6MSG_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	case MRT6MSG_BW_UPCALL:
 	    break;
 #endif
@@ -2143,7 +2146,7 @@ MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
 		return (XORP_ERROR);
 	    }
 	    break;
-#if defined(MRT6MSG_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT6MSG_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	case MRT6MSG_BW_UPCALL:
 	    break;
 #endif
