@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_node.cc,v 1.37 2004/06/17 23:22:29 pavlin Exp $"
-
+#ident "$XORP: xorp/fea/mfea_node.cc,v 1.53 2005/03/25 02:53:10 pavlin Exp $"
 
 //
 // MFEA (Multicast Forwarding Engine Abstraction) implementation.
 //
 
-
 #include "mfea_module.h"
 
 #include "libxorp/xorp.h"
@@ -38,8 +36,6 @@
 #include "mfea_kernel_messages.hh"
 #include "mfea_vif.hh"
 
-// TODO: XXX: PAVPAVPAV: TEMPORARY HERE!!
-#include "fticonfig.hh"
 
 //
 // Exported variables
@@ -69,22 +65,15 @@
  * for IPv4 and IPv6 respectively).
  * @module_id: The module ID (must be %XORP_MODULE_MFEA).
  * @eventloop: The event loop.
- * @ftic the @ref FtiConfig entry to use to obtain the routing
- * table (NOTE (TODO: XXX: PAVPAVPAV: this parameter is only temporary here,
- * and will be removed in the near future).
  * 
  * MFEA node constructor.
  **/
 MfeaNode::MfeaNode(int family, xorp_module_id module_id,
-		   EventLoop& eventloop, FtiConfig& ftic)
+		   EventLoop& eventloop)
     : ProtoNode<MfeaVif>(family, module_id, eventloop),
       _mfea_mrouter(*this),
-      _mrib_table(family),
-      _mrib_table_default_metric_preference(MRIB_TABLE_DEFAULT_METRIC_PREFERENCE),
-      _mrib_table_default_metric(MRIB_TABLE_DEFAULT_METRIC),
       _mfea_dft(*this),
-      _is_log_trace(true),		// XXX: default to print trace logs
-      _ftic(ftic)
+      _is_log_trace(false)
 {
     XLOG_ASSERT(module_id == XORP_MODULE_MFEA);
     
@@ -150,9 +139,20 @@ MfeaNode::~MfeaNode()
 int
 MfeaNode::start()
 {
-    if (is_up() || is_pending_up())
+    if (! is_enabled())
 	return (XORP_OK);
 
+    //
+    // Test the service status
+    //
+    if ((ServiceBase::status() == SERVICE_STARTING)
+	|| (ServiceBase::status() == SERVICE_RUNNING)) {
+	return (XORP_OK);
+    }
+    if (ServiceBase::status() != SERVICE_READY) {
+	return (XORP_ERROR);
+    }
+
     if (ProtoNode<MfeaVif>::pending_start() < 0)
 	return (XORP_ERROR);
 
@@ -162,12 +162,9 @@ MfeaNode::start()
     ProtoNode<MfeaVif>::set_node_status(PROC_STARTUP);
 
     //
-    // Startup the interface manager
+    // Register with the FEA
     //
-    if (ifmgr_startup() != true) {
-	ServiceBase::set_status(FAILED);
-	return (XORP_ERROR);
-    }
+    fea_register_startup();
 
     // Start the MfeaMrouter
     _mfea_mrouter.start();
@@ -206,11 +203,7 @@ MfeaNode::final_start()
     // Start the mfea_vifs
     start_all_vifs();
 
-    // Start the timer to read immediately the MRIB table
-    mrib_table().clear();
-    _mrib_table_read_timer = eventloop().new_oneoff_after(
-	TimeVal(0,0),
-	callback(this, &MfeaNode::mrib_table_read_timer_timeout));
+    XLOG_INFO("MFEA started");
 
     return (XORP_OK);
 }
@@ -220,13 +213,8 @@ MfeaNode::final_start()
  * @: 
  * 
  * Gracefully stop the MFEA.
- * XXX: This function, unlike start(), will stop the MFEA
- * operation on all interfaces.
  * XXX: After the cleanup is completed,
  * MfeaNode::final_stop() is called to complete the job.
- * XXX: If this method is called one-after-another, the second one
- * will force calling immediately MfeaNode::final_stop() to quickly
- * finish the job. TODO: is this a desired semantic?
  * XXX: This function, unlike start(), will stop the MFEA
  * operation on all interfaces.
  * 
@@ -235,10 +223,23 @@ MfeaNode::final_start()
 int
 MfeaNode::stop()
 {
-    if (is_down())
+    //
+    // Test the service status
+    //
+    if ((ServiceBase::status() == SERVICE_SHUTDOWN)
+	|| (ServiceBase::status() == SERVICE_SHUTTING_DOWN)
+	|| (ServiceBase::status() == SERVICE_FAILED)) {
 	return (XORP_OK);
+    }
+    if ((ServiceBase::status() != SERVICE_RUNNING)
+	&& (ServiceBase::status() != SERVICE_STARTING)
+	&& (ServiceBase::status() != SERVICE_PAUSING)
+	&& (ServiceBase::status() != SERVICE_PAUSED)
+	&& (ServiceBase::status() != SERVICE_RESUMING)) {
+	return (XORP_ERROR);
+    }
 
-    if (! (is_up() || is_pending_up() || is_pending_down()))
+    if (ProtoNode<MfeaVif>::pending_stop() < 0)
 	return (XORP_ERROR);
 
     //
@@ -257,26 +258,16 @@ MfeaNode::stop()
     // Stop the MfeaMrouter
     _mfea_mrouter.stop();
 
-    // Clear the MRIB table
-    _mrib_table_read_timer.unschedule();
-    mrib_table().clear();
-    
-    if (ProtoNode<MfeaVif>::pending_stop() < 0)
-	return (XORP_ERROR);
-
-    //
-    // Shutdown the interface manager
-    //
-    if (ifmgr_shutdown() != true) {
-	ServiceBase::set_status(FAILED);
-	return (XORP_ERROR);
-    }
-
     //
     // Set the node status
     //
     ProtoNode<MfeaVif>::set_node_status(PROC_SHUTDOWN);
 
+    //
+    // Update the node status
+    //
+    update_status();
+
     return (XORP_OK);
 }
 
@@ -297,48 +288,37 @@ MfeaNode::final_stop()
     if (ProtoNode<MfeaVif>::stop() < 0)
 	return (XORP_ERROR);
 
+    XLOG_INFO("MFEA stopped");
+
     return (XORP_OK);
 }
 
 /**
- * MfeaNode::has_pending_down_units:
- * @reason_msg: return-by-reference string that contains human-readable
- * information about the status.
+ * Enable the node operation.
  * 
- * Test if there is an unit that is in PENDING_DOWN state.
- * 
- * Return value: True if there is an unit that is in PENDING_DOWN state,
- * otherwise false.
- **/
-bool
-MfeaNode::has_pending_down_units(string& reason_msg)
+ * If an unit is not enabled, it cannot be start, or pending-start.
+ */
+void
+MfeaNode::enable()
 {
-    vector<MfeaVif *>::iterator iter;
-    
-    //
-    // Test the interfaces
-    //
-    for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
-	MfeaVif *mfea_vif = (*iter);
-	if (mfea_vif == NULL)
-	    continue;
-	// TODO: XXX: PAVPAVPAV: vif pending-down state
-	// is not used/implemented yet
-	if (mfea_vif->is_pending_down()) {
-	    reason_msg = c_format("Vif %s is in state %s",
-				  mfea_vif->name().c_str(),
-				  mfea_vif->state_str().c_str());
-	    return (true);
-	}
-    }
-    
-    //
-    // TODO: XXX: PAVPAVPAV: test other units that may be waiting
-    // in PENDING_DOWN state.
-    //
-    
-    reason_msg = "No pending-down units";
-    return (false);
+    ProtoUnit::enable();
+
+    XLOG_INFO("MFEA enabled");
+}
+
+/**
+ * Disable the node operation.
+ * 
+ * If an unit is disabled, it cannot be start or pending-start.
+ * If the unit was runnning, it will be stop first.
+ */
+void
+MfeaNode::disable()
+{
+    stop();
+    ProtoUnit::disable();
+
+    XLOG_INFO("MFEA disabled");
 }
 
 void
@@ -348,7 +328,8 @@ MfeaNode::status_change(ServiceBase*  service,
 {
     if (service == this) {
 	// My own status has changed
-	if ((old_status == STARTING) && (new_status == RUNNING)) {
+	if ((old_status == SERVICE_STARTING)
+	    && (new_status == SERVICE_RUNNING)) {
 	    // The startup process has completed
 	    if (final_start() < 0) {
 		XLOG_ERROR("Cannot complete the startup process; "
@@ -360,7 +341,8 @@ MfeaNode::status_change(ServiceBase*  service,
 	    return;
 	}
 
-	if ((old_status == SHUTTING_DOWN) && (new_status == SHUTDOWN)) {
+	if ((old_status == SERVICE_SHUTTING_DOWN)
+	    && (new_status == SERVICE_SHUTDOWN)) {
 	    // The shutdown process has completed
 	    final_stop();
 	    // Set the node status
@@ -375,7 +357,8 @@ MfeaNode::status_change(ServiceBase*  service,
     }
 
     if (service == ifmgr_mirror_service_base()) {
-	if ((old_status == SHUTTING_DOWN) && (new_status == SHUTDOWN)) {
+	if ((old_status == SERVICE_SHUTTING_DOWN)
+	    && (new_status == SERVICE_SHUTDOWN)) {
 	    MfeaNode::decr_shutdown_requests_n();
 	}
     }
@@ -694,7 +677,7 @@ MfeaNode::add_vif(const Vif& vif, string& error_msg)
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("New vif: %s", mfea_vif->str().c_str());
+    XLOG_INFO("Interface added: %s", mfea_vif->str().c_str());
     
     return (XORP_OK);
 }
@@ -817,7 +800,7 @@ MfeaNode::delete_vif(const string& vif_name, string& error_msg)
     
     delete mfea_vif;
     
-    XLOG_INFO("Deleted vif: %s", vif_name.c_str());
+    XLOG_INFO("Interface deleted: %s", vif_name.c_str());
     
     return (XORP_OK);
 }
@@ -844,8 +827,6 @@ MfeaNode::enable_vif(const string& vif_name, string& error_msg)
     
     mfea_vif->enable();
     
-    XLOG_INFO("Enabled vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -871,8 +852,6 @@ MfeaNode::disable_vif(const string& vif_name, string& error_msg)
     
     mfea_vif->disable();
     
-    XLOG_INFO("Disabled vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -903,8 +882,6 @@ MfeaNode::start_vif(const string& vif_name, string& error_msg)
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("Started vif: %s", vif_name.c_str());
-
     // XXX: add PIM Register vif (if needed)
     add_pim_register_vif();
 
@@ -938,8 +915,6 @@ MfeaNode::stop_vif(const string& vif_name, string& error_msg)
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("Stopped vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -949,31 +924,24 @@ MfeaNode::stop_vif(const string& vif_name, string& error_msg)
  * 
  * Start MFEA on all enabled interfaces.
  * 
- * Return value: The number of virtual interfaces MFEA was started on,
- * or %XORP_ERROR if error occured.
+ * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
 MfeaNode::start_all_vifs()
 {
-    int n = 0;
     vector<MfeaVif *>::iterator iter;
     string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	MfeaVif *mfea_vif = (*iter);
 	if (mfea_vif == NULL)
 	    continue;
-	if (! mfea_vif->is_enabled())
-	    continue;
-	if (mfea_vif->start(error_msg) != XORP_OK) {
-	    XLOG_ERROR("Cannot start vif %s: %s",
-		       mfea_vif->name().c_str(), error_msg.c_str());
-	} else {
-	    n++;
-	}
+	if (start_vif(mfea_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (n);
+    return (ret_value);
 }
 
 /**
@@ -982,29 +950,24 @@ MfeaNode::start_all_vifs()
  * 
  * Stop MFEA on all interfaces it was running on.
  * 
- * Return value: The number of virtual interfaces MFEA was stopped on,
- * or %XORP_ERROR if error occured.
+ * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
 MfeaNode::stop_all_vifs()
 {
-    int n = 0;
     vector<MfeaVif *>::iterator iter;
     string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	MfeaVif *mfea_vif = (*iter);
 	if (mfea_vif == NULL)
 	    continue;
-	if (mfea_vif->stop(error_msg) != XORP_OK) {
-	    XLOG_ERROR("Cannot stop vif %s: %s",
-		       mfea_vif->name().c_str(), error_msg.c_str());
-	} else {
-	    n++;
-	}
+	if (stop_vif(mfea_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (n);
+    return (ret_value);
 }
 
 /**
@@ -1019,15 +982,18 @@ int
 MfeaNode::enable_all_vifs()
 {
     vector<MfeaVif *>::iterator iter;
+    string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	MfeaVif *mfea_vif = (*iter);
 	if (mfea_vif == NULL)
 	    continue;
-	mfea_vif->enable();
+	if (enable_vif(mfea_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (XORP_OK);
+    return (ret_value);
 }
 
 /**
@@ -1042,17 +1008,18 @@ int
 MfeaNode::disable_all_vifs()
 {
     vector<MfeaVif *>::iterator iter;
-
-    stop_all_vifs();
+    string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	MfeaVif *mfea_vif = (*iter);
 	if (mfea_vif == NULL)
 	    continue;
-	mfea_vif->disable();
+	if (disable_vif(mfea_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (XORP_OK);
+    return (ret_value);
 }
 
 /**
@@ -1095,6 +1062,36 @@ MfeaNode::delete_all_vifs()
     }
 }
 
+/**
+ * A method called when a vif has completed its shutdown.
+ * 
+ * @param vif_name the name of the vif that has completed its shutdown.
+ */
+void
+MfeaNode::vif_shutdown_completed(const string& vif_name)
+{
+    vector<MfeaVif *>::iterator iter;
+
+    //
+    // If all vifs have completed the shutdown, then de-register with
+    // the MFEA.
+    //
+    for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
+	MfeaVif *mfea_vif = *iter;
+	if (mfea_vif == NULL)
+	    continue;
+	if (! mfea_vif->is_down())
+	    return;
+    }
+
+    //
+    // De-register with the FEA
+    //
+    fea_register_shutdown();
+
+    UNUSED(vif_name);
+}
+
 /**
  * MfeaNode::add_protocol:
  * @module_instance_name: The module instance name of the protocol to add.
@@ -1183,10 +1180,15 @@ int
 MfeaNode::delete_protocol(const string& module_instance_name,
 			  xorp_module_id module_id)
 {
-    // Delete MRIB registration
-    if (_mrib_messages_register.is_registered(module_instance_name,
-					      module_id)) {
-	delete_allow_mrib_messages(module_instance_name, module_id);
+    vector<MfeaVif *>::iterator iter;
+
+    // Explicitly stop the protocol on all vifs
+    for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
+	MfeaVif *mfea_vif = (*iter);
+	if (mfea_vif == NULL)
+	    continue;
+	// TODO: XXX: PAVPAVPAV: shall we ignore the disabled vifs??
+	mfea_vif->stop_protocol(module_instance_name, module_id);
     }
     
     // Delete kernel signal registration
@@ -1396,147 +1398,6 @@ MfeaNode::delete_allow_kernel_signal_messages(const string& module_instance_name
     return (XORP_OK);
 }
 
-/**
- * MfeaNode::add_allow_mrib_messages:
- * @module_instance_name: The module instance name of the protocol to add to
- * receive MRIB messages.
- * @module_id: The #xorp_module_id of the protocol to add to receive
- * MRIB messages.
- * 
- * Add a protocol to the set of protocols that are interested in
- * receiving MRIB messages.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
-int
-MfeaNode::add_allow_mrib_messages(const string& module_instance_name,
-				  xorp_module_id module_id)
-{
-    // Add the state
-    if (_mrib_messages_register.add_protocol(module_instance_name, module_id)
-	< 0) {
-	XLOG_ERROR("Cannot add protocol instance %s with module_id = %d "
-		   "to receive MRIB messages",
-		   module_instance_name.c_str(), module_id);
-	return (XORP_ERROR);	// Already added
-    }
-    
-    return (XORP_OK);
-}
-
-/**
- * MfeaNode::delete_allow_mrib_messages:
- * @module_instance_name: The module instance name of the protocol to delete
- * from receiving MRIB messages.
- * @module_id: The #xorp_module_id of the protocol to delete from receiving
- * MRIB messages.
- * 
- * Delete a protocol from the set of protocols that are interested in
- * receiving MRIB messages.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
-int
-MfeaNode::delete_allow_mrib_messages(const string& module_instance_name,
-				     xorp_module_id module_id)
-{
-    // Delete the state
-    if (_mrib_messages_register.delete_protocol(module_instance_name,
-						module_id)
-	< 0) {
-	XLOG_ERROR("Cannot delete protocol instance %s with module_id = %d "
-		   "from receiving MRIB messages",
-		   module_instance_name.c_str(), module_id);
-	return (XORP_ERROR);	// Probably not added before
-    }
-    
-    return (XORP_OK);
-}
-
-/**
- * MfeaNode::send_add_mrib:
- * @mrib: The #Mrib entry to add.
- * 
- * Add a #Mrib entry to all user-level protocols that are interested in
- * receiving MRIB messages.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
-int
-MfeaNode::send_add_mrib(const Mrib& mrib)
-{
-    ProtoRegister& pr = _mrib_messages_register;
-    const list<pair<string, xorp_module_id> >& module_list = pr.all_module_instance_name_list();
-    
-    //
-    // Send the message to all upper-layer protocols that expect it.
-    //
-    list<pair<string, xorp_module_id> >::const_iterator iter;
-    for (iter = module_list.begin(); iter != module_list.end(); ++iter) {
-	const string& dst_module_instance_name = (*iter).first;
-	xorp_module_id dst_module_id = (*iter).second;
-	send_add_mrib(dst_module_instance_name, dst_module_id, mrib);
-    }
-    
-    return (XORP_OK);
-}
-
-/**
- * MfeaNode::send_delete_mrib:
- * @mrib: The #Mrib entry to delete.
- * 
- * Delete a #Mrib entry from all user-level protocols that are interested in
- * receiving MRIB messages.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
-int
-MfeaNode::send_delete_mrib(const Mrib& mrib)
-{
-    ProtoRegister& pr = _mrib_messages_register;
-    const list<pair<string, xorp_module_id> >& module_list = pr.all_module_instance_name_list();
-    
-    //
-    // Send the message to all upper-layer protocols that expect it.
-    //
-    list<pair<string, xorp_module_id> >::const_iterator iter;
-    for (iter = module_list.begin(); iter != module_list.end(); ++iter) {
-	const string& dst_module_instance_name = (*iter).first;
-	xorp_module_id dst_module_id = (*iter).second;
-	send_delete_mrib(dst_module_instance_name, dst_module_id, mrib);
-    }
-    
-    return (XORP_OK);
-}
-
-/**
- * MfeaNode::send_set_mrib_done:
- * @: 
- * 
- * Complete a transaction of add/delete #Mrib entries with all user-level
- * protocols that are interested in receiving MRIB messages.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
-int
-MfeaNode::send_set_mrib_done()
-{
-    ProtoRegister& pr = _mrib_messages_register;
-    const list<pair<string, xorp_module_id> >& module_list = pr.all_module_instance_name_list();
-    
-    //
-    // Send the message to all upper-layer protocols that expect it.
-    //
-    list<pair<string, xorp_module_id> >::const_iterator iter;
-    for (iter = module_list.begin(); iter != module_list.end(); ++iter) {
-	const string& dst_module_instance_name = (*iter).first;
-	xorp_module_id dst_module_id = (*iter).second;
-	send_set_mrib_done(dst_module_instance_name, dst_module_id);
-    }
-    
-    return (XORP_OK);
-}
-
 /**
  * MfeaNode::proto_recv:
  * @src_module_instance_name: The module instance name of the module-origin
@@ -1549,7 +1410,7 @@ MfeaNode::send_set_mrib_done()
  * it should be ignored.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * it should be ignored.
- * @router_alert_bool: If true, set the Router Alert IP option for the IP
+ * @is_router_alert: If true, set the Router Alert IP option for the IP
  * packet of the outgoung message.
  * @rcvbuf: The data buffer with the message to send.
  * @rcvlen: The data length in @rcvbuf.
@@ -1564,7 +1425,7 @@ MfeaNode::proto_recv(const string&	, // src_module_instance_name,
 		     xorp_module_id src_module_id,
 		     uint16_t vif_index,
 		     const IPvX& src, const IPvX& dst,
-		     int ip_ttl, int ip_tos, bool router_alert_bool,
+		     int ip_ttl, int ip_tos, bool is_router_alert,
 		     const uint8_t *rcvbuf, size_t rcvlen)
 {
     ProtoComm *proto_comm;
@@ -1582,7 +1443,7 @@ MfeaNode::proto_recv(const string&	, // src_module_instance_name,
 				       src, dst,
 				       ip_ttl,
 				       ip_tos,
-				       router_alert_bool,
+				       is_router_alert,
 				       rcvbuf,
 				       rcvlen) < 0) {
 	return (XORP_ERROR);
@@ -1596,7 +1457,7 @@ int
 MfeaNode::proto_comm_recv(xorp_module_id dst_module_id,
 			  uint16_t vif_index,
 			  const IPvX& src, const IPvX& dst,
-			  int ip_ttl, int ip_tos, bool router_alert_bool,
+			  int ip_ttl, int ip_tos, bool is_router_alert,
 			  const uint8_t *rcvbuf, size_t rcvlen)
 {
     XLOG_TRACE(false & is_log_trace(),	// XXX: unconditionally disabled
@@ -1604,8 +1465,8 @@ MfeaNode::proto_comm_recv(xorp_module_id dst_module_id,
 	       "vif_index = %d src = %s dst = %s ttl = %d tos = %#x "
 	       "router_alert = %d rcvbuf = %p rcvlen = %u",
 	       xorp_module_name(family(), dst_module_id), vif_index,
-	       cstring(src), cstring(dst), ip_ttl, ip_tos, router_alert_bool,
-	       rcvbuf, (uint32_t)rcvlen);
+	       cstring(src), cstring(dst), ip_ttl, ip_tos, is_router_alert,
+	       rcvbuf, XORP_UINT_CAST(rcvlen));
     
     //
     // Test if we should accept or drop the message
@@ -1635,7 +1496,7 @@ MfeaNode::proto_comm_recv(xorp_module_id dst_module_id,
 		   src, dst,
 		   ip_ttl,
 		   ip_tos,
-		   router_alert_bool,
+		   is_router_alert,
 		   rcvbuf,
 		   rcvlen);
     }
@@ -1693,7 +1554,7 @@ MfeaNode::signal_message_recv(const string&	, // src_module_instance_name,
 	switch (family()) {
 	case AF_INET:
 	{
-#if defined(MRT_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	    size_t from = 0;
 	    struct bw_upcall bw_upcall;
 	    IPvX src(family()), dst(family());
@@ -1727,7 +1588,7 @@ MfeaNode::signal_message_recv(const string&	, // src_module_instance_name,
 		    is_geq_upcall,
 		    is_leq_upcall);
 	    }
-#endif // MRT_ADD_BW_UPCALL && ENABLE_ADVANCED_MCAST_API
+#endif // MRT_ADD_BW_UPCALL && ENABLE_ADVANCED_MULTICAST_API
 	}
 	break;
 	
@@ -1740,7 +1601,7 @@ MfeaNode::signal_message_recv(const string&	, // src_module_instance_name,
 	return (XORP_ERROR);
 #else
 	
-#if defined(MRT6_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT6_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	    size_t from = 0;
 	    struct bw6_upcall bw_upcall;
 	    IPvX src(family()), dst(family());
@@ -1774,7 +1635,7 @@ MfeaNode::signal_message_recv(const string&	, // src_module_instance_name,
 		    is_geq_upcall,
 		    is_leq_upcall);
 	    }
-#endif // MRT6_ADD_BW_UPCALL && ENABLE_ADVANCED_MCAST_API
+#endif // MRT6_ADD_BW_UPCALL && ENABLE_ADVANCED_MULTICAST_API
 #endif // HAVE_IPV6_MULTICAST_ROUTING
 	}
 	break;
@@ -2046,7 +1907,7 @@ MfeaNode::add_mfc(const string& , // module_instance_name,
 	if (oiflist_disable_wrongvif.test(i)) {
 	    switch (family()) {
 	    case AF_INET:
-#if defined(MRT_MFC_FLAGS_DISABLE_WRONGVIF) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT_MFC_FLAGS_DISABLE_WRONGVIF) && defined(ENABLE_ADVANCED_MULTICAST_API)
 		oifs_flags[i] |= MRT_MFC_FLAGS_DISABLE_WRONGVIF;
 #endif
 		break;
@@ -2059,7 +1920,7 @@ MfeaNode::add_mfc(const string& , // module_instance_name,
 			   "IPv6 multicast routing not supported");
 		return (XORP_ERROR);
 #else
-#if defined(MRT6_MFC_FLAGS_DISABLE_WRONGVIF) && defined(ENABLE_ADVANCED_MCAST_API)
+#if defined(MRT6_MFC_FLAGS_DISABLE_WRONGVIF) && defined(ENABLE_ADVANCED_MULTICAST_API)
 		oifs_flags[i] |= MRT6_MFC_FLAGS_DISABLE_WRONGVIF;
 #endif
 #endif // HAVE_IPV6_MULTICAST_ROUTING
@@ -2470,205 +2331,3 @@ MfeaNode::proto_comm_find_by_ipproto(int ipproto) const
     
     return (NULL);
 }
-
-/**
- * MfeaNode::get_mrib_table:
- * @mrib_table: A reference to the routing table vector composed
- * of #Mrib elements.
- * 
- * Return value: The number of entries in @mrib_table, or %XORP_ERROR
- * if there was an error.
- **/
-int
-MfeaNode::get_mrib_table(vector<Mrib* >& mrib_table)
-{
-    int mrib_table_n = 0;
-    
-    switch (family()) {
-    case AF_INET:
-    {
-	list<Fte4> fte_list4;
-	list<Fte4>::iterator iter;
-	
-	if (_ftic.get_table4(fte_list4) != true)
-	    return (XORP_ERROR);
-	mrib_table.resize(fte_list4.size());
-	for (iter = fte_list4.begin(); iter != fte_list4.end(); ++iter) {
-	    Fte4& fte = *iter;
-	    Mrib* mrib = new Mrib(family());
-	    mrib_table[mrib_table_n++] = mrib;
-	    MfeaVif *mfea_vif = NULL;
-	    mrib->set_dest_prefix(IPvXNet(fte.net()));
-	    mrib->set_metric_preference(mrib_table_default_metric_preference().get());
-	    mrib->set_metric(mrib_table_default_metric().get());
-	    mrib->set_next_hop_router_addr(IPvX(fte.gateway()));
-
-	    // Set the vif index
-	    mfea_vif = vif_find_by_name(fte.vifname());
-	    if (mfea_vif != NULL)
-		mrib->set_next_hop_vif_index(mfea_vif->vif_index());
-	    if (mrib->next_hop_vif_index() >= maxvifs()) {
-		debug_msg("get_mrib_table() error: "
-			  "cannot find interface toward next-hop "
-			  "router %s for destination %s\n",
-			  cstring(mrib->next_hop_router_addr()),
-			  cstring(mrib->dest_prefix()));
-	    }
-	}
-    }
-    break;
-    
-#ifdef HAVE_IPV6
-    case AF_INET6:
-    {
-	list<Fte6> fte_list6;
-	list<Fte6>::iterator iter;
-	
-	if (_ftic.get_table6(fte_list6) != true)
-	    return (XORP_ERROR);
-	mrib_table.resize(fte_list6.size());
-	for (iter = fte_list6.begin(); iter != fte_list6.end(); ++iter) {
-	    Fte6& fte = *iter;
-	    Mrib* mrib = new Mrib(family());
-	    mrib_table[mrib_table_n++] = mrib;
-	    MfeaVif *mfea_vif = NULL;
-	    mrib->set_dest_prefix(IPvXNet(fte.net()));
-	    mrib->set_metric_preference(mrib_table_default_metric_preference().get());
-	    mrib->set_metric(mrib_table_default_metric().get());
-	    mrib->set_next_hop_router_addr(IPvX(fte.gateway()));
-
-	    // Set the vif index
-	    mfea_vif = vif_find_by_name(fte.vifname());
-	    if (mfea_vif != NULL)
-		mrib->set_next_hop_vif_index(mfea_vif->vif_index());
-	    if (mrib->next_hop_vif_index() >= maxvifs()) {
-		debug_msg("get_mrib_table() error: "
-			  "cannot find interface toward next-hop "
-			  "router %s for destination %s\n",
-			  cstring(mrib->next_hop_router_addr()),
-			  cstring(mrib->dest_prefix()));
-	    }
-	}
-    }
-    break;
-#endif // HAVE_IPV6
-    
-    default:
-	XLOG_UNREACHABLE();
-	break;
-    }
-
-    return (mrib_table_n);
-}
-
-/**
- * MfeaNode::mrib_table_read_timer_timeout:
- * 
- * Timeout handler to read the MRIB table.
- **/
-void
-MfeaNode::mrib_table_read_timer_timeout()
-{
-    MribTable new_mrib_table(family());
-    MribTable::iterator iter;
-    bool is_changed = false;
-
-    ProtoRegister& pr = _mrib_messages_register;
-    const list<pair<string, xorp_module_id> >& module_list = pr.all_module_instance_name_list();
-    
-    if ((maxvifs() == 0) || (module_list.empty())) {
-	//
-	// XXX: If we don't have any vifs yet or if we don't have modules
-	// interested in the MRIB table, then there no point to get
-	// the MRIB table.
-	// Restart the timer after a much shorter interval.
-	//
-	TimeVal timeval(MRIB_TABLE_READ_PERIOD_SEC,
-			MRIB_TABLE_READ_PERIOD_USEC);
-	timeval = timeval/4 + TimeVal(1, 0);
-	
-	//
-	// Start again the timer
-	//
-	_mrib_table_read_timer =
-	    eventloop().new_oneoff_after(timeval,
-					 callback(this,
-						  &MfeaNode::mrib_table_read_timer_timeout));
-	return;
-    }
-    
-    //
-    // Add all new entries to the temporary table
-    //
-    do {
-	vector<Mrib* > new_mrib_vector;
-	int new_mrib_array_size = get_mrib_table(new_mrib_vector);
-	
-	if (new_mrib_array_size > 0) {
-	    for (int i = 0; i < new_mrib_array_size; i++) {
-		Mrib* new_mrib = new_mrib_vector[i];
-		new_mrib_table.insert(*new_mrib);
-	    }
-	}
-	delete_pointers_vector(new_mrib_vector);
-    } while (false);
-    
-    //
-    // Delete the entries that have disappeared
-    //
-    for (iter = mrib_table().begin();
-	 iter != mrib_table().end();
-	 ++iter) {
-	Mrib *mrib = *iter;
-	if (mrib == NULL)
-	    continue;
-	Mrib *new_mrib = new_mrib_table.find_exact(mrib->dest_prefix());
-	if ((new_mrib == NULL) || (*new_mrib != *mrib)) {
-	    send_delete_mrib(*mrib);
-	    is_changed = true;
-	}
-    }
-    
-    //
-    // Add the entries that have appeared
-    //
-    for (iter = new_mrib_table.begin();
-	 iter != new_mrib_table.end();
-	 ++iter) {
-	Mrib *new_mrib = *iter;
-	if (new_mrib == NULL)
-	    continue;
-	Mrib *mrib = mrib_table().find_exact(new_mrib->dest_prefix());
-	if ((mrib == NULL) || (*new_mrib != *mrib)) {
-	    send_add_mrib(*new_mrib);
-	    is_changed = true;
-	}
-    }
-
-    //
-    // Replace the table if changed
-    //
-    if (is_changed) {
-	mrib_table().clear();
-	
-	for (iter = new_mrib_table.begin();
-	     iter != new_mrib_table.end();
-	     ++iter) {
-	    Mrib *new_mrib = *iter;
-	    if (new_mrib == NULL)
-		continue;
-	    mrib_table().insert(*new_mrib);
-	}
-	
-	send_set_mrib_done();
-    }
-    
-    //
-    // Start again the timer
-    //
-    _mrib_table_read_timer =
-	eventloop().new_oneoff_after(TimeVal(MRIB_TABLE_READ_PERIOD_SEC,
-					     MRIB_TABLE_READ_PERIOD_USEC),
-				     callback(this,
-					      &MfeaNode::mrib_table_read_timer_timeout));
-}
