@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_node_cli.cc,v 1.9 2004/06/10 22:40:55 hodson Exp $"
-
+#ident "$XORP: xorp/fea/mfea_node_cli.cc,v 1.16 2005/03/25 02:53:10 pavlin Exp $"
 
 //
 // MFEA (Multicast Forwarding Engine Abstraction) CLI implementation
 //
 
 #include "mfea_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
@@ -72,6 +72,9 @@ MfeaNodeCli::~MfeaNodeCli()
 int
 MfeaNodeCli::start()
 {
+    if (! is_enabled())
+	return (XORP_OK);
+
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
@@ -80,7 +83,9 @@ MfeaNodeCli::start()
 
     if (add_all_cli_commands() < 0)
 	return (XORP_ERROR);
-    
+
+    XLOG_INFO("CLI started");
+
     return (XORP_OK);
 }
 
@@ -97,10 +102,40 @@ MfeaNodeCli::stop()
 
     if (delete_all_cli_commands() < 0)
 	ret_code = XORP_ERROR;
-    
+
+    XLOG_INFO("CLI stopped");
+
     return (ret_code);
 }
 
+/**
+ * Enable the node operation.
+ * 
+ * If an unit is not enabled, it cannot be start, or pending-start.
+ */
+void
+MfeaNodeCli::enable()
+{
+    ProtoUnit::enable();
+
+    XLOG_INFO("CLI enabled");
+}
+
+/**
+ * Disable the node operation.
+ * 
+ * If an unit is disabled, it cannot be start or pending-start.
+ * If the unit was runnning, it will be stop first.
+ */
+void
+MfeaNodeCli::disable()
+{
+    stop();
+    ProtoUnit::disable();
+
+    XLOG_INFO("CLI disabled");
+}
+
 int
 MfeaNodeCli::add_all_cli_commands()
 {
@@ -121,10 +156,6 @@ MfeaNodeCli::add_all_cli_commands()
 	add_cli_command("show mfea interface address",
 			"Display information about addresses of MFEA IPv4 interfaces",
 			callback(this, &MfeaNodeCli::cli_show_mfea_interface_address));
-
-	add_cli_command("show mfea mrib",
-			"Display MRIB IPv4 information inside MFEA",
-			callback(this, &MfeaNodeCli::cli_show_mfea_mrib));
     }
 
     if (mfea_node().is_ipv6()) {
@@ -142,10 +173,6 @@ MfeaNodeCli::add_all_cli_commands()
 	add_cli_command("show mfea6 interface address",
 			"Display information about addresses of MFEA IPv6 interfaces",
 			callback(this, &MfeaNodeCli::cli_show_mfea_interface_address));
-
-	add_cli_command("show mfea6 mrib",
-			"Display MRIB IPv6 information inside MFEA",
-			callback(this, &MfeaNodeCli::cli_show_mfea_mrib));
     }
 
     return (XORP_OK);
@@ -171,10 +198,17 @@ MfeaNodeCli::cli_show_mfea_dataflow(const vector<string>& argv)
 		group_range = IPvXNet(IPvX(argv[0].c_str()),
 				      IPvX::addr_bitlen(family()));
 	    } catch (InvalidString) {
-		cli_print(c_format("ERROR: Invalid group range: %s\n",
+		cli_print(c_format("ERROR: Invalid group range address: %s\n",
 				   argv[0].c_str()));
 		return (XORP_ERROR);
+	    } catch (InvalidNetmaskLength) {
+		XLOG_UNREACHABLE();
+		return (XORP_ERROR);
 	    }
+	} catch (InvalidNetmaskLength) {
+	    cli_print(c_format("ERROR: Invalid group range netmask length: %s\n",
+			       argv[0].c_str()));
+	    return (XORP_ERROR);
 	}
 	if (! group_range.is_multicast()) {
 	    cli_print(c_format("ERROR: Group range is not multicast: %s\n",
@@ -221,17 +255,21 @@ MfeaNodeCli::cli_show_mfea_dataflow(const vector<string>& argv)
 	    threshold_interval = mfea_dfe->threshold_interval();
 	    
 	    // The measured values
-	    if (mfea_dfe->is_threshold_in_packets())
-		s1 = c_format("%u", (uint32_t)mfea_dfe->measured_packets());
-	    else
+	    if (mfea_dfe->is_threshold_in_packets()) {
+		s1 = c_format("%u",
+			      XORP_UINT_CAST(mfea_dfe->measured_packets()));
+	    } else {
 		s1 = "?";
-	    if (mfea_dfe->is_threshold_in_bytes())
-		s2 = c_format("%u", (uint32_t)mfea_dfe->measured_bytes());
-	    else
+	    }
+	    if (mfea_dfe->is_threshold_in_bytes()) {
+		s2 = c_format("%u",
+			      XORP_UINT_CAST(mfea_dfe->measured_bytes()));
+	    } else {
 		s2 = "?";
+	    }
 	    measured_s = c_format("%u.%u|%s|%s",
-				  (uint32_t)start_time.sec(),
-				  (uint32_t)start_time.usec(),
+				  XORP_UINT_CAST(start_time.sec()),
+				  XORP_UINT_CAST(start_time.usec()),
 				  s1.c_str(), s2.c_str());
 	    
 	    // The entry type
@@ -250,8 +288,8 @@ MfeaNodeCli::cli_show_mfea_dataflow(const vector<string>& argv)
 	    else
 		s2 = "?";
 	    thresh_s = c_format("%u.%u|%s|%s",
-				(uint32_t)threshold_interval.sec(),
-				(uint32_t)threshold_interval.usec(),
+				XORP_UINT_CAST(threshold_interval.sec()),
+				XORP_UINT_CAST(threshold_interval.usec()),
 				s1.c_str(), s2.c_str());
 	    
 	    // Remaining time
@@ -259,14 +297,14 @@ MfeaNodeCli::cli_show_mfea_dataflow(const vector<string>& argv)
 	    if (now <= end) {
 		delta = end - now;
 		remain_s = c_format("%u.%u",
-				    (uint32_t)delta.sec(), 
-				    (uint32_t)delta.usec());
+				    XORP_UINT_CAST(delta.sec()),
+				    XORP_UINT_CAST(delta.usec()));
 	    } else {
 		// Negative time
 		delta = now - end;
 		remain_s = c_format("-%u.%u",
-				    (uint32_t)delta.sec(), 
-				    (uint32_t)delta.usec());
+				    XORP_UINT_CAST(delta.sec()),
+				    XORP_UINT_CAST(delta.usec()));
 	    }
 	    
 	    cli_print(c_format("  %-29s %-6s %-30s %-6s\n",
@@ -438,79 +476,3 @@ MfeaNodeCli::cli_show_mfea_interface_address(const vector<string>& argv)
     
     return (XORP_OK);
 }
-
-//
-// CLI COMMAND: "show mfea mrib [dest-address]"
-// CLI COMMAND: "show mfea6 mrib [dest-address]"
-//
-// Display MRIB information inside MFEA.
-//
-int
-MfeaNodeCli::cli_show_mfea_mrib(const vector<string>& argv)
-{
-    string dest_address_name;
-    IPvX dest_address(family());
-    
-    // Check the optional argument
-    if (argv.size()) {
-	dest_address_name = argv[0];
-	try {
-	    dest_address = IPvX(dest_address_name.c_str());
-	} catch (InvalidString) {
-	    cli_print(c_format("ERROR: Invalid destination address: %s\n",
-			       dest_address_name.c_str()));
-	    return (XORP_ERROR);
-	}
-    }
-    
-    // Test if we should print a single entry only
-    if (dest_address_name.size()) {
-	Mrib *mrib = mfea_node().mrib_table().find(dest_address);
-	if (mrib == NULL) {
-	    cli_print(c_format("No matching MRIB entry for %s\n",
-			       dest_address_name.c_str()));
-	    return (XORP_ERROR);
-	}
-	cli_print(c_format("%-18s %-15s %-7s %-8s %10s %6s\n",
-			   "DestPrefix", "NextHopRouter", "VifName", 
-			   "VifIndex", "MetricPref", "Metric"));
-	string vif_name = "UNKNOWN";
-	Vif *vif = mfea_node().vif_find_by_vif_index(mrib->next_hop_vif_index());
-	if (vif != NULL)
-	    vif_name = vif->name();
-	cli_print(c_format("%-18s %-15s %-7s %-8d %10d %6d\n",
-			   cstring(mrib->dest_prefix()),
-			   cstring(mrib->next_hop_router_addr()),
-			   vif_name.c_str(),
-			   mrib->next_hop_vif_index(),
-			   mrib->metric_preference(),
-			   mrib->metric()));
-	return (XORP_OK);
-    }
-    
-    cli_print(c_format("%-18s %-15s %-7s %-8s %10s %6s\n",
-		       "DestPrefix", "NextHopRouter", "VifName", "VifIndex",
-		       "MetricPref", "Metric"));
-    MribTable::iterator iter;
-    for (iter = mfea_node().mrib_table().begin();
-	 iter != mfea_node().mrib_table().end();
-	 ++iter) {
-	Mrib *mrib = *iter;
-	if (mrib == NULL)
-	    continue;
-	
-	string vif_name = "UNKNOWN";
-	Vif *vif = mfea_node().vif_find_by_vif_index(mrib->next_hop_vif_index());
-	if (vif != NULL)
-	    vif_name = vif->name();
-	cli_print(c_format("%-18s %-15s %-7s %-8d %10d %6d\n",
-			   cstring(mrib->dest_prefix()),
-			   cstring(mrib->next_hop_router_addr()),
-			   vif_name.c_str(),
-			   mrib->next_hop_vif_index(),
-			   mrib->metric_preference(),
-			   mrib->metric()));
-    }
-    
-    return (XORP_OK);
-}
