@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,19 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_proto_comm.cc,v 1.18 2004/06/10 22:40:55 hodson Exp $"
-
+#ident "$XORP: xorp/fea/mfea_proto_comm.cc,v 1.29 2005/03/25 02:53:10 pavlin Exp $"
 
 //
 // Multicast-related raw protocol communications.
 //
 
-
 #include "mfea_module.h"
+
 #include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+#include "libxorp/utils.hh"
 
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
@@ -45,15 +48,9 @@
 #include <netinet6/in6_var.h>
 #endif
 
-#include "mrt/include/ip_mroute.h"
-
-#include "libxorp/xlog.h"
-#include "libxorp/debug.h"
-#include "libxorp/ipvx.hh"
-#include "libxorp/utils.hh"
-
 #include "libcomm/comm_api.h"
 
+#include "mrt/include/ip_mroute.h"
 #include "mrt/max_vifs.h"
 #include "mrt/multicast_defs.h"
 
@@ -115,7 +112,7 @@
 //
 #ifdef HAVE_IPV6
 static uint16_t		rtalert_code;
-#ifndef HAVE_RFC2292BIS
+#ifndef HAVE_RFC3542
 static uint8_t		raopt[IP6OPT_RTALERT_LEN];
 #endif
 #endif // HAVE_IPV6
@@ -140,11 +137,11 @@ ProtoComm::ProtoComm(MfeaNode& mfea_node, int ipproto,
     // Init Router Alert related option stuff
 #ifdef HAVE_IPV6
     rtalert_code = htons(IP6OPT_RTALERT_MLD); // XXX: used by MLD only (?)
-#ifndef HAVE_RFC2292BIS
+#ifndef HAVE_RFC3542
     raopt[0] = IP6OPT_ROUTER_ALERT;
     raopt[1] = IP6OPT_RTALERT_LEN - 2;
     memcpy(&raopt[2], (caddr_t)&rtalert_code, sizeof(rtalert_code));
-#endif // ! HAVE_RFC2292BIS
+#endif // ! HAVE_RFC3542
 #endif // HAVE_IPV6
     
     _proto_socket = -1;
@@ -222,12 +219,12 @@ ProtoComm::~ProtoComm()
 int
 ProtoComm::start()
 {
-    if (is_up() || is_pending_up())
-	return (XORP_OK);
-
     // XXX: all ProtoComm are automatically enabled by default
     ProtoUnit::enable();
     
+    if (is_up() || is_pending_up())
+	return (XORP_OK);
+
     if (ProtoUnit::start() < 0)
 	return (XORP_ERROR);
     
@@ -284,7 +281,7 @@ ProtoComm::stop()
  * If set, the IP header of a raw packet should be created
  * by the application itself, otherwise the kernel will build it.
  * XXX: Used only for IPv4.
- * In post-RFC-2292, IPV6_PKTINFO has similar functions,
+ * In RFC-3542, IPV6_PKTINFO has similar functions,
  * but because it requires the interface index and outgoing address,
  * it is of little use for our purpose. Also, in RFC-2292 this option
  * was a flag, so for compatibility reasons we better not set it
@@ -367,7 +364,7 @@ ProtoComm::recv_pktinfo(bool enable_bool)
 	    return (XORP_ERROR);
 	}
 #else
-	// The old option (see RFC 2292)
+	// The old option (see RFC-2292)
 	if (setsockopt(_proto_socket, IPPROTO_IPV6, IPV6_PKTINFO,
 		       (void *)&bool_flag, sizeof(bool_flag)) < 0) {
 	    XLOG_ERROR("setsockopt(IPV6_PKTINFO, %u) failed: %s",
@@ -893,17 +890,37 @@ ProtoComm::open_proto_socket()
 	XLOG_UNREACHABLE();
 	return (XORP_ERROR);
     }
-    
-    // Assign a function to read from this socket
-    if (mfea_node().eventloop().add_selector(_proto_socket, SEL_RD,
-				callback(this, &ProtoComm::proto_socket_read))
-	== false) {
+
+    if (add_proto_socket_selector() < 0) {
+	close_proto_socket();
 	return (XORP_ERROR);
     }
-    
+
     return (XORP_OK);
 }
 
+/**
+ * Add the method to read from the protocol socket.
+ * 
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+int
+ProtoComm::add_proto_socket_selector()
+{
+    if (_proto_socket < 0)
+	return (XORP_ERROR);
+
+    // Assign a method to read from this socket
+    if (mfea_node().eventloop().add_selector(_proto_socket, SEL_RD,
+				callback(this, &ProtoComm::proto_socket_read))
+	== false) {
+	XLOG_ERROR("Cannot add a protocol socket to the set of sockets "
+		   "to read from in the event loop");
+	return (XORP_ERROR);
+    }
+
+    return (XORP_OK);
+}
 
 /**
  * ProtoComm::close_proto_socket:
@@ -931,6 +948,7 @@ ProtoComm::close_proto_socket()
 		    // XXX: the control over the socket passed
 		    // to the MfeaMrouter
 		    //
+		    _proto_socket = -1;
 		    return (XORP_OK);
 	    }
 	}
@@ -961,7 +979,7 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
     IPvX	dst(family());
     int		ip_ttl = -1;		// a.k.a. Hop-Limit in IPv6
     int		ip_tos = -1;
-    bool	router_alert_bool = false; // Router Alert option received
+    bool	is_router_alert = false; // Router Alert option received
     int		pif_index = 0;
     MfeaVif	*mfea_vif = NULL;
 
@@ -1007,7 +1025,8 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 	if (nbytes < (ssize_t)sizeof(*igmpmsg)) {
 	    XLOG_WARNING("proto_socket_read() failed: "
 			 "kernel signal packet size %d is smaller than minimum size %u",
-			 nbytes, (uint32_t)sizeof(*igmpmsg));
+			 XORP_INT_CAST(nbytes),
+			 XORP_UINT_CAST(sizeof(*igmpmsg)));
 	    return;		// Error
 	}
 	if (igmpmsg->im_mbz == 0) {
@@ -1029,7 +1048,7 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 	    XLOG_WARNING("proto_socket_read() failed: "
 			 "packet size %d is smaller than minimum size %u",
 			 nbytes,
-			 (uint32_t)sizeof(struct icmp6_hdr));
+			 XORP_UINT_CAST(sizeof(struct icmp6_hdr)));
 	    return;		// Error
 	}
 #else
@@ -1040,8 +1059,9 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 	    && (nbytes < (ssize_t)sizeof(struct mld_hdr))) {
 	    XLOG_WARNING("proto_socket_read() failed: "
 			 "kernel signal or packet size %d is smaller than minimum size %u",
-			 nbytes,
-			 min((uint32_t)sizeof(*mrt6msg), (uint32_t)sizeof(struct mld_hdr)));
+			 XORP_INT_CAST(nbytes),
+			 XORP_UINT_CAST(min(sizeof(*mrt6msg),
+					    sizeof(struct mld_hdr))));
 	    return;		// Error
 	}
 	if ((mrt6msg->im6_mbz == 0) || (_rcvmh.msg_controllen == 0)) {
@@ -1089,7 +1109,8 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 	if (nbytes < (ssize_t)sizeof(*ip)) {
 	    XLOG_WARNING("proto_socket_read() failed: "
 			 "packet size %d is smaller than minimum size %u",
-			 nbytes, (uint32_t)sizeof(*ip));
+			 XORP_INT_CAST(nbytes),
+			 XORP_UINT_CAST(sizeof(*ip)));
 	    return;		// Error
 	}
 	src.copy_in(_from4);
@@ -1139,7 +1160,8 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 	    XLOG_ERROR("proto_socket_read() failed: "
 		       "RX packet size from %s to %s with %d bytes instead of "
 		       "hdr+datalen=%d+%d=%d",
-		       cstring(src), cstring(dst), nbytes,
+		       cstring(src), cstring(dst),
+		       XORP_INT_CAST(nbytes),
 		       ip_hdr_len, ip_data_len, ip_hdr_len + ip_data_len);
 	    return;		// Error
 	}
@@ -1181,7 +1203,7 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 		if (test_ip_options_len < option_len)
 		    break;
 		if ((option_value == IPOPT_RA) && (option_len == 4)) {
-		    router_alert_bool = true;
+		    is_router_alert = true;
 		    break;
 		}
 		test_ip_options_len -= option_len;
@@ -1203,7 +1225,8 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 	if (_rcvmh.msg_flags & MSG_CTRUNC) {
 	    XLOG_ERROR("proto_socket_read() failed: "
 		       "RX packet from %s with size of %d bytes is truncated",
-		       cstring(src), nbytes);
+		       cstring(src),
+		       XORP_INT_CAST(nbytes));
 	    return;		// Error
 	}
 	size_t controllen =  static_cast<size_t>(_rcvmh.msg_controllen);
@@ -1212,8 +1235,8 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 		       "RX packet from %s has too short msg_controllen "
 		       "(%u instead of %u)",
 		       cstring(src),
-		       (uint32_t)controllen,
-		       (uint32_t)sizeof(struct cmsghdr));
+		       XORP_UINT_CAST(controllen),
+		       XORP_UINT_CAST(sizeof(struct cmsghdr)));
 	    return;		// Error
 	}
 	
@@ -1243,7 +1266,7 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 		    //
 		    // Check for Router Alert option
 		    //
-#ifdef HAVE_RFC2292BIS
+#ifdef HAVE_RFC3542
 		    {
 			struct ip6_hbh *ext;
 			int currentlen;
@@ -1261,12 +1284,12 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 			    if (currentlen == -1)
 				break;
 			    if (type == IP6OPT_ROUTER_ALERT) {
-				router_alert_bool = true;
+				is_router_alert = true;
 				break;
 			    }
 			}
 		    }
-#else // ! HAVE_RFC2292BIS (i.e., the old advanced API)
+#else // ! HAVE_RFC3542 (i.e., the old advanced API)
 		    {
 #ifdef HAVE_IPV6_MULTICAST_ROUTING
 			//
@@ -1277,13 +1300,13 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 			uint8_t  *tptr = NULL;
 			while (inet6_option_next(cmsgp, &tptr) == 0) {
 			    if (*tptr == IP6OPT_ROUTER_ALERT) {
-				router_alert_bool = true;
+				is_router_alert = true;
 				break;
 			    }
 			}
 #endif // HAVE_IPV6_MULTICAST_ROUTING
 		    }
-#endif // ! HAVE_RFC2292BIS
+#endif // ! HAVE_RFC3542
 		    
 		}
 		
@@ -1393,7 +1416,7 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
     if (mfea_node().proto_comm_recv(module_id(),
 				    mfea_vif->vif_index(),
 				    src, dst, ip_ttl, ip_tos,
-				    router_alert_bool,
+				    is_router_alert,
 				    _rcvbuf0 + ip_hdr_len, nbytes - ip_hdr_len)
 	!= XORP_OK) {
 	return;			// Error
@@ -1413,7 +1436,7 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
  * negative value, the TTL will be set here or by the lower layers.
  * @ip_tos: The TOS (a.k.a. Traffic Class in IPv6) of the packet. If it has a
  * negative value, the TOS will be set here or by the lower layers.
- * @router_alert_bool: If true, then the IP packet with the data should
+ * @is_router_alert: If true, then the IP packet with the data should
  * have the Router Alert option included.
  * @databuf: The data buffer.
  * @datalen: The length of the data in @databuf.
@@ -1425,7 +1448,7 @@ ProtoComm::proto_socket_read(int fd, SelectorMask mask)
 int
 ProtoComm::proto_socket_write(uint16_t vif_index,
 			      const IPvX& src, const IPvX& dst,
-			      int ip_ttl, int ip_tos, bool router_alert_bool,
+			      int ip_ttl, int ip_tos, bool is_router_alert,
 			      const uint8_t *databuf, size_t datalen)
 {
     struct ip	*ip;
@@ -1450,14 +1473,14 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
     case AF_INET:
 	// Assign the TTL
 	if (ip_ttl < 0) {
-	    if (router_alert_bool)
+	    if (is_router_alert)
 		ip_ttl = MINTTL;
 	    else
 		ip_ttl = IPDEFTTL;
 	}
 	// Assign the TOS
 	if (ip_tos < 0) {
-	    if (router_alert_bool)
+	    if (is_router_alert)
 		ip_tos = IPTOS_PREC_INTERNETCONTROL;  // Internet Control
 	    else
 		ip_tos = 0;
@@ -1468,14 +1491,14 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
     {
 	// Assign the TTL
 	if (ip_ttl < 0) {
-	    if (router_alert_bool)
+	    if (is_router_alert)
 		ip_ttl = MINTTL;
 	    else
 		ip_ttl = IPDEFTTL;
 	}
 	// Assign the TOS
 	if (ip_tos < 0) {
-	    if (router_alert_bool) {
+	    if (is_router_alert) {
 		// TODO: XXX: IPTOS for IPv6 is bogus??
 		// ip_tos = IPTOS_PREC_INTERNETCONTROL;  // Internet Control
 		ip_tos = 0;
@@ -1501,7 +1524,7 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
     switch (family()) {
     case AF_INET:
 	ip = (struct ip *)_sndbuf0;
-	if (router_alert_bool) {
+	if (is_router_alert) {
 	    // Include the Router Alert option
 	    ip_option	= htonl((IPOPT_RA << 24) | (0x04 << 16));
 	    ip_option_p	= (uint32_t *)(ip + 1);
@@ -1517,9 +1540,16 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
 	ip->ip_p	= _ipproto;
 	ip->ip_ttl	= ip_ttl;
 	ip->ip_tos	= ip_tos;
-	
-	src.copy_out(ip->ip_src);
-	dst.copy_out(ip->ip_dst);
+
+	//
+	// XXX: we need to use a temporary in_addr storage as a work-around
+	// if "struct ip" is __packed
+	//
+	struct in_addr in_addr_tmp;
+	src.copy_out(in_addr_tmp);
+	ip->ip_src = in_addr_tmp;
+	dst.copy_out(in_addr_tmp);
+	ip->ip_dst = in_addr_tmp;
 	ip->ip_len = (ip->ip_hl << 2) + datalen;
 #ifdef IPV4_RAW_OUTPUT_IS_RAW
 	ip->ip_len = htons(ip->ip_len);
@@ -1538,8 +1568,8 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
 		       mfea_vif->name().c_str(),
 		       src.str().c_str(),
 		       dst.str().c_str(),
-		       (uint32_t)datalen,
-		       (uint32_t)IO_BUF_SIZE);
+		       XORP_UINT_CAST(datalen),
+		       XORP_UINT_CAST(IO_BUF_SIZE));
 	    return (XORP_ERROR);
 	}
     	memcpy(_sndbuf1, databuf, datalen); // XXX: goes to _sndiov[1].iov_base
@@ -1568,9 +1598,9 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
 	// Space for IPV6_PKTINFO
 	ctllen = CMSG_SPACE(sizeof(struct in6_pktinfo));
 	
-	if (router_alert_bool) {
+	if (is_router_alert) {
 	    // Space for Router Alert option
-#ifdef HAVE_RFC2292BIS
+#ifdef HAVE_RFC3542
 	    if ((hbhlen = inet6_opt_init(NULL, 0)) == -1)
 		XLOG_ERROR("inet6_opt_init(NULL) failed");
 	    if ((hbhlen = inet6_opt_append(NULL, 0, hbhlen,
@@ -1596,7 +1626,7 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
 #else
 	    UNUSED(hbhlen);
 #endif
-#endif // ! HAVE_RFC2292BIS
+#endif // ! HAVE_RFC3542
 	}
 	// Space for IPV6_TCLASS
 #ifdef IPV6_TCLASS 
@@ -1635,8 +1665,8 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
 	//
 	// Include the Router Alert option if needed
 	//
-	if (router_alert_bool) {
-#ifdef HAVE_RFC2292BIS
+	if (is_router_alert) {
+#ifdef HAVE_RFC3542
 	    int currentlen;
 	    void *hbhbuf, *optp = NULL;
 	    
@@ -1661,7 +1691,7 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
 		return (XORP_ERROR);
 	    }
 	    
-#else  // ! HAVE_RFC2292BIS (i.e., the old advanced API)
+#else  // ! HAVE_RFC3542 (i.e., the old advanced API)
 
 #ifdef HAVE_IPV6_MULTICAST_ROUTING
 	    //
@@ -1679,7 +1709,7 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
 	    }
 #endif // HAVE_IPV6_MULTICAST_ROUTING
 	    
-#endif // ! HAVE_RFC2292BIS
+#endif // ! HAVE_RFC3542
 	    
 	    cmsgp = CMSG_NXTHDR(&_sndmh, cmsgp);
 	}
@@ -1717,8 +1747,8 @@ ProtoComm::proto_socket_write(uint16_t vif_index,
 		       mfea_vif->name().c_str(),
 		       src.str().c_str(),
 		       dst.str().c_str(),
-		       (uint32_t)datalen,
-		       (uint32_t)IO_BUF_SIZE);
+		       XORP_UINT_CAST(datalen),
+		       XORP_UINT_CAST(IO_BUF_SIZE));
 	    return (XORP_ERROR);
 	}
 	memcpy(_sndbuf0, databuf, datalen); // XXX: goes to _sndiov[0].iov_base
