@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_vif.cc,v 1.8 2004/06/18 21:51:52 pavlin Exp $"
-
+#ident "$XORP: xorp/fea/mfea_vif.cc,v 1.14 2005/03/25 02:53:11 pavlin Exp $"
 
 //
 // MFEA virtual interfaces implementation.
 //
 
-
 #include "mfea_module.h"
 
 #include "libxorp/xorp.h"
@@ -114,6 +112,9 @@ MfeaVif::~MfeaVif()
 int
 MfeaVif::start(string& error_msg)
 {
+    if (! is_enabled())
+	return (XORP_OK);
+
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
@@ -142,6 +143,9 @@ MfeaVif::start(string& error_msg)
 	return (XORP_ERROR);
     }
 
+    XLOG_INFO("Interface started: %s%s",
+	      this->str().c_str(), flags_string().c_str());
+
     return (XORP_OK);
 }
 
@@ -156,6 +160,8 @@ MfeaVif::start(string& error_msg)
 int
 MfeaVif::stop(string& error_msg)
 {
+    int ret_value = XORP_OK;
+
     if (is_down())
 	return (XORP_OK);
 
@@ -164,32 +170,33 @@ MfeaVif::stop(string& error_msg)
 	return (XORP_ERROR);
     }
 
+    if (ProtoUnit::pending_stop() < 0) {
+	error_msg = "internal error";
+	ret_value = XORP_ERROR;
+    }
+
     leave_all_multicast_groups();
 
     if (ProtoUnit::stop() < 0) {
 	error_msg = "internal error";
-	return (XORP_ERROR);
+	ret_value = XORP_ERROR;
     }
 
     if (mfea_node().delete_multicast_vif(vif_index()) < 0) {
-	error_msg = "cannot delete the multicast vif from the kernel";
-	return (XORP_ERROR);
+	XLOG_ERROR("Cannot delete multicast vif %s with the kernel",
+		   name().c_str());
+	ret_value = XORP_ERROR;
     }
 
-    XLOG_INFO("STOPPED %s%s",
+    XLOG_INFO("Interface stopped %s%s",
 	      this->str().c_str(), flags_string().c_str());
 
     //
-    // Test if time to completely stop the MfeaNode itself because
-    // of this vif
+    // Inform the node that the vif has completed the shutdown
     //
-    string dummy_string;
-    if (mfea_node().is_pending_down()
-	&& (! mfea_node().has_pending_down_units(dummy_string))) {
-	mfea_node().final_stop();
-    }
+    mfea_node().vif_shutdown_completed(name());
 
-    return (XORP_OK);
+    return (ret_value);
 }
 
 /**
@@ -201,6 +208,9 @@ void
 MfeaVif::enable()
 {
     ProtoUnit::enable();
+
+    XLOG_INFO("Interface enabled %s%s",
+	      this->str().c_str(), flags_string().c_str());
 }
 
 /**
@@ -216,6 +226,9 @@ MfeaVif::disable()
 
     stop(error_msg);
     ProtoUnit::disable();
+
+    XLOG_INFO("Interface disabled %s%s",
+	      this->str().c_str(), flags_string().c_str());
 }
 
 /**
