@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/netlink_socket.cc,v 1.17 2004/06/10 22:40:56 hodson Exp $"
-
+#ident "$XORP: xorp/fea/netlink_socket.cc,v 1.30 2005/03/25 02:53:11 pavlin Exp $"
 
 #include "fea_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
-#include "libcomm/comm_api.h"
-
 #include <algorithm>
 
 #include <sys/types.h>
@@ -39,6 +37,8 @@
 #include <linux/rtnetlink.h>
 #endif
 
+#include "libcomm/comm_api.h"
+
 #include "netlink_socket.hh"
 
 
@@ -49,8 +49,8 @@ pid_t NetlinkSocket::_pid = getpid();
 // Netlink Sockets (see netlink(7)) communication with the kernel
 //
 
-NetlinkSocket::NetlinkSocket(EventLoop& e)
-    : _e(e),
+NetlinkSocket::NetlinkSocket(EventLoop& eventloop)
+    : _eventloop(eventloop),
       _fd(-1),
       _seqno(0),
       _instance_no(_instance_cnt++),
@@ -62,45 +62,61 @@ NetlinkSocket::NetlinkSocket(EventLoop& e)
 
 NetlinkSocket::~NetlinkSocket()
 {
-    stop();
+    string error_msg;
+
+    if (stop(error_msg) != XORP_OK) {
+	XLOG_ERROR("Cannot stop the netlink socket: %s", error_msg.c_str());
+    }
+
     XLOG_ASSERT(_ol.empty());
 }
 
 #ifndef HAVE_NETLINK_SOCKETS
 
 int
-NetlinkSocket::start(int )
+NetlinkSocket::start(int af, string& error_msg)
 {
+    error_msg = c_format("The system does not support netlink sockets");
     XLOG_UNREACHABLE();
     return (XORP_ERROR);
+
+    UNUSED(af);
 }
 
 int
-NetlinkSocket::stop()
+NetlinkSocket::stop(string& error_msg)
 {
-    return (XORP_ERROR);
+    //
+    // XXX: Even if the system doesn not support netlink sockets, we
+    // still allow to call the no-op stop() method and return success.
+    // This is needed to cover the case when a NetlinkSocket is allocated
+    // but not used.
+    //
+    return (XORP_OK);
+    UNUSED(error_msg);
 }
 
-void
-NetlinkSocket::force_read()
+int
+NetlinkSocket::force_read(string& error_msg)
 {
     XLOG_UNREACHABLE();
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
 }
 
 #else // HAVE_NETLINK_SOCKETS
 
 int
-NetlinkSocket::start(int af)
+NetlinkSocket::start(int af, string& error_msg)
 {
     int			socket_protocol = -1;
     struct sockaddr_nl	snl;
     socklen_t		snl_len;
 
-    if (_fd >= 0) {
-	XLOG_ERROR("Cound not start netlink socket operation: "
-		   "already started");
-	return (XORP_ERROR);
-    }
+    if (_fd >= 0)
+	return (XORP_OK);
     
     //
     // Select the protocol
@@ -130,7 +146,8 @@ NetlinkSocket::start(int af)
     //
     _fd = socket(AF_NETLINK, SOCK_RAW, socket_protocol);
     if (_fd < 0) {
-	XLOG_ERROR("Could not open netlink socket: %s", strerror(errno));
+	error_msg = c_format("Could not open netlink socket: %s",
+			     strerror(errno));
 	return (XORP_ERROR);
     }
     //
@@ -149,7 +166,7 @@ NetlinkSocket::start(int af)
     snl.nl_pid    = 0;		// nl_pid = 0 if destination is the kernel
     snl.nl_groups = _nl_groups;
     if (bind(_fd, reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl)) < 0) {
-	XLOG_ERROR("bind(AF_NETLINK) failed: %s", strerror(errno));
+	error_msg = c_format("bind(AF_NETLINK) failed: %s", strerror(errno));
 	close(_fd);
 	_fd = -1;
 	return (XORP_ERROR);
@@ -160,23 +177,24 @@ NetlinkSocket::start(int af)
     //
     snl_len = sizeof(snl);
     if (getsockname(_fd, reinterpret_cast<struct sockaddr*>(&snl), &snl_len) < 0) {
-	XLOG_ERROR("getsockname(AF_NETLINK) failed: %s", strerror(errno));
+	error_msg = c_format("getsockname(AF_NETLINK) failed: %s",
+			     strerror(errno));
 	close(_fd);
 	_fd = -1;
 	return (XORP_ERROR);
     }
     if (snl_len != sizeof(snl)) {
-	XLOG_ERROR("Wrong address length of AF_NETLINK socket: "
-		   "%d instead of %d",
-		   snl_len, sizeof(snl));
+	error_msg = c_format("Wrong address length of AF_NETLINK socket: "
+			     "%d instead of %d",
+			     snl_len, sizeof(snl));
 	close(_fd);
 	_fd = -1;
 	return (XORP_ERROR);
     }
     if (snl.nl_family != AF_NETLINK) {
-	XLOG_ERROR("Wrong address family of AF_NETLINK socket: "
-		   "%d instead of %d",
-		   snl.nl_family, AF_NETLINK);
+	error_msg = c_format("Wrong address family of AF_NETLINK socket: "
+			     "%d instead of %d",
+			     snl.nl_family, AF_NETLINK);
 	close(_fd);
 	_fd = -1;
 	return (XORP_ERROR);
@@ -184,8 +202,9 @@ NetlinkSocket::start(int af)
 #if 0				// XXX
     // XXX: 'nl_pid' is supposed to be defined as 'pid_t'
     if ( (pid_t)snl.nl_pid != pid()) {
-	XLOG_ERROR("Wrong nl_pid of AF_NETLINK socket: %d instead of %d",
-		   snl.nl_pid, pid());
+	error_msg = c_format("Wrong nl_pid of AF_NETLINK socket: "
+			     "%d instead of %d",
+			     snl.nl_pid, pid());
 	close(_fd);
 	_fd = -1;
 	return (XORP_ERROR);
@@ -195,33 +214,30 @@ NetlinkSocket::start(int af)
     //
     // Add the socket to the event loop
     //
-    if (_e.add_selector(_fd, SEL_RD,
-			callback(this, &NetlinkSocket::select_hook))
+    if (_eventloop.add_selector(_fd, SEL_RD,
+				callback(this, &NetlinkSocket::select_hook))
 	== false) {
-	XLOG_ERROR("Failed to add netlink socket to EventLoop");
+	error_msg = c_format("Failed to add netlink socket to EventLoop");
 	close(_fd);
 	_fd = -1;
+	return (XORP_ERROR);
     }
     
     return (XORP_OK);
 }
 
 int
-NetlinkSocket::stop()
-{
-    shutdown();
-    
-    return (XORP_OK);
-}
-
-void
-NetlinkSocket::shutdown()
+NetlinkSocket::stop(string& error_msg)
 {
     if (_fd >= 0) {
-	_e.remove_selector(_fd, SEL_ALL);
+	_eventloop.remove_selector(_fd, SEL_ALL);
 	close(_fd);
 	_fd = -1;
     }
+    
+    return (XORP_OK);
+
+    UNUSED(error_msg);
 }
 
 ssize_t
@@ -239,8 +255,8 @@ NetlinkSocket::sendto(const void* data, size_t nbytes, int flags,
     return ::sendto(_fd, data, nbytes, flags, to, tolen);
 }
 
-void
-NetlinkSocket::force_read()
+int
+NetlinkSocket::force_read(string& error_msg)
 {
     vector<uint8_t> message;
     vector<uint8_t> buffer(NLSOCK_BYTES);
@@ -253,6 +269,8 @@ NetlinkSocket::force_read()
 	do {
 	    got = recv(_fd, &buffer[0], buffer.size(),
 		       MSG_DONTWAIT | MSG_PEEK);
+	    if ((got < 0) && (errno == EINTR))
+		continue;	// XXX: the receive was interrupted by a signal
 	    if ((got < 0) || (got < (ssize_t)buffer.size()))
 		break;		// The buffer is big enough
 	    buffer.resize(buffer.size() + NLSOCK_BYTES);
@@ -262,20 +280,21 @@ NetlinkSocket::force_read()
 	if (got < 0) {
 	    if (errno == EINTR)
 		continue;
-	    XLOG_ERROR("Netlink socket read error: %s", strerror(errno));
-	    shutdown();
-	    return;
+	    error_msg = c_format("Netlink socket read error: %s",
+			      strerror(errno));
+	    return (XORP_ERROR);
 	}
 	message.resize(message.size() + got);
 	memcpy(&message[off], &buffer[0], got);
 	off += got;
 	
 	if ((off - last_mh_off) < (ssize_t)sizeof(struct nlmsghdr)) {
-	    XLOG_ERROR("Netlink socket read failed: message truncated: "
-		       "received %d bytes instead of (at least) %u bytes",
-		       got, (uint32_t)sizeof(struct nlmsghdr));
-	    shutdown();
-	    return;
+	    error_msg = c_format("Netlink socket read failed: "
+				 "message truncated: "
+				 "received %d bytes instead of (at least) %u "
+				 "bytes",
+				 got, XORP_UINT_CAST(sizeof(struct nlmsghdr)));
+	    return (XORP_ERROR);
 	}
 	
 	//
@@ -307,11 +326,13 @@ NetlinkSocket::force_read()
     for (ObserverList::iterator i = _ol.begin(); i != _ol.end(); i++) {
 	(*i)->nlsock_data(&message[0], message.size());
     }
+
+    return (XORP_OK);
 }
 
-void
+int
 NetlinkSocket::force_recvfrom(int flags, struct sockaddr* from,
-			      socklen_t* fromlen)
+			      socklen_t* fromlen, string& error_msg)
 {
     vector<uint8_t> message;
     vector<uint8_t> buffer(NLSOCK_BYTES);
@@ -324,6 +345,8 @@ NetlinkSocket::force_recvfrom(int flags, struct sockaddr* from,
 	do {
 	    got = recv(_fd, &buffer[0], buffer.size(),
 		       MSG_DONTWAIT | MSG_PEEK);
+	    if ((got < 0) && (errno == EINTR))
+		continue;	// XXX: the receive was interrupted by a signal
 	    if ((got < 0) || (got < (ssize_t)buffer.size()))
 		break;		// The buffer is big enough
 	    buffer.resize(buffer.size() + NLSOCK_BYTES);
@@ -333,20 +356,21 @@ NetlinkSocket::force_recvfrom(int flags, struct sockaddr* from,
 	if (got < 0) {
 	    if (errno == EINTR)
 		continue;
-	    XLOG_ERROR("Netlink socket recvfrom error: %s", strerror(errno));
-	    shutdown();
-	    return;
+	    error_msg = c_format("Netlink socket recvfrom error: %s",
+				 strerror(errno));
+	    return (XORP_ERROR);
 	}
 	message.resize(message.size() + got);
 	memcpy(&message[off], &buffer[0], got);
 	off += got;
 	
 	if ((off - last_mh_off) < (ssize_t)sizeof(struct nlmsghdr)) {
-	    XLOG_ERROR("Netlink socket recvfrom failed: message truncated: "
-		       "received %d bytes instead of (at least) %u bytes",
-		       got, (uint32_t)sizeof(struct nlmsghdr));
-	    shutdown();
-	    return;
+	    error_msg = c_format("Netlink socket recvfrom failed: "
+				 "message truncated: "
+				 "received %d bytes instead of (at least) %u "
+				 "bytes",
+				 got, XORP_UINT_CAST(sizeof(struct nlmsghdr)));
+	    return (XORP_ERROR);
 	}
 	
 	//
@@ -378,10 +402,12 @@ NetlinkSocket::force_recvfrom(int flags, struct sockaddr* from,
     for (ObserverList::iterator i = _ol.begin(); i != _ol.end(); i++) {
 	(*i)->nlsock_data(&message[0], message.size());
     }
+
+    return (XORP_OK);
 }
 
-void
-NetlinkSocket::force_recvmsg(int flags)
+int
+NetlinkSocket::force_recvmsg(int flags, string& error_msg)
 {
     vector<uint8_t> message;
     vector<uint8_t> buffer(NLSOCK_BYTES);
@@ -412,6 +438,8 @@ NetlinkSocket::force_recvmsg(int flags)
 	do {
 	    got = recv(_fd, &buffer[0], buffer.size(),
 		       MSG_DONTWAIT | MSG_PEEK);
+	    if ((got < 0) && (errno == EINTR))
+		continue;	// XXX: the receive was interrupted by a signal
 	    if ((got < 0) || (got < (ssize_t)buffer.size()))
 		break;		// The buffer is big enough
 	    buffer.resize(buffer.size() + NLSOCK_BYTES);
@@ -425,27 +453,27 @@ NetlinkSocket::force_recvmsg(int flags)
 	if (got < 0) {
 	    if (errno == EINTR)
 		continue;
-	    XLOG_ERROR("Netlink socket recvmsg error: %s", strerror(errno));
-	    shutdown();
-	    return;
+	    error_msg = c_format("Netlink socket recvmsg error: %s",
+				 strerror(errno));
+	    return (XORP_ERROR);
 	}
 	if (msg.msg_namelen != sizeof(snl)) {
-	    XLOG_ERROR("Netlink socket recvmsg error: "
-		       "sender address length %d instead of %u ",
-		       msg.msg_namelen, sizeof(snl));
-	    shutdown();
-	    return;
+	    error_msg = c_format("Netlink socket recvmsg error: "
+				 "sender address length %d instead of %u ",
+				 msg.msg_namelen, sizeof(snl));
+	    return (XORP_ERROR);
 	}
 	message.resize(message.size() + got);
 	memcpy(&message[off], &buffer[0], got);
 	off += got;
 	
 	if ((off - last_mh_off) < (ssize_t)sizeof(struct nlmsghdr)) {
-	    XLOG_ERROR("Netlink socket recvmsg failed: message truncated: "
-		       "received %d bytes instead of (at least) %u bytes",
-		       got, (uint32_t)sizeof(struct nlmsghdr));
-	    shutdown();
-	    return;
+	    error_msg = c_format("Netlink socket recvmsg failed: "
+				 "message truncated: "
+				 "received %d bytes instead of (at least) %u "
+				 "bytes",
+				 got, XORP_UINT_CAST(sizeof(struct nlmsghdr)));
+	    return (XORP_ERROR);
 	}
 	
 	//
@@ -477,14 +505,21 @@ NetlinkSocket::force_recvmsg(int flags)
     for (ObserverList::iterator i = _ol.begin(); i != _ol.end(); i++) {
 	(*i)->nlsock_data(&message[0], message.size());
     }
+
+    return (XORP_OK);
 }
 
 void
 NetlinkSocket::select_hook(int fd, SelectorMask m)
 {
+    string error_msg;
+
     XLOG_ASSERT(fd == _fd);
     XLOG_ASSERT(m == SEL_RD);
-    force_read();
+    if (force_read(error_msg) != XORP_OK) {
+	XLOG_ERROR("Error force_read() from netlink socket: %s",
+		   error_msg.c_str());
+    }
 }
 
 #endif // HAVE_NETLINK_SOCKETS
@@ -565,15 +600,17 @@ NetlinkSocketReader::~NetlinkSocketReader()
  * Force the reader to receive data from the IPv4 netlink socket.
  *
  * @param seqno the sequence number of the data to receive.
+ * @param error_msg the error message (if error).
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
-NetlinkSocketReader::receive_data4(uint32_t seqno)
+NetlinkSocketReader::receive_data4(uint32_t seqno, string& error_msg)
 {
     _cache_seqno = seqno;
     _cache_valid = false;
     while (_cache_valid == false) {
-	_ns4.force_read();
+	if (_ns4.force_read(error_msg) != XORP_OK)
+	    return (XORP_ERROR);
     }
 
     return (XORP_OK);
@@ -583,15 +620,17 @@ NetlinkSocketReader::receive_data4(uint32_t seqno)
  * Force the reader to receive data from the IPv6 netlink socket.
  *
  * @param seqno the sequence number of the data to receive.
+ * @param error_msg the error message (if error).
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
-NetlinkSocketReader::receive_data6(uint32_t seqno)
+NetlinkSocketReader::receive_data6(uint32_t seqno, string& error_msg)
 {
     _cache_seqno = seqno;
     _cache_valid = false;
     while (_cache_valid == false) {
-	_ns6.force_read();
+	if (_ns6.force_read(error_msg) != XORP_OK)
+	    return (XORP_ERROR);
     }
 
     return (XORP_OK);
@@ -602,15 +641,18 @@ NetlinkSocketReader::receive_data6(uint32_t seqno)
  *
  * @param ns the netlink socket to receive the data from.
  * @param seqno the sequence number of the data to receive.
+ * @param error_msg the error message (if error).
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
-NetlinkSocketReader::receive_data(NetlinkSocket& ns, uint32_t seqno)
+NetlinkSocketReader::receive_data(NetlinkSocket& ns, uint32_t seqno,
+				  string& error_msg)
 {
     _cache_seqno = seqno;
     _cache_valid = false;
     while (_cache_valid == false) {
-	ns.force_read();
+	if (ns.force_read(error_msg) != XORP_OK)
+	    return (XORP_ERROR);
     }
 
     return (XORP_OK);
