@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,24 +12,28 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/test_xrl_sockets4_udp.cc,v 1.5 2004/06/10 22:40:57 hodson Exp $"
+#ident "$XORP: xorp/fea/test_xrl_sockets4_udp.cc,v 1.10 2005/03/25 02:53:15 pavlin Exp $"
 
 #include <sysexits.h>
 
 #include <set>
 
 #include "fea_module.h"
+
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
+#include "libxorp/debug.h"
 #include "libxorp/status_codes.h"
 
 #include "libxipc/sockutil.hh"
 #include "libxipc/xrl_std_router.hh"
 
+#include "xrl/interfaces/socket4_xif.hh"
+#include "xrl/targets/test_socket4_base.hh"
+
 #include "xrl_socket_server.hh"
 #include "addr_table.hh"
 
-#include "xrl/interfaces/socket4_xif.hh"
-#include "xrl/targets/test_socket4_base.hh"
 
 static const uint8_t FILLER_VALUE = 0xe7;
 
@@ -170,8 +174,8 @@ public:
 	verbose_log("Sending bind request (\"%s\", %s/%u)\n",
 		    _ssname.c_str(), addr.str().c_str(), port);
 	bool s = c.send_udp_open_and_bind(
-			_ssname.c_str(), _r->instance_name(), addr, port,
-			callback(this, &TestSocket4UDP::bind_cb));
+	    _ssname.c_str(), _r->instance_name(), addr, port, false,
+	    callback(this, &TestSocket4UDP::bind_cb));
 	return s;
     }
 
@@ -532,12 +536,10 @@ remove_address(TestAddressTable* ta, IPv4 addr)
 int
 test_main(IPv4 finder_host, uint16_t finder_port)
 {
-    const IPv4 localhost("127.0.0.1");
-
     EventLoop e;
     TestAddressTable tat;
 
-    tat.add_address(localhost);
+    tat.add_address(IPv4::LOOPBACK());
 
     vector<XorpTimer> ev;	// Vector for timed events
     bool eflag(false);		// Error flag set by timed events
@@ -568,7 +570,7 @@ test_main(IPv4 finder_host, uint16_t finder_port)
 
     ev.push_back(e.new_oneoff_after_ms(1500,
 				       callback(bind_test_socket, &udp1,
-						localhost, uint16_t(5000),
+						IPv4::LOOPBACK(), uint16_t(5000),
 						&eflag)));
 
     //
@@ -582,7 +584,7 @@ test_main(IPv4 finder_host, uint16_t finder_port)
 
     ev.push_back(e.new_oneoff_after_ms(2500,
 				       callback(bind_test_socket, &udp2,
-						localhost, uint16_t(5001),
+						IPv4::LOOPBACK(), uint16_t(5001),
 						&eflag)));
 
     //
@@ -590,7 +592,7 @@ test_main(IPv4 finder_host, uint16_t finder_port)
     //
     ev.push_back(e.new_oneoff_after_ms(3000,
 				       callback(start_sending, &udp2,
-						localhost, uint16_t(5000))));
+						IPv4::LOOPBACK(), uint16_t(5000))));
 
     ev.push_back(e.new_oneoff_after_ms(10000,
 				       callback(stop_sending, &udp2)));
@@ -600,7 +602,7 @@ test_main(IPv4 finder_host, uint16_t finder_port)
     //
     ev.push_back(e.new_oneoff_after_ms(10000,
 				       callback(start_sending, &udp1,
-						localhost, uint16_t(5001))));
+						IPv4::LOOPBACK(), uint16_t(5001))));
 
     ev.push_back(e.new_oneoff_after_ms(12000,
 				       callback(stop_sending, &udp1)));
@@ -646,12 +648,12 @@ test_main(IPv4 finder_host, uint16_t finder_port)
 						false, &eflag)));
 
     //
-    // Invalidate localhost, which should cause udp2 to be closed by
+    // Invalidate IPv4::LOOPBACK(), which should cause udp2 to be closed by
     // server.
     //
     ev.push_back(e.new_oneoff_after_ms(15000,
 				       callback(remove_address,
-						&tat, localhost)));
+						&tat, IPv4::LOOPBACK())));
 
     //
     // Check udp2 is closed.
@@ -749,7 +751,7 @@ parse_finder_arg(const char* host_colon_port,
 	}
     } else {
 	finder_host = string(host_colon_port);
-	finder_port = FINDER_DEFAULT_PORT;
+	finder_port = FinderConstants::FINDER_DEFAULT_PORT();
     }
 
     try {
@@ -782,8 +784,8 @@ main(int argc, char* const argv[])
     xlog_add_default_output();
     xlog_start();
 
-    IPv4        finder_addr = FINDER_DEFAULT_HOST;
-    uint16_t    finder_port = FINDER_DEFAULT_PORT;
+    IPv4        finder_addr = FinderConstants::FINDER_DEFAULT_HOST();
+    uint16_t    finder_port = FinderConstants::FINDER_DEFAULT_PORT();
 
     int r = 0;	// Return code
 
