@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,20 +12,21 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/xrl_socket_server.cc,v 1.15 2004/06/10 22:40:59 hodson Exp $"
+#ident "$XORP: xorp/fea/xrl_socket_server.cc,v 1.21 2005/03/25 02:53:17 pavlin Exp $"
 
 #include "fea_module.h"
 
-#include "config.h"
 #include "libxorp/xorp.h"
-
+#include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 #include "libxorp/eventloop.hh"
 #include "libxorp/status_codes.h"
-#include "libxorp/xlog.h"
+
+#include "libcomm/comm_api.h"
+
 #include "libxipc/xrl_std_router.hh"
 #include "libxipc/xuid.hh"
-#include "libcomm/comm_api.h"
+
 #include "xrl/interfaces/finder_event_notifier_xif.hh"
 
 #include "xrl_socket_server.hh"
@@ -41,12 +42,13 @@ static const char* NO_IPV6_MSG = "IPv6 is not available on host";
  */
 class ChattyXrlStdRouter : public XrlStdRouter {
 public:
-    ChattyXrlStdRouter(EventLoop&	e,
+    ChattyXrlStdRouter(EventLoop&	eventloop,
 		       const char*	class_name,
 		       IPv4	   	finder_host,
 		       uint16_t		finder_port,
 		       XrlSocketServer* parent)
-	: XrlStdRouter(e, class_name, finder_host, finder_port), _p(parent)
+	: XrlStdRouter(eventloop, class_name, finder_host, finder_port),
+	  _p(parent)
     {}
 
     void
@@ -305,8 +307,8 @@ XrlSocketServer::RemoteSocket<A>::RemoteSocket<A>(XrlSocketServer& ss,
 template <typename A>
 XrlSocketServer::RemoteSocket<A>::~RemoteSocket()
 {
-    EventLoop& e = _ss.eventloop();
-    e.remove_selector(_fd);
+    EventLoop& eventloop = _ss.eventloop();
+    eventloop.remove_selector(_fd);
     comm_close(_fd);
     _owner.decr_socket_count();
 }
@@ -315,16 +317,16 @@ template <typename A>
 void
 XrlSocketServer::RemoteSocket<A>::set_data_recv_enable(bool en)
 {
-    EventLoop& e = _ss.eventloop();
+    EventLoop& eventloop = _ss.eventloop();
     if (en) {
 	debug_msg("Adding selector for %d\n", _fd);
-	if (e.add_selector(_fd, SEL_RD,
-		callback(this, &RemoteSocket::data_sel_cb)) == false) {
+	if (eventloop.add_selector(_fd, SEL_RD,
+			callback(this, &RemoteSocket::data_sel_cb)) == false) {
 	    XLOG_ERROR("FAILED TO ADD SELECTOR %d\n", _fd);
 	}
     } else {
 	debug_msg("Removing selector for %d\n", _fd);
-	e.remove_selector(_fd);
+	eventloop.remove_selector(_fd);
     }
 }
 
@@ -372,12 +374,12 @@ template <typename A>
 void
 XrlSocketServer::RemoteSocket<A>::set_connect_recv_enable(bool en)
 {
-    EventLoop& e = _ss.eventloop();
+    EventLoop& eventloop = _ss.eventloop();
     if (en) {
-	e.remove_selector(_fd, SEL_RD,
+	eventloop.remove_selector(_fd, SEL_RD,
 			  callback(this, &RemoteSocket::connect_sel_cb));
     } else {
-	e.remove_selector(_fd);
+	eventloop.remove_selector(_fd);
     }
 }
 
@@ -488,14 +490,15 @@ private:
 // ----------------------------------------------------------------------------
 //
 
-XrlSocketServer::XrlSocketServer(EventLoop&		e,
+XrlSocketServer::XrlSocketServer(EventLoop&		eventloop,
 				 AddressTableBase&	atable,
 				 const IPv4&		finder_host,
 				 uint16_t		finder_port)
-    : ServiceBase("Socket Server"), _e(e), _atable(atable)
+    : ServiceBase("Socket Server"), _eventloop(eventloop), _atable(atable)
 {
     const char* class_name = "socket_server";
-    _r = new ChattyXrlStdRouter(e, class_name, finder_host, finder_port, this);
+    _r = new ChattyXrlStdRouter(eventloop, class_name, finder_host,
+				finder_port, this);
     if (set_command_map(_r) == false) {
 	XLOG_FATAL("Could not set command map.");
     }
@@ -514,7 +517,7 @@ bool
 XrlSocketServer::startup()
 {
     _r->finalize();
-    set_status(STARTING);
+    set_status(SERVICE_STARTING);
 
     return true;
 }
@@ -522,7 +525,7 @@ XrlSocketServer::startup()
 bool
 XrlSocketServer::shutdown()
 {
-    set_status(SHUTTING_DOWN);
+    set_status(SERVICE_SHUTTING_DOWN);
 
     return true;
 }
@@ -637,9 +640,10 @@ XrlCmdError
 XrlSocketServer::socket4_0_1_tcp_open_and_bind(const string&	creator,
 					       const IPv4&	local_addr,
 					       const uint32_t&	local_port,
+					       const bool&	is_blocking,
 					       string&		sockid)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -650,7 +654,7 @@ XrlSocketServer::socket4_0_1_tcp_open_and_bind(const string&	creator,
 
     in_addr ia;
     local_addr.copy_out(ia);
-    int fd = comm_bind_tcp4(&ia, htons(local_port));
+    int fd = comm_bind_tcp4(&ia, htons(local_port), is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
@@ -670,9 +674,10 @@ XrlCmdError
 XrlSocketServer::socket4_0_1_udp_open_and_bind(const string&	creator,
 					       const IPv4&	local_addr,
 					       const uint32_t&	local_port,
+					       const bool&	is_blocking,
 					       string&		sockid)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -683,7 +688,7 @@ XrlSocketServer::socket4_0_1_udp_open_and_bind(const string&	creator,
 
     in_addr ia;
     local_addr.copy_out(ia);
-    int fd = comm_bind_udp4(&ia, htons(local_port));
+    int fd = comm_bind_udp4(&ia, htons(local_port), is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
@@ -707,13 +712,14 @@ XrlSocketServer::socket4_0_1_udp_open_bind_join(const string&	creator,
 						const IPv4&	mcast_addr,
 						const uint32_t&	ttl,
 						const bool&	reuse,
+						const bool&	is_blocking,
 						string&		sockid)
 {
     debug_msg("udp_open_bind_join(%s, %s, %u, %s, ttl=%u, reuse %d)\n",
 	      creator.c_str(), local_addr.str().c_str(), local_port,
 	      mcast_addr.str().c_str(), ttl, reuse);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -728,7 +734,8 @@ XrlSocketServer::socket4_0_1_udp_open_bind_join(const string&	creator,
     in_addr grp;
     mcast_addr.copy_out(grp);
 
-    int fd = comm_bind_join_udp4(&grp, &ia, htons(local_port), reuse);
+    int fd = comm_bind_join_udp4(&grp, &ia, htons(local_port), reuse,
+				 is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
@@ -772,10 +779,11 @@ XrlSocketServer::socket4_0_1_tcp_open_bind_connect(
     const uint32_t&	local_port,
     const IPv4&		remote_addr,
     const uint32_t&	remote_port,
+    const bool&		is_blocking,
     string&		sockid
     )
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -786,14 +794,15 @@ XrlSocketServer::socket4_0_1_tcp_open_bind_connect(
     in_addr ia;
     local_addr.copy_out(ia);
 
-    int fd = comm_bind_tcp4(&ia, htons(local_port));
+    int fd = comm_bind_tcp4(&ia, htons(local_port), is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
 
     in_addr ra;
     remote_addr.copy_out(ra);
-    if (comm_sock_connect4(fd, &ra, htons(remote_port)) != XORP_OK) {
+    if (comm_sock_connect4(fd, &ra, htons(remote_port), is_blocking)
+	!= XORP_OK) {
 	comm_close(fd);
 	return XrlCmdError::COMMAND_FAILED("Connect failed.");
     }
@@ -817,10 +826,11 @@ XrlSocketServer::socket4_0_1_udp_open_bind_connect(
     const uint32_t& 	local_port,
     const IPv4&		remote_addr,
     const uint32_t&	remote_port,
+    const bool&		is_blocking,
     string&		sockid
     )
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -835,7 +845,8 @@ XrlSocketServer::socket4_0_1_udp_open_bind_connect(
     remote_addr.copy_out(ra);
 
     int fd = comm_bind_connect_udp4(&ia, htons(local_port),
-				    &ra, htons(remote_port));
+				    &ra, htons(remote_port),
+				    is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
@@ -857,7 +868,7 @@ XrlSocketServer::socket4_0_1_udp_join_group(const string&	sockid,
 					    const IPv4&		group,
 					    const IPv4&		if_addr)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V4Sockets::const_iterator ci = find_if(_v4sockets.begin(),
@@ -892,7 +903,7 @@ XrlSocketServer::socket4_0_1_udp_leave_group(const string&	sockid,
 					    const IPv4&		group,
 					    const IPv4&		if_addr)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V4Sockets::const_iterator ci = find_if(_v4sockets.begin(),
@@ -925,7 +936,7 @@ XrlSocketServer::socket4_0_1_udp_leave_group(const string&	sockid,
 XrlCmdError
 XrlSocketServer::socket4_0_1_close(const string& sockid)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V4Sockets::iterator i4;
@@ -943,7 +954,7 @@ XrlCmdError
 XrlSocketServer::socket4_0_1_tcp_listen(const string&	sockid,
 					const uint32_t&	backlog)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V4Sockets::const_iterator ci;
@@ -967,7 +978,7 @@ XrlSocketServer::socket4_0_1_send(
     const vector<uint8_t>&	data
     )
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V4Sockets::const_iterator ci;
@@ -993,7 +1004,7 @@ XrlSocketServer::socket4_0_1_send_with_flags(
     const bool&			end_of_file
     )
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V4Sockets::const_iterator ci;
@@ -1045,7 +1056,7 @@ XrlSocketServer::socket4_0_1_send_to(const string&		sockid,
 				     const uint32_t&		remote_port,
 				     const vector<uint8_t>&	data)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V4Sockets::const_iterator ci;
@@ -1080,7 +1091,7 @@ XrlSocketServer::socket4_0_1_send_to_with_flags(const string&	sockid,
 						const bool&	end_of_record,
 						const bool&	end_of_file)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V4Sockets::const_iterator ci;
@@ -1143,7 +1154,7 @@ XrlSocketServer::socket4_0_1_send_from_multicast_if(
 					const vector<uint8_t>& 	data
 					)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -1187,7 +1198,7 @@ XrlSocketServer::socket4_0_1_set_socket_option(const string&	sockid,
 					       const string&	optname,
 					       const uint32_t&	optval)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     const char* o_cstr = optname.c_str();
@@ -1224,7 +1235,7 @@ XrlSocketServer::socket4_0_1_get_socket_option(const string&	sockid,
 					       const string&	optname,
 					       uint32_t&	optval)
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     const char* o_cstr = optname.c_str();
@@ -1264,12 +1275,13 @@ XrlCmdError
 XrlSocketServer::socket6_0_1_tcp_open_and_bind(const string&	creator,
 					       const IPv6&	local_addr,
 					       const uint32_t&	local_port,
+					       const bool&	is_blocking,
 					       string&		sockid)
 {
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -1279,7 +1291,7 @@ XrlSocketServer::socket6_0_1_tcp_open_and_bind(const string&	creator,
 
     in6_addr ia;
     local_addr.copy_out(ia);
-    int fd = comm_bind_tcp6(&ia, htons(local_port));
+    int fd = comm_bind_tcp6(&ia, htons(local_port), is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
@@ -1299,12 +1311,13 @@ XrlCmdError
 XrlSocketServer::socket6_0_1_udp_open_and_bind(const string&	creator,
 					       const IPv6&	local_addr,
 					       const uint32_t&	local_port,
+					       const bool&	is_blocking,
 					       string&		sockid)
 {
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -1314,7 +1327,7 @@ XrlSocketServer::socket6_0_1_udp_open_and_bind(const string&	creator,
 
     in6_addr ia;
     local_addr.copy_out(ia);
-    int fd = comm_bind_udp6(&ia, htons(local_port));
+    int fd = comm_bind_udp6(&ia, htons(local_port), is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
@@ -1338,12 +1351,13 @@ XrlSocketServer::socket6_0_1_udp_open_bind_join(const string&	creator,
 						const IPv6&	mcast_addr,
 						const uint32_t&	ttl,
 						const bool&	reuse,
+						const bool&	is_blocking,
 						string&		sockid)
 {
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -1360,7 +1374,8 @@ XrlSocketServer::socket6_0_1_udp_open_bind_join(const string&	creator,
     in6_addr grp;
     mcast_addr.copy_out(grp);
 
-    int fd = comm_bind_join_udp6(&grp, pif_index, htons(local_port), reuse);
+    int fd = comm_bind_join_udp6(&grp, pif_index, htons(local_port), reuse,
+				 is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
@@ -1395,13 +1410,14 @@ XrlSocketServer::socket6_0_1_tcp_open_bind_connect(
     const uint32_t&	local_port,
     const IPv6&		remote_addr,
     const uint32_t&	remote_port,
+    const bool&		is_blocking,
     string&		sockid
     )
 {
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -1412,14 +1428,15 @@ XrlSocketServer::socket6_0_1_tcp_open_bind_connect(
     in6_addr ia;
     local_addr.copy_out(ia);
 
-    int fd = comm_bind_tcp6(&ia, htons(local_port));
+    int fd = comm_bind_tcp6(&ia, htons(local_port), is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
 
     in6_addr ra;
     remote_addr.copy_out(ra);
-    if (comm_sock_connect6(fd, &ra, htons(remote_port)) != XORP_OK) {
+    if (comm_sock_connect6(fd, &ra, htons(remote_port), is_blocking)
+	!= XORP_OK) {
 	comm_close(fd);
 	return XrlCmdError::COMMAND_FAILED("Connect failed.");
     }
@@ -1443,13 +1460,14 @@ XrlSocketServer::socket6_0_1_udp_open_bind_connect(
     const uint32_t& 	local_port,
     const IPv6&		remote_addr,
     const uint32_t&	remote_port,
+    const bool&		is_blocking,
     string&		sockid
     )
 {
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -1464,7 +1482,8 @@ XrlSocketServer::socket6_0_1_udp_open_bind_connect(
     remote_addr.copy_out(ra);
 
     int fd = comm_bind_connect_udp6(&ia, htons(local_port),
-				    &ra, htons(remote_port));
+				    &ra, htons(remote_port),
+				    is_blocking);
     if (fd <= 0) {
 	return XrlCmdError::COMMAND_FAILED(last_comm_error());
     }
@@ -1489,7 +1508,7 @@ XrlSocketServer::socket6_0_1_udp_join_group(const string&	sockid,
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V6Sockets::const_iterator ci = find_if(_v6sockets.begin(),
@@ -1529,7 +1548,7 @@ XrlSocketServer::socket6_0_1_udp_leave_group(const string&	sockid,
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V6Sockets::const_iterator ci = find_if(_v6sockets.begin(),
@@ -1567,7 +1586,7 @@ XrlSocketServer::socket6_0_1_close(const string& sockid)
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V6Sockets::iterator i6;
@@ -1588,7 +1607,7 @@ XrlSocketServer::socket6_0_1_tcp_listen(const string&	sockid,
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V6Sockets::const_iterator ci;
@@ -1614,7 +1633,7 @@ XrlSocketServer::socket6_0_1_send(
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V6Sockets::const_iterator ci;
@@ -1643,7 +1662,7 @@ XrlSocketServer::socket6_0_1_send_with_flags(
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V6Sockets::const_iterator ci;
@@ -1698,7 +1717,7 @@ XrlSocketServer::socket6_0_1_send_to(const string&		sockid,
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V6Sockets::const_iterator ci;
@@ -1736,7 +1755,7 @@ XrlSocketServer::socket6_0_1_send_to_with_flags(const string&	sockid,
     if (comm_ipv6_present() != XORP_OK)
 	return XrlCmdError::COMMAND_FAILED(NO_IPV6_MSG);
 
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     V6Sockets::const_iterator ci;
@@ -1800,7 +1819,7 @@ XrlSocketServer::socket6_0_1_send_from_multicast_if(
 					)
 {
 #ifdef HAVE_IPV6
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     string err;
@@ -1854,7 +1873,7 @@ XrlSocketServer::socket6_0_1_set_socket_option(const string&	sockid,
 					       const uint32_t&	optval)
 {
 #ifdef HAVE_IPV6
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     const char* o_cstr = optname.c_str();
@@ -1898,7 +1917,7 @@ XrlSocketServer::socket6_0_1_get_socket_option(const string&	sockid,
 					       uint32_t&	optval)
 {
 #ifdef HAVE_IPV6
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return XrlCmdError::COMMAND_FAILED(NOT_RUNNING_MSG);
 
     const char* o_cstr = optname.c_str();
@@ -1993,6 +2012,6 @@ void
 XrlSocketServer::xrl_router_ready(const string& iname)
 {
     if (iname == _r->instance_name())
-	set_status(RUNNING);
+	set_status(SERVICE_RUNNING);
 }
 
