@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,38 +12,52 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/xrl_target.cc,v 1.43 2004/06/10 22:40:59 hodson Exp $"
+#ident "$XORP: xorp/fea/xrl_target.cc,v 1.71 2005/03/25 02:53:17 pavlin Exp $"
+
+#define PROFILE_UTILS_REQUIRED
 
-#include "config.h"
 #include "fea_module.h"
 
 #include "libxorp/xorp.h"
-
-#include "net/if.h"			/* Included for IFF_ definitions */
-
-#include "libxorp/debug.h"
 #include "libxorp/xlog.h"
+#include "libxorp/debug.h"
 #include "libxorp/status_codes.h"
 #include "libxorp/eventloop.hh"
+
 #include "libxipc/xrl_std_router.hh"
 
+#include "xrl/interfaces/profile_client_xif.hh"
+//
+// XXX: file "libxorp/profile.hh" must be included after
+// "libxipc/xrl_std_router.hh" and "xrl/interfaces/profile_client_xif.hh"
+// Sigh...
+//
+#include "libxorp/profile.hh"
+
+
 #include "fticonfig.hh"
 #include "libfeaclient_bridge.hh"
 #include "xrl_ifupdate.hh"
 #include "xrl_rawsock4.hh"
+#include "xrl_rawsock6.hh"
 #include "xrl_socket_server.hh"
 #include "xrl_target.hh"
+#include "profile_vars.hh"
 
 XrlFeaTarget::XrlFeaTarget(EventLoop&		 	e,
 			   XrlRouter&		 	r,
 			   FtiConfig&  		 	ftic,
 			   InterfaceManager&	 	ifmgr,
 			   XrlIfConfigUpdateReporter&	xifcur,
-			   XrlRawSocket4Manager*	xrsm,
+			   Profile&			profile,
+			   XrlRawSocket4Manager*	xrsm4,
+			   XrlRawSocket6Manager*	xrsm6,
 			   LibFeaClientBridge*		lfcb,
 			   XrlSocketServer*		xss)
-    : XrlFeaTargetBase(&r), _xftm(e, ftic, &r), _xifmgr(e, ifmgr),
-      _xifcur(xifcur), _xrsm(xrsm), _lfcb(lfcb), _xss(xss), _done(false),
+    : XrlFeaTargetBase(&r), _xrl_router(r), _xftm(e, ftic, &r),
+      _xifmgr(e, ifmgr), _xifcur(xifcur), _profile(profile),
+      _xrsm4(xrsm4), _xrsm6(xrsm6),
+      _lfcb(lfcb), _xss(xss), _done(false),
       _have_ipv4(false), _have_ipv6(false)
 {
     _have_ipv4 = _xftm.ftic().have_ipv4();
@@ -120,6 +134,373 @@ XrlFeaTarget::common_0_1_shutdown()
     return XrlCmdError::OKAY();
 }
 
+/**
+ *  Enable/disable Click FEA support.
+ *
+ *  @param enable if true, then enable the Click FEA support, otherwise
+ *  disable it.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_enable_click(
+    // Input values,
+    const bool&	enable)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.enable_click(enable);
+    ftic.enable_click(enable);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Start Click FEA support.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_start_click()
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+    string error_msg;
+
+    if (ifc.start_click(error_msg) < 0) {
+	return XrlCmdError::COMMAND_FAILED(error_msg);
+    }
+    if (ftic.start_click(error_msg) < 0) {
+	string dummy_msg;
+	ifc.stop_click(error_msg);
+	return XrlCmdError::COMMAND_FAILED(error_msg);
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Stop Click FEA support.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_stop_click()
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+    string error_msg1, error_msg2;
+
+    if (ftic.stop_click(error_msg1) < 0) {
+	ifc.stop_click(error_msg2);
+	return XrlCmdError::COMMAND_FAILED(error_msg1);
+    }
+    if (ifc.stop_click(error_msg1) < 0) {
+	return XrlCmdError::COMMAND_FAILED(error_msg1);
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Enable/disable duplicating the Click routes to the system kernel.
+ *
+ *  @param enable if true, then enable duplicating the Click routes to the
+ *  system kernel, otherwise disable it.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_enable_duplicate_routes_to_kernel(
+    // Input values,
+    const bool&	enable)
+{
+    FtiConfig& ftic = _xftm.ftic();
+
+    ftic.enable_duplicate_routes_to_kernel(enable);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Enable/disable kernel-level Click FEA support.
+ *
+ *  @param enable if true, then enable the kernel-level Click FEA support,
+ *  otherwise disable it.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_enable_kernel_click(
+    // Input values,
+    const bool&	enable)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.enable_kernel_click(enable);
+    ftic.enable_kernel_click(enable);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Enable/disable installing kernel-level Click on startup.
+ *
+ *  @param enable if true, then install kernel-level Click on startup.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_enable_kernel_click_install_on_startup(
+    // Input values,
+    const bool&	enable)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.enable_kernel_click_install_on_startup(enable);
+    ftic.enable_kernel_click_install_on_startup(enable);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify the list of kernel Click modules to load on startup if
+ *  installing kernel-level Click on startup is enabled. The file names of
+ *  the kernel modules are separated by colon.
+ *
+ *  @param modules the list of kernel Click modules (separated by colon) to
+ *  load.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_kernel_click_modules(
+    // Input values,
+    const string&	modules)
+{
+    list<string> modules_list;
+
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    //
+    // Split the string with the names of the modules (separated by colon)
+    //
+    string modules_tmp = modules;
+    string::size_type colon;
+    string name;
+    do {
+	if (modules_tmp.empty())
+	    break;
+	colon = modules_tmp.find(':');
+	name = modules_tmp.substr(0, colon);
+	if (colon != string::npos)
+	    modules_tmp = modules_tmp.substr(colon + 1);
+	else
+	    modules_tmp.erase();
+	if (! name.empty())
+	    modules_list.push_back(name);
+    } while (true);
+
+    ifc.set_kernel_click_modules(modules_list);
+    ftic.set_kernel_click_modules(modules_list);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify the kernel-level Click mount directory.
+ *
+ *  @param directory the kernel-level Click mount directory.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_kernel_click_mount_directory(
+    // Input values,
+    const string&	directory)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.set_kernel_click_mount_directory(directory);
+    ftic.set_kernel_click_mount_directory(directory);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify the external program to generate the kernel-level Click
+ *  configuration.
+ *
+ *  @param kernel_click_config_generator_file the name of the external
+ *  program to generate the kernel-level Click configuration.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_kernel_click_config_generator_file(
+    // Input values,
+    const string&	kernel_click_config_generator_file)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.set_kernel_click_config_generator_file(kernel_click_config_generator_file);
+    ftic.set_kernel_click_config_generator_file(kernel_click_config_generator_file);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Enable/disable user-level Click FEA support.
+ *
+ *  @param enable if true, then enable the user-level Click FEA support,
+ *  otherwise disable it.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_enable_user_click(
+    // Input values,
+    const bool&	enable)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.enable_user_click(enable);
+    ftic.enable_user_click(enable);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify the user-level Click command file.
+ *
+ *  @param user_click_command_file the name of the user-level Click command
+ *  file.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_user_click_command_file(
+    // Input values,
+    const string&	user_click_command_file)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.set_user_click_command_file(user_click_command_file);
+    ftic.set_user_click_command_file(user_click_command_file);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify the extra arguments to the user-level Click command.
+ *
+ *  @param user_click_command_extra_arguments the extra arguments to the
+ *  user-level Click command.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_user_click_command_extra_arguments(
+    // Input values,
+    const string&	user_click_command_extra_arguments)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.set_user_click_command_extra_arguments(user_click_command_extra_arguments);
+    ftic.set_user_click_command_extra_arguments(user_click_command_extra_arguments);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify whether to execute on startup the user-level Click command.
+ *
+ *  @param user_click_command_execute_on_startup if true, then execute the
+ *  user-level Click command on startup.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_user_click_command_execute_on_startup(
+    // Input values,
+    const bool&	user_click_command_execute_on_startup)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.set_user_click_command_execute_on_startup(user_click_command_execute_on_startup);
+    ftic.set_user_click_command_execute_on_startup(user_click_command_execute_on_startup);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify the address to use for control access to the user-level
+ *  Click.
+ *
+ *  @param user_click_control_address the address to use for
+ *  control access to the user-level Click.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_user_click_control_address(
+    // Input values,
+    const IPv4&	user_click_control_address)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.set_user_click_control_address(user_click_control_address);
+    ftic.set_user_click_control_address(user_click_control_address);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify the socket port to use for control access to the user-level
+ *  Click.
+ *
+ *  @param user_click_control_socket_port the socket port to use for
+ *  control access to the user-level Click.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_user_click_control_socket_port(
+    // Input values,
+    const uint32_t&	user_click_control_socket_port)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.set_user_click_control_socket_port(user_click_control_socket_port);
+    ftic.set_user_click_control_socket_port(user_click_control_socket_port);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify the configuration file to be used by user-level Click on
+ *  startup.
+ *
+ *  @param user_click_startup_config_file the name of the configuration
+ *  file to be used by user-level Click on startup.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_user_click_startup_config_file(
+    // Input values,
+    const string&	user_click_startup_config_file)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.set_user_click_startup_config_file(user_click_startup_config_file);
+    ftic.set_user_click_startup_config_file(user_click_startup_config_file);
+
+    return XrlCmdError::OKAY();
+}
+
+/**
+ *  Specify the external program to generate the user-level Click
+ *  configuration.
+ *
+ *  @param user_click_config_generator_file the name of the external
+ *  program to generate the user-level Click configuration.
+ */
+XrlCmdError
+XrlFeaTarget::fea_click_0_1_set_user_click_config_generator_file(
+    // Input values,
+    const string&	user_click_config_generator_file)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    FtiConfig& ftic = _xftm.ftic();
+
+    ifc.set_user_click_config_generator_file(user_click_config_generator_file);
+    ftic.set_user_click_config_generator_file(user_click_config_generator_file);
+
+    return XrlCmdError::OKAY();
+}
+
 /**
  *  Add a FIB client.
  *
@@ -128,23 +509,27 @@ XrlFeaTarget::common_0_1_shutdown()
 XrlCmdError
 XrlFeaTarget::fea_fib_0_1_add_fib_client4(
     // Input values,
-    const string&	target_name)
+    const string&	target_name,
+    const bool&		send_updates,
+    const bool&		send_resolves)
 {
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
-    return _xftm.add_fib_client4(target_name);
+    return _xftm.add_fib_client4(target_name, send_updates, send_resolves);
 }
 
 XrlCmdError
 XrlFeaTarget::fea_fib_0_1_add_fib_client6(
     // Input values,
-    const string&	target_name)
+    const string&	target_name,
+    const bool&		send_updates,
+    const bool&		send_resolves)
 {
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
-    return _xftm.add_fib_client6(target_name);
+    return _xftm.add_fib_client6(target_name, send_updates, send_resolves);
 }
 
 /**
@@ -346,6 +731,25 @@ XrlFeaTarget::ifmgr_0_1_get_system_interface_enabled(
     return e;
 }
 
+XrlCmdError
+XrlFeaTarget::ifmgr_0_1_get_system_interface_discard(
+					      // Input values,
+					      const string&	ifname,
+					      // Output values,
+					      bool&		discard)
+{
+    const IfTreeInterface* fi = 0;
+    // XXX: We assume that the discard property exists wholly in the FEA,
+    // and that it is never set by the underlying network stack; therefore
+    // we never 'pull' it.
+    XrlCmdError e = _xifmgr.get_if(ifname, fi);
+
+    if (e == XrlCmdError::OKAY())
+	discard = fi->discard();
+
+    return e;
+}
+
 XrlCmdError
 XrlFeaTarget::ifmgr_0_1_get_configured_interface_enabled(
 					      // Input values,
@@ -362,6 +766,22 @@ XrlFeaTarget::ifmgr_0_1_get_configured_interface_enabled(
     return e;
 }
 
+XrlCmdError
+XrlFeaTarget::ifmgr_0_1_get_configured_interface_discard(
+					      // Input values,
+					      const string&	ifname,
+					      // Output values,
+					      bool&		discard)
+{
+    const IfTreeInterface* fi = 0;
+    XrlCmdError e = _xifmgr.get_if(ifname, fi);
+
+    if (e == XrlCmdError::OKAY())
+	discard = fi->discard();
+
+    return e;
+}
+
 XrlCmdError
 XrlFeaTarget::ifmgr_0_1_get_system_mac(
 				// Input values,
@@ -949,6 +1369,18 @@ XrlFeaTarget::ifmgr_0_1_set_interface_enabled(
 }
 
 
+XrlCmdError
+XrlFeaTarget::ifmgr_0_1_set_interface_discard(
+					      // Input values,
+					      const uint32_t&	tid,
+					      const string&	ifname,
+					      const bool&	discard)
+{
+    IfTree& it = _xifmgr.iftree();
+    return _xifmgr.add(tid, new SetInterfaceDiscard(it, ifname, discard));
+}
+
+
 XrlCmdError
 XrlFeaTarget::ifmgr_0_1_set_mac(
 				// Input values,
@@ -1342,183 +1774,12 @@ XrlFeaTarget::ifmgr_replicator_0_1_unregister_ifmgr_mirror(
 // FTI Related
 
 XrlCmdError
-XrlFeaTarget::fti_0_2_start_transaction(
-	// Output values,
-	uint32_t& tid)
-{
-    return _xftm.start_transaction(tid);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_commit_transaction(
-	// Input values,
-	const uint32_t&	tid)
-{
-    return _xftm.commit_transaction(tid);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_abort_transaction(
-	// Input values,
-	const uint32_t&	tid)
-{
-    return _xftm.abort_transaction(tid);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_add_entry4(
-	// Input values,
-	const uint32_t&	tid,
-	const IPv4Net&	dst,
-	const IPv4&	gateway,
-	const string&	ifname,
-	const string&	vifname,
-	const uint32_t&	metric,
-	const uint32_t& admin_distance,
-	const string&	protocol_origin)
-{
-    if (! have_ipv4())
-	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
-
-    //
-    // XXX: don't add routes for directly-connected subnets,
-    // because that should be managed by the underlying system.
-    //
-    if (protocol_origin == "connected")
-	return XrlCmdError::OKAY();
-
-    // FtiTransactionManager::Operation is a ref_ptr object, allocated
-    // memory here is handed it to to manage.
-    FtiTransactionManager::Operation op(
-	new FtiAddEntry4(_xftm.ftic(), dst, gateway, ifname, vifname, metric,
-			 admin_distance)
-	);
-    return _xftm.add(tid, op);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_add_entry6(
-	// Input values,
-	const uint32_t&	tid,
-	const IPv6Net&	dst,
-	const IPv6&	gateway,
-	const string&	ifname,
-	const string&	vifname,
-	const uint32_t&	metric,
-	const uint32_t& admin_distance,
-	const string&	protocol_origin)
-{
-    if (! have_ipv6())
-	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
-
-    //
-    // XXX: don't add routes for directly-connected subnets,
-    // because that should be managed by the underlying system.
-    //
-    if (protocol_origin == "connected")
-	return XrlCmdError::OKAY();
-
-    // FtiTransactionManager::Operation is a ref_ptr object, allocated
-    // memory here is handed it to to manage.
-    FtiTransactionManager::Operation op(
-	new FtiAddEntry6(_xftm.ftic(), dst, gateway, ifname, vifname, metric,
-			 admin_distance)
-	);
-
-    return _xftm.add(tid, op);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_delete_entry4(
-	// Input values,
-	const uint32_t&	tid,
-	const IPv4Net&	dst)
-{
-    if (! have_ipv4())
-	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
-
-    // FtiTransactionManager::Operation is a ref_ptr object, allocated
-    // memory here is handed it to to manage.
-    FtiTransactionManager::Operation op(
-	new FtiDeleteEntry4(_xftm.ftic(), dst)
-	);
-    return _xftm.add(tid, op);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_delete_entry6(
-	// Input values,
-	const uint32_t&	tid,
-	const IPv6Net&	dst)
-{
-    if (! have_ipv6())
-	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
-
-    // FtiTransactionManager::Operation is a ref_ptr object, allocated
-    // memory here is handed it to to manage.
-    FtiTransactionManager::Operation op(
-	new FtiDeleteEntry6(_xftm.ftic(), dst)
-	);
-
-    return _xftm.add(tid, op);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_delete_all_entries(
-	// Input values,
-	const uint32_t&	tid)
-{
-    // FtiTransactionManager::Operation is a ref_ptr object, allocated
-    // memory here is handed it to to manage.
-
-    FtiTransactionManager::Operation op(
-	new FtiDeleteAllEntries(_xftm.ftic())
-	);
-    return _xftm.add(tid, op);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_delete_all_entries4(
-	// Input values,
-	const uint32_t&	tid)
-{
-    if (! have_ipv4())
-	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
-
-    // FtiTransactionManager::Operation is a ref_ptr object, allocated
-    // memory here is handed it to to manage.
-
-    FtiTransactionManager::Operation op(
-	new FtiDeleteAllEntries4(_xftm.ftic())
-	);
-    return _xftm.add(tid, op);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_delete_all_entries6(
-	// Input values,
-	const uint32_t&	tid)
-{
-    if (! have_ipv6())
-	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
-
-    // FtiTransactionManager::Operation is a ref_ptr object, allocated
-    // memory here is handed it to to manage.
-
-    FtiTransactionManager::Operation op(
-	new FtiDeleteAllEntries6(_xftm.ftic())
-	);
-
-    return _xftm.add(tid, op);
-}
-
-XrlCmdError
-XrlFeaTarget::fti_0_2_lookup_route4(
+XrlFeaTarget::fti_0_2_lookup_route_by_dest4(
 	// Input values,
 	const IPv4&	dst,
 	// Output values,
 	IPv4Net&	netmask,
-	IPv4&		gateway,
+	IPv4&		nexthop,
 	string&		ifname,
 	string&		vifname,
 	uint32_t&	metric,
@@ -1529,15 +1790,14 @@ XrlFeaTarget::fti_0_2_lookup_route4(
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
     Fte4 fte;
-    if (_xftm.ftic().lookup_route4(dst, fte) == true) {
+    if (_xftm.ftic().lookup_route_by_dest4(dst, fte) == true) {
 	netmask = fte.net();
-	gateway = fte.gateway();
+	nexthop = fte.nexthop();
 	ifname = fte.ifname();
 	vifname = fte.vifname();
-	// TODO: set the values of metric, admin_distance and protocol_origin
-	// to something meaningful
-	metric = ~0;
-	admin_distance = ~0;
+	metric = fte.metric();
+	admin_distance = fte.admin_distance();
+	// TODO: set the value of protocol_origin to something meaningful
 	protocol_origin = "NOT_SUPPORTED";
 	return XrlCmdError::OKAY();
     }
@@ -1545,12 +1805,12 @@ XrlFeaTarget::fti_0_2_lookup_route4(
 }
 
 XrlCmdError
-XrlFeaTarget::fti_0_2_lookup_route6(
+XrlFeaTarget::fti_0_2_lookup_route_by_dest6(
 	// Input values,
 	const IPv6&	dst,
 	// Output values,
 	IPv6Net&	netmask,
-	IPv6&		gateway,
+	IPv6&		nexthop,
 	string&		ifname,
 	string&		vifname,
 	uint32_t&	metric,
@@ -1561,15 +1821,14 @@ XrlFeaTarget::fti_0_2_lookup_route6(
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
     Fte6 fte;
-    if (_xftm.ftic().lookup_route6(dst, fte) == true) {
+    if (_xftm.ftic().lookup_route_by_dest6(dst, fte) == true) {
 	netmask = fte.net();
-	gateway = fte.gateway();
+	nexthop = fte.nexthop();
 	ifname = fte.ifname();
 	vifname = fte.vifname();
-	// TODO: set the values of metric, admin_distance and protocol_origin
-	// to something meaningful
-	metric = ~0;
-	admin_distance = ~0;
+	metric = fte.metric();
+	admin_distance = fte.admin_distance();
+	// TODO: set the value of protocol_origin to something meaningful
 	protocol_origin = "NOT_SUPPORTED";
 	return XrlCmdError::OKAY();
     }
@@ -1577,11 +1836,11 @@ XrlFeaTarget::fti_0_2_lookup_route6(
 }
 
 XrlCmdError
-XrlFeaTarget::fti_0_2_lookup_entry4(
+XrlFeaTarget::fti_0_2_lookup_route_by_network4(
 	// Input values,
 	const IPv4Net&	dst,
 	// Output values,
-	IPv4&		gateway,
+	IPv4&		nexthop,
 	string&		ifname,
 	string&		vifname,
 	uint32_t&	metric,
@@ -1592,14 +1851,13 @@ XrlFeaTarget::fti_0_2_lookup_entry4(
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
     Fte4 fte;
-    if (_xftm.ftic().lookup_entry4(dst, fte)) {
-	gateway = fte.gateway();
+    if (_xftm.ftic().lookup_route_by_network4(dst, fte)) {
+	nexthop = fte.nexthop();
 	ifname = fte.ifname();
 	vifname = fte.vifname();
-	// TODO: set the values of metric, admin_distance and protocol_origin
-	// to something meaningful
-	metric = ~0;
-	admin_distance = ~0;
+	metric = fte.metric();
+	admin_distance = fte.admin_distance();
+	// TODO: set the value of protocol_origin to something meaningful
 	protocol_origin = "NOT_SUPPORTED";
 	return XrlCmdError::OKAY();
     }
@@ -1607,11 +1865,11 @@ XrlFeaTarget::fti_0_2_lookup_entry4(
 }
 
 XrlCmdError
-XrlFeaTarget::fti_0_2_lookup_entry6(
+XrlFeaTarget::fti_0_2_lookup_route_by_network6(
 	// Input values,
 	const IPv6Net&	dst,
 	// Output values,
-	IPv6&		gateway,
+	IPv6&		nexthop,
 	string&		ifname,
 	string&		vifname,
 	uint32_t&	metric,
@@ -1622,14 +1880,13 @@ XrlFeaTarget::fti_0_2_lookup_entry6(
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
     Fte6 fte;
-    if (_xftm.ftic().lookup_entry6(dst, fte)) {
-	gateway = fte.gateway();
+    if (_xftm.ftic().lookup_route_by_network6(dst, fte)) {
+	nexthop = fte.nexthop();
 	ifname = fte.ifname();
 	vifname = fte.vifname();
-	// TODO: set the values of metric, admin_distance and protocol_origin
-	// to something meaningful
-	metric = ~0;
-	admin_distance = ~0;
+	metric = fte.metric();
+	admin_distance = fte.admin_distance();
+	// TODO: set the value of protocol_origin to something meaningful
 	protocol_origin = "NOT_SUPPORTED";
 	return XrlCmdError::OKAY();
     }
@@ -1741,28 +1998,45 @@ XrlFeaTarget::redist_transaction4_0_1_add_route(
     // Input values,
     const uint32_t&	tid,
     const IPv4Net&	dst,
-    const IPv4&		nh,
+    const IPv4&		nexthop,
     const string&	ifname,
     const string&	vifname,
     const uint32_t&	metric,
-    const uint32_t&	ad,
+    const uint32_t&	admin_distance,
     const string&	cookie,
     const string&	protocol_origin)
 {
+    bool is_xorp_route;
+    bool is_connected_route = false;
+
+    debug_msg("redist_transaction4_0_1_add_route(): "
+	      "dst = %s nexthop = %s ifname = %s vifname = %s "
+	      "metric = %u admin_distance = %u protocol_origin = %s\n",
+	      dst.str().c_str(),
+	      nexthop.str().c_str(),
+	      ifname.c_str(),
+	      vifname.c_str(),
+	      metric,
+	      admin_distance,
+	      protocol_origin.c_str());
+
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
-    //
-    // XXX: don't add routes for directly-connected subnets,
-    // because that should be managed by the underlying system.
-    //
+    is_xorp_route = true;	// XXX: unconditionally set to true
+
+    // TODO: XXX: get rid of the hard-coded "connected" string here
     if (protocol_origin == "connected")
-	return XrlCmdError::OKAY();
+	is_connected_route = true;
+
+    if (_profile.enabled(profile_route_in))
+	_profile.log(profile_route_in, c_format("add %s", dst.str().c_str()));
 
     // FtiTransactionManager::Operation is a ref_ptr object, allocated
     // memory here is handed it to to manage.
     FtiTransactionManager::Operation op(
-	new FtiAddEntry4(_xftm.ftic(), dst, nh, ifname, vifname, metric, ad)
+	new FtiAddEntry4(_xftm.ftic(), dst, nexthop, ifname, vifname, metric,
+			 admin_distance, is_xorp_route, is_connected_route)
 	);
     return _xftm.add(tid, op);
 
@@ -1774,15 +2048,51 @@ XrlFeaTarget::redist_transaction4_0_1_delete_route(
     // Input values,
     const uint32_t&	tid,
     const IPv4Net&	network,
-    const string&	cookie)
+    const string&	cookie,
+    const string&	protocol_origin)
+{
+    bool is_connected_route = false;
+
+    debug_msg("redist_transaction4_0_1_delete_route(): "
+	      "network = %s protocol_origin = %s\n",
+	      network.str().c_str(),
+	      protocol_origin.c_str());
+
+    if (! have_ipv4())
+	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
+
+    // TODO: XXX: get rid of the hard-coded "connected" string here
+    if (protocol_origin == "connected")
+	is_connected_route = true;
+
+    if (_profile.enabled(profile_route_in))
+	_profile.log(profile_route_in,
+		     c_format("delete %s", network.str().c_str()));
+
+    // FtiTransactionManager::Operation is a ref_ptr object, allocated
+    // memory here is handed it to to manage.
+    FtiTransactionManager::Operation op(
+	new FtiDeleteEntry4(_xftm.ftic(), network, is_connected_route)
+	);
+    return _xftm.add(tid, op);
+
+    UNUSED(cookie);
+}
+
+XrlCmdError
+XrlFeaTarget::redist_transaction4_0_1_delete_all_routes(
+	// Input values,
+	const uint32_t&	tid,
+	const string&	cookie)
 {
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
     // FtiTransactionManager::Operation is a ref_ptr object, allocated
     // memory here is handed it to to manage.
+
     FtiTransactionManager::Operation op(
-	new FtiDeleteEntry4(_xftm.ftic(), network)
+	new FtiDeleteAllEntries4(_xftm.ftic())
 	);
     return _xftm.add(tid, op);
 
@@ -1818,28 +2128,45 @@ XrlFeaTarget::redist_transaction6_0_1_add_route(
     // Input values,
     const uint32_t&	tid,
     const IPv6Net&	dst,
-    const IPv6&		nh,
+    const IPv6&		nexthop,
     const string&	ifname,
     const string&	vifname,
     const uint32_t&	metric,
-    const uint32_t&	ad,
+    const uint32_t&	admin_distance,
     const string&	cookie,
     const string&	protocol_origin)
 {
+    bool is_xorp_route;
+    bool is_connected_route = false;
+
+    debug_msg("redist_transaction6_0_1_add_route(): "
+	      "dst = %s nexthop = %s ifname = %s vifname = %s "
+	      "metric = %u admin_distance = %u protocol_origin = %s\n",
+	      dst.str().c_str(),
+	      nexthop.str().c_str(),
+	      ifname.c_str(),
+	      vifname.c_str(),
+	      metric,
+	      admin_distance,
+	      protocol_origin.c_str());
+
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
-    //
-    // XXX: don't add routes for directly-connected subnets,
-    // because that should be managed by the underlying system.
-    //
+    is_xorp_route = true;	// XXX: unconditionally set to true
+
+    // TODO: XXX: get rid of the hard-coded "connected" string here
     if (protocol_origin == "connected")
-	return XrlCmdError::OKAY();
+	is_connected_route = true;
+
+    if (_profile.enabled(profile_route_in))
+	_profile.log(profile_route_in, c_format("add %s", dst.str().c_str()));
 
     // FtiTransactionManager::Operation is a ref_ptr object, allocated
     // memory here is handed it to to manage.
     FtiTransactionManager::Operation op(
-	new FtiAddEntry6(_xftm.ftic(), dst, nh, ifname, vifname, metric, ad)
+	new FtiAddEntry6(_xftm.ftic(), dst, nexthop, ifname, vifname, metric,
+			 admin_distance, is_xorp_route, is_connected_route)
 	);
     return _xftm.add(tid, op);
 
@@ -1851,15 +2178,51 @@ XrlFeaTarget::redist_transaction6_0_1_delete_route(
     // Input values,
     const uint32_t&	tid,
     const IPv6Net&	network,
-    const string&	cookie)
+    const string&	cookie,
+    const string&	protocol_origin)
+{
+    bool is_connected_route = false;
+
+    debug_msg("redist_transaction6_0_1_delete_route(): "
+	      "network = %s protocol_origin = %s\n",
+	      network.str().c_str(),
+	      protocol_origin.c_str());
+
+    if (! have_ipv6())
+	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
+
+    // TODO: XXX: get rid of the hard-coded "connected" string here
+    if (protocol_origin == "connected")
+	is_connected_route = true;
+
+    if (_profile.enabled(profile_route_in))
+	_profile.log(profile_route_in,
+		     c_format("delete %s", network.str().c_str()));
+
+    // FtiTransactionManager::Operation is a ref_ptr object, allocated
+    // memory here is handed it to to manage.
+    FtiTransactionManager::Operation op(
+	new FtiDeleteEntry6(_xftm.ftic(), network, is_connected_route)
+	);
+    return _xftm.add(tid, op);
+
+    UNUSED(cookie);
+}
+
+XrlCmdError
+XrlFeaTarget::redist_transaction6_0_1_delete_all_routes(
+	// Input values,
+	const uint32_t&	tid,
+	const string&	cookie)
 {
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
     // FtiTransactionManager::Operation is a ref_ptr object, allocated
     // memory here is handed it to to manage.
+
     FtiTransactionManager::Operation op(
-	new FtiDeleteEntry6(_xftm.ftic(), network)
+	new FtiDeleteAllEntries6(_xftm.ftic())
 	);
     return _xftm.add(tid, op);
 
@@ -1867,12 +2230,12 @@ XrlFeaTarget::redist_transaction6_0_1_delete_route(
 }
 
 // ----------------------------------------------------------------------------
-// Raw Socket related
+// IPv4 Raw Socket related
 
-static const char* XRL_RAW_SOCKET_NULL = "XrlRawSocket4Manager not present" ;
+static const char* XRL_RAW_SOCKET4_NULL = "XrlRawSocket4Manager not present" ;
 
 XrlCmdError
-XrlFeaTarget::raw_packet_0_1_send4(
+XrlFeaTarget::raw_packet4_0_1_send(
 				   // Input values,
 				   const IPv4&		  src_address,
 				   const IPv4&		  dst_address,
@@ -1887,15 +2250,15 @@ XrlFeaTarget::raw_packet_0_1_send4(
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
-    if (_xrsm == 0) {
-	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET_NULL);
+    if (_xrsm4 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET4_NULL);
     }
-    return _xrsm->send(src_address, dst_address, vifname,
+    return _xrsm4->send(src_address, dst_address, vifname,
 		       proto, ttl, tos, options, payload);
 }
 
 XrlCmdError
-XrlFeaTarget::raw_packet_0_1_send_raw4(
+XrlFeaTarget::raw_packet4_0_1_send_raw(
 				       // Input values,
 				       const string&		vifname,
 				       const vector<uint8_t>&	packet
@@ -1904,14 +2267,14 @@ XrlFeaTarget::raw_packet_0_1_send_raw4(
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
-    if (_xrsm == 0) {
-	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET_NULL);
+    if (_xrsm4 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET4_NULL);
     }
-    return _xrsm->send(vifname, packet);
+    return _xrsm4->send(vifname, packet);
 }
 
 XrlCmdError
-XrlFeaTarget::raw_packet_0_1_register_vif_receiver(
+XrlFeaTarget::raw_packet4_0_1_register_vif_receiver(
 						   // Input values,
 						   const string&   router_name,
 						   const string&   ifname,
@@ -1919,14 +2282,14 @@ XrlFeaTarget::raw_packet_0_1_register_vif_receiver(
 						   const uint32_t& proto
 						   )
 {
-    if (_xrsm == 0) {
-	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET_NULL);
+    if (_xrsm4 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET4_NULL);
     }
-    return _xrsm->register_vif_receiver(router_name, ifname, vifname, proto);
+    return _xrsm4->register_vif_receiver(router_name, ifname, vifname, proto);
 }
 
 XrlCmdError
-XrlFeaTarget::raw_packet_0_1_unregister_vif_receiver(
+XrlFeaTarget::raw_packet4_0_1_unregister_vif_receiver(
 						     // Input values,
 						     const string& router_name,
 						     const string& ifname,
@@ -1934,12 +2297,73 @@ XrlFeaTarget::raw_packet_0_1_unregister_vif_receiver(
 						     const uint32_t& proto
 						     )
 {
-    if (_xrsm == 0) {
-	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET_NULL);
+    if (_xrsm4 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET4_NULL);
     }
-    return _xrsm->unregister_vif_receiver(router_name, ifname, vifname, proto);
+    return _xrsm4->unregister_vif_receiver(router_name, ifname, vifname, proto);
 }
 
+// ----------------------------------------------------------------------------
+// IPv6 Raw Socket related
+
+static const char* XRL_RAW_SOCKET6_NULL = "XrlRawSocket6Manager not present" ;
+
+XrlCmdError
+XrlFeaTarget::raw_packet6_0_1_send_raw(
+				// Input values,
+				const IPv6&	src_address,
+				const IPv6&	dst_address,
+				const string&	vif_name,
+				const uint32_t&	proto,
+				const uint32_t&	tclass,
+				const uint32_t&	hoplimit,
+				const vector<uint8_t>&	hopopts,
+				const vector<uint8_t>&	packet)
+{
+
+    if (! have_ipv6())
+	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
+
+    if (_xrsm6 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET6_NULL);
+    }
+    return _xrsm6->send(src_address, dst_address, vif_name, proto,
+			tclass, hoplimit, hopopts, packet);
+}
+
+XrlCmdError
+XrlFeaTarget::raw_packet6_0_1_register_vif_receiver(
+						   // Input values,
+						   const string&   router_name,
+						   const string&   ifname,
+						   const string&   vifname,
+						   const uint32_t& proto
+						   )
+{
+    if (_xrsm6 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET6_NULL);
+    }
+    return _xrsm6->register_vif_receiver(router_name, ifname, vifname, proto);
+}
+
+XrlCmdError
+XrlFeaTarget::raw_packet6_0_1_unregister_vif_receiver(
+						     // Input values,
+						     const string& router_name,
+						     const string& ifname,
+						     const string& vifname,
+						     const uint32_t& proto
+						     )
+{
+    if (_xrsm6 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET6_NULL);
+    }
+    return _xrsm6->unregister_vif_receiver(router_name, ifname, vifname, proto);
+}
+
+// ----------------------------------------------------------------------------
+// Socket Server related
+
 XrlCmdError
 XrlFeaTarget::socket4_locator_0_1_find_socket_server_for_addr(
 							      const IPv4& addr,
@@ -1955,7 +2379,7 @@ XrlFeaTarget::socket4_locator_0_1_find_socket_server_for_addr(
     if (_xss == 0) {
 	return XrlCmdError::COMMAND_FAILED("Socket Server is not present.");
     }
-    if (_xss->status() != RUNNING) {
+    if (_xss->status() != SERVICE_RUNNING) {
 	return XrlCmdError::COMMAND_FAILED("Socket Server not running.");
     }
     UNUSED(addr);
@@ -1980,10 +2404,91 @@ XrlFeaTarget::socket6_locator_0_1_find_socket_server_for_addr(
     if (_xss == 0) {
 	return XrlCmdError::COMMAND_FAILED("Socket Server is not present.");
     }
-    if (_xss->status() != RUNNING) {
+    if (_xss->status() != SERVICE_RUNNING) {
 	return XrlCmdError::COMMAND_FAILED("Socket Server not running.");
     }
 
     svr = _xss->instance_name();
     return XrlCmdError::OKAY();
 }
+
+// ----------------------------------------------------------------------------
+// Profiling related
+
+XrlCmdError
+XrlFeaTarget::profile_0_1_enable(const string& pname)
+{
+    debug_msg("enable profile variable %s\n", pname.c_str());
+
+    try {
+	_profile.enable(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlFeaTarget::profile_0_1_disable(const string&	pname)
+{
+    debug_msg("disable profile variable %s\n", pname.c_str());
+
+    try {
+	_profile.disable(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlFeaTarget::profile_0_1_get_entries(const string& pname,
+				      const string& instance_name)
+{
+    debug_msg("get profile variable %s instance %s\n", pname.c_str(),
+	      instance_name.c_str());
+
+    // Lock and initialize.
+    try {
+	_profile.lock_log(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    ProfileUtils::transmit_log(pname,
+			       dynamic_cast<XrlStdRouter *>(&_xrl_router),
+			       instance_name, &_profile);
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlFeaTarget::profile_0_1_clear(const string& pname)
+{
+    debug_msg("clear profile variable %s\n", pname.c_str());
+
+    try {
+	_profile.clear(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlFeaTarget::profile_0_1_list(string& info)
+{
+    debug_msg("\n");
+    
+    info = _profile.list();
+    return XrlCmdError::OKAY();
+}
