@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fib2mrib/fib2mrib_node.cc,v 1.11 2004/06/10 22:41:01 hodson Exp $"
-
+#ident "$XORP: xorp/fib2mrib/fib2mrib_node.cc,v 1.22 2005/04/02 11:48:24 pavlin Exp $"
 
 //
 // Fib2mrib node implementation.
 //
 
-
 #include "fib2mrib_module.h"
 
 #include "libxorp/xorp.h"
@@ -28,6 +26,7 @@
 #include "libxorp/ipvx.hh"
 
 #include "fib2mrib_node.hh"
+#include "fib2mrib_varrw.hh"
 
 
 Fib2mribNode::Fib2mribNode(EventLoop& eventloop)
@@ -38,10 +37,7 @@ Fib2mribNode::Fib2mribNode(EventLoop& eventloop)
       _shutdown_requests_n(0),
       _is_log_trace(true)		// XXX: default to print trace logs
 {
-    //
-    // Set the node status
-    //
-    _node_status = PROC_STARTUP;
+    set_node_status(PROC_STARTUP);
 }
 
 Fib2mribNode::~Fib2mribNode()
@@ -55,37 +51,30 @@ Fib2mribNode::startup()
     //
     // Test the service status
     //
-    if ((ServiceBase::status() == STARTING)
-	|| (ServiceBase::status() == RUNNING))
+    if ((ServiceBase::status() == SERVICE_STARTING)
+	|| (ServiceBase::status() == SERVICE_RUNNING)) {
 	return true;
-
-    if (ServiceBase::status() != READY)
-	return false;
-
-    //
-    // Transition to RUNNING occurs when all transient startup operations
-    // are completed (e.g., after we have the interface/vif/address state
-    // available, after we have registered with the RIB, etc.)
-    //
-    ServiceBase::set_status(STARTING);
-
-    //
-    // Set the node status
-    //
-    _node_status = PROC_STARTUP;
-
-    //
-    // Startup the interface manager
-    //
-    if (ifmgr_startup() != true) {
-	ServiceBase::set_status(FAILED);
+    }
+    if (ServiceBase::status() != SERVICE_READY) {
 	return false;
     }
 
     //
-    // Register as an FEA FIB client
+    // Transition to SERVICE_RUNNING occurs when all transient startup
+    // operations are completed (e.g., after we have the interface/vif/address
+    // state available, after we have registered with the RIB, etc.)
     //
-    fea_fib_client_register_startup();
+    ServiceBase::set_status(SERVICE_STARTING);
+
+    //
+    // Set the node status
+    //
+    set_node_status(PROC_STARTUP);
+
+    //
+    // Register with the FEA
+    //
+    fea_register_startup();
 
     //
     // Register with the RIB
@@ -99,22 +88,27 @@ bool
 Fib2mribNode::shutdown()
 {
     //
-    // We cannot shutdown if our status is SHUTDOWN or FAILED.
+    // Test the service status
     //
-    if ((ServiceBase::status() == SHUTDOWN)
-	|| (ServiceBase::status() == FAILED)) {
+    if ((ServiceBase::status() == SERVICE_SHUTDOWN)
+	|| (ServiceBase::status() == SERVICE_SHUTTING_DOWN)
+	|| (ServiceBase::status() == SERVICE_FAILED)) {
 	return true;
     }
-
-    if (ServiceBase::status() != RUNNING)
+    if ((ServiceBase::status() != SERVICE_RUNNING)
+	&& (ServiceBase::status() != SERVICE_STARTING)
+	&& (ServiceBase::status() != SERVICE_PAUSING)
+	&& (ServiceBase::status() != SERVICE_PAUSED)
+	&& (ServiceBase::status() != SERVICE_RESUMING)) {
 	return false;
+    }
 
     //
-    // Transition to SHUTDOWN occurs when all transient shutdown operations
-    // are completed (e.g., after we have deregistered with the FEA
+    // Transition to SERVICE_SHUTDOWN occurs when all transient shutdown
+    // operations are completed (e.g., after we have deregistered with the FEA
     // and the RIB, etc.)
     //
-    ServiceBase::set_status(SHUTTING_DOWN);
+    ServiceBase::set_status(SERVICE_SHUTTING_DOWN);
 
     //
     // De-register with the RIB
@@ -122,22 +116,19 @@ Fib2mribNode::shutdown()
     rib_register_shutdown();
 
     //
-    // De-register with as an FEA FIB client
+    // De-register with the FEA
     //
-    fea_fib_client_register_shutdown();
-
-    //
-    // Shutdown the interface manager
-    //
-    if (ifmgr_shutdown() != true) {
-	ServiceBase::set_status(FAILED);
-	return false;
-    }
+    fea_register_shutdown();
 
     //
     // Set the node status
     //
-    _node_status = PROC_SHUTDOWN;
+    set_node_status(PROC_SHUTDOWN);
+
+    //
+    // Update the node status
+    //
+    update_status();
 
     return true;
 }
@@ -149,15 +140,17 @@ Fib2mribNode::status_change(ServiceBase*  service,
 {
     if (service == this) {
 	// My own status has changed
-	if ((old_status == STARTING) && (new_status == RUNNING)) {
+	if ((old_status == SERVICE_STARTING)
+	    && (new_status == SERVICE_RUNNING)) {
 	    // The startup process has completed
-	    _node_status = PROC_READY;
+	    set_node_status(PROC_READY);
 	    return;
 	}
 
-	if ((old_status == SHUTTING_DOWN) && (new_status == SHUTDOWN)) {
+	if ((old_status == SERVICE_SHUTTING_DOWN)
+	    && (new_status == SERVICE_SHUTDOWN)) {
 	    // The shutdown process has completed
-	    _node_status = PROC_DONE;
+	    set_node_status(PROC_DONE);
 	    return;
 	}
 
@@ -168,7 +161,8 @@ Fib2mribNode::status_change(ServiceBase*  service,
     }
 
     if (service == ifmgr_mirror_service_base()) {
-	if ((old_status == SHUTTING_DOWN) && (new_status == SHUTDOWN)) {
+	if ((old_status == SERVICE_SHUTTING_DOWN)
+	    && (new_status == SERVICE_SHUTDOWN)) {
 	    decr_shutdown_requests_n();
 	}
     }
@@ -212,36 +206,34 @@ Fib2mribNode::update_status()
     //
     // Test if the startup process has completed
     //
-    if (ServiceBase::status() == STARTING) {
+    if (ServiceBase::status() == SERVICE_STARTING) {
 	if (_startup_requests_n > 0)
 	    return;
 
 	// The startup process has completed
-	ServiceBase::set_status(RUNNING);
-	_node_status = PROC_READY;
+	ServiceBase::set_status(SERVICE_RUNNING);
+	set_node_status(PROC_READY);
 	return;
     }
 
     //
     // Test if the shutdown process has completed
     //
-    if (ServiceBase::status() == SHUTTING_DOWN) {
+    if (ServiceBase::status() == SERVICE_SHUTTING_DOWN) {
 	if (_shutdown_requests_n > 0)
 	    return;
 
 	// The shutdown process has completed
-	ServiceBase::set_status(SHUTDOWN);
-	// Set the node status
-	_node_status = PROC_DONE;
+	ServiceBase::set_status(SERVICE_SHUTDOWN);
+	set_node_status(PROC_DONE);
 	return;
     }
 
     //
     // Test if we have failed
     //
-    if (ServiceBase::status() == FAILED) {
-	// Set the node status
-	_node_status = PROC_DONE;
+    if (ServiceBase::status() == SERVICE_FAILED) {
+	set_node_status(PROC_DONE);
 	return;
     }
 }
@@ -261,7 +253,7 @@ Fib2mribNode::node_status(string& reason_msg)
     case PROC_STARTUP:
 	// Get the message about the startup progress
 	reason_msg = c_format("Waiting for %u startup events",
-			      static_cast<uint32_t>(_startup_requests_n));
+			      XORP_UINT_CAST(_startup_requests_n));
 	break;
     case PROC_NOT_READY:
 	// XXX: this state is unused
@@ -273,7 +265,7 @@ Fib2mribNode::node_status(string& reason_msg)
     case PROC_SHUTDOWN:
 	// Get the message about the shutdown progress
 	reason_msg = c_format("Waiting for %u shutdown events",
-			      static_cast<uint32_t>(_shutdown_requests_n));
+			      XORP_UINT_CAST(_shutdown_requests_n));
 	break;
     case PROC_FAILED:
 	// XXX: this state is unused
@@ -305,14 +297,19 @@ Fib2mribNode::tree_complete()
 void
 Fib2mribNode::updates_made()
 {
-    map<IPvXNet, Fib2mribRoute>::const_iterator route_iter;
+    multimap<IPvXNet, Fib2mribRoute>::iterator route_iter;
 
     for (route_iter = _fib2mrib_routes.begin();
 	 route_iter != _fib2mrib_routes.end();
 	 ++route_iter) {
-	const Fib2mribRoute& fib2mrib_route = route_iter->second;
+	Fib2mribRoute& fib2mrib_route = route_iter->second;
 	bool is_old_interface_up = false;
 	bool is_new_interface_up = false;
+	bool is_old_directly_connected = false;
+	bool is_new_directly_connected = false;
+	bool is_new_up = false;
+
+	update_route(ifmgr_iftree(), fib2mrib_route);
 
 	if (fib2mrib_route.is_interface_route()) {
 	    //
@@ -327,27 +324,32 @@ Fib2mribNode::updates_made()
 					       fib2mrib_route.vifname());
 	    if ((vif_atom != NULL) && (vif_atom->enabled()))
 		is_new_interface_up = true;
-	} else {
-	    //
-	    // Calculate whether the next-hop router was directly connected
-	    // before and now.
-	    //
-	    if (is_directly_connected(_iftree, fib2mrib_route.nexthop()))
-		is_old_interface_up = true;
-	    if (is_directly_connected(ifmgr_iftree(), fib2mrib_route.nexthop()))
-		is_new_interface_up = true;
+	}
+	//
+	// Calculate whether the next-hop router was directly connected
+	// before and now.
+	//
+	if (is_directly_connected(_iftree, fib2mrib_route.nexthop()))
+	    is_old_directly_connected = true;
+	if (is_directly_connected(ifmgr_iftree(), fib2mrib_route.nexthop()))
+	    is_new_directly_connected = true;
+
+	if ((is_old_interface_up == is_new_interface_up)
+	    && (is_old_directly_connected == is_new_directly_connected)) {
+	    continue;			// Nothing changed
 	}
 
 	if (is_old_interface_up == is_new_interface_up)
-	    continue;			// Nothing changed
+	    is_new_up = is_new_directly_connected;
+	else
+	    is_new_up = is_new_interface_up;
 
-	if (is_new_interface_up) {
+	if (is_new_up) {
 	    //
 	    // The interface is now up, hence add the route
 	    //
 	    inform_rib_route_change(fib2mrib_route);
-	}
-	if (! is_new_interface_up) {
+	} else {
 	    //
 	    // The interface went down, hence cancel all pending requests,
 	    // and withdraw the route.
@@ -367,13 +369,13 @@ Fib2mribNode::updates_made()
 }
 
 bool
-Fib2mribNode::is_directly_connected(const IfMgrIfTree& if_tree,
+Fib2mribNode::is_directly_connected(const IfMgrIfTree& iftree,
 				    const IPvX& addr) const
 {
     IfMgrIfTree::IfMap::const_iterator if_iter;
 
-    for (if_iter = if_tree.ifs().begin();
-	 if_iter != if_tree.ifs().end();
+    for (if_iter = iftree.ifs().begin();
+	 if_iter != iftree.ifs().end();
 	 ++if_iter) {
 	const IfMgrIfAtom& iface = if_iter->second;
 
@@ -652,12 +654,19 @@ int
 Fib2mribNode::add_route(const Fib2mribRoute& fib2mrib_route,
 			string& error_msg)
 {
+    Fib2mribRoute updated_route = fib2mrib_route;
+
+    //
+    // Update the route
+    //
+    update_route(_iftree, updated_route);
+
     //
     // Check the entry
     //
-    if (fib2mrib_route.is_valid_entry(error_msg) != true) {
+    if (updated_route.is_valid_entry(error_msg) != true) {
 	error_msg = c_format("Cannot add route for %s: %s",
-			     fib2mrib_route.network().str().c_str(),
+			     updated_route.network().str().c_str(),
 			     error_msg.c_str());
 	return XORP_ERROR;
     }
@@ -666,40 +675,57 @@ Fib2mribNode::add_route(const Fib2mribRoute& fib2mrib_route,
     // Check if the route was added already
     //
     multimap<IPvXNet, Fib2mribRoute>::iterator iter;
-    iter = _fib2mrib_routes.find(fib2mrib_route.network());
+    iter = _fib2mrib_routes.find(updated_route.network());
     for ( ; iter != _fib2mrib_routes.end(); ++iter) {
 	Fib2mribRoute& tmp_route = iter->second;
-	if (tmp_route.network() != fib2mrib_route.network())
+	if (tmp_route.network() != updated_route.network())
 	    break;
-	if ((tmp_route.ifname() != fib2mrib_route.ifname())
-	    || (tmp_route.vifname() != fib2mrib_route.vifname())) {
+	if ((tmp_route.ifname() != updated_route.ifname())
+	    || (tmp_route.vifname() != updated_route.vifname())) {
 	    continue;
 	}
-	error_msg = c_format("Cannot add route for %s: "
-			     "the route already exists",
-			     fib2mrib_route.network().str().c_str());
-	return XORP_ERROR;
+
+	//
+	// XXX: Route found. Ideally, if we receive add_route() from
+	// the FEA, we should have received delete_route() before.
+	// However, on FreeBSD-4.10 (at least), if a route points
+	// to the local address of an interface, and if we delete
+	// that address, the kernel automatically removes all affected
+	// routes without reporting the routing information change
+	// to the process(es) listening on routing sockets.
+	// Therefore, to deal with such (mis)behavior of the FEA,
+	// we just replace the previously received route with the
+	// new one.
+	//
+	updated_route.set_replace_route();
+	return (replace_route(updated_route, error_msg));
     }
 
     //
     // Add the route
     //
-    _fib2mrib_routes.insert(make_pair(fib2mrib_route.network(),
-				      fib2mrib_route));
+    iter = _fib2mrib_routes.insert(make_pair(updated_route.network(),
+					     updated_route));
 
     //
-    // Inform the RIB about the change
+    // Do policy filtering
     //
-    if (fib2mrib_route.is_interface_route()) {
-	const IfMgrVifAtom* vif_atom;
-	vif_atom = _iftree.find_vif(fib2mrib_route.ifname(),
-				    fib2mrib_route.vifname());
-	if ((vif_atom != NULL) && (vif_atom->enabled()))
-	    inform_rib_route_change(fib2mrib_route);
-    } else {
-	if (is_directly_connected(_iftree, fib2mrib_route.nexthop()))
-	    inform_rib_route_change(fib2mrib_route);
-    }
+    Fib2mribRoute& added_route = iter->second;
+
+    //
+    // We do not want to modify original route, so we may re-filter routes on
+    // filter configuration changes. Hence, copy route.
+    //
+    Fib2mribRoute copy_route = added_route;
+
+    bool accepted = do_filtering(copy_route);
+
+    // Tag the original route as filtered or not
+    added_route.set_filtered(!accepted);
+
+    // Inform RIB about the possibly modified route if it was accepted 
+    if (accepted)
+	inform_rib(copy_route);
 
     return XORP_OK;
 }
@@ -716,14 +742,20 @@ int
 Fib2mribNode::replace_route(const Fib2mribRoute& fib2mrib_route,
 			    string& error_msg)
 {
-    bool is_route_replaced = false;
+    Fib2mribRoute updated_route = fib2mrib_route;
+    Fib2mribRoute* route_to_replace_ptr = NULL;
+
+    //
+    // Update the route
+    //
+    update_route(_iftree, updated_route);
 
     //
     // Check the entry
     //
-    if (fib2mrib_route.is_valid_entry(error_msg) != true) {
+    if (updated_route.is_valid_entry(error_msg) != true) {
 	error_msg = c_format("Cannot replace route for %s: %s",
-			     fib2mrib_route.network().str().c_str(),
+			     updated_route.network().str().c_str(),
 			     error_msg.c_str());
 	return XORP_ERROR;
     }
@@ -731,15 +763,16 @@ Fib2mribNode::replace_route(const Fib2mribRoute& fib2mrib_route,
     //
     // Find the route and replace it
     //
-    map<IPvXNet, Fib2mribRoute>::iterator iter, iter2;
-    iter = _fib2mrib_routes.find(fib2mrib_route.network());
-    if (iter == _fib2mrib_routes.end()) {
+    multimap<IPvXNet, Fib2mribRoute>::iterator iter, iter2;
+    iter = _fib2mrib_routes.find(updated_route.network());
+    if ((iter == _fib2mrib_routes.end())
+	|| (iter->second.network() != updated_route.network())) {
 	//
-	// Coudn't find the route to replace
+	// Couldn't find the route to replace
 	//
 	error_msg = c_format("Cannot replace route for %s: "
 			     "no such route",
-			     fib2mrib_route.network().str().c_str());
+			     updated_route.network().str().c_str());
 	return XORP_ERROR;
     }
 
@@ -750,44 +783,65 @@ Fib2mribNode::replace_route(const Fib2mribRoute& fib2mrib_route,
     //
     for (iter2 = iter; iter2 != _fib2mrib_routes.end(); ++iter2) {
 	Fib2mribRoute& tmp_route = iter2->second;
-	if (tmp_route.network() != fib2mrib_route.network())
+	if (tmp_route.network() != updated_route.network())
 	    break;
-	if ((tmp_route.ifname() != fib2mrib_route.ifname())
-	    || (tmp_route.vifname() != fib2mrib_route.vifname())) {
+	if ((tmp_route.ifname() != updated_route.ifname())
+	    || (tmp_route.vifname() != updated_route.vifname())) {
 	    continue;
 	}
 
 	//
-	// Route found. Overwrite its value.
+	// Route found.
 	//
-	tmp_route = fib2mrib_route;
-	is_route_replaced = true;
+	route_to_replace_ptr = &tmp_route;
 	break;
     }
 
-    if (! is_route_replaced) {
+    if (route_to_replace_ptr == NULL) {
 	//
 	// No route found with the same ifname and vifname.
 	// Update the first route for the same subnet.
 	//
 	Fib2mribRoute& tmp_route = iter->second;
 
-	tmp_route = fib2mrib_route;
+	route_to_replace_ptr = &tmp_route;
     }
 
     //
-    // Inform the RIB about the change
+    // Overwrite the route's value.
     //
-    if (fib2mrib_route.is_interface_route()) {
-	const IfMgrVifAtom* vif_atom;
-	vif_atom = _iftree.find_vif(fib2mrib_route.ifname(),
-				    fib2mrib_route.vifname());
-	if ((vif_atom != NULL) && (vif_atom->enabled()))
-	    inform_rib_route_change(fib2mrib_route);
-    } else {
-	if (is_directly_connected(_iftree, fib2mrib_route.nexthop()))
-	    inform_rib_route_change(fib2mrib_route);
-    }
+    do {
+	XLOG_ASSERT(route_to_replace_ptr != NULL);
+	Fib2mribRoute& tmp_route = *route_to_replace_ptr;
+
+	bool was_filtered = tmp_route.is_filtered();
+	tmp_route = updated_route;
+
+	// Do policy filtering
+	Fib2mribRoute copy_route = updated_route;
+	bool accepted = do_filtering(copy_route);
+	tmp_route.set_filtered(!accepted);
+
+	// Decide what to do
+	if (accepted) {
+	    if (was_filtered) {
+		copy_route.set_add_route();
+	    } else {
+	    }
+	} else {
+	    if (was_filtered) {
+		return XORP_OK;
+	    } else {
+		copy_route.set_delete_route();
+	    }
+	}
+	
+	//
+	// Inform the RIB about the change
+	//
+	inform_rib(copy_route);
+
+    } while (false);
 
     return XORP_OK;
 }
@@ -804,12 +858,19 @@ int
 Fib2mribNode::delete_route(const Fib2mribRoute& fib2mrib_route,
 			   string& error_msg)
 {
+    Fib2mribRoute updated_route = fib2mrib_route;
+
+    //
+    // Update the route
+    //
+    update_route(_iftree, updated_route);
+
     //
     // Check the entry
     //
-    if (fib2mrib_route.is_valid_entry(error_msg) != true) {
+    if (updated_route.is_valid_entry(error_msg) != true) {
 	error_msg = c_format("Cannot delete route for %s: %s",
-			     fib2mrib_route.network().str().c_str(),
+			     updated_route.network().str().c_str(),
 			     error_msg.c_str());
 	return XORP_ERROR;
     }
@@ -817,48 +878,72 @@ Fib2mribNode::delete_route(const Fib2mribRoute& fib2mrib_route,
     //
     // Find the route and delete it
     //
-    map<IPvXNet, Fib2mribRoute>::iterator iter;
-    iter = _fib2mrib_routes.find(fib2mrib_route.network());
-    for ( ; iter != _fib2mrib_routes.end(); ++iter) {
-	Fib2mribRoute& tmp_route = iter->second;
-	if (tmp_route.network() != fib2mrib_route.network())
+    multimap<IPvXNet, Fib2mribRoute>::iterator iter, iter2;
+    iter = _fib2mrib_routes.find(updated_route.network());
+    if ((iter == _fib2mrib_routes.end())
+	|| (iter->second.network() != updated_route.network())) {
+	//
+	// Couldn't find the route to delete
+	//
+	error_msg = c_format("Cannot delete route for %s: "
+			     "no such route",
+			     updated_route.network().str().c_str());
+	return XORP_ERROR;
+    }
+
+    //
+    // First check if there is a route with the same ifname and vifname.
+    // If there is such route, then delete it. Otherwise,
+    // delete the first route for the same subnet.
+    //
+    for (iter2 = iter; iter2 != _fib2mrib_routes.end(); ++iter2) {
+	Fib2mribRoute& tmp_route = iter2->second;
+	if (tmp_route.network() != updated_route.network())
 	    break;
-	if ((tmp_route.ifname() != fib2mrib_route.ifname())
-	    || (tmp_route.vifname() != fib2mrib_route.vifname())) {
+	if ((tmp_route.ifname() != updated_route.ifname())
+	    || (tmp_route.vifname() != updated_route.vifname())) {
 	    continue;
 	}
 
 	//
-	// Route found. Create a copy of it and erase it.
+	// Route found
 	//
-	Fib2mribRoute copy_route = tmp_route;
-	copy_route.set_delete_route();
-	_fib2mrib_routes.erase(iter);
+	break;
+    }
 
+    if (iter2 == _fib2mrib_routes.end()
+	|| (iter2->second.network() != updated_route.network())) {
 	//
-	// Inform the RIB about the change
+	// No route found with the same ifname and vifname.
+	// If this is an interface-specific route, then this is an error.
+	// Otherwise, delete the first route for the same subnet.
 	//
-	if (copy_route.is_interface_route()) {
-	    const IfMgrVifAtom* vif_atom;
-	    vif_atom = _iftree.find_vif(copy_route.ifname(),
-					copy_route.vifname());
-	    if ((vif_atom != NULL) && (vif_atom->enabled()))
-		inform_rib_route_change(copy_route);
-	} else {
-	    if (is_directly_connected(_iftree, copy_route.nexthop()))
-		inform_rib_route_change(copy_route);
+	if (updated_route.is_interface_route()) {
+	    error_msg = c_format("Cannot delete route for %s: "
+				 "no such route",
+				 updated_route.network().str().c_str());
+	    return XORP_ERROR;
 	}
-
-	return XORP_OK;
+	iter2 = iter;
     }
 
     //
-    // Coudn't find the route to delete
+    // Create a copy of the route and erase it.
     //
-    error_msg = c_format("Cannot delete route for %s: "
-			 "no such route",
-			 fib2mrib_route.network().str().c_str());
-    return XORP_ERROR;
+    Fib2mribRoute copy_route = iter2->second;
+    copy_route.set_delete_route();
+    _fib2mrib_routes.erase(iter2);
+
+    // If the route is filtered, then RIB doesn't know about it.
+    if (copy_route.is_filtered())
+	return XORP_OK;
+
+    //
+    // Inform the RIB about the change
+    //
+    inform_rib(copy_route);
+
+    return XORP_OK;
 }
 
 /**
@@ -874,3 +959,236 @@ Fib2mribRoute::is_valid_entry(string& error_msg) const
 
     return true;
 }
+
+void
+Fib2mribNode::configure_filter(const uint32_t& filter, const string& conf)
+{
+    _policy_filters.configure(filter, conf);
+}
+
+void
+Fib2mribNode::reset_filter(const uint32_t& filter) {
+    _policy_filters.reset(filter);
+}
+
+void
+Fib2mribNode::push_routes()
+{
+    multimap<IPvXNet, Fib2mribRoute>::iterator iter;
+
+    // XXX: not a background task
+    for (iter = _fib2mrib_routes.begin(); iter != _fib2mrib_routes.end();
+	 ++iter) {
+	Fib2mribRoute& orig_route = iter->second;
+	bool was_filtered = orig_route.is_filtered();
+	
+	Fib2mribRoute copy_route = orig_route;
+	bool accepted = do_filtering(copy_route);
+
+	debug_msg("[FIB2MRIB] Push route: %s, was filtered: %d, accepted %d\n",
+		  orig_route.network().str().c_str(),
+		  was_filtered, accepted);
+
+	orig_route.set_filtered(!accepted);
+
+	if (accepted) {
+	    if (was_filtered) {
+		copy_route.set_add_route();
+	    } else {
+		copy_route.set_replace_route();
+	    }
+	} else {
+	    // not accepted
+	    if (was_filtered) {
+		continue;
+	    } else {
+		copy_route.set_delete_route();
+	    }
+	}
+
+	inform_rib(copy_route);
+    }
+}
+
+void
+Fib2mribNode::inform_rib(const Fib2mribRoute& route)
+{
+    //
+    // Inform the RIB about the change
+    //
+    if (route.is_interface_route()) {
+	const IfMgrVifAtom* vif_atom;
+	vif_atom = _iftree.find_vif(route.ifname(), route.vifname());
+	if ((vif_atom != NULL) && (vif_atom->enabled()))
+	    inform_rib_route_change(route);
+    } else {
+	if (is_directly_connected(_iftree, route.nexthop()))
+	    inform_rib_route_change(route);
+    }
+}
+
+/**
+ * Update a route received from the FEA.
+ * 
+ * This method is needed as a work-around of FEA-related problems
+ * with the routes the FEA sends to interested parties such as FIB2MRIB.
+ * A route is updated with interface-related information or next-hop
+ * address.
+ * 
+ * This method is needed as a work-around of FEA-related problems
+ * with the routes the FEA sends to interested parties such as FIB2MRIB.
+ * 
+ * The routes received from the FEA for the directly-connected subnets
+ * may not contain next-hop information and network interface information.
+ * If the route is for a directly-connected subnet, and if it is missing
+ * that information, then add the interface and next-hop router
+ * information.
+ * Furthermore, on startup, the routes received from the FEA may
+ * contain interface-related information, but later updates of those
+ * routes may be missing this information. This may create a mismatch,
+ * therefore all routes are updated (if possible) to contain
+ * interface-related information.
+ * 
+ * @param iftree the tree with the interface state to update the route.
+ * @param route the route to update.
+ * @return true if the route was updated, otherwise false.
+ */
+bool
+Fib2mribNode::update_route(const IfMgrIfTree& iftree, Fib2mribRoute& route)
+{
+    //
+    // Test if the route needs to be updated
+    //
+    if (route.is_interface_route())
+	return (false);
+
+    //
+    // Find if there is a directly-connected subnet that matches the route
+    // or the address of the next-hop router.
+    //
+    IfMgrIfTree::IfMap::const_iterator if_iter;
+    for (if_iter = iftree.ifs().begin();
+	 if_iter != iftree.ifs().end();
+	 ++if_iter) {
+	const IfMgrIfAtom& iface = if_iter->second;
+
+	IfMgrIfAtom::VifMap::const_iterator vif_iter;
+	for (vif_iter = iface.vifs().begin();
+	     vif_iter != iface.vifs().end();
+	     ++vif_iter) {
+	    const IfMgrVifAtom& vif = vif_iter->second;
+
+	    //
+	    // Check the IPv4 subnets
+	    //
+	    if (route.is_ipv4()) {
+		IfMgrVifAtom::V4Map::const_iterator a4_iter;
+		for (a4_iter = vif.ipv4addrs().begin();
+		     a4_iter != vif.ipv4addrs().end();
+		     ++a4_iter) {
+		    const IfMgrIPv4Atom& a4 = a4_iter->second;
+		    IPvXNet ipvxnet(a4.addr(), a4.prefix_len());
+
+		    // Update a directly-connected route
+		    if (ipvxnet == route.network()) {
+			route.set_ifname(iface.name());
+			route.set_vifname(vif.name());
+			if (route.nexthop().is_zero())
+			    route.set_nexthop(a4.addr());
+			return (true);
+		    }
+
+		    // Update the route if a directly-connected next-hop
+		    if (ipvxnet.contains(route.nexthop())
+			&& (! route.nexthop().is_zero())) {
+			route.set_ifname(iface.name());
+			route.set_vifname(vif.name());
+			return (true);
+		    }
+		}
+	    }
+
+	    //
+	    // Check the IPv6 subnets
+	    //
+	    if (route.is_ipv6()) {
+		IfMgrVifAtom::V6Map::const_iterator a6_iter;
+		for (a6_iter = vif.ipv6addrs().begin();
+		     a6_iter != vif.ipv6addrs().end();
+		     ++a6_iter) {
+		    const IfMgrIPv6Atom& a6 = a6_iter->second;
+		    IPvXNet ipvxnet(a6.addr(), a6.prefix_len());
+
+		    // Update a directly-connected route
+		    if (ipvxnet == route.network()) {
+			route.set_ifname(iface.name());
+			route.set_vifname(vif.name());
+			if (route.nexthop().is_zero())
+			    route.set_nexthop(a6.addr());
+			return (true);
+		    }
+
+		    // Update the route if a directly-connected next-hop
+		    if (ipvxnet.contains(route.nexthop())
+			&& (! route.nexthop().is_zero())) {
+			route.set_ifname(iface.name());
+			route.set_vifname(vif.name());
+			return (true);
+		    }
+		}
+	    }
+	}
+    }
+
+    return (false);
+}
+
+bool
+Fib2mribNode::do_filtering(Fib2mribRoute& route)
+{
+    try {
+	ostringstream trace;
+
+	Fib2mribVarRW varrw(route);
+
+	// Import filtering
+	bool accepted;
+
+	debug_msg("[FIB2MRIB] Running filter: %s on route: %s\n",
+		  filter::filter2str(filter::IMPORT).c_str(),
+		  route.network().str().c_str());
+	accepted = _policy_filters.run_filter(filter::IMPORT, varrw, &trace);
+
+	debug_msg("[FIB2MRIB] filter trace:\n%s\nEnd of trace.\n",
+		  trace.str().c_str());
+
+	// clear trace
+	trace.str("");
+
+	route.set_filtered(!accepted);
+
+	// Route Rejected 
+	if (!accepted) 
+	    return accepted;
+
+	Fib2mribVarRW varrw2(route);
+
+	// Export source-match filtering
+	debug_msg("[FIB2MRIB] Running filter: %s on route: %s\n",
+		  filter::filter2str(filter::EXPORT_SOURCEMATCH).c_str(),
+		  route.network().str().c_str());
+
+	_policy_filters.run_filter(filter::EXPORT_SOURCEMATCH,
+				   varrw2, &trace);
+
+	debug_msg("[FIB2MRIB] filter trace:\n%s\nEnd of trace.\n",
+		  trace.str().c_str());
+
+	return accepted;
+    } catch(const PolicyException& e) {
+	XLOG_FATAL("PolicyException: %s", e.str().c_str());
+
+	// FIXME: What do we do ?
+	XLOG_UNFINISHED();
+    }
+}
