@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2004 International Computer Science Institute
+ * Copyright (c) 2001-2005 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -14,7 +14,7 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/libcomm/test_comm.c,v 1.5 2004/06/10 22:41:02 hodson Exp $"
+#ident "$XORP: xorp/libcomm/test_comm.c,v 1.8 2005/03/25 02:53:22 pavlin Exp $"
 
 
 /*
@@ -25,6 +25,12 @@
 #include "comm_module.h"
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
+
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+
 #include "libcomm/comm_api.h"
 
 
@@ -80,7 +86,7 @@ main(int argc, char *argv[])
     /*
      * Test `open TCP socket'
      */
-    sock = comm_open_tcp(AF_INET);
+    sock = comm_open_tcp(AF_INET, COMM_SOCK_BLOCKING);
     if (sock >= 0) {
 	printf("OK: open TCP socket\n");
 	comm_close(sock);
@@ -91,7 +97,7 @@ main(int argc, char *argv[])
     /*
      * Test `open UDP socket'
      */
-    sock = comm_open_udp(AF_INET);
+    sock = comm_open_udp(AF_INET, COMM_SOCK_BLOCKING);
     if (sock >= 0) {
 	printf("OK: open UDP socket\n");
 	comm_close(sock);
@@ -102,7 +108,7 @@ main(int argc, char *argv[])
     /*
      * Test `bind TCP socket'
      */
-    sock = comm_bind_tcp4(NULL, port);
+    sock = comm_bind_tcp4(NULL, port, COMM_SOCK_BLOCKING);
     if (sock >= 0) {
 	printf("OK: open and bind TCP socket to port %d\n", ntohs(port));
 	comm_close(sock);
@@ -114,7 +120,7 @@ main(int argc, char *argv[])
     /*
      * Test `bind UDP socket'
      */
-    sock = comm_bind_udp4(NULL, port);
+    sock = comm_bind_udp4(NULL, port, COMM_SOCK_BLOCKING);
     if (sock >= 0) {
 	printf("OK: open and bind UDP socket to port %d\n", ntohs(port));
 	comm_close(sock);
@@ -126,7 +132,9 @@ main(int argc, char *argv[])
     /*
      * Test 'bind and join a multicast group'
      */
-    sock = comm_bind_join_udp4(&mcast_addr, NULL, port, ADDR_PORT_REUSE_FLAG);
+    sock = comm_bind_join_udp4(&mcast_addr, NULL, port,
+			       COMM_SOCK_ADDR_PORT_REUSE,
+			       COMM_SOCK_BLOCKING);
     if (sock >= 0) {
 	printf("OK: open, bind and join UDP socket to group %s and port %d\n",
 	       inet_ntoa(mcast_addr), ntohs(port));
