@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/ifmgr_cmds.cc,v 1.9 2004/06/10 22:41:02 hodson Exp $"
+#ident "$XORP: xorp/libfeaclient/ifmgr_cmds.cc,v 1.13 2005/03/25 02:53:22 pavlin Exp $"
 
 #include "libxorp/c_format.hh"
 
@@ -222,6 +222,41 @@ IfMgrIfSetEnabled::str() const
 	+ "\", " + bool2str(en()) + if_str_end();
 }
 
+// ----------------------------------------------------------------------------
+// IfMgrIfSetDiscard
+
+bool
+IfMgrIfSetDiscard::execute(IfMgrIfTree& t) const
+{
+    IfMgrIfTree::IfMap& ifs = t.ifs();
+    const string& n = ifname();
+
+    IfMgrIfTree::IfMap::iterator i = ifs.find(n);
+    if (i != ifs.end()) {
+	IfMgrIfAtom& interface = i->second;
+	interface.set_discard(discard());
+	return true;
+    }
+    return false;
+}
+
+bool
+IfMgrIfSetDiscard::forward(XrlSender&		 sender,
+			   const string&	 xrl_target,
+			   const IfMgrXrlSendCB& xcb) const
+{
+    XrlFeaIfmgrMirrorV0p1Client c(&sender);
+    const char* xt = xrl_target.c_str();
+    return c.send_interface_set_discard(xt, ifname(), discard(), xcb);
+}
+
+string
+IfMgrIfSetDiscard::str() const
+{
+    return if_str_begin(this, "SetDiscard")
+	+ "\", " + bool2str(discard()) + if_str_end();
+}
+
 // ----------------------------------------------------------------------------
 // IfMgrIfSetMtu
 
@@ -254,7 +289,7 @@ string
 IfMgrIfSetMtu::str() const
 {
     return if_str_begin(this, "Mtu") + ", " +
-	c_format("%u", mtu_bytes()) + if_str_end();
+	c_format("%u", XORP_UINT_CAST(mtu_bytes())) + if_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -323,7 +358,7 @@ string
 IfMgrIfSetPifIndex::str() const
 {
     return if_str_begin(this, "SetPifIndex") +
-	c_format(", %u", pif_index()) + if_str_end();
+	c_format(", %u", XORP_UINT_CAST(pif_index())) + if_str_end();
 }
 
 
@@ -599,7 +634,7 @@ string
 IfMgrVifSetPifIndex::str() const
 {
     return vif_str_begin(this, "SetPifIndex")
-	+ ", " + c_format("%u", pif_index()) + vif_str_end();
+	+ ", " + c_format("%u", XORP_UINT_CAST(pif_index())) + vif_str_end();
 }
 
 
@@ -706,7 +741,7 @@ string
 IfMgrIPv4SetPrefix::str() const
 {
     return ipv4_str_begin(this, "SetPrefix") + ", "
-	+ c_format("%u", prefix_len()) + ipv4_str_end();
+	+ c_format("%u", XORP_UINT_CAST(prefix_len())) + ipv4_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -977,7 +1012,7 @@ string
 IfMgrIPv6SetPrefix::str() const
 {
     return ipv6_str_begin(this, "SetPrefix") + ", "
-	+ c_format("%u", prefix_len()) + ipv6_str_end();
+	+ c_format("%u", XORP_UINT_CAST(prefix_len())) + ipv6_str_end();
 }
 
 // ----------------------------------------------------------------------------
