@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_mirror.cc,v 1.10 2004/06/10 22:41:02 hodson Exp $"
+#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_mirror.cc,v 1.16 2005/03/25 02:53:23 pavlin Exp $"
 
 #include "libxorp/status_codes.h"
 #include "libxorp/eventloop.hh"
@@ -82,6 +82,11 @@ protected:
 	const string&	ifname,
 	const bool&	enabled);
 
+    XrlCmdError fea_ifmgr_mirror_0_1_interface_set_discard(
+	// Input values,
+	const string&	ifname,
+	const bool&	discard);
+
     XrlCmdError fea_ifmgr_mirror_0_1_interface_set_mtu(
 	// Input values,
 	const string&	ifname,
@@ -341,6 +346,19 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_enabled(
     return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
 }
 
+XrlCmdError
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_discard(
+	const string&	ifname,
+	const bool&	discard
+	)
+{
+    _dispatcher.push(new IfMgrIfSetDiscard(ifname, discard));
+    if (_dispatcher.execute() == true) {
+	return XrlCmdError::OKAY();
+    }
+    return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
+}
+
 XrlCmdError
 IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_mtu(
 	const string&	ifname,
@@ -776,6 +794,13 @@ public:
 	: XrlStdRouter(e, class_name, finder_addr, finder_port), _o(0)
     {}
 
+    IfMgrXrlMirrorRouter(EventLoop&	e,
+			 const char*	class_name,
+			 const char*	finder_hostname,
+			 uint16_t	finder_port)
+	: XrlStdRouter(e, class_name, finder_hostname, finder_port), _o(0)
+    {}
+
     /**
      * Attach an observer to router.  Only one observer is permitted.
      *
@@ -828,15 +853,25 @@ protected:
 // IfMgrXrlMirror
 
 static const char* CLSNAME = "ifmgr_mirror";
-const char* IfMgrXrlMirror::DEFAULT_REGISTRATION_TARGET = "ifmgr_mirror";
 
 IfMgrXrlMirror::IfMgrXrlMirror(EventLoop&	e,
 			       const char*	rtarget,
-			       IPv4		finder_host,
+			       IPv4		finder_addr,
 			       uint16_t		finder_port)
 
     : ServiceBase("FEA Interface Mirror"),
-      _e(e), _finder_host(finder_host), _finder_port(finder_port),
+      _e(e), _finder_addr(finder_addr), _finder_port(finder_port),
+      _dispatcher(_iftree), _rtarget(rtarget), _rtr(0), _xrl_tgt(0)
+{
+}
+
+IfMgrXrlMirror::IfMgrXrlMirror(EventLoop&	e,
+			       const char*	rtarget,
+			       const char*	finder_hostname,
+			       uint16_t		finder_port)
+
+    : ServiceBase("FEA Interface Mirror"),
+      _e(e), _finder_hostname(finder_hostname), _finder_port(finder_port),
       _dispatcher(_iftree), _rtarget(rtarget), _rtr(0), _xrl_tgt(0)
 {
 }
@@ -861,14 +896,26 @@ IfMgrXrlMirror::~IfMgrXrlMirror()
 bool
 IfMgrXrlMirror::startup()
 {
-    if (status() != READY)
+    if (status() != SERVICE_READY)
 	return false;
 
-    _rtr = new IfMgrXrlMirrorRouter(_e, CLSNAME, _finder_host, _finder_port);
-    _xrl_tgt = new IfMgrXrlMirrorTarget(*_rtr, _dispatcher);
-    _rtr->attach(this);
-    _xrl_tgt->attach(this);
-    set_status(STARTING, "Initializing Xrl Router.");
+    if (_rtr == NULL) {
+	if (! _finder_hostname.empty()) {
+	    _rtr = new IfMgrXrlMirrorRouter(_e, CLSNAME,
+					    _finder_hostname.c_str(),
+					    _finder_port);
+	} else {
+	    _rtr = new IfMgrXrlMirrorRouter(_e, CLSNAME,
+					    _finder_addr,
+					    _finder_port);
+	}
+	_rtr->attach(this);
+    }
+    if (_xrl_tgt == NULL) {
+	_xrl_tgt = new IfMgrXrlMirrorTarget(*_rtr, _dispatcher);
+	_xrl_tgt->attach(this);
+    }
+    set_status(SERVICE_STARTING, "Initializing Xrl Router.");
 
     return true;
 }
@@ -876,10 +923,10 @@ IfMgrXrlMirror::startup()
 bool
 IfMgrXrlMirror::shutdown()
 {
-    if (status() != RUNNING)
+    if (status() != SERVICE_RUNNING)
 	return false;
 
-    set_status(SHUTTING_DOWN);
+    set_status(SERVICE_SHUTTING_DOWN);
     unregister_with_ifmgr();
 
     return true;
@@ -889,10 +936,10 @@ void
 IfMgrXrlMirror::finder_disconnect_event()
 {
     _iftree.clear();
-    if (status() == SHUTTING_DOWN) {
-	set_status(SHUTDOWN);
+    if (status() == SERVICE_SHUTTING_DOWN) {
+	set_status(SERVICE_SHUTDOWN);
     } else {
-	set_status(FAILED);
+	set_status(SERVICE_FAILED);
     }
 }
 
@@ -905,14 +952,14 @@ IfMgrXrlMirror::finder_ready_event()
 void
 IfMgrXrlMirror::tree_complete()
 {
-    if (status() != STARTING) {
+    if (status() != SERVICE_STARTING) {
 	// XXX In case shutdown is called or a failure occurs before
 	// tree_complete message is received.  If either happens we don't
 	// want to advertise the tree as complete.
 	return;
     }
 
-    set_status(RUNNING);
+    set_status(SERVICE_RUNNING);
     list<IfMgrHintObserver*>::const_iterator ci;
     for (ci = _hint_observers.begin(); ci != _hint_observers.end(); ++ci) {
 	IfMgrHintObserver* ho = *ci;
@@ -923,7 +970,9 @@ IfMgrXrlMirror::tree_complete()
 void
 IfMgrXrlMirror::updates_made()
 {
-    if (status() & ServiceStatus(SHUTTING_DOWN | SHUTDOWN | FAILED))
+    if (status() & ServiceStatus(SERVICE_SHUTTING_DOWN
+				 | SERVICE_SHUTDOWN
+				 | SERVICE_FAILED))
 	return;
 
     list<IfMgrHintObserver*>::const_iterator ci;
@@ -936,7 +985,9 @@ IfMgrXrlMirror::updates_made()
 bool
 IfMgrXrlMirror::attach_hint_observer(IfMgrHintObserver* o)
 {
-    if (status() & ServiceStatus(SHUTTING_DOWN | SHUTDOWN | FAILED))
+    if (status() & ServiceStatus(SERVICE_SHUTTING_DOWN
+				 | SERVICE_SHUTDOWN
+				 | SERVICE_FAILED))
 	return false;
 
     if (find(_hint_observers.begin(), _hint_observers.end(), o)
@@ -968,19 +1019,19 @@ IfMgrXrlMirror::register_with_ifmgr()
 				     callback(this,
 					      &IfMgrXrlMirror::register_cb))
 	== false) {
-	set_status(FAILED, "Failed to send registration to ifmgr");
+	set_status(SERVICE_FAILED, "Failed to send registration to ifmgr");
 	return;
     }
-    set_status(STARTING, "Registering with FEA interface manager.");
+    set_status(SERVICE_STARTING, "Registering with FEA interface manager.");
 }
 
 void
 IfMgrXrlMirror::register_cb(const XrlError& e)
 {
     if (e == XrlError::OKAY()) {
-	set_status(STARTING, "Waiting to receive interface data.");
+	set_status(SERVICE_STARTING, "Waiting to receive interface data.");
     } else {
-	set_status(FAILED, "Failed to send registration to ifmgr");
+	set_status(SERVICE_FAILED, "Failed to send registration to ifmgr");
     }
 }
 
@@ -991,10 +1042,11 @@ IfMgrXrlMirror::unregister_with_ifmgr()
     if (c.send_unregister_ifmgr_mirror(_rtarget.c_str(), _rtr->instance_name(),
 		callback(this, &IfMgrXrlMirror::unregister_cb))
 	== false) {
-	set_status(FAILED, "Failed to send unregister to FEA");
+	set_status(SERVICE_FAILED, "Failed to send unregister to FEA");
 	return;
     }
-    set_status(SHUTTING_DOWN, "De-registering with FEA interface manager.");
+    set_status(SERVICE_SHUTTING_DOWN,
+	       "De-registering with FEA interface manager.");
 }
 
 void
@@ -1003,8 +1055,8 @@ IfMgrXrlMirror::unregister_cb(const XrlError& e)
     _iftree.clear();
 
     if (e == XrlError::OKAY()) {
-	set_status(SHUTDOWN);
+	set_status(SERVICE_SHUTDOWN);
     } else {
-	set_status(FAILED, "Failed to de-registration to ifmgr");
+	set_status(SERVICE_FAILED, "Failed to de-registration to ifmgr");
     }
 }
