@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_replicator.cc,v 1.6 2004/06/10 22:41:03 hodson Exp $"
+#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_replicator.cc,v 1.9 2005/03/25 02:53:23 pavlin Exp $"
 
 #include "config.h"
 
@@ -151,8 +151,8 @@ IfMgrXrlReplicationManager::remove_mirror(const string& target_name)
 {
     for (Outputs::iterator i = _outputs.begin(); i != _outputs.end(); ++i) {
 	if ((*i)->xrl_target_name() == target_name) {
-	    _outputs.erase(i);
 	    delete *i;
+	    _outputs.erase(i);
 	    return true;
 	}
     }
