@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder.cc,v 1.15 2004/06/10 22:41:05 hodson Exp $"
+#ident "$XORP: xorp/libxipc/finder.cc,v 1.20 2005/03/25 02:53:25 pavlin Exp $"
 
 #include <set>
 
@@ -337,7 +337,7 @@ validate_finder_classes_and_instances(const Finder::ClassTable&  classes,
 		    debug_msg("Known target \"%s\"\n", ti->first.c_str());
 		}
 		XLOG_FATAL("Missing instance (%s) / %u\n", ii->c_str(),
-			   static_cast<uint32_t>(targets.size()));
+			   XORP_UINT_CAST(targets.size()));
 	    }
 	}
     }
@@ -386,7 +386,7 @@ Finder::~Finder()
 	FinderMessengerBase* old_front = _messengers.front();
 	delete _messengers.front();
 	// Expect death event for messenger to remove item from list
-	assert(_messengers.front() != old_front);
+	assert(_messengers.empty() || (_messengers.front() != old_front));
     }
 }
 
@@ -503,12 +503,14 @@ Finder::add_target(const string& cls,
 	      cookie.c_str());
 
     TargetTable::const_iterator ci = _targets.find(tgt);
-    if (ci->second.messenger() == _active_messenger) {
-	debug_msg("already registered by messenger.\n");
-	return true;
-    } else if (ci != _targets.end()) {
-	debug_msg("Fail registered by another messenger.");
-	return false;
+    if (ci != _targets.end()) {
+	if (ci->second.messenger() == _active_messenger) {
+	    debug_msg("already registered by messenger.\n");
+	    return true;
+	} else {
+	    debug_msg("Fail registered by another messenger.");
+	    return false;
+	}
     }
 
     pair<TargetTable::iterator, bool> r =
@@ -938,7 +940,7 @@ Finder::remove_instance_watch(const string& target,
     return true;
 }
 
-#include "finder_event_observer_xif.hh"
+#include "xrl/interfaces/finder_event_observer_xif.hh"
 
 static void
 dummy_xrl_cb(const XrlError& e)
