@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_main.cc,v 1.13 2004/06/10 22:41:05 hodson Exp $"
+#ident "$XORP: xorp/libxipc/finder_main.cc,v 1.15 2005/03/25 02:53:26 pavlin Exp $"
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -106,7 +106,7 @@ finder_main(int argc, char* const argv[])
 {
     bool	run_verbose = false;
     list<IPv4>  bind_addrs;
-    uint16_t	bind_port = FINDER_DEFAULT_PORT;
+    uint16_t	bind_port = FinderConstants::FINDER_DEFAULT_PORT();
 
     signal(SIGHUP, finder_sig_handler);
     signal(SIGINT, finder_sig_handler);
@@ -195,7 +195,7 @@ finder_main(int argc, char* const argv[])
     XorpUnexpectedHandler x(xorp_unexpected_handler);
     try {
 	EventLoop e;
-	FinderServer fs(e, bind_port);
+	FinderServer fs(e, FinderConstants::FINDER_DEFAULT_HOST(), bind_port);
 
 	list<IPv4>::const_iterator ci = bind_addrs.begin();
 	while (ci != bind_addrs.end()) {
