@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_tcp.cc,v 1.18 2004/06/21 17:31:46 hodson Exp $"
+#ident "$XORP: xorp/libxipc/finder_tcp.cc,v 1.22 2005/03/25 02:53:27 pavlin Exp $"
 
 #include <functional>
 
@@ -149,9 +149,9 @@ FinderTcpBase::read_callback(AsyncFileOperator::Event	ev,
 	// Read length of data to follow
 	_isize = ntohl(_isize);
 
-	if (0 == _isize || _isize > FinderTcpBase::FINDER_TCP_BUFFER_BYTES) {
+	if (0 == _isize) {
 	    XLOG_ERROR("Bad input buffer size (%d bytes) from wire, "
-		       "dropping connection", _isize);
+		       "dropping connection", XORP_INT_CAST(_isize));
 	    error_event();
 	    return;
 	}
@@ -280,7 +280,7 @@ FinderTcpListenerBase::FinderTcpListenerBase(EventLoop& e,
 	xorp_throw(InvalidAddress, "Not a valid interface address");
     }
 
-    _lfd = comm_bind_tcp4(&if_ia, htons(port));
+    _lfd = comm_bind_tcp4(&if_ia, htons(port), COMM_SOCK_NONBLOCKING);
     if (XORP_ERROR == _lfd) {
 	xorp_throw(InvalidPort, strerror(errno));
     }
