@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_finder_events.cc,v 1.9 2004/06/18 17:06:35 hodson Exp $"
+#ident "$XORP: xorp/libxipc/test_finder_events.cc,v 1.16 2005/03/25 02:53:28 pavlin Exp $"
 
 #include <list>
 #include <vector>
@@ -31,8 +31,8 @@
 
 #include "xrl_std_router.hh"
 
-#include "finder_event_notifier_xif.hh"
-#include "test_finder_events_base.hh"
+#include "xrl/interfaces/finder_event_notifier_xif.hh"
+#include "xrl/targets/test_finder_events_base.hh"
 
 ///////////////////////////////////////////////////////////////////////////////
 //
@@ -52,11 +52,12 @@ static const char *program_return_value = "0 on success, 1 if test error, 2 if i
 // Verbosity level control
 //
 
+
 static bool s_verbose = false;
 bool verbose()                  { return s_verbose; }
 void set_verbose(bool v)        { s_verbose = v; }
 
-#define verbose_log(x...) _verbose_log(__FILE__,__LINE__, x)
+#define verbose_log(x...) _verbose_log(__FUNCTION__,__LINE__, x)
 
 #define _verbose_log(file, line, x...)					\
 do {									\
@@ -72,6 +73,20 @@ do {									\
     }									\
 } while(0)
 
+static int ready = 0;
+static bool observing = true;
+#define WAIT_FOR_CALLBACK() 						\
+do {									\
+    ready++;								\
+    verbose_log("up ready: %d\n", ready);				\
+} while(0)
+
+#define CALLBACK_CALLED() 						\
+do {									\
+    ready--;								\
+    verbose_log("down ready: %d\n", ready);				\
+} while(0)
+
 
 /**
  * @short Test class that registers for event notifications with the finder.
@@ -96,10 +111,14 @@ public:
     {
 	XLOG_ASSERT(e == XrlError::OKAY());
 	_watches[class_name];
+
+	CALLBACK_CALLED();
     }
 
     void add_class_to_watch(const string& class_name)
     {
+	WAIT_FOR_CALLBACK();
+
 	XrlFinderEventNotifierV0p1Client cl(&_xrl_router);
 	bool r = cl.send_register_class_event_interest(
 			 "finder", _xrl_router.instance_name(), class_name,
@@ -116,10 +135,14 @@ public:
 	map<string,Watch>::iterator wi = _watches.find(class_name);
 	XLOG_ASSERT(wi != _watches.end());
 	_watches.erase(wi);
+
+	CALLBACK_CALLED();
     }
 
     void remove_class_to_watch(const string& class_name)
     {
+	WAIT_FOR_CALLBACK();
+
 	XrlFinderEventNotifierV0p1Client cl(&_xrl_router);
 	bool r = cl.send_deregister_class_event_interest(
 			 "finder", _xrl_router.instance_name(), class_name,
@@ -139,6 +162,7 @@ public:
 	w.instance_names.insert(instance_name);
 	verbose_log("Birth in class \"%s\", it's a \"%s\"\n",
 		    class_name.c_str(), instance_name.c_str());
+	CALLBACK_CALLED();
     }
 
     void death_event(const string& class_name, const string& instance_name)
@@ -151,6 +175,8 @@ public:
 	w.instance_names.erase(si);
 	verbose_log("Death in class \"%s\", it's a \"%s\"\n",
 		    class_name.c_str(), instance_name.c_str());
+
+	CALLBACK_CALLED();
     }
 
     uint32_t instance_count(const string& class_name)
@@ -381,6 +407,10 @@ create_xrl_target(EventLoop*		e,
 		  const char*		class_name,
 		  list<AnXrlTarget*>* store)
 {
+    if (observing) {
+	WAIT_FOR_CALLBACK();
+    }
+
     AnXrlTarget* a = new AnXrlTarget(*e, class_name, finder_addr, finder_port);
     store->push_back(a);
     verbose_log("Creating Xrl target \"%s\" in class \"%s\"\n",
@@ -390,6 +420,8 @@ create_xrl_target(EventLoop*		e,
 static void
 remove_oldest_xrl_target(list<AnXrlTarget*>* store)
 {
+    WAIT_FOR_CALLBACK();;
+
     XLOG_ASSERT(store != 0 && store->empty() == false);
     AnXrlTarget* a = store->front();
     store->pop_front();
@@ -401,6 +433,8 @@ remove_oldest_xrl_target(list<AnXrlTarget*>* store)
 static void
 remove_any_xrl_target(list<AnXrlTarget*>* store)
 {
+    WAIT_FOR_CALLBACK();;
+
     uint32_t n = store->size();
     XLOG_ASSERT(n > 0);
 
@@ -437,15 +471,23 @@ assert_xrl_target_count(list<AnXrlTarget*>* store,
 // cycles.
 //
 static void
-drip_run(EventLoop& e, list<OneoffTimerCallback>& locb)
+drip_run(EventLoop& e, list<OneoffTimerCallback>& locb, bool delay = false)
 {
-    static const uint32_t tstep = 500;
+    static const uint32_t tstep = 5000;
 
     while (locb.empty() == false) {
 	XorpTimer pause = e.new_oneoff_after_ms(tstep, callback(&no_op));
-	while (pause.scheduled()) {
+	while(pause.scheduled()) {
+	    if (!delay && 0 == ready)
+		    break;
 	    e.run();
 	}
+	verbose_log("Events: %u ready: %d scheduled: %d delay %d\n",
+		    XORP_UINT_CAST(e.timer_list_length()), ready,
+		    pause.scheduled(), delay);
+	if (0 != ready)
+	    XLOG_WARNING("ready is %d not zero", ready);
+	// XLOG_ASSERT(0 == ready);
 	locb.front()->dispatch();
 	locb.pop_front();
     }
@@ -462,7 +504,7 @@ test1(IPv4		finder_addr,
 
     FinderServer* fs = 0;
     if (use_internal_finder) {
-	fs = new FinderServer(e, finder_port, finder_addr);
+	fs = new FinderServer(e, finder_addr, finder_port);
     } else {
 	verbose_log("Using external Finder\n");
     }
@@ -491,7 +533,8 @@ test1(IPv4		finder_addr,
     locb.push_back(callback(remove_watch_from_observer, &pfeo, "class_a"));
     locb.push_back(callback(assert_observer_not_watching, &pfeo, "class_a"));
     locb.push_back(callback(destroy_observer, &pfeo));
-    locb.push_back(callback(assert_xrl_target_count, &tgt_store, 0u));
+    locb.push_back(callback(assert_xrl_target_count, &tgt_store,
+			    static_cast<uint32_t>(0u)));
 
     drip_run(e, locb);
 
@@ -509,7 +552,7 @@ test2(IPv4		finder_addr,
 
     FinderServer* fs = 0;
     if (use_internal_finder) {
-	fs = new FinderServer(e, finder_port, finder_addr);
+	fs = new FinderServer(e, finder_addr, finder_port);
     } else {
 	verbose_log("Using external Finder\n");
     }
@@ -543,21 +586,22 @@ test2(IPv4		finder_addr,
 				finder_addr, finder_port,
 				"class_a", &tgt_store));
 	locb.push_back(callback(assert_observer_class_instance_count,
-		     &pfeo, "class_a", 1u));
+				&pfeo, "class_a", static_cast<uint32_t>(1u)));
 	locb.push_back(callback(remove_oldest_xrl_target, &tgt_store));
 	locb.push_back(callback(assert_observer_class_instance_count,
-		     &pfeo, "class_a", 0u));
+				&pfeo, "class_a", static_cast<uint32_t>(0u)));
     }
 
     locb.push_back(callback(create_xrl_target, &e, finder_addr, finder_port,
 			  "class_b", &tgt_store));
     locb.push_back(callback(assert_observer_class_instance_count, &pfeo,
-			    "class_b", 0u));
+			    "class_b", static_cast<uint32_t>(0u)));
     locb.push_back(callback(remove_oldest_xrl_target, &tgt_store));
     locb.push_back(callback(remove_watch_from_observer, &pfeo, "class_a"));
     locb.push_back(callback(assert_observer_not_watching, &pfeo, "class_a"));
     locb.push_back(callback(destroy_observer, &pfeo));
-    locb.push_back(callback(assert_xrl_target_count, &tgt_store, 0u));
+    locb.push_back(callback(assert_xrl_target_count, &tgt_store,
+			    static_cast<uint32_t>(0u)));
 
     drip_run(e, locb);
 
@@ -575,7 +619,7 @@ test3(IPv4		finder_addr,
 
     FinderServer* fs = 0;
     if (use_internal_finder) {
-	fs = new FinderServer(e, finder_port, finder_addr);
+	fs = new FinderServer(e, finder_addr, finder_port);
     } else {
 	verbose_log("Using external Finder\n");
     }
@@ -616,11 +660,12 @@ test3(IPv4		finder_addr,
 		     "class_a", burst_cnt - i));
     }
     locb.push_back(callback(assert_observer_class_instance_count, &pfeo,
-			    "class_a", 0u));
+			    "class_a", static_cast<uint32_t>(0u)));
     locb.push_back(callback(remove_watch_from_observer, &pfeo, "class_a"));
     locb.push_back(callback(assert_observer_not_watching, &pfeo, "class_a"));
     locb.push_back(callback(destroy_observer, &pfeo));
-    locb.push_back(callback(assert_xrl_target_count, &tgt_store, 0u));
+    locb.push_back(callback(assert_xrl_target_count, &tgt_store,
+			    static_cast<uint32_t>(0u)));
 
     drip_run(e, locb);
 
@@ -638,7 +683,7 @@ test4(IPv4		finder_addr,
 
     FinderServer* fs = 0;
     if (use_internal_finder) {
-	fs = new FinderServer(e, finder_port, finder_addr);
+	fs = new FinderServer(e, finder_addr, finder_port);
     } else {
 	verbose_log("Using external Finder\n");
     }
@@ -661,7 +706,9 @@ test4(IPv4		finder_addr,
 				finder_port, "class_a", &tgt_store));
     }
 
-    drip_run(e, locb);
+    observing = false;
+    drip_run(e, locb, true);
+    observing = true;
 
     create_observer(&e, &pfeo, finder_addr, finder_port);
     while (pfeo->xrl_router_ready() == false &&
@@ -693,7 +740,8 @@ test4(IPv4		finder_addr,
     for (uint32_t i = 1; i <= burst_cnt ; i++) {
 	locb.push_back(callback(remove_any_xrl_target, &tgt_store));
     }
-    locb.push_back(callback(assert_xrl_target_count, &tgt_store, 0u));
+    locb.push_back(callback(assert_xrl_target_count, &tgt_store,
+			    static_cast<uint32_t>(0u)));
 
     drip_run(e, locb);
 
@@ -710,12 +758,34 @@ static int
 test_main(IPv4		 finder_addr,
 	  uint16_t	 finder_port,
 	  const uint32_t burst_cnt,
-	  bool		 use_internal_finder)
+	  bool		 use_internal_finder, int which)
 {
-    test1(finder_addr, finder_port, burst_cnt, use_internal_finder);
-    test2(finder_addr, finder_port, burst_cnt, use_internal_finder);
-    test3(finder_addr, finder_port, burst_cnt, use_internal_finder);
-    test4(finder_addr, finder_port, burst_cnt, use_internal_finder);
+    switch(which) {
+    case 0:
+	test1(finder_addr, finder_port, burst_cnt, use_internal_finder);
+	ready = 0;
+	test2(finder_addr, finder_port, burst_cnt, use_internal_finder);
+	ready = 0;
+	test3(finder_addr, finder_port, burst_cnt, use_internal_finder);
+	break;
+	ready = 0;
+	test4(finder_addr, finder_port, burst_cnt, use_internal_finder);
+	break;
+    case 1:
+	test1(finder_addr, finder_port, burst_cnt, use_internal_finder);
+	break;
+    case 2:
+	test2(finder_addr, finder_port, burst_cnt, use_internal_finder);
+	break;
+    case 3:
+	test3(finder_addr, finder_port, burst_cnt, use_internal_finder);
+	break;
+    case 4:
+	test4(finder_addr, finder_port, burst_cnt, use_internal_finder);
+	break;
+    default:
+	XLOG_FATAL("Unknown test");
+    }
 
     return 0;
 }
@@ -759,6 +829,8 @@ usage(const char* progname)
 	    "Set the number of simulated clients\n");
     fprintf(stderr, "  -r <n>                 "
 	    "Set the number of test runs\n");
+    fprintf(stderr, "  -t <n>                 "
+	    "Test number to run\n");
     fprintf(stderr, "  -h                     Display this information\n");
     fprintf(stderr, "  -v                     Verbose output\n");
 }
@@ -782,7 +854,7 @@ parse_finder_arg(const char* host_colon_port,
 	}
     } else {
 	finder_host = string(host_colon_port);
-	finder_port = FINDER_DEFAULT_PORT;
+	finder_port = FinderConstants::FINDER_DEFAULT_PORT();
     }
 
     try {
@@ -809,7 +881,7 @@ main(int argc, char * const argv[])
     //
     // Defaults
     //
-    IPv4	finder_addr = FINDER_DEFAULT_HOST;
+    IPv4	finder_addr = FinderConstants::FINDER_DEFAULT_HOST();
     uint16_t	finder_port = 16600;	// over-ridden for external finder
     bool	use_internal_finder = true;
     int		reps = 1;
@@ -821,15 +893,16 @@ main(int argc, char * const argv[])
     //
     if (burst_cnt > dtablesize / 20) {
 	burst_cnt = dtablesize / 20;
-	verbose_log("Cropped maximum burst count to %d "
+	verbose_log("Cropped maximum burst count to %u "
 		    "(descriptor table constraint).\n",
-		    burst_cnt);
+		    XORP_UINT_CAST(burst_cnt));
     }
 
     int ch;
     char* bp = NULL;
+    int which = 0;
 
-    while ((ch = getopt(argc, argv, "F:b:r:hv")) != -1) {
+    while ((ch = getopt(argc, argv, "F:b:r:t:hv")) != -1) {
         switch (ch) {
 	case 'F':
 	    if (parse_finder_arg(optarg, finder_addr, finder_port) == false) {
@@ -852,6 +925,9 @@ main(int argc, char * const argv[])
 		return 1;
 	    }
 	    break;
+	case 't':
+	    which = atoi(optarg);
+	    break;
         case 'v':
             set_verbose(true);
             break;
@@ -887,7 +963,7 @@ main(int argc, char * const argv[])
 	    verbose_log("============================\n");
 	    ret_value = test_main(finder_addr, finder_port,
 				  burst_cnt,
-				  use_internal_finder);
+				  use_internal_finder, which);
 	    if (ret_value != 0)
 		break;
 	    iter++;
