@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/xrl_atom.cc,v 1.12 2004/06/10 22:41:10 hodson Exp $"
+#ident "$XORP: xorp/libxipc/xrl_atom.cc,v 1.19 2005/03/25 02:53:31 pavlin Exp $"
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -465,7 +465,7 @@ XrlAtom::XrlAtom(const char* name, XrlAtomType t,
 const string
 XrlAtom::value() const
 {
-    static char tmp[32];
+    char tmp[32];
     tmp[0] = '\0';
 
     switch (_type) {
@@ -477,11 +477,13 @@ XrlAtom::value() const
 	return xrlatom_encode_value(tmp, strlen(tmp));
 	break;
     case xrlatom_int32:
-	snprintf(tmp, sizeof(tmp) / sizeof(tmp[0]), "%d", _i32val);
+	snprintf(tmp, sizeof(tmp) / sizeof(tmp[0]), "%d",
+		 XORP_INT_CAST(_i32val));
 	return xrlatom_encode_value(tmp, strlen(tmp));
 	break;
     case xrlatom_uint32:
-	snprintf(tmp, sizeof(tmp) / sizeof(tmp[0]), "%u", _u32val);
+	snprintf(tmp, sizeof(tmp) / sizeof(tmp[0]), "%u",
+		 XORP_UINT_CAST(_u32val));
 	return xrlatom_encode_value(tmp, strlen(tmp));
 	break;
     case xrlatom_ipv4:
@@ -573,7 +575,7 @@ XrlAtom::operator==(const XrlAtom& other) const
 	}
     }
     //    debug_msg("%d%d%d%d\n", mn, mt, md, mv);
-    return mn == mt == md == mv; // redundant return value
+    return ((mn == true) && (mt == true) && (md == true) && (mv == true));
 }
 
 // ----------------------------------------------------------------------------
@@ -886,7 +888,7 @@ XrlAtom::pack_list(uint8_t* buffer, size_t buffer_bytes) const
 {
     size_t done = 0;
 
-    size_t nelem = htonl(_list->size());
+    uint32_t nelem = htonl(_list->size());
     static_assert(sizeof(nelem) == 4);
 
     memcpy(buffer, &nelem, sizeof(nelem));
@@ -904,7 +906,7 @@ size_t
 XrlAtom::unpack_list(const uint8_t* buffer, size_t buffer_bytes)
 {
     size_t used = 0;
-    size_t nelem;
+    uint32_t nelem;
 
     if (buffer_bytes < sizeof(nelem)) {
 	return 0;
@@ -968,7 +970,7 @@ XrlAtom::pack(uint8_t* buffer, size_t buffer_bytes) const
     size_t pb = packed_bytes();
     if (buffer_bytes < pb) {
 	debug_msg("Buffer too small (%u < %u)\n",
-		  (uint32_t)buffer_bytes, (uint32_t)pb);
+		  XORP_UINT_CAST(buffer_bytes), XORP_UINT_CAST(pb));
 	return 0;
     }
 
@@ -1060,14 +1062,14 @@ XrlAtom::unpack(const uint8_t* buffer, size_t buffer_bytes)
 	_have_data = true;
 
 	debug_msg("Unpacked %u remain %u\n",
-		  static_cast<uint32_t>(unpacked),
-		  static_cast<uint32_t>(buffer_bytes));
+		  XORP_UINT_CAST(unpacked),
+		  XORP_UINT_CAST(buffer_bytes));
 
 	// Check size for fixed width packed types.
 	if (packed_bytes_fixed() && buffer_bytes < packed_bytes()) {
 	    debug_msg("Insufficient space (%u < %u) for type %d\n",
-		      static_cast<uint32_t>(buffer_bytes),
-		      static_cast<uint32_t>(packed_bytes()), _type);
+		      XORP_UINT_CAST(buffer_bytes),
+		      XORP_UINT_CAST(packed_bytes()), _type);
 	    _have_data = false;
 	    return 0;
 	}
