@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +12,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/xrl_std_router.cc,v 1.6 2004/06/10 22:41:13 hodson Exp $"
+#ident "$XORP: xorp/libxipc/xrl_std_router.cc,v 1.9 2005/03/25 02:53:34 pavlin Exp $"
 
 #include "xrl_std_router.hh"
+#include "xrl_pf_inproc.hh"
 #include "xrl_pf_stcp.hh"
+#include "xrl_pf_sudp.hh"
 
 // ----------------------------------------------------------------------------
 // Helper methods
@@ -23,7 +25,15 @@
 static XrlPFListener*
 create_listener(EventLoop& e, XrlDispatcher* d)
 {
-    // Change type here for your protocol family of choice
+    const char* pf = getenv("XORP_PF");
+    if (pf != NULL) {
+	if (pf[0] == 'i') {
+	    return new XrlPFInProcListener(e, d);
+	}
+	if (pf[0] == 'u') {
+	    return new XrlPFSUDPListener(e, d);
+	}
+    }
     return new XrlPFSTCPListener(e, d);
 }
 
@@ -39,7 +49,8 @@ destroy_listener(XrlPFListener*& l)
 
 XrlStdRouter::XrlStdRouter(EventLoop&	eventloop,
 			   const char*	class_name)
-    : XrlRouter(eventloop, class_name)
+    : XrlRouter(eventloop, class_name,  FinderConstants::FINDER_DEFAULT_HOST(),
+		FinderConstants::FINDER_DEFAULT_PORT())
 {
     _l = create_listener(eventloop, this);
     add_listener(_l);
@@ -48,7 +59,8 @@ XrlStdRouter::XrlStdRouter(EventLoop&	eventloop,
 XrlStdRouter::XrlStdRouter(EventLoop&	eventloop,
 			   const char*	class_name,
 			   IPv4		finder_address)
-    : XrlRouter(eventloop, class_name, finder_address)
+    : XrlRouter(eventloop, class_name, finder_address,
+		FinderConstants::FINDER_DEFAULT_PORT())
 {
     _l = create_listener(eventloop, this);
     add_listener(_l);
@@ -67,7 +79,8 @@ XrlStdRouter::XrlStdRouter(EventLoop&	eventloop,
 XrlStdRouter::XrlStdRouter(EventLoop&	eventloop,
 			   const char*	class_name,
 			   const char*	finder_address)
-    : XrlRouter(eventloop, class_name, finder_address)
+    : XrlRouter(eventloop, class_name, finder_address,
+		FinderConstants::FINDER_DEFAULT_PORT())
 {
     _l = create_listener(eventloop, this);
     add_listener(_l);
