@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/xrl_tokens.cc,v 1.4 2004/06/10 22:41:13 hodson Exp $"
+#ident "$XORP: xorp/libxipc/xrl_tokens.cc,v 1.5 2005/03/25 02:53:35 pavlin Exp $"
 
 #include "xrl_tokens.hh"
 
