@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
 // Portions of this code originally derived from:
 // 	FreeBSD dummynet code, (C) 2001 Luigi Rizzo.
 
-#ident "$XORP: xorp/libxorp/heap.cc,v 1.9 2004/06/10 22:41:15 hodson Exp $"
+#ident "$XORP: xorp/libxorp/heap.cc,v 1.11 2005/03/25 02:53:40 pavlin Exp $"
 
 #include <strings.h>
 #include "libxorp_module.h"
@@ -122,7 +122,7 @@ Heap::push(Heap_Key k, void *p, int son)
         int father = HEAP_FATHER(son) ;
         struct heap_entry tmp  ;
 
-        if ( _p[father].key < _p[son].key )
+        if ( _p[father].key <= _p[son].key )
             break ; /* found right position */
         /* son smaller than father, swap and repeat */
         HEAP_SWAP(_p[son], _p[father], tmp) ;
