@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,9 +12,13 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/mac.cc,v 1.6 2004/06/10 22:41:17 hodson Exp $"
+#ident "$XORP: xorp/libxorp/mac.cc,v 1.11 2005/03/25 02:53:42 pavlin Exp $"
 
+#include <vector>
+
+#include "libxorp_module.h"
 #include "xorp.h"
+#include "xlog.h"
 #include "mac.hh"
 #include "ether_compat.h" 
 
@@ -26,7 +30,7 @@ Mac::Mac(const string& s) throw (InvalidString)
 {
     // Empty string valid, nothing to do.
     if (_srep.empty())
-    return;
+	return;
 
     // ------------------------------------------------------------------------
     // I M P O R T A N T !
@@ -35,11 +39,34 @@ Mac::Mac(const string& s) throw (InvalidString)
     //
     // Add new MyMac::valid methods here
     // ------------------------------------------------------------------------
-    if (EtherMac::valid(_srep))
+    if (EtherMac::valid(_srep)) {
 	return;
+    }
 
     xorp_throw(InvalidString,
-	       c_format("Unknown Mac representation: %s", _srep.c_str()));
+	       c_format("Unknown Mac representation: %s", s.c_str()));
+}
+
+string
+Mac::normalized_str() const
+{
+    if (_srep.empty())
+	return _srep;	// XXX: the empty string is valid, so just return it
+
+    // ------------------------------------------------------------------------
+    // I M P O R T A N T !
+    //
+    // Check all known MAC instance classes for whether string is valid
+    // and return the corresponding normalized string.
+    //
+    // Add new MyMac::valid and MyMac::normalize() methods here
+    // ------------------------------------------------------------------------
+    if (EtherMac::valid(_srep)) {
+	return EtherMac::normalize(_srep);
+    }
+
+    XLOG_UNREACHABLE();
+    return (_srep);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -47,30 +74,38 @@ Mac::Mac(const string& s) throw (InvalidString)
 
 EtherMac::EtherMac(const string& s) throw (InvalidString)
 {
-    set_rep(s);
-    if (valid(_srep)) 
+    if (valid(s)) {
+	set_rep(s);
 	return;
+    }
+
     xorp_throw(InvalidString,
-	       c_format("Bad EtherMac representation: %s", _srep.c_str()));
+	       c_format("Bad EtherMac representation: %s", s.c_str()));
 }
 
 EtherMac::EtherMac(const Mac& m) throw (BadMac)
 {
-    set_rep(m.str());
-    if (valid(_srep))
+    string s = m.str();
+
+    if (valid(s)) {
+	set_rep(s);
 	return;
+    }
+
     xorp_throw(BadMac,
-	       c_format("Bad EtherMac representation: %s", _srep.c_str()));
+	       c_format("Bad EtherMac representation: %s", s.c_str()));
 }
 
 EtherMac::EtherMac(const struct ether_addr& ea) throw (BadMac)
 {
+    //
     // XXX: we need to const_cast the ether_ntoa() argument,
-    // because in some OS (e.g., MacOS X 10.2.3) the ether_ntoa(3)
+    // because on some OS (e.g., MacOS X 10.2.3) the ether_ntoa(3)
     // declaration is broken.
-    const char* a = ether_ntoa(const_cast<struct ether_addr *>(&ea));
-    if (a) {
-	set_rep(a);
+    //
+    const char* ap = ether_ntoa(const_cast<struct ether_addr *>(&ea));
+    if (ap != NULL) {
+	set_rep(ap);
 	return;
     }
 
@@ -91,13 +126,13 @@ EtherMac::get_ether_addr(struct ether_addr& ea) const
     // XXX: work-around because of broken ether_aton() declarations that
     // are missing the 'const' in the argument.
     //
-    char buf[_srep.size() + 1];
-    strncpy(buf, _srep.c_str(), sizeof(buf) - 1);
-    buf[sizeof(buf) - 1] = '\0';
+    vector<char> buf(_srep.size() + 1);
+    strncpy(&buf[0], _srep.c_str(), buf.size() - 1);
+    buf[buf.size() - 1] = '\0';
 
-    const struct ether_addr* p = ether_aton(buf);
-    if (p) {
-	memcpy(&ea, p, sizeof(ea));
+    const struct ether_addr* ep = ether_aton(&buf[0]);
+    if (ep != NULL) {
+	memcpy(&ea, ep, sizeof(ea));
 	return true;
     }
     return false;
@@ -110,10 +145,42 @@ EtherMac::valid(const string& s)
     // XXX: work-around because of broken ether_aton() declarations that
     // are missing the 'const' in the argument.
     //
-    char buf[s.size() + 1];
-    strncpy(buf, s.c_str(), sizeof(buf) - 1);
-    buf[sizeof(buf) - 1] = '\0';
+    vector<char> buf(s.size() + 1);
+    strncpy(&buf[0], s.c_str(), buf.size() - 1);
+    buf[buf.size() - 1] = '\0';
 
-    return ether_aton(buf) != 0;
+    return (ether_aton(&buf[0]) != NULL);
 }
 
+string
+EtherMac::normalize(const string& s) throw (InvalidString)
+{
+    //
+    // XXX: work-around because of broken ether_aton() declarations that
+    // are missing the 'const' in the argument.
+    //
+    vector<char> buf(s.size() + 1);
+    strncpy(&buf[0], s.c_str(), buf.size() - 1);
+    buf[buf.size() - 1] = '\0';
+
+    //
+    // Convert the string with an EtherMAC address into
+    // an "struct ether_addr", and then back to a string.
+    // Thus, the string address representation is normalized
+    // to the system's internal preference. Example:
+    // "00:00:00:00:00:00" -> "0:0:0:0:0:0"
+    //
+    struct ether_addr* ep;
+    ep = ether_aton(&buf[0]);
+    if (ep == NULL) {
+	xorp_throw(InvalidString,
+		   c_format("Bad EtherMac representation: %s", s.c_str()));
+    }
+    char* ap = ether_ntoa(ep);
+    if (ap == NULL) {
+	xorp_throw(InvalidString,
+		   c_format("Internal error: bad EtherMac representation: %s",
+			    s.c_str()));
+    }
+    return (string(ap));
+}
