@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +12,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/selector.cc,v 1.21 2004/06/10 22:41:18 hodson Exp $"
+#ident "$XORP: xorp/libxorp/selector.cc,v 1.24 2005/03/25 02:53:45 pavlin Exp $"
 
 #include "libxorp_module.h"
 #include "xorp.h"
 #include "debug.h"
 #include "selector.hh"
 #include "timeval.hh"
+#include "clock.hh"
 #include "utility.h"
 #include "xlog.h"
 
@@ -122,8 +123,8 @@ SelectorList::Node::is_empty()
 // ----------------------------------------------------------------------------
 // SelectorList implementation
 
-SelectorList::SelectorList()
-    : _maxfd(0), _descriptor_count(0), _observer(NULL)
+SelectorList::SelectorList(ClockBase *clock)
+    : _maxfd(0), _descriptor_count(0), _observer(NULL), _clock(clock)
 {
     static_assert(SEL_RD == (1 << SEL_RD_IDX) && SEL_WR == (1 << SEL_WR_IDX)
 		  && SEL_EX == (1 << SEL_EX_IDX) && SEL_MAX_IDX == 3);
@@ -172,7 +173,7 @@ void
 SelectorList::remove_selector(int fd, SelectorMask mask)
 {
     if (fd < 0 || fd >= (int)_selector_entries.size()) {
-	XLOG_ERROR("Attempting to remove fd = %d that is outside range of file descriptors 0..%u", fd,  (uint32_t)_selector_entries.size());
+	XLOG_ERROR("Attempting to remove fd = %d that is outside range of file descriptors 0..%u", fd,  XORP_UINT_CAST(_selector_entries.size()));
 	return;
     }
 
@@ -216,6 +217,10 @@ SelectorList::select(TimeVal* timeout)
 		     &tv_to);
     }
 
+    // We need to re-read the system wallclock time immediately
+    // after returning from a possibly blocking select() syscall.
+    _clock->advance_time();
+
     if (n < 0) {
 	switch (errno) {
 	case EBADF:
