@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipv4.cc,v 1.12 2004/06/10 22:41:20 hodson Exp $"
+#ident "$XORP: xorp/libxorp/test_ipv4.cc,v 1.15 2005/03/25 02:53:45 pavlin Exp $"
 
 #include "libxorp_module.h"
 #include "libxorp/xorp.h"
@@ -573,6 +573,9 @@ test_ipv4_address_const()
     verbose_assert(IPv4::ALL_ONES() == IPv4("255.255.255.255"),
 		   "ALL_ONES()");
 
+    verbose_assert(IPv4::LOOPBACK() == IPv4("127.0.0.1"),
+		   "LOOPBACK()");
+
     verbose_assert(IPv4::MULTICAST_BASE() == IPv4("224.0.0.0"),
 		   "MULTICAST_BASE()");
 
@@ -662,7 +665,7 @@ test_ipv4_invalid_manipulate_address()
 	// Invalid prefix length
 	IPv4 ip(IPv4::make_prefix(IPv4::addr_bitlen() + 1));
 	verbose_log("Cannot catch invalid IPv4 mask prefix with length %u : FAIL\n",
-		    (uint32_t)IPv4::addr_bitlen() + 1);
+		    XORP_UINT_CAST(IPv4::addr_bitlen() + 1));
 	incr_failures();
 	UNUSED(ip);
     } catch (const InvalidNetmaskLength& e) {
@@ -678,7 +681,7 @@ test_ipv4_invalid_manipulate_address()
 	IPv4 ip(addr_string4);
 	ip.mask_by_prefix_len(IPv4::addr_bitlen() + 1);
 	verbose_log("Cannot catch masking with an invalid IPv4 mask prefix with length %u : FAIL\n",
-		    (uint32_t)IPv4::addr_bitlen() + 1);
+		    XORP_UINT_CAST(IPv4::addr_bitlen() + 1));
 	incr_failures();
     } catch (const InvalidNetmaskLength& e) {
 	// The problem was caught
