@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipv6.cc,v 1.12 2004/06/10 22:41:20 hodson Exp $"
+#ident "$XORP: xorp/libxorp/test_ipv6.cc,v 1.15 2005/03/25 02:53:46 pavlin Exp $"
 
 #include "libxorp_module.h"
 #include "libxorp/xorp.h"
@@ -608,6 +608,9 @@ test_ipv6_address_const()
 		   IPv6("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"),
 		   "ALL_ONES()");
 
+    verbose_assert(IPv6::LOOPBACK() == IPv6("::1"),
+		   "LOOPBACK()");
+
     verbose_assert(IPv6::MULTICAST_BASE() == IPv6("ff00::"),
 		   "MULTICAST_BASE()");
 
@@ -712,7 +715,7 @@ test_ipv6_invalid_manipulate_address()
 	// Invalid prefix length
 	IPv6 ip(IPv6::make_prefix(IPv6::addr_bitlen() + 1));
 	verbose_log("Cannot catch invalid IPv6 mask prefix with length %u : FAIL\n",
-		    (uint32_t)IPv6::addr_bitlen() + 1);
+		    XORP_UINT_CAST(IPv6::addr_bitlen() + 1));
 	incr_failures();
 	UNUSED(ip);
     } catch (const InvalidNetmaskLength& e) {
@@ -728,7 +731,7 @@ test_ipv6_invalid_manipulate_address()
 	IPv6 ip(addr_string6);
 	ip.mask_by_prefix_len(IPv6::addr_bitlen() + 1);
 	verbose_log("Cannot catch masking with an invalid IPv6 mask prefix with length %u : FAIL\n",
-		    (uint32_t)IPv6::addr_bitlen() + 1);
+		    XORP_UINT_CAST(IPv6::addr_bitlen() + 1));
 	incr_failures();
     } catch (const InvalidNetmaskLength& e) {
 	// The problem was caught
