@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipvx.cc,v 1.11 2004/06/10 22:41:20 hodson Exp $"
+#ident "$XORP: xorp/libxorp/test_ipvx.cc,v 1.15 2005/03/25 02:53:46 pavlin Exp $"
 
 #include "libxorp_module.h"
 #include "libxorp/xorp.h"
@@ -693,7 +693,7 @@ test_ipvx_invalid_copy_in_out()
     // Mismatch copy-out: copy-out IPv6 address to in_addr structure.
     //
     try {
-	IPvX ip(*addr_string6);
+	IPvX ip(addr_string6);
 	struct in_addr in_addr;
 	ip.copy_out(in_addr);
 	verbose_log("Cannot catch mismatch copy-out : FAIL\n");
@@ -707,7 +707,7 @@ test_ipvx_invalid_copy_in_out()
     // Mismatch copy-out: copy-out IPv4 address to in_addr6 structure.
     //
     try {
-	IPvX ip(*addr_string4);
+	IPvX ip(addr_string4);
 	struct in6_addr in6_addr;
 	ip.copy_out(in6_addr);
 	verbose_log("Cannot catch mismatch copy-out : FAIL\n");
@@ -1046,6 +1046,11 @@ test_ipvx_address_const()
 		   IPvX("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"),
 		   "ALL_ONES()");
 
+    verbose_assert(IPvX::LOOPBACK(AF_INET) == IPvX("127.0.0.1"),
+		   "LOOPBACK()");
+    verbose_assert(IPvX::LOOPBACK(AF_INET6) == IPvX("::1"),
+		   "LOOPBACK()");
+
     verbose_assert(IPvX::MULTICAST_BASE(AF_INET) == IPvX("224.0.0.0"),
 		   "MULTICAST_BASE()");
     verbose_assert(IPvX::MULTICAST_BASE(AF_INET6) == IPvX("ff00::"),
@@ -1285,7 +1290,7 @@ test_ipvx_invalid_manipulate_address()
 	// Invalid prefix length: IPv4
 	IPvX ip(IPvX::make_prefix(AF_INET, IPvX::addr_bitlen(AF_INET) + 1));
 	verbose_log("Cannot catch invalid IPv4 mask prefix with length %u : FAIL\n",
-		    (uint32_t)IPvX::addr_bitlen(AF_INET) + 1);
+		    XORP_UINT_CAST(IPvX::addr_bitlen(AF_INET) + 1));
 	incr_failures();
 	UNUSED(ip);
     } catch (const InvalidNetmaskLength& e) {
@@ -1296,7 +1301,7 @@ test_ipvx_invalid_manipulate_address()
 	// Invalid prefix length: IPv6
 	IPvX ip(IPvX::make_prefix(AF_INET6, IPvX::addr_bitlen(AF_INET6) + 1));
 	verbose_log("Cannot catch invalid IPv6 mask prefix with length %u : FAIL\n",
-		    (uint32_t)IPvX::addr_bitlen(AF_INET6) + 1);
+		    XORP_UINT_CAST(IPvX::addr_bitlen(AF_INET6) + 1));
 	incr_failures();
 	UNUSED(ip);
     } catch (const InvalidNetmaskLength& e) {
@@ -1312,7 +1317,7 @@ test_ipvx_invalid_manipulate_address()
 	IPvX ip(addr_string4);
 	ip.mask_by_prefix_len(IPvX::addr_bitlen(AF_INET) + 1);
 	verbose_log("Cannot catch masking with an invalid IPv4 mask prefix with length %u : FAIL\n",
-		    (uint32_t)IPvX::addr_bitlen(AF_INET) + 1);
+		    XORP_UINT_CAST(IPvX::addr_bitlen(AF_INET) + 1));
 	incr_failures();
     } catch (const InvalidNetmaskLength& e) {
 	// The problem was caught
@@ -1323,7 +1328,7 @@ test_ipvx_invalid_manipulate_address()
 	IPvX ip(addr_string6);
 	ip.mask_by_prefix_len(IPvX::addr_bitlen(AF_INET6) + 1);
 	verbose_log("Cannot catch masking with an invalid IPv6 mask prefix with length %u : FAIL\n",
-		    (uint32_t)IPvX::addr_bitlen(AF_INET6) + 1);
+		    XORP_UINT_CAST(IPvX::addr_bitlen(AF_INET6) + 1));
 	incr_failures();
     } catch (const InvalidNetmaskLength& e) {
 	// The problem was caught
