@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_vif.cc,v 1.5 2004/06/10 22:41:21 hodson Exp $"
+#ident "$XORP: xorp/libxorp/test_vif.cc,v 1.9 2005/03/25 02:53:48 pavlin Exp $"
 
 #include "libxorp_module.h"
 #include "libxorp/xorp.h"
@@ -235,18 +235,21 @@ test_vif_addr_methods()
     //
     // Test whether is the same interface address.
     //
-    verbose_assert(vif_addr_b.is_same_addr(IPvX("22.22.22.22")),
-		   "is_same_addr()");
+    verbose_assert(vif_addr_b.is_my_addr(IPvX("22.22.22.22")),
+		   "is_my_addr()");
 
-    verbose_assert(! vif_addr_b.is_same_addr(IPvX("22.22.22.33")),
-		   "is_same_addr()");
+    verbose_assert(! vif_addr_b.is_my_addr(IPvX("22.22.22.33")),
+		   "is_my_addr()");
     
     //
-    // Test whether is the same subnet.
+    // Test whether a subnet address is a subset of my subnet address.
     //
     verbose_assert(vif_addr_b.is_same_subnet(IPvXNet("22.22.22.0/24")),
 		   "is_same_subnet(IPvXNet)");
 
+    verbose_assert(vif_addr_b.is_same_subnet(IPvXNet("22.22.22.128/25")),
+		   "is_same_subnet(IPvXNet)");
+
     verbose_assert(! vif_addr_b.is_same_subnet(IPvXNet("22.22.33.0/24")),
 		   "is_same_subnet(IPvXNet)");
     
@@ -375,6 +378,14 @@ test_vif_methods()
     vif1.set_loopback(false);
     verbose_assert(! vif1.is_loopback(), "is_loopback()");
 
+    //
+    // Test if this vif is a discard interface.
+    //
+    vif1.set_discard(true);
+    verbose_assert(vif1.is_discard(), "is_discard()");
+    vif1.set_discard(false);
+    verbose_assert(! vif1.is_discard(), "is_discard()");
+
     //
     // Test if this vif is multicast capable.
     //
@@ -432,6 +443,10 @@ test_vif_manipulate_address()
 		       IPvXNet("11.11.11.0/24"),
 		       IPvX("11.11.11.255"),
 		       IPvX("0.0.0.0"));
+    VifAddr vif_addr_aa(IPvX("11.11.11.11"),
+		       IPvXNet("11.11.11.128/25"),
+		       IPvX("11.11.11.127"),
+		       IPvX("0.0.0.0"));
     VifAddr vif_addr_b(IPvX("22.22.22.22"),
 		       IPvXNet("22.22.22.0/24"),
 		       IPvX("22.22.22.255"),
@@ -521,15 +536,19 @@ test_vif_manipulate_address()
     verbose_assert(! vif1.is_my_vif_addr(vif_addr_c), "is_my_vif_addr()");
     
     //
-    // Test if this vif belongs to a given subnet.
+    // Test if a subnet address is a subset of one of the subnet
+    // addresses of this vif.
     //
     verbose_assert(vif1.is_same_subnet(vif_addr_a.subnet_addr()),
 		   "is_same_subnet(IPvXNet)");
+    verbose_assert(vif1.is_same_subnet(vif_addr_aa.subnet_addr()),
+		   "is_same_subnet(IPvXNet)");
     verbose_assert(! vif1.is_same_subnet(IPvXNet("99.99.99.0/24")),
 		   "is_same_subnet(IPvXNet)");
 
     //
-    // Test if this vif belongs to a given subnet.
+    // Test if an IPvX address belongs to one of the subnet
+    // addresses of this vif.
     //
     verbose_assert(vif1.is_same_subnet(IPvX("11.11.11.22")),
 		   "is_same_subnet(IPvX)");
