@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2004 International Computer Science Institute
+ * Copyright (c) 2001-2005 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/libxorp/utility.h,v 1.8 2004/06/10 22:41:23 hodson Exp $
+ * $XORP: xorp/libxorp/utility.h,v 1.10 2005/03/25 02:53:49 pavlin Exp $
  */
 
 #ifndef __LIBXORP_UTILITY_H__
@@ -42,6 +42,27 @@
 #define cstring(s) (s).str().c_str()
 #endif
 
+/*
+ * XORP code uses typedefs (e.g. uint32_t, int32_t) rather than using
+ * the base types, because the 'C' language allows a compiler to use a
+ * natural size for base type. The XORP code is internally consistent
+ * in this usage, one problem arises with format strings in printf
+ * style functions.
+ *
+ * In order to print a size_t or uint32_t with "%u" it is necessary to
+ * cast to an unsigned int. On Mac OS X a size_t is not an unsigned
+ * int. On windows uint32_t is not an unsigned int.
+ *
+ * In order to print a int32_t with a "%d" it is necessary to cast to
+ * a signed int. On windows int32_t is not a signed int.
+ *
+ * The two convenience macros are provided to perform the cast.
+ */
+#ifdef __cplusplus
+#define	XORP_UINT_CAST(x)	static_cast<unsigned int>(x)
+#define	XORP_INT_CAST(x)	static_cast<int>(x)
+#endif
+
 #define ADD_POINTER(pointer, size, type)				\
 	((type)(((uint8_t *)(pointer)) + (size)))
 
