@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/vif.cc,v 1.11 2004/06/10 22:41:23 hodson Exp $"
+#ident "$XORP: xorp/libxorp/vif.cc,v 1.16 2005/03/25 02:53:49 pavlin Exp $"
 
 #include <functional>
 #include <string>
@@ -42,15 +42,13 @@ VifAddr::VifAddr(const IPvX& ipvx_addr, const IPvXNet& ipvxnet_subnet_addr,
 bool
 VifAddr::is_same_subnet(const IPvXNet& ipvxnet) const
 {
-    return (_subnet_addr == ipvxnet);
+    return (_subnet_addr.contains(ipvxnet));
 }
 
 bool
 VifAddr::is_same_subnet(const IPvX& ipvx_addr) const
 {
-    IPvXNet ipvx_net(ipvx_addr, subnet_addr().prefix_len());
-    
-    return (ipvx_net.masked_addr() == subnet_addr().masked_addr());
+    return (_subnet_addr.contains(ipvx_addr));
 }
 
 //
@@ -90,6 +88,7 @@ Vif::Vif(const string& vifname, const string& ifname)
     set_pim_register(false);
     set_p2p(false);
     set_loopback(false);
+    set_discard(false);
     set_multicast_capable(false);
     set_broadcast_capable(false);
     set_underlying_vif_up(false);
@@ -108,6 +107,7 @@ Vif::Vif(const Vif& vif)
     set_pim_register(vif.is_pim_register());
     set_p2p(vif.is_p2p());
     set_loopback(vif.is_loopback());
+    set_discard(vif.is_discard());
     set_multicast_capable(vif.is_multicast_capable());
     set_broadcast_capable(vif.is_broadcast_capable());
     set_underlying_vif_up(vif.is_underlying_vif_up());
@@ -159,6 +159,8 @@ Vif::str() const
 	r += " BROADCAST";
     if (is_loopback())
 	r += " LOOPBACK";
+    if (is_discard())
+	r += " DISCARD";
     if (is_underlying_vif_up())
 	r += " UNDERLYING_VIF_UP";
     
@@ -175,6 +177,7 @@ Vif::operator==(const Vif& other) const
 	    && (is_pim_register() == other.is_pim_register())
 	    && (is_p2p() == other.is_p2p())
 	    && (is_loopback() == other.is_loopback())
+	    && (is_discard() == other.is_discard())
 	    && (is_multicast_capable() == other.is_multicast_capable())
 	    && (is_broadcast_capable() == other.is_broadcast_capable())
 	    && (is_underlying_vif_up() == other.is_underlying_vif_up()));
@@ -226,7 +229,7 @@ Vif::delete_address(const IPvX& ipvx_addr)
     list<VifAddr>::iterator iter;
     
     for (iter = _addr_list.begin(); iter != _addr_list.end(); ++iter) {
-	if ((iter)->is_same_addr(ipvx_addr)) {
+	if ((iter)->is_my_addr(ipvx_addr)) {
 	    _addr_list.erase(iter);
 	    return (XORP_OK);
 	}
@@ -241,7 +244,7 @@ Vif::find_address(const IPvX& ipvx_addr)
     list<VifAddr>::iterator iter;
     
     for (iter = _addr_list.begin(); iter != _addr_list.end(); ++iter) {
-	if ((iter)->is_same_addr(ipvx_addr)) {
+	if ((iter)->is_my_addr(ipvx_addr)) {
 	    return &(*iter);
 	}
     }
@@ -255,7 +258,7 @@ Vif::is_my_addr(const IPvX& ipvx_addr) const
     list<VifAddr>::const_iterator iter;
     
     for (iter = _addr_list.begin(); iter != _addr_list.end(); ++iter) {
-	if ((iter)->is_same_addr(ipvx_addr)) {
+	if ((iter)->is_my_addr(ipvx_addr)) {
 	    return (true);
 	}
     }
@@ -320,7 +323,7 @@ Vif::is_same_p2p(const IPvX& ipvx_addr) const
 	return (false);
     
     for (iter = _addr_list.begin(); iter != _addr_list.end(); ++iter) {
-	if ((iter)->is_same_addr(ipvx_addr)
+	if ((iter)->is_my_addr(ipvx_addr)
 	    || ((iter)->peer_addr() == ipvx_addr)) {
 	    return (true);
 	}
