@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
-/* Copyright (c) 2001-2004 International Computer Science Institute
+/* Copyright (c) 2001-2005 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/libxorp/xlog.c,v 1.7 2004/06/10 22:41:23 hodson Exp $"
+#ident "$XORP: xorp/libxorp/xlog.c,v 1.8 2005/03/25 02:53:49 pavlin Exp $"
 
 
 /*
