@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2004 International Computer Science Institute
+ * Copyright (c) 2001-2005 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/libxorp/xorp.h,v 1.7 2004/06/10 22:41:23 hodson Exp $
+ * $XORP: xorp/libxorp/xorp.h,v 1.9 2005/03/25 02:53:50 pavlin Exp $
  */
 
 
@@ -47,8 +47,6 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <netinet/in.h>
-#include <net/route.h>
-#include <arpa/inet.h>  
 
 #include "utility.h"
 
