@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mibs/bgp4_mib_1657_bgppeertable.cc,v 1.14 2004/06/10 22:41:24 hodson Exp $"
+#ident "$XORP: xorp/mibs/bgp4_mib_1657_bgppeertable.cc,v 1.16 2005/03/25 02:53:52 pavlin Exp $"
 
 
 #include <net-snmp/net-snmp-config.h>
@@ -33,9 +33,9 @@
 typedef struct 
 {
     uint32_t peer_list_token;
-    IPv4     peer_local_ip;
+    string     peer_local_ip;
     uint32_t peer_local_port;
-    IPv4     peer_remote_ip;
+    string     peer_remote_ip;
     uint32_t peer_remote_port;
     bool     more;
     bool     valid;
@@ -45,8 +45,8 @@ typedef struct
 void free_context(void *, struct netsnmp_iterator_info_s *);
 void get_peer_list_start_done( const XrlError&, const uint32_t*, const bool*, 
     PeerLoopContext*);
-void get_peer_list_next_done(const XrlError&, const IPv4*, const uint32_t*,
-    const IPv4*, const uint32_t*, const bool*, PeerDataContext*); 
+void get_peer_list_next_done(const XrlError&, const string*, const uint32_t*,
+    const string*, const uint32_t*, const bool*, PeerDataContext*); 
 void get_peer_id_done(const XrlError&, const IPv4*, netsnmp_delegated_cache *);
 void get_peer_status_done(const XrlError&, const uint32_t*, const uint32_t*, 
     netsnmp_delegated_cache *);
@@ -230,7 +230,20 @@ bgpPeerTable_get_next_data_point(void **my_loop_context, void **my_data_context,
     // data_context goes away so store state in loop_context
     loop_context->more = data_context->more;
 
-    uint32_t raw_ip = ntohl(data_context->peer_remote_ip.addr());
+    // XXX - Atanu 2004-12-5
+    // When this code was originally written the peer_local_ip and
+    // peer_remote_ip variables were of type IPv4. Now that BGP can
+    // form IPv6 peerings its simpler to return a numeric version of
+    // the address that can be IPv4 or IPv6. Its not clear how to deal
+    // with an IPv6 address being returned. Its clearly wrong to
+    // return anyting to the high level code. At the momement lets
+    // just return a value of zero and hope that the higher level code
+    // sorts it out.
+
+    IPvX ip(data_context->peer_remote_ip.c_str());
+    uint32_t raw_ip = ip.is_ipv4() ? ip.get_ipv4().addr() : 0;
+
+    //    uint32_t raw_ip = ntohl(data_context->peer_remote_ip.addr());
 
     snmp_set_var_typed_value(put_index_data, ASN_IPADDRESS, 
 	reinterpret_cast<const u_char *> (&raw_ip), sizeof(uint32_t));
@@ -332,7 +345,14 @@ bgpPeerTable_handler(
 			}
                     case COLUMN_BGPPEERLOCALADDR:	// not delegated
  			{
-			uint32_t raw_ip = cntxt->peer_local_ip.addr();
+			    // XXX
+			    // See comment in bgpPeerTable_get_next_data_point
+
+			    IPvX ip(cntxt->peer_local_ip.c_str());
+			    uint32_t raw_ip = ip.is_ipv4() ?
+				ip.get_ipv4().addr() : 0;
+
+//			uint32_t raw_ip = cntxt->peer_local_ip.addr();
                         snmp_set_var_typed_value(var, ASN_IPADDRESS, 
 			    reinterpret_cast<const u_char *> (&raw_ip),
 			    sizeof(raw_ip));
@@ -348,7 +368,13 @@ bgpPeerTable_handler(
 			}
                     case COLUMN_BGPPEERREMOTEADDR:	// not delegated
  			{
-			uint32_t raw_ip = cntxt->peer_remote_ip.addr();
+			    // XXX
+			    // See comment in bgpPeerTable_get_next_data_point
+
+			    IPvX ip(cntxt->peer_remote_ip.c_str());
+			    uint32_t raw_ip = ip.is_ipv4() ?
+				ip.get_ipv4().addr() : 0;
+//			uint32_t raw_ip = cntxt->peer_remote_ip.addr();
                         snmp_set_var_typed_value(var, ASN_IPADDRESS, 
 			    reinterpret_cast<const u_char *>(&raw_ip), 
 			    sizeof(uint32_t));
@@ -462,9 +488,9 @@ get_peer_list_start_done(
 
 void get_peer_list_next_done(
     const XrlError& e, 
-    const IPv4* local_ip, 
+    const string* local_ip, 
     const uint32_t* local_port, 
-    const IPv4* remote_ip, 
+    const string* remote_ip, 
     const uint32_t* remote_port,
     const bool* more,
     PeerDataContext* data_context)
@@ -477,7 +503,7 @@ void get_peer_list_next_done(
 	data_context->more = (*more);
 	data_context->valid = true;
 	DEBUGMSGTL((BgpMib::the_instance().name(),
-                "local_ip: %s more: %d\n", local_ip->str().c_str(), *more));		 
+                "local_ip: %s more: %d\n", local_ip->c_str(), *more));		 
     } else {
     // XXX: deal with retries
     }
