@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mibs/test_xorpevents.cc,v 1.12 2004/06/10 22:41:25 hodson Exp $"
+#ident "$XORP: xorp/mibs/test_xorpevents.cc,v 1.14 2005/03/25 02:53:53 pavlin Exp $"
 
 
 #include <set>
@@ -348,7 +348,9 @@ run_test_2()
     // test we create a Finder in process as we can't guarantee Finder
     // is already running.  Most XORP processes do not have to do this.
     
-    FinderServer* finder = new FinderServer(e);
+    FinderServer* finder = new FinderServer(e,
+					    FinderConstants::FINDER_DEFAULT_HOST(),
+					    FinderConstants::FINDER_DEFAULT_PORT());
     
     // Create and configure "party_A"
     XrlRouter		party_a(e, "party_A", finder->addr(), finder->port());
