@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mld6igmp/mld6_proto.cc,v 1.29 2004/06/10 22:41:26 hodson Exp $"
+#ident "$XORP: xorp/mld6igmp/mld6_proto.cc,v 1.33 2005/03/25 02:53:54 pavlin Exp $"
 
 
 //
@@ -22,9 +22,15 @@
 
 
 #include "mld6igmp_module.h"
-#include "mld6igmp_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "mrt/inet_cksum.h"
+
 #include "mld6_proto.h"
+#include "mld6igmp_node.hh"
 #include "mld6igmp_vif.hh"
 
 
@@ -60,7 +66,7 @@
  * it should be ignored.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * it should be ignored.
- * @router_alert_bool: True if the received IP packet had the Router Alert
+ * @is_router_alert: True if the received IP packet had the Router Alert
  * IP option set.
  * @buffer: The buffer with the message.
  * 
@@ -72,7 +78,7 @@ int
 Mld6igmpVif::mld6_process(const IPvX& src, const IPvX& dst,
 			  int ip_ttl,
 			  int ip_tos,
-			  bool router_alert_bool,
+			  bool is_router_alert,
 			  buffer_t *buffer)
 {
     uint8_t message_type;
@@ -89,7 +95,7 @@ Mld6igmpVif::mld6_process(const IPvX& src, const IPvX& dst,
 		     module_name(),
 		     cstring(src), cstring(dst),
 		     name().c_str(),
-		     (uint32_t)BUFFER_DATA_SIZE(buffer));
+		     XORP_UINT_CAST(BUFFER_DATA_SIZE(buffer)));
 	return (XORP_ERROR);
     }
     
@@ -150,11 +156,11 @@ Mld6igmpVif::mld6_process(const IPvX& src, const IPvX& dst,
     
     //
     // TODO: if we are running in secure mode, then check ip_ttl, ip_tos and
-    // @router_alert_bool (e.g. (ip_ttl == MINTTL) && (router_alert_bool))
+    // @is_router_alert (e.g. (ip_ttl == MINTTL) && (is_router_alert))
     //
     UNUSED(ip_ttl);
     UNUSED(ip_tos);
-    UNUSED(router_alert_bool);
+    UNUSED(is_router_alert);
 #if 0
     //
     // TTL (aka. Hop-limit in IPv6) and Router Alert option check.
