@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mld6igmp/mld6igmp_node.cc,v 1.30 2004/06/10 22:41:26 hodson Exp $"
+#ident "$XORP: xorp/mld6igmp/mld6igmp_node.cc,v 1.44 2005/03/25 02:53:54 pavlin Exp $"
 
 
 //
@@ -23,7 +23,11 @@
 
 
 #include "mld6igmp_module.h"
-#include "mld6igmp_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "mld6igmp_node.hh"
 #include "mld6igmp_vif.hh"
 
@@ -63,7 +67,7 @@
 Mld6igmpNode::Mld6igmpNode(int family, xorp_module_id module_id,
 			   EventLoop& eventloop)
     : ProtoNode<Mld6igmpVif>(family, module_id, eventloop),
-      _is_log_trace(true)		// XXX: default to print trace logs
+      _is_log_trace(false)
 {
     XLOG_ASSERT(module_id == XORP_MODULE_MLD6IGMP);
     if (module_id != XORP_MODULE_MLD6IGMP) {
@@ -122,9 +126,20 @@ Mld6igmpNode::~Mld6igmpNode()
 int
 Mld6igmpNode::start()
 {
-    if (is_up() || is_pending_up())
+    if (! is_enabled())
 	return (XORP_OK);
 
+    //
+    // Test the service status
+    //
+    if ((ServiceBase::status() == SERVICE_STARTING)
+	|| (ServiceBase::status() == SERVICE_RUNNING)) {
+	return (XORP_OK);
+    }
+    if (ServiceBase::status() != SERVICE_READY) {
+	return (XORP_ERROR);
+    }
+
     if (ProtoNode<Mld6igmpVif>::pending_start() < 0)
 	return (XORP_ERROR);
 
@@ -137,6 +152,10 @@ Mld6igmpNode::start()
     // Set the node status
     //
     ProtoNode<Mld6igmpVif>::set_node_status(PROC_STARTUP);
+
+    //
+    // Update the node status
+    //
     update_status();
 
     return (XORP_OK);
@@ -166,6 +185,8 @@ Mld6igmpNode::final_start()
     // Start the mld6igmp_vifs
     start_all_vifs();
 
+    XLOG_INFO("Protocol started");
+
     return (XORP_OK);
 }
 
@@ -174,13 +195,8 @@ Mld6igmpNode::final_start()
  * @: 
  * 
  * Gracefully stop the MLD or IGMP protocol.
- * XXX: This function, unlike start(), will stop the protocol
- * operation on all interfaces.
  * XXX: After the cleanup is completed,
  * Mld6igmpNode::final_stop() is called to complete the job.
- * XXX: If this method is called one-after-another, the second one
- * will force calling immediately Mld6igmpNode::final_stop() to quickly
- * finish the job. TODO: is this a desired semantic?
  * XXX: This function, unlike start(), will stop the protocol
  * operation on all interfaces.
  * 
@@ -189,12 +205,25 @@ Mld6igmpNode::final_start()
 int
 Mld6igmpNode::stop()
 {
-    if (is_down())
+    //
+    // Test the service status
+    //
+    if ((ServiceBase::status() == SERVICE_SHUTDOWN)
+	|| (ServiceBase::status() == SERVICE_SHUTTING_DOWN)
+	|| (ServiceBase::status() == SERVICE_FAILED)) {
 	return (XORP_OK);
-
-    if (! (is_up() || is_pending_up() || is_pending_down()))
+    }
+    if ((ServiceBase::status() != SERVICE_RUNNING)
+	&& (ServiceBase::status() != SERVICE_STARTING)
+	&& (ServiceBase::status() != SERVICE_PAUSING)
+	&& (ServiceBase::status() != SERVICE_PAUSED)
+	&& (ServiceBase::status() != SERVICE_RESUMING)) {
 	return (XORP_ERROR);
-    
+    }
+
+    if (ProtoNode<Mld6igmpVif>::pending_stop() < 0)
+	return (XORP_ERROR);
+
     //
     // Perform misc. MLD6IGMP-specific stop operations
     //
@@ -203,18 +232,14 @@ Mld6igmpNode::stop()
     // Stop the vifs
     stop_all_vifs();
     
-    if (ProtoNode<Mld6igmpVif>::pending_stop() < 0)
-	return (XORP_ERROR);
-
-    //
-    // De-register with the MFEA
-    //
-    mfea_register_shutdown();
-
     //
     // Set the node status
     //
     ProtoNode<Mld6igmpVif>::set_node_status(PROC_SHUTDOWN);
+
+    //
+    // Update the node status
+    //
     update_status();
 
     return (XORP_OK);
@@ -237,48 +262,37 @@ Mld6igmpNode::final_stop()
     if (ProtoNode<Mld6igmpVif>::stop() < 0)
 	return (XORP_ERROR);
 
+    XLOG_INFO("Protocol stopped");
+
     return (XORP_OK);
 }
 
 /**
- * Mld6igmpNode::has_pending_down_units:
- * @reason_msg: return-by-reference string that contains human-readable
- * information about the status.
+ * Enable the node operation.
  * 
- * Test if there is an unit that is in PENDING_DOWN state.
- * 
- * Return value: True if there is an unit that is in PENDING_DOWN state,
- * otherwise false.
- **/
-bool
-Mld6igmpNode::has_pending_down_units(string& reason_msg)
+ * If an unit is not enabled, it cannot be start, or pending-start.
+ */
+void
+Mld6igmpNode::enable()
 {
-    vector<Mld6igmpVif *>::iterator iter;
-    
-    //
-    // Test the interfaces
-    //
-    for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
-	Mld6igmpVif *mld6igmp_vif = (*iter);
-	if (mld6igmp_vif == NULL)
-	    continue;
-	// TODO: XXX: PAVPAVPAV: vif pending-down state
-	// is not used/implemented yet
-	if (mld6igmp_vif->is_pending_down()) {
-	    reason_msg = c_format("Vif %s is in state %s",
-				  mld6igmp_vif->name().c_str(),
-				  mld6igmp_vif->state_str().c_str());
-	    return (true);
-	}
-    }
-    
-    //
-    // TODO: XXX: PAVPAVPAV: test other units that may be waiting
-    // in PENDING_DOWN state.
-    //
-    
-    reason_msg = "No pending-down units";
-    return (false);
+    ProtoUnit::enable();
+
+    XLOG_INFO("Protocol enabled");
+}
+
+/**
+ * Disable the node operation.
+ * 
+ * If an unit is disabled, it cannot be start or pending-start.
+ * If the unit was runnning, it will be stop first.
+ */
+void
+Mld6igmpNode::disable()
+{
+    stop();
+    ProtoUnit::disable();
+
+    XLOG_INFO("Protocol disabled");
 }
 
 void
@@ -288,7 +302,8 @@ Mld6igmpNode::status_change(ServiceBase*  service,
 {
     XLOG_ASSERT(this == service);
 
-    if ((old_status == STARTING) && (new_status == RUNNING)) {
+    if ((old_status == SERVICE_STARTING)
+	&& (new_status == SERVICE_RUNNING)) {
 	// The startup process has completed
 	if (final_start() < 0) {
 	    XLOG_ERROR("Cannot complete the startup process; "
@@ -300,7 +315,8 @@ Mld6igmpNode::status_change(ServiceBase*  service,
 	return;
     }
 
-    if ((old_status == SHUTTING_DOWN) && (new_status == SHUTDOWN)) {
+    if ((old_status == SERVICE_SHUTTING_DOWN)
+	&& (new_status == SERVICE_SHUTDOWN)) {
 	// The shutdown process has completed
 	final_stop();
 	// Set the node status
@@ -339,8 +355,27 @@ Mld6igmpNode::add_vif(const Vif& vif, string& error_msg)
 	delete mld6igmp_vif;
 	return (XORP_ERROR);
     }
-    
-    XLOG_INFO("New vif: %s", mld6igmp_vif->str().c_str());
+
+    //
+    // Update and check the primary address
+    //
+    do {
+	if (mld6igmp_vif->update_primary_address(error_msg) == XORP_OK)
+	    break;
+	if (mld6igmp_vif->addr_ptr() == NULL) {
+	    // XXX: don't print an error if the vif has no addresses
+	    break;
+	}
+	if (mld6igmp_vif->is_loopback() || mld6igmp_vif->is_pim_register()) {
+	    // XXX: don't print an error if this is a loopback or register_vif
+	    break;
+	}
+	XLOG_ERROR("Error updating primary address for vif %s: %s",
+		   mld6igmp_vif->name().c_str(), error_msg.c_str());
+	return (XORP_ERROR);
+    } while (false);
+
+    XLOG_INFO("Interface added: %s", mld6igmp_vif->str().c_str());
     
     return (XORP_OK);
 }
@@ -407,7 +442,7 @@ Mld6igmpNode::delete_vif(const string& vif_name, string& error_msg)
     
     delete mld6igmp_vif;
     
-    XLOG_INFO("Deleted vif: %s", vif_name.c_str());
+    XLOG_INFO("Interface deleted: %s", vif_name.c_str());
     
     return (XORP_OK);
 }
@@ -455,7 +490,7 @@ Mld6igmpNode::set_vif_flags(const string& vif_name,
     }
     
     if (is_changed)
-	XLOG_INFO("Vif flags changed: %s", mld6igmp_vif->str().c_str());
+	XLOG_INFO("Interface flags changed: %s", mld6igmp_vif->str().c_str());
     
     return (XORP_OK);
 }
@@ -525,7 +560,26 @@ Mld6igmpNode::add_vif_addr(const string& vif_name,
 	XLOG_INFO("Added new address to vif %s: %s",
 		  mld6igmp_vif->name().c_str(), vif_addr.str().c_str());
     }
-    
+
+    //
+    // Update and check the primary address
+    //
+    do {
+	if (mld6igmp_vif->update_primary_address(error_msg) == XORP_OK)
+	    break;
+	if (! (mld6igmp_vif->is_up() || mld6igmp_vif->is_pending_up())) {
+	    // XXX: print an error only if the interface is UP or PENDING_UP
+	    break;
+	}
+	if (mld6igmp_vif->is_loopback() || mld6igmp_vif->is_pim_register()) {
+	    // XXX: don't print an error if this is a loopback or register_vif
+	    break;
+	}
+	XLOG_ERROR("Error updating primary address for vif %s: %s",
+		   mld6igmp_vif->name().c_str(), error_msg.c_str());
+	return (XORP_ERROR);
+    } while (false);
+
     return (XORP_OK);
 }
 
@@ -567,22 +621,28 @@ Mld6igmpNode::delete_vif_addr(const string& vif_name,
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("Deleted address on vif %s: %s",
+    XLOG_INFO("Deleted address on interface %s: %s",
 	      mld6igmp_vif->name().c_str(), vif_addr.str().c_str());
     
     //
-    // Update the primary address
+    // Update and check the primary address.
     // If the vif has no more primary address, then stop it.
     //
-    mld6igmp_vif->update_primary_address(error_msg);
-    if (mld6igmp_vif->is_up()) {
-	// Check the primary address
-	if (mld6igmp_vif->primary_addr() == IPvX::ZERO(family())) {
-	    XLOG_ERROR("Cannot update primary address: %s",
-		       error_msg.c_str());
-	    mld6igmp_vif->stop(error_msg);
+    do {
+	if (mld6igmp_vif->update_primary_address(error_msg) == XORP_OK)
+	    break;
+	if (! (mld6igmp_vif->is_up() || mld6igmp_vif->is_pending_up())) {
+	    // XXX: don't do anything if the interface is not UP or PENDING_UP
+	    break;
 	}
-    }
+	if (mld6igmp_vif->is_loopback() || mld6igmp_vif->is_pim_register()) {
+	    // XXX: don't do anything if this is a loopback or register_vif
+	    break;
+	}
+	XLOG_ERROR("Error updating primary address for vif %s: %s",
+		   mld6igmp_vif->name().c_str(), error_msg.c_str());
+	mld6igmp_vif->stop(error_msg);
+    } while (false);
 
     return (XORP_OK);
 }
@@ -609,8 +669,6 @@ Mld6igmpNode::enable_vif(const string& vif_name, string& error_msg)
     
     mld6igmp_vif->enable();
     
-    XLOG_INFO("Enabled vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -636,8 +694,6 @@ Mld6igmpNode::disable_vif(const string& vif_name, string& error_msg)
     
     mld6igmp_vif->disable();
     
-    XLOG_INFO("Disabled vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -668,8 +724,6 @@ Mld6igmpNode::start_vif(const string& vif_name, string& error_msg)
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("Started vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -700,8 +754,6 @@ Mld6igmpNode::stop_vif(const string& vif_name, string& error_msg)
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("Stopped vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -711,31 +763,24 @@ Mld6igmpNode::stop_vif(const string& vif_name, string& error_msg)
  * 
  * Start MLD/IGMP on all enabled interfaces.
  * 
- * Return value: The number of virtual interfaces MLD/IGMP was started on,
- * or %XORP_ERROR if error occured.
+ * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
 Mld6igmpNode::start_all_vifs()
 {
-    int n = 0;
     vector<Mld6igmpVif *>::iterator iter;
     string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	Mld6igmpVif *mld6igmp_vif = (*iter);
 	if (mld6igmp_vif == NULL)
 	    continue;
-	if (! mld6igmp_vif->is_enabled())
-	    continue;
-	if (mld6igmp_vif->start(error_msg) != XORP_OK) {
-	    XLOG_ERROR("Cannot start vif %s: %s",
-		       mld6igmp_vif->name().c_str(), error_msg.c_str());
-	} else {
-	    n++;
-	}
+	if (start_vif(mld6igmp_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (n);
+    return (ret_value);
 }
 
 /**
@@ -744,29 +789,24 @@ Mld6igmpNode::start_all_vifs()
  * 
  * Stop MLD/IGMP on all interfaces it was running on.
  * 
- * Return value: The number of virtual interfaces MLD/IGMP was stopped on,
- * or %XORP_ERROR if error occured.
+ * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
 Mld6igmpNode::stop_all_vifs()
 {
-    int n = 0;
     vector<Mld6igmpVif *>::iterator iter;
     string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	Mld6igmpVif *mld6igmp_vif = (*iter);
 	if (mld6igmp_vif == NULL)
 	    continue;
-	if (mld6igmp_vif->stop(error_msg) != XORP_OK) {
-	    XLOG_ERROR("Cannot stop vif %s: %s",
-		       mld6igmp_vif->name().c_str(), error_msg.c_str());
-	} else {
-	    n++;
-	}
+	if (stop_vif(mld6igmp_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (n);
+    return (ret_value);
 }
 
 /**
@@ -781,15 +821,18 @@ int
 Mld6igmpNode::enable_all_vifs()
 {
     vector<Mld6igmpVif *>::iterator iter;
+    string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	Mld6igmpVif *mld6igmp_vif = (*iter);
 	if (mld6igmp_vif == NULL)
 	    continue;
-	mld6igmp_vif->enable();
+	if (enable_vif(mld6igmp_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (XORP_OK);
+    return (ret_value);
 }
 
 /**
@@ -805,17 +848,18 @@ int
 Mld6igmpNode::disable_all_vifs()
 {
     vector<Mld6igmpVif *>::iterator iter;
-    
-    stop_all_vifs();
+    string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	Mld6igmpVif *mld6igmp_vif = (*iter);
 	if (mld6igmp_vif == NULL)
 	    continue;
-	mld6igmp_vif->disable();
+	if (disable_vif(mld6igmp_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (XORP_OK);
+    return (ret_value);
 }
 
 /**
@@ -858,6 +902,38 @@ Mld6igmpNode::delete_all_vifs()
     }
 }
 
+/**
+ * A method called when a vif has completed its shutdown.
+ * 
+ * @param vif_name the name of the vif that has completed its shutdown.
+ */
+void
+Mld6igmpNode::vif_shutdown_completed(const string& vif_name)
+{
+    vector<Mld6igmpVif *>::iterator iter;
+
+    //
+    // If all vifs have completed the shutdown, then de-register with
+    // the MFEA.
+    //
+    for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
+	Mld6igmpVif *mld6igmp_vif = *iter;
+	if (mld6igmp_vif == NULL)
+	    continue;
+	if (! mld6igmp_vif->is_down())
+	    return;
+    }
+
+    //
+    // De-register with the MFEA
+    //
+    if (ServiceBase::status() == SERVICE_SHUTTING_DOWN) {
+	mfea_register_shutdown();
+    }
+
+    UNUSED(vif_name);
+}
+
 /**
  * Mld6igmpNode::proto_recv:
  * @src_module_instance_name: The module instance name of the module-origin
@@ -870,7 +946,7 @@ Mld6igmpNode::delete_all_vifs()
  * it should be ignored.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * it should be ignored.
- * @router_alert_bool: If true, the Router Alert IP option for the IP
+ * @is_router_alert: If true, the Router Alert IP option for the IP
  * packet of the incoming message was set.
  * @rcvbuf: The data buffer with the received message.
  * @rcvlen: The data length in @rcvbuf.
@@ -884,7 +960,7 @@ Mld6igmpNode::proto_recv(const string&	, // src_module_instance_name,
 			 xorp_module_id src_module_id,
 			 uint16_t vif_index,
 			 const IPvX& src, const IPvX& dst,
-			 int ip_ttl, int ip_tos, bool router_alert_bool,
+			 int ip_ttl, int ip_tos, bool is_router_alert,
 			 const uint8_t *rcvbuf, size_t rcvlen)
 {
     Mld6igmpVif *mld6igmp_vif = NULL;
@@ -893,7 +969,7 @@ Mld6igmpNode::proto_recv(const string&	, // src_module_instance_name,
     debug_msg("Received message from %s to %s: "
 	      "ip_ttl = %d ip_tos = %#x router_alert = %d rcvlen = %u\n",
 	      cstring(src), cstring(dst),
-	      ip_ttl, ip_tos, router_alert_bool, (uint32_t)rcvlen);
+	      ip_ttl, ip_tos, is_router_alert, XORP_UINT_CAST(rcvlen));
     
     //
     // Check whether the node is up.
@@ -914,7 +990,7 @@ Mld6igmpNode::proto_recv(const string&	, // src_module_instance_name,
     
     // Process the data by the vif
     ret_value = mld6igmp_vif->mld6igmp_recv(src, dst, ip_ttl, ip_tos,
-					    router_alert_bool,
+					    is_router_alert,
 					    _buffer_recv);
     
     return (ret_value);
@@ -935,7 +1011,7 @@ Mld6igmpNode::proto_recv(const string&	, // src_module_instance_name,
  * the TTL will be set by the lower layers.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * the TOS will be set by the lower layers.
- * @router_alert_bool: If true, set the Router Alert IP option for the IP
+ * @is_router_alert: If true, set the Router Alert IP option for the IP
  * packet of the outgoung message.
  * @buffer: The #buffer_t data buffer with the message to send.
  * 
@@ -946,7 +1022,7 @@ Mld6igmpNode::proto_recv(const string&	, // src_module_instance_name,
 int
 Mld6igmpNode::mld6igmp_send(uint16_t vif_index,
 			    const IPvX& src, const IPvX& dst,
-			    int ip_ttl, int ip_tos, bool router_alert_bool,
+			    int ip_ttl, int ip_tos, bool is_router_alert,
 			    buffer_t *buffer)
 {
     if (! is_up())
@@ -956,7 +1032,7 @@ Mld6igmpNode::mld6igmp_send(uint16_t vif_index,
     if (proto_send(xorp_module_name(family(), XORP_MODULE_MFEA),
 		   XORP_MODULE_MFEA,
 		   vif_index, src, dst,
-		   ip_ttl, ip_tos, router_alert_bool,
+		   ip_ttl, ip_tos, is_router_alert,
 		   BUFFER_DATA_HEAD(buffer),
 		   BUFFER_DATA_SIZE(buffer)) < 0) {
 	return (XORP_ERROR);
