@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mld6igmp/mld6igmp_vif.cc,v 1.29 2004/06/18 21:51:52 pavlin Exp $"
+#ident "$XORP: xorp/mld6igmp/mld6igmp_vif.cc,v 1.37 2005/03/25 02:53:55 pavlin Exp $"
 
 
 //
@@ -21,8 +21,13 @@
 
 
 #include "mld6igmp_module.h"
-#include "mld6igmp_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "mrt/inet_cksum.h"
+
 #include "mld6igmp_vif.hh"
 
 
@@ -170,6 +175,9 @@ Mld6igmpVif::proto_is_ssm() const
 int
 Mld6igmpVif::start(string& error_msg)
 {
+    if (! is_enabled())
+	return (XORP_OK);
+
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
@@ -208,9 +216,6 @@ Mld6igmpVif::start(string& error_msg)
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("STARTING %s%s",
-	      this->str().c_str(), flags_string().c_str());
-    
     //
     // Join the appropriate multicast groups: ALL-SYSTEMS and ALL-ROUTERS
     //
@@ -258,6 +263,9 @@ Mld6igmpVif::start(string& error_msg)
 		callback(this, &Mld6igmpVif::query_timer_timeout));
     }
 #endif // HAVE_IPV6_MULTICAST_ROUTING
+
+    XLOG_INFO("Interface started: %s%s",
+	      this->str().c_str(), flags_string().c_str());
     
     return (XORP_OK);
 }
@@ -282,6 +290,11 @@ Mld6igmpVif::stop(string& error_msg)
 	error_msg = "the vif state is not UP or PENDING_UP or PENDING_DOWN";
 	return (XORP_ERROR);
     }
+
+    if (ProtoUnit::pending_stop() < 0) {
+	error_msg = "internal error";
+	ret_value = XORP_ERROR;
+    }
     
     //
     // XXX: we don't have to explicitly leave the multicast groups
@@ -315,24 +328,19 @@ Mld6igmpVif::stop(string& error_msg)
     // Stop the vif with the kernel
     //
     if (mld6igmp_node().stop_protocol_kernel_vif(vif_index()) != XORP_OK) {
-	error_msg = c_format("cannot stop protocol vif %s with the kernel",
-			     name().c_str());
-	return (XORP_ERROR);
+	XLOG_ERROR("Cannot stop protocol vif %s with the kernel",
+		   name().c_str());
+	ret_value = XORP_ERROR;
     }
     
-    XLOG_INFO("STOPPED %s%s",
+    XLOG_INFO("Interface stopped: %s%s",
 	      this->str().c_str(), flags_string().c_str());
 
     //
-    // Test if time to completely stop the Mld6igmpNode itself because
-    // of this vif
+    // Inform the node that the vif has completed the shutdown
     //
-    string dummy_string;
-    if (mld6igmp_node().is_pending_down()
-	&& (! mld6igmp_node().has_pending_down_units(dummy_string))) {
-	mld6igmp_node().final_stop();
-    }
-    
+    mld6igmp_node().vif_shutdown_completed(name());
+
     return (ret_value);
 }
 
@@ -345,6 +353,9 @@ void
 Mld6igmpVif::enable()
 {
     ProtoUnit::enable();
+
+    XLOG_INFO("Interface enabled: %s%s",
+	      this->str().c_str(), flags_string().c_str());
 }
 
 /**
@@ -360,6 +371,9 @@ Mld6igmpVif::disable()
 
     stop(error_msg);
     ProtoUnit::disable();
+
+    XLOG_INFO("Interface disabled: %s%s",
+	      this->str().c_str(), flags_string().c_str());
 }
 
 /**
@@ -466,7 +480,7 @@ Mld6igmpVif::mld6igmp_send(const IPvX& src,
  * it should be ignored.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * it should be ignored.
- * @router_alert_bool: True if the received IP packet had the Router Alert
+ * @is_router_alert: True if the received IP packet had the Router Alert
  * IP option set.
  * @buffer: The buffer with the received message.
  * 
@@ -479,7 +493,7 @@ Mld6igmpVif::mld6igmp_recv(const IPvX& src,
 			   const IPvX& dst,
 			   int ip_ttl,
 			   int ip_tos,
-			   bool router_alert_bool,
+			   bool is_router_alert,
 			   buffer_t *buffer)
 {
     int ret_value = XORP_ERROR;
@@ -490,13 +504,13 @@ Mld6igmpVif::mld6igmp_recv(const IPvX& src,
     do {
 	if (proto_is_igmp()) {
 	    ret_value = igmp_process(src, dst, ip_ttl, ip_tos,
-				     router_alert_bool, buffer);
+				     is_router_alert, buffer);
 	    break;
 	}
 #ifdef HAVE_IPV6_MULTICAST_ROUTING
 	if (proto_is_mld6()) {
 	    ret_value = mld6_process(src, dst, ip_ttl, ip_tos,
-				     router_alert_bool, buffer);
+				     is_router_alert, buffer);
 	    break;
 	}
 #endif
@@ -558,7 +572,6 @@ Mld6igmpVif::update_primary_address(string& error_msg)
 
     //
     // Check that the interface has a primary address.
-    // addresses.
     //
     if (primary_a == IPvX::ZERO(family())) {
 	error_msg = "invalid primary address";
