@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_bsr.cc,v 1.27 2004/06/10 22:41:29 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_bsr.cc,v 1.36 2005/03/25 02:53:58 pavlin Exp $"
 
 
 //
@@ -24,9 +24,14 @@
 #include <math.h>
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
 #include "libxorp/utils.hh"
+
 #include "mrt/random.h"
+
 #include "pim_proto.h"
 #include "pim_bsr.hh"
 #include "pim_node.hh"
@@ -76,6 +81,15 @@ PimBsr::PimBsr(PimNode& pim_node)
  * 
  **/
 PimBsr::~PimBsr()
+{
+    clear();
+}
+
+/**
+ * Clear the entry.
+ */
+void
+PimBsr::clear()
 {
     stop();
     
@@ -97,6 +111,9 @@ PimBsr::~PimBsr()
 int
 PimBsr::start()
 {
+    if (! is_enabled())
+	return (XORP_OK);
+
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
@@ -124,7 +141,9 @@ PimBsr::start()
 	}
 	config_bsr_zone->start_candidate_rp_advertise_timer();
     }
-    
+
+    XLOG_INFO("Bootstrap mechanism started");
+
     return (XORP_OK);
 }
 
@@ -153,11 +172,14 @@ PimBsr::stop()
     
     //
     // Find the first vif that is UP, and use it to unicast the Cand-RP-Adv
-    // messages.
+    // messages. Note that the PIM Register vif is excluded.
+    //
     for (uint16_t i = 0; i < pim_node().maxvifs(); i++) {
 	pim_vif_up = pim_node().vif_find_by_vif_index(i);
 	if (pim_vif_up == NULL)
 	    continue;
+	if (pim_vif_up->is_pim_register())
+	    continue;
 	if (pim_vif_up->is_up())
 	    break;
 	pim_vif_up = NULL;
@@ -263,10 +285,40 @@ PimBsr::stop()
     // because it is a one-time only timers that is used to
     // synchronize internal state.
     //
+
+    XLOG_INFO("Bootstrap mechanism stopped");
     
     return (XORP_OK);
 }
 
+/**
+ * Enable the node operation.
+ * 
+ * If an unit is not enabled, it cannot be start, or pending-start.
+ */
+void
+PimBsr::enable()
+{
+    ProtoUnit::enable();
+
+    XLOG_INFO("Bootstrap mechanism enabled");
+}
+
+/**
+ * Disable the node operation.
+ * 
+ * If an unit is disabled, it cannot be start or pending-start.
+ * If the unit was runnning, it will be stop first.
+ */
+void
+PimBsr::disable()
+{
+    stop();
+    ProtoUnit::disable();
+
+    XLOG_INFO("Bootstrap mechanism disabled");
+}
+
 // Unicast the Bootstrap message(s) to a (new) neighbor
 int
 PimBsr::unicast_pim_bootstrap(PimVif *pim_vif, const IPvX& nbr_addr) const
@@ -362,7 +414,7 @@ PimBsr::add_active_bsr_zone(const BsrZone& bsr_zone, string& error_msg)
 	 ++iter_prefix) {
 	BsrGroupPrefix *bsr_group_prefix = *iter_prefix;
 	BsrGroupPrefix *active_bsr_group_prefix = active_bsr_zone->find_bsr_group_prefix(bsr_group_prefix->group_prefix());
-	if (active_bsr_zone == NULL)
+	if (active_bsr_group_prefix == NULL)
 	    continue;
 	list<BsrRp *>::const_iterator iter_rp;
 	for (iter_rp = bsr_group_prefix->rp_list().begin();
@@ -1066,12 +1118,14 @@ PimBsr::send_test_cand_rp_adv()
     
     //
     // Find the first vif that is UP, and use it to unicast the Cand-RP-Adv
-    // messages.
+    // messages. Note that the PIM Register vif is excluded.
     //
     for (uint16_t i = 0; i < pim_node().maxvifs(); i++) {
 	pim_vif = pim_node().vif_find_by_vif_index(i);
 	if (pim_vif == NULL)
 	    continue;
+	if (pim_vif->is_pim_register())
+	    continue;	
 	if (! pim_vif->is_up())
 	    continue;
 	break;	// Found
@@ -1447,7 +1501,7 @@ BsrZone::can_merge_rp_set(const BsrZone& bsr_zone, string& error_msg) const
 				 "for zone %s is too large: %u while "
 				 "the expected count is %u",
 				 cstring(zone_id()),
-				 rp_count_sum,
+				 XORP_UINT_CAST(rp_count_sum),
 				 active_bsr_group_prefix->expected_rp_count());
 	    return (false);
 	}
@@ -2298,12 +2352,14 @@ BsrZone::candidate_rp_advertise_timer_timeout()
 	
 	//
 	// Find the first vif that is UP, and use it to unicast the Cand-RP-Adv
-	// messages.
+	// messages. Note that the PIM Register vif is excluded.
 	//
 	for (uint16_t i = 0; i < pim_bsr().pim_node().maxvifs(); i++) {
 	    pim_vif = pim_bsr().pim_node().vif_find_by_vif_index(i);
 	    if (pim_vif == NULL)
 		continue;
+	    if (pim_vif->is_pim_register())
+		continue;
 	    if (! pim_vif->is_up())
 		continue;
 	    break;	// Found
