@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_config.cc,v 1.25 2004/06/10 22:41:30 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_config.cc,v 1.29 2005/03/25 02:53:58 pavlin Exp $"
 
 
 //
@@ -21,7 +21,11 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "mrt/random.h"
 
 #include "pim_node.hh"
@@ -1559,6 +1563,63 @@ PimNode::delete_config_static_rp(const IPvXNet& group_prefix,
     return (XORP_OK);
 }
 
+//
+// Delete a statically configured RP for all group prefixes
+//
+// Return: %XORP_OK on success, otherwise %XORP_ERROR.
+//
+// XXX: we don't call end_config(), because config_static_rp_done() will
+// do it when the RP configuration is completed.
+//
+int
+PimNode::delete_config_all_static_group_prefixes_rp(const IPvX& rp_addr,
+						    string& error_msg)
+{
+    if (start_config(error_msg) != XORP_OK)
+	return (XORP_ERROR);
+    
+    if (rp_table().delete_all_group_prefixes_rp(rp_addr,
+						PimRp::RP_LEARNED_METHOD_STATIC)
+	!= XORP_OK) {
+	// XXX: don't call end_config(error_msg);
+	error_msg = c_format("Cannot delete configure static RP with address %s",
+			     cstring(rp_addr));
+	XLOG_ERROR(error_msg.c_str());
+	return (XORP_ERROR);
+    }
+    
+    // XXX: config_static_rp_done() will complete the configuration setup
+    
+    return (XORP_OK);
+}
+
+//
+// Delete all statically configured RPs
+//
+// Return: %XORP_OK on success, otherwise %XORP_ERROR.
+//
+// XXX: we don't call end_config(), because config_static_rp_done() will
+// do it when the RP configuration is completed.
+//
+int
+PimNode::delete_config_all_static_rps(string& error_msg)
+{
+    if (start_config(error_msg) != XORP_OK)
+	return (XORP_ERROR);
+    
+    if (rp_table().delete_all_rps(PimRp::RP_LEARNED_METHOD_STATIC)
+	!= XORP_OK) {
+	// XXX: don't call end_config(error_msg);
+	error_msg = c_format("Cannot delete configure all static RPs");
+	XLOG_ERROR(error_msg.c_str());
+	return (XORP_ERROR);
+    }
+    
+    // XXX: config_static_rp_done() will complete the configuration setup
+    
+    return (XORP_OK);
+}
+
 //
 // Finish with the static RP configuration.
 //
