@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_mre_rpf.cc,v 1.27 2004/06/10 22:41:31 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_mre_rpf.cc,v 1.32 2005/03/25 02:54:00 pavlin Exp $"
 
 //
 // PIM Multicast Routing Entry RPF handling
@@ -20,7 +20,11 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "pim_mre.hh"
 #include "pim_mrt.hh"
 #include "pim_nbr.hh"
@@ -216,13 +220,6 @@ PimMre::set_nbr_mrib_next_hop_rp(PimNbr *v)
     if ((old_pim_nbr == NULL) && (! is_pim_nbr_missing())) {
 	pim_node().delete_pim_mre_no_pim_nbr(this);
     }
-    
-    //
-    // Set RPF'(*,G) if necessary
-    //
-    if (is_wc()) {
-	set_rpfp_nbr_wc(compute_rpfp_nbr_wc());
-    }
 }
 
 // Note: applies only for (S,G)
@@ -255,13 +252,6 @@ PimMre::set_nbr_mrib_next_hop_s(PimNbr *v)
     if ((old_pim_nbr == NULL) && (! is_pim_nbr_missing())) {
 	pim_node().delete_pim_mre_no_pim_nbr(this);
     }
-    
-    //
-    // Set RPF'(S,G) if necessary
-    //
-    if (is_sg()) {
-	set_rpfp_nbr_sg(compute_rpfp_nbr_sg());
-    }
 }
 
 // Note: applies only for (*,G)
@@ -843,7 +833,7 @@ PimMre::compute_rpfp_nbr_sg_rpt() const
 // Note: applies only for (*,*,RP) entries
 //
 void
-PimMre::recompute_nbr_mrib_next_hop_rp_changed()
+PimMre::recompute_nbr_mrib_next_hop_rp_rp_changed()
 {
     PimNbr *old_pim_nbr, *new_pim_nbr;
     uint16_t join_prune_period = PIM_JOIN_PRUNE_PERIOD_DEFAULT;
@@ -941,6 +931,48 @@ PimMre::recompute_nbr_mrib_next_hop_rp_gen_id_changed()
     }
 }
 
+//
+// The NBR(RPF_interface(RP(G)), MRIB.next_hop(RP(G))) has changed.
+// Take the appropriate action.
+// Note: applies only for (*,G) entries
+//
+void
+PimMre::recompute_nbr_mrib_next_hop_rp_wc_changed()
+{
+    PimNbr *old_pim_nbr, *new_pim_nbr;
+    
+    if (! is_wc())
+	return;
+
+    old_pim_nbr = nbr_mrib_next_hop_rp();
+    new_pim_nbr = compute_nbr_mrib_next_hop_rp();
+    if (old_pim_nbr == new_pim_nbr)
+	return;				// Nothing changed
+
+    set_nbr_mrib_next_hop_rp(new_pim_nbr);
+}
+
+//
+// The NBR(RPF_interface(S), MRIB.next_hop(S)) has changed.
+// Take the appropriate action.
+// Note: applies only for (S,G) entries
+//
+void
+PimMre::recompute_nbr_mrib_next_hop_s_changed()
+{
+    PimNbr *old_pim_nbr, *new_pim_nbr;
+    
+    if (! is_sg())
+	return;
+
+    old_pim_nbr = nbr_mrib_next_hop_s();
+    new_pim_nbr = compute_nbr_mrib_next_hop_s();
+    if (old_pim_nbr == new_pim_nbr)
+	return;				// Nothing changed
+
+    set_nbr_mrib_next_hop_s(new_pim_nbr);
+}
+
 //
 // The current next hop towards the RP has changed due to an Assert.
 // Take the appropriate action.
@@ -1004,7 +1036,7 @@ PimMre::recompute_rpfp_nbr_wc_not_assert_changed()
     if (! is_wc())
 	return;
     
-    new_pim_nbr = compute_rpfp_nbr_wc();    
+    new_pim_nbr = compute_rpfp_nbr_wc();
     
     if (is_joined_state())
 	goto joined_state_label;
