@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_mre_task.cc,v 1.10 2004/06/10 22:41:31 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_mre_task.cc,v 1.14 2005/03/25 02:54:00 pavlin Exp $"
 
 //
 // PIM Multicast Routing Entry task
@@ -20,9 +20,13 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
 #include "libxorp/time_slice.hh"
 #include "libxorp/utils.hh"
+
 #include "pim_mfc.hh"
 #include "pim_mre_task.hh"
 #include "pim_mrt.hh"
@@ -194,18 +198,6 @@ PimMreTask::family() const
     return (_pim_mrt.family());
 }
 
-//
-// (Re)schedule the task to run (again).
-// 
-void
-PimMreTask::schedule_task()
-{
-    _time_slice_timer =
-	pim_node().eventloop().new_oneoff_after(
-	    TimeVal(0, 1),
-	    callback(this, &PimMreTask::time_slice_timer_timeout));
-}
-
 //
 // Run the task.
 //
@@ -215,15 +207,17 @@ bool
 PimMreTask::run_task()
 {
     _time_slice.reset();
-    
+
     if (run_task_rp()) {
-	// Time slice has expired. Schedule a new time slice.
-	schedule_task();
+	// The time slice has expired. Keep processing this task.
 	return (true);
     }
-    
+
+    //
+    // The task has been completed, hence delete the task.
+    //
     delete this;	// XXX: should be right before the return
-    
+
     return (false);
 }
 
@@ -639,7 +633,7 @@ PimMreTask::run_task_wc()
 			time_slice_expired = true;
 		}
 		if (time_slice_expired || _time_slice.is_expired()) {
-		    // Time slice has expired. Save state to continue late
+		    // Time slice has expired. Save state to continue later
 		    // from here.
 		    _is_processing_wc = true;
 		    _is_processing_pim_nbr_addr_wc = true;
@@ -1678,12 +1672,3 @@ PimMreTask::add_pim_mfc_delete(PimMfc *pim_mfc)
 {
     _pim_mfc_delete_list.push_back(pim_mfc);
 }
-
-//
-// A timeout handler to continue processing a task.
-//
-void
-PimMreTask::time_slice_timer_timeout()
-{
-    run_task();
-}
