@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_mrt.cc,v 1.6 2004/06/10 22:41:31 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_mrt.cc,v 1.9 2005/03/25 02:54:01 pavlin Exp $"
 
 //
 // PIM Multicast Routing Table implementation.
@@ -20,10 +20,15 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
 #include "libxorp/utils.hh"
+
 #include "pim_mfc.hh"
 #include "pim_mre.hh"
+#include "pim_mre_task.hh"
 #include "pim_mrt.hh"
 #include "pim_node.hh"
 #include "pim_vif.hh"
@@ -64,7 +69,7 @@ PimMrt::PimMrt(PimNode& pim_node)
 
 PimMrt::~PimMrt()
 {
-    
+    clear();
 }
 
 PimMrtSg::PimMrtSg(PimMrt& pim_mrt)
@@ -147,6 +152,18 @@ PimMrt::pim_register_vif_index() const
     return (pim_node().pim_register_vif_index());
 }
 
+void
+PimMrt::clear()
+{
+    delete_pointers_list(_pim_mre_task_list);
+
+    _pim_mrt_mfc.clear();
+    _pim_mrt_sg.clear();
+    _pim_mrt_sg_rpt.clear();
+    _pim_mrt_g.clear();
+    _pim_mrt_rp.clear();
+}
+
 //
 // XXX: @create_flags must be a subset of @lookup_flags
 // XXX: if no creation allowed, the entry that it returns may be the
