@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_node.cc,v 1.49 2004/06/10 22:41:32 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_node.cc,v 1.68 2005/03/25 02:54:01 pavlin Exp $"
 
 
 //
@@ -23,8 +23,13 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "fea/mfea_kernel_messages.hh"		// TODO: XXX: yuck!
+
 #include "pim_mre.hh"
 #include "pim_node.hh"
 #include "pim_vif.hh"
@@ -68,14 +73,13 @@ PimNode::PimNode(int family, xorp_module_id module_id,
     : ProtoNode<PimVif>(family, module_id, eventloop),
       _pim_mrt(*this),
       _pim_mrib_table(*this),
-      _pim_bsr(*this),
       _rp_table(*this),
       _pim_scope_zone_table(*this),
+      _pim_bsr(*this),
       _is_switch_to_spt_enabled(false),	// XXX: disabled by defailt
       _switch_to_spt_threshold_interval_sec(0),
       _switch_to_spt_threshold_bytes(0),
-      _is_receive_mrib_from_mfea(false),
-      _is_log_trace(true),		// XXX: default to print trace logs
+      _is_log_trace(false),
       _test_jp_header(*this)
 {
     // TODO: XXX: PIMDM not implemented yet
@@ -119,6 +123,22 @@ PimNode::~PimNode()
 
     stop();
 
+    //
+    // XXX: Explicitly clear the PimBsr and RpTable now to avoid
+    // cross-referencing of lists that may be deleted prematurely
+    // at the end of the PimNode destructor.
+    //
+    _pim_bsr.clear();
+    _rp_table.clear();
+
+    //
+    // XXX: explicitly clear the PimMrt table now, because PimMrt may utilize
+    // some lists in the PimNode class (e.g., _processing_pim_nbr_list) that
+    // may be deleted prematurely at the end of the PimNode destructor
+    // (depending on the declaration ordering).
+    //
+    _pim_mrt.clear();
+    
     ProtoNode<PimVif>::set_node_status(PROC_NULL);
 
     delete_all_vifs();
@@ -141,9 +161,20 @@ PimNode::~PimNode()
 int
 PimNode::start()
 {
-    if (is_up() || is_pending_up())
+    if (! is_enabled())
 	return (XORP_OK);
 
+    //
+    // Test the service status
+    //
+    if ((ServiceBase::status() == SERVICE_STARTING)
+	|| (ServiceBase::status() == SERVICE_RUNNING)) {
+	return (XORP_OK);
+    }
+    if (ServiceBase::status() != SERVICE_READY) {
+	return (XORP_ERROR);
+    }
+
     if (ProtoNode<PimVif>::pending_start() < 0)
 	return (XORP_ERROR);
 
@@ -161,6 +192,10 @@ PimNode::start()
     // Set the node status
     //
     ProtoNode<PimVif>::set_node_status(PROC_STARTUP);
+
+    //
+    // Update the node status
+    //
     update_status();
 
     return (XORP_OK);
@@ -190,11 +225,11 @@ PimNode::final_start()
     // Start the pim_vifs
     start_all_vifs();
     
-    // Start the BSR module only if we are enabled to run the BSR mechanism
-    if (_pim_bsr.is_enabled()) {
-	if (_pim_bsr.start() < 0)
-	    return (XORP_ERROR);
-    }
+    // Start the BSR module
+    if (_pim_bsr.start() < 0)
+	return (XORP_ERROR);
+
+    XLOG_INFO("Protocol started");
 
     return (XORP_OK);
 }
@@ -209,9 +244,6 @@ PimNode::final_start()
  * state with neighbors.
  * XXX: After the multicast routing entries cleanup is completed,
  * PimNode::final_stop() is called to complete the job.
- * XXX: If this method is called one-after-another, the second one
- * will force calling immediately PimNode::final_stop() to quickly
- * finish the job. TODO: is this a desired semantic?
  * XXX: This function, unlike start(), will stop the protocol
  * operation on all interfaces.
  * 
@@ -220,12 +252,25 @@ PimNode::final_start()
 int
 PimNode::stop()
 {
-    if (is_down())
+    //
+    // Test the service status
+    //
+    if ((ServiceBase::status() == SERVICE_SHUTDOWN)
+	|| (ServiceBase::status() == SERVICE_SHUTTING_DOWN)
+	|| (ServiceBase::status() == SERVICE_FAILED)) {
 	return (XORP_OK);
-
-    if (! (is_up() || is_pending_up() || is_pending_down()))
+    }
+    if ((ServiceBase::status() != SERVICE_RUNNING)
+	&& (ServiceBase::status() != SERVICE_STARTING)
+	&& (ServiceBase::status() != SERVICE_PAUSING)
+	&& (ServiceBase::status() != SERVICE_PAUSED)
+	&& (ServiceBase::status() != SERVICE_RESUMING)) {
 	return (XORP_ERROR);
-    
+    }
+
+    if (ProtoNode<PimVif>::pending_stop() < 0)
+	return (XORP_ERROR);
+
     //
     // Perform misc. PIM-specific stop operations
     //
@@ -234,23 +279,14 @@ PimNode::stop()
     // Stop the vifs
     stop_all_vifs();
     
-    if (ProtoNode<PimVif>::pending_stop() < 0)
-	return (XORP_ERROR);
-
-    //
-    // De-register with the RIB
-    //
-    rib_register_shutdown();
-
-    //
-    // De-register with the MFEA
-    //
-    mfea_register_shutdown();
-
     //
     // Set the node status
     //
     ProtoNode<PimVif>::set_node_status(PROC_SHUTDOWN);
+
+    //
+    // Update the node status
+    //
     update_status();
 
     return (XORP_OK);
@@ -273,48 +309,37 @@ PimNode::final_stop()
     if (ProtoNode<PimVif>::stop() < 0)
 	return (XORP_ERROR);
 
+    XLOG_INFO("Protocol stopped");
+
     return (XORP_OK);
 }
 
 /**
- * PimNode::has_pending_down_units:
- * @reason_msg: return-by-reference string that contains human-readable
- * information about the status.
+ * Enable the node operation.
  * 
- * Test if there is an unit that is in PENDING_DOWN state.
- * 
- * Return value: True if there is an unit that is in PENDING_DOWN state,
- * otherwise false.
- **/
-bool
-PimNode::has_pending_down_units(string& reason_msg)
+ * If an unit is not enabled, it cannot be start, or pending-start.
+ */
+void
+PimNode::enable()
 {
-    vector<PimVif *>::iterator iter;
-    
-    //
-    // Test the interfaces
-    //
-    for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
-	PimVif *pim_vif = (*iter);
-	if (pim_vif == NULL)
-	    continue;
-	if (pim_vif->is_pending_down()) {
-	    reason_msg = c_format("Vif %s is in state %s: remaining %u tasks",
-				  pim_vif->name().c_str(),
-				  pim_vif->state_str().c_str(),
-				  (uint32_t)pim_vif->usage_by_pim_mre_task());
-	    return (true);
-	}
-    }
-    
-    //
-    // TODO: XXX: PAVPAVPAV: test other units that may be waiting
-    // in PENDING_DOWN state: PimMrt, PimMribTable, PimBsr, RpTable,
-    // PimScopeZoneTable, etc.
-    //
-    
-    reason_msg = "No pending-down units";
-    return (false);
+    ProtoUnit::enable();
+
+    XLOG_INFO("Protocol enabled");
+}
+
+/**
+ * Disable the node operation.
+ * 
+ * If an unit is disabled, it cannot be start or pending-start.
+ * If the unit was runnning, it will be stop first.
+ */
+void
+PimNode::disable()
+{
+    stop();
+    ProtoUnit::disable();
+
+    XLOG_INFO("Protocol disabled");
 }
 
 void
@@ -324,7 +349,8 @@ PimNode::status_change(ServiceBase*  service,
 {
     XLOG_ASSERT(this == service);
 
-    if ((old_status == STARTING) && (new_status == RUNNING)) {
+    if ((old_status == SERVICE_STARTING)
+	&& (new_status == SERVICE_RUNNING)) {
 	// The startup process has completed
 	if (final_start() < 0) {
 	    XLOG_ERROR("Cannot complete the startup process; "
@@ -336,7 +362,8 @@ PimNode::status_change(ServiceBase*  service,
 	return;
     }
 
-    if ((old_status == SHUTTING_DOWN) && (new_status == SHUTDOWN)) {
+    if ((old_status == SERVICE_SHUTTING_DOWN)
+	&& (new_status == SERVICE_SHUTDOWN)) {
 	// The shutdown process has completed
 	final_stop();
 	// Set the node status
@@ -387,7 +414,29 @@ PimNode::add_vif(const Vif& vif, string& error_msg)
     _pim_mrib_table.resolve_prefixes_by_vif_name(pim_vif->name(),
 						 pim_vif->vif_index());
 
-    XLOG_INFO("New vif: %s", pim_vif->str().c_str());
+    //
+    // Update and check the primary and domain-wide addresses
+    //
+    do {
+	if (pim_vif->update_primary_and_domain_wide_address(error_msg)
+	    == XORP_OK) {
+	    break;
+	}
+	if (pim_vif->addr_ptr() == NULL) {
+	    // XXX: don't print an error if the vif has no addresses
+	    break;
+	}
+	if (pim_vif->is_loopback()) {
+	    // XXX: don't print an error if this is a loopback interface
+	    break;
+	}
+	XLOG_ERROR("Error updating primary and domain-wide addresses "
+		   "for vif %s: %s",
+		   pim_vif->name().c_str(), error_msg.c_str());
+	return (XORP_ERROR);
+    } while (false);
+
+    XLOG_INFO("Interface added: %s", pim_vif->str().c_str());
     
     return (XORP_OK);
 }
@@ -457,7 +506,7 @@ PimNode::delete_vif(const string& vif_name, string& error_msg)
     
     delete pim_vif;
     
-    XLOG_INFO("Deleted vif: %s", vif_name.c_str());
+    XLOG_INFO("Interface deleted: %s", vif_name.c_str());
     
     return (XORP_OK);
 }
@@ -509,7 +558,7 @@ PimNode::set_vif_flags(const string& vif_name,
 	_pim_register_vif_index = pim_vif->vif_index();
     
     if (is_changed)
-	XLOG_INFO("Vif flags changed: %s", pim_vif->str().c_str());
+	XLOG_INFO("Interface flags changed: %s", pim_vif->str().c_str());
     
     return (XORP_OK);
 }
@@ -570,7 +619,8 @@ PimNode::add_vif_addr(const string& vif_name,
 
     if (node_vif_addr != NULL) {
 	// Update the address
-	XLOG_INFO("Updated existing address on vif %s: old is %s new is %s",
+	XLOG_INFO("Updated existing address on interface %s: "
+		  "old is %s new is %s",
 		  pim_vif->name().c_str(), node_vif_addr->str().c_str(),
 		  vif_addr.str().c_str());
 	*node_vif_addr = vif_addr;
@@ -578,10 +628,32 @@ PimNode::add_vif_addr(const string& vif_name,
 	// Add a new address
 	pim_vif->add_address(vif_addr);
 	
-	XLOG_INFO("Added new address to vif %s: %s",
+	XLOG_INFO("Added new address to interface %s: %s",
 		  pim_vif->name().c_str(), vif_addr.str().c_str());
     }
-    
+
+    //
+    // Update and check the primary and domain-wide addresses
+    //
+    do {
+	if (pim_vif->update_primary_and_domain_wide_address(error_msg)
+	    == XORP_OK) {
+	    break;
+	}
+	if (! (pim_vif->is_up() || pim_vif->is_pending_up())) {
+	    // XXX: print an error only if the interface is UP or PENDING_UP
+	    break;
+	}
+	if (pim_vif->is_loopback()) {
+	    // XXX: don't print an error if this is a loopback interface
+	    break;
+	}
+	XLOG_ERROR("Error updating primary and domain-wide addresses "
+		   "for vif %s: %s",
+		   pim_vif->name().c_str(), error_msg.c_str());
+	return (XORP_ERROR);
+    } while (false);
+
     //
     // Spec:
     // "If an interface changes one of its secondary IP addresses,
@@ -639,23 +711,31 @@ PimNode::delete_vif_addr(const string& vif_name,
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("Deleted address on vif %s: %s",
+    XLOG_INFO("Deleted address on interface %s: %s",
 	      pim_vif->name().c_str(), vif_addr.str().c_str());
 
     //
     // Update the primary and domain-wide addresses.
     // If the vif has no more primary or a domain-wide address, then stop it.
     //
-    pim_vif->update_primary_and_domain_wide_address(error_msg);
-    if (pim_vif->is_up()) {
-	// Check the primary and domain-wide addresses
-	if ((pim_vif->primary_addr() == IPvX::ZERO(family()))
-	    || (pim_vif->domain_wide_addr() == IPvX::ZERO(family()))) {
-	    XLOG_ERROR("Cannot update primary and domain-wide addresses: %s",
-		       error_msg.c_str());
-	    pim_vif->stop(error_msg);
+    do {
+	if (pim_vif->update_primary_and_domain_wide_address(error_msg)
+	    == XORP_OK) {
+	    break;
 	}
-    }
+	if (! (pim_vif->is_up() || pim_vif->is_pending_up())) {
+	    // XXX: don't do anything if the interface is not UP or PENDING_UP
+	    break;
+	}
+	if (pim_vif->is_loopback()) {
+	    // XXX: don't do anything if this is a loopback interface
+	    break;
+	}
+	XLOG_ERROR("Error updating primary and domain-wide addresses "
+		   "for vif %s: %s",
+		   pim_vif->name().c_str(), error_msg.c_str());
+	pim_vif->stop(error_msg);
+    } while (false);
 
     //
     // Spec:
@@ -695,8 +775,6 @@ PimNode::enable_vif(const string& vif_name, string& error_msg)
     
     pim_vif->enable();
     
-    XLOG_INFO("Enabled vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -722,8 +800,6 @@ PimNode::disable_vif(const string& vif_name, string& error_msg)
     
     pim_vif->disable();
     
-    XLOG_INFO("Disabled vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -754,8 +830,6 @@ PimNode::start_vif(const string& vif_name, string& error_msg)
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("Started vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -786,8 +860,6 @@ PimNode::stop_vif(const string& vif_name, string& error_msg)
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("Stopped vif: %s", vif_name.c_str());
-    
     return (XORP_OK);
 }
 
@@ -797,31 +869,24 @@ PimNode::stop_vif(const string& vif_name, string& error_msg)
  * 
  * Start PIM on all enabled interfaces.
  * 
- * Return value: The number of virtual interfaces PIM was started on,
- * or %XORP_ERROR if error occured.
+ * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
 PimNode::start_all_vifs()
 {
-    int n = 0;
     vector<PimVif *>::iterator iter;
     string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	PimVif *pim_vif = (*iter);
 	if (pim_vif == NULL)
 	    continue;
-	if (! pim_vif->is_enabled())
-	    continue;
-	if (pim_vif->start(error_msg) != XORP_OK) {
-	    XLOG_ERROR("Cannot start vif %s: %s",
-		       pim_vif->name().c_str(), error_msg.c_str());
-	} else {
-	    n++;
-	}
+	if (start_vif(pim_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (n);
+    return (ret_value);
 }
 
 /**
@@ -830,29 +895,24 @@ PimNode::start_all_vifs()
  * 
  * Stop PIM on all interfaces it was running on.
  * 
- * Return value: The number of virtual interfaces PIM was stopped on,
- * or %XORP_ERROR if error occured.
+ * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
 PimNode::stop_all_vifs()
 {
-    int n = 0;
     vector<PimVif *>::iterator iter;
     string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	PimVif *pim_vif = (*iter);
 	if (pim_vif == NULL)
 	    continue;
-	if (pim_vif->stop(error_msg) != XORP_OK) {
-	    XLOG_ERROR("Cannot stop vif %s: %s",
-		       pim_vif->name().c_str(), error_msg.c_str());
-	} else {
-	    n++;
-	}
+	if (stop_vif(pim_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (n);
+    return (ret_value);
 }
 
 /**
@@ -867,15 +927,18 @@ int
 PimNode::enable_all_vifs()
 {
     vector<PimVif *>::iterator iter;
+    string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	PimVif *pim_vif = (*iter);
 	if (pim_vif == NULL)
 	    continue;
-	pim_vif->enable();
+	if (enable_vif(pim_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (XORP_OK);
+    return (ret_value);
 }
 
 /**
@@ -890,17 +953,18 @@ int
 PimNode::disable_all_vifs()
 {
     vector<PimVif *>::iterator iter;
-    
-    stop_all_vifs();
+    string error_msg;
+    int ret_value = XORP_OK;
     
     for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
 	PimVif *pim_vif = (*iter);
 	if (pim_vif == NULL)
 	    continue;
-	pim_vif->disable();
+	if (disable_vif(pim_vif->name(), error_msg) != XORP_OK)
+	    ret_value = XORP_ERROR;
     }
     
-    return (XORP_OK);
+    return (ret_value);
 }
 
 /**
@@ -943,6 +1007,43 @@ PimNode::delete_all_vifs()
     }
 }
 
+/**
+ * A method called when a vif has completed its shutdown.
+ * 
+ * @param vif_name the name of the vif that has completed its shutdown.
+ */
+void
+PimNode::vif_shutdown_completed(const string& vif_name)
+{
+    vector<PimVif *>::iterator iter;
+
+    //
+    // If all vifs have completed the shutdown, then de-register with
+    // the RIB and the MFEA.
+    //
+    for (iter = proto_vifs().begin(); iter != proto_vifs().end(); ++iter) {
+	PimVif *pim_vif = *iter;
+	if (pim_vif == NULL)
+	    continue;
+	if (! pim_vif->is_down())
+	    return;
+    }
+
+    if (ServiceBase::status() == SERVICE_SHUTTING_DOWN) {
+	//
+	// De-register with the RIB
+	//
+	rib_register_shutdown();
+
+	//
+	// De-register with the MFEA
+	//
+	mfea_register_shutdown();
+    }
+
+    UNUSED(vif_name);
+}
+
 /**
  * PimNode::proto_recv:
  * @src_module_instance_name: The module instance name of the module-origin
@@ -955,7 +1056,7 @@ PimNode::delete_all_vifs()
  * it should be ignored.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * it should be ignored.
- * @router_alert_bool: If true, the Router Alert IP option for the IP
+ * @is_router_alert: If true, the Router Alert IP option for the IP
  * packet of the incoming message was set.
  * @rcvbuf: The data buffer with the received message.
  * @rcvlen: The data length in @rcvbuf.
@@ -969,7 +1070,7 @@ PimNode::proto_recv(const string&	, // src_module_instance_name,
 		    xorp_module_id src_module_id,
 		    uint16_t vif_index,
 		    const IPvX& src, const IPvX& dst,
-		    int ip_ttl, int ip_tos, bool router_alert_bool,
+		    int ip_ttl, int ip_tos, bool is_router_alert,
 		    const uint8_t *rcvbuf, size_t rcvlen)
 {
     PimVif *pim_vif = NULL;
@@ -978,7 +1079,7 @@ PimNode::proto_recv(const string&	, // src_module_instance_name,
     debug_msg("Received message from %s to %s on vif_index %d: "
 	      "ip_ttl = %d ip_tos = %#x router_alert = %d rcvlen = %u\n",
 	      cstring(src), cstring(dst), vif_index,
-	      ip_ttl, ip_tos, router_alert_bool, (uint32_t)rcvlen);
+	      ip_ttl, ip_tos, is_router_alert, XORP_UINT_CAST(rcvlen));
     
     //
     // Check whether the node is up.
@@ -999,7 +1100,7 @@ PimNode::proto_recv(const string&	, // src_module_instance_name,
     
     // Process the data by the vif
     ret_value = pim_vif->pim_recv(src, dst, ip_ttl, ip_tos,
-				  router_alert_bool,
+				  is_router_alert,
 				  _buffer_recv);
     
     return (ret_value);
@@ -1020,7 +1121,7 @@ PimNode::proto_recv(const string&	, // src_module_instance_name,
  * the TTL will be set by the lower layers.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * the TOS will be set by the lower layers.
- * @router_alert_bool: If true, set the Router Alert IP option for the IP
+ * @is_router_alert: If true, set the Router Alert IP option for the IP
  * packet of the outgoung message.
  * @buffer: The #buffer_t data buffer with the message to send.
  * 
@@ -1031,7 +1132,7 @@ PimNode::proto_recv(const string&	, // src_module_instance_name,
 int
 PimNode::pim_send(uint16_t vif_index,
 		  const IPvX& src, const IPvX& dst,
-		  int ip_ttl, int ip_tos, bool router_alert_bool,
+		  int ip_ttl, int ip_tos, bool is_router_alert,
 		  buffer_t *buffer)
 {
     if (! (is_up() || is_pending_down()))
@@ -1041,7 +1142,7 @@ PimNode::pim_send(uint16_t vif_index,
     if (proto_send(xorp_module_name(family(), XORP_MODULE_MFEA),
 		   XORP_MODULE_MFEA,
 		   vif_index, src, dst,
-		   ip_ttl, ip_tos, router_alert_bool,
+		   ip_ttl, ip_tos, is_router_alert,
 		   BUFFER_DATA_HEAD(buffer),
 		   BUFFER_DATA_SIZE(buffer)) < 0) {
 	return (XORP_ERROR);
@@ -1382,6 +1483,7 @@ PimNode::pim_nbr_rpf_find(const IPvX& dst_addr)
 PimNbr *
 PimNode::pim_nbr_rpf_find(const IPvX& dst_addr, const Mrib *mrib)
 {
+    bool is_same_subnet = false;
     PimNbr *pim_nbr = NULL;
     
     //
@@ -1394,22 +1496,42 @@ PimNode::pim_nbr_rpf_find(const IPvX& dst_addr, const Mrib *mrib)
     // Find the vif toward the destination address
     //
     PimVif *pim_vif = vif_find_by_vif_index(mrib->next_hop_vif_index());
-    
+
+    //
+    // Test if the destination is on the same subnet.
+    //
+    // Note that we need to capture the case if the next-hop router
+    // address toward a destination on the same subnet is set to one
+    // of the addresses of the interface for that subnet.
+    //
+    do {
+	if (mrib->next_hop_router_addr() == IPvX::ZERO(family())) {
+	    is_same_subnet = true;
+	    break;
+	}
+	if ((pim_vif != NULL)
+	    && pim_vif->is_my_addr(mrib->next_hop_router_addr())) {
+	    is_same_subnet = true;
+	    break;
+	}
+	break;
+    } while (false);
+
     //
     // Search for the neighbor router
     //
-    if (mrib->next_hop_router_addr() != IPvX::ZERO(family())) {
-	// Not a destination on the same subnet
-	if (pim_vif != NULL)
-	    pim_nbr = pim_vif->pim_nbr_find(mrib->next_hop_router_addr());
-    } else {
-	// Probably a destination on the same subnet
+    if (is_same_subnet) {
+	// A destination on the same subnet
 	if (pim_vif != NULL)
 	    pim_nbr = pim_vif->pim_nbr_find(dst_addr);
 	else
 	    pim_nbr = pim_nbr_find(dst_addr);
+    } else {
+	// Not a destination on the same subnet
+	if (pim_vif != NULL)
+	    pim_nbr = pim_vif->pim_nbr_find(mrib->next_hop_router_addr());
     }
-    
+
     return (pim_nbr);
 }
 
@@ -1420,7 +1542,7 @@ PimNode::pim_nbr_rpf_find(const IPvX& dst_addr, const Mrib *mrib)
  * Find a PIM neighbor by its address.
  * 
  * Note: this method should be used in very limited cases, because
- * in case of IPv6 a neighbor's IP address may be non-unique within
+ * in case of IPv6 a neighbor's IP address may not be unique within
  * the PIM neighbor database due to scope issues.
  * 
  * Return value: The #PimNbr entry for the neighbor if found, otherwise %NULL.
@@ -1432,6 +1554,9 @@ PimNode::pim_nbr_find(const IPvX& nbr_addr)
 	PimVif *pim_vif = vif_find_by_vif_index(i);
 	if (pim_vif == NULL)
 	    continue;
+	// Exclude the PIM Register vif (as a safe-guard)
+	if (pim_vif->is_pim_register())
+	    continue;
 	PimNbr *pim_nbr = pim_vif->pim_nbr_find(nbr_addr);
 	if (pim_nbr != NULL)
 	    return (pim_nbr);
@@ -1462,12 +1587,15 @@ PimNode::add_pim_mre_no_pim_nbr(PimMre *pim_mre)
     }
     
     if (pim_nbr == NULL) {
-	// Find the first vif
+	// Find the first vif. Note that the PIM Register vif is excluded.
 	PimVif *pim_vif = NULL;
 	for (uint16_t i = 0; i < maxvifs(); i++) {
 	    pim_vif = vif_find_by_vif_index(i);
-	    if (pim_vif != NULL)
-		break;
+	    if (pim_vif == NULL)
+		continue;
+	    if (pim_vif->is_pim_register())
+		continue;
+	    break;
 	}
 	XLOG_ASSERT(pim_vif != NULL);
 	pim_nbr = new PimNbr(*pim_vif, ipvx_zero, PIM_VERSION_DEFAULT);
