@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_node_cli.cc,v 1.28 2004/06/10 22:41:32 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_node_cli.cc,v 1.35 2005/03/25 02:54:02 pavlin Exp $"
 
 
 //
@@ -21,7 +21,11 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "pim_mfc.hh"
 #include "pim_mre.hh"
 #include "pim_node.hh"
@@ -72,6 +76,9 @@ PimNodeCli::~PimNodeCli()
 int
 PimNodeCli::start()
 {
+    if (! is_enabled())
+	return (XORP_OK);
+
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
@@ -80,6 +87,8 @@ PimNodeCli::start()
 
     if (add_all_cli_commands() < 0)
 	return (XORP_ERROR);
+
+    XLOG_INFO("CLI started");
     
     return (XORP_OK);
 }
@@ -97,10 +106,40 @@ PimNodeCli::stop()
     
     if (delete_all_cli_commands() < 0)
 	ret_code = XORP_ERROR;
-    
+
+    XLOG_INFO("CLI stopped");
+
     return (ret_code);
 }
 
+/**
+ * Enable the node operation.
+ * 
+ * If an unit is not enabled, it cannot be start, or pending-start.
+ */
+void
+PimNodeCli::enable()
+{
+    ProtoUnit::enable();
+
+    XLOG_INFO("CLI enabled");
+}
+
+/**
+ * Disable the node operation.
+ * 
+ * If an unit is disabled, it cannot be start or pending-start.
+ * If the unit was runnning, it will be stop first.
+ */
+void
+PimNodeCli::disable()
+{
+    stop();
+    ProtoUnit::disable();
+
+    XLOG_INFO("CLI disabled");
+}
+
 int
 PimNodeCli::add_all_cli_commands()
 {
@@ -236,7 +275,11 @@ PimNodeCli::cli_show_pim_bootstrap(const vector<string>& argv)
 		    zone_id = PimScopeZoneId(argv[0].c_str(), true);
 	    }
 	} catch (InvalidString) {
-	    cli_print(c_format("ERROR: Invalid zone ID: %s\n",
+	    cli_print(c_format("ERROR: Invalid zone ID address: %s\n",
+			       argv[0].c_str()));
+	    return (XORP_ERROR);
+	} catch (InvalidNetmaskLength) {
+	    cli_print(c_format("ERROR: Invalid zone ID netmask length: %s\n",
 			       argv[0].c_str()));
 	    return (XORP_ERROR);
 	}
@@ -451,7 +494,11 @@ PimNodeCli::cli_show_pim_bootstrap_rps(const vector<string>& argv)
 		    zone_id = PimScopeZoneId(argv[0].c_str(), true);
 	    }
 	} catch (InvalidString) {
-	    cli_print(c_format("ERROR: Invalid zone ID: %s\n",
+	    cli_print(c_format("ERROR: Invalid zone ID address: %s\n",
+			       argv[0].c_str()));
+	    return (XORP_ERROR);
+	} catch (InvalidNetmaskLength) {
+	    cli_print(c_format("ERROR: Invalid zone ID netmask length: %s\n",
 			       argv[0].c_str()));
 	    return (XORP_ERROR);
 	}
@@ -738,7 +785,11 @@ PimNodeCli::cli_show_pim_join(const vector<string>& argv)
 		group_range = IPvXNet(IPvX(argv[0].c_str()),
 				      IPvX::addr_bitlen(family()));
 	    } catch (InvalidString) {
-		cli_print(c_format("ERROR: Invalid group range: %s\n",
+		cli_print(c_format("ERROR: Invalid group range address: %s\n",
+				   argv[0].c_str()));
+		return (XORP_ERROR);
+	    } catch (InvalidNetmaskLength) {
+		cli_print(c_format("ERROR: Invalid group range netmask length: %s\n",
 				   argv[0].c_str()));
 		return (XORP_ERROR);
 	    }
@@ -775,10 +826,17 @@ PimNodeCli::cli_show_pim_join_all(const vector<string>& argv)
 		group_range = IPvXNet(IPvX(argv[0].c_str()),
 				      IPvX::addr_bitlen(family()));
 	    } catch (InvalidString) {
-		cli_print(c_format("ERROR: Invalid group range: %s\n",
+		cli_print(c_format("ERROR: Invalid group range address: %s\n",
 				   argv[0].c_str()));
 		return (XORP_ERROR);
+	    } catch (InvalidNetmaskLength) {
+		XLOG_UNREACHABLE();
+		return (XORP_ERROR);
 	    }
+	} catch (InvalidNetmaskLength) {
+	    cli_print(c_format("ERROR: Invalid group range netmask length: %s\n",
+			       argv[0].c_str()));
+	    return (XORP_ERROR);
 	}
 	if (! group_range.is_multicast()) {
 	    cli_print(c_format("ERROR: Group range is not multicast: %s\n",
@@ -812,10 +870,17 @@ PimNodeCli::cli_show_pim_mfc(const vector<string>& argv)
 		group_range = IPvXNet(IPvX(argv[0].c_str()),
 				      IPvX::addr_bitlen(family()));
 	    } catch (InvalidString) {
-		cli_print(c_format("ERROR: Invalid group range: %s\n",
+		cli_print(c_format("ERROR: Invalid group range address: %s\n",
 				   argv[0].c_str()));
 		return (XORP_ERROR);
+	    } catch (InvalidNetmaskLength) {
+		XLOG_UNREACHABLE();
+		return (XORP_ERROR);
 	    }
+	} catch (InvalidNetmaskLength) {
+	    cli_print(c_format("ERROR: Invalid group range netmask length: %s\n",
+			       argv[0].c_str()));
+	    return (XORP_ERROR);
 	}
 	if (! group_range.is_multicast()) {
 	    cli_print(c_format("ERROR: Group range is not multicast: %s\n",
@@ -1049,6 +1114,11 @@ PimNodeCli::cli_print_pim_mre(const PimMre *pim_mre)
 	cli_print(c_format("    Join timer:                %d\n",
 			   left_sec));
     }
+    if (pim_mre->is_sg()) {
+	cli_print(c_format("    KAT(S,G) running:          %s\n",
+			   pim_mre->is_keepalive_timer_running()?
+			   "true" : "false"));
+    }
     
     
     //
@@ -1452,8 +1522,8 @@ PimNodeCli::cli_show_pim_rps(const vector<string>& argv)
 			   pim_rp->rp_priority(),
 			   holdtime,
 			   left_sec,
-			   (uint32_t)(pim_rp->pim_mre_wc_list().size()
-				      + pim_rp->processing_pim_mre_wc_list().size()),
+			   XORP_UINT_CAST(pim_rp->pim_mre_wc_list().size()
+					  + pim_rp->processing_pim_mre_wc_list().size()),
 			   cstring(pim_rp->group_prefix())));
     }
     
