@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2004 International Computer Science Institute
+ * Copyright (c) 2001-2005 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/pim/pim_proto.h,v 1.10 2004/06/10 22:41:33 hodson Exp $
+ * $XORP: xorp/pim/pim_proto.h,v 1.12 2005/03/25 02:54:02 pavlin Exp $
  */
 
 
@@ -45,6 +45,19 @@
 #endif
 
 
+/*
+ * Conditionally define constants that may be missing from <netinet/pim.h>
+ */
+#ifndef PIM_MINLEN
+#define PIM_MINLEN		8	/* PIM message min. length	*/
+#endif
+#ifndef PIM_REG_MINLEN
+#define PIM_REG_MINLEN	(PIM_MINLEN+20)	/* PIM Register hdr + inner IPv4 hdr */
+#endif
+#ifndef PIM6_REG_MINLEN
+#define PIM6_REG_MINLEN	(PIM_MINLEN+40)	/* PIM Register hdr + inner IPv6 hdr */
+#endif
+
 /*
  * Constants definitions
  */
