@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_proto_bootstrap.cc,v 1.15 2004/06/10 22:41:33 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_proto_bootstrap.cc,v 1.18 2005/03/25 02:54:02 pavlin Exp $"
 
 
 //
@@ -21,7 +21,11 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "pim_bsr.hh"
 #include "pim_node.hh"
 #include "pim_vif.hh"
@@ -346,7 +350,10 @@ PimVif::pim_bootstrap_send(const IPvX& dst_addr, const BsrZone& bsr_zone)
 {
     size_t avail_buffer_size = 0;
     IPvX src_addr = primary_addr();
-    
+
+    if (is_pim_register())
+	return (XORP_ERROR);	// Never send on the PIM Register vif
+
     if (bsr_zone.bsr_addr() == dst_addr)
 	return (XORP_ERROR);	// Never send-back to the BSR
     
