@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_proto_join_prune_message.cc,v 1.17 2004/06/10 22:41:33 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_proto_join_prune_message.cc,v 1.20 2005/03/25 02:54:02 pavlin Exp $"
 
 
 //
@@ -22,8 +22,12 @@
 #include <map>
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
 #include "libxorp/utils.hh"
+
 #include "pim_mre.hh"
 #include "pim_node.hh"
 #include "pim_proto.h"
@@ -188,19 +192,25 @@ PimJpHeader::jp_entry_add(const IPvX& source_addr, const IPvX& group_addr,
     }
     _holdtime = holdtime;	// XXX: the older holdtime may be modified
     XLOG_ASSERT(jp_group != NULL);
-    
+
+    //
     // Perform sanity check for conflicting entries,
     // and at the same time find the type of entry.
     // XXX: the '?' entries in the J/P rules table are accepted
     // (see the J/P messages format section in the spec).
+    //
     switch(mrt_entry_type) {
     case MRT_ENTRY_RP:
 	if (action_jp == ACTION_JOIN) {
 	    // (*,*,RP) Join
 	    if (jp_group->rp()->j_list_found(source_addr))
 		return (XORP_OK);		// Already added; ignore.
+	    if (jp_group->rp()->p_list_found(source_addr))
+		return (XORP_ERROR);		// Combination not allowed
 	} else {
 	    // (*,*,RP) Prune
+	    if (jp_group->rp()->j_list_found(source_addr))
+		return (XORP_ERROR);		// Combination not allowed
 	    if (jp_group->rp()->p_list_found(source_addr))
 		return (XORP_OK);		// Already added; ignore.
 	}
