@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_proto_register.cc,v 1.15 2004/06/10 22:41:33 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_proto_register.cc,v 1.19 2005/03/25 02:54:03 pavlin Exp $"
 
 
 //
@@ -21,7 +21,11 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "mrt/inet_cksum.h"
 #include "pim_mfc.hh"
 #include "pim_node.hh"
@@ -657,8 +661,18 @@ PimVif::pim_register_null_send(const IPvX& rp_addr,
 	ip4_header.ip_p		= IPPROTO_PIM;
 	ip4_header.ip_len	= htons(sizeof(ip4_header));
 	ip4_header.ip_ttl	= 0;
-	source_addr.copy_out(ip4_header.ip_src);
-	group_addr.copy_out(ip4_header.ip_dst);
+
+	//
+	// XXX: we need to use a temporary in_addr storage as a work-around
+	// if "struct ip" is __packed
+	//
+	struct in_addr in_addr_tmp;
+	source_addr.copy_out(in_addr_tmp);
+	ip4_header.ip_src = in_addr_tmp;
+	group_addr.copy_out(in_addr_tmp);
+	ip4_header.ip_dst = in_addr_tmp;
+
+	//
 	// XXX: on older Linux 'ip->ip_sum' was named 'ip->ip_csum'.
 	// Later someone has realized that it is not a smart move,
 	// so now Linux is in sync with the rest of the UNIX-es.
@@ -666,6 +680,7 @@ PimVif::pim_register_null_send(const IPvX& rp_addr,
 	// something more decent. If you are stubborn and you don't want
 	// to upgrade, then change 'ip_sum' to 'ip_csum' below, but
 	// you should know that in the future you will get what you deserve :)
+	//
 	ip4_header.ip_sum	= 0;
 	ip4_header.ip_sum	= INET_CKSUM(&ip4_header, sizeof(ip4_header));
 	
