@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_proto_register_stop.cc,v 1.10 2004/06/10 22:41:34 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_proto_register_stop.cc,v 1.13 2005/03/25 02:54:03 pavlin Exp $"
 
 
 //
@@ -21,7 +21,11 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "pim_mre.hh"
 #include "pim_mrt.hh"
 #include "pim_vif.hh"
@@ -157,7 +161,8 @@ PimVif::pim_register_stop_process(const IPvX& rp_addr,
 		     "instead of %u",
 		     PIMTYPE2ASCII(PIM_REGISTER_STOP),
 		     cstring(rp_addr), cstring(domain_wide_addr()),
-		     group_mask_len, (uint32_t)IPvX::addr_bitlen(family()));
+		     group_mask_len,
+		     XORP_UINT_CAST(IPvX::addr_bitlen(family())));
 	return (XORP_ERROR);
     }
     
