@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_rp.cc,v 1.7 2004/06/10 22:41:34 hodson Exp $"
+#ident "$XORP: xorp/pim/pim_rp.cc,v 1.13 2005/03/25 02:54:03 pavlin Exp $"
 
 
 //
@@ -22,9 +22,13 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
-#include "libxorp/time_slice.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
 #include "libxorp/utils.hh"
+#include "libxorp/time_slice.hh"
+
 #include "pim_mfc.hh"
 #include "pim_mre.hh"
 #include "pim_node.hh"
@@ -83,6 +87,15 @@ RpTable::RpTable(PimNode& pim_node)
  * 
  **/
 RpTable::~RpTable()
+{
+    clear();
+}
+
+/**
+ * Clear the entry.
+ */
+void
+RpTable::clear()
 {
     stop();
 }
@@ -98,6 +111,9 @@ RpTable::~RpTable()
 int
 RpTable::start()
 {
+    if (! is_enabled())
+	return (XORP_OK);
+
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
@@ -393,6 +409,86 @@ RpTable::delete_rp(const IPvX& rp_addr,
     return (XORP_ERROR);
 }
 
+//
+// Delete all RP entries for a given RP address
+// Return %XORP_OK on success, otherwise %XORP_ERROR.
+//
+int
+RpTable::delete_all_group_prefixes_rp(
+    const IPvX& rp_addr,
+    PimRp::rp_learned_method_t rp_learned_method)
+{
+    list<pair<IPvX, IPvXNet> > delete_list;
+    list<pair<IPvX, IPvXNet> >::iterator delete_list_iter;
+    int ret_value = XORP_OK;
+
+    //
+    // Find all matching RP entries and add them to the list
+    //
+    list<PimRp *>::iterator iter;
+    for (iter = _rp_list.begin(); iter != _rp_list.end(); ++iter) {
+	PimRp *pim_rp = *iter;
+	if (pim_rp->rp_addr() != rp_addr)
+	    continue;
+	if (pim_rp->rp_learned_method() == rp_learned_method) {
+	    delete_list.push_back(make_pair(pim_rp->rp_addr(),
+					    pim_rp->group_prefix()));
+	}
+    }
+
+    //
+    // Delete all RP entries on the list
+    //
+    for (delete_list_iter = delete_list.begin();
+	 delete_list_iter != delete_list.end();
+	 ++delete_list_iter) {
+	const IPvX& ipvx = delete_list_iter->first;
+	const IPvXNet& ipvxnet = delete_list_iter->second;
+	if (delete_rp(ipvx, ipvxnet, rp_learned_method) != XORP_OK)
+	    ret_value = XORP_ERROR;
+    }
+
+    return (ret_value);
+}
+
+//
+// Delete all RP entries
+// Return %XORP_OK on success, otherwise %XORP_ERROR.
+//
+int
+RpTable::delete_all_rps(PimRp::rp_learned_method_t rp_learned_method)
+{
+    list<pair<IPvX, IPvXNet> > delete_list;
+    list<pair<IPvX, IPvXNet> >::iterator delete_list_iter;
+    int ret_value = XORP_OK;
+
+    //
+    // Find all matching RP entries and add them to the list
+    //
+    list<PimRp *>::iterator iter;
+    for (iter = _rp_list.begin(); iter != _rp_list.end(); ++iter) {
+	PimRp *pim_rp = *iter;
+	if (pim_rp->rp_learned_method() == rp_learned_method) {
+	    delete_list.push_back(make_pair(pim_rp->rp_addr(),
+					    pim_rp->group_prefix()));
+	}
+    }
+
+    //
+    // Delete all RP entries on the list
+    //
+    for (delete_list_iter = delete_list.begin();
+	 delete_list_iter != delete_list.end();
+	 ++delete_list_iter) {
+	const IPvX& ipvx = delete_list_iter->first;
+	const IPvXNet& ipvxnet = delete_list_iter->second;
+	if (delete_rp(ipvx, ipvxnet, rp_learned_method) != XORP_OK)
+	    ret_value = XORP_ERROR;
+    }
+
+    return (ret_value);
+}
+
 //
 // Apply the RP changes to the PIM multicast routing table
 // Return true if the processing was saved for later because it was taking too
