@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_vif.cc,v 1.39 2004/06/18 21:51:53 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_vif.cc,v 1.48 2005/03/25 02:54:03 pavlin Exp $"
 
 
 //
@@ -21,9 +21,14 @@
 
 
 #include "pim_module.h"
-#include "pim_private.hh"
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/ipvx.hh"
+
 #include "mrt/inet_cksum.h"
 #include "mrt/random.h"
+
 #include "pim_node.hh"
 #include "pim_vif.hh"
 
@@ -265,6 +270,9 @@ PimVif::pim_mrt() const
 int
 PimVif::start(string& error_msg)
 {
+    if (! is_enabled())
+	return (XORP_OK);
+
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
@@ -300,9 +308,6 @@ PimVif::start(string& error_msg)
 	return (XORP_ERROR);
     }
     
-    XLOG_INFO("STARTING %s%s",
-	      this->str().c_str(), flags_string().c_str());
-    
     if (! is_pim_register()) {    
 	//
 	// Join the appropriate multicast groups: ALL-PIM-ROUTERS
@@ -328,6 +333,9 @@ PimVif::start(string& error_msg)
     pim_node().pim_mrt().add_task_start_vif(vif_index());
     pim_node().pim_mrt().add_task_my_ip_address(vif_index());
     pim_node().pim_mrt().add_task_my_ip_subnet_address(vif_index());
+
+    XLOG_INFO("Interface started: %s%s",
+	      this->str().c_str(), flags_string().c_str());
     
     return (XORP_OK);
 }
@@ -342,9 +350,6 @@ PimVif::start(string& error_msg)
  * state with neighbors.
  * XXX: After the multicast routing entries cleanup is completed,
  * PimVif::final_stop() is called to complete the job.
- * XXX: If this method is called one-after-another, the second one
- * will force calling immediately PimVif::final_stop() to quickly
- * finish the job. TODO: is this a desired semantic?
  * 
  * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
@@ -360,22 +365,25 @@ PimVif::stop(string& error_msg)
 	error_msg = "the vif state is not UP or PENDING_UP or PENDING_DOWN";
 	return (XORP_ERROR);
     }
-    
+
+    if (ProtoUnit::pending_stop() < 0) {
+	error_msg = "internal error";
+	ret_value = XORP_ERROR;
+    }
+
     //
     // Add the tasks to take care of the PimMre processing
     //
-    if (is_up()) {
-	pim_node().pim_mrt().add_task_stop_vif(vif_index());
-	pim_node().pim_mrt().add_task_my_ip_address(vif_index());
-	pim_node().pim_mrt().add_task_my_ip_subnet_address(vif_index());
-    }
-    if ((_usage_by_pim_mre_task == 0)
-	|| is_pending_up()
-	|| is_pending_down()) {
-	ret_value = final_stop(error_msg);
-	return (ret_value);
-    }
-    
+    pim_node().pim_mrt().add_task_stop_vif(vif_index());
+    pim_node().pim_mrt().add_task_my_ip_address(vif_index());
+    pim_node().pim_mrt().add_task_my_ip_subnet_address(vif_index());
+
+    //
+    // Add the shutdown operation of this vif as a shutdown task
+    // for the node.
+    //
+    pim_node().incr_shutdown_requests_n();
+
     if (! is_pim_register()) {
 	//
 	// Delete MLD6/IGMP membership tracking
@@ -391,16 +399,8 @@ PimVif::stop(string& error_msg)
 	set_i_am_dr(false);
     }
     
-    if (ProtoUnit::pending_stop() < 0) {
-	error_msg = "internal error";
-	ret_value = XORP_ERROR;
-    }
-    
     _dr_addr = IPvX::ZERO(family());
     
-    XLOG_INFO("PENDING STOP %s%s",
-	      this->str().c_str(), flags_string().c_str());
-    
     return (ret_value);
 }
 
@@ -461,23 +461,25 @@ PimVif::final_stop(string& error_msg)
     // Stop the vif with the kernel
     //
     if (pim_node().stop_protocol_kernel_vif(vif_index()) != XORP_OK) {
-	error_msg = c_format("cannot stop protocol vif %s with the kernel",
-			     name().c_str());
-	return (XORP_ERROR);
+	XLOG_ERROR("Cannot stop protocol vif %s with the kernel",
+		   name().c_str());
+	ret_value = XORP_ERROR;
     }
     
-    XLOG_INFO("STOPPED %s%s",
+    XLOG_INFO("Interface stopped: %s%s",
 	      this->str().c_str(), flags_string().c_str());
-    
+
     //
-    // Test if time to completely stop the PimNode itself because of this vif
+    // Inform the node that the vif has completed the shutdown
     //
-    string dummy_string;
-    if (pim_node().is_pending_down()
-	&& (! pim_node().has_pending_down_units(dummy_string))) {
-	pim_node().final_stop();
-    }
-    
+    pim_node().vif_shutdown_completed(name());
+
+    //
+    // Remove the shutdown operation of this vif as a shutdown task
+    // for the node.
+    //
+    pim_node().decr_shutdown_requests_n();
+
     return (ret_value);
 }
 
@@ -490,6 +492,9 @@ void
 PimVif::enable()
 {
     ProtoUnit::enable();
+
+    XLOG_INFO("Interface enabled: %s%s",
+	      this->str().c_str(), flags_string().c_str());
 }
 
 /**
@@ -505,6 +510,9 @@ PimVif::disable()
 
     stop(error_msg);
     ProtoUnit::disable();
+
+    XLOG_INFO("Interface disabled: %s%s",
+	      this->str().c_str(), flags_string().c_str());
 }
 
 /**
@@ -531,7 +539,7 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
     int ret_value;
     size_t datalen;
     int ttl = MINTTL;
-    bool router_alert_bool = true;
+    bool is_router_alert = true;
 
     if (! (is_up() || is_pending_down()))
 	return (XORP_ERROR);
@@ -546,7 +554,7 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
 	case PIM_REGISTER_STOP:
 	case PIM_CAND_RP_ADV:
 	    ttl = IPDEFTTL;
-	    router_alert_bool = false;
+	    is_router_alert = false;
 	    break;
 	default:
 	    break;
@@ -669,7 +677,7 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
     // Send the message
     //
     ret_value = pim_node().pim_send(vif_index(), src, dst, ttl, ip_tos,
-				    router_alert_bool, buffer);
+				    is_router_alert, buffer);
     
     //
     // Actions after the message is sent
@@ -748,7 +756,7 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
  * it should be ignored.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * it should be ignored.
- * @router_alert_bool: True if the received IP packet had the Router Alert
+ * @is_router_alert: True if the received IP packet had the Router Alert
  * IP option set.
  * @buffer: The buffer with the received message.
  * 
@@ -761,7 +769,7 @@ PimVif::pim_recv(const IPvX& src,
 		 const IPvX& dst,
 		 int ip_ttl,
 		 int ip_tos,
-		 bool router_alert_bool,
+		 bool is_router_alert,
 		 buffer_t *buffer)
 {
     int ret_value = XORP_ERROR;
@@ -771,7 +779,7 @@ PimVif::pim_recv(const IPvX& src,
 	return (XORP_ERROR);
     }
     
-    ret_value = pim_process(src, dst, ip_ttl, ip_tos, router_alert_bool,
+    ret_value = pim_process(src, dst, ip_ttl, ip_tos, is_router_alert,
 			    buffer);
     
     return (ret_value);
@@ -785,7 +793,7 @@ PimVif::pim_recv(const IPvX& src,
  * it should be ignored.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * it should be ignored.
- * @router_alert_bool: True if the received IP packet had the Router Alert
+ * @is_router_alert: True if the received IP packet had the Router Alert
  * IP option set.
  * @buffer: The buffer with the message.
  * 
@@ -797,7 +805,7 @@ int
 PimVif::pim_process(const IPvX& src, const IPvX& dst,
 		    int ip_ttl,
 		    int ip_tos,
-		    bool router_alert_bool,
+		    bool is_router_alert,
 		    buffer_t *buffer)
 {
     uint8_t pim_vt;
@@ -819,7 +827,7 @@ PimVif::pim_process(const IPvX& src, const IPvX& dst,
 		     "too short data field (%u bytes)",
 		     cstring(src), cstring(dst),
 		     name().c_str(),
-		     (uint32_t)BUFFER_DATA_SIZE(buffer));
+		     XORP_UINT_CAST(BUFFER_DATA_SIZE(buffer)));
 	++_pimstat_bad_length_messages;
 	return (XORP_ERROR);
     }
@@ -868,16 +876,44 @@ PimVif::pim_process(const IPvX& src, const IPvX& dst,
     default:
 	cksum = INET_CKSUM(BUFFER_DATA_HEAD(buffer), BUFFER_DATA_SIZE(buffer));
 	cksum = INET_CKSUM_ADD(cksum, cksum2);
-	
-	if (cksum != 0) {
-	    XLOG_WARNING("RX packet from %s to %s on vif %s: "
-			 "checksum error",
-			 cstring(src), cstring(dst),
-			 name().c_str());
-	    ++_pimstat_bad_checksum_messages;
-	    return (XORP_ERROR);
+
+	if (cksum == 0)
+	    break;
+
+	//
+	// If this is a PIM Register packet, and if it was truncated
+	// by the kernel (e.g., in some *BSD systems), then ignore the
+	// checksum error.
+	//
+	if (message_type == PIM_REGISTER) {
+	    bool is_truncated = false;
+
+	    switch (family()) {
+	    case AF_INET:
+		if (BUFFER_DATA_SIZE(buffer) == PIM_REG_MINLEN)
+		    is_truncated = true;
+		break;
+#ifdef HAVE_IPV6	
+	    case AF_INET6:
+		if (BUFFER_DATA_SIZE(buffer) == PIM6_REG_MINLEN)
+		    is_truncated = true;
+		break;
+#endif // HAVE_IPV6
+	    default:
+		XLOG_UNREACHABLE();
+		return (XORP_ERROR);
+	    }
+
+	    if (is_truncated)
+		break;		// XXX: accept the truncated PIM Register
 	}
-	break;
+
+	XLOG_WARNING("RX packet from %s to %s on vif %s: "
+		     "checksum error",
+		     cstring(src), cstring(dst),
+		     name().c_str());
+	++_pimstat_bad_checksum_messages;
+	return (XORP_ERROR);
     }
     
     //
@@ -909,11 +945,11 @@ PimVif::pim_process(const IPvX& src, const IPvX& dst,
     
     //
     // TODO: if we are running in secure mode, then check ip_ttl, ip_tos and
-    // @router_alert_bool (e.g. (ip_ttl == MINTTL) && (router_alert_bool))
+    // @is_router_alert (e.g. (ip_ttl == MINTTL) && (is_router_alert))
     //
     UNUSED(ip_ttl);
     UNUSED(ip_tos);
-    UNUSED(router_alert_bool);
+    UNUSED(is_router_alert);
 #if 0
     //
     // TTL (aka. Hop-limit in IPv6) and Router Alert option checks.
