@@ -1,7 +1,7 @@
 #!/bin/sh
 
 #
-# $XORP: xorp/pim/xrl_pim_shell_funcs.sh,v 1.23 2004/03/02 00:32:33 pavlin Exp $
+# $XORP: xorp/pim/xrl_pim_shell_funcs.sh,v 1.25 2005/03/23 10:49:59 pavlin Exp $
 #
 
 #
@@ -629,6 +629,33 @@ pim_delete_config_static_rp6()
     call_xrl_wrapper -r 0 $XRL$XRL_ARGS
 }
 
+pim_delete_config_all_static_group_prefixes_rp4()
+{
+    if [ $# -lt 1 ] ; then
+	echo "Usage: pim_delete_config_all_static_group_prefixes_rp4 <rp_addr:ipv4>"
+	exit 1
+    fi
+    rp_addr=$1
+    
+    echo "pim_delete_config_all_static_group_prefixes_rp4" $*
+    XRL="finder://$PIM_TARGET/pim/0.1/delete_config_all_static_group_prefixes_rp4"
+    XRL_ARGS="?rp_addr:ipv4=$rp_addr"
+    call_xrl_wrapper -r 0 $XRL$XRL_ARGS
+}
+
+pim_delete_config_all_static_rps()
+{
+    if [ $# -lt 0 ] ; then
+	echo "Usage: pim_delete_config_all_static_rps"
+	exit 1
+    fi
+    
+    echo "pim_delete_config_all_static_rps" $*
+    XRL="finder://$PIM_TARGET/pim/0.1/delete_config_all_static_rps"
+    XRL_ARGS=""
+    call_xrl_wrapper -r 0 $XRL$XRL_ARGS
+}
+
 pim_config_static_rp_done()
 {
     if [ $# -lt 0 ] ; then
