@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,15 +12,16 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/parser.cc,v 1.12 2004/06/10 22:41:37 hodson Exp $"
+#ident "$XORP: xorp/rib/parser.cc,v 1.15 2005/03/25 02:54:19 pavlin Exp $"
 
 #include <stdexcept>
 
 #include "rib_module.h"
 
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 #include "libxorp/c_format.hh"
-#include "libxorp/xlog.h"
 
 #include "parser.hh"
 
@@ -29,11 +30,11 @@
 // Argument Parsing methods
 
 Datum*
-IntArgumentParser::parse(const string& str) const 
+Uint32ArgumentParser::parse(const string& str) const 
 {
     try {
 	printf(">>>str=%s\n", str.c_str());
-	return new IntDatum(str);
+	return new Uint32Datum(str);
     } catch (const InvalidString&) {
 	return NULL;
     }
@@ -110,9 +111,9 @@ Command::bind(int n, DatumVariableBinding* d)
 }
 
 void
-Command::bind_int(int n, int& i)
+Command::bind_uint32(int n, uint32_t& i)
 {
-    bind(n, new DatumIntBinding(i));
+    bind(n, new DatumUint32Binding(i));
 }
 
 void
@@ -149,7 +150,7 @@ Command::set_arg(int n, Datum* d) throw (Parse_error)
 Parser::Parser()
     : _separator(' ')
 {
-    add_argtype(new IntArgumentParser());
+    add_argtype(new Uint32ArgumentParser());
     add_argtype(new StringArgumentParser());
     add_argtype(new IPv4ArgumentParser());
     add_argtype(new IPv4NetArgumentParser());
