@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,15 +12,18 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/redist_xrl.cc,v 1.12 2004/06/10 22:41:38 hodson Exp $"
+#ident "$XORP: xorp/rib/redist_xrl.cc,v 1.20 2005/03/25 02:54:20 pavlin Exp $"
 
 #include <list>
 #include <string>
 
 #include "rib_module.h"
-#include "config.h"
 
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
 #include "libxorp/safe_callback_obj.hh"
+
 #include "libxipc/xrl_router.hh"
 
 #include "xrl/interfaces/redist4_xif.hh"
@@ -30,6 +33,7 @@
 
 #include "route.hh"
 #include "redist_xrl.hh"
+#include "profile_vars.hh"
 
 /**
  * Base class for RedistXrlOutput Tasks.  Classes derived from this
@@ -49,7 +53,7 @@ public:
      * @return true on success, false if XrlRouter could not dispatch
      * request.
      */
-    virtual bool dispatch(XrlRouter& xrl_router) = 0;
+    virtual bool dispatch(XrlRouter& xrl_router, Profile& profile) = 0;
 
     /**
      * Get number of times dispatch() invoked on instance.
@@ -78,11 +82,11 @@ class AddRoute : public RedistXrlTask<A>
 {
 public:
     AddRoute(RedistXrlOutput<A>* parent, const IPRouteEntry<A>& ipr);
-    virtual bool dispatch(XrlRouter& xrl_router);
+    virtual bool dispatch(XrlRouter& xrl_router, Profile& profile);
     void dispatch_complete(const XrlError& xe);
 protected:
     IPNet<A>	_net;
-    A		_nh;
+    A		_nexthop;
     string	_ifname;
     string	_vifname;
     uint32_t	_metric;
@@ -95,10 +99,11 @@ class DeleteRoute : public RedistXrlTask<A>
 {
 public:
     DeleteRoute(RedistXrlOutput<A>* parent, const IPRouteEntry<A>& ipr);
-    virtual bool dispatch(XrlRouter& xrl_router);
+    virtual bool dispatch(XrlRouter& xrl_router, Profile& profile);
     void dispatch_complete(const XrlError& xe);
 protected:
     IPNet<A>	_net;
+    string	_protocol_origin;
 };
 
 template <typename A>
@@ -106,7 +111,7 @@ class StartingRouteDump : public RedistXrlTask<A>
 {
 public:
     StartingRouteDump(RedistXrlOutput<A>* parent);
-    virtual bool dispatch(XrlRouter& xrl_router);
+    virtual bool dispatch(XrlRouter& xrl_router, Profile& profile);
     void dispatch_complete(const XrlError& xe);
 };
 
@@ -115,7 +120,7 @@ class FinishingRouteDump : public RedistXrlTask<A>
 {
 public:
     FinishingRouteDump(RedistXrlOutput<A>* parent);
-    virtual bool dispatch(XrlRouter& xrl_router);
+    virtual bool dispatch(XrlRouter& xrl_router, Profile& profile);
     void dispatch_complete(const XrlError& xe);
 };
 
@@ -124,7 +129,7 @@ class Pause : public RedistXrlTask<A>
 {
 public:
     Pause(RedistXrlOutput<A>* parent, uint32_t ms);
-    virtual bool dispatch(XrlRouter&  xrl_router);
+    virtual bool dispatch(XrlRouter&  xrl_router, Profile& profile);
     void expire();
 private:
     XorpTimer _t;
@@ -137,7 +142,7 @@ private:
 
 template <typename A>
 AddRoute<A>::AddRoute(RedistXrlOutput<A>* parent, const IPRouteEntry<A>& ipr)
-    : RedistXrlTask<A>(parent), _net(ipr.net()), _nh(ipr.nexthop_addr()),
+    : RedistXrlTask<A>(parent), _net(ipr.net()), _nexthop(ipr.nexthop_addr()),
       _ifname(ipr.vif()->ifname()), _vifname(ipr.vif()->name()),
       _metric(ipr.metric()), _admin_distance(ipr.admin_distance()),
       _protocol_origin(ipr.protocol().name())
@@ -146,13 +151,17 @@ AddRoute<A>::AddRoute(RedistXrlOutput<A>* parent, const IPRouteEntry<A>& ipr)
 
 template <>
 bool
-AddRoute<IPv4>::dispatch(XrlRouter& xrl_router)
+AddRoute<IPv4>::dispatch(XrlRouter& xrl_router, Profile& profile)
 {
+    if (profile.enabled(profile_route_rpc_out))
+	profile.log(profile_route_rpc_out,
+		    c_format("add %s", _net.str().c_str()));
+
     RedistXrlOutput<IPv4>* p = this->parent();
 
     XrlRedist4V0p1Client cl(&xrl_router);
     return cl.send_add_route(p->xrl_target_name().c_str(),
-			     _net, _nh, _ifname, _vifname, _metric,
+			     _net, _nexthop, _ifname, _vifname, _metric,
 			     _admin_distance, p->cookie(),
 			     callback(this, &AddRoute<IPv4>::dispatch_complete)
 			     );
@@ -160,13 +169,17 @@ AddRoute<IPv4>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-AddRoute<IPv6>::dispatch(XrlRouter& xrl_router)
+AddRoute<IPv6>::dispatch(XrlRouter& xrl_router, Profile& profile)
 {
+    if (profile.enabled(profile_route_rpc_out))
+	profile.log(profile_route_rpc_out,
+		    c_format("add %s", _net.str().c_str()));
+
     RedistXrlOutput<IPv6>* p = this->parent();
 
     XrlRedist6V0p1Client cl(&xrl_router);
     return cl.send_add_route(p->xrl_target_name().c_str(),
-			     _net, _nh, _ifname, _vifname, _metric,
+			     _net, _nexthop, _ifname, _vifname, _metric,
 			     _admin_distance, p->cookie(),
 			     callback(this, &AddRoute<IPv6>::dispatch_complete)
 			     );
@@ -181,13 +194,14 @@ AddRoute<A>::dispatch_complete(const XrlError& xe)
 	this->signal_complete_ok();
 	return;
     } else if (xe == XrlError::COMMAND_FAILED()) {
-	XLOG_ERROR("Failed to redistribute route add for %s",
-		   _net.str().c_str());
+	XLOG_ERROR("Failed to redistribute route add for %s: %s",
+		   _net.str().c_str(),
+		   xe.str().c_str());
 	this->signal_complete_ok();
 	return;
     }
     // For now all errors are signalled fatal
-    XLOG_ERROR("Fatal error during route redistribution \"%s\"",
+    XLOG_ERROR("Fatal error during route redistribution: %s",
 	       xe.str().c_str());
 
     this->signal_fatal_failure();
@@ -200,14 +214,20 @@ AddRoute<A>::dispatch_complete(const XrlError& xe)
 template <typename A>
 DeleteRoute<A>::DeleteRoute(RedistXrlOutput<A>* parent,
 			    const IPRouteEntry<A>& ipr)
-    : RedistXrlTask<A>(parent), _net(ipr.net())
+    : RedistXrlTask<A>(parent),
+      _net(ipr.net()),
+      _protocol_origin(ipr.protocol().name())
 {
 }
 
 template <>
 bool
-DeleteRoute<IPv4>::dispatch(XrlRouter& xrl_router)
+DeleteRoute<IPv4>::dispatch(XrlRouter& xrl_router, Profile& profile)
 {
+    if (profile.enabled(profile_route_rpc_out))
+	profile.log(profile_route_rpc_out,
+		    c_format("delete %s", _net.str().c_str()));
+
     RedistXrlOutput<IPv4>* p = this->parent();
 
     XrlRedist4V0p1Client cl(&xrl_router);
@@ -220,8 +240,12 @@ DeleteRoute<IPv4>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-DeleteRoute<IPv6>::dispatch(XrlRouter& xrl_router)
+DeleteRoute<IPv6>::dispatch(XrlRouter& xrl_router, Profile& profile)
 {
+    if (profile.enabled(profile_route_rpc_out))
+	profile.log(profile_route_rpc_out,
+		    c_format("delete %s", _net.str().c_str()));
+
     RedistXrlOutput<IPv6>* p = this->parent();
 
     XrlRedist6V0p1Client cl(&xrl_router);
@@ -240,13 +264,14 @@ DeleteRoute<A>::dispatch_complete(const XrlError& xe)
 	this->signal_complete_ok();
 	return;
     } else if (xe == XrlError::COMMAND_FAILED()) {
-	XLOG_ERROR("Failed to redistribute route delete for %s",
-		   _net.str().c_str());
+	XLOG_ERROR("Failed to redistribute route delete for %s: %s",
+		   _net.str().c_str(),
+		   xe.str().c_str());
 	this->signal_complete_ok();
 	return;
     }
     // XXX For now all errors signalled as fatal
-    XLOG_ERROR("Fatal error during route redistribution \"%s\"",
+    XLOG_ERROR("Fatal error during route redistribution: %s",
 	       xe.str().c_str());
     this->signal_fatal_failure();
 }
@@ -263,7 +288,7 @@ StartingRouteDump<A>::StartingRouteDump(RedistXrlOutput<A>* parent)
 
 template <>
 bool
-StartingRouteDump<IPv4>::dispatch(XrlRouter& xrl_router)
+StartingRouteDump<IPv4>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistXrlOutput<IPv4>* p = this->parent();
 
@@ -277,7 +302,7 @@ StartingRouteDump<IPv4>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-StartingRouteDump<IPv6>::dispatch(XrlRouter& xrl_router)
+StartingRouteDump<IPv6>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistXrlOutput<IPv6>* p = this->parent();
 
@@ -297,12 +322,13 @@ StartingRouteDump<A>::dispatch_complete(const XrlError& xe)
 	this->signal_complete_ok();
 	return;
     } else if (xe == XrlError::COMMAND_FAILED()) {
-	XLOG_ERROR("Failed to send starting route dump.");
+	XLOG_ERROR("Failed to send starting route dump: %s",
+		   xe.str().c_str());
 	this->signal_complete_ok();
 	return;
     }
     // XXX For now all errors signalled as fatal
-    XLOG_ERROR("Fatal error during route redistribution \"%s\"",
+    XLOG_ERROR("Fatal error during route redistribution: %s",
 	       xe.str().c_str());
     this->signal_fatal_failure();
 }
@@ -319,7 +345,7 @@ FinishingRouteDump<A>::FinishingRouteDump(RedistXrlOutput<A>* parent)
 
 template <>
 bool
-FinishingRouteDump<IPv4>::dispatch(XrlRouter& xrl_router)
+FinishingRouteDump<IPv4>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistXrlOutput<IPv4>* p = this->parent();
 
@@ -333,7 +359,7 @@ FinishingRouteDump<IPv4>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-FinishingRouteDump<IPv6>::dispatch(XrlRouter& xrl_router)
+FinishingRouteDump<IPv6>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistXrlOutput<IPv6>* p = this->parent();
 
@@ -353,12 +379,13 @@ FinishingRouteDump<A>::dispatch_complete(const XrlError& xe)
 	this->signal_complete_ok();
 	return;
     } else if (xe == XrlError::COMMAND_FAILED()) {
-	XLOG_ERROR("Failed to send finishing route dump.");
+	XLOG_ERROR("Failed to send finishing route dump: %s",
+		   xe.str().c_str());
 	this->signal_complete_ok();
 	return;
     }
     // XXX For now all errors signalled as fatal
-    XLOG_ERROR("Fatal error during route redistribution \"%s\"",
+    XLOG_ERROR("Fatal error during route redistribution: %s",
 	       xe.str().c_str());
     this->signal_fatal_failure();
 }
@@ -375,7 +402,7 @@ Pause<A>::Pause(RedistXrlOutput<A>* parent, uint32_t ms)
 
 template <typename A>
 bool
-Pause<A>::dispatch(XrlRouter& xrl_router)
+Pause<A>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     this->incr_dispatch_attempts();
     EventLoop& e = xrl_router.eventloop();
@@ -397,10 +424,12 @@ Pause<A>::expire()
 template <typename A>
 RedistXrlOutput<A>::RedistXrlOutput(Redistributor<A>*	redistributor,
 				    XrlRouter&		xrl_router,
+				    Profile&		profile,
 				    const string&	from_protocol,
 				    const string&	xrl_target_name,
 				    const string&	cookie)
-    : RedistOutput<A>(redistributor), _xrl_router(xrl_router),
+    : RedistOutput<A>(redistributor), _xrl_router(xrl_router), 
+      _profile(profile),
       _from_protocol(from_protocol), _target_name(xrl_target_name),
       _cookie(cookie), _n_tasks(0)
 {
@@ -465,6 +494,10 @@ template <typename A>
 void
 RedistXrlOutput<A>::add_route(const IPRouteEntry<A>& ipr)
 {
+    if (_profile.enabled(profile_route_rpc_in))
+	_profile.log(profile_route_rpc_in,
+		     c_format("add %s", ipr.net().str().c_str()));
+    
     enqueue_task(new AddRoute<A>(this, ipr));
     if (task_count() == 1) {
 	start_running_tasks();
@@ -475,6 +508,10 @@ template <typename A>
 void
 RedistXrlOutput<A>::delete_route(const IPRouteEntry<A>& ipr)
 {
+    if (_profile.enabled(profile_route_rpc_in))
+	_profile.log(profile_route_rpc_in,
+		     c_format("delete %s", ipr.net().str().c_str()));
+
     enqueue_task(new DeleteRoute<A>(this, ipr));
     if (task_count() == 1) {
 	start_running_tasks();
@@ -516,10 +553,10 @@ RedistXrlOutput<A>::start_next_task()
     XLOG_ASSERT(task_count() >= 1);
     RedistXrlTask<A>* t = _tasks.front();
 
-    if (t->dispatch(_xrl_router) == false) {
+    if (t->dispatch(_xrl_router, _profile) == false) {
 	if (t->dispatch_attempts() > MAX_RETRIES) {
 	    XLOG_ERROR("Failed to dispatch command after %u attempts.",
-		       t->dispatch_attempts());
+		       XORP_UINT_CAST(t->dispatch_attempts()));
 	    // XXX signal failure
 	}
 	// Dispatch of task failed.  XrlRouter is presumeably backlogged.
@@ -528,7 +565,7 @@ RedistXrlOutput<A>::start_next_task()
 	t = new Pause<A>(this, RETRY_PAUSE_MS);
 	_tasks.push_front(t);
 	incr_task_count();
-	t->dispatch(_xrl_router);
+	t->dispatch(_xrl_router, _profile);
     }
 }
 
@@ -566,7 +603,7 @@ public:
 	: AddRoute<A>(parent, ipr) {
 	parent->incr_transaction_size();
     }
-    virtual bool dispatch(XrlRouter& xrl_router);
+    virtual bool dispatch(XrlRouter& xrl_router, Profile& profile);
 };
 
 template <typename A>
@@ -577,7 +614,7 @@ public:
 	: DeleteRoute<A>(parent, ipr) {
 	parent->incr_transaction_size();
     }
-    virtual bool dispatch(XrlRouter& xrl_router);
+    virtual bool dispatch(XrlRouter& xrl_router, Profile& profile);
 };
 
 template <typename A>
@@ -587,7 +624,7 @@ public:
 	: RedistXrlTask<A>(parent) {
 	parent->reset_transaction_size();
     }
-    virtual bool dispatch(XrlRouter&  xrl_router);
+    virtual bool dispatch(XrlRouter&  xrl_router, Profile& profile);
     void dispatch_complete(const XrlError& xe, const uint32_t* tid);
 };
 
@@ -598,7 +635,7 @@ public:
 	: RedistXrlTask<A>(parent) {
 	parent->reset_transaction_size();
     }
-    virtual bool dispatch(XrlRouter&  xrl_router);
+    virtual bool dispatch(XrlRouter&  xrl_router, Profile& profile);
     void dispatch_complete(const XrlError& xe);
 };
 
@@ -609,7 +646,7 @@ public:
 	: RedistXrlTask<A>(parent) {
 	parent->reset_transaction_size();
     }
-    virtual bool dispatch(XrlRouter&  xrl_router);
+    virtual bool dispatch(XrlRouter&  xrl_router, Profile& profile);
     void dispatch_complete(const XrlError& xe);
 };
 
@@ -619,7 +656,7 @@ public:
 
 template <>
 bool
-AddTransactionRoute<IPv4>::dispatch(XrlRouter& xrl_router)
+AddTransactionRoute<IPv4>::dispatch(XrlRouter& xrl_router, Profile& profile)
 {
     RedistTransactionXrlOutput<IPv4>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv4>*>(this->parent());
@@ -631,10 +668,18 @@ AddTransactionRoute<IPv4>::dispatch(XrlRouter& xrl_router)
 	return true;	// XXX: we return true to avoid retransmission
     }
 
+    if (profile.enabled(profile_route_rpc_out))
+	profile.log(profile_route_rpc_out,
+		     c_format("add %s %s %s %u", 
+			      p->xrl_target_name().c_str(),
+			      _net.str().c_str(),
+			      _nexthop.str().c_str(),
+			      XORP_UINT_CAST(_metric)));
+
     XrlRedistTransaction4V0p1Client cl(&xrl_router);
     return cl.send_add_route(p->xrl_target_name().c_str(),
 			     p->tid(),
-			     _net, _nh, _ifname, _vifname, _metric,
+			     _net, _nexthop, _ifname, _vifname, _metric,
 			     _admin_distance, p->cookie(),
 			     _protocol_origin,
 			     callback(static_cast<AddRoute<IPv4>*>(this),
@@ -644,7 +689,7 @@ AddTransactionRoute<IPv4>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-AddTransactionRoute<IPv6>::dispatch(XrlRouter& xrl_router)
+AddTransactionRoute<IPv6>::dispatch(XrlRouter& xrl_router, Profile& profile)
 {
     RedistTransactionXrlOutput<IPv6>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv6>*>(this->parent());
@@ -656,10 +701,18 @@ AddTransactionRoute<IPv6>::dispatch(XrlRouter& xrl_router)
 	return true;	// XXX: we return true to avoid retransmission
     }
 
+    if (profile.enabled(profile_route_rpc_out))
+	profile.log(profile_route_rpc_out,
+		     c_format("add %s %s %s %u", 
+			      p->xrl_target_name().c_str(),
+			      _net.str().c_str(),
+			      _nexthop.str().c_str(),
+			      XORP_UINT_CAST(_metric)));
+
     XrlRedistTransaction6V0p1Client cl(&xrl_router);
     return cl.send_add_route(p->xrl_target_name().c_str(),
 			     p->tid(),
-			     _net, _nh, _ifname, _vifname, _metric,
+			     _net, _nexthop, _ifname, _vifname, _metric,
 			     _admin_distance, p->cookie(),
 			     _protocol_origin,
 			     callback(static_cast<AddRoute<IPv6>*>(this),
@@ -673,7 +726,7 @@ AddTransactionRoute<IPv6>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-DeleteTransactionRoute<IPv4>::dispatch(XrlRouter& xrl_router)
+DeleteTransactionRoute<IPv4>::dispatch(XrlRouter& xrl_router, Profile& profile)
 {
     RedistTransactionXrlOutput<IPv4>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv4>*>(this->parent());
@@ -685,11 +738,18 @@ DeleteTransactionRoute<IPv4>::dispatch(XrlRouter& xrl_router)
 	return true;	// XXX: we return true to avoid retransmission
     }
 
+    if (profile.enabled(profile_route_rpc_out))
+	profile.log(profile_route_rpc_out,
+		     c_format("delete %s %s", 
+			      p->xrl_target_name().c_str(),
+			      _net.str().c_str()));
+
     XrlRedistTransaction4V0p1Client cl(&xrl_router);
     return cl.send_delete_route(
 	p->xrl_target_name().c_str(),
 	p->tid(),
 	_net, p->cookie(),
+	_protocol_origin,
 	callback(static_cast<DeleteRoute<IPv4>*>(this),
 		 &DeleteRoute<IPv4>::dispatch_complete)
 	);
@@ -697,7 +757,7 @@ DeleteTransactionRoute<IPv4>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-DeleteTransactionRoute<IPv6>::dispatch(XrlRouter& xrl_router)
+DeleteTransactionRoute<IPv6>::dispatch(XrlRouter& xrl_router, Profile& profile)
 {
     RedistTransactionXrlOutput<IPv6>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv6>*>(this->parent());
@@ -709,11 +769,18 @@ DeleteTransactionRoute<IPv6>::dispatch(XrlRouter& xrl_router)
 	return true;	// XXX: we return true to avoid retransmission
     }
 
+    if (profile.enabled(profile_route_rpc_out))
+	profile.log(profile_route_rpc_out,
+		     c_format("delete %s %s", 
+			      p->xrl_target_name().c_str(),
+			      _net.str().c_str()));
+
     XrlRedistTransaction6V0p1Client cl(&xrl_router);
     return cl.send_delete_route(
 	p->xrl_target_name().c_str(),
 	p->tid(),
 	_net, p->cookie(),
+	_protocol_origin,
 	callback(static_cast<DeleteRoute<IPv6>*>(this),
 		 &DeleteRoute<IPv6>::dispatch_complete)
 	);
@@ -725,7 +792,7 @@ DeleteTransactionRoute<IPv6>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-StartTransaction<IPv4>::dispatch(XrlRouter& xrl_router)
+StartTransaction<IPv4>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistTransactionXrlOutput<IPv4>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv4>*>(this->parent());
@@ -742,7 +809,7 @@ StartTransaction<IPv4>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-StartTransaction<IPv6>::dispatch(XrlRouter& xrl_router)
+StartTransaction<IPv6>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistTransactionXrlOutput<IPv6>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv6>*>(this->parent());
@@ -768,14 +835,14 @@ StartTransaction<A>::dispatch_complete(const XrlError& xe, const uint32_t* tid)
 	this->signal_complete_ok();
 	return;
     } else if (xe == XrlError::COMMAND_FAILED()) {
-	XLOG_ERROR("Failed to start transaction");
+	XLOG_ERROR("Failed to start transaction: %s", xe.str().c_str());
 	p->set_transaction_in_progress(false);
 	p->set_transaction_in_error(true);
 	this->signal_complete_ok();
 	return;
     }
     // For now all errors are signalled fatal
-    XLOG_ERROR("Fatal error during start transaction \"%s\"",
+    XLOG_ERROR("Fatal error during start transaction: %s",
 	       xe.str().c_str());
 
     this->signal_fatal_failure();
@@ -787,7 +854,7 @@ StartTransaction<A>::dispatch_complete(const XrlError& xe, const uint32_t* tid)
 
 template <>
 bool
-CommitTransaction<IPv4>::dispatch(XrlRouter& xrl_router)
+CommitTransaction<IPv4>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistTransactionXrlOutput<IPv4>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv4>*>(this->parent());
@@ -807,7 +874,7 @@ CommitTransaction<IPv4>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-CommitTransaction<IPv6>::dispatch(XrlRouter& xrl_router)
+CommitTransaction<IPv6>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistTransactionXrlOutput<IPv6>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv6>*>(this->parent());
@@ -833,12 +900,12 @@ CommitTransaction<A>::dispatch_complete(const XrlError& xe)
 	this->signal_complete_ok();
 	return;
     } else if (xe == XrlError::COMMAND_FAILED()) {
-	XLOG_ERROR("Failed to commit transaction");
+	XLOG_ERROR("Failed to commit transaction: %s", xe.str().c_str());
 	this->signal_complete_ok();
 	return;
     }
     // For now all errors are signalled fatal
-    XLOG_ERROR("Fatal error during commit transaction \"%s\"",
+    XLOG_ERROR("Fatal error during commit transaction: %s",
 	       xe.str().c_str());
 
     this->signal_fatal_failure();
@@ -850,7 +917,7 @@ CommitTransaction<A>::dispatch_complete(const XrlError& xe)
 
 template <>
 bool
-AbortTransaction<IPv4>::dispatch(XrlRouter& xrl_router)
+AbortTransaction<IPv4>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistTransactionXrlOutput<IPv4>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv4>*>(this->parent());
@@ -870,7 +937,7 @@ AbortTransaction<IPv4>::dispatch(XrlRouter& xrl_router)
 
 template <>
 bool
-AbortTransaction<IPv6>::dispatch(XrlRouter& xrl_router)
+AbortTransaction<IPv6>::dispatch(XrlRouter& xrl_router, Profile&)
 {
     RedistTransactionXrlOutput<IPv6>* p =
 	reinterpret_cast<RedistTransactionXrlOutput<IPv6>*>(this->parent());
@@ -896,12 +963,12 @@ AbortTransaction<A>::dispatch_complete(const XrlError& xe)
 	this->signal_complete_ok();
 	return;
     } else if (xe == XrlError::COMMAND_FAILED()) {
-	XLOG_ERROR("Failed to abort transaction");
+	XLOG_ERROR("Failed to abort transaction: %s", xe.str().c_str());
 	this->signal_complete_ok();
 	return;
     }
     // For now all errors are signalled fatal
-    XLOG_ERROR("Fatal error during abort transaction \"%s\"",
+    XLOG_ERROR("Fatal error during abort transaction: %s",
 	       xe.str().c_str());
 
     this->signal_fatal_failure();
@@ -915,11 +982,12 @@ template <typename A>
 RedistTransactionXrlOutput<A>::RedistTransactionXrlOutput(
 				Redistributor<A>*	redistributor,
 				XrlRouter&		xrl_router,
+				Profile&		profile,
 				const string&		from_protocol,
 				const string&		xrl_target_name,
 				const string&		cookie
 				)
-    : RedistXrlOutput<A>(redistributor, xrl_router, from_protocol,
+    : RedistXrlOutput<A>(redistributor, xrl_router, profile, from_protocol,
 			 xrl_target_name, cookie),
       _tid(0),
       _transaction_in_progress(false),
@@ -932,6 +1000,14 @@ template <typename A>
 void
 RedistTransactionXrlOutput<A>::add_route(const IPRouteEntry<A>& ipr)
 {
+    if (this->_profile.enabled(profile_route_rpc_in))
+	this->_profile.log(profile_route_rpc_in,
+			   c_format("add %s %s %s %u",
+				    ipr.protocol().name().c_str(),
+				    ipr.net().str().c_str(),
+				    ipr.nexthop()->str().c_str(),
+				    XORP_UINT_CAST(ipr.metric())));
+
     bool no_running_tasks = (this->task_count() == 0);
 
     if (this->transaction_size() == 0)
@@ -955,6 +1031,12 @@ template <typename A>
 void
 RedistTransactionXrlOutput<A>::delete_route(const IPRouteEntry<A>& ipr)
 {
+    if (this->_profile.enabled(profile_route_rpc_in))
+	this->_profile.log(profile_route_rpc_in,
+			   c_format("add %s %s",
+				    ipr.protocol().name().c_str(),
+				    ipr.net().str().c_str()));
+
     bool no_running_tasks = (this->task_count() == 0);
 
     if (this->transaction_size() == 0)
