@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rib.cc,v 1.35 2004/07/02 05:03:51 pavlin Exp $"
+#ident "$XORP: xorp/rib/rib.cc,v 1.55 2005/03/25 02:54:20 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -25,6 +26,7 @@
 #include "rib_manager.hh"
 #include "rib.hh"
 
+
 // ----------------------------------------------------------------------------
 // Inline table utility methods
 
@@ -121,10 +123,10 @@ RIB<A>::remove_table(const string& tablename)
 }
 
 template <typename A>
-inline int
+inline uint32_t
 RIB<A>::admin_distance(const string& tablename)
 {
-    map<const string, int>::iterator mi;
+    map<string, uint32_t>::iterator mi;
 
     mi = _admin_distances.find(tablename);
     if (mi == _admin_distances.end()) {
@@ -139,12 +141,14 @@ template <typename A>
 inline Vif*
 RIB<A>::find_vif(const A& addr)
 {
-    map<const string, Vif>::iterator mi;
+    map<string, Vif>::iterator mi;
 
     for (mi = _vifs.begin(); mi != _vifs.end(); ++mi) {
 	Vif& v = mi->second;
 	if (v.is_my_addr(addr))
 	    return &v;
+	if (v.is_p2p() && v.is_same_p2p(addr))
+	    return &v;
     }
     return NULL;
 }
@@ -153,7 +157,7 @@ template <typename A>
 inline IPExternalNextHop<A>*
 RIB<A>::find_external_nexthop(const A& addr)
 {
-    typename map<const A, IPExternalNextHop<A> >::iterator mi;
+    typename map<A, IPExternalNextHop<A> >::iterator mi;
 
     mi = _external_nexthops.find(addr);
     if (mi == _external_nexthops.end())
@@ -165,7 +169,7 @@ template <typename A>
 inline IPPeerNextHop<A>*
 RIB<A>::find_peer_nexthop(const A& addr)
 {
-    typename map<const A, IPPeerNextHop<A> >::iterator mi;
+    typename map<A, IPPeerNextHop<A> >::iterator mi;
 
     mi = _peer_nexthops.find(addr);
     if (mi == _peer_nexthops.end())
@@ -181,7 +185,7 @@ RIB<A>::find_or_create_external_nexthop(const A& addr)
     if (nexthop != NULL)
 	return nexthop;
 
-    typedef map<const A,IPExternalNextHop<A> > C;	// for convenience
+    typedef map<A, IPExternalNextHop<A> > C;	// for convenience
     typename C::value_type vt(addr, IPExternalNextHop<A>(addr));
     typename C::iterator iter;
     iter = _external_nexthops.insert(_external_nexthops.end(), vt);
@@ -196,7 +200,7 @@ RIB<A>::find_or_create_peer_nexthop(const A& addr)
     if (nexthop != NULL)
 	return nexthop;
 
-    typedef map<const A,IPPeerNextHop<A> > C;		// for convenience
+    typedef map<A, IPPeerNextHop<A> > C;		// for convenience
     typename C::value_type vt(addr, addr);
     typename C::iterator iter;
     iter = _peer_nexthops.insert(_peer_nexthops.end(), vt);
@@ -206,12 +210,6 @@ RIB<A>::find_or_create_peer_nexthop(const A& addr)
 // ----------------------------------------------------------------------------
 // Naming utility methods
 
-static inline const char*
-export_tablename()
-{
-    return "ExportToRibClients";
-}
-
 static inline string
 redist_tablename(const string& from_table)
 {
@@ -227,7 +225,8 @@ RIB<A>::RIB(RibTransportType t, RibManager& rib_manager, EventLoop& eventloop)
       _eventloop(eventloop),
       _final_table(NULL),
       _register_table(NULL),
-      _errors_are_fatal(false)
+      _errors_are_fatal(false),
+      _policy_redist_table(NULL)
 {
     if (t == UNICAST) {
 	_multicast = false;
@@ -263,18 +262,60 @@ RIB<A>::~RIB()
 }
 
 template <typename A>
-int
-RIB<A>::initialize_export(list<RibClient* >* rib_clients_list)
+void
+RIB<A>::initialize(RegisterServer& register_server)
 {
-    ExportTable<A>* et = new ExportTable<A>(export_tablename(), 0,
-					    rib_clients_list);
-
-    if (add_table(et) != XORP_OK) {
-	XLOG_FATAL("Export already initialized.");
-	//delete et;
-	//return XORP_ERROR;
+    if (initialize_register(register_server) != XORP_OK) {
+	XLOG_FATAL("Could not initialize register table for %s",
+		   name().c_str());
     }
-    _final_table = et;
+
+    if (initialize_policy_redist() != XORP_OK) {
+	XLOG_FATAL("Could not initialize policy redistribution table for %s",
+		   name().c_str());
+    }
+    
+    //
+    // XXX: we must initialize the final RedistTable after the
+    // RegistTable has been initialized.
+    //
+    if (initialize_redist_all("all") != XORP_OK) {
+	XLOG_FATAL("Could not initialize all-protocol redistribution "
+		   "table for %s",
+		   name().c_str());
+    }
+    if (add_igp_table("connected", "", "") != XORP_OK) {
+	XLOG_FATAL("Could not add igp table \"connected\" for %s",
+		   name().c_str());
+    }
+}
+
+template <typename A>
+int
+RIB<A>::initialize_policy_redist()
+{
+    if (_register_table == NULL) {
+	XLOG_ERROR("Register table is not yet initialized");
+	return XORP_ERROR;
+    }
+
+    if (_policy_redist_table != NULL) {
+	return XORP_OK;	// done already
+    }
+
+    _policy_redist_table =
+	new PolicyRedistTable<A>(_register_table, _rib_manager.xrl_router(),
+				 _rib_manager.policy_redist_map(),
+				 _multicast);
+    if (add_table(_policy_redist_table) != XORP_OK) {
+	delete _policy_redist_table;
+	_policy_redist_table = NULL;
+	return XORP_ERROR;
+    }
+
+    if (_final_table == NULL || _final_table == _register_table)
+	_final_table = _policy_redist_table;
+    
     return XORP_OK;
 }
 
@@ -282,8 +323,8 @@ template <typename A>
 int
 RIB<A>::initialize_redist_all(const string& all)
 {
-    if (_register_table == NULL) {
-	XLOG_ERROR("Register table is not yet initialized");
+    if (_policy_redist_table == NULL) {
+	XLOG_ERROR("Policy redist table is not yet initialized");
 	return XORP_ERROR;
     }
 
@@ -292,18 +333,24 @@ RIB<A>::initialize_redist_all(const string& all)
     }
 
     RedistTable<A>* r;
-    r = new RedistTable<A>(redist_tablename(all), _register_table);
+    r = new RedistTable<A>(redist_tablename(all), _policy_redist_table);
     if (add_table(r) != XORP_OK) {
 	delete r;
 	return XORP_ERROR;
     }
 
+    //
+    // Set the RedistTable as the final table
+    //
+    if (_final_table == NULL || _final_table == _policy_redist_table)
+	_final_table = r;
+
     return XORP_OK;
 }
 
 template <typename A>
 int
-RIB<A>::initialize_register(RegisterServer* regserv)
+RIB<A>::initialize_register(RegisterServer& register_server)
 {
     if (_register_table != NULL) {
 	XLOG_WARNING("Register table already initialized.");
@@ -311,7 +358,7 @@ RIB<A>::initialize_register(RegisterServer* regserv)
     }
 
     RegisterTable<A>* rt;
-    rt = new RegisterTable<A>("RegisterTable", regserv, _multicast);
+    rt = new RegisterTable<A>("RegisterTable", register_server, _multicast);
     if (add_table(rt) != XORP_OK) {
 	XLOG_WARNING("Add RegisterTable failed.");
 	delete rt;
@@ -319,11 +366,7 @@ RIB<A>::initialize_register(RegisterServer* regserv)
     }
     _register_table = rt;
 
-    if (find_table(export_tablename()) == NULL) {
-	// No ExportTable<A> - perhaps we're an MRIB.
-	if (_final_table != NULL) {
-	    XLOG_FATAL("No export table when initializing register table");
-	}
+    if (_final_table == NULL) {
 	_final_table = _register_table;
     } else {
 	_final_table->replumb(NULL, _register_table);
@@ -337,7 +380,7 @@ int
 RIB<A>::new_origin_table(const string&	tablename,
 			 const string&	target_class,
 			 const string&	target_instance,
-			 int		admin_distance,
+			 uint32_t	admin_distance,
 			 ProtocolType	protocol_type)
 {
     OriginTable<A>* ot = new OriginTable<A>(tablename, admin_distance,
@@ -346,7 +389,9 @@ RIB<A>::new_origin_table(const string&	tablename,
 	XLOG_WARNING("Could not add origin table %s", tablename.c_str());
 	delete ot;
 	return XORP_ERROR;
-    } else if (_final_table == NULL) {
+    }
+
+    if (_final_table == NULL) {
 	_final_table = ot;
     }
 
@@ -370,11 +415,11 @@ RIB<IPv4>::new_vif(const string& vifname, const Vif& vif)
     debug_msg("RIB::new_vif: %s\n", vifname.c_str());
     if (_vifs.find(vifname) == _vifs.end()) {
 	// Can't use _vifs[vifname] = vif because no Vif() constructor
-	map<const string, Vif>::value_type v(vifname, vif);
+	map<string, Vif>::value_type v(vifname, vif);
 	_vifs.insert(_vifs.end(), v);
 
 	// We need to add the routes from the VIF to the connected table
-	map<const string, Vif>::iterator iter = _vifs.find(vifname);
+	map<string, Vif>::iterator iter = _vifs.find(vifname);
 	XLOG_ASSERT(iter != _vifs.end());
 	Vif* new_vif = &(iter->second);
 	XLOG_ASSERT(new_vif != NULL);
@@ -384,7 +429,14 @@ RIB<IPv4>::new_vif(const string& vifname, const Vif& vif)
 	     ai++) {
 	    if (ai->addr().is_ipv4()) {
 		add_route("connected", ai->subnet_addr().get_ipv4net(),
-			  ai->addr().get_ipv4(), "", "", 0);
+			  ai->addr().get_ipv4(), "", "", 0, PolicyTags());
+		if (new_vif->is_p2p() && (ai->peer_addr() != IPv4::ZERO())) {
+		    add_route("connected",
+			      IPv4Net(ai->peer_addr().get_ipv4(),
+				      IPv4::addr_bitlen()),
+			      ai->peer_addr().get_ipv4(), "", "", 0,
+			      PolicyTags());
+		}
 	    }
 	}
 
@@ -400,11 +452,11 @@ RIB<IPv6>::new_vif(const string& vifname, const Vif& vif)
     debug_msg("RIB::new_vif: %s\n", vifname.c_str());
     if (_vifs.find(vifname) == _vifs.end()) {
 	// Can't use _vifs[vifname] = vif because no Vif() constructor
-	map<const string, Vif>::value_type v(vifname, vif);
+	map<string, Vif>::value_type v(vifname, vif);
 	_vifs.insert(_vifs.end(), v);
 
 	// We need to add the routes from the VIF to the connected table
-	map<const string, Vif>::iterator iter = _vifs.find(vifname);
+	map<string, Vif>::iterator iter = _vifs.find(vifname);
 	XLOG_ASSERT(iter != _vifs.end());
 	Vif* new_vif = &(iter->second);
 	XLOG_ASSERT(new_vif != NULL);
@@ -414,7 +466,14 @@ RIB<IPv6>::new_vif(const string& vifname, const Vif& vif)
 	     ai++) {
 	    if (ai->addr().is_ipv6()) {
 		add_route("connected", ai->subnet_addr().get_ipv6net(),
-			  ai->addr().get_ipv6(), "", "", 0);
+			  ai->addr().get_ipv6(), "", "", 0, PolicyTags());
+		if (new_vif->is_p2p() && (ai->peer_addr() != IPv6::ZERO())) {
+		    add_route("connected",
+			      IPv6Net(ai->peer_addr().get_ipv6(),
+				      IPv6::addr_bitlen()),
+			      ai->peer_addr().get_ipv6(), "", "", 0,
+			      PolicyTags());
+		}
 	    }
 	}
 
@@ -428,7 +487,7 @@ int
 RIB<A>::delete_vif(const string& vifname)
 {
     debug_msg("RIB::delete_vif: %s\n", vifname.c_str());
-    map<const string, Vif>::iterator iter;
+    map<string, Vif>::iterator iter;
     iter = _vifs.find(vifname);
     if (iter == _vifs.end()) {
 	return XORP_ERROR;
@@ -463,7 +522,7 @@ RIB<A>::set_vif_flags(const string& vifname,
 		      bool is_broadcast,
 		      bool is_up)
 {
-    map<const string, Vif>::iterator vi = _vifs.find(vifname);
+    map<string, Vif>::iterator vi = _vifs.find(vifname);
     if (vi == _vifs.end()) {
 	XLOG_ERROR("Attempting to set flags to non-existant Vif \"%s\"",
 		   vifname.c_str());
@@ -488,7 +547,7 @@ RIB<A>::add_vif_address(const string&	vifname,
 			const A&	broadcast_addr,
 			const A&	peer_addr)
 {
-    map<const string, Vif>::iterator vi = _vifs.find(vifname);
+    map<string, Vif>::iterator vi = _vifs.find(vifname);
     if (vi == _vifs.end()) {
 	XLOG_ERROR("Attempting to add address to non-existant Vif \"%s\"",
 		   vifname.c_str());
@@ -496,7 +555,13 @@ RIB<A>::add_vif_address(const string&	vifname,
     }
     vi->second.add_address(VifAddr(addr, subnet, broadcast_addr, peer_addr));
     // Add a route for this subnet
-    add_route("connected", subnet, addr, "", "", /* best possible metric */ 0);
+    add_route("connected", subnet, addr, "", "", /* best possible metric */ 0,
+	      PolicyTags());
+    if (vi->second.is_p2p() && (peer_addr != A::ZERO())) {
+	add_route("connected",
+		  IPNet<A>(peer_addr, A::addr_bitlen()),
+		  peer_addr, "", "", 0, PolicyTags());
+    }
     return XORP_OK;
 }
 
@@ -505,7 +570,7 @@ int
 RIB<A>::delete_vif_address(const string& vifname,
 			   const A& addr)
 {
-    map<const string, Vif>::iterator vi = _vifs.find(vifname);
+    map<string, Vif>::iterator vi = _vifs.find(vifname);
     if (vi == _vifs.end()) {
 	XLOG_ERROR("Attempting to delete address from non-existant Vif \"%s\"",
 		   vifname.c_str());
@@ -541,7 +606,8 @@ RIB<A>::add_route(const string&		tablename,
 		  const A&		nexthop_addr,
 		  const string&		ifname,
 		  const string&		vifname,
-		  uint32_t		metric)
+		  uint32_t		metric,
+		  const PolicyTags&	policytags)
 {
     RouteTable<A>* rt = find_table(tablename);
     if (rt == NULL) {
@@ -583,7 +649,7 @@ RIB<A>::add_route(const string&		tablename,
 	//
 	// Add a route with explicitly specified network interface
 	//
-	map<const string, Vif>::iterator iter = _vifs.find(vifname);
+	map<string, Vif>::iterator iter = _vifs.find(vifname);
 	if (iter == _vifs.end()) {
 	    XLOG_ERROR("Attempting to add route to table \"%s\" "
 		       "(prefix %s next-hop %s ifname %s vifname %s): "
@@ -595,67 +661,49 @@ RIB<A>::add_route(const string&		tablename,
 	}
 	Vif* vif = &iter->second;
 	IPNextHop<A>* nexthop = find_or_create_peer_nexthop(nexthop_addr);
-	ot->add_route(IPRouteEntry<A>(net, vif, nexthop, *protocol, metric));
+	ot->add_route(IPRouteEntry<A>(net, vif, nexthop, *protocol,
+		      metric, policytags));
 	flush();
 	return XORP_OK;
     }
 
     //
-    // Find the vif so we can see if the nexthop is a peer.  First
-    // lookup the nexthop addr, and see it's the subnet is a directly
-    // connected one.
+    // Find the vif so we can see if the nexthop is directly connected
     //
-    const IPRouteEntry<A>* re = _final_table->lookup_route(nexthop_addr);
-    if (re != NULL) {
-	// We found a route.  Is the subnet directly connected?
-	Vif* vif = re->vif();
-	if ((vif != NULL)
-	    && (vif->is_same_subnet(IPvXNet(re->net()))
-		|| vif->is_same_p2p(IPvX(nexthop_addr)))) {
-	    debug_msg("**directly connected route found for nexthop\n");
-	    IPNextHop<A>* nexthop = find_or_create_peer_nexthop(nexthop_addr);
-	    ot->add_route(IPRouteEntry<A>(net, re->vif(), nexthop,
-					  *protocol, metric));
-	    flush();
-	    return XORP_OK;
-	} else {
-	    debug_msg("**not directly connected route found for nexthop\n");
-	    //
-	    // XXX: If the route came from an IGP, then we must have
-	    // a directly-connected interface toward the next-hop router
-	    //
-	    if (protocol->protocol_type() == IGP) {
-		XLOG_ERROR("Attempting to add IGP route to table \"%s\" "
-			   "(prefix %s next-hop %s): no directly connected "
-			   "interface toward the next-hop router",
-			   tablename.c_str(), net.str().c_str(),
-			   nexthop_addr.str().c_str());
-		return XORP_ERROR;
+    Vif* vif = NULL;
+    IPNextHop<A>* nexthop = NULL;
+    do {
+	//
+	// Search for a route to a directly-connected destination
+	//
+	const IPRouteEntry<A>* re = _final_table->lookup_route(nexthop_addr);
+	if (re != NULL) {
+	    // We found a route for the nexthop
+	    vif = re->vif();
+	    if ((vif != NULL)
+		&& (vif->is_same_subnet(IPvXNet(re->net()))
+		    || vif->is_same_p2p(IPvX(nexthop_addr)))) {
+		debug_msg("**directly connected route found for nexthop\n");
+		break;
 	    }
-
-	    IPNextHop<A>* nexthop = find_or_create_external_nexthop(nexthop_addr);
-	    ot->add_route(IPRouteEntry<A>(net, /* No vif */ NULL, nexthop,
-					  *protocol, metric));
-	    flush();
-	    return XORP_OK;
 	}
-    }
 
-    debug_msg("** no route\n");
-    //
-    // We failed to find a route.  One final possibility is that this
-    // route is the route for a directly connected subnet, so we need
-    // to test all the Vifs to see if this is the case.
-    //
-    Vif* vif = find_vif(nexthop_addr);
-    debug_msg("Vif %p\n", vif);
-    IPNextHop<A>* nexthop;
-    if (vif != NULL) {
-	nexthop = find_or_create_peer_nexthop(nexthop_addr);
-    } else {
+	//
+	// We failed to find a route, or the route wasn't for a
+	// directly-connected destination. One final possibility is that
+	// we are trying to add a route for a directly connected subnet,
+	// so we need to test all the Vifs to see if this is the case.
+	//
+	vif = find_vif(nexthop_addr);
+	debug_msg("Vif %p\n", vif);
+	break;
+    } while (false);
+
+    if (vif == NULL) {
+	debug_msg("**not directly connected route found for nexthop\n");
 	//
 	// XXX: If the route came from an IGP, then we must have
-	// a directly-connected interface toward the next-hop router
+	// a directly-connected interface toward the next-hop router.
 	//
 	if (protocol->protocol_type() == IGP) {
 	    XLOG_ERROR("Attempting to add IGP route to table \"%s\" "
@@ -665,19 +713,30 @@ RIB<A>::add_route(const string&		tablename,
 		       nexthop_addr.str().c_str());
 	    return XORP_ERROR;
 	}
+    }
 
+    if (vif != NULL) {
+	nexthop = find_or_create_peer_nexthop(nexthop_addr);
+    } else {
 	nexthop = find_or_create_external_nexthop(nexthop_addr);
     }
     XLOG_ASSERT(nexthop->addr() == nexthop_addr);
 
+    //
     // Only accept the least significant 16 bits of metric.
+    //
     if (metric > 0xffff) {
-	XLOG_WARNING("IGP metric value %d is greater than 0xffff", metric);
+	XLOG_WARNING("Protocol metric value %u is greater than 0xffff from "
+		     "table \"%s\"",  XORP_UINT_CAST(metric),
+		     tablename.c_str());
 	metric &= 0xffff;
     }
 
-    ot->add_route(IPRouteEntry<A>(net, vif, nexthop, *protocol, metric));
-
+    //
+    // Add the route
+    //
+    ot->add_route(IPRouteEntry<A>(net, vif, nexthop,
+				  *protocol, metric, policytags));
     flush();
     return XORP_OK;
 }
@@ -689,7 +748,8 @@ RIB<A>::replace_route(const string&	tablename,
 		      const A&		nexthop_addr,
 		      const string&	ifname,
 		      const string&	vifname,
-		      uint32_t		metric)
+		      uint32_t		metric,
+		      const PolicyTags&	policytags)
 {
     RouteTable<A>* rt = find_table(tablename);
     if (NULL == rt)
@@ -704,7 +764,7 @@ RIB<A>::replace_route(const string&	tablename,
 	return response;
 
     response = add_route(tablename, net, nexthop_addr, ifname, vifname,
-			 metric);
+			 metric, policytags);
 
     // No need to flush here, as add_route will do it for us.
 
@@ -743,15 +803,17 @@ int
 RIB<A>::verify_route(const A& lookup_addr,
 		     const string& ifname,
 		     const A& nexthop_addr,
-		     uint32_t metric)
+		     uint32_t metric,
+		     RibVerifyType matchtype)
 {
     const IPRouteEntry<A>* re;
 
+    // 1. Check for an expected route miss.
+    // 2. Check table entry validity and existence.
     re = _final_table->lookup_route(lookup_addr);
     if (re == NULL || re->vif() == NULL) {
-	// TODO: XXX: hard-coded interface name!!
-	if (ifname == "discard") {
-	    debug_msg("****ROUTE FAILURE SUCCESSFULLY VERIFIED****\n");
+	if (matchtype == RibVerifyType(MISS)) {
+	    debug_msg("****ROUTE MISS SUCCESSFULLY VERIFIED****\n");
 	    return XORP_OK;
 	}
 	if (re == NULL) {
@@ -763,6 +825,22 @@ RIB<A>::verify_route(const A& lookup_addr,
 	return XORP_ERROR;
     }
 
+#ifdef notyet
+    // 3. Check for discard (blackhole) routes.
+    // XXX: re->vif() must be non-NULL and valid. Revisit this in future.
+    DiscardNextHop* dnh = dynamic_cast<DiscardNextHop*>(re->nexthop());
+    if (matchtype == RibVerifyType(DISCARD)) {
+	    if (dnh == NULL) {
+		    debug_msg("Next hop is not a DiscardNextHop");
+		    return XORP_ERROR;
+	    } else {
+		debug_msg("****DISCARD ROUTE SUCCESSFULLY VERIFIED****\n");
+		return XORP_OK;
+	    }
+    }
+#endif
+
+    // 4. Check for protocol level routes (specifically IP).
     IPNextHop<A>* route_nexthop = dynamic_cast<IPNextHop<A>* >(re->nexthop());
     if (route_nexthop == NULL) {
 	debug_msg("Next hop is not an IPNextHop\n");
@@ -787,14 +865,15 @@ RIB<A>::verify_route(const A& lookup_addr,
 		  re->vif()->name().c_str());
     }
     if (metric != re->metric()) {
-	XLOG_ERROR("Metric \"%d\" does not match expected \"%d\".",
-		   re->metric(), metric);
+	XLOG_ERROR("Metric \"%u\" does not match expected \"%u\".",
+		   XORP_UINT_CAST(re->metric()), XORP_UINT_CAST(metric));
 	return XORP_ERROR;
     } else {
-	debug_msg("Metric: Exp: %d == Got: %d\n", metric,
-		  re->metric());
+	debug_msg("Metric: Exp: %u == Got: %u\n",
+		  XORP_UINT_CAST(metric),
+		  XORP_UINT_CAST(re->metric()));
     }
-    debug_msg("****ROUTE SUCCESSFULLY VERIFIED****\n");
+    debug_msg("****IP ROUTE SUCCESSFULLY VERIFIED****\n");
     return XORP_OK;
 }
 
@@ -805,12 +884,30 @@ RIB<A>::lookup_route(const A& lookupaddr)
     debug_msg("looking up %s\n", lookupaddr.str().c_str());
 
     const IPRouteEntry<A>* re = _final_table->lookup_route(lookupaddr);
-    if (re == NULL || re->vif() == NULL) {
+    // Case 1: Route miss. Return the null IP address.
+    // Vif cannot be NULL for a valid route.
+    if (re == NULL || re->vif() == NULL)
 	return A::ZERO();
-    }
 
+#ifdef notyet
+    DiscardNextHop* discard_nexthop =
+	dynamic_cast<DiscardNextHop* >(re->nexthop());
+    // Case 2: Discard route. Return the loopback address.
+    if (discard_nexthop != NULL)
+	return A::LOOPBACK();
+
+    IPNextHop<A>* ip_nexthop = dynamic_cast<IPNextHop<A>* >(re->nexthop());
+    // Case 3: IP protocol route. Return the nexthop address.
+    if (ip_nexthop != NULL)
+	return ip_nexthop->addr();
+
+    // Default: Return the null IP address.
+    return A::ZERO();
+#else
+    // Default: Assume the route points to a resolved IPNextHop.
     IPNextHop<A>* route_nexthop = static_cast<IPNextHop<A>* >(re->nexthop());
     return route_nexthop->addr();
+#endif
 }
 
 template <typename A>
@@ -841,10 +938,10 @@ RedistTable<A>*
 RIB<A>::protocol_redist_table(const string& protocol)
 {
     RouteTable<A>* rt = find_table(redist_tablename(protocol));
-    if (rt) {
+    if (rt != NULL) {
 	return dynamic_cast<RedistTable<A>*>(rt);
     }
-    return 0;
+    return NULL;
 }
 
 template <typename A>
@@ -865,6 +962,21 @@ RIB<A>::add_igp_table(const string& tablename,
     r = add_redist_table(tablename);
     if (r != XORP_OK) {
 	delete_origin_table(tablename, target_class, target_instance);
+	return r;
+    }
+    RouteTable<A>* rt = find_table(redist_tablename(tablename));
+    XLOG_ASSERT(rt != NULL);
+
+    if (tablename == "connected") {
+	r = add_policy_connected_table(rt->tablename());
+	if (r != XORP_OK) {
+	    delete_origin_table(tablename, target_class, target_instance);
+	    //
+	    // XXX: we don't delete the redist table but keep it around,
+	    // because delete_origin_table() does similar.
+	    //
+	    return r;
+	}
     }
 
     return r;
@@ -890,11 +1002,38 @@ RIB<A>::add_egp_table(const string& tablename,
     r = add_redist_table(tablename);
     if (r != XORP_OK) {
 	delete_origin_table(tablename, target_class, target_instance);
+	return r;
     }
+    RouteTable<A>* rt = find_table(redist_tablename(tablename));
+    XLOG_ASSERT(rt != NULL);
 #endif
     return r;
 }
 
+template <typename A>
+int
+RIB<A>::add_policy_connected_table(const string& parent_tablename)
+{
+    RouteTable<A>* parent = find_table(parent_tablename);
+    if (parent == NULL) {
+	XLOG_WARNING("add_policy_connected_table: parent table %s does not exist",
+		     parent_tablename.c_str());
+	return XORP_ERROR;
+    }
+
+    if (find_table(PolicyConnectedTable<A>::table_name) != NULL)
+	return XORP_OK;
+
+    PolicyConnectedTable<A>* pt = 
+	new PolicyConnectedTable<A>(parent, _rib_manager.policy_filters());
+    if (add_table(pt) != XORP_OK) {
+	delete pt;
+	return XORP_ERROR;
+    }
+
+    return XORP_OK;
+}
+
 template <typename A>
 int
 RIB<A>::add_redist_table(const string& parent_tablename)
@@ -906,7 +1045,7 @@ RIB<A>::add_redist_table(const string& parent_tablename)
 	return XORP_ERROR;
     }
 
-    if (find_table(redist_tablename(parent_tablename)) != 0) {
+    if (find_table(redist_tablename(parent_tablename)) != NULL) {
 	return XORP_OK;		// RedistTable already exists, no sweat
     }
 
@@ -945,12 +1084,27 @@ RIB<A>::add_origin_table(const string& tablename,
     // Check if table exists and check type if so
     RouteTable<A>* rt = find_table(tablename);
     if (rt != NULL) {
-	if (dynamic_cast<OriginTable<A>* >(rt) == NULL) {
+	OriginTable<A>* ot = dynamic_cast<OriginTable<A>* >(rt);
+	if (ot == NULL) {
 	    XLOG_ERROR("add_origin_table: table \"%s\" already exists, but is "
 		       "not is an OriginTable.", tablename.c_str());
 	    return XORP_ERROR;
 	} else {
-	    return XORP_OK; 	    // table already exists, use that
+	    //
+	    // Table already exists, hence use it.
+	    //
+
+	    //
+	    // Store the XRL target instance, so we know which OriginTable to
+	    // shutdown if the routing protocol dies.
+	    //
+	    if (!target_instance.empty()) {
+		_rib_manager.register_interest_in_target(target_class);
+		_routing_protocol_instances[tablename + " "
+					    + target_class + " "
+					    + target_instance] = ot;
+	    }
+	    return XORP_OK;
 	}
     }
 
@@ -964,9 +1118,7 @@ RIB<A>::add_origin_table(const string& tablename,
     OriginTable<A>* new_table = static_cast<OriginTable<A>* >(find_table(tablename));
     if (_final_table == new_table) {
 	//
-	// This is the first table, so no need to plumb anything
-	// this won't occur if there's an ExportTable<A>, because that
-	// would have been created already.
+	// This is the first table, so no need to plumb anything.
 	//
 	debug_msg("first table\n");
 	return XORP_OK;
@@ -1033,22 +1185,23 @@ RIB<A>::add_origin_table(const string& tablename,
 	if ((egp_table == NULL) && (igp_table == NULL)) {
 	    //
 	    // There are tables, but neither IGP or EGP origin tables
-	    // Therefore the final table must be an ExportTable<A> or
-	    // a RegisterTable (MRIB's have no ExportTable<A>).
+	    // Therefore the final table must be a RedistTable
+	    // or a RegisterTable.
 	    //
-	    if (_final_table->type() != EXPORT_TABLE
-		&& _final_table->type() != REGISTER_TABLE) {
+	    if ((_final_table->type() != REDIST_TABLE)
+		&& (_final_table->type() != POLICY_REDIST_TABLE)
+		&& (_final_table->type() != REGISTER_TABLE)) {
 		XLOG_UNREACHABLE();
 	    }
 
 	    //
 	    // There may be existing single-parent tables before the
-	    // ExportTable such as RegisterTable - track back to be
+	    // final table such as RegisterTable - track back to be
 	    // first of them.
 	    //
 	    RouteTable<A>* rt = track_back(_final_table,
-					   EXPORT_TABLE
-					   | REDIST_TABLE
+					   REDIST_TABLE
+					   | POLICY_REDIST_TABLE
 					   | REGISTER_TABLE);
 
 	    //
@@ -1067,8 +1220,8 @@ RIB<A>::add_origin_table(const string& tablename,
 	// of the ExtIntTable.
 	//
 	RouteTable<A>* next_table = track_back(_final_table,
-					       EXPORT_TABLE
-					       | REDIST_TABLE
+					       REDIST_TABLE
+					       | POLICY_REDIST_TABLE
 					       | REGISTER_TABLE);
 	RouteTable<A>* existing_table = next_table->parent();
 	if (protocol_type == IGP) {
@@ -1085,7 +1238,8 @@ RIB<A>::add_origin_table(const string& tablename,
 	    return XORP_ERROR;
 	}
 
-	if (_final_table->type() & (EXPORT_TABLE | REGISTER_TABLE)) {
+	if (_final_table->type() & (REDIST_TABLE | POLICY_REDIST_TABLE | 
+				    REGISTER_TABLE)) {
 	    ei_table->set_next_table(next_table);
 	    next_table->replumb(existing_table, ei_table);
 	} else {
@@ -1102,7 +1256,8 @@ RIB<A>::add_origin_table(const string& tablename,
 
     // Skip past any RedistTables
     RouteTable<A>* new_prev_table = track_forward(existing_table,
-						  REDIST_TABLE);
+						  (REDIST_TABLE |
+						   POLICY_CONNECTED_TABLE));
     if (new_prev_table != existing_table) {
 	existing_table = new_prev_table;
 	next_table = existing_table->next_table();
@@ -1180,7 +1335,7 @@ RIB<A>::target_death(const string& target_class,
 		     const string& target_instance)
 {
     string s = " " + target_class + " " + target_instance;
-    typename map<const string, OriginTable<A>* >::iterator iter;
+    typename map<string, OriginTable<A>* >::iterator iter;
     for (iter = _routing_protocol_instances.begin();
 	 iter != _routing_protocol_instances.end();
 	 ++iter) {
@@ -1282,5 +1437,19 @@ RIB<A>::name() const
 		    A::ip_version_str().c_str());
 }
 
+
+template <typename A>
+void
+RIB<A>::push_routes()
+{
+    RouteTable<A>* rt = find_table(PolicyConnectedTable<A>::table_name);
+    XLOG_ASSERT(rt != NULL);
+
+    PolicyConnectedTable<A>* pct = dynamic_cast<PolicyConnectedTable<A>*>(rt);
+    XLOG_ASSERT(pct != NULL);
+
+    pct->push_routes();
+}
+
 template class RIB<IPv4>;
 template class RIB<IPv6>;
