@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rib_manager.cc,v 1.38 2004/06/11 23:33:33 hodson Exp $"
+#ident "$XORP: xorp/rib/rib_manager.cc,v 1.48 2005/03/25 02:54:21 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -28,35 +29,7 @@
 #include "rib_manager.hh"
 #include "redist_xrl.hh"
 #include "redist_policy.hh"
-
-template <typename A>
-void
-initialize_rib(RIB<A>& 			rib,
-	       list<RibClient*>& 	rib_clients_list,
-	       RegisterServer& 		register_server)
-{
-    if (rib.initialize_export(&rib_clients_list) != XORP_OK) {
-	XLOG_FATAL("Could not initialize export table for %s",
-		   rib.name().c_str());
-    }
-    if (rib.initialize_register(&register_server) != XORP_OK) {
-	XLOG_FATAL("Could not initialize register table for %s",
-		   rib.name().c_str());
-    }
-    if (rib.initialize_redist_all("all") != XORP_OK) {
-	XLOG_FATAL("Could not initialize all-protocol redistribution "
-		   "table for %s",
-		   rib.name().c_str());
-    }
-    if (rib.add_igp_table("connected", "", "") != XORP_OK) {
-	XLOG_FATAL("Could not add igp table \"connected\" for %s",
-		   rib.name().c_str());
-    }
-    if (rib.add_igp_table("static", "", "") != XORP_OK) {
-	XLOG_FATAL("Could not add igp table \"connected\" for %s",
-		   rib.name().c_str());
-    }
-}
+#include "profile_vars.hh"
 
 RibManager::RibManager(EventLoop& eventloop, XrlStdRouter& xrl_std_router,
 		       const string& fea_target)
@@ -74,22 +47,18 @@ RibManager::RibManager(EventLoop& eventloop, XrlStdRouter& xrl_std_router,
 		      _vif_manager, this),
       _fea_target(fea_target)
 {
-    initialize_rib(_urib4, _urib4_clients_list, _register_server);
-    initialize_rib(_mrib4, _mrib4_clients_list, _register_server);
-    initialize_rib(_urib6, _urib6_clients_list, _register_server);
-    initialize_rib(_mrib6, _mrib6_clients_list, _register_server);
+    _urib4.initialize(_register_server);
+    _mrib4.initialize(_register_server);
+    _urib6.initialize(_register_server);
+    _mrib6.initialize(_register_server);
     PeriodicTimerCallback cb = callback(this, &RibManager::status_updater);
     _status_update_timer = _eventloop.new_periodic(1000, cb);
+    initialize_profiling_variables(_profile);
 }
 
 RibManager::~RibManager()
 {
     stop();
-
-    delete_pointers_list(_urib4_clients_list);
-    delete_pointers_list(_mrib4_clients_list);
-    delete_pointers_list(_urib6_clients_list);
-    delete_pointers_list(_mrib6_clients_list);
 }
 
 int
@@ -123,59 +92,62 @@ bool
 RibManager::status_updater()
 {
     ProcessStatus s = PROC_READY;
-    string reason;
+    string reason = "Ready";
+    bool ret = true;
 
     //
     // Check the VifManager's status
     //
     ServiceStatus vif_mgr_status = _vif_manager.status();
     switch (vif_mgr_status) {
-    case STARTING:
+    case SERVICE_READY:
+	break;
+    case SERVICE_STARTING:
 	s = PROC_NOT_READY;
 	reason = "VifManager starting";
 	break;
-    case RUNNING:
+    case SERVICE_RUNNING:
 	break;
-    case FAILED:
+    case SERVICE_PAUSING:
+	s = PROC_NOT_READY;
+	reason = "VifManager pausing";
+	break;
+    case SERVICE_PAUSED:
+	s = PROC_NOT_READY;
+	reason = "VifManager paused";
+	break;
+    case SERVICE_RESUMING:
+	s = PROC_NOT_READY;
+	reason = "VifManager resuming";
+	break;
+    case SERVICE_SHUTTING_DOWN:
+	s = PROC_SHUTDOWN;
+	reason = "VifManager shutting down";
+	break;
+    case SERVICE_SHUTDOWN:
+	s = PROC_DONE;
+	reason = "VifManager Shutdown";
+       break;
+    case SERVICE_FAILED:
 	// VifManager failed: set process state to failed.
 	// TODO: XXX: Should we exit here, or wait to be restarted?
-	_status_code = PROC_FAILED;
-	_status_reason = "VifManager Failed";
-	return false;
-    default:
-	// TODO: XXX: PAVPAVPAV: more cases may need to be added
+	s = PROC_FAILED;
+	reason = "VifManager Failed";
+	ret = false;
+	break;
+    case SERVICE_ALL:
+	XLOG_UNREACHABLE();
 	break;
     }
 
     //
-    // Check the unicast RIBs can still talk to the FEA
+    // PROC_SHUTDOWN and PROC_DOWN states are persistent unless
+    // there's a failure.
     //
-    RibClient* fea_client;
-    fea_client = find_rib_client(_fea_target, AF_INET, true, false);
-    if (fea_client != NULL) {
-	if (fea_client->failed()) {
-	    _status_code = PROC_FAILED;
-	    _status_reason = "Fatal error talking to FEA (IPv4, unicast)";
-	    return false;
-	}
-    }
-    fea_client = find_rib_client(_fea_target, AF_INET6, true, false);
-    if (fea_client != NULL) {
-	if (fea_client->failed()) {
-	    _status_code = PROC_FAILED;
-	    _status_reason = "Fatal error talking to FEA (IPv6, unicast)";
-	    return false;
-	}
-    }
+    _status_code = s;
+    _status_reason = reason;
 
-    //
-    // SHUTDOWN state is persistent unless there's a failure.
-    //
-    if (_status_code != PROC_SHUTDOWN) {
-	_status_code = s;
-	_status_reason = "Ready";
-    }
-    return true;
+    return ret;
 }
 
 
@@ -355,194 +327,6 @@ RibManager::delete_vif_address(const string& 	vifn,
 }
 
 
-//
-// Select the appropriate list of RIB clients
-//
-list<RibClient* >*
-RibManager::select_rib_clients_list(int family, bool unicast, bool multicast)
-{
-    if (! (unicast ^ multicast))
-	return (NULL);	// Only one of the flags must be set
-
-    //
-    // Select the appropriate list
-    //
-    switch (family) {
-    case AF_INET:
-	if (unicast)
-	    return (&_urib4_clients_list);
-	if (multicast)
-	    return (&_mrib4_clients_list);
-	break;
-    case AF_INET6:
-	if (unicast)
-	    return (&_urib6_clients_list);
-	if (multicast)
-	    return (&_mrib6_clients_list);
-	break;
-    default:
-	XLOG_FATAL("Invalid address family: %d", family);
-	break;
-    }
-
-    //
-    // Nothing found
-    //
-    return (NULL);
-}
-
-//
-// Find a RIB client for a given target name, address family, and
-// unicast/multicast flags.
-//
-RibClient*
-RibManager::find_rib_client(const string& target_name, int family,
-			    bool unicast, bool multicast)
-{
-    list<RibClient* >* rib_clients_list;
-
-    rib_clients_list = select_rib_clients_list(family, unicast, multicast);
-    if (rib_clients_list == NULL)
-	return (NULL);
-
-    //
-    // Search for a RIB client with the same target name
-    //
-    list<RibClient* >::iterator iter;
-    for (iter = rib_clients_list->begin();
-	 iter != rib_clients_list->end();
-	 ++iter) {
-	RibClient* rib_client = *iter;
-	if (rib_client->target_name() == target_name)
-	    return (rib_client);
-    }
-
-    //
-    // Nothing found
-    //
-    return (NULL);
-}
-
-//
-// Add a RIB client for a given target name, address family, and
-// unicast/multicast flags.
-//
-// Return: XORP_OK on success, otherwise XORP_ERROR.
-//
-int
-RibManager::add_rib_client(const string& target_name, int family,
-			   bool unicast, bool multicast)
-{
-    list<RibClient* >* rib_clients_list;
-
-    //
-    // Check if this RIB client has been added already
-    //
-    if (find_rib_client(target_name, family, unicast, multicast) != NULL)
-	return (XORP_OK);
-
-    //
-    // Find the list of RIB clients to add the new client to.
-    //
-    rib_clients_list = select_rib_clients_list(family, unicast, multicast);
-    if (rib_clients_list == NULL)
-	return (XORP_ERROR);
-
-    //
-    // Create a new RibClient, and add it to the list
-    //
-    RibClient* rib_client = new RibClient(_xrl_router, target_name);
-    rib_clients_list->push_back(rib_client);
-
-    return (XORP_OK);
-}
-
-//
-// Delete a RIB client for a given target name, address family, and
-// unicast/multicast flags.
-//
-// Return: XORP_OK on success, otherwise XORP_ERROR.
-//
-int
-RibManager::delete_rib_client(const string& target_name, int family,
-			      bool unicast, bool multicast)
-{
-    RibClient* rib_client;
-    list<RibClient* >* rib_clients_list;
-
-    //
-    // Find the RIB client
-    //
-    rib_client = find_rib_client(target_name, family, unicast, multicast);
-    if (rib_client == NULL)
-	return (XORP_ERROR);
-
-    //
-    // Find the list of RIB clients
-    //
-    rib_clients_list = select_rib_clients_list(family, unicast, multicast);
-    if (rib_clients_list == NULL)
-	return (XORP_ERROR);
-
-    list<RibClient* >::iterator iter;
-    iter = find(rib_clients_list->begin(),
-		rib_clients_list->end(),
-		rib_client);
-    XLOG_ASSERT(iter != rib_clients_list->end());
-    rib_clients_list->erase(iter);
-    delete rib_client;
-
-    return (XORP_OK);
-}
-
-//
-// Enable a RIB client for a given target name, address family, and
-// unicast/multicast flags.
-//
-// Return: XORP_OK on success, otherwise XORP_ERROR.
-//
-int
-RibManager::enable_rib_client(const string& target_name, int family,
-			      bool unicast, bool multicast)
-{
-    RibClient* rib_client;
-
-    //
-    // Find the RIB client
-    //
-    rib_client = find_rib_client(target_name, family, unicast, multicast);
-    if (rib_client == NULL)
-	return (XORP_ERROR);
-
-    rib_client->set_enabled(true);
-
-    return (XORP_OK);
-}
-
-//
-// Disable a RIB client for a given target name, address family, and
-// unicast/multicast flags.
-//
-// Return: XORP_OK on success, otherwise XORP_ERROR.
-//
-int
-RibManager::disable_rib_client(const string& target_name, int family,
-			       bool unicast, bool multicast)
-{
-    RibClient* rib_client;
-
-    //
-    // Find the RIB client
-    //
-    rib_client = find_rib_client(target_name, family, unicast, multicast);
-    if (rib_client == NULL)
-	return (XORP_ERROR);
-
-    rib_client->set_enabled(false);
-
-    return (XORP_OK);
-}
-
 void
 RibManager::make_errors_fatal()
 {
@@ -575,6 +359,29 @@ RibManager::register_interest_in_target_done(const XrlError& e)
     }
 }
 
+void
+RibManager::deregister_interest_in_target(const string& target_class)
+{
+    if (_targets_of_interest.find(target_class)
+	!= _targets_of_interest.end()) {
+	_targets_of_interest.erase(target_class);
+	XrlFinderEventNotifierV0p1Client finder(&_xrl_router);
+	XrlFinderEventNotifierV0p1Client::RegisterClassEventInterestCB cb =
+	    callback(this, &RibManager::deregister_interest_in_target_done);
+	finder.send_deregister_class_event_interest("finder",
+						    _xrl_router.instance_name(),
+						    target_class, cb);
+    }
+}
+
+void
+RibManager::deregister_interest_in_target_done(const XrlError& e)
+{
+    if (e != XrlError::OKAY()) {
+	XLOG_ERROR("Failed to deregister interest in an XRL target\n");
+    }
+}
+
 void
 RibManager::target_death(const string& target_class,
 			 const string& target_instance)
@@ -585,58 +392,14 @@ RibManager::target_death(const string& target_class,
 	exit(0);
     }
 
+    // Deregister interest in the target
+    deregister_interest_in_target(target_class);
+
     // Inform the RIBs in case this was a routing protocol that died.
     _urib4.target_death(target_class, target_instance);
     _urib6.target_death(target_class, target_instance);
     _mrib4.target_death(target_class, target_instance);
     _mrib6.target_death(target_class, target_instance);
-
-    //
-    // Remove any RibClients that died.
-    //
-    list<RibClient* >::iterator iter;
-    for (iter = _urib4_clients_list.begin();
-	 iter != _urib4_clients_list.end(); ++iter) {
-	if (((*iter)->target_name() == target_instance)
-	    || ((*iter)->target_name() == target_class)) {
-	    delete (*iter);
-	    _urib4_clients_list.erase(iter);
-	    break;
-	}
-    }
-
-    for (iter = _mrib4_clients_list.begin();
-	 iter != _mrib4_clients_list.end();
-	 ++iter) {
-	if (((*iter)->target_name() == target_instance)
-	    || ((*iter)->target_name() == target_class)) {
-	    delete (*iter);
-	    _mrib4_clients_list.erase(iter);
-	    break;
-	}
-    }
-
-    for (iter = _urib6_clients_list.begin();
-	 iter != _urib6_clients_list.end();
-	 ++iter) {
-	if (((*iter)->target_name() == target_instance)
-	    || ((*iter)->target_name() == target_class)) {
-	    delete (*iter);
-	    _urib6_clients_list.erase(iter);
-	    break;
-	}
-    }
-
-    for (iter = _mrib6_clients_list.begin();
-	 iter != _mrib6_clients_list.end();
-	 ++iter) {
-	if (((*iter)->target_name() == target_instance)
-	    || ((*iter)->target_name() == target_class)) {
-	    delete (*iter);
-	    _mrib6_clients_list.erase(iter);
-	    break;
-	}
-    }
 }
 
 static inline
@@ -658,6 +421,7 @@ template <typename A>
 static int
 redist_enable_xrl_output(EventLoop&	eventloop,
 			 XrlRouter&	rtr,
+			 Profile&	profile,
 			 RIB<A>&	rib,
 			 const string&	to_xrl_target,
 			 const string&	proto,
@@ -704,11 +468,14 @@ redist_enable_xrl_output(EventLoop&	eventloop,
     if (is_xrl_transaction_output) {
 	redist->set_output(
 		    new RedistTransactionXrlOutput<A>(redist, rtr,
+						      profile,
 						      protocol, to_xrl_target,
 						      cookie)
 		    );
     } else {
-	redist->set_output(new RedistXrlOutput<A>(redist, rtr, protocol,
+	redist->set_output(new RedistXrlOutput<A>(redist, rtr, 
+						  profile,
+						  protocol,
 						  to_xrl_target, cookie));
     }
 
@@ -757,7 +524,8 @@ RibManager::add_redist_xrl_output4(const string&	to_xrl_target,
 				   bool			is_xrl_transaction_output)
 {
     if (unicast) {
-	int e = redist_enable_xrl_output(_eventloop, _xrl_router, _urib4,
+	int e = redist_enable_xrl_output(_eventloop, _xrl_router, _profile,
+					 _urib4,
 					 to_xrl_target, from_protocol, cookie,
 					 is_xrl_transaction_output);
 	if (e != XORP_OK) {
@@ -765,7 +533,8 @@ RibManager::add_redist_xrl_output4(const string&	to_xrl_target,
 	}
     }
     if (multicast) {
-	int e = redist_enable_xrl_output(_eventloop, _xrl_router, _mrib4,
+	int e = redist_enable_xrl_output(_eventloop, _xrl_router, _profile,
+					 _mrib4,
 					 to_xrl_target, from_protocol, cookie,
 					 is_xrl_transaction_output);
 	if (e != XORP_OK && unicast) {
@@ -787,7 +556,8 @@ RibManager::add_redist_xrl_output6(const string&	to_xrl_target,
 				   bool			is_xrl_transaction_output)
 {
     if (unicast) {
-	int e = redist_enable_xrl_output(_eventloop, _xrl_router, _urib6,
+	int e = redist_enable_xrl_output(_eventloop, _xrl_router, _profile,
+					 _urib6,
 					 to_xrl_target, from_protocol, cookie,
 					 is_xrl_transaction_output);
 	if (e != XORP_OK) {
@@ -795,7 +565,8 @@ RibManager::add_redist_xrl_output6(const string&	to_xrl_target,
 	}
     }
     if (multicast) {
-	int e = redist_enable_xrl_output(_eventloop, _xrl_router, _mrib6,
+	int e = redist_enable_xrl_output(_eventloop, _xrl_router, _profile,
+					 _mrib6,
 					 to_xrl_target, from_protocol, cookie,
 					 is_xrl_transaction_output);
 	if (e != XORP_OK && unicast) {
@@ -841,3 +612,38 @@ RibManager::delete_redist_xrl_output6(const string&	to_xrl_target,
 				  is_xrl_transaction_output);
     return XORP_OK;
 }
+
+void
+RibManager::push_routes()
+{
+    _urib4.push_routes();
+    _urib6.push_routes();
+    
+    _mrib4.push_routes();
+    _mrib6.push_routes();
+}
+
+void
+RibManager::configure_filter(const uint32_t& filter, const string& conf)
+{
+    _policy_filters.configure(filter, conf);
+}
+
+void
+RibManager::reset_filter(const uint32_t& filter)
+{
+    _policy_filters.reset(filter);
+}
+
+void
+RibManager::insert_policy_redist_tags(const string& protocol,
+				      const PolicyTags& tags)
+{
+    _policy_redist_map.insert(protocol, tags);
+}				      
+
+void
+RibManager::reset_policy_redist_tags()
+{
+    _policy_redist_map.reset();
+}
