@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +13,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rt_tab_base.cc,v 1.6 2004/06/10 22:41:39 hodson Exp $"
+#ident "$XORP: xorp/rib/rt_tab_base.cc,v 1.10 2005/03/25 02:54:21 pavlin Exp $"
 
 #include "rib_module.h"
 
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+
 #include "rt_tab_base.hh"
 
 template <typename A>
@@ -30,6 +35,18 @@ RouteTable<A>::set_next_table(RouteTable<A>* next_table)
     _next_table = next_table;
 }
 
+template <typename A>
+void
+RouteTable<A>::replace_policytags(const IPRouteEntry<A>& route,
+				  const PolicyTags& prevtags,
+				  RouteTable* caller)
+{
+    XLOG_ASSERT(_next_table);
+    UNUSED(caller);
+    _next_table->replace_policytags(route, prevtags, this);
+}				  
+
+
 template class RouteTable<IPv4>;
 typedef RouteTable<IPv4> IPv4RouteTable;
 
