@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,28 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rt_tab_expect.cc,v 1.8 2004/06/10 22:41:40 hodson Exp $"
+#ident "$XORP: xorp/rib/rt_tab_expect.cc,v 1.12 2005/03/25 02:54:22 pavlin Exp $"
 
 #include "rib_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
+#include "libxorp/debug.h"
 
 #include "rt_tab_expect.hh"
 
 
-template <typename A>
-class ExpectedRouteChange {
-public:
-    ExpectedRouteChange(bool add, const IPRouteEntry<A>& route);
-    bool matches_add(const IPRouteEntry<A>& route) const;
-    bool matches_delete(const IPRouteEntry<A>* route) const;
-    string str() const;
-
-private:
-    bool		_add; 			// true = add, false = delete
-    IPRouteEntry<A>	_route;
-};
-
 template<class A>
 ExpectedRouteChange<A>::ExpectedRouteChange<A>(bool add,
 					       const IPRouteEntry<A>& route)
@@ -113,21 +102,21 @@ ExpectTable<A>::ExpectTable(const string&   tablename,
 template<class A>
 ExpectTable<A>::~ExpectTable()
 {
-    XLOG_ASSERT(_expected.empty());
+    XLOG_ASSERT(_expected_route_changes.empty());
 }
 
 template<class A>
 void
 ExpectTable<A>::expect_add(const IPRouteEntry<A>& route)
 {
-    _expected.push_back(ExpectedRouteChange<A>(true, route));
+    _expected_route_changes.push_back(ExpectedRouteChange<A>(true, route));
 }
 
 template<class A>
 void
 ExpectTable<A>::expect_delete(const IPRouteEntry<A>& route)
 {
-    _expected.push_back(ExpectedRouteChange<A>(false, route));
+    _expected_route_changes.push_back(ExpectedRouteChange<A>(false, route));
 }
 
 template<class A>
@@ -138,16 +127,16 @@ ExpectTable<A>::add_route(const IPRouteEntry<A>& 	route,
     XLOG_ASSERT(caller == _parent);
     debug_msg("DT[%s]: Adding route %s\n", this->tablename().c_str(),
 	      route.str().c_str());
-    if (_expected.empty()) {
+    if (_expected_route_changes.empty()) {
 	XLOG_FATAL("ExpectTable: unexpected add_route received");
     }
-    if (_expected.front().matches_add(route)) {
-	_expected.pop_front();
+    if (_expected_route_changes.front().matches_add(route)) {
+	_expected_route_changes.pop_front();
 	return XORP_OK;
     }
     XLOG_FATAL("ExpectTable: unexpected add_route received. "
 	       "Expected: %s; Received: Add of %s",
-	       _expected.front().str().c_str(),
+	       _expected_route_changes.front().str().c_str(),
 	       route.str().c_str());
     return XORP_ERROR;
 }
@@ -160,16 +149,16 @@ ExpectTable<A>::delete_route(const IPRouteEntry<A>* 	route,
     XLOG_ASSERT(caller == _parent);
     debug_msg("DT[%s]: Deleting route %s\n", this->tablename().c_str(),
 	      route->str().c_str());
-    if (_expected.empty()) {
+    if (_expected_route_changes.empty()) {
 	XLOG_FATAL("ExpectTable: unexpected delete_route received");
     }
-    if (_expected.front().matches_delete(route)) {
-	_expected.pop_front();
+    if (_expected_route_changes.front().matches_delete(route)) {
+	_expected_route_changes.pop_front();
 	return XORP_OK;
     }
     XLOG_FATAL("ExpectTable: unexpected delete_route received. "
 	       "Expected: %s; Received: Delete of %s",
-	       _expected.front().str().c_str(),
+	       _expected_route_changes.front().str().c_str(),
 	       route->str().c_str());
     return XORP_ERROR;
 }
