@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,13 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rt_tab_extint.cc,v 1.19 2004/06/10 22:41:40 hodson Exp $"
+#ident "$XORP: xorp/rib/rt_tab_extint.cc,v 1.25 2005/03/25 02:54:22 pavlin Exp $"
 
 #include "rib_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
+#include "libxorp/debug.h"
 
 #include "rt_tab_extint.hh"
 
@@ -48,15 +50,6 @@ ExtIntTable<A>::add_route(const IPRouteEntry<A>& route, RouteTable<A>* caller)
 	// The new route comes from the IGP table
 	debug_msg("route comes from IGP\n");
 	const IPRouteEntry<A>* found;
-	found = lookup_in_resolved_table(route.net());
-	if (found != NULL) {
-	    if (found->admin_distance() > route.admin_distance()) {
-		// The admin distance of the existing route is worse
-		delete_route(found, _ext_table);
-	    } else {
-		return XORP_ERROR;
-	    }
-	}
 
 	if (route.nexthop()->type() == EXTERNAL_NEXTHOP) {
 	    // An IGP route must have a local nexthop.
@@ -66,6 +59,25 @@ ExtIntTable<A>::add_route(const IPRouteEntry<A>& route, RouteTable<A>* caller)
 	    return XORP_ERROR;
 	}
 
+	found = lookup_route_in_egp_parent(route.net());
+	if (found != NULL) {
+	    if (found->admin_distance() < route.admin_distance()) {
+		// The admin distance of the existing EGP route is better
+		return XORP_ERROR;
+	    }
+	}
+
+	found = lookup_in_resolved_table(route.net());
+	if (found != NULL) {
+	    if (found->admin_distance() < route.admin_distance()) {
+		// The admin distance of the existing route is better
+		return XORP_ERROR;
+	    } else {
+		bool is_delete_propagated = false;
+		this->delete_ext_route(found, is_delete_propagated);
+	    }
+	}
+
 	if (this->next_table() != NULL)
 	    this->next_table()->add_route(route, this);
 
@@ -84,12 +96,12 @@ ExtIntTable<A>::add_route(const IPRouteEntry<A>& route, RouteTable<A>* caller)
 	const IPRouteEntry<A>* found;
 	found = lookup_route_in_igp_parent(route.net());
 	if (found != NULL) {
-	    if (found->admin_distance() > route.admin_distance()) {
-		// The admin distance of the existing route is worse
+	    if (found->admin_distance() < route.admin_distance()) {
+		// The admin distance of the existing IGP route is better
+		return XORP_ERROR;
+	    } else {
 		if (this->next_table() != NULL)
 		    this->next_table()->delete_route(found, this);
-	    } else {
-		return XORP_ERROR;
 	    }
 	}
 	IPNextHop<A>* rt_nexthop;
@@ -176,7 +188,6 @@ ExtIntTable<A>::delete_route(const IPRouteEntry<A>* route,
 
     if (caller == _int_table) {
 	debug_msg("  called from _int_table\n");
-	const ResolvedIPRouteEntry<A>* found;
 	const IPRouteEntry<A>* egp_parent;
 
 	if (route->nexthop()->type() == EXTERNAL_NEXTHOP) {
@@ -184,6 +195,16 @@ ExtIntTable<A>::delete_route(const IPRouteEntry<A>* route,
 	    return XORP_ERROR;
 	}
 
+	const IPRouteEntry<A>* found_egp_route;
+	found_egp_route = lookup_route_in_egp_parent(route->net());
+	if (found_egp_route != NULL) {
+	    if (found_egp_route->admin_distance() < route->admin_distance()) {
+		// The admin distance of the existing EGP route is better
+		return XORP_ERROR;
+	    }
+	}
+
+	const ResolvedIPRouteEntry<A>* found;
 	found = lookup_by_igp_parent(route);
 
 	if (found != NULL) {
@@ -218,32 +239,24 @@ ExtIntTable<A>::delete_route(const IPRouteEntry<A>* route,
 
     } else if (caller == _ext_table) {
 	debug_msg("  called from _ext_table\n");
-	const ResolvedIPRouteEntry<A>* found;
-	found = lookup_in_resolved_table(route->net());
-	if (found != NULL) {
-	    // Erase from table first to prevent lookups on this entry
-	    _ip_route_table.erase(found->net());
-	    _ip_igp_parents.erase(found->backlink());
 
-	    // Delete the route's IGP parent from _resolving_routes if
-	    // no-one is using it anymore
-	    if (lookup_by_igp_parent(found->igp_parent()) == NULL) {
-		_resolving_routes.erase(found->igp_parent()->net());
+	const IPRouteEntry<A>* found_igp_route;
+	found_igp_route = lookup_route_in_igp_parent(route->net());
+	if (found_igp_route != NULL) {
+	    if (found_igp_route->admin_distance() < route->admin_distance()) {
+		// The admin distance of the existing IGP route is better
+		return XORP_ERROR;
 	    }
+	}
 
-	    // Propagate the delete next
-	    if (this->next_table() != NULL)
-		this->next_table()->delete_route(found, this);
-
-	    // Now delete the locally modified copy
-	    delete found;
-	} else {
-	    // Propagate the delete only if the route wasn't found in
-	    // the unresolved nexthops table.
-	    if (delete_unresolved_nexthop(route) == false)
-		if (this->next_table() != NULL)
-		    this->next_table()->delete_route(route, this);
-
+	bool is_delete_propagated = false;
+	this->delete_ext_route(route, is_delete_propagated);
+	if (is_delete_propagated) {
+	    // It is possible the external route had masked an internal one.
+	    const IPRouteEntry<A>* masked_route;
+	    masked_route = _int_table->lookup_route(route->net());
+	    if (masked_route != NULL)
+		add_route(*masked_route, _int_table);
 	}
     } else {
 	XLOG_FATAL("ExtIntTable::delete_route called from a class that "
@@ -252,6 +265,49 @@ ExtIntTable<A>::delete_route(const IPRouteEntry<A>* route,
     return XORP_OK;
 }
 
+template<class A>
+int
+ExtIntTable<A>::delete_ext_route(const IPRouteEntry<A>* route,
+				 bool& is_delete_propagated)
+{
+    const ResolvedIPRouteEntry<A>* found;
+
+    is_delete_propagated = false;
+
+    found = lookup_in_resolved_table(route->net());
+    if (found != NULL) {
+	// Erase from table first to prevent lookups on this entry
+	_ip_route_table.erase(found->net());
+	_ip_igp_parents.erase(found->backlink());
+
+	// Delete the route's IGP parent from _resolving_routes if
+	// no-one is using it anymore
+	if (lookup_by_igp_parent(found->igp_parent()) == NULL) {
+	    _resolving_routes.erase(found->igp_parent()->net());
+	}
+
+	// Propagate the delete next
+	if (this->next_table() != NULL) {
+	    this->next_table()->delete_route(found, this);
+	    is_delete_propagated = true;
+	}
+
+	// Now delete the locally modified copy
+	delete found;
+    } else {
+	// Propagate the delete only if the route wasn't found in
+	// the unresolved nexthops table.
+	if (delete_unresolved_nexthop(route) == false) {
+	    if (this->next_table() != NULL) {
+		this->next_table()->delete_route(route, this);
+		is_delete_propagated = true;
+	    }
+	}
+    }
+
+    return XORP_OK;
+}
+
 template<class A>
 const ResolvedIPRouteEntry<A>*
 ExtIntTable<A>::lookup_in_resolved_table(const IPNet<A>& net)
@@ -271,7 +327,7 @@ template<class A>
 void
 ExtIntTable<A>::resolve_unresolved_nexthops(const IPRouteEntry<A>& nexthop_route)
 {
-    typename map<A, const IPRouteEntry<A>* >::iterator rpair, nextpair;
+    typename multimap<A, const IPRouteEntry<A>* >::iterator rpair, nextpair;
 
     A unresolved_nexthop, new_subnet;
     size_t prefix_len = nexthop_route.net().prefix_len();
@@ -324,7 +380,7 @@ ExtIntTable<A>::delete_unresolved_nexthop(const IPRouteEntry<A>* route)
 {
     debug_msg("delete_unresolved_nexthop %s\n", route->str().c_str());
 
-    typename map<A, const IPRouteEntry<A>* >::iterator rpair;
+    typename multimap<A, const IPRouteEntry<A>* >::iterator rpair;
     IPNextHop<A>* rt_nexthop;
     rt_nexthop = reinterpret_cast<IPNextHop<A>* >(route->nexthop());
     rpair = _ip_unresolved_nexthops.find(rt_nexthop->addr());
@@ -343,8 +399,8 @@ template<class A>
 const ResolvedIPRouteEntry<A>*
 ExtIntTable<A>::lookup_by_igp_parent(const IPRouteEntry<A>* route)
 {
-    debug_msg("lookup_by_igp_parent %x -> %s\n",
-	      (u_int)route, route->net().str().c_str());
+    debug_msg("lookup_by_igp_parent %p -> %s\n",
+	      route, route->net().str().c_str());
 
     typename RouteBackLink::iterator iter;
     iter = _ip_igp_parents.find(route);
@@ -352,8 +408,8 @@ ExtIntTable<A>::lookup_by_igp_parent(const IPRouteEntry<A>* route)
 	debug_msg("Found no routes with this IGP parent\n");
 	return NULL;
     } else {
-	debug_msg("Found route with IGP parent %x:\n    %s\n",
-	       (u_int)(route), (iter->second)->str().c_str());
+	debug_msg("Found route with IGP parent %p:\n    %s\n",
+		  route, (iter->second)->str().c_str());
 	return iter->second;
     }
 }
@@ -363,8 +419,8 @@ const ResolvedIPRouteEntry<A>*
 ExtIntTable<A>::lookup_next_by_igp_parent(const IPRouteEntry<A>* route,
 				  const ResolvedIPRouteEntry<A>* previous)
 {
-    debug_msg("lookup_next_by_igp_parent %x -> %s\n",
-	   (u_int)route, route->net().str().c_str());
+    debug_msg("lookup_next_by_igp_parent %p -> %s\n",
+	      route, route->net().str().c_str());
 
     //
     // TODO: if we have a large number of routes with the same IGP parent,
@@ -390,8 +446,8 @@ ExtIntTable<A>::lookup_next_by_igp_parent(const IPRouteEntry<A>* route,
 	return NULL;
     }
 
-    debug_msg("Found next route with IGP parent %x:\n    %s\n",
-	   (u_int)(route), (iter->second)->str().c_str());
+    debug_msg("Found next route with IGP parent %p:\n    %s\n",
+	      route, (iter->second)->str().c_str());
     return iter->second;
 }
 
@@ -552,16 +608,16 @@ ExtIntTable<A>::lookup_route(const A& addr) const
     }
 
     // Retain only the routes with the lowest admin_distance
-    int lowest_ad = 0xffff;
+    uint32_t lowest_admin_distance = 0xffff;
     for (iter = found.begin(); iter != found.end(); ++iter) {
-	if ((*iter)->admin_distance() < lowest_ad) {
-	    lowest_ad = (*iter)->admin_distance();
+	if ((*iter)->admin_distance() < lowest_admin_distance) {
+	    lowest_admin_distance = (*iter)->admin_distance();
 	}
     }
     for (iter = found.begin(); iter != found.end(); ) {
 	iter2 = iter;
 	++iter2;
-	if ((*iter)->admin_distance() > lowest_ad) {
+	if ((*iter)->admin_distance() > lowest_admin_distance) {
 	    found.erase(iter);
 	}
 	iter = iter2;
@@ -609,6 +665,28 @@ ExtIntTable<A>::lookup_route_in_igp_parent(const A& addr) const
     return found;
 }
 
+template<class A>
+const IPRouteEntry<A>*
+ExtIntTable<A>::lookup_route_in_egp_parent(const IPNet<A>& ipnet) const
+{
+    const IPRouteEntry<A>* found;
+
+    found = _ext_table->lookup_route(ipnet);
+
+    return found;
+}
+
+template<class A>
+const IPRouteEntry<A>*
+ExtIntTable<A>::lookup_route_in_egp_parent(const A& addr) const
+{
+    const IPRouteEntry<A>* found;
+
+    found = _ext_table->lookup_route(addr);
+
+    return found;
+}
+
 template<class A>
 void
 ExtIntTable<A>::replumb(RouteTable<A>* old_parent,
