@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,19 +12,20 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rt_tab_redist.cc,v 1.19 2004/06/10 22:41:41 hodson Exp $"
+#ident "$XORP: xorp/rib/rt_tab_redist.cc,v 1.21 2005/03/25 02:54:23 pavlin Exp $"
 
 #include "rib_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 #include "libxorp/eventloop.hh"
 
 #include "rt_tab_redist.hh"
 #include "rt_tab_origin.hh"
-
 #include "redist_policy.hh"
 
+
 // ----------------------------------------------------------------------------
 // RedistOutput<A>
 
