@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,18 +12,21 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/test_rib_direct.cc,v 1.12 2004/06/10 22:41:42 hodson Exp $"
+#ident "$XORP: xorp/rib/test_rib_direct.cc,v 1.19 2005/03/25 02:54:24 pavlin Exp $"
 
 #include "rib_module.h"
 
 #include "libxorp/xorp.h"
-#include "libxorp/exceptions.hh"
 #include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/exceptions.hh"
 #include "libxorp/eventloop.hh"
 
+#include "libxipc/finder_server.hh"
+
 #include "parser.hh"
 #include "parser_direct_cmds.hh"
-#include "dummy_rib_manager.hh"
+#include "rib_manager.hh"
 #include "rib.hh"
 #include "dummy_register_server.hh"
 
@@ -41,9 +44,13 @@ private:
 RibParser::RibParser(RIB<IPv4>& rib)
     : _rib(rib)
 {
+    add_command(new DirectDiscardVifCommand(_rib));
     add_command(new DirectEtherVifCommand(_rib));
+    add_command(new DirectLoopbackVifCommand(_rib));
+
     add_command(new DirectRouteAddCommand(_rib));
     add_command(new DirectRouteDeleteCommand(_rib));
+    add_command(new DirectRouteVifAddCommand(_rib));
 
     add_command(new DirectRouteVerifyCommand(_rib));
     add_command(new DirectTableOriginCommand(_rib));
@@ -59,11 +66,26 @@ static void
 parser_main()
 {
     EventLoop eventloop;
-    RibManager rib_manager;
-    RIB<IPv4> rib(UNICAST, rib_manager, eventloop);
-    DummyRegisterServer regserv;
 
-    rib.initialize_register(&regserv);
+    // Finder Server
+    FinderServer fs(eventloop, FinderConstants::FINDER_DEFAULT_HOST(),
+		    FinderConstants::FINDER_DEFAULT_PORT());
+
+    // Rib Server component
+    XrlStdRouter xrl_std_router_rib(eventloop, "rib", fs.addr(), fs.port());
+
+    //
+    // The RIB manager
+    //
+    RibManager rib_manager(eventloop, xrl_std_router_rib, "fea");
+    rib_manager.enable();
+
+    RIB<IPv4> rib(UNICAST, rib_manager, eventloop);
+    DummyRegisterServer register_server;
+
+    rib.initialize_register(register_server);
+
+    wait_until_xrl_router_is_ready(eventloop, xrl_std_router_rib);
 
     RibParser parser(rib);
 
