@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/test_rib_xrls.cc,v 1.33 2004/06/10 22:41:42 hodson Exp $"
+#ident "$XORP: xorp/rib/test_rib_xrls.cc,v 1.40 2005/03/25 02:54:24 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -24,11 +24,10 @@
 #include "libxipc/finder_server.hh"
 #include "libxipc/xrl_std_router.hh"
 
-#include "dummy_rib_manager.hh"
+#include "rib_manager.hh"
 #include "parser.hh"
 #include "parser_direct_cmds.hh"
 #include "parser_xrl_cmds.hh"
-#include "rib_client.hh"
 #include "dummy_register_server.hh"
 #include "xrl_target.hh"
 
@@ -49,12 +48,15 @@ public:
 	add_command(new XrlDeleteEGPTableCommand(e, xrl_client, cv));
 
 	// The following do not exist in XRL interface so use direct methods
+	add_command(new DirectRouteVifAddCommand(rib));
 	add_command(new DirectRouteVerifyCommand(rib));
 	add_command(new DirectTableOriginCommand(rib));
 
 	// XXX The following should probably use XRL's but punting for
 	// time being.
+	add_command(new DirectDiscardVifCommand(rib));
 	add_command(new DirectEtherVifCommand(rib));
+	add_command(new DirectLoopbackVifCommand(rib));
     }
 };
 
@@ -64,37 +66,27 @@ parser_main()
     EventLoop eventloop;
 
     // Finder Server
-    FinderServer fs(eventloop);
+    FinderServer fs(eventloop, FinderConstants::FINDER_DEFAULT_HOST(),
+		    FinderConstants::FINDER_DEFAULT_PORT());
 
     // Rib Server component
-    XrlStdRouter xrl_router(eventloop, "rib", fs.addr(), fs.port());
-    RibClient rib_client(xrl_router, "fea");
+    XrlStdRouter xrl_std_router_rib(eventloop, "rib", fs.addr(), fs.port());
 
-    RibManager rib_manager;
+    //
+    // The RIB manager
+    //
+    RibManager rib_manager(eventloop, xrl_std_router_rib, "fea");
+    rib_manager.enable();
 
-    // RIB Instantiations for XrlRibTarget
-    RIB<IPv4> urib4(UNICAST, rib_manager, eventloop);
-    DummyRegisterServer regserv;
-    urib4.initialize_register(&regserv);
+    wait_until_xrl_router_is_ready(eventloop, xrl_std_router_rib);
 
-    // Instantiated but not used
-    RIB<IPv4> mrib4(MULTICAST, rib_manager, eventloop);
-    RIB<IPv6> urib6(UNICAST, rib_manager, eventloop);
-    RIB<IPv6> mrib6(MULTICAST, rib_manager, eventloop);
+    rib_manager.start();
 
-    VifManager vif_manager(xrl_router, eventloop, NULL, "fea");
-    vif_manager.enable();
-    vif_manager.start();
-    XrlRibTarget xrt(&xrl_router, urib4, mrib4, urib6, mrib6, vif_manager,
-		     NULL);
-
-    wait_until_xrl_router_is_ready(eventloop, xrl_router);
-
-    XrlRibV0p1Client xrl_client(&xrl_router);
+    XrlRibV0p1Client xrl_client(&xrl_std_router_rib);
 
     // Variable used to signal completion of Xrl parse completion
     XrlCompletion cv;
-    XrlRibParser parser(eventloop, xrl_client, urib4, cv);
+    XrlRibParser parser(eventloop, xrl_client, rib_manager.urib4(), cv);
 
     string cmd;
     int line = 0;
