@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/tools/show_routes.cc,v 1.4 2004/06/12 14:50:20 hodson Exp $"
+#ident "$XORP: xorp/rib/tools/show_routes.cc,v 1.11 2005/03/25 02:54:26 pavlin Exp $"
 
 #include "rib/rib_module.h"
 
@@ -143,11 +143,11 @@ public:
     XrlCmdError redist4_0_1_starting_route_dump(const string&	cookie);
     XrlCmdError redist4_0_1_finishing_route_dump(const string&	cookie);
     XrlCmdError redist4_0_1_add_route(const IPv4Net&	dst,
-				      const IPv4&	nh,
+				      const IPv4&	nexthop,
 				      const string&	ifname,
 				      const string&	vifname,
 				      const uint32_t&	metric,
-				      const uint32_t&	ad,
+				      const uint32_t&	admin_distance,
 				      const string&	cookie);
     XrlCmdError redist4_0_1_delete_route(const IPv4Net&	network,
 					 const string&	cookie);
@@ -155,11 +155,11 @@ public:
     XrlCmdError redist6_0_1_starting_route_dump(const string&	cookie);
     XrlCmdError redist6_0_1_finishing_route_dump(const string&	cookie);
     XrlCmdError redist6_0_1_add_route(const IPv6Net&	dst,
-				      const IPv6&	nh,
+				      const IPv6&	nexthop,
 				      const string&	ifname,
 				      const string&	vifname,
 				      const uint32_t&	metric,
-				      const uint32_t&	ad,
+				      const uint32_t&	admin_distance,
 				      const string&	cookie);
     XrlCmdError redist6_0_1_delete_route(const IPv6Net&	network,
 					 const string&	cookie);
@@ -202,25 +202,30 @@ protected:
 
 ShowRoutesProcessor::ShowRoutesProcessor(EventLoop&		e,
 					 ShowRoutesOptions&	o)
-    : _e(e), _opts(o)
+    : _e(e),
+      _opts(o),
+      _rtr(NULL)
 {
-    _rtr = 0;
 }
 
 ShowRoutesProcessor::~ShowRoutesProcessor()
 {
-    delete _rtr;
+    if (_rtr != NULL) {
+	delete _rtr;
+	_rtr = NULL;
+    }
 }
 
 bool
 ShowRoutesProcessor::startup()
 {
-    if (status() != READY) {
+    if (status() != SERVICE_READY) {
 	return false;
     }
 
     // Create XrlRouter
-    _rtr = new XrlStdRouter(_e, "show_routes",
+    string process = c_format("show_routes<%d>", getpid());
+    _rtr = new XrlStdRouter(_e, process.c_str(),
 			    _opts.finder_host.c_str(), _opts.finder_port);
 
     // Glue the router to the Xrl methods class exports
@@ -234,18 +239,23 @@ ShowRoutesProcessor::startup()
     _t = _e.new_periodic(250,
 			 callback(this,
 				  &ShowRoutesProcessor::poll_ready_failed));
-    set_status(STARTING);
+    set_status(SERVICE_STARTING);
     return true;
 }
 
 bool
 ShowRoutesProcessor::shutdown()
 {
+    // Withdraw the Xrl methods
+    this->set_command_map(NULL);
+
     ServiceStatus st = this->status();
-    if (st == FAILED || st == SHUTTING_DOWN || st == SHUTDOWN)
+    if (st == SERVICE_FAILED
+	|| st == SERVICE_SHUTTING_DOWN
+	|| st == SERVICE_SHUTDOWN)
 	return false;
 
-    set_status(SHUTTING_DOWN);
+    set_status(SERVICE_SHUTTING_DOWN);
     step_1000_request_cease();
     return true;
 }
@@ -256,11 +266,11 @@ ShowRoutesProcessor::poll_ready_failed()
     if (_rtr == 0) {
 	return false;
     } else if (_rtr->ready()) {
-	set_status(RUNNING);
+	set_status(SERVICE_RUNNING);
 	step_100_watch_rib();
 	return false;
     } else if (_rtr->failed()) {
-	set_status(FAILED, "Failed: No Finder?");
+	set_status(SERVICE_FAILED, "Failed: No Finder?");
 	return false;
     }
     return true;
@@ -274,8 +284,9 @@ ShowRoutesProcessor::step_100_watch_rib()
     if (fen.send_register_class_event_interest(
 		"finder", _rtr->instance_name(), _opts.xrl_target,
 		callback(this, &ShowRoutesProcessor::watch_rib_cb)) == false) {
-	set_status(FAILED, c_format("Failed to register interest in %s",
-				   _opts.xrl_target.c_str()));
+	set_status(SERVICE_FAILED,
+		   c_format("Failed to register interest in %s",
+			    _opts.xrl_target.c_str()));
 	return;
     }
 }
@@ -287,8 +298,9 @@ ShowRoutesProcessor::watch_rib_cb(const XrlError& xe)
 	step_200_request_redist();
 	return;
     }
-    set_status(FAILED, c_format("Failed to register interest in %s",
-				_opts.xrl_target.c_str()));
+    set_status(SERVICE_FAILED,
+	       c_format("Failed to register interest in %s",
+			_opts.xrl_target.c_str()));
     return;
 }
 
@@ -327,7 +339,7 @@ ShowRoutesProcessor::step_200_request_redist()
     }
 
     if (sent == false) {
-	set_status(FAILED, "Failed to request redist.");
+	set_status(SERVICE_FAILED, "Failed to request redist.");
     }
 }
 
@@ -337,7 +349,7 @@ ShowRoutesProcessor::request_redist_cb(const XrlError& xe)
     if (xe == XrlError::OKAY()) {
 	return;
     }
-    set_status(FAILED,
+    set_status(SERVICE_FAILED,
 	       c_format("Request for routes to be redistributed from %s "
 			"failed.\nThe protocol is probably not active.",
 			_opts.protocol));
@@ -369,16 +381,16 @@ ShowRoutesProcessor::step_1000_request_cease()
     }
 
     if (sent == false) {
-	set_status(FAILED, "Failed to request redistribution end.");
+	set_status(SERVICE_FAILED, "Failed to request redistribution end.");
 	return;
     }
-    set_status(SHUTTING_DOWN);
+    set_status(SERVICE_SHUTTING_DOWN);
 }
 
 void
 ShowRoutesProcessor::request_cease_cb(const XrlError& /* xe */)
 {
-    set_status(SHUTDOWN);
+    set_status(SERVICE_SHUTDOWN);
     return;
 }
 
@@ -413,16 +425,16 @@ ShowRoutesProcessor::common_0_1_get_status(uint32_t&	status,
 					   string&	/* reason */)
 {
     switch (this->status()) {
-    case READY:		status = PROC_NULL;		break;
-    case STARTING:	status = PROC_STARTUP;		break;
-    case RUNNING:	status = PROC_READY;		break;
-    case PAUSED:					/* FALLTHRU */
-    case PAUSING:					/* FALLTHRU */
-    case RESUMING:	status = PROC_NOT_READY;	break;
-    case SHUTTING_DOWN:	status = PROC_SHUTDOWN;		break;
-    case SHUTDOWN:	status = PROC_DONE;		break;
-    case FAILED:	status = PROC_FAILED;		break;
-    case ALL:						break;
+    case SERVICE_READY:		status = PROC_NULL;		break;
+    case SERVICE_STARTING:	status = PROC_STARTUP;		break;
+    case SERVICE_RUNNING:	status = PROC_READY;		break;
+    case SERVICE_PAUSED:					/* FALLTHRU */
+    case SERVICE_PAUSING:					/* FALLTHRU */
+    case SERVICE_RESUMING:	status = PROC_NOT_READY;	break;
+    case SERVICE_SHUTTING_DOWN:	status = PROC_SHUTDOWN;		break;
+    case SERVICE_SHUTDOWN:	status = PROC_DONE;		break;
+    case SERVICE_FAILED:	status = PROC_FAILED;		break;
+    case SERVICE_ALL:						break;
     }
     return XrlCmdError::OKAY();
 }
@@ -471,18 +483,18 @@ ShowRoutesProcessor::redist4_0_1_finishing_route_dump(const string& cookie)
 
 XrlCmdError
 ShowRoutesProcessor::redist4_0_1_add_route(const IPv4Net&	dst,
-					   const IPv4&		nh,
+					   const IPv4&		nexthop,
 					   const string&	ifname,
 					   const string&	vifname,
 					   const uint32_t&	metric,
-					   const uint32_t&	ad,
+					   const uint32_t&	admin_distance,
 					   const string&	cookie)
 {
-    if (this->status() != RUNNING || check_cookie(cookie) == false) {
+    if (this->status() != SERVICE_RUNNING || check_cookie(cookie) == false) {
 	return XrlCmdError::OKAY();
     }
 
-    display_route(dst, nh, ifname, vifname, metric, ad);
+    display_route(dst, nexthop, ifname, vifname, metric, admin_distance);
     return XrlCmdError::OKAY();
 }
 
@@ -511,17 +523,17 @@ ShowRoutesProcessor::redist6_0_1_finishing_route_dump(const string& cookie)
 
 XrlCmdError
 ShowRoutesProcessor::redist6_0_1_add_route(const IPv6Net&	dst,
-					   const IPv6&		nh,
+					   const IPv6&		nexthop,
 					   const string&	ifname,
 					   const string&	vifname,
 					   const uint32_t&	metric,
-					   const uint32_t&	ad,
+					   const uint32_t&	admin_distance,
 					   const string&	cookie)
 {
-    if (this->status() != RUNNING || check_cookie(cookie) == false) {
+    if (this->status() != SERVICE_RUNNING || check_cookie(cookie) == false) {
 	return XrlCmdError::OKAY();
     }
-    display_route(dst, nh, ifname, vifname, metric, ad);
+    display_route(dst, nexthop, ifname, vifname, metric, admin_distance);
     return XrlCmdError::OKAY();
 }
 
@@ -565,7 +577,8 @@ parse_finder_args(const string& host_colon_port, string& host, uint16_t& port)
         string s_port = string(host_colon_port, sp + 1, 14);
         uint32_t t_port = atoi(s_port.c_str());
         if (t_port == 0 || t_port > 65535) {
-            XLOG_ERROR("Finder port %d is not in range 1--65535.\n", t_port);
+            XLOG_ERROR("Finder port %u is not in range 1--65535.\n",
+		       XORP_UINT_CAST(t_port));
             return false;
         }
         port = (uint16_t)t_port;
@@ -624,8 +637,8 @@ main(int argc, char* const argv[])
 
 	ShowRoutesOptions sr_opts;
 
-	sr_opts.finder_host = FINDER_DEFAULT_HOST.str();
-        sr_opts.finder_port = FINDER_DEFAULT_PORT;
+	sr_opts.finder_host = FinderConstants::FINDER_DEFAULT_HOST().str();
+        sr_opts.finder_port = FinderConstants::FINDER_DEFAULT_PORT();
 	sr_opts.xrl_target  = "rib";
 
 	int ch;
@@ -675,11 +688,13 @@ main(int argc, char* const argv[])
 	ShowRoutesProcessor srp(e, sr_opts);
 
 	srp.startup();
-	while (srp.status() != FAILED && srp.status() != SHUTDOWN) {
+	while ((srp.status() != SERVICE_FAILED)
+	       && (srp.status() != SERVICE_SHUTDOWN)) {
 	    e.run();
 	}
 
-	if (srp.status() == FAILED) {
+	if (srp.status() == SERVICE_FAILED) {
+	    srp.shutdown();
 	    if (srp.status_note().empty() == false) {
 		cout << srp.status_note() << endl;
 	    } else {
