@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,21 +13,28 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/xrl_target.cc,v 1.37 2004/06/10 22:41:43 hodson Exp $"
+#ident "$XORP: xorp/rib/xrl_target.cc,v 1.52 2005/03/30 06:45:42 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
+#define PROFILE_UTILS_REQUIRED
 
 #include "rib_module.h"
 
 #include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
 #include "libxorp/status_codes.h"
 
+#include "libxipc/xrl_std_router.hh"
+
+#include "xrl/interfaces/profile_client_xif.hh"
+
 #include "xrl_target.hh"
 #include "rt_tab_register.hh"
 #include "rib_manager.hh"
 #include "vifmanager.hh"
-
+#include "profile_vars.hh"
 
 XrlCmdError
 XrlRibTarget::common_0_1_get_target_name(string& name)
@@ -91,146 +99,6 @@ XrlRibTarget::rib_0_1_stop_rib()
     return XrlCmdError::OKAY();
 }
 
-XrlCmdError
-XrlRibTarget::rib_0_1_add_rib_client4(
-    // Input values,
-    const string&	target_name,
-    const bool&		unicast,
-    const bool&		multicast)
-{
-    if (_rib_manager->add_rib_client(target_name, AF_INET, unicast,
-				     multicast) != XORP_OK) {
-	return XrlCmdError::COMMAND_FAILED("Failed to add rib client");
-    }
-    return XrlCmdError::OKAY();
-}
-
-XrlCmdError
-XrlRibTarget::rib_0_1_add_rib_client6(
-    // Input values,
-    const string&	target_name,
-    const bool&		unicast,
-    const bool&		multicast)
-{
-#ifndef HAVE_IPV6
-    UNUSED(target_name);
-    UNUSED(unicast);
-    UNUSED(multicast);
-    return XrlCmdError::COMMAND_FAILED("IPv6 not supported.");
-#else
-    if (_rib_manager->add_rib_client(target_name, AF_INET6, unicast,
-				     multicast) != XORP_OK) {
-	return XrlCmdError::COMMAND_FAILED("Failed to add rib client");
-    }
-    return XrlCmdError::OKAY();
-#endif // HAVE_IPV6
-}
-
-XrlCmdError
-XrlRibTarget::rib_0_1_delete_rib_client4(
-    // Input values,
-    const string&	target_name,
-    const bool&		unicast,
-    const bool&		multicast)
-{
-    if (_rib_manager->delete_rib_client(target_name, AF_INET, unicast,
-					multicast) != XORP_OK) {
-	return XrlCmdError::COMMAND_FAILED("Failed to delete rib client");
-    }
-    return XrlCmdError::OKAY();
-}
-
-XrlCmdError
-XrlRibTarget::rib_0_1_delete_rib_client6(
-    // Input values,
-    const string&	target_name,
-    const bool&		unicast,
-    const bool&		multicast)
-{
-#ifndef HAVE_IPV6
-    UNUSED(target_name);
-    UNUSED(unicast);
-    UNUSED(multicast);
-    return XrlCmdError::COMMAND_FAILED("IPv6 not supported");
-#else
-    if (_rib_manager->delete_rib_client(target_name, AF_INET6, unicast,
-					multicast) != XORP_OK) {
-	return XrlCmdError::COMMAND_FAILED("Failed to delete rib client");
-    }
-    return XrlCmdError::OKAY();
-#endif // HAVE_IPV6
-}
-
-XrlCmdError
-XrlRibTarget::rib_0_1_enable_rib_client4(
-    // Input values,
-    const string&	target_name,
-    const bool&		unicast,
-    const bool&		multicast)
-{
-    if (_rib_manager->enable_rib_client(target_name, AF_INET, unicast,
-					multicast) != XORP_OK) {
-	return XrlCmdError::COMMAND_FAILED("Failed to enable rib client");
-    }
-    return XrlCmdError::OKAY();
-}
-
-XrlCmdError
-XrlRibTarget::rib_0_1_enable_rib_client6(
-    // Input values,
-    const string&	target_name,
-    const bool&		unicast,
-    const bool&		multicast)
-{
-#ifndef HAVE_IPV6
-    UNUSED(target_name);
-    UNUSED(unicast);
-    UNUSED(multicast);
-    return XrlCmdError::COMMAND_FAILED("IPv6 not supported");
-#else
-    if (_rib_manager->enable_rib_client(target_name, AF_INET6, unicast,
-					multicast) != XORP_OK) {
-	return XrlCmdError::COMMAND_FAILED("Failed to enable rib client");
-    }
-    return XrlCmdError::OKAY();
-#endif // HAVE_IPV6
-}
-
-XrlCmdError
-XrlRibTarget::rib_0_1_disable_rib_client4(
-    // Input values,
-    const string&	target_name,
-    const bool&		unicast,
-    const bool&		multicast)
-{
-    if (_rib_manager->disable_rib_client(target_name, AF_INET, unicast,
-					 multicast) != XORP_OK) {
-	return XrlCmdError::COMMAND_FAILED("Failed to disable rib client");
-    }
-    return XrlCmdError::OKAY();
-}
-
-XrlCmdError
-XrlRibTarget::rib_0_1_disable_rib_client6(
-    // Input values,
-    const string&	target_name,
-    const bool&		unicast,
-    const bool&		multicast)
-{
-#ifndef HAVE_IPV6
-    UNUSED(target_name);
-    UNUSED(unicast);
-    UNUSED(multicast);
-    return XrlCmdError::COMMAND_FAILED("IPv6 not supported");
-#else
-    if (_rib_manager->disable_rib_client(target_name, AF_INET6, unicast,
-					 multicast) != XORP_OK) {
-	return XrlCmdError::COMMAND_FAILED("Failed to disable rib client");
-    }
-    return XrlCmdError::OKAY();
-#endif // HAVE_IPV6
-}
-
 XrlCmdError
 XrlRibTarget::rib_0_1_make_errors_fatal()
 {
@@ -452,21 +320,44 @@ XrlRibTarget::rib_0_1_add_route4(const string&	protocol,
 				 const bool&	multicast,
 				 const IPv4Net&	network,
 				 const IPv4&	nexthop,
-				 const uint32_t& metric)
+				 const uint32_t& metric,
+				 const XrlAtomList& policytags)
 {
-    debug_msg("#### XRL: ADD ROUTE net %s, nexthop: %s\n",
-	      network.str().c_str(), nexthop.str().c_str());
+    debug_msg("add_route4 protocol: %s unicast: %s multicast: %s "
+	      "network %s nexthop %s metric %u\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
+	      network.str().c_str(),
+	      nexthop.str().c_str(),
+	      XORP_UINT_CAST(metric));
+ 
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("add %s %s%s %s %s %u",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str(),
+					     nexthop.str().c_str(),
+					     XORP_UINT_CAST(metric)));
+    }
+    
     if (unicast &&
-	_urib4.add_route(protocol, network, nexthop, "", "", metric)
+	_urib4.add_route(protocol, network, nexthop, "", "", metric, policytags)
 	!= XORP_OK) {
-	string err = "Could not add IPv4 route to unicast RIB";
+	string err = c_format("Could not add IPv4 route "
+			      "net %s, nexthop: %s to unicast RIB",
+			      network.str().c_str(), nexthop.str().c_str());
 	return XrlCmdError::COMMAND_FAILED(err);
     }
 
     if (multicast &&
-	_mrib4.add_route(protocol, network, nexthop, "", "", metric)
+	_mrib4.add_route(protocol, network, nexthop, "", "", metric, policytags)
 	!= XORP_OK) {
-	string err = "Could not add IPv4 route to multicast RIB";
+	string err = c_format("Could not add IPv4 route "
+			      "net %s, nexthop: %s to multicast RIB",
+			      network.str().c_str(), nexthop.str().c_str());
 	return XrlCmdError::COMMAND_FAILED(err);
     }
 
@@ -479,19 +370,46 @@ XrlRibTarget::rib_0_1_add_route6(const string&	protocol,
 				 const bool&	multicast,
 				 const IPv6Net&	network,
 				 const IPv6&	nexthop,
-				 const uint32_t& metric)
+				 const uint32_t& metric,
+				 const XrlAtomList& policytags)
 {
+    debug_msg("add_route6 protocol: %s unicast: %s multicast: %s "
+	      "network %s nexthop %s metric %u\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
+	      network.str().c_str(),
+	      nexthop.str().c_str(),
+	      XORP_UINT_CAST(metric));
+
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("add %s %s%s %s %s %u",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str(),
+					     nexthop.str().c_str(),
+					     XORP_UINT_CAST(metric)));
+    }
+
     if (unicast &&
-	_urib6.add_route(protocol, network, nexthop, "", "", metric)
+	_urib6.add_route(protocol, network, nexthop, "", "", metric,
+			 policytags)
 	!= XORP_OK) {
-	string err = "Could not add IPv6 route to unicast RIB";
+	string err = c_format("Could not add IPv6 route "
+			      "net %s, nexthop: %s to unicast RIB",
+			      network.str().c_str(), nexthop.str().c_str());
 	return XrlCmdError::COMMAND_FAILED(err);
     }
 
     if (multicast &&
-	_mrib6.add_route(protocol, network, nexthop, "", "", metric)
+	_mrib6.add_route(protocol, network, nexthop, "", "", metric,
+			 policytags)
 	!= XORP_OK) {
-	string err = "Could not add IPv6 route to multicast RIB";
+	string err = c_format("Could not add IPv6 route "
+			      "net %s, nexthop: %s to multicast RIB",
+			      network.str().c_str(), nexthop.str().c_str());
 	return XrlCmdError::COMMAND_FAILED(err);
     }
 
@@ -504,17 +422,40 @@ XrlRibTarget::rib_0_1_replace_route4(const string&	protocol,
 				     const bool&	multicast,
 				     const IPv4Net&	network,
 				     const IPv4&	nexthop,
-				     const uint32_t&	metric)
+				     const uint32_t&	metric,
+				     const XrlAtomList& policytags)
 {
+    debug_msg("replace_route4 protocol: %s unicast: %s multicast: %s "
+	      "network %s nexthop %s metric %u\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
+	      network.str().c_str(),
+	      nexthop.str().c_str(),
+	      XORP_UINT_CAST(metric));
+
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("replace %s %s%s %s %s %u",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str(),
+					     nexthop.str().c_str(),
+					     XORP_UINT_CAST(metric)));
+    }
+
     if (unicast &&
-	_urib4.replace_route(protocol, network, nexthop, "", "", metric)
+	_urib4.replace_route(protocol, network, nexthop, "", "",
+					metric, policytags)
 	!= XORP_OK) {
 	string err = "Could not replace IPv4 route in unicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
     }
 
     if (multicast &&
-	_mrib4.replace_route(protocol, network, nexthop, "", "", metric)
+	_mrib4.replace_route(protocol, network, nexthop, "", "",
+					metric, policytags)
 	!= XORP_OK) {
 	string err = "Could not replace IPv4 route in multicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -529,17 +470,40 @@ XrlRibTarget::rib_0_1_replace_route6(const string&	protocol,
 				     const bool&	multicast,
 				     const IPv6Net&	network,
 				     const IPv6&	nexthop,
-				     const uint32_t&	metric)
+				     const uint32_t&	metric,
+				     const XrlAtomList& policytags)
 {
+    debug_msg("replace_route6 protocol: %s unicast: %s multicast: %s "
+	      "network %s nexthop %s metric %u\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
+	      network.str().c_str(),
+	      nexthop.str().c_str(),
+	      XORP_UINT_CAST(metric));
+
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("replace %s %s%s %s %s %u",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str(),
+					     nexthop.str().c_str(),
+					     XORP_UINT_CAST(metric)));
+    }
+
     if (unicast &&
-	_urib6.replace_route(protocol, network, nexthop, "", "", metric)
+	_urib6.replace_route(protocol, network, nexthop, "", "", metric,
+			     policytags)
 	!= XORP_OK) {
 	string err = "Could not replace IPv6 route in unicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
     }
 
     if (multicast &&
-	_mrib6.replace_route(protocol, network, nexthop, "", "", metric)
+	_mrib6.replace_route(protocol, network, nexthop, "", "", metric,
+			     policytags)
 	!= XORP_OK) {
 	string err = "Could not add IPv6 route in multicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -554,8 +518,22 @@ XrlRibTarget::rib_0_1_delete_route4(const string&	protocol,
 				    const bool&		multicast,
 				    const IPv4Net&	network)
 {
-    debug_msg("#### XRL: DELETE ROUTE net %s\n",
+    debug_msg("delete_route4 protocol: %s unicast: %s multicast: %s "
+	      "network %s\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
 	      network.str().c_str());
+
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("delete %s %s%s %s",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str()));
+    }
+
     if (unicast && _urib4.delete_route(protocol, network) != XORP_OK) {
 	string err = "Could not delete IPv4 route from unicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -575,6 +553,22 @@ XrlRibTarget::rib_0_1_delete_route6(const string&	protocol,
 				    const bool&		multicast,
 				    const IPv6Net&	network)
 {
+    debug_msg("delete_route6 protocol: %s unicast: %s multicast: %s "
+	      "network %s\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
+	      network.str().c_str());
+
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("delete %s %s%s %s",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str()));
+    }
+
     if (unicast && _urib6.delete_route(protocol, network) != XORP_OK) {
 	string err = "Could not delete IPv6 route from unicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -596,21 +590,44 @@ XrlRibTarget::rib_0_1_add_interface_route4(const string&	protocol,
 					   const IPv4&		nexthop,
 					   const string&	ifname,
 					   const string&	vifname,
-					   const uint32_t&	metric)
+					   const uint32_t&	metric,
+					   const XrlAtomList&	policytags)
 {
-    debug_msg("#### XRL: ADD INTERFACE ROUTE net %s, nexthop: %s "
-	      "ifname: %s vifname: %s\n",
-	      network.str().c_str(), nexthop.str().c_str(),
-	      ifname.c_str(), vifname.c_str());
+    debug_msg("add_interface_route4 protocol: %s unicast: %s multicast: %s "
+	      "network %s nexthop %s ifname %s vifname %s metric %u\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
+	      network.str().c_str(),
+	      nexthop.str().c_str(),
+	      ifname.c_str(),
+	      vifname.c_str(),
+	      XORP_UINT_CAST(metric));
+
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("add %s %s%s %s %s %s/%s %u",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str(),
+					     nexthop.str().c_str(),
+					     ifname.c_str(),
+					     vifname.c_str(),
+					     XORP_UINT_CAST(metric)));
+    }
+
     if (unicast &&
-	_urib4.add_route(protocol, network, nexthop, ifname, vifname, metric)
+	_urib4.add_route(protocol, network, nexthop, ifname, vifname, metric,
+			 policytags)
 	!= XORP_OK) {
 	string err = "Could not add IPv4 interface route to unicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
     }
 
     if (multicast &&
-	_mrib4.add_route(protocol, network, nexthop, ifname, vifname, metric)
+	_mrib4.add_route(protocol, network, nexthop, ifname, vifname, metric,
+			 policytags)
 	!= XORP_OK) {
 	string err = "Could not add IPv4 interface route to multicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -627,21 +644,44 @@ XrlRibTarget::rib_0_1_add_interface_route6(const string&	protocol,
 					   const IPv6&		nexthop,
 					   const string&	ifname,
 					   const string&	vifname,
-					   const uint32_t&	metric)
+					   const uint32_t&	metric,
+					   const XrlAtomList&	policytags)
 {
-    debug_msg("#### XRL: ADD INTERFACE ROUTE net %s, nexthop: %s "
-	      "ifname: %s vifname: %s\n",
-	      network.str().c_str(), nexthop.str().c_str(),
-	      ifname.c_str(), vifname.c_str());
+    debug_msg("add_interface_route6 protocol: %s unicast: %s multicast: %s "
+	      "network %s nexthop %s ifname %s vifname %s metric %u\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
+	      network.str().c_str(),
+	      nexthop.str().c_str(),
+	      ifname.c_str(),
+	      vifname.c_str(),
+	      XORP_UINT_CAST(metric));
+
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("add %s %s%s %s %s %s/%s %u",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str(),
+					     nexthop.str().c_str(),
+					     ifname.c_str(),
+					     vifname.c_str(),
+					     XORP_UINT_CAST(metric)));
+    }
+
     if (unicast &&
-	_urib6.add_route(protocol, network, nexthop, ifname, vifname, metric)
+	_urib6.add_route(protocol, network, nexthop, ifname, vifname,
+					metric, policytags)
 	!= XORP_OK) {
 	string err = "Could not add IPv6 interface route to unicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
     }
 
     if (multicast &&
-	_mrib6.add_route(protocol, network, nexthop, ifname, vifname, metric)
+	_mrib6.add_route(protocol, network, nexthop, ifname, vifname, metric,
+					policytags)
 	!= XORP_OK) {
 	string err = "Could not add IPv6 interface route to multicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -651,18 +691,43 @@ XrlRibTarget::rib_0_1_add_interface_route6(const string&	protocol,
 }
 
 XrlCmdError
-XrlRibTarget::rib_0_1_replace_interface_route4(const string&	protocol,
-					       const bool&	unicast,
-					       const bool&	multicast,
-					       const IPv4Net&	network,
-					       const IPv4&	nexthop,
-					       const string&	ifname,
-					       const string&	vifname,
-					       const uint32_t&	metric)
+XrlRibTarget::rib_0_1_replace_interface_route4(const string&	    protocol,
+					       const bool&	    unicast,
+					       const bool&	    multicast,
+					       const IPv4Net&	    network,
+					       const IPv4&	    nexthop,
+					       const string&	    ifname,
+					       const string&	    vifname,
+					       const uint32_t&	    metric,
+					       const XrlAtomList&   policytags)
 {
+    debug_msg("replace_interface_route4 protocol: %s unicast: %s multicast: %s "
+	      "network %s nexthop %s ifname %s vifname %s metric %u\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
+	      network.str().c_str(),
+	      nexthop.str().c_str(),
+	      ifname.c_str(),
+	      vifname.c_str(),
+	      XORP_UINT_CAST(metric));
+
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("replace %s %s%s %s %s %s/%s %u",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str(),
+					     nexthop.str().c_str(),
+					     ifname.c_str(),
+					     vifname.c_str(),
+					     XORP_UINT_CAST(metric)));
+    }
+
     if (unicast &&
 	_urib4.replace_route(protocol, network, nexthop, ifname, vifname,
-			     metric)
+			     metric, policytags)
 	!= XORP_OK) {
 	string err = "Could not replace IPv4 interface route in unicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -670,7 +735,7 @@ XrlRibTarget::rib_0_1_replace_interface_route4(const string&	protocol,
 
     if (multicast &&
 	_mrib4.replace_route(protocol, network, nexthop, ifname, vifname,
-			     metric)
+			     metric, policytags)
 	!= XORP_OK) {
 	string err = "Could not replace IPv4 interface route in multicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -680,18 +745,43 @@ XrlRibTarget::rib_0_1_replace_interface_route4(const string&	protocol,
 }
 
 XrlCmdError
-XrlRibTarget::rib_0_1_replace_interface_route6(const string&	protocol,
-					       const bool&	unicast,
-					       const bool&	multicast,
-					       const IPv6Net&	network,
-					       const IPv6&	nexthop,
-					       const string&	ifname,
-					       const string&	vifname,
-					       const uint32_t&	metric)
+XrlRibTarget::rib_0_1_replace_interface_route6(const string&	    protocol,
+					       const bool&	    unicast,
+					       const bool&	    multicast,
+					       const IPv6Net&	    network,
+					       const IPv6&	    nexthop,
+					       const string&	    ifname,
+					       const string&	    vifname,
+					       const uint32_t&	    metric,
+					       const XrlAtomList&   policytags)
 {
+    debug_msg("replace_interface_route4 protocol: %s unicast: %s multicast: %s "
+	      "network %s nexthop %s ifname %s vifname %s metric %u\n",
+	      protocol.c_str(),
+	      unicast ? "true" : "false",
+	      multicast ? "true" : "false",
+	      network.str().c_str(),
+	      nexthop.str().c_str(),
+	      ifname.c_str(),
+	      vifname.c_str(),
+	      XORP_UINT_CAST(metric));
+
+    if (_rib_manager->profile().enabled(profile_route_ribin)) {
+	_rib_manager->profile().log(profile_route_ribin,
+				    c_format("replace %s %s%s %s %s %s/%s %u",
+					     protocol.c_str(),
+					     unicast ? "u" : "",
+					     multicast ? "m" : "",
+					     network.str().c_str(),
+					     nexthop.str().c_str(),
+					     ifname.c_str(),
+					     vifname.c_str(),
+					     XORP_UINT_CAST(metric)));
+    }
+
     if (unicast &&
 	_urib6.replace_route(protocol, network, nexthop, ifname, vifname,
-			     metric)
+			     metric, policytags)
 	!= XORP_OK) {
 	string err = "Could not replace IPv6 interface route in unicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -699,7 +789,7 @@ XrlRibTarget::rib_0_1_replace_interface_route6(const string&	protocol,
 
     if (multicast &&
 	_mrib6.replace_route(protocol, network, nexthop, ifname, vifname,
-			     metric)
+			     metric, policytags)
 	!= XORP_OK) {
 	string err = "Could not replace IPv6 interface route in multicast RIB";
 	return XrlCmdError::COMMAND_FAILED(err);
@@ -709,12 +799,13 @@ XrlRibTarget::rib_0_1_replace_interface_route6(const string&	protocol,
 }
 
 XrlCmdError
-XrlRibTarget::rib_0_1_lookup_route4(// Input values,
-				    const IPv4&	addr,
-				    const bool&	unicast,
-				    const bool&	multicast,
-				    // Output values,
-				    IPv4& nexthop)
+XrlRibTarget::rib_0_1_lookup_route_by_dest4(
+    // Input values,
+    const IPv4&	addr,
+    const bool&	unicast,
+    const bool&	multicast,
+    // Output values,
+    IPv4& nexthop)
 {
     // if unicast and multicast then fail, can only look one place at time
     if (unicast == multicast) {
@@ -728,12 +819,13 @@ XrlRibTarget::rib_0_1_lookup_route4(// Input values,
 }
 
 XrlCmdError
-XrlRibTarget::rib_0_1_lookup_route6(// Input values,
-				    const IPv6&	addr,
-				    const bool&	unicast,
-				    const bool&	multicast,
-				    // Output values,
-				    IPv6& nexthop)
+XrlRibTarget::rib_0_1_lookup_route_by_dest6(
+    // Input values,
+    const IPv6&	addr,
+    const bool&	unicast,
+    const bool&	multicast,
+    // Output values,
+    IPv6& nexthop)
 {
     // Must look in exactly one RIB
     if (unicast == multicast) {
@@ -980,19 +1072,20 @@ XrlRibTarget::rib_0_1_register_interest4(// Input values,
 					 IPv4&	nexthop,
 					 uint32_t& metric)
 {
-    debug_msg("#### XRL: REGISTER INTEREST addr %s\n",
-	   addr.str().c_str());
+    debug_msg("register_interest4 target = %s addr = %s\n",
+	      target.c_str(), addr.str().c_str());
+
     RouteRegister<IPv4>* rt_reg = _urib4.route_register(addr, target);
     if (rt_reg->route() == NULL) {
 	base_addr = rt_reg->valid_subnet().masked_addr();
 	prefix_len = real_prefix_len = rt_reg->valid_subnet().prefix_len();
 	resolves = false;
 	debug_msg("#### XRL -> REGISTER INTEREST UNRESOLVABLE %s\n",
-	       rt_reg->valid_subnet().str().c_str());
+		  rt_reg->valid_subnet().str().c_str());
     } else {
 	metric = rt_reg->route()->metric();
 	base_addr = rt_reg->valid_subnet().masked_addr();
-	prefix_len = rt_reg->valid_subnet().prefix_len();
+	prefix_len = real_prefix_len = rt_reg->valid_subnet().prefix_len();
 	NextHop *nh = rt_reg->route()->nexthop();
 	switch (nh->type()) {
 	case GENERIC_NEXTHOP:
@@ -1025,7 +1118,7 @@ XrlRibTarget::rib_0_1_deregister_interest4(// Input values,
 				    "prefix %s/%u",
 				    target.c_str(),
 				    addr.str().c_str(),
-				    prefix_len);
+				    XORP_UINT_CAST(prefix_len));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     return XrlCmdError::OKAY();
@@ -1043,15 +1136,20 @@ XrlRibTarget::rib_0_1_register_interest6(// Input values,
 					 IPv6&	nexthop,
 					 uint32_t& metric)
 {
+    debug_msg("register_interest6 target = %s addr = %s\n",
+	      target.c_str(), addr.str().c_str());
+
     RouteRegister<IPv6>* rt_reg = _urib6.route_register(addr, target);
     if (rt_reg->route() == NULL) {
 	base_addr = rt_reg->valid_subnet().masked_addr();
 	prefix_len = real_prefix_len = rt_reg->valid_subnet().prefix_len();
 	resolves = false;
+	debug_msg("#### XRL -> REGISTER INTEREST UNRESOLVABLE %s\n",
+		  rt_reg->valid_subnet().str().c_str());
     } else {
 	metric = rt_reg->route()->metric();
 	base_addr = rt_reg->valid_subnet().masked_addr();
-	prefix_len = rt_reg->valid_subnet().prefix_len();
+	prefix_len = real_prefix_len = rt_reg->valid_subnet().prefix_len();
 	NextHop *nh = rt_reg->route()->nexthop();
 	switch (nh->type()) {
 	case GENERIC_NEXTHOP:
@@ -1084,7 +1182,7 @@ XrlRibTarget::rib_0_1_deregister_interest6(// Input values,
 				    "prefix %s/%u",
 				    target.c_str(),
 				    addr.str().c_str(),
-				    prefix_len);
+				    XORP_UINT_CAST(prefix_len));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     return XrlCmdError::OKAY();
@@ -1095,8 +1193,9 @@ XrlRibTarget::finder_event_observer_0_1_xrl_target_birth(
         const string&	target_class,
 	const string&	target_instance)
 {
-    debug_msg(("Target Birth: " + target_class + " " + target_instance
-	       + "\n").c_str());
+    debug_msg("Target Birth: class = %s instance = %s\n",
+	      target_class.c_str(), target_instance.c_str());
+
     UNUSED(target_class);
     UNUSED(target_instance);
 
@@ -1108,8 +1207,147 @@ XrlRibTarget::finder_event_observer_0_1_xrl_target_death(
 	const string&	target_class,
 	const string&	target_instance)
 {
-    debug_msg(("Target Death: " + target_class + " " + target_instance
-	       + "\n").c_str());
+    debug_msg("Target Death: class = %s instance = %s\n",
+	      target_class.c_str(), target_instance.c_str());
+
     _rib_manager->target_death(target_class, target_instance);
     return XrlCmdError::OKAY();
 }
+
+
+XrlCmdError
+XrlRibTarget::policy_backend_0_1_configure(const uint32_t& filter,
+					   const string&   conf)
+{
+    try {
+	_rib_manager->configure_filter(filter, conf);
+    } catch(const PolicyException& e) {
+	return XrlCmdError::COMMAND_FAILED("Filter configure failed: " +
+					   e.str());
+    }
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::policy_backend_0_1_reset(const uint32_t& filter)
+{
+    try {
+	_rib_manager->reset_filter(filter);
+    } catch(const PolicyException& e) {
+	return XrlCmdError::COMMAND_FAILED("Filter reset failed: " +
+					   e.str());
+    }
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::policy_backend_0_1_push_routes()
+{
+    _rib_manager->push_routes();
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::rib_0_1_insert_policy_redist_tags(const string& protocol,
+						const XrlAtomList& policytags)
+{
+    // doubt these will ever be used
+    try {
+	_rib_manager->insert_policy_redist_tags(protocol, policytags);
+    } catch(const PolicyException& e) {
+	return XrlCmdError::COMMAND_FAILED("Insert policy redist tags failed: "
+					   + e.str());
+    }
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::rib_0_1_reset_policy_redist_tags()
+{
+    // just a guard for the future.
+    try {
+	_rib_manager->reset_policy_redist_tags();
+    } catch(const PolicyException& e) {
+	return XrlCmdError::COMMAND_FAILED("Reset policy redist tags failed: " +
+					   e.str());
+    }
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::profile_0_1_enable(const string& pname)
+{
+    debug_msg("enable profile variable %s\n", pname.c_str());
+
+    try {
+	_rib_manager->profile().enable(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::profile_0_1_disable(const string&	pname)
+{
+    debug_msg("disable profile variable %s\n", pname.c_str());
+
+    try {
+	_rib_manager->profile().disable(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlRibTarget::profile_0_1_get_entries(const string& pname,
+				      const string& instance_name)
+{
+    debug_msg("profile variable %s instance %s\n", pname.c_str(),
+	      instance_name.c_str());
+
+    // Lock and initialize.
+    try {
+	_rib_manager->profile().lock_log(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    ProfileUtils::transmit_log(pname,
+			       &_rib_manager->xrl_router(), instance_name,
+			       &_rib_manager->profile());
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::profile_0_1_clear(const string& pname)
+{
+    debug_msg("clear profile variable %s\n", pname.c_str());
+
+    try {
+	_rib_manager->profile().clear(pname);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::profile_0_1_list(string& info)
+{
+    debug_msg("\n");
+    
+    info = _rib_manager->profile().list();
+    return XrlCmdError::OKAY();
+}
