@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/port.cc,v 1.35 2004/06/10 22:41:45 hodson Exp $"
+#ident "$XORP: xorp/rip/port.cc,v 1.40 2005/03/25 02:54:27 pavlin Exp $"
 
 #include "rip_module.h"
 
@@ -203,7 +203,8 @@ Port<A>::triggered_update_timeout()
     {
 	RouteDB<A>& rdb = _pm.system().route_db();
 	UNUSED(rdb);
-	debug_msg("- Route DB routes = %u\n", rdb.route_count());
+	debug_msg("- Route DB routes = %u\n",
+		  XORP_UINT_CAST(rdb.route_count()));
     }
 
     // Table dump is running, we should not be doing triggered updates.
@@ -568,7 +569,7 @@ Port<A>::kill_peer_routes()
 	for (ri = routes.begin(); ri != routes.end(); ++ri) {
 	    const RouteEntry<A>* r = *ri;
 	    rdb.update_route(r->net(), r->nexthop(), RIP_INFINITY, r->tag(),
-			     p);
+			     p, r->policytags());
 	}
 	pli++;
     }
@@ -752,8 +753,8 @@ Port<A>::port_io_receive(const A&	src_address,
 
     if (rip_packet_bytes < RIPv2_MIN_PACKET_BYTES) {
 	record_bad_packet(c_format("Packet size less than minimum (%u < %u)",
-				   uint32_t(rip_packet_bytes),
-				   uint32_t(RIPv2_MIN_PACKET_BYTES)),
+				   XORP_UINT_CAST(rip_packet_bytes),
+				   XORP_UINT_CAST(RIPv2_MIN_PACKET_BYTES)),
 			  src_address, src_port, p);
 	return;
     }
@@ -918,7 +919,7 @@ Port<IPv4>::parse_response(const Addr&				src_addr,
 	    nh = src_addr;
 	}
 
-	metric += metric + cost();
+	metric += cost();
 	if (metric > RIP_INFINITY) {
 	    metric = RIP_INFINITY;
 	}
@@ -929,7 +930,7 @@ Port<IPv4>::parse_response(const Addr&				src_addr,
 	//
 	uint16_t tag = entries[i].tag();
 
-	rdb.update_route(net, nh, metric, tag, p);
+	rdb.update_route(net, nh, metric, tag, p, PolicyTags());
     }
 }
 
@@ -960,7 +961,7 @@ Port<IPv6>::parse_response(const Addr&				src_addr,
 
     // ALL_ONES is used as a magic value to indicate no nexthop has been set.
     IPv6 nh = IPv6::ALL_ONES();
-    for(uint32_t i = 0; i < n_entries; i++) {
+    for (uint32_t i = 0; i < n_entries; i++) {
 	if (entries[i].is_nexthop()) {
 	    nh = entries[i].nexthop();
 	    if (nh == IPv6::ZERO()) {
@@ -1021,7 +1022,7 @@ Port<IPv6>::parse_response(const Addr&				src_addr,
 	//
 	uint16_t tag = entries[i].tag();
 
-	rdb.update_route(net, nh, metric, tag, p);
+	rdb.update_route(net, nh, metric, tag, p, PolicyTags());
     }
 
 }
