@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/test_outputs.cc,v 1.14 2004/06/10 22:41:46 hodson Exp $"
+#ident "$XORP: xorp/rip/test_outputs.cc,v 1.18 2005/03/25 02:54:30 pavlin Exp $"
 
 #include <set>
 
@@ -346,10 +346,11 @@ public:
 	    } else {
 		// Not a test peer net and not an other peer net
 		// ==> it's bogus
-		verbose_log("Failed Processing entry %d / %d %s cost %u\n",
-			    rr.packet_entry(), p->max_entries(),
+		verbose_log("Failed Processing entry %u / %u %s cost %u\n",
+			    XORP_UINT_CAST(rr.packet_entry()),
+			    XORP_UINT_CAST(p->max_entries()),
 			    n.str().c_str(),
-			    cost);
+			    XORP_UINT_CAST(cost));
 		return false;
 	    }
 	}
@@ -359,13 +360,14 @@ public:
     bool valid_in_sum() const
     {
 	if (this->test_peer_routes_seen() != this->_tpn.size()) {
-	    verbose_log("Test routes seen (%d) does not match expected (%d)\n",
-			this->test_peer_routes_seen(), 
-			(int32_t)this->_tpn.size());
+	    verbose_log("Test routes seen (%u) does not match expected (%u)\n",
+			XORP_UINT_CAST(this->test_peer_routes_seen()),
+			XORP_UINT_CAST(this->_tpn.size()));
 	    return false;
 	}
-	verbose_log("total routes seen %d, test peer routes seen = %d\n",
-		    this->total_routes_seen(), this->test_peer_routes_seen());
+	verbose_log("total routes seen %u, test peer routes seen = %u\n",
+		    XORP_UINT_CAST(this->total_routes_seen()),
+		    XORP_UINT_CAST(this->test_peer_routes_seen()));
 	return this->test_peer_routes_seen() * 2 == this->total_routes_seen();
     }
 };
@@ -390,9 +392,11 @@ public:
 	    if (this->_opn.find(n) == this->_opn.end()) {
 		verbose_log("Saw own or alien route with split horizon\n");
 		// ==> it's bogus
-		verbose_log("Failed Processing entry %d / %d %s cost %u\n",
-			    rr.packet_entry(), p->max_entries(),
-			    n.str().c_str(), cost);
+		verbose_log("Failed Processing entry %u / %u %s cost %u\n",
+			    XORP_UINT_CAST(rr.packet_entry()),
+			    XORP_UINT_CAST(p->max_entries()),
+			    n.str().c_str(),
+			    XORP_UINT_CAST(cost));
 		return false;
 	    }
 	}
@@ -402,12 +406,15 @@ public:
     bool valid_in_sum() const
     {
 	if (this->test_peer_routes_seen() != 0) {
-	    verbose_log("Test peer routes seen (%d) does not match expected "
-			"(%d)\n", this->test_peer_routes_seen(), 0);
+	    verbose_log("Test peer routes seen (%u) does not match expected "
+			"(%u)\n",
+			XORP_UINT_CAST(this->test_peer_routes_seen()),
+			XORP_UINT_CAST(0));
 	    return false;
 	}
-	verbose_log("total routes seen %d, test peer routes seen = %d\n",
-		    this->total_routes_seen(), this->test_peer_routes_seen());
+	verbose_log("total routes seen %u, test peer routes seen = %u\n",
+		    XORP_UINT_CAST(this->total_routes_seen()),
+		    XORP_UINT_CAST(this->test_peer_routes_seen()));
 	return this->total_routes_seen() == (uint32_t)this->_opn.size();
     }
 };
@@ -439,9 +446,11 @@ public:
 	    } else {
 		// Not a test peer net and not an other peer net
 		// ==> it's bogus
-		verbose_log("Failed Processing entry %d / %d %s cost %u\n",
-			    rr.packet_entry(), p->max_entries(),
-			    n.str().c_str(), cost);
+		verbose_log("Failed Processing entry %u / %u %s cost %u\n",
+			    XORP_UINT_CAST(rr.packet_entry()),
+			    XORP_UINT_CAST(p->max_entries()),
+			    n.str().c_str(),
+			    XORP_UINT_CAST(cost));
 		return false;
 	    }
 	}
@@ -451,13 +460,14 @@ public:
     bool valid_in_sum() const
     {
 	if (this->test_peer_routes_seen() != this->_tpn.size()) {
-	    verbose_log("Test routes seen (%d) does not match expected (%d)\n",
-			this->test_peer_routes_seen(), 
-			(int32_t)this->_tpn.size());
+	    verbose_log("Test routes seen (%u) does not match expected (%u)\n",
+			XORP_UINT_CAST(this->test_peer_routes_seen()),
+			XORP_UINT_CAST(this->_tpn.size()));
 	    return false;
 	}
-	verbose_log("total routes seen %d, test peer routes seen = %d\n",
-		    this->total_routes_seen(), this->test_peer_routes_seen());
+	verbose_log("total routes seen %u, test peer routes seen = %u\n",
+		    XORP_UINT_CAST(this->total_routes_seen()),
+		    XORP_UINT_CAST(this->test_peer_routes_seen()));
 	return this->test_peer_routes_seen() * 2 == this->total_routes_seen();
     }
 };
@@ -514,7 +524,7 @@ public:
 	for (typename set<IPNet<A> >::const_iterator n = _tpn.begin();
 	     n != _tpn.end(); n++) {
 	    RouteEntryOrigin<A>* reo = _pm.test_peer();
-	    if (rdb.update_route(*n, A::ZERO(), 5u, 0u, reo) == false) {
+	    if (rdb.update_route(*n, A::ZERO(), 5u, 0u, reo, PolicyTags()) == false) {
 		verbose_log("Failed to add route for %s\n",
 			    n->str().c_str());
 		return 1;
@@ -525,7 +535,7 @@ public:
 	for (typename set<IPNet<A> >::const_iterator n = this->_opn.begin();
 	     n != this->_opn.end(); n++) {
 	    RouteEntryOrigin<A>* reo = _pm.other_peer();
-	    if (rdb.update_route(*n, A::ZERO(), 5u, 0u, reo) == false) {
+	    if (rdb.update_route(*n, A::ZERO(), 5u, 0u, reo, PolicyTags()) == false) {
 		verbose_log("Failed to add route for %s\n",
 			    n->str().c_str());
 		return 1;
@@ -537,10 +547,10 @@ public:
 	ou.start();
 
 	while (ou.running() && timeout == false) {
-	    _e.run();
+  	    _e.run();
 	}
-	verbose_log("%d bytes buffered in packet queue.\n",
-		    op_out.buffered_bytes());
+	verbose_log("%u bytes buffered in packet queue.\n",
+		    XORP_UINT_CAST(op_out.buffered_bytes()));
 	if (timeout) {
 	    verbose_log("Timed out!\n");
 	    return 1;
@@ -629,7 +639,7 @@ run_all_test_cases()
     //
     {
 	verbose_log("=== IPv%u No Horizon updates test ===\n",
-		    A::ip_version());
+		    XORP_UINT_CAST(A::ip_version()));
 	OutputTester<A, OutputUpdates<A> > tester(tpn, opn);
 	NoHorizonValidator<A> nohv(tpn, opn);
 	rval |= tester.run_test(NONE, nohv);
@@ -638,7 +648,7 @@ run_all_test_cases()
     }
     {
 	verbose_log("=== IPv%u Split Horizon updates test ===\n",
-		    A::ip_version());
+		    XORP_UINT_CAST(A::ip_version()));
 	OutputTester<A, OutputUpdates<A> > tester(tpn, opn);
 	SplitHorizonValidator<A> shv(tpn, opn);
 	rval |= tester.run_test(SPLIT, shv);
@@ -647,7 +657,7 @@ run_all_test_cases()
     }
     {
 	verbose_log("=== IPv%u Split Horizon Poison Reverse test ===\n",
-		    A::ip_version());
+		    XORP_UINT_CAST(A::ip_version()));
 	OutputTester<A, OutputUpdates<A> > tester(tpn, opn);
 	PoisonReverseValidator<A> prv(tpn, opn);
 	rval |= tester.run_test(SPLIT_POISON_REVERSE, prv);
@@ -660,7 +670,7 @@ run_all_test_cases()
     //
     {
 	verbose_log("=== IPv%u No Horizon table test ===\n",
-		    A::ip_version());
+		    XORP_UINT_CAST(A::ip_version()));
 	OutputTester<A, OutputTable<A> > tester(tpn, opn);
 	NoHorizonValidator<A> nohv(tpn, opn);
 	rval |= tester.run_test(NONE, nohv);
@@ -669,7 +679,7 @@ run_all_test_cases()
     }
     {
 	verbose_log("=== IPv%u Split Horizon table test ===\n",
-		    A::ip_version());
+		    XORP_UINT_CAST(A::ip_version()));
 	OutputTester<A, OutputTable<A> > tester(tpn, opn);
 	SplitHorizonValidator<A> shv(tpn, opn);
 	rval |= tester.run_test(SPLIT, shv);
@@ -678,7 +688,7 @@ run_all_test_cases()
     }
     {
 	verbose_log("=== IPv%u Split Horizon Poison Reverse table test ===\n",
-		    A::ip_version());
+		    XORP_UINT_CAST(A::ip_version()));
 	OutputTester<A, OutputTable<A> > tester(tpn, opn);
 	PoisonReverseValidator<A> prv(tpn, opn);
 	rval |= tester.run_test(SPLIT_POISON_REVERSE, prv);
