@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/test_timers.cc,v 1.9 2004/06/10 22:41:47 hodson Exp $"
+#ident "$XORP: xorp/rip/test_timers.cc,v 1.13 2005/03/25 02:54:30 pavlin Exp $"
 
 #include <set>
 
@@ -91,12 +91,12 @@ public:
     {
 	this->_peers.push_back(new Peer<A>(*this, addr));
 	verbose_log("Constructing SpoofPort<IPv%u> instance\n",
-		    A::ip_version());
+		    XORP_UINT_CAST(A::ip_version()));
     }
     ~SpoofPort()
     {
 	verbose_log("Destructing SpoofPort<IPv%u> instance\n",
-		    A::ip_version());
+		    XORP_UINT_CAST(A::ip_version()));
 	while (this->_peers.empty() == false) {
 	    delete this->_peers.front();
 	    this->_peers.pop_front();
@@ -169,16 +169,20 @@ test_main()
 
     if (peer->route_count() != n_routes) {
 	verbose_log("Routes lost (count %u != %u)\n",
-		    (uint32_t)peer->route_count(), n_routes);
+		    XORP_UINT_CAST(peer->route_count()),
+		    XORP_UINT_CAST(n_routes));
 	return 1;
     }
 
+    e.timer_list().advance_time(); // XXX
+
     {
 	// Quick route dump test
 	TimeVal t1, t2;
 	e.current_time(t1);
 	vector<typename RouteDB<A>::ConstDBRouteEntry> l;
 	rdb.dump_routes(l);
+	e.timer_list().advance_time(); // XXX
 	e.current_time(t2);
 	t2 -= t1;
 	fprintf(stderr, "route db route dump took %d.%06d seconds\n",
@@ -190,6 +194,7 @@ test_main()
 	e.current_time(t1);
 	vector<const typename RouteEntryOrigin<A>::Route*> l;
 	peer->dump_routes(l);
+	e.timer_list().advance_time(); // XXX
 	e.current_time(t2);
 	t2 -= t1;
 	fprintf(stderr, "peer route dump took %d.%06d seconds\n",
@@ -201,6 +206,7 @@ test_main()
 	e.current_time(t1);
 	vector<typename RouteDB<A>::ConstDBRouteEntry> l;
 	rdb.dump_routes(l);
+	e.timer_list().advance_time(); // XXX
 	e.current_time(t2);
 	t2 -= t1;
 	fprintf(stderr, "route db route dump took %d.%06d seconds\n",
@@ -219,7 +225,7 @@ test_main()
 
     while (timeout == false) {
 	size_t route_count = peer->route_count();
-	verbose_log("Route count = %u\n", (uint32_t)route_count);
+	verbose_log("Route count = %u\n", XORP_UINT_CAST(route_count));
 	if (route_count == 0)
 	    break;
 	e.run();
