@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/test_update_queue.cc,v 1.8 2004/06/10 22:41:47 hodson Exp $"
+#ident "$XORP: xorp/rip/test_update_queue.cc,v 1.13 2005/03/25 02:54:30 pavlin Exp $"
 
 #include <set>
 
@@ -143,7 +143,8 @@ public:
     {
 	const uint32_t n_routes = 20000;
 
-	verbose_log("Running test for IPv%u\n", A::ip_version());
+	verbose_log("Running test for IPv%u\n",
+		    XORP_UINT_CAST(A::ip_version()));
 
 	RouteDB<A>& rdb = _rip_system.route_db();
 	UpdateQueue<A>& uq = _rip_system.route_db().update_queue();
@@ -167,14 +168,15 @@ public:
 
 	if (uq.updates_queued() != n_routes) {
 	    verbose_log("%u updates queued, expected %u\n",
-			uq.updates_queued(), n_routes);
+			XORP_UINT_CAST(uq.updates_queued()),
+			XORP_UINT_CAST(n_routes));
 	    return 1;
 	}
 
 	uint32_t n = 0;
 	for (n = 0; n < n_routes; n++) {
 	    if (uq.get(_fast_reader) == 0) {
-		verbose_log("Ran out of updates at %u.\n", n);
+		verbose_log("Ran out of updates at %u.\n", XORP_UINT_CAST(n));
 		return 1;
 	    }
 	    uq.next(_fast_reader);
@@ -192,11 +194,12 @@ public:
 	}
 
 	verbose_log("%u updates queued, expected %u\n",
-		    uq.updates_queued(), n_routes);
+		    XORP_UINT_CAST(uq.updates_queued()),
+		    XORP_UINT_CAST(n_routes));
 
 	for (n = 0; n < n_routes; n++) {
 	    if (uq.get(_fast_reader) == 0) {
-		verbose_log("Ran out of updates at %u.\n", n);
+		verbose_log("Ran out of updates at %u.\n", XORP_UINT_CAST(n));
 		return 1;
 	    }
 	    uq.next(_fast_reader);
@@ -204,7 +207,7 @@ public:
 
 	for (n = 0; n < 2 * n_routes; n++) {
 	    if (uq.get(_slow_reader) == 0) {
-		verbose_log("Ran out of updates at %u.\n", n);
+		verbose_log("Ran out of updates at %u.\n", XORP_UINT_CAST(n));
 		return 1;
 	    }
 	    uq.next(_slow_reader);
@@ -220,7 +223,7 @@ public:
 
 	if (uq.updates_queued() != 0) {
 	    verbose_log("Updates queued (%u) when no readers present\n",
-			uq.updates_queued());
+			XORP_UINT_CAST(uq.updates_queued()));
 	    return 1;
 	}
 
@@ -232,7 +235,7 @@ public:
 	for (typename set<IPNet<A> >::const_iterator n = nets.begin();
 	     n != nets.end(); ++n) {
 	    if (rdb.update_route(*n, A::ZERO(), 5, 0,
-				 _pm.the_peer()) == false) {
+				 _pm.the_peer(), PolicyTags()) == false) {
 		verbose_log("Failed to add route for %s\n",
 			    n->str().c_str());
 		return 1;
@@ -267,13 +270,13 @@ public:
 
 	for (n = 0; n < n_routes; n++) {
 	    if (uq.get(_slow_reader) == 0) {
-		verbose_log("Ran out of updates at %u.\n", n);
+		verbose_log("Ran out of updates at %u.\n", XORP_UINT_CAST(n));
 		return 1;
 	    }
 	    uq.next(_slow_reader);
 
 	    if (uq.get(_fast_reader) == 0) {
-		verbose_log("Ran out of updates at %u.\n", n);
+		verbose_log("Ran out of updates at %u.\n", XORP_UINT_CAST(n));
 		return 1;
 	    }
 	    uq.next(_fast_reader);
