@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/cli.cc,v 1.54 2004/06/12 01:38:48 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/cli.cc,v 1.64 2005/03/25 02:54:34 pavlin Exp $"
 
 #include <pwd.h>
 
@@ -629,7 +629,7 @@ RouterCLI::display_config_mode_users() const
 	}
 	pwent = getpwuid(*iter);
 	if (pwent == NULL)
-	    _cli_client.cli_print(c_format("UID:%d", *iter));
+	    _cli_client.cli_print(c_format("UID:%d", XORP_UINT_CAST(*iter)));
 	else
 	    _cli_client.cli_print(pwent->pw_name);
     }
@@ -1012,6 +1012,10 @@ RouterCLI::add_immediate_commands(CliCommand& current_cli_node,
 
     list<TemplateTreeNode*>::const_iterator tti;
     for (tti = ttn->children().begin(); tti != ttn->children().end(); ++tti) {
+	// XXX: ignore deprecated subtrees
+	 if ((*tti)->is_deprecated())
+	     continue;
+
 	// We don't need to consider this child if it's already added
 	if (existing_children.find((*tti)->segname())
 	    != existing_children.end()) {
@@ -1091,6 +1095,7 @@ RouterCLI::add_edit_subtree()
 
     cmds.push_back("%create");
     cmds.push_back("%activate");
+    cmds.push_back("%update");
     SlaveConfigTreeNode *current_config_node = config_tree()->find_node(_path);
     XLOG_ASSERT(current_config_node != NULL);
 
@@ -1152,6 +1157,7 @@ RouterCLI::add_delete_subtree()
 
     cmds.push_back("%create");
     cmds.push_back("%activate");
+    cmds.push_back("%update");
     cmds.push_back("%set");
     cmds.push_back("%delete");
     SlaveConfigTreeNode *current_config_node = config_tree()->find_node(_path);
@@ -1272,6 +1278,10 @@ RouterCLI::add_text_entry_commands(CliCommand* com0)
 	 CliCommand* com;
 	 string subpath;
 
+	// XXX: ignore deprecated subtrees
+	 if ((*tti)->is_deprecated())
+	     continue;
+
 	 if (pathstr().empty())
 	     subpath = (*tti)->segname();
 	 else
@@ -1387,7 +1397,7 @@ RouterCLI::new_config_user(uid_t user_id)
     struct passwd* pwent = getpwuid(user_id);
     string username;
     if (pwent == NULL)
-	username = c_format("UID:%d", user_id);
+	username = c_format("UID:%u", XORP_UINT_CAST(user_id));
     else
 	username = pwent->pw_name;
     string alert = c_format("User %s entered configuration mode\n",
@@ -1736,6 +1746,9 @@ RouterCLI::text_entry_func(const string& ,
 	    for (iter = tag_ttn->children().begin();
 		 iter != tag_ttn->children().end();
 		 ++iter) {
+		// XXX: ignore deprecated subtrees
+		if ((*iter)->is_deprecated())
+		    continue;
 		if ((*iter)->type_match(argv[0])) {
 		    data_ttn = (*iter);
 		    break;
@@ -1949,7 +1962,7 @@ RouterCLI::text_entry_func(const string& ,
 	//
 	if (brace_ctn == NULL) {
 	    _braces.push_front(0);
-	    brace_ctn = &(config_tree()->root_node());
+	    brace_ctn = &(config_tree()->slave_root_node());
 	} else {
 	    _braces.push_front(brace_ctn->depth());
 	}
@@ -1983,7 +1996,7 @@ RouterCLI::text_entry_func(const string& ,
     if (ctn == NULL)
 	ctn = brace_ctn;
     if (ctn == NULL)
-	ctn = &(config_tree()->root_node());
+	ctn = &(config_tree()->slave_root_node());
 
     //
     // At this point, path_segments contains the path of nodes that
@@ -2053,6 +2066,9 @@ RouterCLI::text_entry_func(const string& ,
 		for (tti = ttn->children().begin();
 		     tti != ttn->children().end();
 		     ++tti) {
+		    // XXX: ignore deprecated subtrees
+		    if ((*tti)->is_deprecated())
+			continue;
 		    if ((*tti)->type_match(value)) {
 			data_ttn = (*tti);
 			break;
@@ -2119,17 +2135,28 @@ RouterCLI::text_entry_func(const string& ,
 		    _cli_client.cli_print("ERROR: mismatched \"}\".\n");
 		    goto cleanup;
 		}
-		// The last brace we entered should match the current depth
 		XLOG_TRACE(_verbose, "braces: %u ctn depth: %u ctn: %s\n", 
-			   _braces.back(),
+			   XORP_UINT_CAST(_braces.back()),
 			   ctn->depth(), ctn->segname().c_str());
+		while (ctn->depth() > _braces.back()) {
+		    //looks like one or more close braces on the same
+		    //line as some config - just pop out
+		    //appropriately before we can handle this correctly.
+		    ctn = ctn->parent();
+		    XLOG_TRACE(_verbose, "jumping out one level\n");
+		    XLOG_TRACE(_verbose, "braces: %u ctn depth: %u ctn: %s\n", 
+			       XORP_UINT_CAST(_braces.back()),
+			       ctn->depth(), ctn->segname().c_str());
+		}
+		// The last brace we entered should match the current depth
 		XLOG_ASSERT(_braces.back() == ctn->depth());
 		_braces.pop_back();
 		// The next brace should be where we're jumping to.
 		uint32_t new_depth = 0;
 		XLOG_ASSERT(!_braces.empty());
 		new_depth = _braces.back();
-		XLOG_TRACE(_verbose, "new_depth: %u\n", new_depth);
+		XLOG_TRACE(_verbose, "new_depth: %u\n",
+			   XORP_UINT_CAST(new_depth));
 		XLOG_ASSERT(ctn->depth() > new_depth);
 		while (ctn->depth() > new_depth) {
 		    ctn = ctn->parent();
@@ -2155,6 +2182,19 @@ RouterCLI::text_entry_func(const string& ,
 		_cli_client.cli_print(errmsg);
 		goto cleanup;
 	    }
+	    //
+	    // Test if some part of the configuration path is deprecated
+	    //
+	    const TemplateTreeNode* deprecated_ttn;
+	    deprecated_ttn = ttn->find_first_deprecated_ancestor();
+	    if (deprecated_ttn != NULL) {
+		string errmsg = c_format("ERROR: path \"%s\" "
+					 "is not valid: %s.\n",
+					 makepath(path_segments).c_str(),
+					 deprecated_ttn->deprecated_reason().c_str());
+		_cli_client.cli_print(errmsg);
+		goto cleanup;
+	    }
 
 	    XLOG_TRACE(_verbose, "creating node %s\n", ttn->segname().c_str());
 	    ctn = new SlaveConfigTreeNode(ttn->segname(), 
@@ -2262,10 +2302,13 @@ RouterCLI::text_entry_children_func(const string& path,
     const TemplateTreeNode *ttn = template_tree()->find_node(path_segments);
     is_executable = true;
     can_pipe = false;
-    if (ttn != NULL) {
+    if (ttn != NULL && (! ttn->is_deprecated())) {
 	list<TemplateTreeNode*>::const_iterator tti;
 	for (tti = ttn->children().begin(); tti != ttn->children().end(); 
 	     ++tti) {
+	    // XXX: ignore deprecated subtrees
+	    if ((*tti)->is_deprecated())
+		continue;
 	    string help;
 	    help = (*tti)->help();
 	    string subpath;
@@ -2389,7 +2432,7 @@ RouterCLI::run_set_command(const string& path, const vector<string>& argv)
     list<string> path_parts;
     path_parts = split(path, ' ');
 
-    ConfigTreeNode* ctn = config_tree()->find_node(path_parts);
+    SlaveConfigTreeNode* ctn = config_tree()->find_node(path_parts);
     const TemplateTreeNode* ttn;
     bool create_needed = false;
     string nodename;
@@ -2433,11 +2476,12 @@ RouterCLI::run_set_command(const string& path, const vector<string>& argv)
 	    else
 		newpath += " " + *iter;
 	}
-	ConfigTreeNode* newnode = new ConfigTreeNode(path_parts.back(),
-						     newpath, ttn,
-						     ctn,
-						     getuid(),
-						     _verbose);
+	SlaveConfigTreeNode* newnode;
+	newnode = new SlaveConfigTreeNode(path_parts.back(),
+					  newpath, ttn,
+					  ctn,
+					  getuid(),
+					  _verbose);
 	ctn = newnode;
 	ctn->set_value(argv[0], getuid());
     } else {
@@ -2447,7 +2491,6 @@ RouterCLI::run_set_command(const string& path, const vector<string>& argv)
     //
     // We don't execute commands until a commit is received
     //
-    // string result = config_tree()->execute_node_command(ctn, "%set");
     _changes_made = true;
 
     string result = "OK";
@@ -2597,7 +2640,7 @@ RouterCLI::op_mode_func(const string& ,
 
 	// Verify that any previous command has been disposed of.
 	XLOG_ASSERT(0 == _op_mode_cmd);
-	_op_mode_cmd = op_cmd_list()->execute(&(_xorpsh.eventloop()),
+	_op_mode_cmd = op_cmd_list()->execute(_xorpsh.eventloop(),
 					      path_segments,
 			       callback(this, &RouterCLI::op_mode_cmd_print),
 			       callback(this, &RouterCLI::op_mode_cmd_done));
