@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/master_conf_tree.cc,v 1.37 2004/06/10 22:41:52 hodson Exp $"
+#ident "$XORP: xorp/rtrmgr/master_conf_tree.cc,v 1.47 2005/03/25 02:54:35 pavlin Exp $"
 
 
 #include <sys/stat.h>
@@ -28,8 +28,8 @@
 #include "module_command.hh"
 #include "rtrmgr_error.hh"
 #include "template_commands.hh"
-#include "template_tree.hh"
-#include "template_tree_node.hh"
+#include "master_template_tree.hh"
+#include "master_template_tree_node.hh"
 #include "util.hh"
 
 
@@ -38,16 +38,20 @@
  *************************************************************************/
 
 MasterConfigTree::MasterConfigTree(const string& config_file,
-				   TemplateTree* tt,
+				   MasterTemplateTree* tt,
 				   ModuleManager& mmgr,
 				   XorpClient& xclient,
 				   bool global_do_exec,
 				   bool verbose) throw (InitError)
     : ConfigTree(tt, verbose),
-      _task_manager(*this, mmgr, xclient, global_do_exec, verbose),
+      _root_node(verbose),
       _commit_in_progress(false),
       _config_failed(false)
 {
+    _current_node = &_root_node;
+    _task_manager = new TaskManager(*this, mmgr, xclient, 
+				    global_do_exec, verbose);
+
     string configuration;
     string errmsg;
 
@@ -65,6 +69,10 @@ MasterConfigTree::MasterConfigTree(const string& config_file,
     //
     add_default_children();
 
+    if (root_node().check_config_tree(errmsg) != true) {
+	xorp_throw(InitError, errmsg);
+    }
+
     //
     // If we got this far, it looks like we have a good bootfile.
     // The bootfile is now stored in the ConfigTree.
@@ -73,6 +81,34 @@ MasterConfigTree::MasterConfigTree(const string& config_file,
     execute();
 }
 
+MasterConfigTree::MasterConfigTree(TemplateTree* tt, bool verbose)
+    : ConfigTree(tt, verbose),
+      _root_node(verbose),
+      _task_manager(NULL)
+{
+    _current_node = &_root_node;
+}
+
+MasterConfigTree::~MasterConfigTree()
+{
+    delete _task_manager;
+}
+
+MasterConfigTree&
+MasterConfigTree::operator=(const MasterConfigTree& orig_tree)
+{
+    master_root_node().clone_subtree(orig_tree.const_master_root_node());
+    return *this;
+}
+
+ConfigTree* MasterConfigTree::create_tree(TemplateTree *tt, bool verbose)
+{
+    MasterConfigTree *mct;
+    mct = new MasterConfigTree(tt, verbose);
+    return mct;
+}
+
+
 bool
 MasterConfigTree::read_file(string& configuration,
 			    const string& config_file,
@@ -163,13 +199,28 @@ MasterConfigTree::config_done(bool success, string errmsg)
     debug_msg("MasterConfigTree::config_done returning\n");
 }
 
+ConfigTreeNode*
+MasterConfigTree::create_node(const string& segment, const string& path,
+			      const TemplateTreeNode* ttn, 
+			      ConfigTreeNode* parent_node, 
+			      uid_t user_id, bool verbose)
+{
+    MasterConfigTreeNode *ctn, *parent;
+    parent = dynamic_cast<MasterConfigTreeNode *>(parent_node);
+    if (parent_node != NULL)
+	XLOG_ASSERT(parent != NULL);
+    ctn = new MasterConfigTreeNode(segment, path, ttn, parent, 
+				   user_id, verbose);
+    return reinterpret_cast<ConfigTreeNode*>(ctn);
+}
+
 list<string>
 MasterConfigTree::find_changed_modules() const
 {
     debug_msg("Find changed modules\n");
 
     set<string> changed_modules;
-    _root_node.find_changed_modules(changed_modules);
+    const_master_root_node().find_changed_modules(changed_modules);
 
     list<string> ordered_modules;
     order_module_list(changed_modules, ordered_modules);
@@ -183,7 +234,7 @@ MasterConfigTree::find_active_modules() const
     debug_msg("Find active modules\n");
 
     set<string> active_modules;
-    _root_node.find_active_modules(active_modules);
+    const_master_root_node().find_active_modules(active_modules);
 
     list<string> ordered_modules;
     order_module_list(active_modules, ordered_modules);
@@ -200,28 +251,30 @@ MasterConfigTree::find_inactive_modules() const
     list<string> ordered_all_modules;
 
     debug_msg("All modules:\n");
-    _root_node.find_all_modules(all_modules);
+    const_master_root_node().find_all_modules(all_modules);
     order_module_list(all_modules, ordered_all_modules);
 
     set<string> active_modules;
     list<string> ordered_active_modules;
     debug_msg("Active modules:\n");
-    _root_node.find_active_modules(active_modules);
+    master_root_node().find_active_modules(active_modules);
     order_module_list(active_modules, ordered_active_modules);
 
     // Remove things that are common to both lists
-    list<string>::iterator iter;
     while (!ordered_active_modules.empty()) {
+	list<string>::iterator iter;
+	bool found = false;
 	for (iter = ordered_all_modules.begin();
 	     iter != ordered_all_modules.end();
 	     ++iter) {
 	    if (*iter == ordered_active_modules.front()) {
 		ordered_all_modules.erase(iter);
 		ordered_active_modules.pop_front();
+		found = true;
 		break;
 	    }
 	}
-	XLOG_ASSERT(iter != ordered_all_modules.end());
+	XLOG_ASSERT(found == true);
     }
 
     debug_msg("Inactive Module Order: ");
@@ -304,9 +357,9 @@ MasterConfigTree::order_module_list(const set<string>& module_set,
 	if (mc->depends().empty()) {
 	    debug_msg("%s has no dependencies\n", (*iter).c_str());
 	    additional_done.insert(*iter);
-	    additional_modules.erase(iter);
 	    ordered_modules.push_back(*iter);
 	    satisfied.insert(*iter);
+	    additional_modules.erase(iter);
 	    continue;
 	}
 
@@ -403,13 +456,13 @@ MasterConfigTree::commit_changes_pass1(CallBack cb)
     /* Pass 1: check for errors without actually doing anything        */
     /*******************************************************************/
 
-    _task_manager.reset();
-    _task_manager.set_do_exec(false);
+    _task_manager->reset();
+    _task_manager->set_do_exec(false);
     _commit_cb = cb;
 
-    _root_node.initialize_commit();
+    master_root_node().initialize_commit();
 
-    if (_root_node.check_config_tree(result) == false) {
+    if (root_node().check_config_tree(result) == false) {
 	// Something went wrong - return the error message.
 	_commit_in_progress = false;
 	cb->dispatch(false, result);
@@ -427,10 +480,12 @@ MasterConfigTree::commit_changes_pass1(CallBack cb)
 	}
     }
 
-    if (_root_node.commit_changes(_task_manager,
-				  /* do_commit = */ false,
-				  0, 0,
-				  result)
+    bool needs_update = false;
+    if (master_root_node().commit_changes(*_task_manager,
+					  /* do_commit = */ false,
+					  0, 0,
+					  result,
+					  needs_update)
 	== false) {
 	// Something went wrong - return the error message.
 	_commit_in_progress = false;
@@ -438,12 +493,18 @@ MasterConfigTree::commit_changes_pass1(CallBack cb)
 	return;
     }
 
+#if 0
+    //
+    // XXX: don't shutdown any modules yet, because in this stage
+    // we are checking only for errors.
+    //
     for (iter = inactive_modules.begin();
 	 iter != inactive_modules.end(); ++iter) {
-	_task_manager.shutdown_module(*iter);
+	_task_manager->shutdown_module(*iter);
     }
+#endif // 0
 
-    _task_manager.run(callback(this, &MasterConfigTree::commit_pass1_done));
+    _task_manager->run(callback(this, &MasterConfigTree::commit_pass1_done));
 }
 
 void
@@ -471,8 +532,8 @@ MasterConfigTree::commit_changes_pass2()
 
     _commit_in_progress = true;
 
-    if (_root_node.check_config_tree(result) == false) {
-	XLOG_ERROR("Commit failed in deciding startups");
+    if (root_node().check_config_tree(result) == false) {
+	XLOG_ERROR("Configuration tree error: %s", result.c_str());
 	_commit_in_progress = false;
 	_commit_cb->dispatch(false, result);
 	return;
@@ -486,10 +547,10 @@ MasterConfigTree::commit_changes_pass2()
     list<string> inactive_modules = find_inactive_modules();
     list<string>::const_iterator iter;
 
-    _task_manager.reset();
-    _task_manager.set_do_exec(true);
+    _task_manager->reset();
+    _task_manager->set_do_exec(true);
 
-    _root_node.initialize_commit();
+    master_root_node().initialize_commit();
     // Sort the changes in order of module dependencies
     for (iter = changed_modules.begin();
 	 iter != changed_modules.end();
@@ -502,9 +563,12 @@ MasterConfigTree::commit_changes_pass2()
 	}
     }
 
-    if (!_root_node.commit_changes(_task_manager,
+    bool needs_update = false;
+    if (!master_root_node().commit_changes(*_task_manager,
 				   /* do_commit = */ true,
-				   0, 0, result)) {
+				   0, 0,
+				   result,
+				   needs_update)) {
 	// Abort the commit
 	XLOG_ERROR("Commit failed in config tree");
 	_commit_in_progress = false;
@@ -515,10 +579,10 @@ MasterConfigTree::commit_changes_pass2()
     for (iter = inactive_modules.begin();
 	 iter != inactive_modules.end();
 	 ++iter) {
-	_task_manager.shutdown_module(*iter);
+	_task_manager->shutdown_module(*iter);
     }
 
-    _task_manager.run(callback(this, &MasterConfigTree::commit_pass2_done));
+    _task_manager->run(callback(this, &MasterConfigTree::commit_pass2_done));
 }
 
 void
@@ -540,12 +604,12 @@ bool
 MasterConfigTree::check_commit_status(string& result)
 {
     debug_msg("check_commit_status\n");
-    bool success = _root_node.check_commit_status(result);
+    bool success = master_root_node().check_commit_status(result);
 
     if (success) {
 	// If the commit was successful, clear all the temporary state.
 	debug_msg("commit was successful, finalizing...\n");
-	_root_node.finalize_commit();
+	master_root_node().finalize_commit();
 	debug_msg("finalizing done\n");
     }
     return success;
@@ -556,7 +620,7 @@ MasterConfigTree::discard_changes()
 {
     debug_msg("##############################################################\n");
     debug_msg("MasterConfigTree::discard_changes\n");
-    string result = _root_node.discard_changes(0, 0);
+    string result = root_node().discard_changes(0, 0);
     debug_msg("##############################################################\n");
     return result;
 }
@@ -583,8 +647,8 @@ MasterConfigTree::mark_subtree_for_deletion(const list<string>& path_segments,
 void
 MasterConfigTree::delete_entire_config()
 {
-    _root_node.mark_subtree_for_deletion(0);
-    _root_node.undelete();
+    root_node().mark_subtree_for_deletion(0);
+    root_node().undelete();
 
     // We don't need a verification pass - this isn't allowed to fail.
     commit_changes_pass2();
@@ -630,8 +694,8 @@ MasterConfigTree::save_to_file(const string& filename,
     orig_gid = getgid();
     gid = grp->gr_gid;
     if (setegid(gid) < 0) {
-	errmsg = c_format("Failed to seteuid to group \"xorp\", gid %d\n",
-			  gid);
+	errmsg = c_format("Failed to seteuid to group \"xorp\", gid %u\n",
+			  XORP_UINT_CAST(gid));
 	return false;
     }
 
@@ -639,7 +703,8 @@ MasterConfigTree::save_to_file(const string& filename,
     uid_t orig_uid;
     orig_uid = getuid();
     if (seteuid(user_id) < 0) {
-	errmsg = c_format("Failed to seteuid to uid %d\n", user_id);
+	errmsg = c_format("Failed to seteuid to uid %u\n",
+			  XORP_UINT_CAST(user_id));
 	setegid(orig_gid);
 	return false;
     }
@@ -791,7 +856,7 @@ MasterConfigTree::run_save_hook(uid_t userid, const string& save_hook,
     argv.reserve(2);
     argv.push_back(save_hook);
     argv.push_back(filename);
-    _task_manager.shell_execute(userid, argv, 
+    _task_manager->shell_execute(userid, argv, 
            callback(this, &MasterConfigTree::save_hook_complete));
 }
 
@@ -829,8 +894,8 @@ MasterConfigTree::load_from_file(const string& filename, uid_t user_id,
     orig_gid = getgid();
     gid = grp->gr_gid;
     if (setegid(gid) < 0) {
-	errmsg = c_format("Failed to seteuid to group \"xorp\", gid %d\n",
-			  gid);
+	errmsg = c_format("Failed to seteuid to group \"xorp\", gid %u\n",
+			  XORP_UINT_CAST(gid));
 	return false;
     }
 
@@ -838,7 +903,8 @@ MasterConfigTree::load_from_file(const string& filename, uid_t user_id,
     uid_t orig_uid;
     orig_uid = getuid();
     if (seteuid(user_id) < 0) {
-	errmsg = c_format("Failed to seteuid to user_id %d\n", user_id);
+	errmsg = c_format("Failed to seteuid to user_id %u\n",
+			  XORP_UINT_CAST(user_id));
 	setegid(orig_gid);
 	return false;
     }
@@ -858,33 +924,35 @@ MasterConfigTree::load_from_file(const string& filename, uid_t user_id,
     // Test out parsing the config on a new config tree to detect any
     // parse errors before we reconfigure ourselves with the new config.
     //
-    ConfigTree new_tree(_template_tree, _verbose);
-    if (new_tree.parse(configuration, filename, errmsg) != true)
+    MasterConfigTree new_tree(_template_tree, _verbose);
+    if (new_tree.parse(configuration, filename, errmsg) != true) {
 	return false;
+    }
 
     //
     // Go through the config tree, and create nodes for any defaults
     // specified in the template tree that aren't already configured.
     //
     new_tree.add_default_children();
+    
 
     //
     // Ok, so the new config parses.  Now we need to figure out how it
     // differs from the existing config so we don't need to modify
     // anything that hasn't changed.
     //
-    ConfigTree delta_tree(_template_tree, _verbose);
-    ConfigTree deletion_tree(_template_tree, _verbose);
+    MasterConfigTree delta_tree(_template_tree, _verbose);
+    MasterConfigTree deletion_tree(_template_tree, _verbose);
     diff_configs(new_tree, delta_tree, deletion_tree);
 
     string response;
-    if (! _root_node.merge_deltas(user_id, delta_tree.const_root_node(),
+    if (! root_node().merge_deltas(user_id, delta_tree.const_root_node(),
 				  /* provisional change */ true, response)) {
 	discard_changes();
 	errmsg = response;
 	return false;
     }
-    if (! _root_node.merge_deletions(user_id, deletion_tree.const_root_node(),
+    if (! root_node().merge_deletions(user_id, deletion_tree.const_root_node(),
 				     /* provisional change */ true,
 				     response)) {
 	discard_changes();
@@ -904,15 +972,15 @@ MasterConfigTree::load_from_file(const string& filename, uid_t user_id,
 }
 
 void
-MasterConfigTree::diff_configs(const ConfigTree& new_tree,
-			       ConfigTree& delta_tree,
-			       ConfigTree& deletion_tree)
+MasterConfigTree::diff_configs(const MasterConfigTree& new_tree,
+			       MasterConfigTree& delta_tree,
+			       MasterConfigTree& deletion_tree)
 {
     //
     // Clone the existing config tree into the delta tree.
     // Clone the new config tree into the delta tree.
     //
-    deletion_tree = *((ConfigTree*)(this));
+    deletion_tree = *((MasterConfigTree*)(this));
     delta_tree = new_tree;
 
     deletion_tree.retain_different_nodes(new_tree, false);
@@ -945,7 +1013,7 @@ MasterConfigTree::module_config_start(const string& module_name,
 	result = "Module " + module_name + " is not registered with the TemplateTree, but is needed to satisfy a dependency\n";
 	return false;
     }
-    _task_manager.add_module(*cmd);
+    _task_manager->add_module(*cmd);
     return true;
 }
 
@@ -960,8 +1028,10 @@ MasterConfigTree::module_shutdown(const string& module_name,
 	result = "Module " + module_name + " is not registered with the TemplateTree, but is needed to satisfy a dependency\n";
 	return false;
     }
-    _task_manager.shutdown_module(*cmd);
+    _task_manager->shutdown_module(*cmd);
     return true;
 }
 
 #endif // 0
+
+
