@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/module_manager.cc,v 1.32 2004/06/10 22:41:52 hodson Exp $"
+#ident "$XORP: xorp/rtrmgr/module_manager.cc,v 1.42 2005/03/25 02:54:36 pavlin Exp $"
 
 #include <signal.h>
 #include <glob.h>
@@ -31,12 +31,39 @@
 #include "libxorp/debug.h"
 
 #include "module_manager.hh"
+#include "conf_tree_node.hh"
+#include "master_conf_tree.hh"
 #include "template_commands.hh"
+//#include "xrl_rtrmgr_interface.hh"
+#include "main_rtrmgr.hh"
 
 
 static map<pid_t, string> module_pids;
 static multimap<string, Module*> module_paths;
 
+
+int
+restart_module(Module* module)
+{
+    XLOG_INFO("Restarting module %s ...", module->name().c_str());
+    XorpCallback1<void, bool>::RefPtr run_cb
+	= callback(module, &Module::module_run_done);
+    if (module->run(true, run_cb) < 0) {
+	XLOG_ERROR("Failed to restart module %s.", module->name().c_str());
+	return (XORP_ERROR);
+    }
+
+    MasterConfigTree* mct = module->module_manager().master_config_tree();
+    ConfigTreeNode* ctn = mct->find_config_module(module->name());
+    if (ctn != NULL) {
+	XLOG_INFO("Configuring module %s ...", module->name().c_str());
+	ctn->mark_subtree_as_uncommitted();
+	mct->execute();
+    }
+
+    return (XORP_OK);
+}
+
 static void
 child_handler(int x)
 {
@@ -83,12 +110,22 @@ child_handler(int x)
 		module->abnormal_exit(child_wait_status);
 		module_paths.erase(path_iter);
 		path_iter = path_iter_next;
+		//
+		// Restart the module because it exited abnormally.
+		// It probably crashed.
+		//
+		if (module->status() == Module::MODULE_FAILED) {
+		    if (module->module_manager().do_restart())
+			restart_module(module);
+		}
 	    }
 	}
 	module_pids.erase(pid_iter);
     } else if (WIFSIGNALED(child_wait_status)) {
-	debug_msg("process was killed with signal %d\n",
-	       WTERMSIG(child_wait_status));
+	int sig = WTERMSIG(child_wait_status);
+
+	debug_msg("process was killed with signal %d\n", sig);
+
 	//
 	// Set the status for all appropriate modules, and at the same
 	// time delete the entries from the "module_paths" multimap.
@@ -104,6 +141,21 @@ child_handler(int x)
 	    module->killed();
 	    module_paths.erase(path_iter);
 	    path_iter = path_iter_next;
+	    //
+	    // XXX: if the child was killed by SIGTERM or SIGKILL, then
+	    // DO NOT restart it, because probably it was killed for a reason.
+	    //
+	    if (module->status() == Module::MODULE_FAILED) {
+		switch (sig) {
+		case SIGTERM:
+		case SIGKILL:
+		    break;
+		default:
+		    if (module->module_manager().do_restart())
+			restart_module(module);
+		    break;
+		}
+	    }
 	}
 	module_pids.erase(pid_iter);
     } else if (WIFSTOPPED(child_wait_status)) {
@@ -126,11 +178,9 @@ child_handler(int x)
 }
 
 Module::Module(ModuleManager& mmgr, const string& name, bool verbose)
-    : _mmgr(mmgr),
-      _name(name),
+    : GenericModule(name), _mmgr(mmgr),
       _userid(NO_SETUID_ON_EXEC),
       _pid(0),
-      _status(MODULE_NOT_STARTED),
       _do_exec(false),
       _verbose(verbose)
 {
@@ -163,7 +213,7 @@ Module::terminate(XorpCallback0<void>::RefPtr cb)
 	return;
     }
 
-    XLOG_INFO("Terminating module: %s\n", _name.c_str());
+    XLOG_INFO("Terminating module: %s", _name.c_str());
 
     //
     // Find whether this is the last module running within this process
@@ -209,8 +259,7 @@ Module::terminate(XorpCallback0<void>::RefPtr cb)
     //
     // We need to kill the process
     //
-    XLOG_INFO("Killing module: %s (pid = %d)\n",
-	      _name.c_str(), _pid);
+    XLOG_INFO("Killing module: %s (pid = %d)", _name.c_str(), _pid);
     new_status(MODULE_SHUTTING_DOWN);
     kill(_pid, SIGTERM);
 
@@ -284,8 +333,7 @@ Module::set_execution_path(const string& path)
 	// Add the XORP root path to the front
 	_path = _mmgr.xorp_root_dir() + "/" + path;
     }
-    XLOG_TRACE(_verbose, "New module: %s (%s)\n",
-	       _name.c_str(), _path.c_str());
+    XLOG_TRACE(_verbose, "New module: %s (%s)", _name.c_str(), _path.c_str());
 
     if (_path[0] != '/') {
 	// we're going to call glob, but don't want to allow wildcard expansion
@@ -353,7 +401,7 @@ Module::run(bool do_exec, XorpCallback1<void, bool>::RefPtr cb)
 {
     bool is_process_running = false;
 
-    XLOG_INFO("Running module: %s (%s)\n", _name.c_str(), _path.c_str());
+    XLOG_INFO("Running module: %s (%s)", _name.c_str(), _path.c_str());
 
     _do_exec = do_exec;
 
@@ -398,7 +446,8 @@ Module::run(bool do_exec, XorpCallback1<void, bool>::RefPtr cb)
 	    //set userid as required.
 	    if (_userid != NO_SETUID_ON_EXEC) {
 		if (setuid(_userid) != 0) {
-		    XLOG_ERROR("Failed to setuid(%d) on exec\n", _userid);
+		    XLOG_ERROR("Failed to setuid(%u) on exec",
+			       XORP_UINT_CAST(_userid));
 		    exit(1);
 		}
 	    }
@@ -407,7 +456,7 @@ Module::run(bool do_exec, XorpCallback1<void, bool>::RefPtr cb)
 	    setsid();
 	    if (_argv.empty()) {
 		if (execl(_expath.c_str(), _expath.c_str(), NULL) < 0) {
-		    XLOG_ERROR("Execution of %s failed\n", _expath.c_str());
+		    XLOG_ERROR("Execution of %s failed", _expath.c_str());
 		    exit(1);
 		}
 	    } else {
@@ -420,7 +469,7 @@ Module::run(bool do_exec, XorpCallback1<void, bool>::RefPtr cb)
 		argv[i] = NULL;
 		
 		if (execv(_expath.c_str(),const_cast<char*const*>(argv)) < 0) {
-		    XLOG_ERROR("Execution of %s failed\n", _expath.c_str());
+		    XLOG_ERROR("Execution of %s failed", _expath.c_str());
 		    exit(1);
 		}
 	    }
@@ -465,7 +514,7 @@ Module::module_run_done(bool success)
 void
 Module::set_stalled()
 {
-    XLOG_INFO("Module stalled: %s\n", _name.c_str());
+    XLOG_INFO("Module stalled: %s", _name.c_str());
 
     new_status(MODULE_STALLED);
 }
@@ -473,7 +522,7 @@ Module::set_stalled()
 void
 Module::normal_exit()
 {
-    XLOG_INFO("Module normal exit: %s\n", _name.c_str());
+    XLOG_INFO("Module normal exit: %s", _name.c_str());
 
     new_status(MODULE_NOT_STARTED);
     _pid = 0;
@@ -482,7 +531,7 @@ Module::normal_exit()
 void
 Module::abnormal_exit(int child_wait_status)
 {
-    XLOG_INFO("Module abnormal exit: %s status:%d\n",
+    XLOG_INFO("Module abnormal exit: %s status:%d",
 	      _name.c_str(), child_wait_status);
 
     new_status(MODULE_FAILED);
@@ -494,12 +543,12 @@ Module::killed()
 {
     if (_status == MODULE_SHUTTING_DOWN) {
 	// It may have been shutting down already, in which case this is OK.
-	XLOG_INFO("Module killed during shutdown: %s\n",
+	XLOG_INFO("Module killed during shutdown: %s",
 		  _name.c_str());
 	new_status(MODULE_NOT_STARTED);
     } else {
 	// We don't know why it was killed.
-	XLOG_INFO("Module abnormally killed: %s\n", _name.c_str());
+	XLOG_INFO("Module abnormally killed: %s", _name.c_str());
 	new_status(MODULE_FAILED);
     }
     _pid = 0;
@@ -528,11 +577,15 @@ Module::str() const
     return s;
 }
 
-ModuleManager::ModuleManager(EventLoop& eventloop, bool verbose,
-			     const string& xorp_root_dir)
-    : _eventloop(eventloop),
+ModuleManager::ModuleManager(EventLoop& eventloop, 
+			     Rtrmgr *rtrmgr, bool do_restart,
+			     bool verbose, const string& xorp_root_dir)
+    : GenericModuleManager(eventloop, verbose),
+      _do_restart(do_restart),
       _verbose(verbose),
-      _xorp_root_dir(xorp_root_dir)
+      _xorp_root_dir(xorp_root_dir),
+      _master_config_tree(NULL),
+      _rtrmgr(rtrmgr)
 {
 }
 
@@ -545,28 +598,18 @@ bool
 ModuleManager::new_module(const string& module_name, const string& path)
 {
     debug_msg("ModuleManager::new_module %s\n", module_name.c_str());
-
-    map<string, Module*>::iterator found_mod;
-    found_mod = _modules.find(module_name);
-    if (found_mod == _modules.end()) {
-	Module* new_module;
-	new_module = new Module(*this, module_name, _verbose);
-	_modules[module_name] = new_module;
-	if (new_module->set_execution_path(path) != XORP_OK)
-	    return false;
-	return true;
-    } else {
-	XLOG_TRACE(_verbose, "Module %s already exists\n",
-		   module_name.c_str());
-	return true;
-    }
+    Module* module = new Module(*this, module_name, _verbose);
+    store_new_module(module);
+    if (module->set_execution_path(path) != XORP_OK)
+	return false;
+    return true;
 }
 
 int
 ModuleManager::start_module(const string& module_name, bool do_exec,
 			    XorpCallback1<void, bool>::RefPtr cb)
 {
-    Module* module = find_module(module_name);
+    Module* module = (Module*)find_module(module_name);
 
     XLOG_ASSERT(module != NULL);
     return module->run(do_exec, cb);
@@ -576,7 +619,7 @@ int
 ModuleManager::kill_module(const string& module_name,
 			   XorpCallback0<void>::RefPtr cb)
 {
-    Module* module = find_module(module_name);
+    Module* module = (Module*)find_module(module_name);
 
     XLOG_ASSERT(module != NULL);
     module->terminate(cb);
@@ -586,7 +629,7 @@ ModuleManager::kill_module(const string& module_name,
 bool
 ModuleManager::module_is_running(const string& module_name) const
 {
-    const Module* module = const_find_module(module_name);
+    const Module* module = (const Module*)const_find_module(module_name);
 
     if (module == NULL)
 	return false;
@@ -596,57 +639,29 @@ ModuleManager::module_is_running(const string& module_name) const
 bool
 ModuleManager::module_has_started(const string& module_name) const
 {
-    const Module* module = const_find_module(module_name);
+    const Module* module = (const Module*)const_find_module(module_name);
 
     if (module == NULL)
 	return false;
     return (module->status() != Module::MODULE_NOT_STARTED);
 }
 
-Module*
-ModuleManager::find_module(const string& module_name)
-{
-    map<string, Module*>::iterator found;
-
-    found = _modules.find(module_name);
-    if (found == _modules.end()) {
-	debug_msg("ModuleManager: Failed to find module %s\n",
-		  module_name.c_str());
-	return NULL;
-    } else {
-	debug_msg("ModuleManager: Found module %s\n", module_name.c_str());
-	return found->second;
-    }
-}
-
-const Module*
-ModuleManager::const_find_module(const string& module_name) const
-{
-    map<string, Module*>::const_iterator found;
-
-    found = _modules.find(module_name);
-    if (found == _modules.end()) {
-	return NULL;
-    } else {
-	return found->second;
-    }
-}
-
-bool
-ModuleManager::module_exists(const string& module_name) const
-{
-    return _modules.find(module_name) != _modules.end();
-}
-
 void
 ModuleManager::module_status_changed(const string& module_name,
 				     Module::ModuleStatus old_status,
 				     Module::ModuleStatus new_status)
 {
-    // XXX eventually we'll tell someone that things changed
-    UNUSED(module_name);
     UNUSED(old_status);
-    UNUSED(new_status);
+    _rtrmgr->module_status_changed(module_name, new_status);
+}
+
+void
+ModuleManager::get_module_list(list <string>& modules)
+{
+    map<string, GenericModule *>::iterator i;
+    for (i = _modules.begin(); i != _modules.end(); i++) {
+	modules.push_back(i->first);
+    }
 }
 
 void
@@ -654,10 +669,10 @@ ModuleManager::shutdown()
 {
     debug_msg("ModuleManager::shutdown\n");
 
-    map<string, Module*>::iterator iter;
+    map<string, GenericModule*>::iterator iter;
     for (iter = _modules.begin(); iter != _modules.end(); ++iter) {
-	iter->second->terminate(callback(this,
-					 &ModuleManager::module_shutdown_cb));
+	Module *module = (Module*)iter->second;
+	module->terminate(callback(this, &ModuleManager::module_shutdown_cb));
     }
 }
 
@@ -672,7 +687,7 @@ ModuleManager::shutdown_complete()
 {
     bool complete = true;
 
-    map<string, Module*>::iterator iter, iter2;
+    map<string, GenericModule*>::iterator iter, iter2;
     for (iter = _modules.begin(); iter != _modules.end(); ) {
 	if (iter->second->status() == Module::MODULE_NOT_STARTED) {
 	    delete iter->second;
@@ -695,7 +710,7 @@ ModuleManager::shell_execute(uid_t userid, const vector<string>& argv,
     if (!new_module(argv[0], argv[0])) {
 	return XORP_ERROR;
     }
-    Module *module = find_module(argv[0]);
+    Module *module = (Module*)find_module(argv[0]);
     module->set_userid(userid);
     module->set_argv(argv);
     XorpCallback1<void, bool>::RefPtr run_cb 
