@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/op_commands.cc,v 1.32 2004/06/12 03:53:24 atanu Exp $"
+#ident "$XORP: xorp/rtrmgr/op_commands.cc,v 1.46 2005/03/25 02:54:36 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -22,66 +22,71 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/signal.h>
+#include <unistd.h>
 
 #include "rtrmgr_module.h"
 
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
+#include "libxorp/popen.hh"
 
 #include "cli.hh"
 #include "op_commands.hh"
-#include "popen.hh"
 #include "slave_conf_tree.hh"
 #include "template_tree.hh"
+#include "slave_module_manager.hh"
 #include "util.hh"
 #include "y.opcmd_tab.h"
 
 
+
 extern int init_opcmd_parser(const char *filename, OpCommandList *o);
 extern void parse_opcmd() throw (ParseError);
 extern int opcmderror(const char *s);
 
-OpInstance::OpInstance(EventLoop* eventloop, const string& executable_filename,
-		       const string& command_arguments,
-		       RouterCLI::OpModePrintCallback print_cb,
-		       RouterCLI::OpModeDoneCallback done_cb,
-		       OpCommand* op_command)
+OpInstance::OpInstance(EventLoop&			eventloop,
+		       const string&			executable_filename,
+		       const string&			command_arguments,
+		       RouterCLI::OpModePrintCallback	print_cb,
+		       RouterCLI::OpModeDoneCallback	done_cb,
+		       OpCommand*			op_command)
     : _executable_filename(executable_filename),
       _command_arguments(command_arguments),
       _op_command(op_command),
       _stdout_file_reader(NULL),
       _stderr_file_reader(NULL),
+      _stdout_stream(NULL),
+      _stderr_stream(NULL),
       _pid(0),
-      _out_stream(NULL),
-      _err_stream(NULL),
-      _error(false),
+      _is_error(false),
       _last_offset(0),
       _print_callback(print_cb),
       _done_callback(done_cb)
 {
     XLOG_ASSERT(op_command->is_executable());
-    debug_msg("\n");
 
-    memset(_outbuffer, 0, OP_BUF_SIZE);
-    memset(_errbuffer, 0, OP_BUF_SIZE);
+    memset(_stdout_buffer, 0, OP_BUF_SIZE);
+    memset(_stderr_buffer, 0, OP_BUF_SIZE);
 
     string execute = executable_filename + " " + command_arguments;
 
-    _pid = popen2(execute, _out_stream, _err_stream);
-    if (_out_stream == NULL) {
+    _pid = popen2(execute, _stdout_stream, _stderr_stream);
+    if (_stdout_stream == NULL) {
 	done_cb->dispatch(false, "Failed to execute command");
     }
 
-    _stdout_file_reader = new AsyncFileReader(*eventloop, fileno(_out_stream));
-    _stdout_file_reader->add_buffer((uint8_t*)_outbuffer, OP_BUF_SIZE,
+    _stdout_file_reader = new AsyncFileReader(eventloop,
+					      fileno(_stdout_stream));
+    _stdout_file_reader->add_buffer(_stdout_buffer, OP_BUF_SIZE,
 				    callback(this, &OpInstance::append_data));
     if (! _stdout_file_reader->start()) {
 	done_cb->dispatch(false, "Failed to execute command");
     }
 
-    _stderr_file_reader = new AsyncFileReader(*eventloop, fileno(_err_stream));
-    _stderr_file_reader->add_buffer((uint8_t*)_errbuffer, OP_BUF_SIZE,
+    _stderr_file_reader = new AsyncFileReader(eventloop,
+					      fileno(_stderr_stream));
+    _stderr_file_reader->add_buffer(_stderr_buffer, OP_BUF_SIZE,
 				    callback(this, &OpInstance::append_data));
     if (! _stderr_file_reader->start()) {
 	done_cb->dispatch(false, "Failed to execute command");
@@ -94,47 +99,45 @@ OpInstance::~OpInstance()
 }
 
 void
-OpInstance::append_data(AsyncFileOperator::Event e,
+OpInstance::append_data(AsyncFileOperator::Event event,
 			const uint8_t* buffer,
 			size_t /* buffer_bytes */,
 			size_t offset)
 {
-    debug_msg("\n");
-
-    if ((const char *)buffer == _errbuffer) {
-	if (_error == false) {
+    if (buffer == _stderr_buffer) {
+	if (_is_error == false) {
 	    // We hadn't previously seen any error output
-	    if (e == AsyncFileOperator::END_OF_FILE) {
+	    if (event == AsyncFileOperator::END_OF_FILE) {
 		// We just got EOF on stderr - ignore this and wait for
 		// EOF on stdout
 		return;
 	    }
-	    _error = true;
+	    _is_error = true;
 	    // Reset for reading error response
 	    _error_msg = "";
 	    _last_offset = 0;
 	}
     } else {
-	if (_error == true) {
+	if (_is_error == true) {
 	    // Discard further output
 	    return;
 	}
     }
 
-    if ((e == AsyncFileOperator::DATA)
-	|| (e == AsyncFileOperator::END_OF_FILE)) {
+    if ((event == AsyncFileOperator::DATA)
+	|| (event == AsyncFileOperator::END_OF_FILE)) {
 	XLOG_ASSERT(offset >= _last_offset);
 	if (offset == _last_offset) {
-	    XLOG_ASSERT(e == AsyncFileOperator::END_OF_FILE);
-	    done(!_error);
+	    XLOG_ASSERT(event == AsyncFileOperator::END_OF_FILE);
+	    done(!_is_error);
 	    return;
 	}
 
 	if (offset != _last_offset) {
 	    const char* p   = (const char*)buffer + _last_offset;
 	    size_t 	len = offset - _last_offset;
-	    debug_msg("len = %u\n", static_cast<uint32_t>(len));
-	    if (!_error) {
+	    debug_msg("len = %u\n", XORP_UINT_CAST(len));
+	    if (!_is_error) {
 		_print_callback->dispatch(string(p, len));
 	    } else {
 		_error_msg.append(p, p + len);
@@ -142,24 +145,22 @@ OpInstance::append_data(AsyncFileOperator::Event e,
 	    _last_offset = offset;
 	}
 
-	if (e == AsyncFileOperator::END_OF_FILE) {
-	    done(!_error);
+	if (event == AsyncFileOperator::END_OF_FILE) {
+	    done(!_is_error);
 	    return;
 	}
 
 	if (offset == OP_BUF_SIZE) {
 	    // The buffer is exhausted
 	    _last_offset = 0;
-	    if (_error) {
-		memset(_errbuffer, 0, OP_BUF_SIZE);
-		_stderr_file_reader->add_buffer((uint8_t*)_errbuffer,
-						OP_BUF_SIZE,
+	    if (_is_error) {
+		memset(_stderr_buffer, 0, OP_BUF_SIZE);
+		_stderr_file_reader->add_buffer(_stderr_buffer, OP_BUF_SIZE,
 				callback(this, &OpInstance::append_data));
 		_stderr_file_reader->start();
 	    } else {
-		memset(_outbuffer, 0, OP_BUF_SIZE);
-		_stdout_file_reader->add_buffer((uint8_t*)_outbuffer,
-						OP_BUF_SIZE,
+		memset(_stdout_buffer, 0, OP_BUF_SIZE);
+		_stdout_file_reader->add_buffer(_stdout_buffer, OP_BUF_SIZE,
 				callback(this, &OpInstance::append_data));
 		_stdout_file_reader->start();
 	    }
@@ -175,8 +176,6 @@ OpInstance::append_data(AsyncFileOperator::Event e,
 void
 OpInstance::done(bool success)
 {
-    debug_msg("\n");
-
     _op_command->remove_instance(this);
     _done_callback->dispatch(success, _error_msg);
     // The callback will delete us. Don't do anything more in this method.
@@ -196,8 +195,6 @@ OpInstance::operator<(const OpInstance& them) const
 void
 OpInstance::terminate()
 {
-    debug_msg("\n");
-
     // Kill the process group
     if (0 != _pid)
 	killpg(_pid, SIGTERM);
@@ -210,14 +207,15 @@ OpInstance::terminate()
 	delete _stderr_file_reader;
 	_stderr_file_reader = NULL;
     }
-    if (_out_stream != NULL) {
-	pclose2(_out_stream);
-	_out_stream = NULL;
+    if (_stdout_stream != NULL) {
+	pclose2(_stdout_stream);
+	_stdout_stream = NULL;
     }
-//     if (_err_stream != NULL) {
-// 	fclose(_err_stream);
-// 	_err_stream = NULL;
-//     }
+    //
+    // XXX: don't call pclose2(_stderr_stream), because pclose2(_stdout_stream)
+    // automatically takes care of the _stderr_stream as well.
+    //
+    _stderr_stream = NULL;
 }
 
 OpCommand::OpCommand(const list<string>& command_parts)
@@ -306,7 +304,7 @@ OpCommand::select_positional_argument(const list<string>& arguments,
 			     "expected values must be in interval "
 			     "[0, %u]",
 			     position.c_str(),
-			     static_cast<uint32_t>(arguments.size()));
+			     XORP_UINT_CAST(arguments.size()));
 	return string("");
     }
 
@@ -336,7 +334,7 @@ OpCommand::select_positional_argument(const list<string>& arguments,
 }
 
 OpInstance *
-OpCommand::execute(EventLoop* eventloop, const list<string>& command_line,
+OpCommand::execute(EventLoop& eventloop, const list<string>& command_line,
 		   RouterCLI::OpModePrintCallback print_cb,
 		   RouterCLI::OpModeDoneCallback done_cb)
 {
@@ -393,7 +391,7 @@ OpCommand::execute(EventLoop* eventloop, const list<string>& command_line,
 
 bool
 OpCommand::command_match(const list<string>& path_parts,
-			 SlaveConfigTree* conf_tree,
+			 SlaveConfigTree* slave_config_tree,
 			 bool exact_match) const
 {
     list<string>::const_iterator them, us;
@@ -420,7 +418,7 @@ OpCommand::command_match(const list<string>& path_parts,
 	if ((*us)[0] == '$') {
 	    // Matching against a varname
 	    list<string> varmatches;
-	    conf_tree->expand_varname_to_matchlist(*us, varmatches);
+	    slave_config_tree->expand_varname_to_matchlist(*us, varmatches);
 	    list<string>::const_iterator vi;
 	    bool ok = false;
 	    for (vi = varmatches.begin(); vi != varmatches.end(); ++vi) {
@@ -431,6 +429,8 @@ OpCommand::command_match(const list<string>& path_parts,
 	    }
 	    if (ok == false)
 		return false;
+	} else if ((*us)[0] == '<') {
+	    /* any single word matches a wildcard */
 	} else if (*them != *us) {
 	    return false;
 	}
@@ -458,7 +458,7 @@ OpCommand::command_match(const list<string>& path_parts,
 }
 
 void
-OpCommand::get_matches(size_t wordnum, SlaveConfigTree* conf_tree,
+OpCommand::get_matches(size_t wordnum, SlaveConfigTree* slave_config_tree,
 		       map<string, string>& return_matches,
 		       bool& is_executable,
 		       bool& can_pipe) const
@@ -486,7 +486,7 @@ OpCommand::get_matches(size_t wordnum, SlaveConfigTree* conf_tree,
 	    XLOG_ASSERT(match[1] == '(');
 	    XLOG_ASSERT(match[match.size() - 1] == ')');
 	    list<string> var_matches;
-	    conf_tree->expand_varname_to_matchlist(match, var_matches);
+	    slave_config_tree->expand_varname_to_matchlist(match, var_matches);
 	    list<string>::const_iterator vi;
 	    for (vi = var_matches.begin(); vi != var_matches.end(); ++vi) {
 		return_matches.insert(make_pair(*vi, _help_string));
@@ -513,7 +513,9 @@ OpCommand::remove_instance(OpInstance* instance)
 }
 
 OpCommandList::OpCommandList(const string& config_template_dir,
-			     const TemplateTree* tt) throw (InitError)
+			     const TemplateTree* tt,
+			     SlaveModuleManager& mmgr) throw (InitError)
+    : _mmgr(mmgr)
 {
     list<string> files;
     string errmsg;
@@ -607,21 +609,22 @@ OpCommandList::command_match(const list<string>& command_parts,
 {
     list<OpCommand*>::const_iterator iter;
     for (iter = _op_commands.begin(); iter != _op_commands.end(); ++iter) {
-	if ((*iter)->command_match(command_parts, _conf_tree, exact_match))
+	if ((*iter)->command_match(command_parts, _slave_config_tree,
+				   exact_match))
 	    return true;
     }
     return false;
 }
 
-OpInstance *
-OpCommandList::execute(EventLoop *eventloop, const list<string>& command_parts,
+OpInstance*
+OpCommandList::execute(EventLoop& eventloop, const list<string>& command_parts,
 		       RouterCLI::OpModePrintCallback print_cb,
 		       RouterCLI::OpModeDoneCallback done_cb) const
 {
     list<OpCommand*>::const_iterator iter;
     for (iter = _op_commands.begin(); iter != _op_commands.end(); ++iter) {
 	// Find the right command
-	if ((*iter)->command_match(command_parts, _conf_tree, true)) {
+	if ((*iter)->command_match(command_parts, _slave_config_tree, true)) {
 	    // Execute it
 	    return (*iter)->execute(eventloop,
 				    command_parts, print_cb, done_cb);
@@ -656,9 +659,9 @@ OpCommandList::find_executable_filename(const string& command_filename,
 
     if (command_filename[0] == '/') {
 	// Absolute path name
-	if ((stat(command_filename.c_str(), &statbuf) == 0)
-	    && (statbuf.st_mode & S_IFREG > 0)
-	    && (statbuf.st_mode & S_IXUSR > 0)) {
+	if (stat(command_filename.c_str(), &statbuf) == 0 &&
+	    //access(command_filename.c_str(), X_OK) == 0 &&
+	    S_ISREG(statbuf.st_mode)) {
 	    executable_filename = command_filename;
 	    return true;
 	}
@@ -678,11 +681,12 @@ OpCommandList::find_executable_filename(const string& command_filename,
 	path.splice(path.end(), l2);
     }
 
+    // Search each path component
     while (!path.empty()) {
 	string full_path_executable = path.front() + "/" + command_filename;
-	if ((stat(full_path_executable.c_str(), &statbuf) == 0)
-	    && (statbuf.st_mode & S_IFREG > 0)
-	    && (statbuf.st_mode & S_IXUSR > 0)) {
+	if (stat(full_path_executable.c_str(), &statbuf) == 0 &&
+	    //access(command_filename.c_str(), X_OK) == 0 &&
+	    S_ISREG(statbuf.st_mode)) {
 	    executable_filename = full_path_executable;
 	    return true;
 	}
@@ -736,11 +740,21 @@ OpCommandList::childlist(const string& path, bool& is_executable,
     list<OpCommand*>::const_iterator iter;
     for (iter = _op_commands.begin(); iter != _op_commands.end(); ++iter) {
 	const OpCommand* op_command = *iter;
-	if (op_command->command_match(path_parts, _conf_tree, false)) {
+	if (op_command->command_match(path_parts, _slave_config_tree, false)) {
+	    //
+	    // XXX: If the module has not been started, then don't add its
+	    // commands. However, add all commands that are not associated
+	    // with any module.
+	    //
+	    if ((_mmgr.module_is_active(op_command->module()) == false)
+		&& (! op_command->module().empty())) {
+		continue;
+	    }
+
 	    map<string, string> matches;
 	    bool tmp_is_executable;
 	    bool tmp_can_pipe;
-	    op_command->get_matches(path_parts.size(), _conf_tree,
+	    op_command->get_matches(path_parts.size(), _slave_config_tree,
 				    matches, tmp_is_executable, tmp_can_pipe);
 	    if (tmp_is_executable)
 		is_executable = true;
