@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,8 +12,10 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/task.cc,v 1.39 2004/06/11 00:53:46 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/task.cc,v 1.47 2005/03/25 02:54:38 pavlin Exp $"
 
+// #define DEBUG_LOGGING
+// #define DEBUG_PRINT_FUNCTION_NAME
 
 #include "rtrmgr_module.h"
 
@@ -21,7 +23,7 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
-#include "conf_tree.hh"
+#include "master_conf_tree.hh"
 #include "module_command.hh"
 #include "module_manager.hh"
 #include "task.hh"
@@ -90,7 +92,7 @@ XrlStatusValidation::validate(CallBack cb)
 	Xrl* xrl = NULL;
 	do {
 	    // Try to expand using the configuration tree
-	    const ConfigTreeNode* ctn;
+	    const MasterConfigTreeNode* ctn;
 	    ctn = _task_manager.config_tree().find_config_module(_module_name);
 	    if (ctn != NULL) {
 		if (_xrl_action.expand_xrl_variables(*ctn, xrl_request,
@@ -164,40 +166,46 @@ XrlStatusValidation::dummy_response()
 void
 XrlStatusValidation::xrl_done(const XrlError& e, XrlArgs* xrl_args)
 {
-    if (e == XrlError::OKAY()) {
+    switch (e.error_code()) {
+    case OKAY:
 	try {
 	    ProcessStatus status;
 	    status = static_cast<ProcessStatus>(xrl_args->get_uint32("status"));
 	    string reason(xrl_args->get_string("reason"));
 	    handle_status_response(status, reason);
-	    return;
 	} catch (XrlArgs::XrlAtomNotFound) {
 	    // Not a valid response
 	    XLOG_ERROR("Bad XRL response to get_status");
 	    _cb->dispatch(false);
-	    return;
 	}
-    } else if (e == XrlError::NO_FINDER()) {
-	// We're in trouble now! This shouldn't be able to happen.
-	XLOG_UNREACHABLE();
-    } else if (e == XrlError::NO_SUCH_METHOD()) {
+	break;
+
+    case BAD_ARGS:
+    case COMMAND_FAILED:
+    case NO_SUCH_METHOD:
+	//
 	// The template file must have been wrong - the target doesn't
 	// support the common interface.
+	// Just return true and hope everything's OK
+	//
 	XLOG_ERROR("Module %s doesn't support status validation",
 		   _module_name.c_str());
-
-	// Just return true and hope everything's OK
 	_cb->dispatch(true);
-    } else if ((e == XrlError::RESOLVE_FAILED())
-	       || (e == XrlError::SEND_FAILED()))  {
+	break;
+
+    case RESOLVE_FAILED:
+    case SEND_FAILED:
+    case REPLY_TIMED_OUT:
+    case SEND_FAILED_TRANSIENT:
 	//
+	// REPLY_TIMED_OUT => We did not yet receive a reply.
 	// RESOLVE_FAILED => It's not yet registered with the finder -
 	// retry after a short delay.
+	// SEND_FAILED and SEND_FAILED_TRANSIENT=> ???  We're dealing with
+	// startup conditions here, so give the problem a chance to resolve
+	// itself.
 	//
-
-	//
-	// SEND_FAILED => ???  We're dealing with startup conditions
-	// here, so give the problem a chance to resolve itself.
+	// TODO: Make retries and delay configurable (no magic numbers).
 	//
 	_retries++;
 	if (_retries > MAX_STATUS_RETRIES) {
@@ -205,10 +213,18 @@ XrlStatusValidation::xrl_done(const XrlError& e, XrlArgs* xrl_args)
 	}
 	_retry_timer = eventloop().new_oneoff_after_ms(1000,
 			callback(this, &XrlStatusValidation::validate, _cb));
-    } else {
+	break;
+
+    case INTERNAL_ERROR:
 	XLOG_ERROR("Error while validating module %s", _module_name.c_str());
 	XLOG_WARNING("Continuing anyway, cross your fingers...");
 	_cb->dispatch(true);
+	break;
+
+    case NO_FINDER:
+	// We're in trouble now! This shouldn't be able to happen.
+	XLOG_UNREACHABLE();
+	break;
     }
 }
 
@@ -354,7 +370,7 @@ StatusShutdownValidation::handle_status_response(ProcessStatus status,
     case PROC_NOT_READY:
     case PROC_READY:
     case PROC_FAILED:
-	// The process should be in SHUTDOWN state, or it should not
+	// The process should be in PROC_SHUTDOWN state, or it should not
 	// be able to respond to Xrls because it's in NULL state.
 	_cb->dispatch(false);
 	return;
@@ -375,40 +391,55 @@ StatusShutdownValidation::handle_status_response(ProcessStatus status,
 void
 StatusShutdownValidation::xrl_done(const XrlError& e, XrlArgs* xrl_args)
 {
-    if (e == XrlError::OKAY()) {
+    switch (e.error_code()) {
+    case OKAY:
 	try {
 	    ProcessStatus status;
 	    status = static_cast<ProcessStatus>(xrl_args->get_uint32("status"));
 	    string reason(xrl_args->get_string("reason"));
 	    handle_status_response(status, reason);
-	    return;
 	} catch (XrlArgs::XrlAtomNotFound) {
 	    // Not a valid response
 	    XLOG_ERROR("Bad XRL response to get_status");
 	    _cb->dispatch(false);
-	    return;
 	}
-    } else if (e == XrlError::NO_FINDER()) {
-	// We're in trouble now! This shouldn't be able to happen.
-	XLOG_UNREACHABLE();
-    } else if (e == XrlError::NO_SUCH_METHOD()) {
+	break;
+
+    case BAD_ARGS:
+    case NO_SUCH_METHOD:
+    case COMMAND_FAILED:
+	//
 	// The template file must have been wrong - the target doesn't
 	// support the common interface.
+	// Return false, and we can shut it down using kill.
+	//
 	XLOG_ERROR("Module %s doesn't support shutdown validation",
 		   _module_name.c_str());
-	// Return false, and we can shut it down using kill.
 	_cb->dispatch(false);
-    } else if (e == XrlError::RESOLVE_FAILED()) {
-	// The process appears to have shutdown correctly.
+	break;
+
+    case RESOLVE_FAILED:
+    case REPLY_TIMED_OUT:
+    case SEND_FAILED:
+    case SEND_FAILED_TRANSIENT:
+	//
+	// The process appears to have shutdown correctly or the process
+	// is probably gone.
+	//
 	_cb->dispatch(true);
-    } else if (e == XrlError::SEND_FAILED())  {
-	// The process is probably gone.
-	_cb->dispatch(true);
-    } else {
+	break;
+
+    case INTERNAL_ERROR:
+	// Return false, and we can shut it down using kill.
 	XLOG_ERROR("Error while shutdown validation of module %s",
 		   _module_name.c_str());
-	// Return false, and we can shut it down using kill.
 	_cb->dispatch(false);
+	break;
+
+    case NO_FINDER:
+	// We're in trouble now! This shouldn't be able to happen.
+	XLOG_UNREACHABLE();
+	break;
     }
 }
 
@@ -417,7 +448,8 @@ StatusShutdownValidation::xrl_done(const XrlError& e, XrlArgs* xrl_args)
 // Startup implementation
 
 Startup::Startup(const string& module_name, bool verbose)
-    : _module_name(module_name, verbose)
+    : _module_name(module_name),
+      _verbose(verbose)
 {
 }
 
@@ -449,7 +481,7 @@ XrlStartup::startup(CallBack cb)
 	Xrl* xrl = NULL;
 	do {
 	    // Try to expand using the configuration tree
-	    const ConfigTreeNode* ctn;
+	    const MasterConfigTreeNode* ctn;
 	    ctn = _task_manager.config_tree().find_config_module(_module_name);
 	    if (ctn != NULL) {
 		if (_xrl_action.expand_xrl_variables(*ctn, xrl_request,
@@ -573,7 +605,7 @@ XrlShutdown::shutdown(CallBack cb)
 	Xrl* xrl = NULL;
 	do {
 	    // Try to expand using the configuration tree
-	    const ConfigTreeNode* ctn;
+	    const MasterConfigTreeNode* ctn;
 	    ctn = _task_manager.config_tree().find_config_module(_module_name);
 	    if (ctn != NULL) {
 		if (_xrl_action.expand_xrl_variables(*ctn, xrl_request,
@@ -646,29 +678,50 @@ XrlShutdown::dummy_response()
 void
 XrlShutdown::shutdown_done(const XrlError& err, XrlArgs* xrl_args)
 {
-    UNUSED(xrl_args);
-
-    if ((err == XrlError::OKAY())
-	|| (err == XrlError::RESOLVE_FAILED())
-	|| (err == XrlError::SEND_FAILED())) {
+    switch (err.error_code()) {
+    case OKAY:
+    case RESOLVE_FAILED:
+    case SEND_FAILED:
+    case REPLY_TIMED_OUT:
+    case SEND_FAILED_TRANSIENT:
 	// Success - either it said it would shutdown, or it's already gone.
 	_cb->dispatch(true);
-    } else {
+	break;
+
+    case BAD_ARGS:
+    case COMMAND_FAILED:
+    case NO_SUCH_METHOD:
+    case INTERNAL_ERROR:	// TODO: should be XLOG_UNREACHABLE()?
 	// We may have to kill it.
 	_cb->dispatch(false);
+	break;
+
+    case NO_FINDER:
+	XLOG_UNREACHABLE();
+	break;
     }
+
+    UNUSED(xrl_args);
 }
 
 
 // ----------------------------------------------------------------------------
 // TaskXrlItem implementation
 
+const uint32_t	TaskXrlItem::DEFAULT_RESEND_COUNT = 10;
+const int	TaskXrlItem::DEFAULT_RESEND_DELAY_MS = 1000;
+
 TaskXrlItem::TaskXrlItem(const UnexpandedXrl& uxrl,
-			 const XrlRouter::XrlCallback& cb, Task& task)
+			 const XrlRouter::XrlCallback& cb,
+			 Task& task,
+			 uint32_t xrl_resend_count,
+			 int xrl_resend_delay_ms)
     : _unexpanded_xrl(uxrl),
       _xrl_callback(cb),
       _task(task),
-      _resend_counter(0),
+      _xrl_resend_count_limit(xrl_resend_count),
+      _xrl_resend_count(_xrl_resend_count_limit),
+      _xrl_resend_delay_ms(xrl_resend_delay_ms),
       _verbose(task.verbose())
 {
 }
@@ -677,7 +730,9 @@ TaskXrlItem::TaskXrlItem(const TaskXrlItem& them)
     : _unexpanded_xrl(them._unexpanded_xrl),
       _xrl_callback(them._xrl_callback),
       _task(them._task),
-      _resend_counter(0),
+      _xrl_resend_count_limit(them._xrl_resend_count_limit),
+      _xrl_resend_count(_xrl_resend_count_limit),
+      _xrl_resend_delay_ms(them._xrl_resend_delay_ms),
       _verbose(them._verbose)
 {
 }
@@ -702,7 +757,7 @@ TaskXrlItem::execute(string& errmsg)
     // For debugging purposes, record what we were doing
     errmsg = _unexpanded_xrl.str();
 
-    _resend_counter = 0;
+    _xrl_resend_count = _xrl_resend_count_limit;
     _task.xorp_client().send_now(*xrl,
 				 callback(this, &TaskXrlItem::execute_done),
 				 xrl_return_spec,
@@ -749,69 +804,64 @@ TaskXrlItem::execute_done(const XrlError& err, XrlArgs* xrl_args)
 
     debug_msg("TaskXrlItem::execute_done\n");
 
-    if (err != XrlError::OKAY()) {
-	// XXX: handle XRL errors here
-	if ((err == XrlError::NO_FINDER())
-	    || (err == XrlError::SEND_FAILED())
-	    || (err == XrlError::NO_SUCH_METHOD())) {
-	    //
+    switch (err.error_code()) {
+    case OKAY:
+	break;
+
+    case BAD_ARGS:
+    case COMMAND_FAILED:
+	// Non-fatal error for the target - typically this is
+	// because the user entered bad information - we can't
+	// change the target configuration to what the user
+	// specified, but this doesn't mean the target is fatally
+	// wounded.
+	fatal = false;
+	break;
+
+    case NO_FINDER:
+    case SEND_FAILED:
+	// The error was a fatal one for the target - we now
+	// consider the target to be fatally wounded.
+	XLOG_ERROR("%s", err.str().c_str());
+	fatal = true;
+	break;
+
+    case REPLY_TIMED_OUT:
+    case RESOLVE_FAILED:
+    case SEND_FAILED_TRANSIENT:
+	// REPLY_TIMED_OUT shouldn't happen on a reliable
+	// transport, but at this point we don't know for sure
+	// which Xrls are reliable and while are unreliable, so
+	// best handle this as if it were packet loss.
+	//
+	// RESOLVE_FAILED shouldn't happen if the startup
+	// validation has done it's job correctly, but just in case
+	// we'll be lenient and give it ten more seconds to be
+	// functioning before we declare it dead.
+	if (--_xrl_resend_count > 0) {
+	    // Re-send the Xrl after a short delay.
+	    _xrl_resend_timer = _task.eventloop().new_oneoff_after_ms(
+		_xrl_resend_delay_ms,
+		callback(this, &TaskXrlItem::resend));
+	    return;
+	} else {
+	    // Give up.
 	    // The error was a fatal one for the target - we now
 	    // consider the target to be fatally wounded.
-	    //
 	    XLOG_ERROR("%s", err.str().c_str());
 	    fatal = true;
-	} else if ((err == XrlError::COMMAND_FAILED())
-		   || (err == XrlError::BAD_ARGS())) {
-	    //
-	    // Non-fatal error for the target - typically this is
-	    // because the user entered bad information - we can't
-	    // change the target configuration to what the user
-	    // specified, but this doesn't mean the target is fatally
-	    // wounded.
-	    //
-	    fatal = false;
-	} else if ((err == XrlError::REPLY_TIMED_OUT())
-		   || (err == XrlError::RESOLVE_FAILED())) {
-	    //
-	    // REPLY_TIMED_OUT shouldn't happen on a reliable
-	    // transport, but at this point we don't know for sure
-	    // which Xrls are reliable and while are unreliable, so
-	    // best handle this as if it were packet loss.
-	    //
-	    // RESOLVE_FAILED shouldn't happen if the startup
-	    // validation has done it's job correctly, but just in case
-	    // we'll be lenient and give it ten more seconds to be
-	    // functioning before we declare it dead.
-	    //
-	    // TODO: get rid of the hard-coded value of "10" below.
-	    if (_resend_counter > 10) {
-		// Give up.
-		// The error was a fatal one for the target - we now
-		// consider the target to be fatally wounded.
-		XLOG_ERROR("%s", err.str().c_str());
-		fatal = true;
-	    } else {
-		// Re-send the Xrl after a short delay.
-		_resend_counter++;
-		// TODO: get rid of the hard-coded value of "1000" below.
-		_resend_timer = _task.eventloop().new_oneoff_after_ms(1000,
-					callback(this, &TaskXrlItem::resend));
-		return;
-	    }
-	} else if (err == XrlError::INTERNAL_ERROR()) {
-	    //
-	    // Something bad happened but it's not clear what.  Don't
-	    // consider these to be fatal errors, but they may well
-	    // prove to be so.  XXX revisit this issue when we've more
-	    // experience with XRL errors.
-	    //
-	    XLOG_ERROR("%s", err.str().c_str());
-	    fatal = false;
-	} else {
-	    // We intended to explicitly handle all errors above, so if
-	    // we got here there's an error type we didn't know about.
-	    XLOG_UNREACHABLE();
 	}
+	break;
+
+    case INTERNAL_ERROR:
+    case NO_SUCH_METHOD:
+	// Something bad happened but it's not clear what.  Don't
+	// consider these to be fatal errors, but they may well
+	// prove to be so.  XXX revisit this issue when we've more
+	// experience with XRL errors.
+	XLOG_ERROR("%s", err.str().c_str());
+	fatal = false;
+	break;
     }
 
     if (!_xrl_callback.is_empty())
@@ -868,6 +918,7 @@ Task::start_module(const string& module_name,
 		   Validation* config_validation,
 		   Startup* startup)
 {
+    XLOG_ASSERT(!module_name.empty());
     XLOG_ASSERT(_start_module == false);
     XLOG_ASSERT(_stop_module == false);
     XLOG_ASSERT(_startup_validation == NULL);
@@ -885,6 +936,7 @@ void
 Task::shutdown_module(const string& module_name, Validation* validation,
 		      Shutdown* shutdown)
 {
+    XLOG_ASSERT(!module_name.empty());
     XLOG_ASSERT(_start_module == false);
     XLOG_ASSERT(_stop_module == false);
     XLOG_ASSERT(_shutdown_validation == NULL);
@@ -911,7 +963,7 @@ Task::set_ready_validation(Validation* validation)
 void
 Task::run(CallBack cb)
 {
-    debug_msg("Task::run %s\n", _module_name.c_str());
+    debug_msg("Task::run (%s)\n", _module_name.c_str());
 
     _task_complete_cb = cb;
     step1_start();
@@ -1073,6 +1125,8 @@ Task::step4_wait()
 void
 Task::step4_done(bool success)
 {
+    debug_msg("step4_done (%s)\n", _module_name.c_str());
+
     if (success) {
 	step5_stop();
     } else {
@@ -1168,8 +1222,8 @@ Task::task_fail(string errmsg, bool fatal)
 {
     debug_msg("%s\n", errmsg.c_str());
 
-    if (fatal) {
-	XLOG_ERROR("Shutting down fatally wounded process %s",
+    if (fatal && !_module_name.empty()) {
+	XLOG_ERROR("Shutting down fatally wounded process (%s)",
 		   _module_name.c_str());
 	_taskmgr.kill_process(_module_name);
     }
@@ -1198,7 +1252,7 @@ Task::eventloop() const
 // ----------------------------------------------------------------------------
 // TaskManager implementation
 
-TaskManager::TaskManager(ConfigTree& config_tree, ModuleManager& mmgr,
+TaskManager::TaskManager(MasterConfigTree& config_tree, ModuleManager& mmgr,
 			 XorpClient& xclient, bool global_do_exec,
 			 bool verbose)
     : _config_tree(config_tree),
