@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/template_commands.cc,v 1.46 2004/06/10 22:41:54 hodson Exp $"
+#ident "$XORP: xorp/rtrmgr/template_commands.cc,v 1.50 2005/03/25 02:54:38 pavlin Exp $"
 
 
 #include "rtrmgr_module.h"
@@ -31,6 +31,7 @@
 #include "util.hh"
 #include "xrldb.hh"
 
+#include "libxipc/xrl_atom_encoding.hh"
 
 Action::Action(TemplateTreeNode& template_tree_node,
 	       const list<string>& action)
@@ -194,7 +195,7 @@ XrlAction::XrlAction(TemplateTreeNode& template_tree_node,
 		segment = " " + segment.substr(1, segment.size() - 1);
 	}
 	string::size_type start = segment.find("->");
-	debug_msg("start=%u\n", (uint32_t)start);
+	debug_msg("start=%u\n", XORP_UINT_CAST(start));
 	if (start != string::npos) {
 	    debug_msg("found return spec\n");
 	    string::size_type orig_start = orig_segment.find("->");
@@ -420,7 +421,7 @@ XrlAction::check_xrl_is_valid(const list<string>& action, const XRLdb& xrldb,
 }
 
 int
-XrlAction::execute(const ConfigTreeNode& ctn,
+XrlAction::execute(const MasterConfigTreeNode& ctn,
 		   TaskManager& task_manager,
 		   XrlRouter::XrlCallback cb) const
 {
@@ -524,6 +525,7 @@ XrlAction::expand_xrl_variables(const TreeNode& tn,
 	if (segment[0] == '`') {
 	    expand_done = tn.expand_expression(segment, expanded_var);
 	    if (expand_done) {
+	    	expanded_var = xrlatom_encode_value(expanded_var);
 		word += unquote(expanded_var);
 	    } else {
 		// Error
@@ -535,6 +537,7 @@ XrlAction::expand_xrl_variables(const TreeNode& tn,
 	} else if (segment[0] == '$') {
 	    expand_done = tn.expand_variable(segment, expanded_var);
 	    if (expand_done) {
+	    	expanded_var = xrlatom_encode_value(expanded_var);
 		word += unquote(expanded_var);
 	    } else {
 		// Error
@@ -590,9 +593,9 @@ XrlAction::affected_module() const
 }
 
 Command::Command(TemplateTreeNode& template_tree_node, const string& cmd_name)
-    : _template_tree_node(template_tree_node)
+    : BaseCommand(template_tree_node, cmd_name) 
 {
-    _cmd_name = cmd_name;
+    debug_msg("Command constructor: %s\n", cmd_name.c_str());
 }
 
 Command::~Command()
@@ -614,7 +617,7 @@ Command::add_action(const list<string>& action, const XRLdb& xrldb)
 }
 
 int
-Command::execute(ConfigTreeNode& ctn, TaskManager& task_manager) const
+Command::execute(MasterConfigTreeNode& ctn, TaskManager& task_manager) const
 {
     int result = 0;
     int actions = 0;
@@ -645,7 +648,7 @@ Command::execute(ConfigTreeNode& ctn, TaskManager& task_manager) const
 void
 Command::action_complete(const XrlError& err,
 			 XrlArgs* ,
-			 ConfigTreeNode* ctn) const
+			 MasterConfigTreeNode* ctn) const
 {
     debug_msg("Command::action_complete\n");
 
@@ -663,7 +666,8 @@ Command::affected_xrl_modules() const
     list<Action*>::const_iterator iter;
 
     for (iter = _actions.begin(); iter != _actions.end(); ++iter) {
-	XrlAction* xa = dynamic_cast<XrlAction*>(*iter);
+	Action *a = (*iter);
+	XrlAction* xa = dynamic_cast<XrlAction*>(a);
 	if (xa != NULL) {
 	    string affected = xa->affected_module();
 	    affected_modules.insert(affected);
@@ -711,177 +715,13 @@ Command::check_referred_variables(string& errmsg) const
     return true;
 }
 
-
-// ----------------------------------------------------------------------------
-// AllowCommand implementation
 
-AllowCommand::AllowCommand(TemplateTreeNode& template_tree_node,
-			   const string& cmd_name)
-    : Command(template_tree_node, cmd_name)
-{
-}
-
-
-// ----------------------------------------------------------------------------
-// AllowOptionsCommand implementation
-
-AllowOptionsCommand::AllowOptionsCommand(TemplateTreeNode& 	ttn,
-					 const string&		cmd_name)
-    : AllowCommand(ttn, cmd_name)
-{
-}
-
-void
-AllowOptionsCommand::add_action(const list<string>& action) throw (ParseError)
-{
-    debug_msg("AllowOptionsCommand::add_action\n");
-
-    if (action.size() < 2) {
-	xorp_throw(ParseError, "Allow command with less than two parameters");
-    }
-
-    list<string>::const_iterator iter;
-    iter = action.begin();
-    if ((_varname.size() != 0) && (_varname != *iter)) {
-	xorp_throw(ParseError,
-		   "Currently only one variable per node can be specified "
-		   "using \"allow\" commands");
-    }
-    _varname = *iter;
-    ++iter;
-    for ( ; iter != action.end(); ++iter) {
-	_allowed_values.push_back(unquote(*iter));
-    }
-}
-
-bool
-AllowOptionsCommand::verify_variable_value(const ConfigTreeNode& ctn,
-					   string&		 errmsg) const
-{
-    string value;
-
-    if (ctn.expand_variable(_varname, value) == false) {
-	// Error: cannot expand the variable
-	errmsg = c_format("Variable %s is not defined.", _varname.c_str());
-	return false;
-    }
-
-    list<string>::const_iterator iter;
-    for (iter = _allowed_values.begin();
-	iter != _allowed_values.end();
-	++iter) {
-	if (value == *iter)
-	    return true;		// OK
-    }
-
-    // Error: variable value is not allowed
-    errmsg = c_format("Value \"%s\" is not a valid value for variable %s. ",
-		      value.c_str(), _varname.c_str());
-    list<string> values = _allowed_values;
-    if (values.size() == 1) {
-	errmsg += c_format("The only value allowed is %s.",
-			   values.front().c_str());
-    } else {
-	errmsg += "Allowed values are: ";
-	errmsg += values.front();
-	values.pop_front();
-	while (! values.empty()) {
-	    if (values.size() == 1)
-		errmsg += " and ";
-	    else
-		errmsg += ", ";
-	    errmsg += values.front();
-	    values.pop_front();
-	}
-	errmsg += ".";
-    }
-    return false;
-}
-
-string
-AllowOptionsCommand::str() const
-{
-    string tmp;
-
-    tmp = "AllowOptionsCommand: varname = " + _varname + " \n       Allowed values: ";
-
-    list<string>::const_iterator iter;
-    for (iter = _allowed_values.begin();
-	 iter != _allowed_values.end();
-	 ++iter) {
-	tmp += "  " + *iter;
-    }
-    tmp += "\n";
-    return tmp;
-}
-
-// ----------------------------------------------------------------------------
-// AllowRangeCommand implementation
-
-AllowRangeCommand::AllowRangeCommand(TemplateTreeNode& 	ttn,
-				     const string&	cmd_name)
-    : AllowCommand(ttn, cmd_name)
-{
-}
-
-void
-AllowRangeCommand::add_action(const list<string>& action) throw (ParseError)
-{
-    debug_msg("AllowRangeCommand::add_action\n");
-
-    if (action.size() < 3) {
-	xorp_throw(ParseError, "Allow range command with less than three parameters");
-    }
-
-    list<string>::const_iterator iter;
-    iter = action.begin();
-    if ((_varname.size() != 0) && (_varname != *iter)) {
-	xorp_throw(ParseError,
-		   "Currently only one variable per node can be specified "
-		   "using \"allow range\" commands");
-    }
-    _varname = *iter;
-    ++iter;
-    _lower = atoi(unquote(*iter).c_str());
-    ++iter;
-    _upper = atoi(unquote(*iter).c_str());
-
-    if (_lower > _upper)
-	swap(_lower, _upper);
-}
-
-bool
-AllowRangeCommand::verify_variable_value(const ConfigTreeNode&	ctn,
-					 string&		errmsg) const
-{
-    string value;
-
-    if (ctn.expand_variable(_varname, value) == false) {
-	// Error: cannot expand the variable
-	errmsg = c_format("Variable %s is not defined.", _varname.c_str());
-	return false;
-    }
-
-    int32_t ival = atoi(value.c_str());
-    if (ival < _lower || ival > _upper) {
-	errmsg = c_format("Value %s is outside valid range, %d...%d,  for variable %s.", value.c_str(), _lower, _upper, _varname.c_str());
-	return false;
-    }
-
-    return true;
-}
-
-string
-AllowRangeCommand::str() const
-{
-    return c_format("AllowRangeCommand: varname = %s\n       Allowed range: %d...%d", _varname.c_str(), _lower, _upper);
-}
 
 //
 // Template explicit instatiation
 //
-template int XrlAction::expand_xrl_variables<class ConfigTreeNode>(
-    const ConfigTreeNode& ctn,
+template int XrlAction::expand_xrl_variables<class MasterConfigTreeNode>(
+    const MasterConfigTreeNode& ctn,
     string& result,
     string& errmsg) const;
 template int XrlAction::expand_xrl_variables<class TemplateTreeNode>(
