@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/template_tree_node.cc,v 1.27 2004/06/10 22:41:54 hodson Exp $"
+#ident "$XORP: xorp/rtrmgr/template_tree_node.cc,v 1.35 2005/03/25 02:54:39 pavlin Exp $"
 
 
 #include <glob.h>
@@ -36,20 +36,24 @@ TemplateTreeNode::TemplateTreeNode(TemplateTree& template_tree,
 				   TemplateTreeNode* parent,
 				   const string& path,
 				   const string& varname)
-    : _template_tree(template_tree),
-      _parent(parent),
+    : _parent(parent),
+      _template_tree(template_tree),
       _module_name(""),
       _default_target_name(""),
       _segname(path),
       _varname(varname),
       _has_default(false),
       _is_tag(false),
-      _verbose(template_tree.verbose())
+      _verbose(template_tree.verbose()),
+      _is_deprecated(false)
 {
     if (_parent != NULL) {
 	_parent->add_child(this);
 	_module_name = _parent->module_name();
 	_default_target_name = _parent->default_target_name();
+	_child_number = _parent->children().size();
+    } else {
+	_child_number = 0;
     }
 }
 
@@ -60,7 +64,7 @@ TemplateTreeNode::~TemplateTreeNode()
 	_children.pop_front();
     }
 
-    map<string, Command *>::iterator iter1, iter2;
+    map<string, BaseCommand *>::iterator iter1, iter2;
     iter1 = _cmd_map.begin();
     while (iter1 != _cmd_map.end()) {
 	iter2 = iter1;
@@ -77,18 +81,13 @@ TemplateTreeNode::add_child(TemplateTreeNode* child)
 }
 
 void
-TemplateTreeNode::add_cmd(const string& cmd, TemplateTree& tt)
+TemplateTreeNode::add_cmd(const string& cmd)
     throw (ParseError)
 {
-    Command* command;
-    map<string, Command*>::iterator iter;
+    BaseCommand* command;
 
     if (cmd == "%modinfo") {
-	iter = _cmd_map.find("%modinfo");
-	if (iter == _cmd_map.end()) {
-	    command = new ModuleCommand(tt, *this, cmd);
-	    _cmd_map[cmd] = command;
-	}
+	// only the MasterTemplateTree cares about this
     } else if (cmd == "%allow") {
 	// If the command already exists, no need to create it again.
 	// The command action will simply be added to the existing command.
@@ -105,8 +104,11 @@ TemplateTreeNode::add_cmd(const string& cmd, TemplateTree& tt)
 	}
     } else if (cmd == "%help") {
 	// Nothing to do - the work is done by add_action
+    } else if (cmd == "%deprecated") {
+	// Nothing to do - the work is done by add_action
     } else if ((cmd == "%create")
 	       || (cmd == "%activate")
+	       || (cmd == "%update")
 	       || (cmd == "%list")
 	       || (cmd == "%delete")
 	       || (cmd == "%set")
@@ -116,7 +118,9 @@ TemplateTreeNode::add_cmd(const string& cmd, TemplateTree& tt)
 	// If the command already exists, no need to create it again.
 	// The command action will simply be added to the existing command.
 	if (_cmd_map.find(cmd) == _cmd_map.end()) {
-	    command = new Command(*this, cmd);
+	    // we just create a placeholder here - if we were a master
+	    // template tree node we'd create a real command.
+	    command = new BaseCommand(*this, cmd);
 	    _cmd_map[cmd] = command;
 	}
     } else if (cmd == "%mandatory") {
@@ -124,7 +128,7 @@ TemplateTreeNode::add_cmd(const string& cmd, TemplateTree& tt)
     } else {
 	string err = "Invalid command \"" + cmd + "\"\n";
 	err += "Valid commands are %create, %delete, %set, %unset, %get, ";
-	err += "%default, %modinfo, %activate, %allow, %allow-range, %mandatory\n";
+	err += "%default, %modinfo, %activate, %update, %allow, %allow-range, %mandatory, %deprecated\n";
 	xorp_throw(ParseError, err);
     }
 }
@@ -133,7 +137,7 @@ set<string>
 TemplateTreeNode::commands() const
 {
     set<string> cmds;
-    map<string, Command *>::const_iterator iter;
+    map<string, BaseCommand *>::const_iterator iter;
 
     for (iter = _cmd_map.begin(); iter != _cmd_map.end(); ++iter) {
 	cmds.insert(iter->first);
@@ -143,19 +147,13 @@ TemplateTreeNode::commands() const
 
 void
 TemplateTreeNode::add_action(const string& cmd,
-			     const list<string>& action_list,
-			     const XRLdb& xrldb)
+			     const list<string>& action_list)
 {
-    Command* command;
-    map<string, Command*>::iterator iter;
+    BaseCommand* command;
+    map<string, BaseCommand*>::iterator iter;
 
     if (cmd == "%modinfo") {
-	iter = _cmd_map.find("%modinfo");
-	XLOG_ASSERT(iter != _cmd_map.end());
-	command = iter->second;
-	ModuleCommand* module_command = dynamic_cast<ModuleCommand*>(command);
-	XLOG_ASSERT(module_command != NULL);
-	module_command->add_action(action_list, xrldb);
+	// only the Master tree cares about this
     } else if (cmd == "%allow") {
 	iter = _cmd_map.find("%allow");
 	XLOG_ASSERT(iter != _cmd_map.end());
@@ -174,7 +172,7 @@ TemplateTreeNode::add_action(const string& cmd,
 	if (action_list.size() == 2) {
 	    list<string>::const_iterator li = action_list.begin();
 	    li++;
-	    //trim off quotes if present
+	    // Trim off quotes if present
 	    string help = unquote(*li);
 	    if (action_list.front() == "short") {
 		_help = help;
@@ -187,9 +185,24 @@ TemplateTreeNode::add_action(const string& cmd,
 				    + "or \"long\" expectted").c_str());
 	    }
 	} else {
-	    //XXX really should say why it's bad.
+	    // XXX really should say why it's bad.
 	    XLOG_WARNING("Bad help specification in template file ignored\n");
 	}
+    } else if (cmd == "%deprecated") {
+	if (action_list.size() == 1) {
+	    list<string>::const_iterator li = action_list.begin();
+	    // Trim off quotes if present
+	    string reason = unquote(*li);
+	    _is_deprecated = true;
+	    _deprecated_reason = reason;
+	    if ((_parent != NULL) && (_parent->is_tag())) {
+		_parent->set_deprecated(true);
+		_parent->set_deprecated_reason(reason);
+	    }
+	} else {
+	    // XXX really should say why it's bad.
+	    XLOG_WARNING("Bad %%deprecated specification in template file ignored\n");
+	}
     } else if (cmd == "%mandatory") {
 	// Add all new mandatory variables
 	list<string>::const_iterator li;
@@ -203,10 +216,7 @@ TemplateTreeNode::add_action(const string& cmd,
 	    }
 	}
     } else {
-	iter = _cmd_map.find(cmd);
-	XLOG_ASSERT(iter != _cmd_map.end());
-	command = iter->second;
-	command->add_action(action_list, xrldb);
+	// the master tree will deal with these
     }
 }
 
@@ -252,7 +262,7 @@ TemplateTreeNode::str() const
 	tmp += " = " + default_str();
     }
     if (!_cmd_map.empty()) {
-	map<string, Command*>::const_iterator rpair;
+	map<string, BaseCommand*>::const_iterator rpair;
 	rpair = _cmd_map.begin();
 	while (rpair != _cmd_map.end()) {
 	    tmp += "\n    " + rpair->second->str();
@@ -283,10 +293,10 @@ TemplateTreeNode::type_match(const string& ) const
     return true;
 }
 
-Command*
+BaseCommand*
 TemplateTreeNode::command(const string& cmd_name)
 {
-    map<string, Command *>::iterator iter;
+    map<string, BaseCommand *>::iterator iter;
 
     iter = _cmd_map.find(cmd_name);
     if (iter == _cmd_map.end())
@@ -294,10 +304,10 @@ TemplateTreeNode::command(const string& cmd_name)
     return iter->second;
 }
 
-const Command*
+const BaseCommand*
 TemplateTreeNode::const_command(const string& cmd_name) const
 {
-    map<string, Command *>::const_iterator iter;
+    map<string, BaseCommand *>::const_iterator iter;
 
     iter = _cmd_map.find(cmd_name);
     if (iter == _cmd_map.end())
@@ -544,6 +554,25 @@ TemplateTreeNode::find_child_varname_node(const list<string>& var_parts) const
     return NULL;
 }
 
+const TemplateTreeNode*
+TemplateTreeNode::find_first_deprecated_ancestor() const
+{
+    const TemplateTreeNode* ttn = _parent;
+    const TemplateTreeNode* deprecated_ttn = NULL;
+
+    if (is_deprecated())
+	deprecated_ttn = this;
+
+    while (ttn != NULL) {
+	if (ttn->is_deprecated())
+	    deprecated_ttn = ttn;
+	ttn = ttn->parent();
+    }
+
+    return (deprecated_ttn);
+}
+
+#if 0
 bool
 TemplateTreeNode::check_template_tree(string& errmsg) const
 {
@@ -554,11 +583,14 @@ TemplateTreeNode::check_template_tree(string& errmsg) const
     //
     // Check whether all referred variable names are valid
     //
-    map<string, Command *>::const_iterator iter1;
+    map<string, BaseCommand *>::const_iterator iter1;
     for (iter1 = _cmd_map.begin(); iter1 != _cmd_map.end(); ++iter1) {
-	const Command* command = iter1->second;
-	if (! command->check_referred_variables(errmsg))
-	    return false;
+	const Command* command;
+	command = dynamic_cast<Command*>(iter1->second);
+	if (command) {
+	    if (! command->check_referred_variables(errmsg))
+		return false;
+	}
     }
 
     //
@@ -573,6 +605,7 @@ TemplateTreeNode::check_template_tree(string& errmsg) const
 
     return true;
 }
+#endif
 
 bool
 TemplateTreeNode::check_command_tree(const list<string>& cmd_names,
@@ -581,8 +614,12 @@ TemplateTreeNode::check_command_tree(const list<string>& cmd_names,
 {
     bool instantiated = false;
 
+    // XXX: ignore deprecated subtrees
+    if (is_deprecated())
+	    return false;
+
     debug_msg("TTN:check_command_tree %s type %s depth %u\n",
-	      _segname.c_str(), typestr().c_str(), (uint32_t)depth);
+	      _segname.c_str(), typestr().c_str(), XORP_UINT_CAST(depth));
 
     if (_parent != NULL && _parent->is_tag() && (depth == 0)) {
 	//
