@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/xrl_rtrmgr_interface.cc,v 1.23 2004/06/10 22:41:56 hodson Exp $"
+#ident "$XORP: xorp/rtrmgr/xrl_rtrmgr_interface.cc,v 1.34 2005/03/25 02:54:41 pavlin Exp $"
 
 
 #include <sys/stat.h>
@@ -36,18 +36,19 @@ XrlRtrmgrInterface::XrlRtrmgrInterface(XrlRouter& r, UserDB& userdb,
 				       RandomGen& randgen,
 				       Rtrmgr& rtrmgr) 
     : XrlRtrmgrTargetBase(&r),
+      _xrl_router(r),
       _client_interface(&r),
+      _finder_notifier_interface(&r),
       _userdb(userdb),
-      _conf_tree(NULL),
+      _master_config_tree(NULL),
       _eventloop(eventloop), 
       _randgen(randgen),
       _rtrmgr(rtrmgr),
       _exclusive(false),
       _config_locked(false),
-      _lock_holder((uint32_t)-1),
+      _lock_holder_token(""),
       _verbose(rtrmgr.verbose())
 {
-
 }
 
 XrlRtrmgrInterface::~XrlRtrmgrInterface()
@@ -143,7 +144,8 @@ XrlRtrmgrInterface::rtrmgr_0_1_register_client(
     user = _userdb.find_user_by_user_id(user_id);
     if (user == NULL) {
 	unlink(filename.c_str());
-	string err = c_format("User ID %d not found", user_id);
+	string err = c_format("User ID %u not found",
+			      XORP_UINT_CAST(user_id));
 	return XrlCmdError::COMMAND_FAILED(err);
     }
 
@@ -168,7 +170,7 @@ XrlRtrmgrInterface::rtrmgr_0_1_register_client(
     if (fchown(fileno(file), user_id, (gid_t)-1) != 0) {
 	unlink(filename.c_str());
 	string err = c_format("Failed to chown temporary file %s to user_id %u",
-			      filename.c_str(), user_id);
+			      filename.c_str(), XORP_UINT_CAST(user_id));
 	return XrlCmdError::COMMAND_FAILED(err);
     }
     if (fclose(file) != 0) {
@@ -220,6 +222,7 @@ XrlRtrmgrInterface::rtrmgr_0_1_authenticate_client(
     }
     if (token == user->authtoken()) {
 	user->set_authenticated(true);
+	initialize_client_state(user_id, user);
 	return XrlCmdError::OKAY();
     } else {
 	string err = "Bad authtoken";
@@ -227,6 +230,87 @@ XrlRtrmgrInterface::rtrmgr_0_1_authenticate_client(
     }
 }
 
+void
+XrlRtrmgrInterface::initialize_client_state(uint32_t user_id, 
+					    UserInstance *user)
+{
+    // We need to register interest via the finder in the XRL target
+    // associated with this user instance, so that when the user
+    // instance goes away without deregistering, we can clean up the
+    // state we associated with them.
+
+    XrlFinderEventNotifierV0p1Client::RegisterInstanceEventInterestCB cb; 
+    cb = callback(this, &XrlRtrmgrInterface::finder_register_done,
+		  user->clientname());
+    _finder_notifier_interface.
+	send_register_class_event_interest("finder", 
+					   _xrl_router.instance_name(),
+					   user->clientname(), cb);
+    XLOG_TRACE(_verbose, 
+	       "registering interest in %s\n", user->clientname().c_str());
+
+
+    //
+    // We need to send the running config and module state to the
+    // client, but we first need to return from the current XRL, so we
+    // schedule this on a zero-second timer.
+    //
+    XorpTimer t;
+    uint32_t delay = 0;
+    if (!_rtrmgr.ready()) {
+	delay = 2000;
+    }
+    t = _eventloop.new_oneoff_after_ms(delay,
+             callback(this, &XrlRtrmgrInterface::send_client_state, 
+		      user_id, user));
+    _background_tasks.push_front(t);
+}
+
+void 
+XrlRtrmgrInterface::finder_register_done(const XrlError& e, string clientname) 
+{
+    if (e != XrlError::OKAY()) {
+	XLOG_ERROR("Failed to register with finder about XRL %s (err: %s)\n",
+		   clientname.c_str(), e.error_msg());
+    }
+}
+
+void
+XrlRtrmgrInterface::send_client_state(uint32_t user_id, UserInstance *user)
+{
+    debug_msg("send_client_state %s\n", user->clientname().c_str());
+    if (!_rtrmgr.ready()) {
+	//we're still in the process of reconfiguring, so delay this
+	//til later
+	initialize_client_state(user_id, user);
+	return;
+    }
+
+    string config = _master_config_tree->show_tree();
+    string client = user->clientname();
+    GENERIC_CALLBACK cb2;
+    cb2 = callback(this, &XrlRtrmgrInterface::client_updated, 
+		   user_id, user);
+    debug_msg("Sending config changed to %s\n", client.c_str());
+    _client_interface.send_config_changed(client.c_str(),
+					  0, config, "", cb2);
+
+    debug_msg("Sending mod status changed to %s\n", client.c_str());
+    list <string> module_names;
+    ModuleManager &mmgr(_master_config_tree->module_manager());
+    mmgr.get_module_list(module_names);
+    list <string>::iterator i;
+    for (i = module_names.begin(); i != module_names.end(); i++) {
+	debug_msg("module: %s\n", (*i).c_str());
+	Module::ModuleStatus status = mmgr.module_status(*i);
+	if (status != Module::NO_SUCH_MODULE) {
+	    debug_msg("status %d\n", status);
+	    _client_interface.send_module_status(client.c_str(),
+                 *i, (uint32_t)status, cb2);
+	}
+    } 
+}
+
 XrlCmdError 
 XrlRtrmgrInterface::rtrmgr_0_1_enter_config_mode(
 	// Input values, 
@@ -244,7 +328,8 @@ XrlRtrmgrInterface::rtrmgr_0_1_enter_config_mode(
 	response = "You do not have permission for this operation.";
 	return XrlCmdError::COMMAND_FAILED(response);
     }
-    XLOG_TRACE(_verbose, "user %d entering config mode\n", user_id);
+    XLOG_TRACE(_verbose, "user %u entering config mode\n",
+	       XORP_UINT_CAST(user_id));
     if (exclusive)
 	XLOG_TRACE(_verbose, "user requested exclusive config\n");
     else
@@ -287,8 +372,8 @@ XrlRtrmgrInterface::rtrmgr_0_1_leave_config_mode(
 	string err = "AUTH_FAIL";
 	return XrlCmdError::COMMAND_FAILED(err);
     }
-    XLOG_TRACE(_verbose, "user %d leaving config mode\n",
-	       get_user_id_from_token(token));
+    XLOG_TRACE(_verbose, "user %u leaving config mode\n",
+	       XORP_UINT_CAST(get_user_id_from_token(token)));
     multimap<uint32_t, UserInstance*>::iterator iter;
     for (iter = _config_users.begin(); iter != _config_users.end(); ++iter) {
 	if (iter->second->authtoken() == token) {
@@ -318,11 +403,25 @@ XrlRtrmgrInterface::rtrmgr_0_1_get_config_users(
     }
     multimap<uint32_t, UserInstance*>::const_iterator iter;
     for (iter = _config_users.begin(); iter != _config_users.end(); ++iter) {
-	users.append(XrlAtom(iter->second->user_id()));
+	UserInstance* user = iter->second;
+	if (!user->is_a_zombie()) {
+	    users.append(XrlAtom(user->user_id()));
+	} else {
+	    XLOG_WARNING("user %u is a zombie\n",
+			 XORP_UINT_CAST(user->user_id()));
+	}
     }
     return XrlCmdError::OKAY();
 }
 
+/**
+ * This interface is deprecated as the main way for xorpsh to get the
+ * running config from the router manager.  It is retained for
+ * debugging purposes.  xorpsh now gets its config automatically
+ * immediatedly after registering which removes a potential timing
+ * hole where the client could be told about a change to the config
+ * before it has asked what the current config is.
+*/
 XrlCmdError
 XrlRtrmgrInterface::rtrmgr_0_1_get_running_config(
 	// Input values, 
@@ -337,7 +436,7 @@ XrlRtrmgrInterface::rtrmgr_0_1_get_running_config(
     }
     if (_rtrmgr.ready()) {
 	ready = true;
-	config = _conf_tree->show_tree();
+	config = _master_config_tree->show_tree();
     } else {
 	ready = false;
 	config = "";
@@ -372,28 +471,28 @@ XrlRtrmgrInterface::rtrmgr_0_1_apply_config_change(
 	       deltas.c_str(), deletions.c_str());
 
     string response;
-    if (_conf_tree->apply_deltas(user_id, deltas, 
-				 /* provisional change */ true,
-				 response) == false) {
+    if (_master_config_tree->apply_deltas(user_id, deltas, 
+					  /* provisional change */ true,
+					  response) == false) {
 	return XrlCmdError::COMMAND_FAILED(response);
     }
-    if (_conf_tree->apply_deletions(user_id, deletions, 
-				    /* provisional change */ true,
-				    response) == false) {
+    if (_master_config_tree->apply_deletions(user_id, deletions, 
+					     /* provisional change */ true,
+					     response) == false) {
 	return XrlCmdError::COMMAND_FAILED(response);
     }
     // Add nodes providing default values.  Note: they shouldn't be
     // needed, but adding them here acts as a safety mechanism against
     // a client that forgets to add them.
-    _conf_tree->add_default_children();
+    _master_config_tree->add_default_children();
 
     CallBack cb;
     cb = callback(this, &XrlRtrmgrInterface::apply_config_change_done,
 		  user_id, string(target), string(deltas), string(deletions));
 
-    _conf_tree->commit_changes_pass1(cb);
-    if (_conf_tree->config_failed()) {
-	string err = _conf_tree->config_failed_msg();
+    _master_config_tree->commit_changes_pass1(cb);
+    if (_master_config_tree->config_failed()) {
+	string err = _master_config_tree->config_failed_msg();
 	return XrlCmdError::COMMAND_FAILED(err);
     }
     return XrlCmdError::OKAY();
@@ -413,7 +512,7 @@ XrlRtrmgrInterface::apply_config_change_done(bool success,
 
     if (success) {
 	// Check everything really worked, and finalize the commit
-	if (_conf_tree->check_commit_status(errmsg) == false) {
+	if (_master_config_tree->check_commit_status(errmsg) == false) {
 	    XLOG_TRACE(_verbose,
 		       "check commit status indicates failure: >%s<\n",
 		       errmsg.c_str());
@@ -435,7 +534,7 @@ XrlRtrmgrInterface::apply_config_change_done(bool success,
 	    GENERIC_CALLBACK cb2;
 	    cb2 = callback(this, &XrlRtrmgrInterface::client_updated, 
 			   iter->first, iter->second);
-	    debug_msg("Sending config changed to %s\n", target.c_str());
+	    debug_msg("Sending config changed to %s\n", client.c_str());
 	    _client_interface.send_config_changed(client.c_str(),
 						  user_id,
 						  deltas, deletions, cb2);
@@ -445,7 +544,7 @@ XrlRtrmgrInterface::apply_config_change_done(bool success,
 						  true, "", cb1);
     } else {
 	// Something went wrong
-	_conf_tree->discard_changes();
+	_master_config_tree->discard_changes();
 	debug_msg("Sending config change failed to %s\n", target.c_str());
 	_client_interface.send_config_change_done(target.c_str(),
 						  false, errmsg, cb1);
@@ -485,6 +584,23 @@ XrlRtrmgrInterface::client_updated(const XrlError& e, uid_t user_id,
     }
 }
 
+void 
+XrlRtrmgrInterface::module_status_changed(const string& modname,
+					  GenericModule::ModuleStatus status)
+{
+    multimap<uint32_t, UserInstance*>::iterator iter;
+    for (iter = _users.begin(); iter != _users.end(); ++iter) {
+	string client = iter->second->clientname();
+	GENERIC_CALLBACK cb2;
+	cb2 = callback(this, &XrlRtrmgrInterface::client_updated, 
+		       iter->first, iter->second);
+	debug_msg("Sending mod statis changed to %s\n", client.c_str());
+	_client_interface.send_module_status(client.c_str(), 
+					     modname, (uint32_t)status, cb2);
+    }
+}
+
+
 XrlCmdError
 XrlRtrmgrInterface::rtrmgr_0_1_lock_config(
 	// Input values, 
@@ -507,12 +623,12 @@ XrlRtrmgrInterface::rtrmgr_0_1_lock_config(
     if (_config_locked) {
 	// Can't return COMMAND_FAILED and return the lock holder
 	success = false;
-	holder = _lock_holder;
+	holder = get_user_id_from_token(_lock_holder_token);
 	return XrlCmdError::OKAY();
     } else {
 	success = true;
 	_config_locked = true;
-	_lock_holder = get_user_id_from_token(token);
+	_lock_holder_token = token;
 	_lock_timer = _eventloop.new_oneoff_after_ms(
 	    timeout, 
 	    callback(this, &XrlRtrmgrInterface::lock_timeout));
@@ -523,7 +639,7 @@ XrlRtrmgrInterface::rtrmgr_0_1_lock_config(
 void XrlRtrmgrInterface::lock_timeout()
 {
     _config_locked = false;
-    _lock_holder = (uint32_t)-1;
+    _lock_holder_token = "";
 }
 
 XrlCmdError
@@ -536,14 +652,14 @@ XrlRtrmgrInterface::rtrmgr_0_1_unlock_config(
 	return XrlCmdError::COMMAND_FAILED(err);
     }
     if (_config_locked) {
-	uint32_t user_id = get_user_id_from_token(token);
-	if (user_id != _lock_holder) {
-	    string err = c_format("Config not held by USER_ID %d.", user_id);
+	if (token != _lock_holder_token) {
+	    string err = c_format("Config not held by process %s.", 
+				  token.c_str());
 	    return XrlCmdError::COMMAND_FAILED(err);
 	}
 	_config_locked = false;
 	_lock_timer.unschedule();
-	_lock_holder = (uint32_t)-1;
+	_lock_holder_token = "";
 	return XrlCmdError::OKAY();
     } else {
 	string err = "Config not locked.";
@@ -574,7 +690,7 @@ XrlRtrmgrInterface::rtrmgr_0_1_lock_node(
 	// Shouldn't be possible as we already checked the token
 	XLOG_UNREACHABLE();
     }
-    success = _conf_tree->lock_node(node, user_id, timeout, holder);
+    success = _master_config_tree->lock_node(node, user_id, timeout, holder);
     return XrlCmdError::OKAY();
 }
 
@@ -594,7 +710,7 @@ XrlRtrmgrInterface::rtrmgr_0_1_unlock_node(
 	XLOG_UNREACHABLE();
     }
     bool success;
-    success = _conf_tree->unlock_node(node, user_id);
+    success = _master_config_tree->unlock_node(node, user_id);
     if (success) {
 	return XrlCmdError::OKAY();
     } else {
@@ -620,15 +736,16 @@ XrlRtrmgrInterface::rtrmgr_0_1_save_config(// Input values:
 	return XrlCmdError::COMMAND_FAILED(response);
     }
     if (_config_locked) {
-	string holder = _userdb.find_user_by_user_id(_lock_holder)->username();
+	uint32_t uid = get_user_id_from_token(_lock_holder_token);
+	string holder = _userdb.find_user_by_user_id(uid)->username();
 	response = "ERROR: The config is currently locked by user " +
 	    holder + 
 	    ", and so may be in an inconsistent state.  \n" + 
 	    "Save was NOT performed.\n";
 	return XrlCmdError::COMMAND_FAILED(response);
     }
-    if (_conf_tree->save_to_file(filename, user_id, 
-				 _rtrmgr.save_hook(), response)) {
+    if (_master_config_tree->save_to_file(filename, user_id, 
+					  _rtrmgr.save_hook(), response)) {
 	return XrlCmdError::OKAY();
     } else {
 	return XrlCmdError::COMMAND_FAILED(response);
@@ -652,9 +769,10 @@ XrlRtrmgrInterface::rtrmgr_0_1_load_config(// Input values:
 	response = "You do not have permission to reload the configuration.";
 	return XrlCmdError::COMMAND_FAILED(response);
     }
-    if (_config_locked) {
+    if (_config_locked && (token != _lock_holder_token)) {
+	uint32_t uid = get_user_id_from_token(_lock_holder_token);
 	string holder = 
-	    _userdb.find_user_by_user_id(_lock_holder)->username();
+	    _userdb.find_user_by_user_id(uid)->username();
 	response = "ERROR: The config is currently locked by user " +
 	    holder + 
 	    "\n" + 
@@ -663,17 +781,17 @@ XrlRtrmgrInterface::rtrmgr_0_1_load_config(// Input values:
     }
 
     string deltas, deletions;	// These are filled in by load_from_file
-    if (_conf_tree->load_from_file(filename, user_id, response, deltas,
-				   deletions)) {
+    if (_master_config_tree->load_from_file(filename, user_id, response,
+					    deltas, deletions)) {
 	CallBack cb;
 	cb = callback(this, &XrlRtrmgrInterface::apply_config_change_done,
 		      user_id, string(target), 
 		      string(deltas), string(deletions));
 	debug_msg("here1: success\n");
 	response = "";
-	_conf_tree->commit_changes_pass1(cb);
-	if (_conf_tree->config_failed()) {
-	    string err = _conf_tree->config_failed_msg();
+	_master_config_tree->commit_changes_pass1(cb);
+	if (_master_config_tree->config_failed()) {
+	    string err = _master_config_tree->config_failed_msg();
 	    return XrlCmdError::COMMAND_FAILED(err);
 	}
     } else {
@@ -718,13 +836,15 @@ uint32_t
 XrlRtrmgrInterface::get_user_id_from_token(const string& token) const
 {
     uint32_t user_id;
+    unsigned int u;
     size_t tlen = token.size();
 
     if (token.size() < 10)
 	return (uint32_t)-1;
     string uidstr = token.substr(0, 10);
     string authstr = token.substr(10, tlen - 10);
-    sscanf(uidstr.c_str(), "%u", &user_id);
+    sscanf(uidstr.c_str(), "%u", &u);
+    user_id = u;
     return user_id;
 }
 
@@ -733,7 +853,7 @@ XrlRtrmgrInterface::generate_auth_token(const uint32_t& user_id,
 					const string& clientname)
 {
     string token;
-    token = c_format("%10u", user_id);
+    token = c_format("%10u", XORP_UINT_CAST(user_id));
     string shortcname;
     size_t clen = clientname.size();
 
@@ -755,3 +875,45 @@ XrlRtrmgrInterface::generate_auth_token(const uint32_t& user_id,
 
     return token;
 }
+
+
+XrlCmdError 
+XrlRtrmgrInterface::finder_event_observer_0_1_xrl_target_birth(
+        const string&	target_class,
+	const string&	target_instance)
+{
+    XLOG_TRACE(_verbose, "XRL Birth: class %s instance %s\n",
+	       target_class.c_str(), target_instance.c_str());
+    return XrlCmdError::OKAY();
+}
+
+
+XrlCmdError 
+XrlRtrmgrInterface::finder_event_observer_0_1_xrl_target_death(
+	const string&	target_class,
+	const string&	target_instance)
+{
+    XLOG_TRACE(_verbose, "XRL Death: class %s instance %s\n",
+	       target_class.c_str(), target_instance.c_str());
+
+    multimap<uint32_t, UserInstance*>::iterator iter;
+    for (iter = _config_users.begin(); iter != _config_users.end(); ++iter) {
+	UserInstance* user = iter->second;
+	if (user->clientname() == target_class) {
+	    debug_msg("removing user %s from list of config users\n",
+		      target_class.c_str());
+	    _config_users.erase(iter);
+	    break;
+	}
+    }
+    for (iter = _users.begin(); iter != _users.end(); ++iter) {
+	UserInstance* user = iter->second;
+	if (user->clientname() == target_class) {
+	    debug_msg("removing user %s from list of regular users\n",
+		      target_class.c_str());
+	    _users.erase(iter);
+	    break;
+	}
+    }
+    return XrlCmdError::OKAY();
+}
