@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +13,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/static_routes/static_routes_node.cc,v 1.12 2004/06/10 22:41:57 hodson Exp $"
-
+#ident "$XORP: xorp/static_routes/static_routes_node.cc,v 1.25 2005/03/25 02:54:42 pavlin Exp $"
 
 //
 // StaticRoutes node implementation.
 //
 
-
 #include "static_routes_module.h"
 
 #include "libxorp/xorp.h"
@@ -28,7 +27,7 @@
 #include "libxorp/ipvx.hh"
 
 #include "static_routes_node.hh"
-
+#include "static_routes_varrw.hh"
 
 StaticRoutesNode::StaticRoutesNode(EventLoop& eventloop)
     : ServiceBase("StaticRoutes"),
@@ -38,10 +37,7 @@ StaticRoutesNode::StaticRoutesNode(EventLoop& eventloop)
       _shutdown_requests_n(0),
       _is_log_trace(true)		// XXX: default to print trace logs
 {
-    //
-    // Set the node status
-    //
-    _node_status = PROC_STARTUP;
+    set_node_status(PROC_STARTUP);
 }
 
 StaticRoutesNode::~StaticRoutesNode()
@@ -55,32 +51,30 @@ StaticRoutesNode::startup()
     //
     // Test the service status
     //
-    if ((ServiceBase::status() == STARTING)
-	|| (ServiceBase::status() == RUNNING))
+    if ((ServiceBase::status() == SERVICE_STARTING)
+	|| (ServiceBase::status() == SERVICE_RUNNING)) {
 	return true;
-
-    if (ServiceBase::status() != READY)
+    }
+    if (ServiceBase::status() != SERVICE_READY) {
 	return false;
+    }
 
     //
-    // Transition to RUNNING occurs when all transient startup operations
-    // are completed (e.g., after we have the interface/vif/address state
-    // available, after we have registered with the RIB, etc.)
+    // Transition to SERVICE_RUNNING occurs when all transient startup
+    // operations are completed (e.g., after we have the interface/vif/address
+    // state available, after we have registered with the RIB, etc.)
     //
-    ServiceBase::set_status(STARTING);
+    ServiceBase::set_status(SERVICE_STARTING);
 
     //
     // Set the node status
     //
-    _node_status = PROC_STARTUP;
+    set_node_status(PROC_STARTUP);
 
     //
-    // Startup the interface manager
+    // Register with the FEA
     //
-    if (ifmgr_startup() != true) {
-	ServiceBase::set_status(FAILED);
-	return false;
-    }
+    fea_register_startup();
 
     //
     // Register with the RIB
@@ -94,22 +88,27 @@ bool
 StaticRoutesNode::shutdown()
 {
     //
-    // We cannot shutdown if our status is SHUTDOWN or FAILED.
+    // Test the service status
     //
-    if ((ServiceBase::status() == SHUTDOWN)
-	|| (ServiceBase::status() == FAILED)) {
+    if ((ServiceBase::status() == SERVICE_SHUTDOWN)
+	|| (ServiceBase::status() == SERVICE_SHUTTING_DOWN)
+	|| (ServiceBase::status() == SERVICE_FAILED)) {
 	return true;
     }
-
-    if (ServiceBase::status() != RUNNING)
+    if ((ServiceBase::status() != SERVICE_RUNNING)
+	&& (ServiceBase::status() != SERVICE_STARTING)
+	&& (ServiceBase::status() != SERVICE_PAUSING)
+	&& (ServiceBase::status() != SERVICE_PAUSED)
+	&& (ServiceBase::status() != SERVICE_RESUMING)) {
 	return false;
+    }
 
     //
-    // Transition to SHUTDOWN occurs when all transient shutdown operations
-    // are completed (e.g., after we have deregistered with the FEA
+    // Transition to SERVICE_SHUTDOWN occurs when all transient shutdown
+    // operations are completed (e.g., after we have deregistered with the FEA
     // and the RIB, etc.)
     //
-    ServiceBase::set_status(SHUTTING_DOWN);
+    ServiceBase::set_status(SERVICE_SHUTTING_DOWN);
 
     //
     // De-register with the RIB
@@ -117,17 +116,19 @@ StaticRoutesNode::shutdown()
     rib_register_shutdown();
 
     //
-    // Shutdown the interface manager
+    // De-register with the FEA
     //
-    if (ifmgr_shutdown() != true) {
-	ServiceBase::set_status(FAILED);
-	return false;
-    }
+    fea_register_shutdown();
 
     //
     // Set the node status
     //
-    _node_status = PROC_SHUTDOWN;
+    set_node_status(PROC_SHUTDOWN);
+
+    //
+    // Update the node status
+    //
+    update_status();
 
     return true;
 }
@@ -139,15 +140,17 @@ StaticRoutesNode::status_change(ServiceBase*  service,
 {
     if (service == this) {
 	// My own status has changed
-	if ((old_status == STARTING) && (new_status == RUNNING)) {
+	if ((old_status == SERVICE_STARTING)
+	    && (new_status == SERVICE_RUNNING)) {
 	    // The startup process has completed
-	    _node_status = PROC_READY;
+	    set_node_status(PROC_READY);
 	    return;
 	}
 
-	if ((old_status == SHUTTING_DOWN) && (new_status == SHUTDOWN)) {
+	if ((old_status == SERVICE_SHUTTING_DOWN)
+	    && (new_status == SERVICE_SHUTDOWN)) {
 	    // The shutdown process has completed
-	    _node_status = PROC_DONE;
+	    set_node_status(PROC_DONE);
 	    return;
 	}
 
@@ -158,7 +161,8 @@ StaticRoutesNode::status_change(ServiceBase*  service,
     }
 
     if (service == ifmgr_mirror_service_base()) {
-	if ((old_status == SHUTTING_DOWN) && (new_status == SHUTDOWN)) {
+	if ((old_status == SERVICE_SHUTTING_DOWN)
+	    && (new_status == SERVICE_SHUTDOWN)) {
 	    decr_shutdown_requests_n();
 	}
     }
@@ -202,36 +206,34 @@ StaticRoutesNode::update_status()
     //
     // Test if the startup process has completed
     //
-    if (ServiceBase::status() == STARTING) {
+    if (ServiceBase::status() == SERVICE_STARTING) {
 	if (_startup_requests_n > 0)
 	    return;
 
 	// The startup process has completed
-	ServiceBase::set_status(RUNNING);
-	_node_status = PROC_READY;
+	ServiceBase::set_status(SERVICE_RUNNING);
+	set_node_status(PROC_READY);
 	return;
     }
 
     //
     // Test if the shutdown process has completed
     //
-    if (ServiceBase::status() == SHUTTING_DOWN) {
+    if (ServiceBase::status() == SERVICE_SHUTTING_DOWN) {
 	if (_shutdown_requests_n > 0)
 	    return;
 
 	// The shutdown process has completed
-	ServiceBase::set_status(SHUTDOWN);
-	// Set the node status
-	_node_status = PROC_DONE;
+	ServiceBase::set_status(SERVICE_SHUTDOWN);
+	set_node_status(PROC_DONE);
 	return;
     }
 
     //
     // Test if we have failed
     //
-    if (ServiceBase::status() == FAILED) {
-	// Set the node status
-	_node_status = PROC_DONE;
+    if (ServiceBase::status() == SERVICE_FAILED) {
+	set_node_status(PROC_DONE);
 	return;
     }
 }
@@ -251,7 +253,7 @@ StaticRoutesNode::node_status(string& reason_msg)
     case PROC_STARTUP:
 	// Get the message about the startup progress
 	reason_msg = c_format("Waiting for %u startup events",
-			      static_cast<uint32_t>(_startup_requests_n));
+			      XORP_UINT_CAST(_startup_requests_n));
 	break;
     case PROC_NOT_READY:
 	// XXX: this state is unused
@@ -263,7 +265,7 @@ StaticRoutesNode::node_status(string& reason_msg)
     case PROC_SHUTDOWN:
 	// Get the message about the shutdown progress
 	reason_msg = c_format("Waiting for %u shutdown events",
-			      static_cast<uint32_t>(_shutdown_requests_n));
+			      XORP_UINT_CAST(_shutdown_requests_n));
 	break;
     case PROC_FAILED:
 	// XXX: this state is unused
@@ -295,14 +297,17 @@ StaticRoutesNode::tree_complete()
 void
 StaticRoutesNode::updates_made()
 {
-    list<StaticRoute>::const_iterator route_iter;
+    list<StaticRoute>::iterator route_iter;
 
     for (route_iter = _static_routes.begin();
 	 route_iter != _static_routes.end();
 	 ++route_iter) {
-	const StaticRoute& static_route = *route_iter;
+	StaticRoute& static_route = *route_iter;
 	bool is_old_interface_up = false;
 	bool is_new_interface_up = false;
+	bool is_old_directly_connected = false;
+	bool is_new_directly_connected = false;
+	bool is_new_up = false;
 
 	if (static_route.is_interface_route()) {
 	    //
@@ -317,27 +322,32 @@ StaticRoutesNode::updates_made()
 					       static_route.vifname());
 	    if ((vif_atom != NULL) && (vif_atom->enabled()))
 		is_new_interface_up = true;
-	} else {
-	    //
-	    // Calculate whether the next-hop router was directly connected
-	    // before and now.
-	    //
-	    if (is_directly_connected(_iftree, static_route.nexthop()))
-		is_old_interface_up = true;
-	    if (is_directly_connected(ifmgr_iftree(), static_route.nexthop()))
-		is_new_interface_up = true;
+	}
+	//
+	// Calculate whether the next-hop router was directly connected
+	// before and now.
+	//
+	if (is_directly_connected(_iftree, static_route.nexthop()))
+	    is_old_directly_connected = true;
+	if (is_directly_connected(ifmgr_iftree(), static_route.nexthop()))
+	    is_new_directly_connected = true;
+
+	if ((is_old_interface_up == is_new_interface_up)
+	    && (is_old_directly_connected == is_new_directly_connected)) {
+	    continue;			// Nothing changed
 	}
 
 	if (is_old_interface_up == is_new_interface_up)
-	    continue;			// Nothing changed
+	    is_new_up = is_new_directly_connected;
+	else
+	    is_new_up = is_new_interface_up;
 
-	if (is_new_interface_up) {
+	if (is_new_up) {
 	    //
 	    // The interface is now up, hence add the route
 	    //
 	    inform_rib_route_change(static_route);
-	}
-	if (! is_new_interface_up) {
+	} else {
 	    //
 	    // The interface went down, hence cancel all pending requests,
 	    // and withdraw the route.
@@ -357,13 +367,13 @@ StaticRoutesNode::updates_made()
 }
 
 bool
-StaticRoutesNode::is_directly_connected(const IfMgrIfTree& if_tree,
+StaticRoutesNode::is_directly_connected(const IfMgrIfTree& iftree,
 					const IPvX& addr) const
 {
     IfMgrIfTree::IfMap::const_iterator if_iter;
 
-    for (if_iter = if_tree.ifs().begin();
-	 if_iter != if_tree.ifs().end();
+    for (if_iter = iftree.ifs().begin();
+	 if_iter != iftree.ifs().end();
 	 ++if_iter) {
 	const IfMgrIfAtom& iface = if_iter->second;
 
@@ -636,12 +646,19 @@ int
 StaticRoutesNode::add_route(const StaticRoute& static_route,
 			    string& error_msg)
 {
+    StaticRoute updated_route = static_route;
+
+    //
+    // Update the route
+    //
+    update_route(_iftree, updated_route);
+
     //
     // Check the entry
     //
-    if (static_route.is_valid_entry(error_msg) != true) {
+    if (updated_route.is_valid_entry(error_msg) != true) {
 	error_msg = c_format("Cannot add route for %s: %s",
-			     static_route.network().str().c_str(),
+			     updated_route.network().str().c_str(),
 			     error_msg.c_str());
 	return XORP_ERROR;
     }
@@ -652,37 +669,44 @@ StaticRoutesNode::add_route(const StaticRoute& static_route,
     list<StaticRoute>::iterator iter;
     for (iter = _static_routes.begin(); iter != _static_routes.end(); ++iter) {
 	StaticRoute& tmp_route = *iter;
-	if (tmp_route.network() != static_route.network())
+	if (tmp_route.network() != updated_route.network())
 	    continue;
-	if ((tmp_route.unicast() != static_route.unicast())
-	    || (tmp_route.multicast() != static_route.multicast())) {
+	if ((tmp_route.unicast() != updated_route.unicast())
+	    || (tmp_route.multicast() != updated_route.multicast())) {
 	    continue;
 	}
 	error_msg = c_format("Cannot add %s route for %s: "
 			     "the route already exists",
-			     (static_route.unicast())? "unicast" : "multicast",
-			     static_route.network().str().c_str());
+			     (updated_route.unicast())?
+			     "unicast" : "multicast",
+			     updated_route.network().str().c_str());
 	return XORP_ERROR;
     }
 
     //
     // Add the route
     //
-    _static_routes.push_back(static_route);
+    _static_routes.push_back(updated_route);
 
     //
-    // Inform the RIB about the change
+    // Do policy filtering
     //
-    if (static_route.is_interface_route()) {
-	const IfMgrVifAtom* vif_atom;
-	vif_atom = _iftree.find_vif(static_route.ifname(),
-				    static_route.vifname());
-	if ((vif_atom != NULL) && (vif_atom->enabled()))
-	    inform_rib_route_change(static_route);
-    } else {
-	if (is_directly_connected(_iftree, static_route.nexthop()))
-	    inform_rib_route_change(static_route);
-    }
+    StaticRoute& added_route = _static_routes.back();
+
+    //
+    // We do not want to modify original route, so we may re-filter routes on
+    // filter configuration changes. Hence, copy route.
+    //
+    StaticRoute copy_route = added_route;
+    
+    bool accepted = do_filtering(copy_route);
+
+    // Tag the original route as filtered or not
+    added_route.set_filtered(!accepted);
+
+    // Inform RIB about the possibly modified route if it was accepted 
+    if (accepted)
+	inform_rib(copy_route);
 
     return XORP_OK;
 }
@@ -699,12 +723,19 @@ int
 StaticRoutesNode::replace_route(const StaticRoute& static_route,
 				string& error_msg)
 {
+    StaticRoute updated_route = static_route;
+
+    //
+    // Update the route
+    //
+    update_route(_iftree, updated_route);
+
     //
     // Check the entry
     //
-    if (static_route.is_valid_entry(error_msg) != true) {
+    if (updated_route.is_valid_entry(error_msg) != true) {
 	error_msg = c_format("Cannot replace route for %s: %s",
-			     static_route.network().str().c_str(),
+			     updated_route.network().str().c_str(),
 			     error_msg.c_str());
 	return XORP_ERROR;
     }
@@ -715,41 +746,53 @@ StaticRoutesNode::replace_route(const StaticRoute& static_route,
     list<StaticRoute>::iterator iter;
     for (iter = _static_routes.begin(); iter != _static_routes.end(); ++iter) {
 	StaticRoute& tmp_route = *iter;
-	if (tmp_route.network() != static_route.network())
+	if (tmp_route.network() != updated_route.network())
 	    continue;
-	if ((tmp_route.unicast() != static_route.unicast())
-	    || (tmp_route.multicast() != static_route.multicast())) {
+	if ((tmp_route.unicast() != updated_route.unicast())
+	    || (tmp_route.multicast() != updated_route.multicast())) {
 	    continue;
 	}
+
 	//
 	// Route found. Overwrite its value.
 	//
-	tmp_route = static_route;
+	bool was_filtered = tmp_route.is_filtered();
+	tmp_route = updated_route;
 
+	// Do policy filtering
+	StaticRoute copy_route = updated_route;
+	bool accepted = do_filtering(copy_route);
+	tmp_route.set_filtered(!accepted);
+
+	// Decide what to do
+	if (accepted) {
+	    if (was_filtered) {
+		copy_route.set_add_route();
+	    } else {
+	    }
+	} else {
+	    if (was_filtered) {
+		return XORP_OK;
+	    } else {
+		copy_route.set_delete_route();
+	    }
+	}
+	
 	//
 	// Inform the RIB about the change
 	//
-	if (static_route.is_interface_route()) {
-	    const IfMgrVifAtom* vif_atom;
-	    vif_atom = _iftree.find_vif(static_route.ifname(),
-					static_route.vifname());
-	    if ((vif_atom != NULL) && (vif_atom->enabled()))
-		inform_rib_route_change(static_route);
-	} else {
-	    if (is_directly_connected(_iftree, static_route.nexthop()))
-		inform_rib_route_change(static_route);
-	}
+	inform_rib(copy_route);
 
 	return XORP_OK;
     }
 
     //
-    // Coudn't find the route to replace
+    // Couldn't find the route to replace
     //
     error_msg = c_format("Cannot replace %s route for %s: "
 			 "no such route",
-			 (static_route.unicast())? "unicast" : "multicast",
-			 static_route.network().str().c_str());
+			 (updated_route.unicast())? "unicast" : "multicast",
+			 updated_route.network().str().c_str());
     return XORP_ERROR;
 }
 
@@ -765,12 +808,19 @@ int
 StaticRoutesNode::delete_route(const StaticRoute& static_route,
 			       string& error_msg)
 {
+    StaticRoute updated_route = static_route;
+
+    //
+    // Update the route
+    //
+    update_route(_iftree, updated_route);
+
     //
     // Check the entry
     //
-    if (static_route.is_valid_entry(error_msg) != true) {
+    if (updated_route.is_valid_entry(error_msg) != true) {
 	error_msg = c_format("Cannot delete route for %s: %s",
-			     static_route.network().str().c_str(),
+			     updated_route.network().str().c_str(),
 			     error_msg.c_str());
 	return XORP_ERROR;
     }
@@ -781,10 +831,10 @@ StaticRoutesNode::delete_route(const StaticRoute& static_route,
     list<StaticRoute>::iterator iter;
     for (iter = _static_routes.begin(); iter != _static_routes.end(); ++iter) {
 	StaticRoute& tmp_route = *iter;
-	if (tmp_route.network() != static_route.network())
+	if (tmp_route.network() != updated_route.network())
 	    continue;
-	if ((tmp_route.unicast() != static_route.unicast())
-	    || (tmp_route.multicast() != static_route.multicast())) {
+	if ((tmp_route.unicast() != updated_route.unicast())
+	    || (tmp_route.multicast() != updated_route.multicast())) {
 	    continue;
 	}
 
@@ -795,30 +845,25 @@ StaticRoutesNode::delete_route(const StaticRoute& static_route,
 	copy_route.set_delete_route();
 	_static_routes.erase(iter);
 
+	// If the route is filtered, then RIB doesn't know about it.
+	if (copy_route.is_filtered())
+	    return XORP_OK;
+
 	//
 	// Inform the RIB about the change
 	//
-	if (copy_route.is_interface_route()) {
-	    const IfMgrVifAtom* vif_atom;
-	    vif_atom = _iftree.find_vif(copy_route.ifname(),
-					copy_route.vifname());
-	    if ((vif_atom != NULL) && (vif_atom->enabled()))
-		inform_rib_route_change(copy_route);
-	} else {
-	    if (is_directly_connected(_iftree, copy_route.nexthop()))
-		inform_rib_route_change(copy_route);
-	}
+	inform_rib(copy_route);
 
 	return XORP_OK;
     }
 
     //
-    // Coudn't find the route to delete
+    // Couldn't find the route to delete
     //
     error_msg = c_format("Cannot delete %s route for %s: "
 			 "no such route",
-			 (static_route.unicast())? "unicast" : "multicast",
-			 static_route.network().str().c_str());
+			 (updated_route.unicast())? "unicast" : "multicast",
+			 updated_route.network().str().c_str());
     return XORP_ERROR;
 }
 
@@ -845,3 +890,137 @@ StaticRoute::is_valid_entry(string& error_msg) const
 
     return true;
 }
+
+void
+StaticRoutesNode::configure_filter(const uint32_t& filter, const string& conf)
+{
+    _policy_filters.configure(filter, conf);
+}
+
+void
+StaticRoutesNode::reset_filter(const uint32_t& filter) {
+    _policy_filters.reset(filter);
+}
+
+void
+StaticRoutesNode::push_routes()
+{
+    list<StaticRoute>::iterator iter;
+
+    // XXX: not a background task
+    for (iter = _static_routes.begin(); iter != _static_routes.end(); ++iter) {
+	StaticRoute& orig_route = *iter;
+	bool was_filtered = orig_route.is_filtered();
+
+	StaticRoute copy_route = orig_route;
+	bool accepted = do_filtering(copy_route);
+
+	debug_msg("[STATIC] Push route: %s, was filtered: %d, accepted %d\n",
+		  orig_route.network().str().c_str(),
+		  was_filtered, accepted);
+
+	orig_route.set_filtered(!accepted);
+
+	if (accepted) {
+	    if (was_filtered) {
+		copy_route.set_add_route();
+	    } else {
+		copy_route.set_replace_route();
+	    }
+	} else {
+	    // not accepted
+	    if (was_filtered) {
+		continue;
+	    } else {
+		copy_route.set_delete_route();
+	    }
+	}
+
+	inform_rib(copy_route);
+    }
+}
+
+void
+StaticRoutesNode::inform_rib(const StaticRoute& route)
+{
+    //
+    // Inform the RIB about the change
+    //
+    if (route.is_interface_route()) {
+	const IfMgrVifAtom* vif_atom;
+	vif_atom = _iftree.find_vif(route.ifname(), route.vifname());
+	if ((vif_atom != NULL) && (vif_atom->enabled()))
+	    inform_rib_route_change(route);
+    } else {
+	if (is_directly_connected(_iftree, route.nexthop()))
+	    inform_rib_route_change(route);
+    }
+}
+
+/**
+ * Update a route received from the user configuration.
+ *
+ * Currently, this method is a no-op.
+ *
+ * @param iftree the tree with the interface state to update the route.
+ * @param route the route to update.
+ * @return true if the route was updated, otherwise false.
+ */
+bool
+StaticRoutesNode::update_route(const IfMgrIfTree& iftree, StaticRoute& route)
+{
+    UNUSED(iftree);
+    UNUSED(route);
+
+    return (false);
+}
+
+bool
+StaticRoutesNode::do_filtering(StaticRoute& route)
+{
+    try {
+	ostringstream trace;
+
+	StaticRoutesVarRW varrw(route);
+
+	// Import filtering
+	bool accepted;
+
+	debug_msg("[STATIC] Running filter: %s on route: %s\n",
+		  filter::filter2str(filter::IMPORT).c_str(),
+		  route.network().str().c_str());
+	accepted = _policy_filters.run_filter(filter::IMPORT, varrw, &trace);
+
+	debug_msg("[STATIC] filter trace:\n%s\nEnd of trace.\n",
+		  trace.str().c_str());
+
+	// clear trace
+	trace.str("");
+
+	route.set_filtered(!accepted);
+
+	// Route Rejected 
+	if (!accepted) 
+	    return accepted;
+
+	StaticRoutesVarRW varrw2(route);
+
+	// Export source-match filtering
+	debug_msg("[STATIC] Running filter: %s on route: %s\n",
+		  filter::filter2str(filter::EXPORT_SOURCEMATCH).c_str(),
+		  route.network().str().c_str());
+
+	_policy_filters.run_filter(filter::EXPORT_SOURCEMATCH,
+				   varrw2, &trace);
+
+	debug_msg("[STATIC] filter trace:\n%s\nEnd of trace.\n",
+		  trace.str().c_str());
+
+	return accepted;
+    } catch(const PolicyException& e) {
+	XLOG_FATAL("PolicyException: %s", e.str().c_str());
+
+	// FIXME: What do we do ?
+	XLOG_UNFINISHED();
+    }
+}
