@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/static_routes/xorp_static_routes.cc,v 1.7 2004/06/10 22:41:57 hodson Exp $"
-
+#ident "$XORP: xorp/static_routes/xorp_static_routes.cc,v 1.11 2005/03/25 02:54:42 pavlin Exp $"
 
 //
 // XORP StaticRoutes module implementation.
 //
 
-
 #include "static_routes_module.h"
 
 #include "libxorp/xorp.h"
@@ -29,8 +27,6 @@
 #include "libxorp/eventloop.hh"
 #include "libxorp/exceptions.hh"
 
-#include "libxipc/xrl_std_router.hh"
-
 #include "xrl_static_routes_node.hh"
 
 
@@ -83,7 +79,7 @@ usage(const char *argv0, int exit_value)
 }
 
 static void
-static_routes_main(const char* finder_hostname, uint16_t finder_port)
+static_routes_main(const string& finder_hostname, uint16_t finder_port)
 {
     //
     // Init stuff
@@ -93,16 +89,16 @@ static_routes_main(const char* finder_hostname, uint16_t finder_port)
     //
     // StaticRoutes node
     //
-    XrlStdRouter xrl_std_router_static_routes(
-	eventloop,
-	"static_routes",
-	finder_hostname, finder_port);
     XrlStaticRoutesNode xrl_static_routes_node(
 	eventloop,
-	&xrl_std_router_static_routes,
+	"static_routes",
+	finder_hostname,
+	finder_port,
+	"finder",
 	"fea",
 	"rib");
-    wait_until_xrl_router_is_ready(eventloop, xrl_std_router_static_routes);
+    wait_until_xrl_router_is_ready(eventloop,
+				   xrl_static_routes_node.xrl_router());
 
     // Startup
     xrl_static_routes_node.startup();
@@ -121,8 +117,8 @@ main(int argc, char *argv[])
     int ch;
     string::size_type idx;
     const char *argv0 = argv[0];
-    string finder_hostname = FINDER_DEFAULT_HOST.str();
-    uint16_t finder_port = FINDER_DEFAULT_PORT;	// XXX: default (in host order)
+    string finder_hostname = FinderConstants::FINDER_DEFAULT_HOST().str();
+    uint16_t finder_port = FinderConstants::FINDER_DEFAULT_PORT();
 
     //
     // Initialize and start xlog
@@ -177,7 +173,7 @@ main(int argc, char *argv[])
     // Run everything
     //
     try {
-	static_routes_main(finder_hostname.c_str(), finder_port);
+	static_routes_main(finder_hostname, finder_port);
     } catch(...) {
 	xorp_catch_standard_exceptions();
     }
