@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2004 International Computer Science Institute
+// Copyright (c) 2001-2005 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/xrl/tests/test_generated.cc,v 1.12 2004/06/10 22:42:19 hodson Exp $"
+#ident "$XORP: xorp/xrl/tests/test_generated.cc,v 1.14 2005/03/25 02:55:15 pavlin Exp $"
 
 #include <iostream>
 
@@ -36,7 +36,8 @@ static void
 run_test()
 {
     EventLoop e;
-    FinderServer finder(e);
+    FinderServer finder(e, FinderConstants::FINDER_DEFAULT_HOST(),
+			FinderConstants::FINDER_DEFAULT_PORT());
 
     // Configure target for test
     XrlStdRouter tgt_router(e, g_tgt_name, finder.addr(), finder.port());
